@@ -205,6 +205,10 @@ DEFINE_int32(kudu_client_connection_negotiation_timeout_ms, 3000,
     "(Advanced) Timeout for connection negotiation between Kudu client and "
     "Kudu masters and tablet servers, in milliseconds");
 
+// SASL protocol name for Kudu used in the FE and BE when creating Kudu client.
+// The default name is "kudu".
+DEFINE_string(kudu_sasl_protocol_name, "kudu", "SASL protocol name for Kudu");
+
 DEFINE_int64(inc_stats_size_limit_bytes, 200 * (1LL<<20), "Maximum size of "
     "incremental stats the catalog is allowed to serialize per table. "
     "This limit is set as a safety check, to prevent the JVM from "
@@ -262,19 +266,17 @@ DEFINE_bool(invalidate_tables_on_memory_pressure, false, "Configure catalogd to
     "invalidate_table_timeout_s. To enable this feature, a true flag must be applied to "
     "both catalogd and impalad.");
 
-DEFINE_int32(hms_event_polling_interval_s, 0,
-    "Configure catalogd to invalidate cached table metadata based on metastore events. "
+DEFINE_int32(hms_event_polling_interval_s, 1,
+    "Configure catalogd to refresh cached table metadata based on metastore events. "
     "These metastore events could be generated by external systems like Apache Hive or "
     "a different Impala cluster using the same Hive metastore server as this one. "
     "A non-zero value of this flag sets the polling interval of catalogd in seconds to "
     "fetch new metastore events. A value of zero disables this feature. When enabled, "
-    "this flag has the same effect as \"INVALIDATE METADATA\" statement on the table "
+    "this flag has the same effect as \"REFRESH\" statement on the table "
     "for certain metastore event types. Additionally, in case of events which detect "
     "creation or removal of objects from metastore, catalogd adds or removes such "
     "objects from its cached metadata. This feature is independent of time and memory "
-    "based automatic invalidation of tables. Note that this is still an experimental "
-    "feature and not recommended to be deployed on production systems until it is "
-    "made generally available.");
+    "based automatic invalidation of tables.");
 
 DEFINE_bool(enable_insert_events, true,
     "Enables insert events in the events processor. When this configuration is set to "
@@ -346,6 +348,11 @@ DEFINE_bool(enable_incremental_metadata_updates, true,
     "propagated as a whole object in the statestore topic updates. Note that legacy "
     "coordinators can apply incremental or full table updates so don't need this flag.");
 
+DEFINE_bool(enable_catalogd_hms_cache, true,
+    "If true, response for the HMS APIs that are implemented in catalogd will be served "
+    "from catalogd. If this flag is false or a given API is not implemented in catalogd,"
+    " it will be redirected to HMS.");
+
 DEFINE_bool(enable_legacy_avx_support, false,
     "If true, Impala relaxes its x86_64 CPU feature requirement to allow running on "
     "machines with AVX but no AVX2. This allows running Impala on older machines "
@@ -389,6 +396,7 @@ REMOVED_FLAG(coordinator_rpc_threads);
 REMOVED_FLAG(disable_admission_control);
 REMOVED_FLAG(disable_mem_pools);
 REMOVED_FLAG(enable_accept_queue_server);
+REMOVED_FLAG(enable_orc_scanner);
 REMOVED_FLAG(enable_partitioned_aggregation);
 REMOVED_FLAG(enable_partitioned_hash_join);
 REMOVED_FLAG(enable_phj_probe_side_filtering);
