@@ -46,13 +46,11 @@ class OrcComplexColumnReader;
 ///   * 'ProcessFileTail' to create orc::Reader with OrcMemPool and ScanRangeInputStream
 ///   * Resolve TupleDescriptors to get a list of mapped orc::Types (a.k.a column/node).
 ///     Init 'row_reader_options_' with these selected type ids.
-///   * Build a map 'col_id_path_map_' from each orc::Type id to a SchemaPath. Will be
-///     used in creating OrcColumnReaders.
 ///   * Create temporary orc::RowReader with 'row_reader_options_' to get the selected
 ///     subset of the schema (a tree of the selected orc::Types, i.e. the local variable
 ///     'root_type' in 'HdfsOrcScanner::Open').
-///   * Create OrcColumnReaders recursively with 'root_type', 'col_id_path_map_' and
-///     TupleDescriptors (HdfsOrcScanner::Open)
+///   * Create OrcColumnReaders recursively with 'root_type' and TupleDescriptors
+///     (HdfsOrcScanner::Open)
 ///   * At the begining of processing a Stripe, we update 'row_reader_options_' to have
 ///     the range of the Stripe boundaries. Then create a orc::RowReader for this Stripe
 ///     (HdfsOrcScanner::NextStripe)
@@ -91,6 +89,39 @@ class HdfsOrcScanner : public HdfsColumnarScanner {
     boost::unordered_map<char*, uint64_t> chunk_sizes_;
   };
 
+  struct ColumnRange {
+    uint64_t offset_;
+    uint64_t length_;
+    uint64_t current_position_;
+    orc::StreamKind kind_;
+    uint64_t type_id_;
+    HdfsOrcScanner* scanner_;
+    ScannerContext::Stream* stream_ = nullptr;
+    int io_reservation = 0;
+
+    ColumnRange(uint64_t length, uint64_t offset, orc::StreamKind kind, uint64_t type_id,
+        HdfsOrcScanner* scanner)
+      : offset_(offset),
+        length_(length),
+        current_position_(offset),
+        kind_(kind),
+        type_id_(type_id),
+        scanner_(scanner) {}
+
+    bool operator<(const ColumnRange& other) const { return offset_ < other.offset_; }
+
+    /// Read 'length' bytes from the stream_ starting at 'offset' into 'buf'.
+    Status read(void* buf, uint64_t length, uint64_t offset);
+
+    std::string debug() {
+      return strings::Substitute(
+          "colrange_offset: $0 colrange_length: $1 colrange_pos: $2 "
+          "typeId: $3 kind: $4 filename: $5",
+          offset_, length_, current_position_, type_id_, orc::streamKindToString(kind_),
+          scanner_->filename());
+    }
+  };
+
   /// A wrapper of DiskIoMgr to be used by the ORC lib.
   class ScanRangeInputStream : public orc::InputStream {
    public:
@@ -101,18 +132,21 @@ class HdfsOrcScanner : public HdfsColumnarScanner {
           scanner->context_->partition_descriptor()->id(), filename_);
     }
 
+    /// Get the total length of the file in bytes.
     uint64_t getLength() const override {
       return file_desc_->file_length;
     }
 
-    uint64_t getNaturalReadSize() const override {
-      return ExecEnv::GetInstance()->disk_io_mgr()->max_buffer_size();
-    }
+    /// Get the natural size for reads.
+    /// Return 0 to let ORC lib decide what is the best buffer size depending on
+    /// compression method that is being used.
+    uint64_t getNaturalReadSize() const override { return 0; }
 
     /// Read 'length' bytes from the file starting at 'offset' into the buffer starting
     /// at 'buf'.
     void read(void* buf, uint64_t length, uint64_t offset) override;
 
+    /// Get the name of the stream for error messages.
     const std::string& getName() const override {
       return filename_;
     }
@@ -121,6 +155,9 @@ class HdfsOrcScanner : public HdfsColumnarScanner {
     HdfsOrcScanner* scanner_;
     const HdfsFileDesc* file_desc_;
     std::string filename_;
+
+    /// Default read implementation for non async IO.
+    Status readRandom(void* buf, uint64_t length, uint64_t offset);
   };
 
   HdfsOrcScanner(HdfsScanNodeBase* scan_node, RuntimeState* state);
@@ -131,10 +168,24 @@ class HdfsOrcScanner : public HdfsColumnarScanner {
   static Status IssueInitialRanges(HdfsScanNodeBase* scan_node,
       const std::vector<HdfsFileDesc*>& files) WARN_UNUSED_RESULT;
 
+  /// Size of the file footer for ORC.
+  /// This is a guess, matched with orc::DIRECTORY_SIZE_GUESS from ORC lib.
+  /// If this value is too little, we will need to issue another read.
+  static const int64_t ORC_FOOTER_SIZE = 1024 * 16;
+  static_assert(ORC_FOOTER_SIZE <= READ_SIZE_MIN_VALUE,
+      "ORC_FOOTER_SIZE can not be greater than READ_SIZE_MIN_VALUE.\n"
+      "You can increase ORC_FOOTER_SIZE if you want, "
+      "just don't forget to increase READ_SIZE_MIN_VALUE as well.");
+
   virtual Status Open(ScannerContext* context) override WARN_UNUSED_RESULT;
   virtual Status ProcessSplit() override WARN_UNUSED_RESULT;
   virtual void Close(RowBatch* row_batch) override;
 
+ protected:
+  virtual int64_t GetNumberOfRowsInFile() const override {
+    return static_cast<int64_t>(reader_->getNumberOfRows());
+  }
+
  private:
   friend class OrcColumnReader;
   friend class OrcDateColumnReader;
@@ -145,23 +196,10 @@ class HdfsOrcScanner : public HdfsColumnarScanner {
   friend class OrcStructReader;
   friend class OrcListReader;
   friend class OrcMapReader;
-  friend class HdfsOrcScannerTest;
 
   /// Memory guard of the tuple_mem_
   uint8_t* tuple_mem_end_ = nullptr;
 
-  /// Index of the current stripe being processed. Stripe in ORC is equivalent to
-  /// RowGroup in Parquet. Initialized to -1 which indicates that we have not started
-  /// processing the first stripe yet (GetNext() has not yet been called).
-  int32_t stripe_idx_ = -1;
-
-  /// Counts the number of rows processed for the current stripe.
-  int64_t stripe_rows_read_ = 0;
-
-  /// Indicates whether we should advance to the next stripe in the next GetNext().
-  /// Starts out as true to move to the very first stripe.
-  bool advance_stripe_ = true;
-
   /// Indicates whether we are at the end of a stripe.
   bool end_of_stripe_ = true;
 
@@ -177,6 +215,12 @@ class HdfsOrcScanner : public HdfsColumnarScanner {
   /// Pool to copy non-dictionary buffer into. This pool is responsible for handling
   /// vector batches that do not necessarily fit into one row batch.
   boost::scoped_ptr<MemPool> data_batch_pool_;
+  /// Pool to copy values into when building search arguments. Freed on Close().
+  boost::scoped_ptr<MemPool> search_args_pool_;
+
+  /// Clone of statistics conjunct evaluators. Has the same lifetime as the scanner.
+  /// Stored in 'obj_pool_'.
+  vector<ScalarExprEvaluator*> stats_conjunct_evals_;
 
   std::unique_ptr<OrcSchemaResolver> schema_resolver_ = nullptr;
 
@@ -214,9 +258,6 @@ class HdfsOrcScanner : public HdfsColumnarScanner {
   std::unordered_map<const SlotDescriptor*, uint64_t> slot_to_col_id_;
   std::unordered_map<const TupleDescriptor*, uint64_t> tuple_to_col_id_;
 
-  /// Scan range for the metadata (file tail).
-  const io::ScanRange* metadata_range_ = nullptr;
-
   /// With the help of it we can check the validity of ACID write ids.
   ValidWriteIdList valid_write_ids_;
 
@@ -236,27 +277,29 @@ class HdfsOrcScanner : public HdfsColumnarScanner {
   /// For files not written by Streaming Ingestion we can assume that every row is valid.
   bool row_batches_need_validation_ = false;
 
-  /// Timer for materializing rows. This ignores time getting the next buffer.
-  ScopedTimer<MonotonicStopWatch> assemble_rows_timer_;
-
-  /// Average and min/max time spent processing the footer by each split.
-  RuntimeProfile::SummaryStatsCounter* process_footer_timer_stats_ = nullptr;
-
-  /// Number of columns that need to be read.
-  RuntimeProfile::Counter* num_cols_counter_ = nullptr;
+  /// Scan range for column streams.
+  /// StartColumnReading() guarantees that columnRanges_ is sorted by the element's
+  /// offset, and there are no two overlapping range.
+  vector<ColumnRange> columnRanges_;
 
   /// Number of stripes that need to be read.
   RuntimeProfile::Counter* num_stripes_counter_ = nullptr;
 
-  /// Number of scanners that end up doing no reads because their splits don't overlap
-  /// with the midpoint of any stripe in the file.
-  RuntimeProfile::Counter* num_scanners_with_no_reads_counter_ = nullptr;
+  /// Number of predicates that are pushed down to the ORC reader.
+  RuntimeProfile::Counter* num_pushed_down_predicates_counter_ = nullptr;
+
+  /// Number of runtime filters that are pushed down to the ORC reader.
+  RuntimeProfile::Counter* num_pushed_down_runtime_filters_counter_ = nullptr;
+
+  /// Number of arrived runtime IN-list filters that can be pushed down.
+  /// Used in ShouldUpdateSearchArgument(). Init to -1 so the check can pass at first.
+  int num_pushable_in_list_filters_ = -1;
 
   /// Number of collection items read in current row batch. It is a scanner-local counter
   /// used to reduce the frequency of updating HdfsScanNode counter. It is updated by the
   /// callees of AssembleRows() and is merged into the HdfsScanNode counter at the end of
   /// AssembleRows() and then is reset to 0.
-  int64_t coll_items_read_counter_;
+  int64_t coll_items_read_counter_ = 0;
 
   const char *filename() const { return metadata_range_->file(); }
 
@@ -266,6 +309,19 @@ class HdfsOrcScanner : public HdfsColumnarScanner {
   /// row_reader_ to scan it.
   Status NextStripe() WARN_UNUSED_RESULT;
 
+  /// Begin reading columns of given stripe.
+  Status StartColumnReading(const orc::StripeInformation& stripe);
+
+  /// Find scan ColumnRange that where range [offset, offset+length) lies in.
+  /// Return nullptr if such range does not exist.
+  inline ColumnRange* FindColumnRange(uint64_t length, uint64_t offset) {
+    auto in_range = [length, offset](ColumnRange c) {
+      return (offset >= c.offset_) && (offset + length <= c.offset_ + c.length_);
+    };
+    auto range = std::find_if(columnRanges_.begin(), columnRanges_.end(), in_range);
+    return range != columnRanges_.end() ? &(*range) : nullptr;
+  }
+
   /// Reads data to materialize instances of 'tuple_desc'.
   /// Returns a non-OK status if a non-recoverable error was encountered and execution
   /// of this query should be terminated immediately.
@@ -284,7 +340,7 @@ class HdfsOrcScanner : public HdfsColumnarScanner {
   Status TransferTuples(RowBatch* dst_batch) WARN_UNUSED_RESULT;
 
   /// Process the file footer and parse file_metadata_.  This should be called with the
-  /// last FOOTER_SIZE bytes in context_.
+  /// last ORC_FOOTER_SIZE bytes in context_.
   Status ProcessFileTail() WARN_UNUSED_RESULT;
 
   /// Resolve SchemaPath in TupleDescriptors and translate them to ORC type ids into
@@ -311,6 +367,57 @@ class HdfsOrcScanner : public HdfsColumnarScanner {
 
   void SetSyntheticAcidFieldForOriginalFile(const SlotDescriptor* slot_desc,
       Tuple* template_tuple);
+
+  bool PrepareBinaryPredicate(const string& fn_name, uint64_t orc_column_id,
+      const ColumnType& type, ScalarExprEvaluator* eval,
+      orc::SearchArgumentBuilder* sarg);
+  bool PrepareInListPredicate(uint64_t orc_column_id, const ColumnType& type,
+      ScalarExprEvaluator* eval, orc::SearchArgumentBuilder* sarg);
+  bool PrepareInListPredicate(uint64_t orc_column_id, const ColumnType& type,
+      const std::vector<orc::Literal>& in_list, orc::SearchArgumentBuilder* sarg);
+  void PrepareIsNullPredicate(bool is_not_null, uint64_t orc_column_id,
+      const ColumnType& type, orc::SearchArgumentBuilder* sarg);
+
+  /// Builds ORC search arguments from the conjuncts and arrived runtime filters.
+  /// The search arguments will be re-built each time we start reading a new stripe,
+  /// because we may have new runtime filters arrive.
+  Status PrepareSearchArguments() WARN_UNUSED_RESULT;
+
+  /// Helper function for GetLiteralSearchArguments. The template parameter T is the
+  /// type of val, and U is the destination type that the constructor of orc::Literal
+  /// accepts. The conversion is required here, since otherwise multiple implicit
+  /// conversions would be possible.
+  template<typename T, typename U>
+  orc::Literal GetOrcPrimitiveLiteral(orc::PredicateDataType predicate_type, void* val);
+
+  /// Helper function for mapping ColumnType to orc::PredicateDataType.
+  static orc::PredicateDataType GetOrcPredicateDataType(const ColumnType& type);
+
+  /// Returns the literal specified by 'child_idx' from the stats conjuct 'eval',
+  /// with the assumption that the specifit child is a literal.
+  orc::Literal GetSearchArgumentLiteral(ScalarExprEvaluator* eval, int child_idx,
+      const ColumnType& dst_type, orc::PredicateDataType* predicate_type);
+
+  /// Return true if [offset, offset+length) is within the remaining part of the footer
+  /// stream (initial scan range issued by IssueInitialRanges).
+  inline bool IsInFooterRange(uint64_t offset, uint64_t length) const {
+    DCHECK(stream_ != nullptr);
+    return offset >= stream_->file_offset() && length <= stream_->bytes_left();
+  }
+
+  Status ReadFooterStream(void* buf, uint64_t length, uint64_t offset);
+
+  /// Updates the SearchArgument based on arrived runtime filters.
+  /// Returns true if any filter is applied.
+  bool UpdateSearchArgumentWithFilters(orc::SearchArgumentBuilder* sarg);
+
+  /// Decides whether we should rebuild the SearchArgument. It returns true at the first
+  /// call and whenever a new and usable IN-list filter arrives.
+  bool ShouldUpdateSearchArgument();
+
+  /// Checks whether the runtime filter is a usable IN-list filter that can be pushed
+  /// down.
+  bool IsPushableInListFilter(const RuntimeFilter* filter);
 };
 
 } // namespace impala
