@@ -50,6 +50,7 @@
 #include "runtime/runtime-state.h"
 #include "util/compression-util.h"
 #include "util/disk-info.h"
+#include "util/flat_buffer.h"
 #include "util/hdfs-util.h"
 #include "util/impalad-metrics.h"
 #include "util/metrics.h"
@@ -213,15 +214,15 @@ Status HdfsScanPlanNode::Init(const TPlanNode& tnode, FragmentState* state) {
   DCHECK(conjuncts_map_[tuple_id].empty());
   conjuncts_map_[tuple_id] = conjuncts_;
 
-  // Add min max conjuncts
-  if (tnode.hdfs_scan_node.__isset.min_max_tuple_id) {
-    TupleDescriptor* min_max_tuple_desc =
-        state->desc_tbl().GetTupleDescriptor(tnode.hdfs_scan_node.min_max_tuple_id);
-    DCHECK(min_max_tuple_desc != nullptr);
-    RowDescriptor* min_max_row_desc = state->obj_pool()->Add(
-        new RowDescriptor(min_max_tuple_desc, /* is_nullable */ false));
-    RETURN_IF_ERROR(ScalarExpr::Create(tnode.hdfs_scan_node.min_max_conjuncts,
-        *min_max_row_desc, state, &min_max_conjuncts_));
+  // Add stats conjuncts
+  if (tnode.hdfs_scan_node.__isset.stats_tuple_id) {
+    TupleDescriptor* stats_tuple_desc =
+        state->desc_tbl().GetTupleDescriptor(tnode.hdfs_scan_node.stats_tuple_id);
+    DCHECK(stats_tuple_desc != nullptr);
+    RowDescriptor* stats_row_desc = state->obj_pool()->Add(
+        new RowDescriptor(stats_tuple_desc, /* is_nullable */ false));
+    RETURN_IF_ERROR(ScalarExpr::Create(tnode.hdfs_scan_node.stats_conjuncts,
+        *stats_row_desc, state, &stats_conjuncts_));
   }
 
   // Transfer overlap predicate descs.
@@ -250,6 +251,11 @@ Status HdfsScanPlanNode::ProcessScanRangesAndInitSharedState(FragmentState* stat
     for (const ScanRangeParamsPB& params : ranges->second.scan_ranges()) {
       DCHECK(params.scan_range().has_hdfs_file_split());
       const HdfsFileSplitPB& split = params.scan_range().hdfs_file_split();
+      const org::apache::impala::fb::FbFileMetadata* file_metadata = nullptr;
+      if (params.scan_range().has_file_metadata()) {
+        file_metadata = flatbuffers::GetRoot<org::apache::impala::fb::FbFileMetadata>(
+            params.scan_range().file_metadata().c_str());
+      }
       HdfsPartitionDescriptor* partition_desc =
           hdfs_table_->GetPartition(split.partition_id());
       if (template_tuple_map_.find(split.partition_id()) == template_tuple_map_.end()) {
@@ -284,6 +290,7 @@ Status HdfsScanPlanNode::ProcessScanRangesAndInitSharedState(FragmentState* stat
         file_desc->mtime = split.mtime();
         file_desc->file_compression = CompressionTypePBToThrift(split.file_compression());
         file_desc->file_format = partition_desc->file_format();
+        file_desc->file_metadata = file_metadata;
         RETURN_IF_ERROR(HdfsFsCache::instance()->GetConnection(
             native_file_path, &file_desc->fs, &fs_cache));
         shared_state_.per_type_files_[partition_desc->file_format()].push_back(file_desc);
@@ -372,7 +379,7 @@ void HdfsScanPlanNode::Close() {
     if (tid_conjunct.first == tuple_id) continue;
     ScalarExpr::Close(tid_conjunct.second);
   }
-  ScalarExpr::Close(min_max_conjuncts_);
+  ScalarExpr::Close(stats_conjuncts_);
   if (shared_state_.template_pool_.get() != nullptr) {
     shared_state_.template_pool_->FreeAll();
   }
@@ -392,18 +399,17 @@ Status HdfsScanPlanNode::CreateExecNode(RuntimeState* state, ExecNode** node) co
 HdfsScanNodeBase::HdfsScanNodeBase(ObjectPool* pool, const HdfsScanPlanNode& pnode,
     const THdfsScanNode& hdfs_scan_node, const DescriptorTbl& descs)
   : ScanNode(pool, pnode, descs),
-    min_max_tuple_id_(
-        hdfs_scan_node.__isset.min_max_tuple_id ? hdfs_scan_node.min_max_tuple_id : -1),
-    min_max_conjuncts_(pnode.min_max_conjuncts_),
-    min_max_tuple_desc_(
-        min_max_tuple_id_ == -1 ? nullptr : descs.GetTupleDescriptor(min_max_tuple_id_)),
+    stats_tuple_id_(
+        hdfs_scan_node.__isset.stats_tuple_id ? hdfs_scan_node.stats_tuple_id : -1),
+    stats_conjuncts_(pnode.stats_conjuncts_),
+    stats_tuple_desc_(
+        stats_tuple_id_ == -1 ? nullptr : descs.GetTupleDescriptor(stats_tuple_id_)),
     skip_header_line_count_(hdfs_scan_node.__isset.skip_header_line_count ?
             hdfs_scan_node.skip_header_line_count :
             0),
     tuple_id_(pnode.tuple_id_),
-    parquet_count_star_slot_offset_(
-        hdfs_scan_node.__isset.parquet_count_star_slot_offset ?
-            hdfs_scan_node.parquet_count_star_slot_offset :
+    count_star_slot_offset_(hdfs_scan_node.__isset.count_star_slot_offset ?
+            hdfs_scan_node.count_star_slot_offset :
             -1),
     is_partition_key_scan_(hdfs_scan_node.is_partition_key_scan),
     tuple_desc_(pnode.tuple_desc_),
@@ -441,10 +447,10 @@ Status HdfsScanNodeBase::Prepare(RuntimeState* state) {
     }
   }
 
-  // Prepare min max statistics conjuncts.
-  if (min_max_tuple_id_ != -1) {
-    RETURN_IF_ERROR(ScalarExprEvaluator::Create(min_max_conjuncts_, state, pool_,
-        expr_perm_pool(), expr_results_pool(), &min_max_conjunct_evals_));
+  // Prepare stats statistics conjuncts.
+  if (stats_tuple_id_ != -1) {
+    RETURN_IF_ERROR(ScalarExprEvaluator::Create(stats_conjuncts_, state, pool_,
+        expr_perm_pool(), expr_results_pool(), &stats_conjunct_evals_));
   }
 
   // Check if reservation was enough to allocate at least one buffer. The
@@ -459,7 +465,6 @@ Status HdfsScanNodeBase::Prepare(RuntimeState* state) {
   }
 
   // One-time initialization of state that is constant across scan ranges
-  scan_node_pool_.reset(new MemPool(mem_tracker()));
   runtime_profile()->AddInfoString("Table Name", hdfs_table_->fully_qualified_name());
 
   if (HasRowBatchQueue()) {
@@ -521,8 +526,8 @@ Status HdfsScanNodeBase::Open(RuntimeState* state) {
     RETURN_IF_ERROR(ScalarExprEvaluator::Open(entry.second, state));
   }
 
-  // Open min max conjuncts
-  RETURN_IF_ERROR(ScalarExprEvaluator::Open(min_max_conjunct_evals_, state));
+  // Open stats conjuncts
+  RETURN_IF_ERROR(ScalarExprEvaluator::Open(stats_conjunct_evals_, state));
 
   RETURN_IF_ERROR(ClaimBufferReservation(state));
   reader_context_ = ExecEnv::GetInstance()->disk_io_mgr()->RegisterContext();
@@ -621,8 +626,6 @@ void HdfsScanNodeBase::Close(RuntimeState* state) {
   // There should be no active hdfs read threads.
   DCHECK_EQ(active_hdfs_read_thread_counter_.value(), 0);
 
-  if (scan_node_pool_.get() != NULL) scan_node_pool_->FreeAll();
-
   // Close collection conjuncts
   for (auto& tid_conjunct_eval : conjunct_evals_map_) {
     // ExecNode::conjunct_evals_ are already closed in ExecNode::Close()
@@ -630,8 +633,8 @@ void HdfsScanNodeBase::Close(RuntimeState* state) {
     ScalarExprEvaluator::Close(tid_conjunct_eval.second, state);
   }
 
-  // Close min max conjunct
-  ScalarExprEvaluator::Close(min_max_conjunct_evals_, state);
+  // Close stats conjunct
+  ScalarExprEvaluator::Close(stats_conjunct_evals_, state);
   ScanNode::Close(state);
 }
 
@@ -886,7 +889,7 @@ void HdfsScanNodeBase::InitNullCollectionValues(const TupleDescriptor* tuple_des
       continue;
     }
     // Recursively traverse collection items.
-    const TupleDescriptor* item_desc = slot_desc->collection_item_descriptor();
+    const TupleDescriptor* item_desc = slot_desc->children_tuple_descriptor();
     if (item_desc->collection_slots().empty()) continue;
     for (int i = 0; i < slot->num_tuples; ++i) {
       int item_offset = i * item_desc->byte_size();
@@ -919,8 +922,7 @@ bool HdfsScanNodeBase::PartitionPassesFilters(int32_t partition_id,
   if (template_tuple == nullptr) return true;
   TupleRow* tuple_row_mem = reinterpret_cast<TupleRow*>(&template_tuple);
   for (const FilterContext& ctx: filter_ctxs) {
-    int target_ndx = ctx.filter->filter_desc().planid_to_target_ndx.at(id_);
-    if (!ctx.filter->filter_desc().targets[target_ndx].is_bound_by_partition_columns) {
+    if (!ctx.filter->IsBoundByPartitionColumn(id_)) {
       continue;
     }
 
@@ -983,8 +985,8 @@ void HdfsScanPlanNode::ComputeSlotMaterializationOrder(
   }
 }
 
-void HdfsScanNodeBase::TransferToScanNodePool(MemPool* pool) {
-  scan_node_pool_->AcquireData(pool, false);
+void HdfsScanNodeBase::TransferToSharedStatePool(MemPool* pool) {
+  shared_state_->TransferToSharedStatePool(pool);
 }
 
 void HdfsScanNodeBase::UpdateHdfsSplitStats(
@@ -1188,6 +1190,11 @@ Tuple* ScanRangeSharedState::GetTemplateTupleForPartitionId(int64_t partition_id
   return partition_template_tuple_map_[partition_id];
 }
 
+void ScanRangeSharedState::TransferToSharedStatePool(MemPool* pool) {
+  unique_lock<mutex> l(metadata_lock_);
+  template_pool_->AcquireData(pool, false);
+}
+
 void ScanRangeSharedState::UpdateRemainingScanRangeSubmissions(int32_t delta) {
   int new_val = remaining_scan_range_submissions_.Add(delta);
   DCHECK_GE(new_val, 0);
