@@ -30,6 +30,7 @@
 #include <boost/unordered_map.hpp>
 
 #include "codegen/codegen-fn-ptr.h"
+#include "exec/acid-metadata-utils.h"
 #include "exec/filter-context.h"
 #include "exec/scan-node.h"
 #include "runtime/descriptors.h"
@@ -41,6 +42,10 @@
 #include "util/spinlock.h"
 #include "util/unique-id-hash.h"
 
+namespace org { namespace apache { namespace impala { namespace fb {
+struct FbFileMetadata;
+}}}}
+
 namespace impala {
 
 class ScannerContext;
@@ -83,6 +88,9 @@ struct HdfsFileDesc {
   /// Splits (i.e. raw byte ranges) for this file, assigned to this scan node.
   std::vector<io::ScanRange*> splits;
 
+  /// Extra file metadata, e.g. Iceberg-related file-level info.
+  const ::org::apache::impala::fb::FbFileMetadata* file_metadata;
+
   /// Some useful typedefs for creating HdfsFileDesc related data structures.
   /// This is a pair for partition ID and filename which uniquely identifies a file.
   typedef pair<int64_t, std::string> PartitionFileKey;
@@ -183,6 +191,9 @@ class ScanRangeSharedState {
   /// cancellation. Must be called before adding or removing scan ranges to the queue.
   void AddCancellationHook(RuntimeState* state);
 
+  /// Transfers all memory from 'pool' to 'template_pool_'.
+  void TransferToSharedStatePool(MemPool* pool);
+
  private:
   friend class HdfsScanPlanNode;
 
@@ -276,13 +287,13 @@ class HdfsScanPlanNode : public ScanPlanNode {
   typedef std::unordered_map<TupleId, std::vector<ScalarExpr*>> ConjunctsMap;
   ConjunctsMap conjuncts_map_;
 
-  /// Conjuncts to evaluate on parquet::Statistics.
-  std::vector<ScalarExpr*> min_max_conjuncts_;
+  /// Conjuncts to evaluate on parquet::Statistics or to be pushed down to ORC reader.
+  std::vector<ScalarExpr*> stats_conjuncts_;
 
   /// The list of overlap predicate descs. Each is used to find out whether the range
   /// of values of a data item overlap with a min/max filter. Data structure wise, each
   /// desc is composed of the ID of the min/max filter, the slot index in
-  /// 'min_max_tuple_desc_' to hold the min value of the data item and the actual overlap
+  /// 'stats_tuple_desc_' to hold the min value of the data item and the actual overlap
   /// predicate. The next slot after the slot index implicitly holds the max value of
   /// the data item.
   std::vector<TOverlapPredicateDesc> overlap_predicate_descs_;
@@ -418,22 +429,24 @@ class HdfsScanNodeBase : public ScanNode {
   /// Returns number of partition key slots.
   int num_materialized_partition_keys() const { return partition_key_slots_.size(); }
 
-  int min_max_tuple_id() const { return min_max_tuple_id_; }
+  int stats_tuple_id() const { return stats_tuple_id_; }
 
-  const std::vector<ScalarExprEvaluator*>& min_max_conjunct_evals() const {
-    return min_max_conjunct_evals_;
+  const std::vector<ScalarExprEvaluator*>& stats_conjunct_evals() const {
+    return stats_conjunct_evals_;
   }
 
-  const TupleDescriptor* min_max_tuple_desc() const { return min_max_tuple_desc_; }
+  const TupleDescriptor* stats_tuple_desc() const { return stats_tuple_desc_; }
   const TupleDescriptor* tuple_desc() const { return tuple_desc_; }
   const HdfsTableDescriptor* hdfs_table() const { return hdfs_table_; }
   const AvroSchemaElement& avro_schema() const { return avro_schema_; }
   int skip_header_line_count() const { return skip_header_line_count_; }
   io::RequestContext* reader_context() const { return reader_context_.get(); }
-  bool optimize_parquet_count_star() const {
-    return parquet_count_star_slot_offset_ != -1;
+  bool optimize_count_star() const {
+    bool is_optimized = count_star_slot_offset_ != -1;
+    DCHECK(!hdfs_table_->IsTableFullAcid() || !is_optimized);
+    return is_optimized;
   }
-  int parquet_count_star_slot_offset() const { return parquet_count_star_slot_offset_; }
+  int count_star_slot_offset() const { return count_star_slot_offset_; }
   bool is_partition_key_scan() const { return is_partition_key_scan_; }
 
   typedef std::unordered_map<TupleId, std::vector<ScalarExprEvaluator*>>
@@ -566,8 +579,34 @@ class HdfsScanNodeBase : public ScanNode {
     return materialized_slots().empty() && tuple_desc()->tuple_path().empty();
   }
 
-  /// Transfers all memory from 'pool' to 'scan_node_pool_'.
-  virtual void TransferToScanNodePool(MemPool* pool);
+  /// Return true if scan over 'filename' require row validation.
+  /// Hive Streaming Ingestion allocates multiple write ids, hence create delta
+  /// directories like delta_5_10. Then it continuously appends new stripes (and footers)
+  /// to the ORC files of the delte dir. So it's possible that the file has rows that
+  /// Impala is not allowed to see based on its valid write id list. In such cases we need
+  /// to validate the write ids of the row batches.
+  inline bool RequireRowValidation(std::string filename) const {
+    if (!hdfs_table()->IsTableFullAcid()) return false;
+    if (ValidWriteIdList::IsCompacted(filename)) return false;
+    ValidWriteIdList valid_write_ids;
+    std::pair<int64_t, int64_t> acid_write_id_range =
+        valid_write_ids.GetWriteIdRange(filename);
+    valid_write_ids.InitFrom(hdfs_table()->ValidWriteIdList());
+    ValidWriteIdList::RangeResponse rows_valid = valid_write_ids.IsWriteIdRangeValid(
+        acid_write_id_range.first, acid_write_id_range.second);
+    DCHECK_NE(rows_valid, ValidWriteIdList::NONE);
+    return rows_valid == ValidWriteIdList::SOME;
+  }
+
+  /// Return true if scan over 'filename 'can be served only by reading the file metadata,
+  /// such as a count(*) over the table.
+  inline bool ReadsFileMetadataOnly(std::string filename) const {
+    return !RequireRowValidation(filename)
+        && (IsZeroSlotTableScan() || optimize_count_star());
+  }
+
+  /// Transfers all memory from 'pool' to shared state of all scanners.
+  void TransferToSharedStatePool(MemPool* pool);
 
   /// map from volume id to <number of split, per volume split lengths>
   typedef boost::unordered_map<int32_t, std::pair<int, int64_t>> PerVolumeStats;
@@ -626,14 +665,14 @@ class HdfsScanNodeBase : public ScanNode {
   friend class HdfsScanner;
 
   /// Tuple id of the tuple used to evaluate conjuncts on parquet::Statistics.
-  const int min_max_tuple_id_;
+  const int stats_tuple_id_;
 
   /// Conjuncts to evaluate on parquet::Statistics.
-  const vector<ScalarExpr*>& min_max_conjuncts_;
-  vector<ScalarExprEvaluator*> min_max_conjunct_evals_;
+  const vector<ScalarExpr*>& stats_conjuncts_;
+  vector<ScalarExprEvaluator*> stats_conjunct_evals_;
 
   /// Descriptor for the tuple used to evaluate conjuncts on parquet::Statistics.
-  TupleDescriptor* min_max_tuple_desc_ = nullptr;
+  TupleDescriptor* stats_tuple_desc_ = nullptr;
 
   // Number of header lines to skip at the beginning of each file of this table. Only set
   // to values > 0 for hdfs text files.
@@ -642,11 +681,11 @@ class HdfsScanNodeBase : public ScanNode {
   /// Tuple id of the tuple descriptor to be used.
   const int tuple_id_;
 
-  /// The byte offset of the slot for Parquet metadata if Parquet count star optimization
+  /// The byte offset of the slot for Parquet/ORC metadata if count star optimization
   /// is enabled. When set, this scan node can optimize a count(*) query by populating
   /// the tuple with data from the Parquet num rows statistic. See
-  /// applyParquetCountStartOptimization() in HdfsScanNode.java.
-  const int parquet_count_star_slot_offset_;
+  /// applyCountStarOptimization() in ScanNode.java.
+  const int count_star_slot_offset_;
 
   // True if this is a partition key scan that needs only to return at least one row from
   // each scan range. If true, the scan node and scanner implementations should attempt
@@ -744,10 +783,6 @@ class HdfsScanNodeBase : public ScanNode {
   /// taken where there are i concurrent hdfs read thread running. Created in Open().
   std::vector<RuntimeProfile::Counter*>* hdfs_read_thread_concurrency_bucket_ = nullptr;
 
-  /// Pool for allocating some amounts of memory that is shared between scanners.
-  /// e.g. partition key tuple and their string buffers
-  boost::scoped_ptr<MemPool> scan_node_pool_;
-
   /// Status of failed operations.  This is set in the ScannerThreads
   /// Returned in GetNext() if an error occurred.  An non-ok status triggers cleanup
   /// scanner threads.
