@@ -102,6 +102,9 @@ bool HdfsScanner::EvalRuntimeFilter(int i, TupleRow* row) {
   const FilterContext* ctx = filter_ctxs_[i];
   ++stats->total_possible;
   if (stats->enabled_for_row && ctx->filter->HasFilter()) {
+    // Evaluating IN-list filter is much slower than evaluating the corresponding bloom
+    // filter. Skip it until we improve its performance.
+    if (ctx->filter->is_in_list_filter()) return true;
     ++stats->considered;
     if (!ctx->Eval(row)) {
       ++stats->rejected;
