@@ -20,6 +20,7 @@
 #include "codegen/codegen-anyval.h"
 #include "exec/base-sequence-scanner.h"
 #include "exec/exec-node.inline.h"
+#include "exec/file-metadata-utils.h"
 #include "exec/hdfs-scan-node.h"
 #include "exec/hdfs-scan-node-mt.h"
 #include "exec/read-write-util.h"
@@ -46,11 +47,11 @@ DEFINE_double(min_filter_reject_ratio, 0.1, "(Advanced) If the percentage of "
 
 const char* FieldLocation::LLVM_CLASS_NAME = "struct.impala::FieldLocation";
 const char* HdfsScanner::LLVM_CLASS_NAME = "class.impala::HdfsScanner";
-const int64_t HdfsScanner::FOOTER_SIZE;
 
 HdfsScanner::HdfsScanner(HdfsScanNodeBase* scan_node, RuntimeState* state)
     : scan_node_(scan_node),
       state_(state),
+      file_metadata_utils_(scan_node, state),
       expr_perm_pool_(new MemPool(scan_node->expr_mem_tracker())),
       template_tuple_pool_(new MemPool(scan_node->mem_tracker())),
       tuple_byte_size_(scan_node->tuple_desc()->byte_size()),
@@ -66,6 +67,7 @@ HdfsScanner::HdfsScanner(HdfsScanNodeBase* scan_node, RuntimeState* state)
 HdfsScanner::HdfsScanner()
     : scan_node_(nullptr),
       state_(nullptr),
+      file_metadata_utils_(nullptr, nullptr),
       tuple_byte_size_(0) {
   DCHECK(TestInfo::is_test());
 }
@@ -75,6 +77,7 @@ HdfsScanner::~HdfsScanner() {
 
 Status HdfsScanner::Open(ScannerContext* context) {
   context_ = context;
+  file_metadata_utils_.Open(context);
   stream_ = context->GetStream();
 
   // Clone the scan node's conjuncts map. The cloned evaluators must be closed by the
@@ -93,7 +96,7 @@ Status HdfsScanner::Open(ScannerContext* context) {
     for (auto& entry : *(scan_node_->thrift_dict_filter_conjuncts_map())) {
       SlotDescriptor* slot_desc = state_->desc_tbl().GetSlotDescriptor(entry.first);
       TupleId tuple_id = (slot_desc->type().IsCollectionType() ?
-          slot_desc->collection_item_descriptor()->id() :
+          slot_desc->children_tuple_descriptor()->id() :
           slot_desc->parent()->id());
       auto conjunct_evals_it = conjunct_evals_map_.find(tuple_id);
       DCHECK(conjunct_evals_it != conjunct_evals_map_.end());
@@ -109,14 +112,7 @@ Status HdfsScanner::Open(ScannerContext* context) {
     }
   }
 
-  // Initialize the template_tuple_, it is copied from the template tuple map in the
-  // HdfsScanNodeBase.
-  Tuple* template_tuple =
-      scan_node_->GetTemplateTupleForPartitionId(context_->partition_descriptor()->id());
-  if (template_tuple != nullptr) {
-    template_tuple_ =
-        template_tuple->DeepCopy(*scan_node_->tuple_desc(), template_tuple_pool_.get());
-  }
+  template_tuple_ = file_metadata_utils_.CreateTemplateTuple(template_tuple_pool_.get());
   template_tuple_map_[scan_node_->tuple_desc()] = template_tuple_;
 
   decompress_timer_ = ADD_TIMER(scan_node_->runtime_profile(), "DecompressionTime");
@@ -824,12 +820,14 @@ void HdfsScanner::CheckFiltersEffectiveness() {
 }
 
 Status HdfsScanner::IssueFooterRanges(HdfsScanNodeBase* scan_node,
-    const THdfsFileFormat::type& file_type, const vector<HdfsFileDesc*>& files) {
+    const THdfsFileFormat::type& file_type, const vector<HdfsFileDesc*>& files,
+    int64_t footer_size_estimate) {
   DCHECK(!files.empty());
+  DCHECK_LE(footer_size_estimate, READ_SIZE_MIN_VALUE);
   vector<ScanRange*> footer_ranges;
   for (int i = 0; i < files.size(); ++i) {
     // Compute the offset of the file footer.
-    int64_t footer_size = min(FOOTER_SIZE, files[i]->file_length);
+    int64_t footer_size = min(footer_size_estimate, files[i]->file_length);
     int64_t footer_start = files[i]->file_length - footer_size;
     DCHECK_GE(footer_start, 0);
 
@@ -840,12 +838,13 @@ Status HdfsScanner::IssueFooterRanges(HdfsScanNodeBase* scan_node,
       ScanRange* split = files[i]->splits[j];
 
       DCHECK_LE(split->offset() + split->len(), files[i]->file_length);
-      // If there are no materialized slots (such as count(*) over the table), we can
+      // If scan only reads file metadata (such as count(*) over the table), we can
       // get the result with the file metadata alone and don't need to read any row
       // groups. We only want a single node to process the file footer in this case,
       // which is the node with the footer split.  If it's not a count(*), we create a
       // footer range for the split always.
-      if (!scan_node->IsZeroSlotTableScan() || footer_split == split) {
+      if (!scan_node->ReadsFileMetadataOnly(files[i]->filename)
+          || footer_split == split) {
         ScanRangeMetadata* split_metadata =
             static_cast<ScanRangeMetadata*>(split->meta_data());
         // Each split is processed by first issuing a scan range for the file footer, which
