@@ -31,6 +31,7 @@
 #include "common/object-pool.h"
 #include "common/status.h"
 #include "exec/exec-node.inline.h"
+#include "exec/file-metadata-utils.h"
 #include "exec/hdfs-scan-node-base.h"
 #include "exec/scanner-context.h"
 #include "runtime/io/disk-io-mgr.h"
@@ -175,6 +176,9 @@ class HdfsScanner {
     return eos_;
   }
 
+  /// Return the plan id of the scan node that this scanner is associated with.
+  int GetScanNodeId() const { return scan_node_->id(); }
+
   /// Not inlined in IR so it can be replaced with a constant.
   int IR_NO_INLINE tuple_byte_size() const { return tuple_byte_size_; }
   int IR_NO_INLINE tuple_byte_size(const TupleDescriptor& desc) const {
@@ -193,6 +197,13 @@ class HdfsScanner {
     return -1;
   }
 
+  /// Return the runtime filter by index 'filter_idx'.
+  const RuntimeFilter* GetFilter(int filter_idx) {
+    DCHECK_GE(filter_idx, 0);
+    DCHECK_LT(filter_idx, filter_ctxs_.size());
+    return filter_ctxs_[filter_idx]->filter;
+  }
+
   /// Scanner subclasses must implement these static functions as well.  Unfortunately,
   /// c++ does not allow static virtual functions.
 
@@ -229,6 +240,9 @@ class HdfsScanner {
   /// Context for this scanner
   ScannerContext* context_ = nullptr;
 
+  /// Utility class for handling file metadata.
+  FileMetadataUtils file_metadata_utils_;
+
   /// Object pool for objects with same lifetime as scanner.
   ObjectPool obj_pool_;
 
@@ -371,14 +385,6 @@ class HdfsScanner {
   /// GetNext() API (i.e. mt_dop > 0), not the ProcessSplit() API.
   int64_t getnext_batches_returned_ = 0;
 
-  /// Size of the file footer for ORC and Parquet. This is a guess. If this value is too
-  /// little, we will need to issue another read.
-  static const int64_t FOOTER_SIZE = 1024 * 100;
-  static_assert(FOOTER_SIZE <= READ_SIZE_MIN_VALUE,
-      "FOOTER_SIZE can not be greater than READ_SIZE_MIN_VALUE.\n"
-      "You can increase FOOTER_SIZE if you want, "
-      "just don't forget to increase READ_SIZE_MIN_VALUE as well.");
-
   /// Check runtime filters' effectiveness every BATCHES_PER_FILTER_SELECTIVITY_CHECK
   /// row batches. Will update 'filter_stats_'.
   void CheckFiltersEffectiveness();
@@ -400,8 +406,8 @@ class HdfsScanner {
   /// Issue just the footer range for each file. This function is only used in parquet
   /// and orc scanners. We'll then parse the footer and pick out the columns we want.
   static Status IssueFooterRanges(HdfsScanNodeBase* scan_node,
-      const THdfsFileFormat::type& file_type, const std::vector<HdfsFileDesc*>& files)
-      WARN_UNUSED_RESULT;
+      const THdfsFileFormat::type& file_type, const std::vector<HdfsFileDesc*>& files,
+      int64_t footer_size_estimate) WARN_UNUSED_RESULT;
 
   /// Implements GetNext(). Should be overridden by subclasses.
   /// Only valid to call if the parent scan node is multi-threaded.
