@@ -17,20 +17,47 @@
 
 #include "exec/hdfs-sequence-scanner.h"
 
-#include "codegen/llvm-codegen.h"
+#include <string.h>
+#include <algorithm>
+#include <map>
+#include <memory>
+#include <ostream>
+#include <string>
+#include <utility>
+
+#include "common/compiler-util.h"
+#include "common/logging.h"
+#include "exec/delimited-text-parser.h"
 #include "exec/delimited-text-parser.inline.h"
-#include "exec/hdfs-scan-node.h"
+#include "exec/hdfs-scan-node-base.h"
+#include "exec/read-write-util.h"
+#include "exec/scanner-context.h"
 #include "exec/scanner-context.inline.h"
-#include "exec/text-converter.inline.h"
+#include "exec/text-converter.h"
+#include "gen-cpp/ErrorCodes_types.h"
 #include "runtime/descriptors.h"
+#include "runtime/fragment-state.h"
+#include "runtime/mem-pool.h"
+#include "runtime/row-batch.h"
 #include "runtime/runtime-state.h"
 #include "runtime/tuple.h"
-#include "runtime/tuple-row.h"
 #include "util/codec.h"
+#include "util/error-util.h"
 #include "util/runtime-profile-counters.h"
+#include "util/stopwatch.h"
 
 #include "common/names.h"
 
+namespace impala {
+class LlvmCodeGen;
+class ScalarExpr;
+class TupleRow;
+}
+
+namespace llvm {
+class Function;
+}
+
 using namespace impala;
 
 const char* const HdfsSequenceScanner::SEQFILE_VALUE_CLASS_NAME =
@@ -48,7 +75,7 @@ HdfsSequenceScanner::~HdfsSequenceScanner() {
 }
 
 // Codegen for materialized parsed data into tuples.
-Status HdfsSequenceScanner::Codegen(HdfsScanPlanNode* node, RuntimeState* state,
+Status HdfsSequenceScanner::Codegen(HdfsScanPlanNode* node, FragmentState* state,
     llvm::Function** write_aligned_tuples_fn) {
   *write_aligned_tuples_fn = nullptr;
   DCHECK(state->ShouldCodegen());
@@ -297,23 +324,18 @@ Status HdfsSequenceScanner::ProcessDecompressedBlock(RowBatch* row_batch) {
 
   // Need to copy out strings if they may reference the original I/O buffer.
   const bool copy_strings = !header_->is_compressed && !string_slot_offsets_.empty();
-  // Call jitted function if possible
-  int tuples_returned;
+
   if (write_tuples_fn_ != nullptr) {
     // HdfsScanner::InitializeWriteTuplesFn() will skip codegen if there are string slots
     // and escape characters. TextConverter::WriteSlot() will be used instead.
     DCHECK(scan_node_->tuple_desc()->string_slots().empty() ||
         delimited_text_parser_->escape_char() == '\0');
-    // last argument: seq always starts at record_location[0]
-    tuples_returned = write_tuples_fn_(this, row_batch->tuple_data_pool(), tuple_row,
-        field_locations_.data(), num_to_process,
-        max_added_tuples, scan_node_->materialized_slots().size(), 0, copy_strings);
-  } else {
-    tuples_returned = WriteAlignedTuples(row_batch->tuple_data_pool(), tuple_row,
-        field_locations_.data(), num_to_process,
-        max_added_tuples, scan_node_->materialized_slots().size(), 0, copy_strings);
   }
 
+  int tuples_returned = WriteAlignedTuplesCodegenOrInterpret(row_batch->tuple_data_pool(),
+      tuple_row, field_locations_.data(), num_to_process, max_added_tuples,
+      scan_node_->materialized_slots().size(), 0, copy_strings);
+
   if (tuples_returned == -1) return parse_status_;
   COUNTER_ADD(scan_node_->rows_read_counter(), num_to_process);
   RETURN_IF_ERROR(CommitRows(tuples_returned, row_batch));
