@@ -104,6 +104,10 @@ HdfsTableSink::HdfsTableSink(TDataSinkId sink_id, const HdfsTableSinkConfig& sin
   if (hdfs_sink.__isset.external_output_partition_depth) {
     external_output_partition_depth_ = hdfs_sink.external_output_partition_depth;
   }
+
+  if (hdfs_sink.__isset.parquet_bloom_filter_col_info) {
+    parquet_bloom_filter_columns_ = hdfs_sink.parquet_bloom_filter_col_info;
+  }
 }
 
 Status HdfsTableSink::Prepare(RuntimeState* state, MemTracker* parent_mem_tracker) {
@@ -135,9 +139,9 @@ Status HdfsTableSink::Prepare(RuntimeState* state, MemTracker* parent_mem_tracke
     }
   }
   // Sanity check.
-  DCHECK_LE(partition_key_expr_evals_.size(), table_desc_->num_cols())
-      << DebugString();
   if (!IsIceberg()) {
+    DCHECK_LE(partition_key_expr_evals_.size(), table_desc_->num_cols())
+        << DebugString();
     DCHECK_EQ(partition_key_expr_evals_.size(), table_desc_->num_clustering_cols())
         << DebugString();
   }
@@ -456,6 +460,8 @@ Status HdfsTableSink::CreateNewTmpFile(RuntimeState* state,
       IsABFSPath(tmp_hdfs_file_name_cstr) ||
       IsADLSPath(tmp_hdfs_file_name_cstr) ||
       IsGcsPath(tmp_hdfs_file_name_cstr) ||
+      IsCosPath(tmp_hdfs_file_name_cstr) ||
+      IsSFSPath(tmp_hdfs_file_name_cstr) ||
       IsOzonePath(tmp_hdfs_file_name_cstr)) {
     // On S3A, the file cannot be stat'ed until after it's closed, and even so, the block
     // size reported will be just the filesystem default. Similarly, the block size
@@ -494,7 +500,7 @@ Status HdfsTableSink::CreateNewTmpFile(RuntimeState* state,
 string HdfsTableSink::GetPartitionName(int i) {
   if (IsIceberg()) {
     DCHECK_LT(i, partition_key_expr_evals_.size());
-    return table_desc_->IcebergPartitionNames()[i];
+    return table_desc_->IcebergNonVoidPartitionNames()[i];
   } else {
     DCHECK_LT(i, table_desc_->num_clustering_cols());
     return table_desc_->col_descs()[i].name();
@@ -731,7 +737,8 @@ Status HdfsTableSink::FinalizePartitionFile(
     state->dml_exec_state()->UpdatePartition(
         partition->partition_name, partition->current_file_rows,
         &partition->writer->stats());
-    state->dml_exec_state()->AddCreatedFile(*partition);
+    state->dml_exec_state()->AddCreatedFile(*partition, IsIceberg(),
+        partition->writer->iceberg_file_stats());
   }
 
   RETURN_IF_ERROR(ClosePartitionFile(state, partition));
