@@ -17,7 +17,9 @@
 
 #include "exec/join-builder.h"
 
+#include "service/hs2-util.h"
 #include "util/debug-util.h"
+#include "util/min-max-filter.h"
 #include "util/runtime-profile-counters.h"
 
 #include "common/names.h"
@@ -124,4 +126,75 @@ void JoinBuilder::HandoffToProbesAndWait(RuntimeState* build_side_state) {
             << " cancelled=" << build_side_state->is_cancelled();
   }
 }
+
+void JoinBuilder::PublishRuntimeFilters(const std::vector<FilterContext>& filter_ctxs,
+    RuntimeState* runtime_state, float minmax_filter_threshold, int64_t num_build_rows) {
+  VLOG(3) << name() << " publishing "
+          << filter_ctxs.size() << " filters.";
+  int32_t num_enabled_filters = 0;
+  for (const FilterContext& ctx : filter_ctxs) {
+    BloomFilter* bloom_filter = nullptr;
+    if (ctx.local_bloom_filter != nullptr) {
+      bloom_filter = ctx.local_bloom_filter;
+      ++num_enabled_filters;
+    } else if (ctx.local_min_max_filter != nullptr) {
+      /// Apply the column min/max stats (if applicable) to shut down the min/max
+      /// filter early by setting always true flag for the filter. Do this only if
+      /// the min/max filter is too close in area to the column stats of all target
+      /// scan columns.
+      const TRuntimeFilterDesc& filter_desc = ctx.filter->filter_desc();
+      VLOG(3) << "Check out the usefulness of the local minmax filter:"
+              << " id=" << ctx.filter->id()
+              << ", filter details=" << ctx.local_min_max_filter->DebugString()
+              << ", column stats:"
+              << " low=" << PrintTColumnValue(filter_desc.targets[0].low_value)
+              << ", high=" << PrintTColumnValue(filter_desc.targets[0].high_value)
+              << ", threshold=" << minmax_filter_threshold
+              << ", #targets=" << filter_desc.targets.size();
+      bool all_overlap = true;
+      for (const auto& target_desc : filter_desc.targets) {
+        if (!FilterContext::ShouldRejectFilterBasedOnColumnStats(
+                target_desc, ctx.local_min_max_filter, minmax_filter_threshold)) {
+          all_overlap = false;
+          break;
+        }
+      }
+      if (all_overlap) {
+        ctx.local_min_max_filter->SetAlwaysTrue();
+        VLOG(3) << "The local minmax filter is set to always true:"
+                << " id=" << ctx.filter->id();
+      }
+
+      if (!ctx.local_min_max_filter->AlwaysTrue()) {
+        ++num_enabled_filters;
+      }
+    } else if (ctx.local_in_list_filter != nullptr) {
+      if (!ctx.local_in_list_filter->AlwaysTrue()) {
+        ++num_enabled_filters;
+      }
+    }
+
+    runtime_state->filter_bank()->UpdateFilterFromLocal(ctx.filter->id(),
+        bloom_filter, ctx.local_min_max_filter, ctx.local_in_list_filter);
+
+    if (ctx.local_min_max_filter != nullptr) {
+      VLOG(3) << name() << " published min/max filter: "
+              << " id=" << ctx.filter->id()
+              << ", details=" << ctx.local_min_max_filter->DebugString();
+    }
+  }
+
+  if (filter_ctxs.size() > 0) {
+    string info_string;
+    if (num_enabled_filters == filter_ctxs.size()) {
+      info_string = Substitute("$0 of $0 Runtime Filter$1 Published", filter_ctxs.size(),
+          filter_ctxs.size() == 1 ? "" : "s");
+    } else {
+      info_string = Substitute("$0 of $1 Runtime Filter$2 Published, $3 Disabled",
+          num_enabled_filters, filter_ctxs.size(), filter_ctxs.size() == 1 ? "" : "s",
+          filter_ctxs.size() - num_enabled_filters);
+    }
+    profile()->AddInfoString("Runtime filters", info_string);
+  }
+}
 } // namespace impala
