@@ -48,7 +48,22 @@ OrcColumnReader* OrcColumnReader::Create(const orc::Type* node,
   DCHECK(slot_desc != nullptr);
   OrcColumnReader* reader = nullptr;
   if (node->getKind() == orc::TypeKind::STRUCT) {
-    reader = new OrcStructReader(node, slot_desc, scanner);
+    DCHECK_EQ(scanner->slot_to_col_id_.count(slot_desc), 1);
+    const bool node_matches_slot_desc =
+        scanner->slot_to_col_id_[slot_desc] == node->getColumnId();
+    if (node_matches_slot_desc && slot_desc->type().IsStructType()) {
+      // 'node' corresponds to 'slot_desc' and 'slot_desc' refers to a struct: this is the
+      // case where we should materialize the struct and its children.
+      // Note: 'node' could also correspond to 'slot_desc' if 'slot_desc' refers to an
+      // array.
+      DCHECK(slot_desc->children_tuple_descriptor() != nullptr);
+      reader = new OrcStructReader(node, slot_desc,
+          slot_desc->children_tuple_descriptor(), scanner);
+    } else {
+      // A descendant of 'node' corresponds to 'slot_desc'. The struct referenced by
+      // 'node' will not be materialised.
+      reader = new OrcStructReader(node, slot_desc, scanner);
+    }
   } else if (node->getKind() == orc::TypeKind::LIST) {
     reader = new OrcListReader(node, slot_desc, scanner);
   } else if (node->getKind() == orc::TypeKind::MAP) {
@@ -174,7 +189,77 @@ Status OrcStringColumnReader::InitBlob(orc::DataBuffer<char>* blob, MemPool* poo
   return Status::OK();
 }
 
+Status OrcStringColumnReader::ReadValueBatch(int row_idx,
+    ScratchTupleBatch* scratch_batch, MemPool* pool, int scratch_batch_idx) {
+  switch (slot_desc_->type().type) {
+    case TYPE_STRING:
+      if (batch_->isEncoded) {
+        return ReadValueBatchInternal<TYPE_STRING, true>(
+            row_idx, scratch_batch, scratch_batch_idx);
+      }
+      return ReadValueBatchInternal<TYPE_STRING, false>(
+          row_idx, scratch_batch, scratch_batch_idx);
+    case TYPE_CHAR:
+      if (batch_->isEncoded) {
+        return ReadValueBatchInternal<TYPE_CHAR, true>(
+            row_idx, scratch_batch, scratch_batch_idx);
+      }
+      return ReadValueBatchInternal<TYPE_CHAR, false>(
+          row_idx, scratch_batch, scratch_batch_idx);
+    case TYPE_VARCHAR:
+      if (batch_->isEncoded) {
+        return ReadValueBatchInternal<TYPE_VARCHAR, true>(
+            row_idx, scratch_batch, scratch_batch_idx);
+      }
+      return ReadValueBatchInternal<TYPE_VARCHAR, false>(
+          row_idx, scratch_batch, scratch_batch_idx);
+    default:
+      return Status("Illegal string type: " + TypeToString(slot_desc_->type().type));
+  }
+}
+
 Status OrcStringColumnReader::ReadValue(int row_idx, Tuple* tuple, MemPool* pool) {
+  switch (slot_desc_->type().type) {
+    case TYPE_STRING:
+      if (batch_->isEncoded) {
+        return ReadValueInternal<TYPE_STRING, true>(row_idx, tuple);
+      }
+      return ReadValueInternal<TYPE_STRING, false>(row_idx, tuple);
+    case TYPE_CHAR:
+      if (batch_->isEncoded) {
+        return ReadValueInternal<TYPE_CHAR, true>(row_idx, tuple);
+      }
+      return ReadValueInternal<TYPE_CHAR, false>(row_idx, tuple);
+    case TYPE_VARCHAR:
+      if (batch_->isEncoded) {
+        return ReadValueInternal<TYPE_VARCHAR, true>(row_idx, tuple);
+      }
+      return ReadValueInternal<TYPE_VARCHAR, false>(row_idx, tuple);
+    default:
+      return Status("Illegal string type: " + TypeToString(slot_desc_->type().type));
+  }
+}
+
+template<PrimitiveType SLOT_TYPE, bool ENCODED>
+Status OrcStringColumnReader::ReadValueBatchInternal(int row_idx,
+    ScratchTupleBatch* scratch_batch, int scratch_batch_idx) {
+  int num_to_read = std::min<int>(scratch_batch->capacity - scratch_batch_idx,
+      NumElements() - row_idx);
+  DCHECK_LE(row_idx + num_to_read, NumElements());
+  for (int i = 0; i < num_to_read; ++i) {
+    int scratch_batch_pos = i + scratch_batch_idx;
+    uint8_t* next_tuple = scratch_batch->tuple_mem +
+        scratch_batch_pos * OrcColumnReader::scanner_->tuple_byte_size();
+    Tuple* tuple = reinterpret_cast<Tuple*>(next_tuple);
+    Status s = ReadValueInternal<SLOT_TYPE, ENCODED>(row_idx + i, tuple);
+    RETURN_IF_ERROR(s);
+  }
+  scratch_batch->num_tuples = scratch_batch_idx + num_to_read;
+  return Status::OK();
+}
+
+template<PrimitiveType SLOT_TYPE, bool ENCODED>
+Status OrcStringColumnReader::ReadValueInternal(int row_idx, Tuple* tuple) {
   if (IsNull(DCHECK_NOTNULL(batch_), row_idx)) {
     SetNullSlot(tuple);
     return Status::OK();
@@ -182,7 +267,8 @@ Status OrcStringColumnReader::ReadValue(int row_idx, Tuple* tuple, MemPool* pool
   char* src_ptr;
   int src_len;
 
-  if (batch_->isEncoded) {
+  if (ENCODED) {
+    DCHECK(batch_->isEncoded);
     orc::EncodedStringVectorBatch* currentBatch =
         static_cast<orc::EncodedStringVectorBatch*>(batch_);
 
@@ -200,7 +286,8 @@ Status OrcStringColumnReader::ReadValue(int row_idx, Tuple* tuple, MemPool* pool
     src_len = batch_->length[row_idx];
   }
   int dst_len = slot_desc_->type().len;
-  if (slot_desc_->type().type == TYPE_CHAR) {
+  if (SLOT_TYPE == TYPE_CHAR) {
+    DCHECK_EQ(slot_desc_->type().type, TYPE_CHAR);
     int unpadded_len = min(dst_len, static_cast<int>(src_len));
     char* dst_char = reinterpret_cast<char*>(GetSlot(tuple));
     memcpy(dst_char, src_ptr, unpadded_len);
@@ -208,7 +295,7 @@ Status OrcStringColumnReader::ReadValue(int row_idx, Tuple* tuple, MemPool* pool
     return Status::OK();
   }
   StringValue* dst = reinterpret_cast<StringValue*>(GetSlot(tuple));
-  if (slot_desc_->type().type == TYPE_VARCHAR && src_len > dst_len) {
+  if (SLOT_TYPE == TYPE_VARCHAR && src_len > dst_len) {
     dst->len = dst_len;
   } else {
     dst->len = src_len;
@@ -225,7 +312,7 @@ Status OrcTimestampReader::ReadValue(int row_idx, Tuple* tuple, MemPool* pool) {
   int64_t secs = batch_->data.data()[row_idx];
   int64_t nanos = batch_->nanoseconds.data()[row_idx];
   auto slot = reinterpret_cast<TimestampValue*>(GetSlot(tuple));
-  *slot = TimestampValue::FromUnixTimeNanos(secs, nanos, UTCPTR);
+  *slot = TimestampValue::FromUnixTimeNanos(secs, nanos, timezone_);
   if (UNLIKELY(!slot->HasDate())) {
     SetNullSlot(tuple);
     TErrorCode::type errorCode = TErrorCode::ORC_TIMESTAMP_OUT_OF_RANGE;
@@ -291,7 +378,7 @@ bool OrcStructReader::EndOfBatch() {
 inline uint64_t OrcComplexColumnReader::GetTargetColId(
     const SlotDescriptor* slot_desc) const {
   return slot_desc->type().IsCollectionType() ?
-         GetColId(slot_desc->collection_item_descriptor()):
+         GetColId(slot_desc->children_tuple_descriptor()):
          GetColId(slot_desc);
 }
 
@@ -381,6 +468,16 @@ OrcStructReader::OrcStructReader(const orc::Type* node,
   }
 }
 
+OrcStructReader::OrcStructReader(const orc::Type* node, const SlotDescriptor* slot_desc,
+    const TupleDescriptor* children_tuple, HdfsOrcScanner* scanner)
+    : OrcComplexColumnReader(node, slot_desc, scanner) {
+  tuple_desc_ = children_tuple;
+  materialize_tuple_ = true;
+  for (SlotDescriptor* child_slot : tuple_desc_->slots()) {
+    CreateChildForSlot(node, child_slot);
+  }
+}
+
 OrcStructReader::OrcStructReader(const orc::Type* node,
     const SlotDescriptor* slot_desc, HdfsOrcScanner* scanner)
     : OrcComplexColumnReader(node, slot_desc, scanner) {
@@ -399,7 +496,7 @@ Status OrcStructReader::ReadValue(int row_idx, Tuple* tuple, MemPool* pool) {
     return child->ReadValue(row_idx, tuple, pool);
   }
   if (IsNull(DCHECK_NOTNULL(batch_), row_idx)) {
-    for (OrcColumnReader* child : children_) child->SetNullSlot(tuple);
+    SetNullSlot(tuple);
     return Status::OK();
   }
   for (OrcColumnReader* child : children_) {
@@ -433,18 +530,7 @@ Status OrcStructReader::TopLevelReadValueBatch(ScratchTupleBatch* scratch_batch,
   }
   int num_rows_read = scratch_batch->num_tuples - scratch_batch_idx;
   if (children_.empty()) {
-    // We allow empty 'children_' for original files, because we might select the
-    // synthetic 'rowid' field which is not present in original files.
-    // We also allow empty 'children_' when we need to validate row batches of a zero slot
-    // scan. In that case 'children_' is empty and only 'row_validator_' owns an ORC
-    // vector batch (the write id batch).
-    bool valid_empty_children = scanner_->acid_original_file_ ||
-         (scanner_->row_batches_need_validation_ &&
-          scanner_->scan_node_->IsZeroSlotTableScan());
-    if (!valid_empty_children) {
-      return Status(Substitute("Parse error in possibly corrupt ORC file: '$0'",
-          scanner_->filename()));
-    }
+    // We allow empty 'children_' in all cases.
     DCHECK_EQ(0, num_rows_read);
     num_rows_read = std::min(scratch_batch->capacity - scratch_batch->num_tuples,
                              NumElements() - row_idx_);
@@ -472,6 +558,10 @@ void OrcStructReader::FillSyntheticRowId(ScratchTupleBatch* scratch_batch,
 
 Status OrcStructReader::ReadValueBatch(int row_idx, ScratchTupleBatch* scratch_batch,
     MemPool* pool, int scratch_batch_idx) {
+  if (materialize_tuple_) {
+    return OrcBatchedReader::ReadValueBatch(row_idx, scratch_batch, pool,
+        scratch_batch_idx);
+  }
   for (OrcColumnReader* child : children_) {
     RETURN_IF_ERROR(
         child->ReadValueBatch(row_idx, scratch_batch, pool, scratch_batch_idx));
@@ -515,7 +605,7 @@ OrcCollectionReader::OrcCollectionReader(const orc::Type* node,
     // This is a collection SlotDescriptor whose item TupleDescriptor matches
     // 'node'. We should materialize the slot (creating a CollectionValue) and its
     // collection tuples (see more in HdfsOrcScanner::AssembleCollection).
-    tuple_desc_ = slot_desc->collection_item_descriptor();
+    tuple_desc_ = slot_desc->children_tuple_descriptor();
     materialize_tuple_ = true;
   }
 }
@@ -527,7 +617,9 @@ Status OrcCollectionReader::AssembleCollection(int row_idx, Tuple* tuple, MemPoo
   }
   auto coll_slot = reinterpret_cast<CollectionValue*>(GetSlot(tuple));
   *coll_slot = CollectionValue();
-  const TupleDescriptor* tuple_desc = slot_desc_->collection_item_descriptor();
+  const TupleDescriptor* tuple_desc = slot_desc_->children_tuple_descriptor();
+  DCHECK(tuple_desc != nullptr) << "There is no children tuple for slot ID: " <<
+      slot_desc_->id();
   CollectionValueBuilder builder(coll_slot, *tuple_desc, pool, scanner_->state_);
   return scanner_->AssembleCollection(*this, row_idx, &builder);
 }
@@ -605,7 +697,7 @@ void OrcListReader::CreateChildForSlot(const orc::Type* node,
   // We have a position slot descriptor if it refers to this LIST ORC type, but it isn't
   // a collection slot.
   bool is_pos_slot = slot_col_id == node->getColumnId() &&
-                     slot_desc->collection_item_descriptor() == nullptr;
+                     slot_desc->children_tuple_descriptor() == nullptr;
   if (is_pos_slot) {
     DCHECK(pos_slot_desc_ == nullptr) << "Should have unique pos slot";
     pos_slot_desc_ = slot_desc;
