@@ -69,7 +69,7 @@ class OrcColumnReader {
  public:
   /// Create a column reader for the given 'slot_desc' based on the ORC 'node'. We say
   /// the 'slot_desc' and ORC 'node' match iff
-  ///     scanner->col_id_path_map_[node->getColumnId()] == slot_desc->col_path
+  ///     scanner->slot_to_col_id_[slot_desc] == node->getColumnId()
   /// Caller should guarantee that 'slot_desc' matches to ORC 'node' or one of its
   /// descendants. If 'node' is a primitive type, 'slot_desc' should match it since
   /// primitive types don't have descendants.
@@ -337,8 +337,8 @@ class OrcStringColumnReader : public OrcPrimitiveColumnReader<OrcStringColumnRea
     }
     DCHECK(static_cast<orc::EncodedStringVectorBatch*>(batch_) ==
         dynamic_cast<orc::EncodedStringVectorBatch*>(orc_batch));
-    if (last_stripe_idx_ != scanner_->stripe_idx_) {
-      last_stripe_idx_ = scanner_->stripe_idx_;
+    if (last_stripe_idx_ != scanner_->group_idx_) {
+      last_stripe_idx_ = scanner_->group_idx_;
       auto current_batch = static_cast<orc::EncodedStringVectorBatch*>(batch_);
       return InitBlob(&current_batch->dictionary->dictionaryBlob,
           scanner_->dictionary_pool_.get());
@@ -346,11 +346,30 @@ class OrcStringColumnReader : public OrcPrimitiveColumnReader<OrcStringColumnRea
     return Status::OK();
   }
 
-  Status ReadValue(int row_idx, Tuple* tuple, MemPool* pool) final WARN_UNUSED_RESULT;
+  /// Overrides the method to invoke templated ReadValueBatchInternal().
+  Status ReadValueBatch(int row_idx, ScratchTupleBatch* scratch_batch,
+      MemPool* pool, int scratch_batch_idx) override WARN_UNUSED_RESULT;
+
+  /// Overrides ReadValue() to invoke templated ReadValueBatchInternal().
+  /// It's only used in materializing collection (i.e. list/map) values.
+  Status ReadValue(int row_idx, Tuple* tuple, MemPool* pool) override WARN_UNUSED_RESULT;
 
  private:
   friend class OrcPrimitiveColumnReader<OrcStringColumnReader>;
 
+  /// Template implementation for ReadValueBatch() to avoid checks inside the inner loop.
+  /// When 'ENCODED' is true, the orc string vector batch is expected to be dictionary
+  /// encoded.
+  template<PrimitiveType SLOT_TYPE, bool ENCODED>
+  Status ReadValueBatchInternal(int row_idx, ScratchTupleBatch* scratch_batch,
+      int scratch_batch_idx) WARN_UNUSED_RESULT;
+
+  /// Template implementation for ReadValue(). We use this method instead of ReadValue()
+  /// in ReadValueBatch() so we can check types out of the loop.
+  template<PrimitiveType SLOT_TYPE, bool ENCODED>
+  ALWAYS_INLINE inline Status ReadValueInternal(int row_idx, Tuple* tuple)
+      WARN_UNUSED_RESULT;
+
   orc::StringVectorBatch* batch_ = nullptr;
   // We copy the blob from the batch, so the memory will be handled by Impala, and not
   // by the ORC lib.
@@ -370,7 +389,11 @@ class OrcTimestampReader : public OrcPrimitiveColumnReader<OrcTimestampReader> {
  public:
   OrcTimestampReader(const orc::Type* node, const SlotDescriptor* slot_desc,
       HdfsOrcScanner* scanner)
-      : OrcPrimitiveColumnReader<OrcTimestampReader>(node, slot_desc, scanner) { }
+      : OrcPrimitiveColumnReader<OrcTimestampReader>(node, slot_desc, scanner) {
+    if (node->getKind() == orc::TIMESTAMP_INSTANT) {
+      timezone_ = scanner_->state_->local_time_zone();
+    }
+  }
 
   virtual ~OrcTimestampReader() { }
 
@@ -386,6 +409,7 @@ class OrcTimestampReader : public OrcPrimitiveColumnReader<OrcTimestampReader> {
   friend class OrcPrimitiveColumnReader<OrcTimestampReader>;
 
   orc::TimestampVectorBatch* batch_ = nullptr;
+  const Timezone* timezone_ = UTCPTR;
 };
 
 class OrcDateColumnReader : public OrcPrimitiveColumnReader<OrcDateColumnReader> {
@@ -474,9 +498,8 @@ class OrcDecimal16ColumnReader
 /// sub queries). The root reader is always an OrcStructReader since the root of the ORC
 /// schema is represented as a STRUCT type.
 ///
-/// For collection readers, they can be divided into two kinds by whether they should
-/// materialize collection tuples (reflected by materialize_tuple_). (STRUCTs always
-/// delegate materialization to their children.)
+/// For complex readers, they can be divided into two kinds by whether they should
+/// materialize their tuples (reflected by materialize_tuple_).
 ///
 /// For collection type readers that materialize a CollectionValue they create a
 /// CollectionValueBuilder when 'ReadValue' is called. Then recursively delegate the
@@ -564,6 +587,13 @@ class OrcStructReader : public OrcComplexColumnReader {
   OrcStructReader(const orc::Type* node, const TupleDescriptor* table_tuple_desc,
       HdfsOrcScanner* scanner);
 
+  /// Constructor for a slot that materializes all it's children. E.g. when a struct is
+  /// given in the select list.
+  OrcStructReader(const orc::Type* node, const SlotDescriptor* slot_desc,
+      const TupleDescriptor* children_tuple, HdfsOrcScanner* scanner);
+
+  /// Constructor for a struct that is not mapped directly to a slot instead it refers to
+  /// a descendant column.
   OrcStructReader(const orc::Type* node, const SlotDescriptor* slot_desc,
       HdfsOrcScanner* scanner);
 
@@ -589,7 +619,9 @@ class OrcStructReader : public OrcComplexColumnReader {
   int GetChildBatchOffset(int row_idx) const override { return row_idx; }
 
   int NumElements() const final {
-    if (MaterializeTuple()) return vbatch_->numElements;
+    if (MaterializeTuple()) {
+      return vbatch_ ? vbatch_->numElements : 0;
+    }
     DCHECK_EQ(children().size(), 1);
     OrcColumnReader* child = children()[0];
     return child->NumElements();
@@ -618,6 +650,7 @@ class OrcStructReader : public OrcComplexColumnReader {
 
   void SetNullSlot(Tuple* tuple) override {
     for (OrcColumnReader* child : children_) child->SetNullSlot(tuple);
+    tuple->SetNull(DCHECK_NOTNULL(slot_desc_)->null_indicator_offset());
   }
 
   void CreateChildForSlot(const orc::Type* curr_node, const SlotDescriptor* slot_desc);
