@@ -17,12 +17,14 @@
 
 #include "exec/parquet/hdfs-parquet-table-writer.h"
 
+#include <boost/algorithm/string.hpp>
 #include <boost/unordered_set.hpp>
 
 #include "common/version.h"
 #include "exec/hdfs-table-sink.h"
 #include "exec/parquet/parquet-column-stats.inline.h"
 #include "exec/parquet/parquet-metadata-utils.h"
+#include "exec/parquet/parquet-bloom-filter-util.h"
 #include "exprs/scalar-expr-evaluator.h"
 #include "exprs/scalar-expr.h"
 #include "rpc/thrift-util.h"
@@ -32,6 +34,7 @@
 #include "runtime/raw-value.h"
 #include "runtime/row-batch.h"
 #include "runtime/runtime-state.h"
+#include "runtime/scoped-buffer.h"
 #include "runtime/string-value.inline.h"
 #include "util/bit-stream-utils.h"
 #include "util/bit-util.h"
@@ -40,11 +43,13 @@
 #include "util/debug-util.h"
 #include "util/dict-encoding.h"
 #include "util/hdfs-util.h"
+#include "util/parquet-bloom-filter.h"
 #include "util/pretty-printer.h"
 #include "util/rle-encoding.h"
 #include "util/string-util.h"
 
 #include <sstream>
+#include <string>
 
 #include "gen-cpp/ImpalaService_types.h"
 
@@ -119,7 +124,7 @@ class HdfsParquetTableWriter::BaseColumnWriter {
     : parent_(parent),
       expr_eval_(expr_eval),
       codec_info_(codec_info),
-      page_size_(DEFAULT_DATA_PAGE_SIZE),
+      plain_page_size_(parent->default_plain_page_size()),
       current_page_(nullptr),
       num_values_(0),
       total_compressed_byte_size_(0),
@@ -307,6 +312,12 @@ class HdfsParquetTableWriter::BaseColumnWriter {
   // the expression to an int64.
   virtual void* ConvertValue(void* value) { return value; }
 
+  // Some subclasses may write a ParquetBloomFilter, in which case they should override
+  // this method.
+  virtual const ParquetBloomFilter* GetParquetBloomFilter() const {
+    return nullptr;
+  }
+
   // Encodes out all data for the current page and updates the metadata.
   virtual Status FinalizeCurrentPage() WARN_UNUSED_RESULT;
 
@@ -345,12 +356,14 @@ class HdfsParquetTableWriter::BaseColumnWriter {
   // compressed.
   scoped_ptr<Codec> compressor_;
 
-  // Size of newly created pages. Defaults to DEFAULT_DATA_PAGE_SIZE and is increased
-  // when pages are not big enough. This only happens when there are enough unique values
-  // such that we switch from PLAIN_DICTIONARY/RLE_DICTIONARY to PLAIN encoding and then
-  // have very large values (i.e. greater than DEFAULT_DATA_PAGE_SIZE).
+  // Size of newly created PLAIN encoded pages. Defaults to DEFAULT_DATA_PAGE_SIZE or to
+  // the value of 'write.parquet.page-size-bytes' table property for Iceberg tables.
+  // Its value is increased when pages are not big enough. This only happens when there
+  // are enough unique values such that we switch from PLAIN_DICTIONARY/RLE_DICTIONARY to
+  // PLAIN encoding and then have very large values (i.e. greater than
+  // DEFAULT_DATA_PAGE_SIZE).
   // TODO: Consider removing and only creating a single large page as necessary.
-  int64_t page_size_;
+  int64_t plain_page_size_;
 
   // Pages belong to this column chunk. We need to keep them in memory in order to write
   // them together.
@@ -430,9 +443,34 @@ class HdfsParquetTableWriter::ColumnWriter :
       const Codec::CodecInfo& codec_info, const std::string& col_name)
     : BaseColumnWriter(parent, eval, codec_info, col_name),
       num_values_since_dict_size_check_(0),
+      parquet_bloom_filter_bytes_(0),
+      parquet_bloom_filter_buffer_(table_sink_mem_tracker_),
       plain_encoded_value_size_(
           ParquetPlainEncoder::EncodedByteSize(eval->root().type())) {
     DCHECK_NE(eval->root().type().type, TYPE_BOOLEAN);
+
+    const std::map<string, int64_t>& col_to_size =
+      parent->parent_->GetParquetBloomFilterColumns();
+    const auto it = col_to_size.find(column_name());
+
+    if (GetBloomFilterWriteOption() == TParquetBloomFilterWrite::NEVER
+        || it == col_to_size.end()) {
+      // Parquet Bloom filtering is disabled altogether or is not turned on for this
+      // column.
+      parquet_bloom_filter_state_ = ParquetBloomFilterState::DISABLED;
+    } else {
+      // Parquet Bloom filtering is enabled for this column either immediately or if
+      // falling back from dict encoding.
+      parquet_bloom_filter_state_ = ParquetBloomFilterState::UNINITIALIZED;
+
+      // It is the responsibility of the FE to enforce the below constraints.
+      parquet_bloom_filter_bytes_ = it->second;
+      DCHECK_LE(parquet_bloom_filter_bytes_, ParquetBloomFilter::MAX_BYTES);
+      DCHECK_GE(parquet_bloom_filter_bytes_, ParquetBloomFilter::MIN_BYTES);
+      DCHECK(BitUtil::IsPowerOf2(parquet_bloom_filter_bytes_));
+    }
+    parquet_type_ = ParquetMetadataUtils::ConvertInternalToParquetType(type().type,
+        parent_->timestamp_type_);
   }
 
   virtual void Reset() {
@@ -450,6 +488,16 @@ class HdfsParquetTableWriter::ColumnWriter :
     page_stats_base_ = page_stats_.get();
     row_group_stats_.reset(
         new ColumnStats<T>(parent_->per_file_mem_pool_.get(), plain_encoded_value_size_));
+    if (parquet_bloom_filter_state_ != ParquetBloomFilterState::DISABLED) {
+      Status status = InitParquetBloomFilter();
+      if (!status.ok()) {
+        VLOG(google::WARNING)
+            << "Failed to initialise Parquet Bloom filter for column "
+            << column_name() << "."
+            << " Error message: " << status.msg().msg();
+        ReleaseParquetBloomFilterResources();
+      }
+    }
     row_group_stats_base_ = row_group_stats_.get();
   }
 
@@ -460,13 +508,16 @@ class HdfsParquetTableWriter::ColumnWriter :
       if (UNLIKELY(num_values_since_dict_size_check_ >=
                    DICTIONARY_DATA_PAGE_SIZE_CHECK_PERIOD)) {
         num_values_since_dict_size_check_ = 0;
-        if (dict_encoder_->EstimatedDataEncodedSize() >= page_size_) return false;
+        if (dict_encoder_->EstimatedDataEncodedSize() >= parent_->dict_page_size()) {
+          return false;
+        }
       }
       ++num_values_since_dict_size_check_;
       *bytes_needed = dict_encoder_->Put(*val);
       // If the dictionary contains the maximum number of values, switch to plain
       // encoding for the next page. The current page is full and must be written out.
       if (UNLIKELY(*bytes_needed < 0)) {
+        FlushDictionaryToParquetBloomFilterIfNeeded();
         next_page_encoding_ = parquet::Encoding::PLAIN;
         return false;
       }
@@ -475,7 +526,8 @@ class HdfsParquetTableWriter::ColumnWriter :
       *bytes_needed = plain_encoded_value_size_ < 0 ?
           ParquetPlainEncoder::ByteSize<T>(*val) :
           plain_encoded_value_size_;
-      if (current_page_->header.uncompressed_page_size + *bytes_needed > page_size_) {
+      if (current_page_->header.uncompressed_page_size + *bytes_needed >
+          plain_page_size_) {
         return false;
       }
       uint8_t* dst_ptr = values_buffer_ + current_page_->header.uncompressed_page_size;
@@ -497,9 +549,15 @@ class HdfsParquetTableWriter::ColumnWriter :
     }
 
     page_stats_->Update(*val);
+    UpdateParquetBloomFilterIfNeeded(val);
+
     return true;
   }
 
+  virtual const ParquetBloomFilter* GetParquetBloomFilter() const {
+    return parquet_bloom_filter_.get();
+  }
+
  private:
   // The period, in # of rows, to check the estimated dictionary page size against
   // the data page size. We want to start a new data page when the estimated size
@@ -525,10 +583,163 @@ class HdfsParquetTableWriter::ColumnWriter :
   // Tracks statistics per row group. This gets reset when starting a new row group.
   scoped_ptr<ColumnStats<T>> row_group_stats_;
 
+
+  enum struct ParquetBloomFilterState {
+    /// Parquet Bloom filtering is turned off either completely or for this column.
+    DISABLED,
+
+    /// The Parquet Bloom filter needs to be initialised before being used.
+    UNINITIALIZED,
+
+    /// The Bloom filter has been initialised but it is not being used as the dictionary
+    /// can hold all elements. If the dictionary becomes full and there are still new
+    /// elements, we fall back from dictionary encoding to plain encoding and start using
+    /// the Bloom filter.
+    WAIT_FOR_FALLBACK_FROM_DICT,
+
+    /// The Parquet Bloom filter is being used.
+    ENABLED,
+
+    /// An error occured with the Parquet Bloom filter so we are not using it.
+    FAILED
+  };
+
+  ParquetBloomFilterState parquet_bloom_filter_state_;
+  uint64_t parquet_bloom_filter_bytes_;
+  ScopedBuffer parquet_bloom_filter_buffer_;
+
+  // The parquet type corresponding to this->type(). Needed by the Parquet Bloom filter.
+  parquet::Type::type parquet_type_;
+
+  // Buffer used when converting values to the form that is used for hashing and insertion
+  // into the ParquetBloomFilter. The conversion function, 'BytesToParquetType' requires a
+  // vector to be able to allocate space if necessary. However, by caching the allocated
+  // buffer here we avoid the overhead of allocation for every conversion - when
+  // 'BytesToParquetType' calls 'resize' on the vector it will already have at least the
+  // desired length in most (or all) cases.
+  //
+  // We prellocate 16 bytes because that is the longest fixed size type (except for fixed
+  // length arrays).
+  std::vector<uint8_t> parquet_bloom_conversion_buffer{16, 0};
+
+  // The ParquetBloomFilter object if one is being written. If
+  // 'ShouldInitParquetBloomFilter()' is false, the combination of the impala type and the
+  // parquet type is not supported or some error occurs during the initialisation of the
+  // ParquetBloomFilter object, it is set to NULL.
+  unique_ptr<ParquetBloomFilter> parquet_bloom_filter_;
+
   // Converts a slot pointer to a raw value suitable for encoding
   inline T* CastValue(void* value) {
     return reinterpret_cast<T*>(value);
   }
+
+  Status InitParquetBloomFilter() WARN_UNUSED_RESULT {
+    DCHECK(parquet_bloom_filter_state_ != ParquetBloomFilterState::DISABLED);
+    const ColumnType& impala_type = type();
+    if (!IsParquetBloomFilterSupported(parquet_type_, impala_type)) {
+      stringstream ss;
+      ss << "Parquet Bloom filtering not supported for parquet type " << parquet_type_
+          << " and impala type " << impala_type << ".";
+      return Status::Expected(ss.str());
+    }
+
+    parquet_bloom_filter_buffer_.Release();
+    if (!parquet_bloom_filter_buffer_.TryAllocate(parquet_bloom_filter_bytes_)) {
+      parquet_bloom_filter_state_ = ParquetBloomFilterState::FAILED;
+      return Status(Substitute("Could not allocate buffer of $0 bytes for Parquet "
+            "Bloom filter data when writing column '$1'.",
+            parquet_bloom_filter_bytes_, column_name()));
+    }
+    std::memset(parquet_bloom_filter_buffer_.buffer(), 0,
+        parquet_bloom_filter_buffer_.Size());
+
+    parquet_bloom_filter_ = make_unique<ParquetBloomFilter>();
+    Status status = parquet_bloom_filter_->Init(parquet_bloom_filter_buffer_.buffer(),
+          parquet_bloom_filter_buffer_.Size(), true);
+
+    if (!status.ok()) {
+      parquet_bloom_filter_state_ = ParquetBloomFilterState::FAILED;
+      return status;
+    } else {
+      const bool should_wait_for_fallback =
+          (GetBloomFilterWriteOption() == TParquetBloomFilterWrite::IF_NO_DICT)
+          && IsDictionaryEncoding(current_encoding_);
+      parquet_bloom_filter_state_ = should_wait_for_fallback
+          ? ParquetBloomFilterState::WAIT_FOR_FALLBACK_FROM_DICT
+          : ParquetBloomFilterState::ENABLED;;
+      return Status::OK();
+    }
+  }
+
+  void UpdateParquetBloomFilterIfNeeded(const void* val) {
+    if (parquet_bloom_filter_state_ == ParquetBloomFilterState::ENABLED) {
+      Status status = UpdateParquetBloomFilter(val);
+      if (!status.ok()) {
+        // If an error happens, for example conversion to the form expected by the Bloom
+        // filter fails, we stop writing the Bloom filter and release resources associated
+        // with it.
+        VLOG(google::WARNING)
+            << "An error happened updating Parquet Bloom filter in column "
+            << column_name() << " at row idx " << parent_->row_idx_ << "."
+            << " Error message: " << status.msg().msg();
+        parquet_bloom_filter_state_ = ParquetBloomFilterState::FAILED;
+        ReleaseParquetBloomFilterResources();
+      }
+    }
+  }
+
+  Status UpdateParquetBloomFilter(const void* val) WARN_UNUSED_RESULT {
+    DCHECK(parquet_bloom_filter_state_ == ParquetBloomFilterState::ENABLED);
+    DCHECK(parquet_bloom_filter_ != nullptr);
+
+    uint8_t* ptr = nullptr;
+    size_t len = -1;
+    const ColumnType& impala_type = type();
+    RETURN_IF_ERROR(BytesToParquetType(val, impala_type, parquet_type_,
+        &parquet_bloom_conversion_buffer, &ptr, &len));
+    DCHECK(ptr != nullptr);
+    DCHECK(len != -1);
+    parquet_bloom_filter_->HashAndInsert(ptr, len);
+
+    return Status::OK();
+  }
+
+  void ReleaseParquetBloomFilterResources() {
+    parquet_bloom_filter_ = nullptr;
+    parquet_bloom_filter_buffer_.Release();
+  }
+
+  void FlushDictionaryToParquetBloomFilterIfNeeded() {
+    if (parquet_bloom_filter_state_
+        == ParquetBloomFilterState::WAIT_FOR_FALLBACK_FROM_DICT) {
+      parquet_bloom_filter_state_ = ParquetBloomFilterState::ENABLED;
+
+      // Write dictionary keys to Parquet Bloom filter if we haven't been filling it so
+      // far (and Bloom filtering is enabled). If there are too many values for a
+      // dictionary, a Bloom filter may still be useful.
+      Status status = DictKeysToParquetBloomFilter();
+      if (!status.ok()) {
+        VLOG(google::WARNING)
+            << "Failed to add dictionary keys to Parquet Bloom filter for column "
+            << column_name()
+            << " when falling back from dictionary encoding to plain encoding."
+            << " Error message: " << status.msg().msg();
+        parquet_bloom_filter_state_ = ParquetBloomFilterState::FAILED;
+        ReleaseParquetBloomFilterResources();
+      }
+    }
+  }
+
+  Status DictKeysToParquetBloomFilter() {
+    return dict_encoder_->ForEachDictKey([this](const T& value) {
+        return UpdateParquetBloomFilter(&value);
+        });
+  }
+
+  TParquetBloomFilterWrite::type GetBloomFilterWriteOption() {
+    return parent_->state_->query_options().parquet_bloom_filter_write;
+  }
+
  protected:
   // Size of each encoded value in plain encoding. -1 if the type is variable-length.
   int64_t plain_encoded_value_size_;
@@ -711,7 +922,7 @@ inline Status HdfsParquetTableWriter::BaseColumnWriter::AppendRow(TupleRow* row)
 
     // Check how much space is needed to write this value. If that is larger than the
     // page size then increase page size and try again.
-    if (UNLIKELY(bytes_needed > page_size_)) {
+    if (UNLIKELY(bytes_needed > plain_page_size_)) {
       if (bytes_needed > MAX_DATA_PAGE_SIZE) {
         stringstream ss;
         ss << "Cannot write value of size "
@@ -720,8 +931,8 @@ inline Status HdfsParquetTableWriter::BaseColumnWriter::AppendRow(TupleRow* row)
            << PrettyPrinter::Print(MAX_DATA_PAGE_SIZE , TUnit::BYTES) << ".";
         return Status(ss.str());
       }
-      page_size_ = bytes_needed;
-      values_buffer_len_ = page_size_;
+      plain_page_size_ = bytes_needed;
+      values_buffer_len_ = plain_page_size_;
       values_buffer_ = parent_->reusable_col_mem_pool_->Allocate(values_buffer_len_);
     }
     NewPage();
@@ -999,25 +1210,72 @@ HdfsParquetTableWriter::HdfsParquetTableWriter(HdfsTableSink* parent, RuntimeSta
     file_size_limit_(0),
     reusable_col_mem_pool_(new MemPool(parent_->mem_tracker())),
     per_file_mem_pool_(new MemPool(parent_->mem_tracker())),
-    row_idx_(0) {
+    row_idx_(0),
+    default_block_size_(0),
+    default_plain_page_size_(0),
+    dict_page_size_(0) {
   is_iceberg_file_ = table_desc->IsIcebergTable();
 }
 
 HdfsParquetTableWriter::~HdfsParquetTableWriter() {
 }
 
-void HdfsParquetTableWriter::ConfigureTimestampType() {
-  if (is_iceberg_file_) {
-    // The Iceberg spec states that timestamps are stored as INT64 micros.
-    timestamp_type_ = TParquetTimestampType::INT64_MICROS;
-    return;
-  }
+void HdfsParquetTableWriter::Configure(int num_cols) {
+  DCHECK(!is_iceberg_file_);
+
   timestamp_type_ = state_->query_options().parquet_timestamp_type;
+
+  string_utf8_ = state_->query_options().parquet_annotate_strings_utf8;
+
+  if (state_->query_options().__isset.parquet_file_size &&
+      state_->query_options().parquet_file_size > 0) {
+    // If the user specified a value explicitly, use it. InitNewFile() will verify that
+    // the actual file's block size is sufficient.
+    default_block_size_ = state_->query_options().parquet_file_size;
+  } else {
+    default_block_size_ = HDFS_BLOCK_SIZE;
+    // Blocks are usually HDFS_BLOCK_SIZE bytes, unless there are many columns, in
+    // which case a per-column minimum kicks in.
+    default_block_size_ = max(default_block_size_, MinBlockSize(num_cols));
+  }
+  // HDFS does not like block sizes that are not aligned
+  default_block_size_ = BitUtil::RoundUp(default_block_size_, HDFS_BLOCK_ALIGNMENT);
+
+  default_plain_page_size_ = DEFAULT_DATA_PAGE_SIZE;
+  dict_page_size_ = DEFAULT_DATA_PAGE_SIZE;
 }
 
-void HdfsParquetTableWriter::ConfigureStringType() {
-  string_utf8_ = is_iceberg_file_ ||
-                 state_->query_options().parquet_annotate_strings_utf8;
+void HdfsParquetTableWriter::ConfigureForIceberg(int num_cols) {
+  DCHECK(is_iceberg_file_);
+
+  // The Iceberg spec states that timestamps are stored as INT64 micros.
+  timestamp_type_ = TParquetTimestampType::INT64_MICROS;
+
+  string_utf8_ = true;
+
+  if (state_->query_options().__isset.parquet_file_size &&
+      state_->query_options().parquet_file_size > 0) {
+    // If the user specified a value explicitly, use it. InitNewFile() will verify that
+    // the actual file's block size is sufficient.
+    default_block_size_ = state_->query_options().parquet_file_size;
+  } else if (table_desc_->IcebergParquetRowGroupSize() > 0) {
+    // If the user specified a value explicitly, use it. InitNewFile() will verify that
+    // the actual file's block size is sufficient.
+    default_block_size_ = table_desc_->IcebergParquetRowGroupSize();
+  } else {
+    default_block_size_ = HDFS_BLOCK_SIZE;
+    // Blocks are usually HDFS_BLOCK_SIZE bytes, unless there are many columns, in
+    // which case a per-column minimum kicks in.
+    default_block_size_ = max(default_block_size_, MinBlockSize(num_cols));
+  }
+  // HDFS does not like block sizes that are not aligned
+  default_block_size_ = BitUtil::RoundUp(default_block_size_, HDFS_BLOCK_ALIGNMENT);
+
+  default_plain_page_size_ = table_desc_->IcebergParquetPlainPageSize();
+  if (default_plain_page_size_ <= 0) default_plain_page_size_ = DEFAULT_DATA_PAGE_SIZE;
+
+  dict_page_size_ = table_desc_->IcebergParquetDictPageSize();
+  if (dict_page_size_ <= 0) dict_page_size_ = DEFAULT_DATA_PAGE_SIZE;
 }
 
 Status HdfsParquetTableWriter::Init() {
@@ -1037,6 +1295,12 @@ Status HdfsParquetTableWriter::Init() {
   if (query_options.__isset.compression_codec) {
     codec = query_options.compression_codec.codec;
     clevel = query_options.compression_codec.compression_level;
+  } else if (table_desc_->IsIcebergTable()) {
+    TCompressionCodec compression_codec = table_desc_->IcebergParquetCompressionCodec();
+    codec = compression_codec.codec;
+    if (compression_codec.__isset.compression_level) {
+      clevel = compression_codec.compression_level;
+    }
   }
 
   if (!(codec == THdfsCompression::NONE ||
@@ -1084,8 +1348,11 @@ Status HdfsParquetTableWriter::Init() {
 
   Codec::CodecInfo codec_info(codec, clevel);
 
-  ConfigureTimestampType();
-  ConfigureStringType();
+  if (is_iceberg_file_) {
+    ConfigureForIceberg(num_cols);
+  } else {
+    Configure(num_cols);
+  }
 
   columns_.resize(num_cols);
   // Initialize each column structure.
@@ -1233,23 +1500,6 @@ int64_t HdfsParquetTableWriter::MinBlockSize(int64_t num_file_cols) const {
   return 3 * DEFAULT_DATA_PAGE_SIZE * num_file_cols;
 }
 
-uint64_t HdfsParquetTableWriter::default_block_size() const {
-  int64_t block_size;
-  if (state_->query_options().__isset.parquet_file_size &&
-      state_->query_options().parquet_file_size > 0) {
-    // If the user specified a value explicitly, use it. InitNewFile() will verify that
-    // the actual file's block size is sufficient.
-    block_size = state_->query_options().parquet_file_size;
-  } else {
-    block_size = HDFS_BLOCK_SIZE;
-    // Blocks are usually HDFS_BLOCK_SIZE bytes, unless there are many columns, in
-    // which case a per-column minimum kicks in.
-    block_size = max(block_size, MinBlockSize(columns_.size()));
-  }
-  // HDFS does not like block sizes that are not aligned
-  return BitUtil::RoundUp(block_size, HDFS_BLOCK_ALIGNMENT);
-}
-
 Status HdfsParquetTableWriter::InitNewFile() {
   DCHECK(current_row_group_ == nullptr);
 
@@ -1386,9 +1636,56 @@ Status HdfsParquetTableWriter::WriteFileHeader() {
   return Status::OK();
 }
 
+Status HdfsParquetTableWriter::WriteParquetBloomFilter(BaseColumnWriter* col_writer,
+    parquet::ColumnMetaData* meta_data) {
+  DCHECK(col_writer != nullptr);
+  DCHECK(meta_data != nullptr);
+
+  const ParquetBloomFilter* bloom_filter = col_writer->GetParquetBloomFilter();
+  if (bloom_filter == nullptr || bloom_filter->AlwaysFalse()) {
+    // If there is no Bloom filter for this column or if it is empty we don't need to do
+    // anything.
+    // If bloom_filter->AlwaysFalse() is true, it means the Bloom filter was initialised
+    // but no element was inserted, probably because we have not fallen back to plain
+    // encoding from dictionary encoding.
+    return Status::OK();
+  }
+
+  // Update metadata.
+  meta_data->__set_bloom_filter_offset(file_pos_);
+
+  // Write the header to the file.
+  parquet::BloomFilterHeader header = CreateBloomFilterHeader(*bloom_filter);
+  uint8_t* buffer = nullptr;
+  uint32_t len = 0;
+  RETURN_IF_ERROR(thrift_serializer_->SerializeToBuffer(&header, &len, &buffer));
+  DCHECK(buffer != nullptr);
+  DCHECK_GT(len, 0);
+  RETURN_IF_ERROR(Write(buffer, len));
+  file_pos_ += len;
+
+  // Write the Bloom filter directory (bitset) to the file.
+  const uint8_t* directory = bloom_filter->directory();
+  const int64_t directory_size = bloom_filter->directory_size();
+  RETURN_IF_ERROR(Write(directory, directory_size));
+  file_pos_ += directory_size;
+
+  return Status::OK();
+}
+
+void HdfsParquetTableWriter::CollectIcebergDmlFileColumnStats(int field_id,
+    const BaseColumnWriter* col_writer) {
+  // Each data file consists of a single row group, so row group null_count / min / max
+  // stats can be used as data file stats.
+  // Get column_size from column writer.
+  col_writer->row_group_stats_base_->GetIcebergStats(col_writer->total_compressed_size(),
+      &iceberg_file_stats_[field_id]);
+}
+
 Status HdfsParquetTableWriter::FlushCurrentRowGroup() {
   if (current_row_group_ == nullptr) return Status::OK();
 
+  const int num_clustering_cols = table_desc_->num_clustering_cols();
   for (int i = 0; i < columns_.size(); ++i) {
     int64_t data_page_offset, dict_page_offset;
     // Flush this column.  This updates the final metadata sizes for this column.
@@ -1414,6 +1711,14 @@ Status HdfsParquetTableWriter::FlushCurrentRowGroup() {
         parquet_dml_stats_.mutable_per_column_size();
     (*column_size_map)[col_name] += col_writer->total_compressed_size();
 
+    if (is_iceberg_file_ ) {
+      const ColumnDescriptor& col_desc =
+          table_desc_->col_descs()[i + num_clustering_cols];
+      DCHECK_EQ(col_desc.name(), col_name);
+      const int field_id = col_desc.field_id();
+      if (field_id != -1) CollectIcebergDmlFileColumnStats(field_id, col_writer);
+    }
+
     // Write encodings and encoding stats for this column
     col_metadata.encodings.clear();
     for (parquet::Encoding::type encoding : col_writer->column_encodings_) {
@@ -1442,6 +1747,10 @@ Status HdfsParquetTableWriter::FlushCurrentRowGroup() {
     // Build column statistics and add them to the header.
     col_writer->EncodeRowGroupStats(&current_row_group_->columns[i].meta_data);
 
+    // Write Bloom filter and update metadata.
+    RETURN_IF_ERROR(WriteParquetBloomFilter(col_writer,
+        &current_row_group_->columns[i].meta_data));
+
     // Since we don't supported complex schemas, all columns should have the same
     // number of values.
     DCHECK_EQ(current_row_group_->columns[0].meta_data.num_values,
