@@ -22,6 +22,7 @@
 
 #include "exec/parquet/hdfs-parquet-scanner.h"
 #include "exec/parquet/parquet-bool-decoder.h"
+#include "exec/parquet/parquet-data-converter.h"
 #include "exec/parquet/parquet-level-decoder.h"
 #include "exec/parquet/parquet-metadata-utils.h"
 #include "exec/scratch-tuple-batch.h"
@@ -208,10 +209,7 @@ class ScalarColumnReader : public BaseScalarColumnReader {
   /// Most column readers never require conversion, so we can avoid branches by
   /// returning constant false. Column readers for types that require conversion
   /// must specialize this function.
-  inline bool NeedsConversionInline() const {
-    DCHECK(!needs_conversion_);
-    return false;
-  }
+  inline bool NeedsConversionInline() const;
 
   /// Similar to NeedsConversion(), most column readers do not require validation,
   /// so to avoid branches, we return constant false. In general, types where not
@@ -223,8 +221,7 @@ class ScalarColumnReader : public BaseScalarColumnReader {
 
   /// Converts and writes 'src' into 'slot' based on desc_->type()
   bool ConvertSlot(const InternalType* src, void* slot) {
-    DCHECK(false);
-    return false;
+    return data_converter_.ConvertSlot(src, slot);
   }
 
   /// Checks if 'val' is invalid, e.g. due to being out of the valid value range. If it
@@ -251,21 +248,22 @@ class ScalarColumnReader : public BaseScalarColumnReader {
         PrintThriftEnum(page_encoding_), col_chunk_reader_.stream()->file_offset());
   }
 
+  ParquetTimestampDecoder& GetTimestampDecoder() {
+    return data_converter_.timestamp_decoder();
+  }
+
   /// Dictionary decoder for decoding column values.
   DictDecoder<InternalType> dict_decoder_;
 
   /// True if dict_decoder_ has been initialized with a dictionary page.
   bool dict_decoder_init_ = false;
 
-  /// true if decoded values must be converted before being written to an output tuple.
-  bool needs_conversion_ = false;
-
   /// The size of this column with plain encoding for FIXED_LEN_BYTE_ARRAY, or
   /// the max length for VARCHAR columns. Unused otherwise.
   int fixed_len_size_;
 
-  /// Contains extra data needed for Timestamp decoding.
-  ParquetTimestampDecoder timestamp_decoder_;
+  /// Converts values if needed.
+  ParquetDataConverter<InternalType, MATERIALIZED> data_converter_;
 
   /// Contains extra state required to decode boolean values. Only initialised for
   /// BOOLEAN columns.
@@ -279,7 +277,9 @@ template <typename InternalType, parquet::Type::type PARQUET_TYPE, bool MATERIAL
 ScalarColumnReader<InternalType, PARQUET_TYPE, MATERIALIZED>::ScalarColumnReader(
     HdfsParquetScanner* parent, const SchemaNode& node, const SlotDescriptor* slot_desc)
   : BaseScalarColumnReader(parent, node, slot_desc),
-    dict_decoder_(parent->scan_node_->mem_tracker()) {
+    dict_decoder_(parent->scan_node_->mem_tracker()),
+    data_converter_(node.element,
+                    MATERIALIZED ? &slot_desc->type() : nullptr) {
   if (!MATERIALIZED) {
     // We're not materializing any values, just counting them. No need (or ability) to
     // initialize state used to materialize values.
@@ -297,18 +297,43 @@ ScalarColumnReader<InternalType, PARQUET_TYPE, MATERIALIZED>::ScalarColumnReader
     fixed_len_size_ = -1;
   }
 
-  needs_conversion_ = slot_desc_->type().type == TYPE_CHAR;
-
   if (slot_desc_->type().type == TYPE_TIMESTAMP) {
-    timestamp_decoder_ = parent->CreateTimestampDecoder(*node.element);
-    dict_decoder_.SetTimestampHelper(timestamp_decoder_);
-    needs_conversion_ = timestamp_decoder_.NeedsConversion();
+    data_converter_.SetTimestampDecoder(parent->CreateTimestampDecoder(*node.element));
+    dict_decoder_.SetTimestampHelper(GetTimestampDecoder());
   }
   if (slot_desc_->type().type == TYPE_BOOLEAN) {
     bool_decoder_ = make_unique<ParquetBoolDecoder>();
   }
 }
 
+template <typename T>
+struct IsDecimalValue {
+  static constexpr bool value =
+    std::is_same<T, Decimal4Value>::value ||
+    std::is_same<T, Decimal8Value>::value ||
+    std::is_same<T, Decimal16Value>::value;
+};
+
+template <typename InternalType, parquet::Type::type PARQUET_TYPE, bool MATERIALIZED>
+inline bool ScalarColumnReader<InternalType, PARQUET_TYPE, MATERIALIZED>
+::NeedsConversionInline() const {
+  //TODO: use constexpr ifs when we switch to C++17.
+  if /* constexpr */ (MATERIALIZED) {
+    if /* constexpr */ (IsDecimalValue<InternalType>::value) {
+      return data_converter_.NeedsConversion();
+    }
+    if /* constexpr */ (std::is_same<InternalType, TimestampValue>::value) {
+      return data_converter_.NeedsConversion();
+    }
+    if /* constexpr */ (std::is_same<InternalType, StringValue>::value &&
+                        PARQUET_TYPE == parquet::Type::BYTE_ARRAY) {
+      return data_converter_.NeedsConversion();
+    }
+  }
+  DCHECK(!data_converter_.NeedsConversion());
+  return false;
+}
+
 // TODO: consider performing filter selectivity checks in this function.
 template <typename InternalType, parquet::Type::type PARQUET_TYPE, bool MATERIALIZED>
 Status ScalarColumnReader<InternalType, PARQUET_TYPE, MATERIALIZED>::InitDataPage(
@@ -675,14 +700,14 @@ bool ScalarColumnReader<InternalType, PARQUET_TYPE, MATERIALIZED>::ReadSlot(
       reinterpret_cast<InternalType*>(NEEDS_CONVERSION ? val_buf : slot);
 
   if (UNLIKELY(!DecodeValue<ENCODING>(&data_, data_end_, val_ptr))) return false;
-  if (UNLIKELY(NeedsValidationInline() && !ValidateValue(val_ptr))) {
+  if (UNLIKELY(NeedsValidationInline() && !ValidateValue(val_ptr)) ||
+      (NEEDS_CONVERSION && UNLIKELY(!ConvertSlot(val_ptr, slot)))) {
     if (UNLIKELY(!parent_->parse_status_.ok())) return false;
     // The value is invalid but execution should continue - set the null indicator and
     // skip conversion.
     tuple->SetNull(null_indicator_offset_);
     return true;
   }
-  if (NEEDS_CONVERSION && UNLIKELY(!ConvertSlot(val_ptr, slot))) return false;
   return true;
 }
 
@@ -709,16 +734,14 @@ bool ScalarColumnReader<InternalType, PARQUET_TYPE, MATERIALIZED>::ReadAndConver
   uint8_t* curr_tuple = tuple_mem;
   for (int64_t i = 0; i < num_to_read; ++i, ++curr_val, curr_tuple += tuple_size) {
     Tuple* tuple = reinterpret_cast<Tuple*>(curr_tuple);
-    if (NeedsValidationInline() && UNLIKELY(!ValidateValue(curr_val))) {
+    if ((NeedsValidationInline() && UNLIKELY(!ValidateValue(curr_val))) ||
+        UNLIKELY(!ConvertSlot(curr_val, tuple->GetSlot(tuple_offset_)))) {
       if (UNLIKELY(!parent_->parse_status_.ok())) return false;
-      // The value is invalid but execution should continue - set the null indicator and
-      // skip conversion.
+      // The value or the conversion is invalid but execution should continue - set the
+      // null indicator.
       tuple->SetNull(null_indicator_offset_);
       continue;
     }
-    if (UNLIKELY(!ConvertSlot(curr_val, tuple->GetSlot(tuple_offset_)))) {
-      return false;
-    }
   }
   return true;
 }
@@ -772,14 +795,15 @@ bool ScalarColumnReader<InternalType, PARQUET_TYPE, MATERIALIZED>::DecodeValue(
 }
 
 // Specialise for decoding INT64 timestamps from PLAIN decoding, which need to call
-// out to 'timestamp_decoder_'.
+// out to the timestamp decoder.
 template <>
 template <>
 bool ScalarColumnReader<TimestampValue, parquet::Type::INT64,
     true>::DecodeValue<Encoding::PLAIN>(uint8_t** RESTRICT data,
     const uint8_t* RESTRICT data_end, TimestampValue* RESTRICT val) RESTRICT {
   DCHECK_EQ(page_encoding_, Encoding::PLAIN);
-  int encoded_len = timestamp_decoder_.Decode<parquet::Type::INT64>(*data, data_end, val);
+  int encoded_len =
+      GetTimestampDecoder().Decode<parquet::Type::INT64>(*data, data_end, val);
   if (UNLIKELY(encoded_len < 0)) {
     SetPlainDecodeError();
     return false;
@@ -834,14 +858,14 @@ bool ScalarColumnReader<InternalType, PARQUET_TYPE, MATERIALIZED>::DecodeValues(
 }
 
 // Specialise for decoding INT64 timestamps from PLAIN decoding, which need to call
-// out to 'timestamp_decoder_'.
+// out to the timestamp decoder.
 template <>
 template <>
 bool ScalarColumnReader<TimestampValue, parquet::Type::INT64,
     true>::DecodeValues<Encoding::PLAIN>(int64_t stride, int64_t count,
     TimestampValue* RESTRICT out_vals) RESTRICT {
   DCHECK_EQ(page_encoding_, Encoding::PLAIN);
-  int64_t encoded_len = timestamp_decoder_.DecodeBatch<parquet::Type::INT64>(
+  int64_t encoded_len = GetTimestampDecoder().DecodeBatch<parquet::Type::INT64>(
       data_, data_end_, count, stride, out_vals);
   if (UNLIKELY(encoded_len < 0)) {
     SetPlainDecodeError();
@@ -880,64 +904,6 @@ void ScalarColumnReader<InternalType, PARQUET_TYPE, MATERIALIZED>::ReadPositions
   }
 }
 
-template <>
-inline bool ScalarColumnReader<StringValue, parquet::Type::BYTE_ARRAY,
-    true>::NeedsConversionInline() const {
-  return needs_conversion_;
-}
-
-template <>
-bool ScalarColumnReader<StringValue, parquet::Type::BYTE_ARRAY, true>::ConvertSlot(
-    const StringValue* src, void* slot) {
-  DCHECK(slot_desc() != nullptr);
-  DCHECK(slot_desc()->type().type == TYPE_CHAR);
-  int char_len = slot_desc()->type().len;
-  int unpadded_len = min(char_len, src->len);
-  char* dst_char = reinterpret_cast<char*>(slot);
-  memcpy(dst_char, src->ptr, unpadded_len);
-  StringValue::PadWithSpaces(dst_char, char_len, unpadded_len);
-  return true;
-}
-
-template <>
-inline bool ScalarColumnReader<TimestampValue, parquet::Type::INT96, true>
-::NeedsConversionInline() const {
-  return needs_conversion_;
-}
-
-template <>
-inline bool ScalarColumnReader<TimestampValue, parquet::Type::INT64, true>
-::NeedsConversionInline() const {
-  return needs_conversion_;
-}
-
-template <>
-bool ScalarColumnReader<TimestampValue, parquet::Type::INT96, true>::ConvertSlot(
-    const TimestampValue* src, void* slot) {
-  // Conversion should only happen when this flag is enabled.
-  DCHECK(timestamp_decoder_.NeedsConversion());
-  TimestampValue* dst_ts = reinterpret_cast<TimestampValue*>(slot);
-  *dst_ts = *src;
-  // TODO: IMPALA-7862: converting timestamps after validating them can move them out of
-  // range. We should either validate after conversion or require conversion to produce an
-  // in-range value.
-  timestamp_decoder_.ConvertToLocalTime(dst_ts);
-  return true;
-}
-
-template <>
-bool ScalarColumnReader<TimestampValue, parquet::Type::INT64, true>::ConvertSlot(
-    const TimestampValue* src, void* slot) {
-  DCHECK(timestamp_decoder_.NeedsConversion());
-  TimestampValue* dst_ts = reinterpret_cast<TimestampValue*>(slot);
-  *dst_ts = *src;
-  // TODO: IMPALA-7862: converting timestamps after validating them can move them out of
-  // range. We should either validate after conversion or require conversion to produce an
-  // in-range value.
-  timestamp_decoder_.ConvertToLocalTime(static_cast<TimestampValue*>(dst_ts));
-  return true;
-}
-
 template <>
 inline bool ScalarColumnReader<TimestampValue, parquet::Type::INT96, true>
 ::NeedsValidationInline() const {
@@ -1168,6 +1134,65 @@ Status BaseScalarColumnReader::ReadDataPage() {
   return Status::OK();
 }
 
+Status BaseScalarColumnReader::ReadNextDataPageHeader() {
+  // We're about to move to the next data page. The previous data page is
+  // now complete, free up any memory allocated for it. If the data page contained
+  // strings we need to attach it to the returned batch.
+  col_chunk_reader_.ReleaseResourcesOfLastPage(parent_->scratch_batch_->aux_mem_pool);
+
+  DCHECK_EQ(num_buffered_values_, 0);
+  if ((DoesPageFiltering() && candidate_page_idx_ == candidate_data_pages_.size() - 1)
+      || num_values_read_ == metadata_->num_values) {
+    // No more pages to read
+    // TODO: should we check for stream_->eosr()?
+    return Status::OK();
+  } else if (num_values_read_ > metadata_->num_values) {
+    RETURN_IF_ERROR(LogCorruptNumValuesInMetadataError());
+    return Status::OK();
+  }
+
+  bool eos;
+  int data_size;
+  RETURN_IF_ERROR(col_chunk_reader_.ReadNextDataPage(&eos, &data_, &data_size,
+      false /*Read next data page's header only*/));
+  if (eos) return HandleTooEarlyEos();
+  const parquet::PageHeader& current_page_header = col_chunk_reader_.CurrentPageHeader();
+  int num_values = current_page_header.data_page_header.num_values;
+  if (UNLIKELY(num_values < 0)) {
+    return Status(Substitute("Error reading data page in Parquet file '$0'. "
+                             "Invalid number of values in metadata: $1",
+        filename(), num_values));
+  }
+  num_buffered_values_ = num_values;
+  num_values_read_ += num_buffered_values_;
+  if (parent_->candidate_ranges_.empty()) COUNTER_ADD(parent_->num_pages_counter_, 1);
+  return Status::OK();
+}
+
+Status BaseScalarColumnReader::ReadCurrentDataPage() {
+  int data_size;
+  RETURN_IF_ERROR(col_chunk_reader_.ReadDataPageData(&data_, &data_size));
+  data_end_ = data_ + data_size;
+  const parquet::PageHeader& current_page_header = col_chunk_reader_.CurrentPageHeader();
+  /// TODO: Move the level decoder initialisation to ParquetPageReader to abstract away
+  /// the differences between Parquet header V1 and V2.
+  // Initialize the repetition level data
+  RETURN_IF_ERROR(rep_levels_.Init(filename(),
+      &current_page_header.data_page_header.repetition_level_encoding,
+      parent_->perm_pool_.get(), parent_->state_->batch_size(), max_rep_level(), &data_,
+      &data_size));
+  // Initialize the definition level data
+  RETURN_IF_ERROR(def_levels_.Init(filename(),
+      &current_page_header.data_page_header.definition_level_encoding,
+      parent_->perm_pool_.get(), parent_->state_->batch_size(), max_def_level(), &data_,
+      &data_size));
+  // Data can be empty if the column contains all NULLs
+  RETURN_IF_ERROR(InitDataPage(data_, data_size));
+  // Skip rows if needed.
+  RETURN_IF_ERROR(StartPageFiltering());
+  return Status::OK();
+}
+
 template <bool ADVANCE_REP_LEVEL>
 bool BaseScalarColumnReader::NextLevels() {
   if (!ADVANCE_REP_LEVEL) DCHECK_EQ(max_rep_level(), 0) << slot_desc()->DebugString();
@@ -1183,7 +1208,9 @@ bool BaseScalarColumnReader::NextLevels() {
         auto current_range = parent_->candidate_ranges_[current_row_range_];
         int64_t skip_rows = current_range.first - current_row_ - 1;
         DCHECK_GE(skip_rows, 0);
-        if (!SkipTopLevelRows(skip_rows)) return false;
+        int64_t remaining = 0;
+        if (!SkipTopLevelRows(skip_rows, &remaining)) return false;
+        DCHECK_EQ(remaining, 0);
       } else {
         if (!JumpToNextPage()) return parent_->parse_status_.ok();
       }
@@ -1241,39 +1268,59 @@ Status BaseScalarColumnReader::StartPageFiltering() {
   int64_t range_start = candidate_row_ranges[current_row_range_].first;
   if (range_start > current_row_ + 1) {
     int64_t skip_rows = range_start - current_row_ - 1;
-    if (!SkipTopLevelRows(skip_rows)) {
-      return Status(Substitute("Couldn't skip rows in file $0.", filename()));
+    int64_t remaining = 0;
+    if (!SkipTopLevelRows(skip_rows, &remaining)) {
+      return Status(ErrorMsg(TErrorCode::PARQUET_ROWS_SKIPPING,
+          schema_element().name, filename()));
     }
+    DCHECK_EQ(remaining, 0);
     DCHECK_EQ(current_row_, range_start - 1);
   }
   return Status::OK();
 }
 
-bool BaseScalarColumnReader::SkipTopLevelRows(int64_t num_rows) {
-  DCHECK_GE(num_buffered_values_, num_rows);
+template <bool MULTI_PAGE>
+bool BaseScalarColumnReader::SkipTopLevelRows(int64_t num_rows, int64_t* remaining) {
   DCHECK_GT(num_rows, 0);
+  DCHECK_GT(num_buffered_values_, 0);
+  if (!MULTI_PAGE) {
+    DCHECK_GE(num_buffered_values_, num_rows);
+  }
   // Fastest path: field is required and not nested.
   // So row count equals value count, and every value is stored in the page data.
   if (max_def_level() == 0 && max_rep_level() == 0) {
-    current_row_ += num_rows;
-    num_buffered_values_ -= num_rows;
-    return SkipEncodedValuesInPage(num_rows);
+    int rows_skipped;
+    if (MULTI_PAGE) {
+      rows_skipped = std::min((int64_t)num_buffered_values_, num_rows);
+    } else {
+      rows_skipped = num_rows;
+    }
+    current_row_ += rows_skipped;
+    num_buffered_values_ -= rows_skipped;
+    *remaining = num_rows - rows_skipped;
+    return SkipEncodedValuesInPage(rows_skipped);
   }
   int64_t num_values_to_skip = 0;
   if (max_rep_level() == 0) {
     // No nesting, but field is not required.
     // Skip as many values in the page data as many non-NULL values encountered.
     int i = 0;
-    while (i < num_rows) {
+    while (i < num_rows && num_buffered_values_ > 0) {
       int repeated_run_length = def_levels_.NextRepeatedRunLength();
       if (repeated_run_length > 0) {
         int read_count = min<int64_t>(num_rows - i, repeated_run_length);
+        // IMPALA-11134: there can be a mismatch between page_header.num_values and
+        // encoded def levels in old Parquet files.
+        read_count = min(num_buffered_values_, read_count);
         int16_t def_level = def_levels_.GetRepeatedValue(read_count);
         if (def_level >= max_def_level_) num_values_to_skip += read_count;
         i += read_count;
         num_buffered_values_ -= read_count;
       } else if (def_levels_.CacheHasNext()) {
         int read_count = min<int64_t>(num_rows - i, def_levels_.CacheRemaining());
+        // IMPALA-11134: there can be a mismatch between page_header.num_values and
+        // encoded def levels in old Parquet files.
+        read_count = min(num_buffered_values_, read_count);
         for (int j = 0; j < read_count; ++j) {
           if (def_levels_.CacheGetNext() >= max_def_level_) ++num_values_to_skip;
         }
@@ -1283,11 +1330,13 @@ bool BaseScalarColumnReader::SkipTopLevelRows(int64_t num_rows) {
         if (!def_levels_.CacheNextBatch(num_buffered_values_).ok()) return false;
       }
     }
-    current_row_ += num_rows;
+    DCHECK_LE(i, num_rows);
+    current_row_ += i;
+    *remaining = num_rows - i;
   } else {
     // 'rep_level_' being zero denotes the start of a new top-level row.
     // From the 'def_level_' we can determine the number of non-NULL values.
-    while (true) {
+    while (num_buffered_values_ > 0) {
       if (!def_levels_.CacheNextBatchIfEmpty(num_buffered_values_).ok()) return false;
       if (!rep_levels_.CacheNextBatchIfEmpty(num_buffered_values_).ok()) return false;
       if (num_rows == 0 && rep_levels_.PeekLevel() == 0) {
@@ -1304,10 +1353,33 @@ bool BaseScalarColumnReader::SkipTopLevelRows(int64_t num_rows) {
         --num_rows;
       }
     }
+    *remaining = num_rows;
   }
   return SkipEncodedValuesInPage(num_values_to_skip);
 }
 
+bool BaseScalarColumnReader::SetRowGroupAtEnd() {
+  if (RowGroupAtEnd()) {
+    return true;
+  }
+  if (num_buffered_values_ == 0) {
+    NextPage();
+  }
+  if (DoesPageFiltering() && RowsRemainingInCandidateRange() == 0) {
+    if (max_rep_level() == 0 || rep_levels_.PeekLevel() == 0) {
+      if (!IsLastCandidateRange()) AdvanceCandidateRange();
+      if (!PageHasRemainingCandidateRows()) {
+        JumpToNextPage();
+      }
+    }
+  }
+  bool status = RowGroupAtEnd();
+  if (!status) {
+    return false;
+  }
+  return parent_->parse_status_.ok();
+}
+
 int BaseScalarColumnReader::FillPositionsInCandidateRange(int rows_remaining,
     int max_values, uint8_t* RESTRICT tuple_mem, int tuple_size) {
   DCHECK_GT(max_rep_level_, 0);
@@ -1369,7 +1441,8 @@ bool BaseScalarColumnReader::SkipRowsInPage() {
   DCHECK_LT(current_row_, current_range.first);
   int64_t skip_rows = current_range.first - current_row_ - 1;
   DCHECK_GE(skip_rows, 0);
-  return SkipTopLevelRows(skip_rows);
+  int64_t remaining = 0;
+  return SkipTopLevelRows(skip_rows, &remaining);
 }
 
 bool BaseScalarColumnReader::JumpToNextPage() {
@@ -1411,6 +1484,21 @@ bool BaseScalarColumnReader::NextPage() {
   return true;
 }
 
+bool BaseScalarColumnReader::AdvanceNextPageHeader() {
+  num_buffered_values_ = 0;
+  parent_->assemble_rows_timer_.Stop();
+  parent_->parse_status_ = ReadNextDataPageHeader();
+  if (UNLIKELY(!parent_->parse_status_.ok())) return false;
+  if (num_buffered_values_ == 0) {
+    rep_level_ = ParquetLevel::ROW_GROUP_END;
+    def_level_ = ParquetLevel::ROW_GROUP_END;
+    pos_current_value_ = ParquetLevel::INVALID_POS;
+    return false;
+  }
+  parent_->assemble_rows_timer_.Start();
+  return true;
+}
+
 void BaseScalarColumnReader::SetLevelDecodeError(
     const char* level_name, int decoded_level, int max_level) {
   if (decoded_level < 0) {
@@ -1426,6 +1514,122 @@ void BaseScalarColumnReader::SetLevelDecodeError(
   }
 }
 
+/// Wrapper around 'SkipTopLevelRows' to skip across multiple pages.
+/// Function handles 3 scenarios:
+/// 1. Page Filtering: When this is enabled this function can be used
+///    to skip to a particular 'skip_row_id'.
+/// 2. Collection: When this scalar reader is reading elements of a collection
+/// 3. Rest of the cases.
+/// For page filtering, we keep track of first and last page indexes and keep
+/// traversing to next page until we find a page that contains 'skip_row_id'.
+/// At that point, we can just skip to the required row id.
+/// If the page of 'skip_row_id' is not a candidate page, we will stop at the
+/// next candidate page and 'skip_row_id' is skipped by the way.
+/// Difference between scenario 2 and 3 is that in scenario 2, we end up
+/// decompressing all the pages being skipped, whereas in scenario 3 we only
+/// decompress pages required and avoid decompression needed. This is possible
+/// because in scenario 3 'data_page_header.num_values' corresponds to number
+/// of rows stored in the page. This is not true in scenario 2 because multiple
+/// consecutive values can belong to same row.
+template <bool IN_COLLECTION>
+bool BaseScalarColumnReader::SkipRowsInternal(int64_t num_rows, int64_t skip_row_id) {
+  if (DoesPageFiltering() && skip_row_id > 0) {
+    // Checks if its the beginning of row group and advances to next page.
+    if (candidate_page_idx_ < 0) {
+      if (UNLIKELY(!NextPage())) {
+        return false;
+      }
+    }
+    // Keep advancing until we hit reach required page containing 'skip_row_id' or
+    // jump over it if that page is not a candidate page.
+    while (skip_row_id > LastRowIdxInCurrentPage()) {
+      COUNTER_ADD(parent_->num_pages_skipped_by_late_materialization_counter_, 1);
+      if (UNLIKELY(!JumpToNextPage())) {
+        return false;
+      }
+    }
+    int64_t last_row = LastProcessedRow();
+    int64_t remaining = 0;
+    if (skip_row_id >= FirstRowIdxInCurrentPage()) {
+      // Skip to the required row id within the page. Only needs this when row id locates
+      // in current page.
+      if (last_row < skip_row_id) {
+        if (UNLIKELY(!SkipTopLevelRows(skip_row_id - last_row, &remaining))) {
+          return false;
+        }
+      }
+    }
+    // also need to adjust 'candidate_row_ranges' as we skipped to new row id.
+    auto& candidate_row_ranges = parent_->candidate_ranges_;
+    while (current_row_ > candidate_row_ranges[current_row_range_].last) {
+      DCHECK_LT(current_row_range_, candidate_row_ranges.size());
+      ++current_row_range_;
+    }
+    return true;
+  } else if (IN_COLLECTION) {
+    DCHECK_GT(num_rows, 0);
+    // if all the values of current page are consumed, move to next page.
+    if (num_buffered_values_ == 0) {
+      if (!NextPage()) {
+        return false;
+      }
+    }
+    DCHECK_GT(num_buffered_values_, 0);
+    int64_t remaining = 0;
+    // Try to skip 'num_rows' and see if something remains.
+    if (!SkipTopLevelRows<true>(num_rows, &remaining)) {
+      return false;
+    }
+    // Again invoke the same method on remaining rows.
+    if (remaining > 0) {
+      return SkipRowsInternal<IN_COLLECTION>(remaining, skip_row_id);
+    }
+    return true;
+  } else {
+    // If everything consumed in current page, skip data pages (multiple skips if needed)
+    // to reach required page.
+    if (num_buffered_values_ == 0) {
+      if (!AdvanceNextPageHeader()) {
+        return false;
+      }
+      const parquet::PageHeader& current_page_header =
+          col_chunk_reader_.CurrentPageHeader();
+      int32_t current_page_values = current_page_header.data_page_header.num_values;
+      if (UNLIKELY(current_page_values <= 0)) {
+        return false;
+      }
+      // Keep advancing to next page header if rows to be skipped are more than number
+      // of values in the page. Note we will just be reading headers and skipping
+      // pages without decompressing them as we advance.
+      while (num_rows > current_page_values) {
+        COUNTER_ADD(parent_->num_pages_skipped_by_late_materialization_counter_, 1);
+        num_rows -= current_page_values;
+        if (!col_chunk_reader_.SkipPageData().ok() || !AdvanceNextPageHeader()) {
+          return false;
+        }
+        current_page_values =
+            col_chunk_reader_.CurrentPageHeader().data_page_header.num_values;
+      }
+      // Read the data page (includes decompressing them if required).
+      Status page_read = ReadCurrentDataPage();
+      if (!page_read.ok()) {
+        return false;
+      }
+    }
+
+    // Skip the remaining rows in the page.
+    DCHECK_GT(num_buffered_values_, 0);
+    int64_t remaining = 0;
+    if (!SkipTopLevelRows<true>(num_rows, &remaining)) {
+      return false;
+    }
+    if (remaining > 0) {
+      return SkipRowsInternal<IN_COLLECTION>(remaining, skip_row_id);
+    }
+    return true;
+  }
+};
+
 /// Returns a column reader for decimal types based on its size and parquet type.
 static ParquetColumnReader* CreateDecimalColumnReader(
     const SchemaNode& node, const SlotDescriptor* slot_desc, HdfsParquetScanner* parent) {
@@ -1457,11 +1661,9 @@ static ParquetColumnReader* CreateDecimalColumnReader(
       }
       break;
     case parquet::Type::INT32:
-      DCHECK_EQ(sizeof(Decimal4Value::StorageType), slot_desc->type().GetByteSize());
       return new ScalarColumnReader<Decimal4Value, parquet::Type::INT32, true>(
           parent, node, slot_desc);
     case parquet::Type::INT64:
-      DCHECK_EQ(sizeof(Decimal8Value::StorageType), slot_desc->type().GetByteSize());
       return new ScalarColumnReader<Decimal8Value, parquet::Type::INT64, true>(
           parent, node, slot_desc);
     default:
