@@ -21,6 +21,7 @@
 #include <string>
 #include <type_traits>
 
+#include "exec/hdfs-table-writer.h"
 #include "exec/parquet/parquet-common.h"
 #include "runtime/date-value.h"
 #include "runtime/decimal-value.h"
@@ -112,6 +113,10 @@ class ColumnStatsBase {
   /// Encodes the current values into a Statistics thrift message.
   virtual void EncodeToThrift(parquet::Statistics* out) const = 0;
 
+  /// Writes the current values into IcebergColumnStats struct.
+  /// Min and max stats are Single-value serialized.
+  virtual void GetIcebergStats(int64_t column_size, IcebergColumnStats* out) const = 0;
+
   /// Resets the state of this object.
   void Reset();
 
@@ -203,6 +208,9 @@ class ColumnStats : public ColumnStatsBase {
   virtual int64_t BytesNeeded() const override;
   virtual void EncodeToThrift(parquet::Statistics* out) const override;
 
+  virtual void GetIcebergStats(int64_t column_size,
+      IcebergColumnStats* out) const override;
+
   /// Decodes the plain encoded stats value from 'buffer' and writes the result into the
   /// buffer pointed to by 'slot'. Returns true if decoding was successful, false
   /// otherwise. For timestamps and dates an additional validation will be performed.
@@ -210,11 +218,18 @@ class ColumnStats : public ColumnStatsBase {
       parquet::Type::type parquet_type);
 
  protected:
+  /// For BE tests.
+  FRIEND_TEST(SerializeSingleValueTest, Decimal);
+
   /// Encodes a single value using parquet's plain encoding and stores it into the binary
   /// string 'out'. String values are stored without additional encoding. 'bytes_needed'
   /// must be positive.
   static void EncodePlainValue(const T& v, int64_t bytes_needed, std::string* out);
 
+  /// Single-value serialize value 'v'.
+  /// https://iceberg.apache.org/spec/#appendix-d-single-value-serialization
+  static void SerializeIcebergSingleValue(const T& v, std::string* out);
+
   /// Returns the number of bytes needed to encode value 'v'.
   int64_t BytesNeeded(const T& v) const;
 
@@ -252,8 +267,9 @@ public:
   /// the minimum or maximum value.
   enum class StatsField { MIN, MAX };
 
-  ColumnStatsReader(const parquet::ColumnChunk& col_chunk,  const ColumnType& col_type,
-      const parquet::ColumnOrder* col_order, const parquet::SchemaElement& element)
+  ColumnStatsReader(const parquet::ColumnChunk& col_chunk,
+      const ColumnType& col_type, const parquet::ColumnOrder* col_order,
+      const parquet::SchemaElement& element)
   : col_chunk_(col_chunk),
     col_type_(col_type),
     col_order_(col_order),
@@ -281,6 +297,40 @@ public:
       const std::string* paired_stats_value,
       void* slot) const;
 
+  /// Batch read encoded values from a range ['start_idx', 'end_idx'] in
+  /// vector<string> 'encoded_value' into 'slots'.
+  ///
+  /// Return 'true' when all values in the range are decoded successfully, 'false'
+  /// otherwise.
+  bool ReadFromStringsBatch(StatsField stats_field,
+      const vector<std::string>& encoded_values, int64_t start_index,
+      int64_t end_idx, int fixed_len_size, void* slots) const;
+
+  /// Batch decode routine that reads from a vector of std::string 'source' and decodes
+  /// each value in the vector into a value of Impala type 'InternalType'.
+  ///
+  /// Return the number of decoded values from 'source' or -1 if there was an error
+  /// decoding (e.g. invalid data) or invalid specification of the range.
+  ///
+  /// This routine breaks 'source' into batches of 8 elements and unrolls each batch.
+  ///
+  /// For each string value, this routine calls
+  /// ColumnStats<InternalType>::DecodePlainValue() which in turn calls
+  /// ParquetPlainEncoder::DecodeByParquetType() that switches on Parquet type.
+  template <typename InternalType>
+  int64_t DecodeBatchOneBoundsCheck(const vector<std::string>& source, int64_t start_idx,
+      int64_t end_idx, int fixed_len_size, InternalType* v,
+      parquet::Type::type PARQUET_TYPE) const;
+
+  /// A fast track version of the above batch decode routine DecodeBatchOneBoundsCheck().
+  ///
+  /// For each string value, this routine calls ParquetPlainEncoder::DecodeNoBoundsCheck()
+  /// that simply memcpy out the value. This version is used when the Impala internal type
+  /// and the Parquet type are identical.
+  template <typename InternalType>
+  int64_t DecodeBatchOneBoundsCheckFastTrack(const vector<std::string>& source,
+      int64_t start_idx, int64_t end_idx, int fixed_len_size, InternalType* v) const;
+
   // Gets the null_count statistics from the column chunk's metadata and returns
   // it via an output parameter.
   // Returns true if the null_count stats were read successfully, false otherwise.
@@ -313,6 +363,10 @@ private:
   /// order 'col_order_'. Otherwise, returns false.
   bool CanUseDeprecatedStats() const;
 
+  /// Decodes decimal value into slot. Does conversion if needed.
+  template <typename DecimalType>
+  bool DecodeDecimal(const std::string& stat_value, DecimalType* slot) const;
+
   /// Decodes 'stat_value' and does INT64->TimestampValue and timezone conversions if
   /// necessary. Returns true if the decoding and conversions were successful.
   bool DecodeTimestamp(const std::string& stat_value,
