@@ -19,6 +19,7 @@
 
 #include <strings.h>
 #include <sstream>
+#include <stack>
 #include <string>
 #include <vector>
 
@@ -180,6 +181,41 @@ void SetTimestampLogicalType(TParquetTimestampType::type parquet_timestamp_type,
   col_schema->__set_logicalType(logical_type);
 }
 
+bool IsScaleSet(const parquet::SchemaElement& schema_element) {
+  // Scale is required in DecimalType
+  return (schema_element.__isset.logicalType
+             && schema_element.logicalType.__isset.DECIMAL)
+      || schema_element.__isset.scale;
+}
+
+bool IsPrecisionSet(const parquet::SchemaElement& schema_element) {
+  // Precision is required in DecimalType
+  return (schema_element.__isset.logicalType
+             && schema_element.logicalType.__isset.DECIMAL)
+      || schema_element.__isset.precision;
+}
+
+int32_t GetScale(const parquet::SchemaElement& schema_element) {
+  if (schema_element.__isset.logicalType && schema_element.logicalType.__isset.DECIMAL) {
+    return schema_element.logicalType.DECIMAL.scale;
+  }
+
+  if (schema_element.__isset.scale) return schema_element.scale;
+
+  // If not specified, the scale is 0
+  return 0;
+}
+
+// Precision is required, this should be called after checking IsPrecisionSet()
+int32_t GetPrecision(const parquet::SchemaElement& schema_element) {
+  DCHECK(IsPrecisionSet(schema_element));
+  if (schema_element.__isset.logicalType && schema_element.logicalType.__isset.DECIMAL) {
+    return schema_element.logicalType.DECIMAL.precision;
+  }
+
+  return schema_element.precision;
+}
+
 /// Mapping of impala's internal types to parquet storage types. This is indexed by
 /// PrimitiveType enum
 const parquet::Type::type INTERNAL_TO_PARQUET_TYPES[] = {
@@ -318,55 +354,35 @@ Status ParquetMetadataUtils::ValidateColumn(const char* filename,
   bool is_converted_type_decimal = schema_element.__isset.converted_type
       && schema_element.converted_type == parquet::ConvertedType::DECIMAL;
   if (slot_desc->type().type == TYPE_DECIMAL) {
-    // TODO: allow converting to wider type (IMPALA-2515)
-    if (schema_element.type == parquet::Type::INT32 &&
-        sizeof(int32_t) != slot_desc->type().GetByteSize()) {
-      return Status(Substitute("File '$0' decimal column '$1' is stored as INT32, but "
-          "based on the precision in the table metadata, another type would needed.",
-          filename, schema_element.name));
-    }
-    if (schema_element.type == parquet::Type::INT64 &&
-        sizeof(int64_t) != slot_desc->type().GetByteSize()) {
-      return Status(Substitute("File '$0' decimal column '$1' is stored as INT64, but "
-          "based on the precision in the table metadata, another type would needed.",
-          filename, schema_element.name));
-    }
     // We require that the scale and byte length be set.
     if (schema_element.type == parquet::Type::FIXED_LEN_BYTE_ARRAY) {
       if (!schema_element.__isset.type_length) {
         return Status(Substitute("File '$0' column '$1' does not have type_length set.",
             filename, schema_element.name));
       }
-
-      int expected_len = ParquetPlainEncoder::DecimalSize(slot_desc->type());
-      if (schema_element.type_length < expected_len) {
-        return Status(Substitute("File '$0' column '$1' has an invalid type length. "
-            "Expecting: len >= $2 in file: $3", filename, schema_element.name,
-            expected_len, schema_element.type_length));
+      if (schema_element.type_length <= 0) {
+        return Status(Substitute("File '$0' column '$1' has invalid type length: $2",
+            filename, schema_element.name, schema_element.type_length));
       }
     }
-    if (!schema_element.__isset.scale) {
-      return Status(Substitute("File '$0' column '$1' does not have the scale set.",
-          filename, schema_element.name));
-    }
 
-    if (schema_element.scale != slot_desc->type().scale) {
-      // TODO: we could allow a mismatch and do a conversion at this step.
-      return Status(Substitute("File '$0' column '$1' has a scale that does not match "
-          "the table metadata scale. File metadata scale: $2 Table metadata scale: $3",
-          filename, schema_element.name, schema_element.scale, slot_desc->type().scale));
-    }
-
-    // The other decimal metadata should be there but we don't need it.
-    if (!schema_element.__isset.precision) {
+    // We require that the precision be a positive value, and not larger than the
+    // precision in table schema.
+    if (!IsPrecisionSet(schema_element)) {
       ErrorMsg msg(TErrorCode::PARQUET_MISSING_PRECISION, filename, schema_element.name);
-      RETURN_IF_ERROR(state->LogOrReturnError(msg));
+      return Status(msg);
     } else {
-      if (schema_element.precision != slot_desc->type().precision) {
-        // TODO: we could allow a mismatch and do a conversion at this step.
+      int32_t precision = GetPrecision(schema_element);
+      if (precision > slot_desc->type().precision || precision <= 0) {
         ErrorMsg msg(TErrorCode::PARQUET_WRONG_PRECISION, filename, schema_element.name,
-            schema_element.precision, slot_desc->type().precision);
-        RETURN_IF_ERROR(state->LogOrReturnError(msg));
+            precision, slot_desc->type().precision);
+        return Status(msg);
+      }
+      int32_t scale = GetScale(schema_element);
+      if (scale < 0 || scale > precision) {
+        return Status(
+            Substitute("File '$0' column '$1' has invalid scale: $2. Precision is $3.",
+                filename, schema_element.name, scale, precision));
       }
     }
 
@@ -377,7 +393,7 @@ Status ParquetMetadataUtils::ValidateColumn(const char* filename,
           schema_element.name);
       RETURN_IF_ERROR(state->LogOrReturnError(msg));
     }
-  } else if (schema_element.__isset.scale || schema_element.__isset.precision
+  } else if (IsScaleSet(schema_element) || IsPrecisionSet(schema_element)
       || is_converted_type_decimal) {
     ErrorMsg msg(TErrorCode::PARQUET_INCOMPATIBLE_DECIMAL, filename, schema_element.name,
         slot_desc->type().DebugString());
@@ -713,7 +729,7 @@ SchemaNode* ParquetSchemaResolver::NextSchemaNode(
 
   int file_idx;
   int table_idx = path[next_idx];
-  if (fallback_schema_resolution_ == TParquetFallbackSchemaResolution::type::NAME) {
+  if (fallback_schema_resolution_ == TSchemaResolutionStrategy::type::NAME) {
     if (next_idx == 0) {
       // Resolve top-level table column by name.
       DCHECK_LT(table_idx, tbl_desc_.col_descs().size());
@@ -745,7 +761,7 @@ SchemaNode* ParquetSchemaResolver::NextSchemaNode(
       }
     }
   } else if (fallback_schema_resolution_ ==
-        TParquetFallbackSchemaResolution::type::FIELD_ID) {
+        TSchemaResolutionStrategy::type::FIELD_ID) {
     // Resolution by field id for Iceberg table.
     if (next_idx == 0) {
       // Resolve top-level table column by field id.
@@ -772,7 +788,7 @@ SchemaNode* ParquetSchemaResolver::NextSchemaNode(
   } else {
     // Resolution by position.
     DCHECK_EQ(fallback_schema_resolution_,
-        TParquetFallbackSchemaResolution::type::POSITION);
+        TSchemaResolutionStrategy::type::POSITION);
     if (next_idx == 0) {
       // For top-level columns, the first index in a path includes the table's partition
       // keys.
@@ -784,9 +800,9 @@ SchemaNode* ParquetSchemaResolver::NextSchemaNode(
 
   if (file_idx >= node->children.size()) {
     string schema_resolution_mode = "unknown";
-    auto entry = _TParquetFallbackSchemaResolution_VALUES_TO_NAMES.find(
+    auto entry = _TSchemaResolutionStrategy_VALUES_TO_NAMES.find(
         fallback_schema_resolution_);
-    if (entry != _TParquetFallbackSchemaResolution_VALUES_TO_NAMES.end()) {
+    if (entry != _TSchemaResolutionStrategy_VALUES_TO_NAMES.end()) {
       schema_resolution_mode = entry->second;
     }
     VLOG_FILE << Substitute(
@@ -795,6 +811,13 @@ SchemaNode* ParquetSchemaResolver::NextSchemaNode(
     *missing_field = true;
     return NULL;
   }
+
+  if (UNLIKELY(file_idx == INVALID_ID)) {
+    VLOG_FILE << Substitute("File '$0' is corrupted", filename_);
+    *missing_field = true;
+    return NULL;
+  }
+
   return &node->children[file_idx];
 }
 
@@ -811,7 +834,17 @@ int ParquetSchemaResolver::FindChildWithFieldId(SchemaNode* node,
     const int& field_id) const {
   int idx;
   for (idx = 0; idx < node->children.size(); ++idx) {
-    if (node->children[idx].element->field_id == field_id) break;
+    SchemaNode* child = &node->children[idx];
+
+    int child_field_id = 0;
+
+    if (LIKELY(child->element->__isset.field_id)) {
+      child_field_id = child->element->field_id;
+    } else {
+      child_field_id = GetGeneratedFieldID(child);
+    }
+    if (child_field_id == field_id) return idx;
+    if (UNLIKELY(child_field_id == INVALID_ID)) return INVALID_ID;
   }
   return idx;
 }
@@ -935,4 +968,53 @@ Status ParquetSchemaResolver::ValidateScalarNode(const SchemaNode& node,
   return Status::OK();
 }
 
+void ParquetSchemaResolver::GenerateFieldIDs() {
+  std::stack<SchemaNode*> nodes;
+
+  nodes.push(&schema_);
+
+  int fieldID = 1;
+
+  while (!nodes.empty()) {
+    SchemaNode* current = nodes.top();
+    nodes.pop();
+
+    uint64_t size = current->children.size();
+
+    for (uint64_t i = 0; i < size; i++) {
+      auto retval = schema_node_to_field_id_.emplace(&current->children[i], fieldID++);
+
+      // Emplace has to be successful, otherwise we visited the same node twice
+      DCHECK(retval.second);
+
+      // Push children in reverse order to the stack so they are processed in the original
+      // order
+      const uint64_t reverse_idx = size - i - 1;
+
+      SchemaNode& current_child = current->children[reverse_idx];
+
+      const parquet::ConvertedType::type child_type =
+          current_child.element->converted_type;
+
+      if (child_type == parquet::ConvertedType::type::LIST
+          || child_type == parquet::ConvertedType::type::MAP) {
+        // Skip middle level
+        DCHECK(current_child.children.size() == 1);
+
+        nodes.push(&current_child.children[0]);
+      } else {
+        nodes.push(&current_child);
+      }
+    }
+  }
+}
+
+int ParquetSchemaResolver::GetGeneratedFieldID(SchemaNode* node) const {
+  auto it = schema_node_to_field_id_.find(node);
+
+  // First column has field ID, this one does not, file is corrupted
+  if (UNLIKELY(it == schema_node_to_field_id_.end())) return INVALID_ID;
+
+  return it->second;
+}
 }
