@@ -140,17 +140,12 @@ struct SchemaNode {
 class ParquetSchemaResolver {
  public:
   ParquetSchemaResolver(const HdfsTableDescriptor& tbl_desc,
-      TParquetFallbackSchemaResolution::type fallback_schema_resolution,
+      TSchemaResolutionStrategy::type fallback_schema_resolution,
       TParquetArrayResolution::type array_resolution)
     : tbl_desc_(tbl_desc),
       fallback_schema_resolution_(fallback_schema_resolution),
       array_resolution_(array_resolution),
-      filename_(NULL) {
-    // We set FIELD_ID for Iceberg tables.
-    if (tbl_desc_.IsIcebergTable()) {
-      fallback_schema_resolution_ = TParquetFallbackSchemaResolution::type::FIELD_ID;
-    }
-  }
+      filename_(NULL) {}
 
   /// Parses the schema of the given file metadata into an internal schema
   /// representation used in path resolution. Remembers the filename for error
@@ -158,7 +153,16 @@ class ParquetSchemaResolver {
   Status Init(const parquet::FileMetaData* file_metadata, const char* filename) {
     DCHECK(filename != NULL);
     filename_ = filename;
-    return CreateSchemaTree(file_metadata->schema, &schema_);
+    // Use FIELD_ID-based column resolution for Iceberg tables if possible.
+    const auto& schema = file_metadata->schema;
+    Status status = CreateSchemaTree(file_metadata->schema, &schema_);
+    if (tbl_desc_.IsIcebergTable()) {
+      fallback_schema_resolution_ = TSchemaResolutionStrategy::type::FIELD_ID;
+
+      // schema[0] is the 'root', schema[1] is the first column.
+      if (schema.size() > 1 && !schema[1].__isset.field_id) GenerateFieldIDs();
+    }
+    return status;
   }
 
   /// Traverses 'schema_' according to 'path', returning the result in 'node'. If 'path'
@@ -188,6 +192,9 @@ class ParquetSchemaResolver {
   /// Used to sanity-check Parquet schemas.
   static const int SCHEMA_NODE_CHILDREN_SANITY_LIMIT = 64 * 1024;
 
+  /// Invalid ID used to signal corrupted file
+  static const int INVALID_ID = -1;
+
   /// Maps from the array-resolution policy to the ordered array encodings that should
   /// be tried during path resolution. All entries have the ONE_LEVEL encoding at the end
   /// because there is no ambiguity between the one-level and the other encodings (there
@@ -222,6 +229,7 @@ class ParquetSchemaResolver {
   /// then the index of the first match is returned.
   int FindChildWithName(SchemaNode* node, const std::string& name) const;
   /// Returns the index of 'node's child with 'field id' for Iceberg tables.
+  /// Return -1 if the file is corrupted
   int FindChildWithFieldId(SchemaNode* node, const int& field_id) const;
 
   /// The ResolvePathHelper() logic for arrays.
@@ -237,10 +245,18 @@ class ParquetSchemaResolver {
   Status ValidateScalarNode(const SchemaNode& node, const ColumnType& col_type,
       const SchemaPath& path, int idx) const;
 
+  /// Generates field ids for the columns in the same order as Iceberg. The traversal is
+  /// preorder, but the assigned field IDs are not. When a node is visited, its child
+  /// nodes are assigned an ID, hence the difference.
+  void GenerateFieldIDs();
+
+  inline int GetGeneratedFieldID(SchemaNode* node) const;
+
   const HdfsTableDescriptor& tbl_desc_;
-  TParquetFallbackSchemaResolution::type fallback_schema_resolution_;
+  TSchemaResolutionStrategy::type fallback_schema_resolution_;
   const TParquetArrayResolution::type array_resolution_;
   const char* filename_;
+  std::unordered_map<SchemaNode*, int32_t> schema_node_to_field_id_;
 
   /// Root node of our internal schema representation populated in Init().
   SchemaNode schema_;
