@@ -55,10 +55,6 @@ static const string PREPARE_FOR_READ_FAILED_ERROR_MSG =
 using namespace impala;
 using strings::Substitute;
 
-static string ConstructBuilderName(int join_node_id) {
-  return Substitute("Hash Join Builder (join_node_id=$0)", join_node_id);
-}
-
 DataSink* PhjBuilderConfig::CreateSink(RuntimeState* state) const {
   // We have one fragment per sink, so we can use the fragment index as the sink ID.
   TDataSinkId sink_id = state->fragment().idx;
@@ -162,8 +158,8 @@ const char* PhjBuilder::LLVM_CLASS_NAME = "class.impala::PhjBuilder";
 
 PhjBuilder::PhjBuilder(
     TDataSinkId sink_id, const PhjBuilderConfig& sink_config, RuntimeState* state)
-  : JoinBuilder(
-        sink_id, sink_config, ConstructBuilderName(sink_config.join_node_id_), state),
+  : JoinBuilder(sink_id, sink_config,
+      ConstructBuilderName("Hash", sink_config.join_node_id_), state),
     runtime_state_(state),
     hash_seed_(sink_config.hash_seed_),
     resource_profile_(sink_config.resource_profile_),
@@ -200,7 +196,8 @@ PhjBuilder::PhjBuilder(
 PhjBuilder::PhjBuilder(const PhjBuilderConfig& sink_config,
     BufferPool::ClientHandle* buffer_pool_client, int64_t spillable_buffer_size,
     int64_t max_row_buffer_size, RuntimeState* state)
-  : JoinBuilder(-1, sink_config, ConstructBuilderName(sink_config.join_node_id_), state),
+  : JoinBuilder(
+      -1, sink_config, ConstructBuilderName("Hash", sink_config.join_node_id_), state),
     runtime_state_(state),
     hash_seed_(sink_config.hash_seed_),
     resource_profile_(nullptr),
@@ -229,26 +226,6 @@ PhjBuilder::~PhjBuilder() {}
 
 Status PhjBuilder::Prepare(RuntimeState* state, MemTracker* parent_mem_tracker) {
   RETURN_IF_ERROR(DataSink::Prepare(state, parent_mem_tracker));
-  if (is_separate_build_) {
-    const TDebugOptions& instance_debug_options = state->instance_ctx().debug_options;
-    bool debug_option_enabled = instance_debug_options.node_id == -1
-        || instance_debug_options.node_id == join_node_id_;
-    // SET_DENY_RESERVATION_PROBABILITY should behave the same as if it were applied to
-    // the join node.
-    reservation_manager_.Init(Substitute("$0 ptr=$1", name_, this), profile(),
-        state->instance_buffer_reservation(), mem_tracker_.get(), *resource_profile_,
-        debug_option_enabled ? instance_debug_options : TDebugOptions());
-  }
-
-  RETURN_IF_ERROR(HashTableCtx::Create(&obj_pool_, state, hash_table_config_, hash_seed_,
-      MAX_PARTITION_DEPTH, row_desc_->tuple_descriptors().size(), expr_perm_pool_.get(),
-      expr_results_pool_.get(), expr_results_pool_.get(), &ht_ctx_));
-
-  DCHECK_EQ(filter_exprs_.size(), filter_ctxs_.size());
-  for (int i = 0; i < filter_exprs_.size(); ++i) {
-    RETURN_IF_ERROR(ScalarExprEvaluator::Create(*filter_exprs_[i], state, &obj_pool_,
-        expr_perm_pool_.get(), expr_results_pool_.get(), &filter_ctxs_[i].expr_eval));
-  }
 
   partitions_created_ = ADD_COUNTER(profile(), "PartitionsCreated", TUnit::UNIT);
   largest_partition_percent_ =
@@ -264,6 +241,29 @@ Status PhjBuilder::Prepare(RuntimeState* state, MemTracker* parent_mem_tracker)
   num_hash_table_builds_skipped_ =
       ADD_COUNTER(profile(), "NumHashTableBuildsSkipped", TUnit::UNIT);
   repartition_timer_ = ADD_TIMER(profile(), "RepartitionTime");
+
+  if (is_separate_build_) {
+    const TDebugOptions& instance_debug_options = state->instance_ctx().debug_options;
+    bool debug_option_enabled = instance_debug_options.node_id == -1
+        || instance_debug_options.node_id == join_node_id_;
+    // SET_DENY_RESERVATION_PROBABILITY should behave the same as if it were applied to
+    // the join node.
+    reservation_manager_.Init(Substitute("$0 ptr=$1", name_, this), profile(),
+        state->instance_buffer_reservation(), mem_tracker_.get(), *resource_profile_,
+        debug_option_enabled ? instance_debug_options : TDebugOptions());
+  }
+
+  RETURN_IF_ERROR(HashTableCtx::Create(&obj_pool_, state, hash_table_config_, hash_seed_,
+      MAX_PARTITION_DEPTH, row_desc_->tuple_descriptors().size(), expr_perm_pool_.get(),
+      expr_results_pool_.get(), expr_results_pool_.get(), &ht_ctx_));
+
+  RETURN_IF_ERROR(DebugAction(state->query_options(), "PHJ_BUILDER_PREPARE"));
+
+  DCHECK_EQ(filter_exprs_.size(), filter_ctxs_.size());
+  for (int i = 0; i < filter_exprs_.size(); ++i) {
+    RETURN_IF_ERROR(ScalarExprEvaluator::Create(*filter_exprs_[i], state, &obj_pool_,
+        expr_perm_pool_.get(), expr_results_pool_.get(), &filter_ctxs_[i].expr_eval));
+  }
   return Status::OK();
 }
 
@@ -326,9 +326,48 @@ Status PhjBuilder::AddBatch(RowBatch* batch) {
   }
   // Free any expr result allocations made during partitioning.
   expr_results_pool_->Clear();
+  DetermineUsefulnessForMinmaxFilters();
   return Status::OK();
 }
 
+// Set AlwaysTrue to true for each of minmax filter that is not useful.
+void PhjBuilder::DetermineUsefulnessForMinmaxFilters() {
+  // Loop over all remaining min/max filters still enabled (i.e. AlwaysTrue flag is false)
+  for (std::vector<FilterContext*>::const_iterator it = minmax_filter_ctxs_.begin();
+       it != minmax_filter_ctxs_.end();) {
+    DCHECK((*it)->filter->is_min_max_filter() && (*it)->local_min_max_filter);
+    // The filter can be in alwaysTrue state if not a single not-NULL value has been
+    // inserted.
+    if (UNLIKELY((*it)->local_min_max_filter->AlwaysTrue())) {
+      ++it;
+      continue;
+    }
+    const TRuntimeFilterDesc& filter_desc = (*it)->filter->filter_desc();
+    bool useful = true;
+    for (const auto& target_desc : filter_desc.targets) {
+      if (FilterContext::ShouldRejectFilterBasedOnColumnStats(
+              target_desc, (*it)->local_min_max_filter, minmax_filter_threshold_)) {
+        useful = false;
+        break;
+      }
+    }
+    if (LIKELY(!useful)) {
+      (*it)->local_min_max_filter->SetAlwaysTrue();
+      VLOG(3) << "Filter with id " << (*it)->filter->id()
+              << " at join builder (join_node_id_=" << join_node_id_ << ")"
+              << " is not useful. Disabled."
+              << " Fillter details=" << (*it)->local_min_max_filter->DebugString()
+              << ", column stats:"
+              << " low=" << PrintTColumnValue(filter_desc.targets[0].low_value)
+              << ", high=" << PrintTColumnValue(filter_desc.targets[0].high_value)
+              << ", threshold=" << minmax_filter_threshold_;
+      it = minmax_filter_ctxs_.erase(it);
+    } else {
+      ++it;
+    }
+  }
+}
+
 Status PhjBuilder::FlushFinal(RuntimeState* state) {
   SCOPED_TIMER(profile()->total_time_counter());
   return FinalizeBuild(state);
@@ -900,18 +939,26 @@ void PhjBuilder::AllocateRuntimeFilters() {
   DCHECK(join_op_ != TJoinOp::NULL_AWARE_LEFT_ANTI_JOIN || filter_ctxs_.size() == 0)
       << "Runtime filters not supported with NULL_AWARE_LEFT_ANTI_JOIN";
   DCHECK(ht_ctx_ != nullptr);
-  for (int i = 0; i < filter_ctxs_.size(); ++i) {
-    if (filter_ctxs_[i].filter->is_bloom_filter()) {
-      filter_ctxs_[i].local_bloom_filter =
+  for (FilterContext& filter_ctx : filter_ctxs_) {
+    if (filter_ctx.filter->is_bloom_filter()) {
+      filter_ctx.local_bloom_filter =
           runtime_state_->filter_bank()->AllocateScratchBloomFilter(
-              filter_ctxs_[i].filter->id());
-    } else {
-      DCHECK(filter_ctxs_[i].filter->is_min_max_filter());
-      filter_ctxs_[i].local_min_max_filter =
+              filter_ctx.filter->id());
+    } else if (filter_ctx.filter->is_min_max_filter()) {
+      filter_ctx.local_min_max_filter =
           runtime_state_->filter_bank()->AllocateScratchMinMaxFilter(
-              filter_ctxs_[i].filter->id(), filter_ctxs_[i].expr_eval->root().type());
+              filter_ctx.filter->id(), filter_ctx.expr_eval->root().type());
+      minmax_filter_ctxs_.push_back(&filter_ctx);
+    } else {
+      DCHECK(filter_ctx.filter->is_in_list_filter());
+      filter_ctx.local_in_list_filter =
+          runtime_state_->filter_bank()->AllocateScratchInListFilter(
+              filter_ctx.filter->id(), filter_ctx.expr_eval->root().type());
     }
   }
+
+  minmax_filter_threshold_ =
+      (float)(runtime_state_->query_options().minmax_filter_threshold);
 }
 
 void PhjBuilder::InsertRuntimeFilters(
@@ -922,70 +969,8 @@ void PhjBuilder::InsertRuntimeFilters(
 }
 
 void PhjBuilder::PublishRuntimeFilters(int64_t num_build_rows) {
-  VLOG(3) << "Join builder (join_node_id_=" << join_node_id_ << ") publishing "
-          << filter_ctxs_.size() << " filters.";
-  int32_t num_enabled_filters = 0;
-  float threshold = (float)(runtime_state_->query_options().minmax_filter_threshold);
-  for (const FilterContext& ctx : filter_ctxs_) {
-    BloomFilter* bloom_filter = nullptr;
-    if (ctx.local_bloom_filter != nullptr) {
-      bloom_filter = ctx.local_bloom_filter;
-      ++num_enabled_filters;
-    } else if (ctx.local_min_max_filter != nullptr) {
-      /// Apply the column min/max stats (if applicable) to shut down the min/max
-      /// filter early by setting always true flag for the filter. Do this only if
-      /// the min/max filter is too close in area to the column stats of all target
-      /// scan columns.
-      const TRuntimeFilterDesc& filter_desc = ctx.filter->filter_desc();
-      VLOG(3) << "Check out the usefulness of the local minmax filter:"
-              << " id=" << ctx.filter->id()
-              << ", details=" << ctx.local_min_max_filter->DebugString()
-              << ", column stats:"
-              << " low=" << PrintTColumnValue(filter_desc.targets[0].low_value)
-              << ", high=" << PrintTColumnValue(filter_desc.targets[0].high_value)
-              << ", threshold=" << threshold
-              << ", #targets=" << filter_desc.targets.size();
-      bool all_overlap = true;
-      for (auto target_desc : filter_desc.targets) {
-        if (!FilterContext::ShouldRejectFilterBasedOnColumnStats(
-                target_desc, ctx.local_min_max_filter, threshold)) {
-          all_overlap = false;
-          break;
-        }
-      }
-      if (all_overlap) {
-        ctx.local_min_max_filter->SetAlwaysTrue();
-        VLOG(3) << "The local minmax filter is set to always true:"
-                << " id=" << ctx.filter->id();
-      }
-
-      if (!ctx.local_min_max_filter->AlwaysTrue()) {
-        ++num_enabled_filters;
-      }
-    }
-
-    runtime_state_->filter_bank()->UpdateFilterFromLocal(
-        ctx.filter->id(), bloom_filter, ctx.local_min_max_filter);
-
-    if ( ctx.local_min_max_filter != nullptr ) {
-      VLOG(3) << "HJBuilder published min/max filter: "
-              << " id=" << ctx.filter->id()
-              << ", details=" << ctx.local_min_max_filter->DebugString();
-    }
-  }
-
-  if (filter_ctxs_.size() > 0) {
-    string info_string;
-    if (num_enabled_filters == filter_ctxs_.size()) {
-      info_string = Substitute("$0 of $0 Runtime Filter$1 Published", filter_ctxs_.size(),
-          filter_ctxs_.size() == 1 ? "" : "s");
-    } else {
-      info_string = Substitute("$0 of $1 Runtime Filter$2 Published, $3 Disabled",
-          num_enabled_filters, filter_ctxs_.size(), filter_ctxs_.size() == 1 ? "" : "s",
-          filter_ctxs_.size() - num_enabled_filters);
-    }
-    profile()->AddInfoString("Runtime filters", info_string);
-  }
+  JoinBuilder::PublishRuntimeFilters(
+      filter_ctxs_, runtime_state_, minmax_filter_threshold_, num_build_rows);
 }
 
 Status PhjBuilder::BeginSpilledProbe(BufferPool::ClientHandle* probe_client,
