@@ -74,13 +74,15 @@ void LikePredicate::LikePrepareInternal(FunctionContext* context,
     re2::RE2 ends_with_re("(?:%+)([^%_]*)");
     re2::RE2 starts_with_re("([^%_]*)(?:%+)");
     re2::RE2 equals_re("([^%_]*)");
+    re2::RE2 ends_with_escaped_wildcard(".*\\\\%$");
     string pattern_str(pattern.ptr, pattern.len);
     string search_string;
     if (case_sensitive && RE2::FullMatch(pattern_str, substring_re, &search_string)) {
       state->SetSearchString(search_string);
       state->function_ = ConstantSubstringFn;
     } else if (case_sensitive &&
-        RE2::FullMatch(pattern_str, starts_with_re, &search_string)) {
+        RE2::FullMatch(pattern_str, starts_with_re, &search_string) &&
+        !RE2::FullMatch(pattern_str, ends_with_escaped_wildcard)) {
       state->SetSearchString(search_string);
       state->function_ = ConstantStartsWithFn;
     } else if (case_sensitive &&
