@@ -56,6 +56,7 @@ using impala_udf::StringVal;
 using impala_udf::DecimalVal;
 using impala_udf::DateVal;
 using impala_udf::CollectionVal;
+using impala_udf::StructVal;
 
 class FragmentState;
 struct LibCacheEntry;
@@ -232,6 +233,10 @@ class ScalarExpr : public Expr {
   friend class Predicate;
   friend class ScalarExprEvaluator;
   friend class ScalarFnCall;
+  friend class SlotRef;
+
+  // The ORC scanner builds search arguments based on the values it gets from Literal.
+  friend class HdfsOrcScanner;
 
   /// For BE tests
   friend class ExprTest;
@@ -242,7 +247,7 @@ class ScalarExpr : public Expr {
   /// nodes which need FunctionContext in the tree. 'next_fn_ctx_idx' is the index
   /// of the next available entry in the vector. It's updated as this function is
   /// called recursively down the tree.
-  void AssignFnCtxIdx(int* next_fn_ctx_idx);
+  virtual void AssignFnCtxIdx(int* next_fn_ctx_idx);
 
   int fn_ctx_idx() const { return fn_ctx_idx_; }
 
@@ -251,7 +256,7 @@ class ScalarExpr : public Expr {
   static Status CreateNode(const TExprNode& texpr_node, ObjectPool* pool,
       ScalarExpr** expr) WARN_UNUSED_RESULT;
 
-  ScalarExpr(const ColumnType& type, bool is_constant);
+  ScalarExpr(const ColumnType& type, bool is_constant, bool is_codegen_disabled = false);
   ScalarExpr(const TExprNode& node);
 
   /// Implementation of GetCodegendComputeFn() to be overridden by each subclass of
@@ -272,6 +277,7 @@ class ScalarExpr : public Expr {
   DoubleVal GetDoubleVal(ScalarExprEvaluator*, const TupleRow*) const;
   StringVal GetStringVal(ScalarExprEvaluator*, const TupleRow*) const;
   CollectionVal GetCollectionVal(ScalarExprEvaluator*, const TupleRow*) const;
+  StructVal GetStructVal(ScalarExprEvaluator*, const TupleRow*) const;
   TimestampVal GetTimestampVal(ScalarExprEvaluator*, const TupleRow*) const;
   DecimalVal GetDecimalVal(ScalarExprEvaluator*, const TupleRow*) const;
   DateVal GetDateVal(ScalarExprEvaluator*, const TupleRow*) const;
@@ -293,6 +299,8 @@ class ScalarExpr : public Expr {
   virtual StringVal GetStringValInterpreted(ScalarExprEvaluator*, const TupleRow*) const;
   virtual CollectionVal GetCollectionValInterpreted(
       ScalarExprEvaluator*, const TupleRow*) const;
+  virtual StructVal GetStructValInterpreted(
+      ScalarExprEvaluator*, const TupleRow*) const;
   virtual TimestampVal GetTimestampValInterpreted(
       ScalarExprEvaluator*, const TupleRow*) const;
   virtual DecimalVal GetDecimalValInterpreted(
@@ -386,6 +394,10 @@ class ScalarExpr : public Expr {
   /// Set in GetCodegendComputeFn().
   bool added_to_jit_ = false;
 
+  /// True if codegen should be disabled for this scalar expression. Typical use case
+  /// is const expressions in VALUES clause, which are evaluated only once.
+  const bool is_codegen_disabled_;
+
   /// Static wrappers which call the compute function of the given ScalarExpr, passing
   /// it the ScalarExprEvaluator and TupleRow. These are cross-compiled and used by
   /// GetStaticGetValWrapper.
