@@ -45,6 +45,10 @@ class SlotRef : public ScalarExpr {
   /// does not generate the appropriate exprs).
   SlotRef(const SlotDescriptor* desc, const ColumnType& type);
 
+  /// Create a SlotRef based on the given SlotDescriptor 'desc' and make sure the type is
+  /// not TYPE_NULL (if so, replaced it with TYPE_BOOLEAN).
+  static SlotRef* TypeSafeCreate(const SlotDescriptor* desc);
+
   /// Used for testing.  GetValue will return tuple + offset interpreted as 'type'
   SlotRef(const ColumnType& type, int offset, const bool nullable = false);
 
@@ -58,14 +62,24 @@ class SlotRef : public ScalarExpr {
   virtual int GetSlotIds(std::vector<SlotId>* slot_ids) const override;
   const SlotId& slot_id() const { return slot_id_; }
   static const char* LLVM_CLASS_NAME;
+  NullIndicatorOffset GetNullIndicatorOffset() const { return null_indicator_offset_; }
+  int GetSlotOffset() const { return slot_offset_; }
+  virtual const TupleDescriptor* GetCollectionTupleDesc() const override;
 
  protected:
   friend class ScalarExpr;
   friend class ScalarExprEvaluator;
 
+  /// For struct SlotRefs we need a FunctionContext so that we can use it later for
+  /// allocating memory to StructVals.
+  /// If this SlotRef is not a struct then the same function in ScalarExpr is called.
+  virtual void AssignFnCtxIdx(int* next_fn_ctx_idx) override;
+
   GENERATE_GET_VAL_INTERPRETED_OVERRIDES_FOR_ALL_SCALAR_TYPES
   virtual CollectionVal GetCollectionValInterpreted(
       ScalarExprEvaluator*, const TupleRow*) const override;
+  virtual StructVal GetStructValInterpreted(
+      ScalarExprEvaluator*, const TupleRow*) const override;
 
  private:
   int tuple_idx_;  // within row
@@ -73,6 +87,7 @@ class SlotRef : public ScalarExpr {
   NullIndicatorOffset null_indicator_offset_;  // within tuple
   const SlotId slot_id_;
   bool tuple_is_nullable_; // true if the tuple is nullable.
+  const SlotDescriptor* slot_desc_ = nullptr;
 };
 
 }
