@@ -20,7 +20,6 @@
 #include <ctime>
 #include <iomanip>
 
-#include <boost/date_time/compiler_config.hpp>
 #include <boost/date_time/posix_time/posix_time_types.hpp>
 #include <boost/date_time/gregorian/gregorian_types.hpp>
 #include <gutil/strings/substitute.h>
@@ -71,10 +70,7 @@ StringVal TimestampFunctions::StringValFromTimestamp(FunctionContext* context,
     }
   }
 
-  string formatted_timestamp = tv.Format(*dt_ctx);
-  if (formatted_timestamp.empty()) return StringVal::null();
-  StringVal result = AnyValUtil::FromString(context, formatted_timestamp);
-  return result;
+  return tv.ToStringVal(context, *dt_ctx);
 }
 
 template <class TIME>
@@ -82,8 +78,7 @@ StringVal TimestampFunctions::FromUnix(FunctionContext* context, const TIME& int
   if (intp.is_null) return StringVal::null();
   const TimestampValue tv = TimestampValue::FromUnixTime(intp.val,
       context->impl()->state()->time_zone_for_unix_time_conversions());
-  if (!tv.HasDateAndTime()) return StringVal::null();
-  return AnyValUtil::FromString(context, tv.ToString());
+  return tv.ToStringVal(context);
 }
 
 template <class TIME>
@@ -281,6 +276,47 @@ IntVal TimestampFunctions::Millisecond(FunctionContext* context,
   return IntVal(time.total_milliseconds() - time.total_seconds() * 1000);
 }
 
+bool StringToTimeOfDay(
+    const StringVal& str_val, boost::posix_time::time_duration* time) {
+  if (str_val.is_null) return false;
+  boost::gregorian::date dummy_date;
+  return TimestampParser::ParseSimpleDateFormat(
+      reinterpret_cast<char*>(str_val.ptr), str_val.len, &dummy_date, time, true);
+}
+
+IntVal TimestampFunctions::Hour(FunctionContext* context, const StringVal& str_val) {
+  boost::posix_time::time_duration time;
+  if (!StringToTimeOfDay(str_val, &time)) {
+    return IntVal::null();
+  }
+  return IntVal(time.hours());
+}
+
+IntVal TimestampFunctions::Minute(FunctionContext* context, const StringVal& str_val) {
+  boost::posix_time::time_duration time;
+  if (!StringToTimeOfDay(str_val, &time)) {
+    return IntVal::null();
+  }
+  return IntVal(time.minutes());
+}
+
+IntVal TimestampFunctions::Second(FunctionContext* context, const StringVal& str_val) {
+  boost::posix_time::time_duration time;
+  if (!StringToTimeOfDay(str_val, &time)) {
+    return IntVal::null();
+  }
+  return IntVal(time.seconds());
+}
+
+IntVal TimestampFunctions::Millisecond(
+    FunctionContext* context, const StringVal& str_val) {
+  boost::posix_time::time_duration time;
+  if (!StringToTimeOfDay(str_val, &time)) {
+    return IntVal::null();
+  }
+  return IntVal(time.total_milliseconds() - time.total_seconds() * 1000);
+}
+
 TimestampVal TimestampFunctions::Now(FunctionContext* context) {
   const TimestampValue* now = context->impl()->state()->now();
   TimestampVal return_val;
