@@ -29,17 +29,9 @@ template<class T>
 class ClientConnection;
 
 /// Common cache / connection types
-class ImpalaInternalServiceClient;
-typedef ClientCache<ImpalaInternalServiceClient> ImpalaInternalServiceClientCache;
-typedef ClientConnection<ImpalaInternalServiceClient> ImpalaInternalServiceConnection;
-
 class CatalogServiceClientWrapper;
 typedef ClientCache<CatalogServiceClientWrapper> CatalogServiceClientCache;
 typedef ClientConnection<CatalogServiceClientWrapper> CatalogServiceConnection;
-
-class ImpalaBackendClient;
-typedef ClientCache<ImpalaBackendClient> ImpalaBackendClientCache;
-typedef ClientConnection<ImpalaBackendClient> ImpalaBackendConnection;
 }
 
 #endif
