@@ -22,7 +22,6 @@
 #include <boost/algorithm/string.hpp>
 #include <gflags/gflags.h>
 #include <gutil/strings/substitute.h>
-#include <kudu/client/client.h>
 
 #include "catalog/catalog-service-client-wrapper.h"
 #include "common/logging.h"
@@ -101,21 +100,22 @@ DEFINE_int32(admission_control_slots, 0,
     "that it needs. Defaults to number of cores / -num_cores for executors, and 8x that "
     "value for dedicated coordinators).");
 
-DEFINE_bool_hidden(use_local_catalog, false,
-    "Use experimental implementation of a local catalog. If this is set, "
-    "the catalog service is not used and does not need to be started.");
-DEFINE_int32_hidden(local_catalog_cache_mb, -1,
+DEFINE_bool(use_local_catalog, false,
+    "Use the on-demand metadata feature in coordinators. If this is set, coordinators "
+    "pull metadata as needed from catalogd and cache it locally. The cached metadata "
+    "gets evicted automatically under memory pressure or after an expiration time.");
+DEFINE_int32(local_catalog_cache_mb, -1,
     "If --use_local_catalog is enabled, configures the size of the catalog "
     "cache within each impalad. If this is set to -1, the cache is auto-"
     "configured to 60% of the configured Java heap size. Note that the Java "
     "heap size is distinct from and typically smaller than the overall "
     "Impala memory limit.");
-DEFINE_int32_hidden(local_catalog_cache_expiration_s, 60 * 60,
+DEFINE_int32(local_catalog_cache_expiration_s, 60 * 60,
     "If --use_local_catalog is enabled, configures the expiration time "
     "of the catalog cache within each impalad. Even if the configured "
     "cache capacity has not been reached, items are removed from the cache "
     "if they have not been accessed in this amount of time.");
-DEFINE_int32_hidden(local_catalog_max_fetch_retries, 40,
+DEFINE_int32(local_catalog_max_fetch_retries, 40,
     "If --use_local_catalog is enabled, configures the maximum number of times "
     "the frontend retries when fetching a metadata object from the impalad "
     "coordinator's local catalog cache.");
@@ -175,11 +175,11 @@ using namespace impala;
 /// Helper method to forward cluster membership updates to the frontend.
 /// For additional details see comments for PopulateExecutorMembershipRequest()
 /// in cluster-membership-mgr.cc
-void SendClusterMembershipToFrontend(
-    ClusterMembershipMgr::SnapshotPtr& snapshot, Frontend* frontend) {
+void SendClusterMembershipToFrontend(ClusterMembershipMgr::SnapshotPtr& snapshot,
+    const vector<TExecutorGroupSet>& expected_exec_group_sets, Frontend* frontend) {
   TUpdateExecutorMembershipRequest update_req;
 
-  PopulateExecutorMembershipRequest(snapshot, update_req);
+  PopulateExecutorMembershipRequest(snapshot, expected_exec_group_sets, update_req);
 
   Status status = frontend->UpdateExecutorMembership(update_req);
   if (!status.ok()) {
@@ -190,10 +190,6 @@ void SendClusterMembershipToFrontend(
 
 namespace impala {
 
-struct ExecEnv::KuduClientPtr {
-  kudu::client::sp::shared_ptr<kudu::client::KuduClient> kudu_client;
-};
-
 ExecEnv* ExecEnv::exec_env_ = nullptr;
 
 ExecEnv::ExecEnv(bool external_fe)
@@ -440,7 +436,9 @@ Status ExecEnv::Init() {
   if (FLAGS_is_coordinator && frontend_ != nullptr) {
     cluster_membership_mgr_->RegisterUpdateCallbackFn(
         [this](ClusterMembershipMgr::SnapshotPtr snapshot) {
-          SendClusterMembershipToFrontend(snapshot, this->frontend());
+          SendClusterMembershipToFrontend(snapshot,
+              this->cluster_membership_mgr()->GetExpectedExecGroupSets(),
+              this->frontend());
         });
   }
 
@@ -581,20 +579,21 @@ void ExecEnv::InitSystemStateInfo() {
   });
 }
 
-Status ExecEnv::GetKuduClient(
-    const vector<string>& master_addresses, kudu::client::KuduClient** client) {
+Status ExecEnv::GetKuduClient(const vector<string>& master_addresses,
+    kudu::client::sp::shared_ptr<kudu::client::KuduClient>* client) {
   string master_addr_concat = join(master_addresses, ",");
   lock_guard<SpinLock> l(kudu_client_map_lock_);
   auto kudu_client_map_it = kudu_client_map_.find(master_addr_concat);
   if (kudu_client_map_it == kudu_client_map_.end()) {
-    // KuduClient doesn't exist, create it
-    KuduClientPtr* kudu_client_ptr = new KuduClientPtr;
-    RETURN_IF_ERROR(CreateKuduClient(master_addresses, &kudu_client_ptr->kudu_client));
-    kudu_client_map_[master_addr_concat].reset(kudu_client_ptr);
-    *client = kudu_client_ptr->kudu_client.get();
+    // KuduClient doesn't exist, create it.
+    LOG(INFO) << "Creating a new KuduClient for masters=" << master_addr_concat;
+    kudu::client::sp::shared_ptr<kudu::client::KuduClient> kudu_client;
+    RETURN_IF_ERROR(CreateKuduClient(master_addresses, &kudu_client));
+    kudu_client_map_.insert(make_pair(master_addr_concat, kudu_client));
+    *client = kudu_client;
   } else {
     // Return existing KuduClient
-    *client = kudu_client_map_it->second->kudu_client.get();
+    *client = kudu_client_map_it->second;
   }
   return Status::OK();
 }
