@@ -17,15 +17,16 @@
 
 #include <algorithm>
 #include <boost/bind.hpp>
+#include <boost/filesystem.hpp>
 #include <fstream>
 #include <gflags/gflags.h>
-#include <iostream>
 #include <rapidjson/document.h>
 #include <sys/sysinfo.h>
 
 #include "gutil/strings/join.h"
 #include "gutil/strings/util.h"
 #include "runtime/io/data-cache.h"
+#include "runtime/io/data-cache-trace.h"
 #include "runtime/io/request-ranges.h"
 #include "runtime/test-env.h"
 #include "service/fe-support.h"
@@ -33,18 +34,27 @@
 #include "testutil/scoped-flag-setter.h"
 #include "util/counting-barrier.h"
 #include "util/filesystem-util.h"
+#include "util/simple-logger.h"
 #include "util/thread.h"
 
 #include "common/names.h"
 
 #define BASE_CACHE_DIR     "/tmp"
-#define DEFAULT_CACHE_SIZE (4 * 1024 * 1024)
 #define FNAME              ("foobar")
 #define NUM_TEST_DIRS      (16)
 #define NUM_THREADS        (18)
 #define MTIME              (12345)
 #define TEMP_BUFFER_SIZE   (4096)
 #define TEST_BUFFER_SIZE   (8192)
+#define NUM_CACHE_ENTRIES  (1024)
+#define DEFAULT_CACHE_SIZE (NUM_CACHE_ENTRIES * TEMP_BUFFER_SIZE)
+
+// The NUM_CACHE_ENTRIES_NO_EVICT says how many entries can be inserted into the cache
+// and queried without any evictions. This is smaller than NUM_CACHE_ENTRIES to
+// provide a bit of flexibility for cache implementations like LIRS that have multiple
+// segments. The segments may not split cleanly along entries of size TEMP_BUFFER_SIZE,
+// so it can evict something when adding NUM_CACHE_ENTRIES.
+#define NUM_CACHE_ENTRIES_NO_EVICT (NUM_CACHE_ENTRIES - 1)
 
 DECLARE_bool(cache_force_single_shard);
 DECLARE_bool(data_cache_anonymize_trace);
@@ -52,11 +62,17 @@ DECLARE_bool(data_cache_enable_tracing);
 DECLARE_int64(data_cache_file_max_size_bytes);
 DECLARE_int32(data_cache_max_opened_files);
 DECLARE_int32(data_cache_write_concurrency);
+DECLARE_string(data_cache_eviction_policy);
+DECLARE_string(data_cache_trace_dir);
+DECLARE_int32(max_data_cache_trace_file_size);
+DECLARE_int32(data_cache_trace_percentage);
 
 namespace impala {
 namespace io {
 
-class DataCacheTest : public testing::Test {
+using boost::filesystem::path;
+
+class DataCacheBaseTest : public testing::Test {
  public:
   const uint8_t* test_buffer() {
     return reinterpret_cast<const uint8_t*>(test_buffer_);
@@ -66,6 +82,10 @@ class DataCacheTest : public testing::Test {
     return data_cache_dirs_;
   }
 
+  const string& data_cache_trace_dir() {
+    return data_cache_trace_dir_;
+  }
+
   //
   // Use multiple threads to insert and read back a set of ranges from test_buffer().
   // Depending on the setting, the working set may or may not fit in the cache.
@@ -92,7 +112,7 @@ class DataCacheTest : public testing::Test {
       num_misses[i] = 0;
       string thread_name = Substitute("thread-$0", i);
       ASSERT_OK(Thread::Create("data-cache-test", thread_name,
-          boost::bind(&DataCacheTest::ThreadFn, this,
+          boost::bind(&DataCacheBaseTest::ThreadFn, this,
              use_per_thread_filename ? thread_name : "", cache, max_start_offset,
              &barrier, &num_misses[i]), &thread));
       threads.emplace_back(std::move(thread));
@@ -113,7 +133,7 @@ class DataCacheTest : public testing::Test {
   }
 
  protected:
-  DataCacheTest() {
+  DataCacheBaseTest() {
     // Create a buffer of random characters.
     for (int i = 0; i < TEST_BUFFER_SIZE; ++i) {
       test_buffer_[i] = '!' + (rand() % 93);
@@ -121,7 +141,7 @@ class DataCacheTest : public testing::Test {
   }
 
   // Create a bunch of test directories in which the data cache will reside.
-  virtual void SetUp() {
+  void SetupWithParameters(std::string eviction_policy) {
     test_env_.reset(new TestEnv());
     flag_saver_.reset(new google::FlagSaver());
     ASSERT_OK(test_env_->Init());
@@ -130,6 +150,8 @@ class DataCacheTest : public testing::Test {
       ASSERT_OK(FileSystemUtil::RemoveAndCreateDirectory(path));
       data_cache_dirs_.push_back(path);
     }
+    data_cache_trace_dir_ = ("/tmp" / boost::filesystem::unique_path()).string();
+    ASSERT_OK(FileSystemUtil::RemoveAndCreateDirectory(data_cache_trace_dir_));
 
     // Force a single shard to avoid imbalance between shards in the Kudu LRU cache which
     // may lead to unintended eviction. This allows the capacity of the cache is utilized
@@ -138,6 +160,9 @@ class DataCacheTest : public testing::Test {
 
     // Allows full write concurrency for the multi-threaded tests.
     FLAGS_data_cache_write_concurrency = NUM_THREADS;
+
+    // This is parameterized to allow testing LRU and LIRS
+    FLAGS_data_cache_eviction_policy = eviction_policy;
   }
 
   // Delete all the test directories created.
@@ -147,13 +172,10 @@ class DataCacheTest : public testing::Test {
     for (const string& dir_path : data_cache_dirs_) {
       vector<string> entries;
       ASSERT_OK(FileSystemUtil::Directory::GetEntryNames(dir_path, &entries));
-      if (entries.size() == 1) {
-        EXPECT_EQ(DataCache::Partition::TRACE_FILE_NAME, entries[0]);
-      } else {
-        ASSERT_EQ(0, entries.size());
-      }
+      ASSERT_EQ(0, entries.size());
     }
     ASSERT_OK(FileSystemUtil::RemovePaths(data_cache_dirs_));
+    boost::filesystem::remove_all(path(data_cache_trace_dir_));
     flag_saver_.reset();
     test_env_.reset();
   }
@@ -162,6 +184,7 @@ class DataCacheTest : public testing::Test {
   std::unique_ptr<TestEnv> test_env_;
   char test_buffer_[TEST_BUFFER_SIZE];
   vector<string> data_cache_dirs_;
+  string data_cache_trace_dir_;
 
   // Saved configuration flags for restoring the values at the end of the test.
   std::unique_ptr<google::FlagSaver> flag_saver_;
@@ -208,10 +231,27 @@ class DataCacheTest : public testing::Test {
   }
 };
 
+class DataCacheTest :
+    public DataCacheBaseTest,
+    public ::testing::WithParamInterface<std::string> {
+ public:
+  DataCacheTest()
+      : DataCacheBaseTest() {
+  }
+
+  void SetUp() override {
+    const auto& param = GetParam();
+    SetupWithParameters(param);
+  }
+};
+
+INSTANTIATE_TEST_CASE_P(DataCacheTestTypes, DataCacheTest,
+    ::testing::Values("LRU", "LIRS"));
+
 // This test exercises the basic insertion and lookup paths by inserting a known set of
 // offsets which fit in the cache entirely. Also tries reading entries which are never
 // inserted into the cache.
-TEST_F(DataCacheTest, TestBasics) {
+TEST_P(DataCacheTest, TestBasics) {
   const int64_t cache_size = DEFAULT_CACHE_SIZE;
   DataCache cache(Substitute("$0:$1", data_cache_dirs()[0], std::to_string(cache_size)));
   ASSERT_OK(cache.Init());
@@ -221,7 +261,7 @@ TEST_F(DataCacheTest, TestBasics) {
   // Read and then insert a range of offsets. Expected all misses in the first iteration
   // and all hits in the second iteration.
   for (int i = 0; i < 2; ++i) {
-    for (int64_t offset = 0; offset < 1024; ++offset) {
+    for (int64_t offset = 0; offset < NUM_CACHE_ENTRIES_NO_EVICT; ++offset) {
       int expected_bytes = i * TEMP_BUFFER_SIZE;
       memset(buffer, 0, TEMP_BUFFER_SIZE);
       ASSERT_EQ(expected_bytes,
@@ -236,24 +276,27 @@ TEST_F(DataCacheTest, TestBasics) {
   }
 
   // Read the same range inserted previously but with a different filename.
-  for (int64_t offset = 1024; offset < TEST_BUFFER_SIZE; ++offset) {
+  for (int64_t offset = NUM_CACHE_ENTRIES_NO_EVICT; offset < TEST_BUFFER_SIZE;
+       ++offset) {
     const string& alt_fname = "random";
     ASSERT_EQ(0, cache.Lookup(alt_fname, MTIME, offset, TEMP_BUFFER_SIZE, buffer));
   }
 
   // Read the same range inserted previously but with a different mtime.
-  for (int64_t offset = 1024; offset < TEST_BUFFER_SIZE; ++offset) {
+  for (int64_t offset = NUM_CACHE_ENTRIES_NO_EVICT; offset < TEST_BUFFER_SIZE;
+       ++offset) {
     int64_t alt_mtime = 67890;
     ASSERT_EQ(0, cache.Lookup(FNAME, alt_mtime, offset, TEMP_BUFFER_SIZE, buffer));
   }
 
   // Read a range of offsets which should miss in the cache.
-  for (int64_t offset = 1024; offset < TEST_BUFFER_SIZE; ++offset) {
+  for (int64_t offset = NUM_CACHE_ENTRIES_NO_EVICT; offset < TEST_BUFFER_SIZE;
+       ++offset) {
     ASSERT_EQ(0, cache.Lookup(FNAME, MTIME, offset, TEMP_BUFFER_SIZE, buffer));
   }
 
   // Read the same same range inserted previously. They should still all be in the cache.
-  for (int64_t offset = 0; offset < 1024; ++offset) {
+  for (int64_t offset = 0; offset < NUM_CACHE_ENTRIES_NO_EVICT; ++offset) {
     memset(buffer, 0, TEMP_BUFFER_SIZE);
     ASSERT_EQ(TEMP_BUFFER_SIZE,
         cache.Lookup(FNAME, MTIME, offset, TEMP_BUFFER_SIZE + 10, buffer));
@@ -296,17 +339,17 @@ TEST_F(DataCacheTest, TestBasics) {
 
 // Tests backing file rotation by setting FLAGS_data_cache_file_max_size_bytes to be 1/4
 // of the cache size. This forces rotation of backing files.
-TEST_F(DataCacheTest, RotateFiles) {
+TEST_P(DataCacheTest, RotateFiles) {
   // Set the maximum size of backing files to be 1/4 of the cache size.
-  FLAGS_data_cache_file_max_size_bytes = 1024 * 1024;
-  const int64_t cache_size = 4 * FLAGS_data_cache_file_max_size_bytes ;
+  FLAGS_data_cache_file_max_size_bytes = DEFAULT_CACHE_SIZE / 4;
+  const int64_t cache_size = DEFAULT_CACHE_SIZE;
   DataCache cache(Substitute("$0:$1", data_cache_dirs()[0], std::to_string(cache_size)));
   ASSERT_OK(cache.Init());
 
   // Read and then insert a range of offsets. Expected all misses in the first iteration
   // and all hits in the second iteration.
   for (int i = 0; i < 2; ++i) {
-    for (int64_t offset = 0; offset < 1024; ++offset) {
+    for (int64_t offset = 0; offset < NUM_CACHE_ENTRIES_NO_EVICT; ++offset) {
       int expected_bytes = i * TEMP_BUFFER_SIZE;
       uint8_t buffer[TEMP_BUFFER_SIZE];
       memset(buffer, 0, TEMP_BUFFER_SIZE);
@@ -335,13 +378,13 @@ TEST_F(DataCacheTest, RotateFiles) {
 // of the cache size. This forces rotation of backing files. Also sets
 // --data_cache_max_opened_files to 1 so that only one underlying
 // file is allowed. This exercises the lazy deletion path.
-TEST_F(DataCacheTest, RotateAndDeleteFiles) {
+TEST_P(DataCacheTest, RotateAndDeleteFiles) {
   // Set the maximum size of backing files to be 1/4 of the cache size.
-  FLAGS_data_cache_file_max_size_bytes = 1024 * 1024;
+  FLAGS_data_cache_file_max_size_bytes = DEFAULT_CACHE_SIZE / 4;
   // Force to allow one backing file.
   FLAGS_data_cache_max_opened_files = 1;
 
-  const int64_t cache_size = 4 * FLAGS_data_cache_file_max_size_bytes ;
+  const int64_t cache_size = DEFAULT_CACHE_SIZE;
   DataCache cache(Substitute("$0:$1", data_cache_dirs()[0], std::to_string(cache_size)));
   ASSERT_OK(cache.Init());
 
@@ -361,12 +404,18 @@ TEST_F(DataCacheTest, RotateAndDeleteFiles) {
   // is still in the cache.
   int64_t in_cache_offset =
       1024 - FLAGS_data_cache_file_max_size_bytes / TEMP_BUFFER_SIZE;
+  int64_t num_entries_hit = 0;
   for (int64_t offset = in_cache_offset ; offset < 1024; ++offset) {
     memset(buffer, 0, TEMP_BUFFER_SIZE);
-    ASSERT_EQ(TEMP_BUFFER_SIZE, cache.Lookup(FNAME, MTIME, offset,
-        TEMP_BUFFER_SIZE, buffer));
-    ASSERT_EQ(0, memcmp(buffer, test_buffer() + offset, TEMP_BUFFER_SIZE));
+    int64_t lookup_size = cache.Lookup(FNAME, MTIME, offset,
+        TEMP_BUFFER_SIZE, buffer);
+    if (lookup_size != 0) {
+      ++num_entries_hit;
+      EXPECT_EQ(TEMP_BUFFER_SIZE, lookup_size);
+      EXPECT_EQ(0, memcmp(buffer, test_buffer() + offset, TEMP_BUFFER_SIZE));
+    }
   }
+  EXPECT_GE(num_entries_hit, NUM_CACHE_ENTRIES / 4 - 1);
 
   // Make sure only one backing file exists. Allow for 10 seconds latency for the
   // file deleter thread to run.
@@ -386,7 +435,9 @@ TEST_F(DataCacheTest, RotateAndDeleteFiles) {
 
 // Tests eviction in the cache by inserting a large entry which evicts all existing
 // entries in the cache.
-TEST_F(DataCacheTest, Eviction) {
+TEST_P(DataCacheTest, LRUEviction) {
+  // This test is specific to LRU
+  if (FLAGS_data_cache_eviction_policy != "LRU") return;
   const int64_t cache_size = DEFAULT_CACHE_SIZE;
   DataCache cache(Substitute("$0:$1", data_cache_dirs()[0], std::to_string(cache_size)));
   ASSERT_OK(cache.Init());
@@ -439,12 +490,12 @@ TEST_F(DataCacheTest, Eviction) {
 // Tests insertion and lookup with the cache with multiple threads.
 // Inserts a working set which will fit in the cache. Despite potential
 // collision during insertion, all entries in the working set should be found.
-TEST_F(DataCacheTest, MultiThreadedNoMisses) {
+TEST_P(DataCacheTest, MultiThreadedNoMisses) {
   int64_t cache_size = DEFAULT_CACHE_SIZE;
   DataCache cache(Substitute("$0:$1", data_cache_dirs()[0], std::to_string(cache_size)));
   ASSERT_OK(cache.Init());
 
-  int64_t max_start_offset = 1024;
+  int64_t max_start_offset = NUM_CACHE_ENTRIES_NO_EVICT;
   bool use_per_thread_filename = false;
   bool expect_misses = false;
   MultiThreadedReadWrite(&cache, max_start_offset, use_per_thread_filename,
@@ -453,7 +504,7 @@ TEST_F(DataCacheTest, MultiThreadedNoMisses) {
 
 // Inserts a working set which is known to be larger than the cache's capacity.
 // Expect some cache misses in lookups.
-TEST_F(DataCacheTest, MultiThreadedWithMisses) {
+TEST_P(DataCacheTest, MultiThreadedWithMisses) {
   int64_t cache_size = DEFAULT_CACHE_SIZE;
   DataCache cache(Substitute("$0:$1", data_cache_dirs()[0], std::to_string(cache_size)));
   ASSERT_OK(cache.Init());
@@ -466,7 +517,7 @@ TEST_F(DataCacheTest, MultiThreadedWithMisses) {
 }
 
 // Test insertion and lookup with a cache configured with multiple partitions.
-TEST_F(DataCacheTest, MultiPartitions) {
+TEST_P(DataCacheTest, MultiPartitions) {
   StringPiece delimiter(",");
   string cache_base = JoinStrings(data_cache_dirs(), delimiter);
   const int64_t cache_size = DEFAULT_CACHE_SIZE;
@@ -483,7 +534,7 @@ TEST_F(DataCacheTest, MultiPartitions) {
 // Tests insertion of a working set whose size is 1/8 of the total memory size.
 // This likely exceeds the size of the page cache and forces write back of dirty pages in
 // the page cache to the backing files and also read from the backing files during lookup.
-TEST_F(DataCacheTest, LargeFootprint) {
+TEST_P(DataCacheTest, LargeFootprint) {
   struct sysinfo info;
   ASSERT_EQ(0, sysinfo(&info));
   ASSERT_GT(info.totalram, 0);
@@ -506,55 +557,125 @@ TEST_F(DataCacheTest, LargeFootprint) {
   }
 }
 
-TEST_F(DataCacheTest, TestAccessTrace) {
+TEST_P(DataCacheTest, AccessTraceAnonymization) {
   FLAGS_data_cache_enable_tracing = true;
+  FLAGS_data_cache_trace_dir = data_cache_trace_dir();
+  // Use a small trace file size so that the trace is split over multiple files
+  FLAGS_max_data_cache_trace_file_size = 10000;
+  const int64_t cache_size = DEFAULT_CACHE_SIZE;
+  int64_t max_start_offset = 1024;
+  uint64_t expected_total_events = NUM_THREADS * max_start_offset * 2;
   for (bool anon : { false, true }) {
     SCOPED_TRACE(anon);
     FLAGS_data_cache_anonymize_trace = anon;
+    // Remove files between iterations to avoid crosstalk between the anonymized
+    // iteration and non-anonymized iteration
+    ASSERT_OK(FileSystemUtil::RemoveAndCreateDirectory(FLAGS_data_cache_trace_dir));
     {
-      int64_t cache_size = DEFAULT_CACHE_SIZE;
       DataCache cache(Substitute(
           "$0:$1", data_cache_dirs()[0], std::to_string(cache_size)));
       ASSERT_OK(cache.Init());
 
-      int64_t max_start_offset = 1024;
       bool use_per_thread_filename = true;
       bool expect_misses = true;
       MultiThreadedReadWrite(&cache, max_start_offset, use_per_thread_filename,
                              expect_misses);
     }
 
-    // Read the trace file and ensure that all of the lines are valid JSON with the
-    // expected fields.
-    std::ifstream trace(Substitute("$0/$1", data_cache_dirs()[0],
-                                   DataCache::Partition::TRACE_FILE_NAME));
-    int line_num = 1;
-    while (trace.good()) {
-      SCOPED_TRACE(line_num++);
-      string line;
-      std::getline(trace, line);
-      if (line.empty()) {
-        ASSERT_TRUE(trace.eof());
-        break;
-      }
-      SCOPED_TRACE(line);
-      rapidjson::Document d;
-      d.Parse<0>(line.c_str());
-      ASSERT_TRUE(d.IsObject());
-      ASSERT_TRUE(d["ts"].IsDouble());
-      ASSERT_TRUE(d["s"].IsString());
-      ASSERT_TRUE(d["m"].IsInt64());
-      ASSERT_TRUE(d["f"].IsString());
-      if (anon) {
-        // We expect anonymized filenames to be 22-character fingerprints:
-        // - 128 bit fingerprint = 16 bytes
-        // - 16 * 4/3 (base64 encoding) = 21.3333
-        // - Round up to 22.
-        EXPECT_EQ(22, d["f"].GetStringLength());
-      } else {
-        EXPECT_TRUE(MatchPattern(d["f"].GetString(), "thread-*file"));
+    // Part 1: Read the trace files and ensure that the JSON entries are valid
+    // TraceEvent entries
+    vector<string> trace_files;
+    string trace_dir = Substitute("$0/partition-0/", FLAGS_data_cache_trace_dir);
+    ASSERT_OK(SimpleLogger::GetLogFiles(trace_dir, trace::TRACE_FILE_PREFIX,
+        &trace_files));
+
+    for (string filename : trace_files) {
+      trace::TraceFileIterator trace_file_iter(filename);
+      ASSERT_OK(trace_file_iter.Init());
+      while (true) {
+        trace::TraceEvent trace_event;
+        bool done = false;
+        ASSERT_OK(trace_file_iter.GetNextEvent(&trace_event, &done));
+        if (done) break;
+        if (anon) {
+          // We expect anonymized filenames to be 22-character fingerprints:
+          // - 128 bit fingerprint = 16 bytes
+          // - 16 * 4/3 (base64 encoding) = 21.3333
+          // - Round up to 22.
+          EXPECT_EQ(22, trace_event.filename.size()) << trace_event.filename;
+        } else {
+          EXPECT_TRUE(MatchPattern(trace_event.filename, "thread-*file"));
+        }
       }
     }
+
+    // Part 2: Create a replayer and verify that it can successfully replay from that
+    // directory.
+    trace::TraceReplayer replayer(Substitute("/this_directory_doesnt_matter:$0",
+        std::to_string(cache_size)));
+    EXPECT_OK(replayer.Init());
+    EXPECT_OK(replayer.ReplayDirectory(trace_dir));
+    trace::CacheHitStatistics replay_stats = replayer.GetReplayStatistics();
+    int64_t total_events = replay_stats.hits + replay_stats.partial_hits +
+      replay_stats.misses + replay_stats.stores + replay_stats.failed_stores;
+    EXPECT_EQ(total_events, expected_total_events);
+  }
+}
+
+TEST_P(DataCacheTest, AccessTraceSubsetPercentage) {
+  FLAGS_data_cache_enable_tracing = true;
+  FLAGS_data_cache_trace_dir = data_cache_trace_dir();
+  // Use a small trace file size so that the trace is split over multiple files
+  FLAGS_max_data_cache_trace_file_size = 10000;
+  const int64_t cache_size = DEFAULT_CACHE_SIZE;
+  int64_t max_start_offset = 1024;
+  // Each thread does a store and a lookup for each offset
+  uint64_t expected_total_events = NUM_THREADS * max_start_offset * 2;
+  for (int trace_percentage : {100, 50, 20, 10, 5, 1}) {
+    SCOPED_TRACE(trace_percentage);
+    FLAGS_data_cache_trace_percentage = trace_percentage;
+    // Remove files between iterations to avoid crosstalk between the anonymized
+    // iteration and non-anonymized iteration
+    ASSERT_OK(FileSystemUtil::RemoveAndCreateDirectory(FLAGS_data_cache_trace_dir));
+    {
+      DataCache cache(Substitute(
+          "$0:$1", data_cache_dirs()[0], std::to_string(cache_size)));
+      ASSERT_OK(cache.Init());
+
+      bool use_per_thread_filename = true;
+      bool expect_misses = true;
+      MultiThreadedReadWrite(&cache, max_start_offset, use_per_thread_filename,
+                             expect_misses);
+    }
+
+    // Replay the trace and record the counts
+    string trace_dir = Substitute("$0/partition-0/", FLAGS_data_cache_trace_dir);
+    trace::TraceReplayer replayer(Substitute("/this_directory_doesnt_matter:$0",
+        std::to_string(cache_size)));
+    EXPECT_OK(replayer.Init());
+    EXPECT_OK(replayer.ReplayDirectory(trace_dir));
+
+    // The trace percentage is enforced when generating the trace, so verify that
+    // the number of events in the original trace is within +/-1% of the expected value.
+    // This is deterministic, because the actual cache entries accessed are always
+    // the same (even if the order is not).
+    trace::CacheHitStatistics original_trace_stats =
+        replayer.GetOriginalTraceStatistics();
+    int64_t num_original_trace_events = original_trace_stats.hits +
+      original_trace_stats.partial_hits + original_trace_stats.misses +
+      original_trace_stats.stores + original_trace_stats.failed_stores;
+    EXPECT_LE(100 * num_original_trace_events,
+              (expected_total_events * (trace_percentage + 1)));
+    EXPECT_GE(100 * num_original_trace_events,
+              (expected_total_events * (trace_percentage - 1)));
+    LOG(INFO) << "Trace percentage " << trace_percentage << " num events: "
+              << num_original_trace_events;
+
+    // The replay should have the same number of events.
+    trace::CacheHitStatistics replay_stats = replayer.GetReplayStatistics();
+    int64_t num_replay_trace_events = replay_stats.hits + replay_stats.partial_hits +
+      replay_stats.misses + replay_stats.stores + replay_stats.failed_stores;
+    EXPECT_EQ(num_original_trace_events, num_replay_trace_events);
   }
 }
 
