@@ -124,6 +124,9 @@ DEFINE_int32(num_s3_file_oper_io_threads, 16, "Number of S3 file operations I/O
 // The maximum number of ABFS I/O threads. TODO: choose the default empirically.
 DEFINE_int32(num_abfs_io_threads, 16, "Number of ABFS I/O threads");
 
+// The maximum number of OSS/JindoFS I/O threads. TODO: choose the default empirically.
+DEFINE_int32(num_oss_io_threads, 16, "Number of OSS/JindoFS I/O threads");
+
 // The maximum number of ADLS I/O threads. This number is a good default to have for
 // clusters that may vary widely in size, due to an undocumented concurrency limit
 // enforced by ADLS for a cluster, which spans between 500-700. For smaller clusters
@@ -133,9 +136,15 @@ DEFINE_int32(num_adls_io_threads, 16, "Number of ADLS I/O threads");
 // The maximum number of GCS I/O threads. TODO: choose the default empirically.
 DEFINE_int32(num_gcs_io_threads, 16, "Number of GCS I/O threads");
 
+// The maximum number of GCS I/O threads. TODO: choose the default empirically.
+DEFINE_int32(num_cos_io_threads, 16, "Number of COS I/O threads");
+
 // The maximum number of Ozone I/O threads. TODO: choose the default empirically.
 DEFINE_int32(num_ozone_io_threads, 16, "Number of Ozone I/O threads");
 
+// The maximum number of SFS I/O threads.
+DEFINE_int32(num_sfs_io_threads, 16, "Number of SFS I/O threads");
+
 // The number of cached file handles defines how much memory can be used per backend for
 // caching frequently used file handles. Measurements indicate that a single file handle
 // uses about 6kB of memory. 20k file handles will thus reserve ~120MB of memory.
@@ -468,9 +477,15 @@ Status DiskIoMgr::Init() {
     } else if (i == RemoteAdlsDiskId()) {
       num_threads_per_disk = FLAGS_num_adls_io_threads;
       device_name = "ADLS remote";
+    } else if (i == RemoteOSSDiskId()) {
+      num_threads_per_disk = FLAGS_num_oss_io_threads;
+      device_name = "OSS remote";
     } else if (i == RemoteGcsDiskId()) {
       num_threads_per_disk = FLAGS_num_gcs_io_threads;
       device_name = "GCS remote";
+    }  else if (i == RemoteCosDiskId()) {
+      num_threads_per_disk = FLAGS_num_cos_io_threads;
+      device_name = "COS remote";
     } else if (i == RemoteOzoneDiskId()) {
       num_threads_per_disk = FLAGS_num_ozone_io_threads;
       device_name = "Ozone remote";
@@ -480,6 +495,9 @@ Status DiskIoMgr::Init() {
     } else if (i == RemoteS3DiskFileOperId()) {
       num_threads_per_disk = FLAGS_num_s3_file_oper_io_threads;
       device_name = "S3 remote file operations";
+    } else if (i == RemoteSFSDiskId()) {
+      num_threads_per_disk = FLAGS_num_sfs_io_threads;
+      device_name = "SFS remote";
     } else if (DiskInfo::is_rotational(i)) {
       num_threads_per_disk = num_io_threads_per_rotational_disk_;
       // During tests, i may not point to an existing disk.
@@ -617,25 +635,8 @@ Status DiskIoMgr::ValidateScanRange(ScanRange* range) {
 
 Status DiskIoMgr::AllocateBuffersForRange(
     BufferPool::ClientHandle* bp_client, ScanRange* range, int64_t max_bytes) {
-  DCHECK_GE(max_bytes, min_buffer_size_);
-  DCHECK(range->external_buffer_tag() == ScanRange::ExternalBufferTag::NO_BUFFER)
-     << static_cast<int>(range->external_buffer_tag()) << " invalid to allocate buffers "
-     << "when already reading into an external buffer";
-  BufferPool* bp = ExecEnv::GetInstance()->buffer_pool();
-  Status status;
-  vector<unique_ptr<BufferDescriptor>> buffers;
-  for (int64_t buffer_size : ChooseBufferSizes(range->bytes_to_read(), max_bytes)) {
-    BufferPool::BufferHandle handle;
-    status = bp->AllocateBuffer(bp_client, buffer_size, &handle);
-    if (!status.ok()) goto error;
-    buffers.emplace_back(new BufferDescriptor(range, bp_client, move(handle)));
-  }
-  range->AddUnusedBuffers(move(buffers), false);
-  return Status::OK();
- error:
-  DCHECK(!status.ok());
-  range->CleanUpBuffers(move(buffers));
-  return status;
+  return range->AllocateBuffersForRange(bp_client, max_bytes,
+      min_buffer_size_, max_buffer_size_);
 }
 
 vector<int64_t> DiskIoMgr::ChooseBufferSizes(int64_t scan_range_len, int64_t max_bytes) {
@@ -826,15 +827,21 @@ int DiskIoMgr::AssignQueue(
     if (IsS3APath(file, check_default_fs)) return RemoteS3DiskId();
     if (IsABFSPath(file, check_default_fs)) return RemoteAbfsDiskId();
     if (IsADLSPath(file, check_default_fs)) return RemoteAdlsDiskId();
+    if (IsOSSPath(file, check_default_fs)) return RemoteOSSDiskId();
     if (IsGcsPath(file, check_default_fs)) return RemoteGcsDiskId();
+    if (IsCosPath(file, check_default_fs)) return RemoteCosDiskId();
     if (IsOzonePath(file, check_default_fs)) return RemoteOzoneDiskId();
+    if (IsSFSPath(file, check_default_fs)) return RemoteSFSDiskId();
   }
   // Assign to a local disk queue.
   DCHECK(!IsS3APath(file, check_default_fs)); // S3 is always remote.
   DCHECK(!IsABFSPath(file, check_default_fs)); // ABFS is always remote.
   DCHECK(!IsADLSPath(file, check_default_fs)); // ADLS is always remote.
+  DCHECK(!IsOSSPath(file, check_default_fs)); // OSS/JindoFS is always remote.
   DCHECK(!IsGcsPath(file, check_default_fs)); // GCS is always remote.
+  DCHECK(!IsCosPath(file, check_default_fs)); // COS is always remote.
   DCHECK(!IsOzonePath(file, check_default_fs)); // Ozone is always remote.
+  DCHECK(!IsSFSPath(file, check_default_fs)); // SFS is always remote.
   if (disk_id == -1) {
     // disk id is unknown, assign it an arbitrary one.
     disk_id = next_disk_id_.Add(1);
@@ -866,14 +873,12 @@ void DiskIoMgr::ReleaseExclusiveHdfsFileHandle(unique_ptr<ExclusiveHdfsFileHandl
   fid.reset();
 }
 
-Status DiskIoMgr::GetCachedHdfsFileHandle(const hdfsFS& fs,
-    std::string* fname, int64_t mtime, RequestContext *reader,
-    CachedHdfsFileHandle** handle_out) {
+Status DiskIoMgr::GetCachedHdfsFileHandle(const hdfsFS& fs, std::string* fname,
+    int64_t mtime, RequestContext* reader, FileHandleCache::Accessor* accessor) {
   bool cache_hit;
   SCOPED_TIMER(reader->open_file_timer_);
-  RETURN_IF_ERROR(file_handle_cache_.GetFileHandle(fs, fname, mtime, false, handle_out,
-      &cache_hit));
-  ImpaladMetrics::IO_MGR_NUM_FILE_HANDLES_OUTSTANDING->Increment(1L);
+  RETURN_IF_ERROR(
+      file_handle_cache_.GetFileHandle(fs, fname, mtime, false, accessor, &cache_hit));
   if (cache_hit) {
     ImpaladMetrics::IO_MGR_CACHED_FILE_HANDLES_HIT_RATIO->Update(1L);
     ImpaladMetrics::IO_MGR_CACHED_FILE_HANDLES_HIT_COUNT->Increment(1L);
@@ -886,24 +891,17 @@ Status DiskIoMgr::GetCachedHdfsFileHandle(const hdfsFS& fs,
   return Status::OK();
 }
 
-void DiskIoMgr::ReleaseCachedHdfsFileHandle(std::string* fname,
-    CachedHdfsFileHandle* fid) {
-  file_handle_cache_.ReleaseFileHandle(fname, fid, false);
-  ImpaladMetrics::IO_MGR_NUM_FILE_HANDLES_OUTSTANDING->Increment(-1L);
-}
-
 Status DiskIoMgr::ReopenCachedHdfsFileHandle(const hdfsFS& fs, std::string* fname,
-    int64_t mtime, RequestContext* reader, CachedHdfsFileHandle** fid) {
+    int64_t mtime, RequestContext* reader, FileHandleCache::Accessor* accessor) {
   bool cache_hit;
   SCOPED_TIMER(reader->open_file_timer_);
   ImpaladMetrics::IO_MGR_CACHED_FILE_HANDLES_REOPENED->Increment(1L);
-  file_handle_cache_.ReleaseFileHandle(fname, *fid, true);
-  // The old handle has been destroyed, so *fid must be overwritten before returning.
-  *fid = nullptr;
-  Status status = file_handle_cache_.GetFileHandle(fs, fname, mtime, true, fid,
-      &cache_hit);
+
+  accessor->Destroy();
+
+  Status status =
+      file_handle_cache_.GetFileHandle(fs, fname, mtime, true, accessor, &cache_hit);
   if (!status.ok()) {
-    ImpaladMetrics::IO_MGR_NUM_FILE_HANDLES_OUTSTANDING->Increment(-1L);
     return status;
   }
   DCHECK(!cache_hit);
