@@ -149,23 +149,18 @@ Status HdfsFileReader::ReadFromPos(DiskQueue* queue, int64_t file_offset, uint8_
     // If the reader has an exclusive file handle, use it. Otherwise, borrow
     // a file handle from the cache.
     req_context_read_timer.Stop();
-    CachedHdfsFileHandle* borrowed_hdfs_fh = nullptr;
+
+    // RAII accessor, it will release the file handle at destruction
+    FileHandleCache::Accessor accessor;
+
     hdfsFile hdfs_file;
     if (exclusive_hdfs_fh_ != nullptr) {
       hdfs_file = exclusive_hdfs_fh_->file();
     } else {
-      RETURN_IF_ERROR(
-          io_mgr->GetCachedHdfsFileHandle(hdfs_fs_, scan_range_->file_string(),
-              scan_range_->mtime(), request_context, &borrowed_hdfs_fh));
-      hdfs_file = borrowed_hdfs_fh->file();
+      RETURN_IF_ERROR(io_mgr->GetCachedHdfsFileHandle(hdfs_fs_,
+          scan_range_->file_string(), scan_range_->mtime(), request_context, &accessor));
+      hdfs_file = accessor.Get()->file();
     }
-    // Make sure to release any borrowed file handle.
-    auto release_borrowed_hdfs_fh = MakeScopeExitTrigger([this, &borrowed_hdfs_fh]() {
-      if (borrowed_hdfs_fh != nullptr) {
-        scan_range_->io_mgr_->ReleaseCachedHdfsFileHandle(
-            scan_range_->file_string(), borrowed_hdfs_fh);
-      }
-    });
     req_context_read_timer.Start();
 
     while (*bytes_read < bytes_to_read) {
@@ -186,14 +181,14 @@ Status HdfsFileReader::ReadFromPos(DiskQueue* queue, int64_t file_offset, uint8_
       // - first read was not successful
       // and
       // - used a borrowed file handle
-      if (!status.ok() && borrowed_hdfs_fh != nullptr) {
+      if (!status.ok() && accessor.Get()) {
         // The error may be due to a bad file handle. Reopen the file handle and retry.
         // Exclude this time from the read timers.
         req_context_read_timer.Stop();
         RETURN_IF_ERROR(
             io_mgr->ReopenCachedHdfsFileHandle(hdfs_fs_, scan_range_->file_string(),
-                scan_range_->mtime(), request_context, &borrowed_hdfs_fh));
-        hdfs_file = borrowed_hdfs_fh->file();
+                scan_range_->mtime(), request_context, &accessor));
+        hdfs_file = accessor.Get()->file();
         VLOG_FILE << "Reopening file " << scan_range_->file_string() << " with mtime "
                   << scan_range_->mtime() << " offset " << file_offset;
         req_context_read_timer.Start();
