@@ -95,9 +95,11 @@ Status KrpcDataStreamMgr::Init(MemTracker* service_mem_tracker) {
 
 inline uint32_t KrpcDataStreamMgr::GetHashValue(
     const TUniqueId& fragment_instance_id, PlanNodeId dest_node_id) {
-  uint32_t value = RawValue::GetHashValue(&fragment_instance_id.lo, TYPE_BIGINT, 0);
-  value = RawValue::GetHashValue(&fragment_instance_id.hi, TYPE_BIGINT, value);
-  value = RawValue::GetHashValue(&dest_node_id, TYPE_INT, value);
+  uint32_t value = RawValue::GetHashValue(
+      &fragment_instance_id.lo, ColumnType(TYPE_BIGINT), 0);
+  value = RawValue::GetHashValue(
+      &fragment_instance_id.hi, ColumnType(TYPE_BIGINT), value);
+  value = RawValue::GetHashValue(&dest_node_id, ColumnType(TYPE_INT), value);
   return value;
 }
 
@@ -333,13 +335,16 @@ Status KrpcDataStreamMgr::DeregisterRecvr(
   return Status(msg);
 }
 
-void KrpcDataStreamMgr::Cancel(const TUniqueId& finst_id) {
-  VLOG_QUERY << "cancelling active streams for fragment_instance_id="
-             << PrintId(finst_id);
+void KrpcDataStreamMgr::Cancel(const TUniqueId& query_id) {
+  VLOG_QUERY << "cancelling active streams for query_id=" << PrintId(query_id);
   lock_guard<mutex> l(lock_);
+  // Fragment instance IDs are the query ID with the lower bits set to the instance
+  // index. Therefore all finstances for a query are clustered together, starting
+  // after the position in the map where the query_id would be.
   FragmentRecvrSet::iterator iter =
-      fragment_recvr_set_.lower_bound(make_pair(finst_id, 0));
-  while (iter != fragment_recvr_set_.end() && iter->first == finst_id) {
+      fragment_recvr_set_.lower_bound(make_pair(query_id, 0));
+  while (iter != fragment_recvr_set_.end() &&
+         GetQueryId(iter->first) == query_id) {
     bool unused;
     shared_ptr<KrpcDataStreamRecvr> recvr = FindRecvr(iter->first, iter->second, &unused);
     if (recvr != nullptr) {
