@@ -17,30 +17,38 @@
 
 #pragma once
 
-#include <stdint.h>
-#include <map>
+#include <cstdint>
+#include <functional>
+#include <list>
 #include <memory>
 #include <mutex>
+#include <ostream>
 #include <queue>
+#include <string>
+#include <utility>
 #include <vector>
+
 #include <boost/unordered_map.hpp>
 
-#include "common/logging.h"
 #include "common/atomic.h"
+#include "common/compiler-util.h"
+#include "common/logging.h"
+#include "common/status.h"
 #include "runtime/mem-tracker-types.h"
-#include "util/debug-util.h"
-#include "util/internal-queue.h"
 #include "util/metrics-fwd.h"
 #include "util/runtime-profile-counters.h"
 #include "util/spinlock.h"
 
+#include "gen-cpp/StatestoreService.h" // for TPoolStats
 #include "gen-cpp/Types_types.h" // for TUniqueId
+#include <gtest/gtest_prod.h> // for FRIEND_TEST
 
 namespace impala {
 
+class MetricGroup;
 class ObjectPool;
 struct ReservationTrackerCounters;
-class TQueryOptions;
+class RuntimeState;
 
 /// A MemTracker tracks memory consumption; it contains an optional limit
 /// and can be arranged into a tree structure such that the consumption tracked
@@ -85,7 +93,8 @@ class MemTracker {
   /// included
   /// in LogUsage() output if consumption is 0.
   MemTracker(int64_t byte_limit = -1, const std::string& label = std::string(),
-      MemTracker* parent = nullptr, bool log_usage_if_zero = true);
+      MemTracker* parent = nullptr, bool log_usage_if_zero = true,
+      bool is_query_mem_tracker = false, const TUniqueId& query_id = TUniqueId());
 
   /// C'tor for tracker for which consumption counter is created as part of a profile.
   /// The counter is created with name COUNTER_NAME.
@@ -323,8 +332,8 @@ class MemTracker {
   /// for the process MemTracker.
   /// TODO: once all memory is accounted in ReservationTracker hierarchy, move
   /// reporting there.
-  std::string LogUsage(int max_recursive_depth,
-      const std::string& prefix = "", int64_t* logged_consumption = nullptr);
+  std::string LogUsage(int max_recursive_depth, const std::string& prefix = "",
+      int64_t* logged_consumption = nullptr);
   /// Dumping the process MemTracker is expensive. Limiting the recursive depth
   /// to two levels limits the level of detail to a one-line summary for each query
   /// MemTracker, avoiding all MemTrackers below that level. This provides a summary
@@ -338,6 +347,16 @@ class MemTracker {
   /// consumption.
   std::string LogTopNQueries(int limit);
 
+  /// Update the following data members in pool_stats for all queries tracked through
+  /// query memory trackers:
+  ///   heavy_memory_queries: the query Ids of top 'limit' queries in memory consumption
+  ///   (in descending order)
+  ///   min_memory_consumed: the minimal memory consumed among all queries
+  ///   max_memory_consumed: the maximal memory consumed among all queries
+  ///   total_memory_consumed: the total memory consumed by all queries
+  ///   num_running: the total number of all queries
+  void UpdatePoolStatsForQueries(int limit, TPoolStats& pool_stats);
+
   /// Log the memory usage when memory limit is exceeded and return a status object with
   /// details of the allocation which caused the limit to be exceeded.
   /// If 'failed_allocation_size' is greater than zero, logs the allocation size. If
@@ -393,14 +412,17 @@ class MemTracker {
   /// and populates 'min_pq' with 'limit' number of elements (that contain state related
   /// to query MemTrackers) based on maximum total memory consumption.
   void GetTopNQueries(
-      std::priority_queue<pair<int64_t, string>,
-          vector<pair<int64_t, string>>, std::greater<pair<int64_t, string>>>& min_pq,
+      std::priority_queue<pair<int64_t, string>, vector<pair<int64_t, string>>,
+          std::greater<pair<int64_t, string>>>& min_pq,
       int limit);
 
   /// If an ancestor of this tracker is a query MemTracker, return that tracker.
   /// Otherwise return NULL.
   MemTracker* GetQueryMemTracker();
 
+  /// Return the root ancestor of this tracker.
+  MemTracker* GetRootMemTracker();
+
   /// Increases/Decreases the consumption of this tracker and the ancestors up to (but
   /// not including) end_tracker.
   void ChangeConsumption(int64_t bytes, MemTracker* end_tracker) {
@@ -415,6 +437,26 @@ class MemTracker {
     DCHECK(false) << "end_tracker is not an ancestor";
   }
 
+  /// Update the following fields in pool_stats.
+  ///   min_memory_consumed: take the min of min_memory_consumed and mem_consumed
+  ///   max_memory_consumed: take the max of max_memory_consumed and mem_consumed
+  ///   total_memory_consumed: increased by mem_consumed
+  ///   num_running++
+  void UpdatePoolStatsForMemoryConsumed(int64_t mem_consumed, TPoolStats& pool_stats);
+
+  /// Collect the top N queries into a priority queue, and computes the min, the max,
+  /// the total memory consumption, and the total number of all queries that are
+  /// all children query mem trackers. The top element in the queue is the
+  /// min-element such that the remaining elements after a sequence of pop operations
+  /// are the largest in memory consumptions. This method should only be called for
+  /// mem-trackers that are either query mem trackers or higher in the mem tracker
+  /// hierarchy.
+  typedef std::priority_queue<THeavyMemoryQuery, std::vector<THeavyMemoryQuery>,
+      std::greater<THeavyMemoryQuery>>
+      MinPriorityQueue;
+  void GetTopNQueriesAndUpdatePoolStats(
+      MinPriorityQueue& min_pq, int limit, TPoolStats& pool_stats);
+
   /// Lock to protect GcMemory(). This prevents many GCs from occurring at once.
   std::mutex gc_lock_;
 
@@ -499,6 +541,10 @@ class MemTracker {
 
   /// Metric for limit_.
   IntGauge* limit_metric_;
+
+  friend class AdmissionControllerTest;
+  friend class MemTestTest_TopN_Test;
+  FRIEND_TEST(AdmissionControllerTest, TopNQueryCheck);
 };
 
 /// Global registry for query and pool MemTrackers. Owned by ExecEnv.
