@@ -89,6 +89,8 @@ Status QueryDriver::SetExternalPlan(
   // Update local_time_zone in the external request
   exec_request_->query_exec_request.query_ctx.__set_local_time_zone(
       query_ctx.local_time_zone);
+  exec_request_->query_exec_request.query_ctx.__set_now_string(
+      query_ctx.now_string);
   return Status::OK();
 }
 
@@ -174,14 +176,21 @@ void QueryDriver::TryQueryRetry(
         << Substitute("Cannot retry a that has already been retried query_id = $0",
             PrintId(query_id));
 
-    // Update the state and then schedule the retry asynchronously.
-    client_request_state_->MarkAsRetrying(*error);
-
     // Another reference to this QueryDriver (via the shared_ptr) needs to be created and
     // passed to the thread so that a valid shared_ptr exists while the thread is running.
     // Otherwise it is possible that the user cancels the query and this QueryDriver gets
     // deleted by the shared_ptr.
+    DebugActionNoFail(FLAGS_debug_actions, "RETRY_DELAY_GET_QUERY_DRIVER");
     shared_ptr<QueryDriver> query_driver = parent_server_->GetQueryDriver(query_id);
+    if (query_driver.get() == nullptr) {
+      VLOG_QUERY << Substitute(
+          "Skipping retry of query_id=$0 because it has already been unregistered",
+          PrintId(query_id));
+      return;
+    }
+
+    // Update the state and then schedule the retry asynchronously.
+    client_request_state_->MarkAsRetrying(*error);
 
     // Launch the query retry in a separate thread, 'was_retried' is set to true
     // if the query retry was successfully launched.
@@ -354,6 +363,9 @@ void QueryDriver::RetryQueryFromThread(
   // Close the original query.
   QueryHandle query_handle;
   query_handle.SetHandle(query_driver, request_state);
+  // Do the work of close that needs to be done synchronously, otherwise we'll
+  // hit some illegal states in destroying the request_state.
+  RETURN_VOID_IF_ERROR(query_handle->Finalize(true, nullptr));
   parent_server_->CloseClientRequestState(query_handle);
   parent_server_->MarkSessionInactive(session);
 }
@@ -421,6 +433,13 @@ Status QueryDriver::Finalize(
 
 Status QueryDriver::Unregister(ImpalaServer::QueryDriverMap* query_driver_map) {
   DCHECK(finalized_.Load());
+  // Wait until retry_query_thread_ finishes, otherwise the resources for this thread
+  // may not be released.
+  if (retry_query_thread_.get() != nullptr) {
+    retry_query_thread_->Join();
+    retry_query_thread_.reset();
+  }
+  DCHECK(retry_query_thread_.get() == nullptr);
   const TUniqueId* query_id = nullptr;
   const TUniqueId* retry_query_id = nullptr;
   {
