@@ -21,12 +21,14 @@
 #include <string>
 
 #include "codegen/impala-ir.h"
+#include "runtime/collection-value.h"
 #include "runtime/types.h"
 
 namespace impala {
 
 class MemPool;
 class SlotDescriptor;
+struct StringValue;
 class Tuple;
 
 /// Useful utility functions for runtime values (which are passed around as void*).
@@ -46,11 +48,13 @@ class RawValue {
 
   /// Convert 'value' into ascii and write to 'stream'. NULL turns into "NULL". 'scale'
   /// determines how many digits after the decimal are printed for floating point numbers,
-  /// -1 indicates to use the stream's current formatting.
+  /// -1 indicates to use the stream's current formatting. Doesn't support complex types.
+  /// If 'quote_val' is true, write STRING, VARCHAR, CHAR, DATE, TIMESTAMP values in
+  /// quoted form surrounded by double quotes.
   /// TODO: for string types, we just print the result regardless of whether or not it
   /// ascii. This could be undesirable.
   static void PrintValue(const void* value, const ColumnType& type, int scale,
-                         std::stringstream* stream);
+                         std::stringstream* stream, bool quote_val=false);
 
   /// Write ascii value to string instead of stringstream.
   static void PrintValue(const void* value, const ColumnType& type, int scale,
@@ -63,6 +67,10 @@ class RawValue {
     return str;
   }
 
+  /// Similar to PrintValue() but works with array values.
+  /// Converts 'array_val' array into ascii and writes to 'stream'.
+  static void PrintArrayValue(const CollectionValue* array_val,
+      const TupleDescriptor* item_tuple_desc, int scale, std::stringstream *stream);
 
   /// Writes the byte representation of a value to a stringstream character-by-character
   static void PrintValueAsBytes(const void* value, const ColumnType& type,
@@ -125,6 +133,13 @@ class RawValue {
   /// src must be non-NULL.
   static void Write(const void* src, void* dst, const ColumnType& type, MemPool* pool);
 
+  /// Wrapper function for Write() to handle struct slots and its children. Additionally,
+  /// gathers the string slots of the slot tree into 'string_values'.
+  template <bool COLLECT_STRING_VALS>
+  static void Write(const void* value, Tuple* tuple,
+      const SlotDescriptor* slot_desc, MemPool* pool,
+    std::vector<StringValue*>* string_values);
+
   /// Returns true if v1 == v2.
   /// This is more performant than Compare() == 0 for string equality, mostly because of
   /// the length comparison check.
@@ -146,5 +161,20 @@ class RawValue {
 
   // Returns positive zero for floating point types.
   static inline const void* PositiveFloatingZero(const ColumnType& type);
+
+private:
+  /// Recursive helper function for Write() to handle struct slots.
+  template <bool COLLECT_STRING_VALS>
+  static void WriteStruct(const void* value, Tuple* tuple,
+      const SlotDescriptor* slot_desc, MemPool* pool,
+      std::vector<StringValue*>* string_values);
+
+  /// Gets the destination slot from 'tuple' and 'slot_desc', writes value to this slot
+  /// using Write(). Collects pointer of the string slots to 'string_values'. 'slot_desc'
+  /// has to be primitive type.
+  template <bool COLLECT_STRING_VALS>
+  static void WritePrimitive(const void* value, Tuple* tuple,
+      const SlotDescriptor* slot_desc, MemPool* pool,
+      std::vector<StringValue*>* string_values);
 };
 }
