@@ -22,7 +22,10 @@
 #include <numeric>
 
 #include <boost/filesystem.hpp>
+#include <boost/lexical_cast.hpp>
 #include <boost/scoped_ptr.hpp>
+#include <boost/uuid/random_generator.hpp>
+#include <boost/uuid/uuid_io.hpp>
 #include <gtest/gtest.h>
 
 #include "common/init.h"
@@ -44,6 +47,7 @@
 #include "common/names.h"
 
 using boost::filesystem::path;
+using boost::uuids::random_generator;
 
 DECLARE_bool(disk_spill_encryption);
 DECLARE_int64(disk_spill_compression_buffer_limit_bytes);
@@ -53,7 +57,7 @@ DECLARE_bool(disk_spill_punch_holes);
 DECLARE_int32(stress_scratch_write_delay_ms);
 #endif
 DECLARE_string(remote_tmp_file_size);
-DECLARE_bool(allow_spill_to_hdfs);
+DECLARE_int32(wait_for_spill_buffer_timeout_s);
 
 namespace impala {
 
@@ -65,7 +69,7 @@ static const int64_t GIGABYTE = 1024L * MEGABYTE;
 static const int64_t TERABYTE = 1024L * GIGABYTE;
 
 /// For testing spill to remote.
-static const string HDFS_LOCAL_URL = "hdfs://localhost";
+static const string HDFS_LOCAL_URL = "hdfs://localhost:20500/tmp";
 static const string REMOTE_URL = HDFS_LOCAL_URL;
 static const string LOCAL_BUFFER_PATH = "/tmp/tmp-file-mgr-test-buffer";
 
@@ -81,7 +85,6 @@ class TmpFileMgrTest : public ::testing::Test {
     FLAGS_stress_scratch_write_delay_ms = 0;
 #endif
     FLAGS_remote_tmp_file_size = "8MB";
-    FLAGS_allow_spill_to_hdfs = true;
 
     metrics_.reset(new MetricGroup("tmp-file-mgr-test"));
     profile_ = RuntimeProfile::Create(&obj_pool_, "tmp-file-mgr-test");
@@ -101,12 +104,19 @@ class TmpFileMgrTest : public ::testing::Test {
   /// Helper to create a TmpFileMgr and initialise it with InitCustom(). Adds the mgr to
   /// 'obj_pool_' for automatic cleanup at the end of each test. Fails the test if
   /// InitCustom() fails.
-  TmpFileMgr* CreateTmpFileMgr(const string& tmp_dirs_spec) {
+  TmpFileMgr* CreateTmpFileMgr(const string& tmp_dirs_spec, bool expect_ok = true,
+      HdfsFsCache::HdfsFsMap* hdfs_conn_map = nullptr) {
     // Allocate a new metrics group for each TmpFileMgr so they don't get confused by
     // the pre-existing metrics (TmpFileMgr assumes it's a singleton in product code).
     MetricGroup* metrics = obj_pool_.Add(new MetricGroup(""));
     TmpFileMgr* mgr = obj_pool_.Add(new TmpFileMgr());
-    EXPECT_OK(mgr->InitCustom(tmp_dirs_spec, false, "", false, metrics));
+    if (hdfs_conn_map != nullptr) mgr->hdfs_conns_ = *hdfs_conn_map;
+    Status status = mgr->InitCustom(tmp_dirs_spec, false, "", false, metrics);
+    if (expect_ok) {
+      EXPECT_TRUE(status.ok());
+    } else {
+      EXPECT_FALSE(status.ok());
+    }
     return mgr;
   }
 
@@ -161,10 +171,15 @@ class TmpFileMgrTest : public ::testing::Test {
   }
 
   /// Helper to get the private tmp_dirs_ member.
-  static const vector<TmpFileMgr::TmpDir>& GetTmpDirs(TmpFileMgr* mgr) {
+  static const vector<unique_ptr<TmpDir>>& GetTmpDirs(TmpFileMgr* mgr) {
     return mgr->tmp_dirs_;
   }
 
+  /// Helper to get the private tmp_remote_dirs_ pointer.
+  static const unique_ptr<TmpDir>& GetTmpRemoteDir(TmpFileMgr* mgr) {
+    return mgr->tmp_dirs_remote_;
+  }
+
   /// Helper to call the private TmpFileMgr::NewFile() method.
   static void NewFile(TmpFileMgr* mgr, TmpFileGroup* group,
       TmpFileMgr::DeviceId device_id, unique_ptr<TmpFile>* new_file) {
@@ -193,6 +208,32 @@ class TmpFileMgrTest : public ::testing::Test {
     group->io_ctx_->Cancel();
   }
 
+  /// Helper to set an invalid remote path to create an error.
+  static void SetInvalidRemotePath(
+      TmpFileMgr* tmp_file_mgr, TmpFileGroup* group, int tmp_file_idx) {
+    const string& dir = tmp_file_mgr->tmp_dirs_remote_->path();
+    int dev_id = 0;
+    string invalid_path = "";
+    auto tmp_file_shared_ptr = group->tmp_files_remote_[tmp_file_idx];
+    auto org_tmp_file = static_cast<TmpFileRemote*>(tmp_file_shared_ptr.get());
+    auto local_path = org_tmp_file->local_buffer_path_;
+    TmpFileRemote* tmp_file_r =
+        new TmpFileRemote(group, dev_id, invalid_path, local_path, false, dir.c_str());
+    tmp_file_r->at_capacity_ = org_tmp_file->at_capacity_;
+    tmp_file_r->enqueued_ = org_tmp_file->enqueued_;
+    tmp_file_r->local_buffer_path_ = org_tmp_file->local_buffer_path_;
+    tmp_file_r->file_size_ = org_tmp_file->file_size_;
+    tmp_file_r->allocation_offset_ = org_tmp_file->allocation_offset_;
+    tmp_file_r->hdfs_conn_ = org_tmp_file->hdfs_conn_;
+    tmp_file_r->disk_file_->hdfs_conn_ = org_tmp_file->disk_file_->hdfs_conn_;
+    tmp_file_r->disk_buffer_file_.swap(org_tmp_file->disk_buffer_file_);
+    // Set buffer returned to avoid DCHECK failure while destruction.
+    org_tmp_file->buffer_returned_ = true;
+    shared_ptr<TmpFile> tmp_file_remote(move(tmp_file_r));
+    group->tmp_files_remote_[tmp_file_idx] = tmp_file_remote;
+    group->tmp_files_remote_ptrs_.emplace(tmp_file_remote.get(), tmp_file_remote);
+  }
+
   static void SetWriteRangeContext(WriteRange* write_range, TmpFileGroup* group) {
     write_range->SetRequestContext(group->io_ctx_.get());
   }
@@ -927,12 +968,12 @@ TEST_F(TmpFileMgrTest, TestDirectoryLimitParsing) {
                        "/tmp/tmp-file-mgr-test5:200tb:5,"
                        "/tmp/tmp-file-mgr-test6:100MB:6"));
   EXPECT_EQ(6, dirs.size());
-  EXPECT_EQ(5 * GIGABYTE, dirs[0].bytes_limit);
-  EXPECT_EQ(numeric_limits<int64_t>::max(), dirs[1].bytes_limit);
-  EXPECT_EQ(1234, dirs[2].bytes_limit);
-  EXPECT_EQ(99999999, dirs[3].bytes_limit);
-  EXPECT_EQ(200 * TERABYTE, dirs[4].bytes_limit);
-  EXPECT_EQ(100 * MEGABYTE, dirs[5].bytes_limit);
+  EXPECT_EQ(5 * GIGABYTE, dirs[0]->bytes_limit());
+  EXPECT_EQ(numeric_limits<int64_t>::max(), dirs[1]->bytes_limit());
+  EXPECT_EQ(1234, dirs[2]->bytes_limit());
+  EXPECT_EQ(99999999, dirs[3]->bytes_limit());
+  EXPECT_EQ(200 * TERABYTE, dirs[4]->bytes_limit());
+  EXPECT_EQ(100 * MEGABYTE, dirs[5]->bytes_limit());
 
   // Various invalid limit formats result in the directory getting skipped.
   // Include a valid dir on the end to ensure that we don't short-circuit all
@@ -943,8 +984,8 @@ TEST_F(TmpFileMgrTest, TestDirectoryLimitParsing) {
                        "/tmp/tmp-file-mgr-test5:5pb,/tmp/tmp-file-mgr-test6:10%,"
                        "/tmp/tmp-file-mgr-test1:100"));
   EXPECT_EQ(1, dirs2.size());
-  EXPECT_EQ("/tmp/tmp-file-mgr-test1/impala-scratch", dirs2[0].path);
-  EXPECT_EQ(100, dirs2[0].bytes_limit);
+  EXPECT_EQ("/tmp/tmp-file-mgr-test1/impala-scratch", dirs2[0]->path());
+  EXPECT_EQ(100, dirs2[0]->bytes_limit());
 
   // Various valid ways of specifying "unlimited".
   auto& dirs3 =
@@ -952,7 +993,7 @@ TEST_F(TmpFileMgrTest, TestDirectoryLimitParsing) {
                                   "/tmp/tmp-file-mgr-test3,/tmp/tmp-file-mgr-test4:0"));
   EXPECT_EQ(4, dirs3.size());
   for (const auto& dir : dirs3) {
-    EXPECT_EQ(numeric_limits<int64_t>::max(), dir.bytes_limit);
+    EXPECT_EQ(numeric_limits<int64_t>::max(), dir->bytes_limit());
   }
 
   // Extra colons
@@ -967,6 +1008,131 @@ TEST_F(TmpFileMgrTest, TestDirectoryLimitParsing) {
   EXPECT_EQ(2, empty_paths.size());
 }
 
+// Test the scratch directory parsing logic with remote paths.
+TEST_F(TmpFileMgrTest, TestDirectoryLimitParsingRemotePath) {
+  // Create a local directory for the local buffer.
+  RemoveAndCreateDirs({"/tmp/local-buffer-dir", "/tmp/local-buffer-dir1",
+      "/tmp/local-buffer-dir2", "/tmp/local-buffer-dir3"});
+
+  // Successful cases for HDFS paths.
+  // Two types of paths, one with directory, one without.
+  vector<string> hdfs_paths{"hdfs://localhost:20500", "hdfs://localhost:20500/tmp"};
+  for (string hdfs_path : hdfs_paths) {
+    string full_hdfs_path = hdfs_path + "/impala-scratch";
+    auto& dirs1 = GetTmpRemoteDir(CreateTmpFileMgr(hdfs_path + ",/tmp/local-buffer-dir"));
+    EXPECT_NE(nullptr, dirs1);
+
+    auto& dirs2 =
+        GetTmpRemoteDir(CreateTmpFileMgr(hdfs_path + ":100,/tmp/local-buffer-dir"));
+    EXPECT_NE(nullptr, dirs2);
+    EXPECT_EQ(full_hdfs_path, dirs2->path());
+    EXPECT_EQ(100, dirs2->bytes_limit());
+
+    auto& dirs3 =
+        GetTmpRemoteDir(CreateTmpFileMgr(hdfs_path + ":1KB:1,/tmp/local-buffer-dir"));
+    EXPECT_NE(nullptr, dirs3);
+    EXPECT_EQ(full_hdfs_path, dirs3->path());
+    EXPECT_EQ(1024, dirs3->bytes_limit());
+
+    // Multiple local paths with one remote path.
+    auto tmp_mgr_4 = CreateTmpFileMgr(hdfs_path
+        + ",/tmp/local-buffer-dir1,"
+          "/tmp/local-buffer-dir2,/tmp/local-buffer-dir3");
+    auto& dirs4_local = GetTmpDirs(tmp_mgr_4);
+    auto& dirs4_remote = GetTmpRemoteDir(tmp_mgr_4);
+    EXPECT_NE(nullptr, dirs4_remote);
+    EXPECT_EQ(full_hdfs_path, dirs4_remote->path());
+    EXPECT_EQ(2, dirs4_local.size());
+    EXPECT_EQ("/tmp/local-buffer-dir2/impala-scratch", dirs4_local[0]->path());
+    EXPECT_EQ("/tmp/local-buffer-dir3/impala-scratch", dirs4_local[1]->path());
+
+    // Fails the parsing due to no port number for the HDFS path.
+    auto tmp_mgr_5 = CreateTmpFileMgr("hdfs://localhost/tmp,/tmp/local-buffer-dir");
+    auto& dirs5_local = GetTmpDirs(tmp_mgr_5);
+    auto& dirs5_remote = GetTmpRemoteDir(tmp_mgr_5);
+    EXPECT_EQ(1, dirs5_local.size());
+    EXPECT_EQ(nullptr, dirs5_remote);
+
+    // Fails to init due to no local buffer for the remote scratch space.
+    // We expect a non-null tmp_mgr, but the init process should fail.
+    EXPECT_NE(nullptr, CreateTmpFileMgr(hdfs_path, false));
+
+    // Parse successfully, but the parsed HDFS path is unable to connect.
+    // These cases would fail the initialization of TmpFileMgr.
+    auto& dirs7 = GetTmpRemoteDir(
+        CreateTmpFileMgr("hdfs://localhost:1/tmp::1,/tmp/local-buffer-dir", false));
+    EXPECT_EQ(nullptr, dirs7);
+
+    auto& dirs8 = GetTmpRemoteDir(
+        CreateTmpFileMgr("hdfs://localhost:/tmp::,/tmp/local-buffer-dir", false));
+    EXPECT_EQ(nullptr, dirs8);
+
+    auto& dirs9 = GetTmpRemoteDir(
+        CreateTmpFileMgr("hdfs://localhost/tmp::1,/tmp/local-buffer-dir", false));
+    EXPECT_EQ(nullptr, dirs9);
+
+    auto& dirs10 = GetTmpRemoteDir(
+        CreateTmpFileMgr("hdfs://localhost/tmp:1,/tmp/local-buffer-dir", false));
+    EXPECT_EQ(nullptr, dirs10);
+
+    // Multiple remote paths, should support only one.
+    auto& dirs11 = GetTmpRemoteDir(CreateTmpFileMgr(hdfs_path
+        + ",hdfs://localhost:20501/tmp,"
+          "/tmp/local-buffer-dir"));
+    EXPECT_NE(nullptr, dirs11);
+    EXPECT_EQ(full_hdfs_path, dirs11->path());
+
+    // The order of the buffer and the remote dir should not affect the result.
+    auto& dirs12 = GetTmpRemoteDir(CreateTmpFileMgr(
+        "/tmp/local-buffer-dir, " + hdfs_path + ",hdfs://localhost:20501/tmp"));
+    EXPECT_NE(nullptr, dirs12);
+    EXPECT_EQ(full_hdfs_path, dirs12->path());
+  }
+
+  // Successful cases for parsing S3 paths.
+  // Create a fake s3 connection in order to pass the connection verification.
+  HdfsFsCache::HdfsFsMap fake_hdfs_conn_map;
+  hdfsFS fake_conn = reinterpret_cast<hdfsFS>(1);
+  fake_hdfs_conn_map.insert(make_pair("s3a://fake_host/", fake_conn));
+  // Two types of paths, one with directory, one without.
+  vector<string> s3_paths{"s3a://fake_host", "s3a://fake_host/dir"};
+  for (string s3_path : s3_paths) {
+    string full_s3_path = s3_path + "/impala-scratch";
+    auto& dirs13 = GetTmpRemoteDir(
+        CreateTmpFileMgr("/tmp/local-buffer-dir, " + s3_path, true, &fake_hdfs_conn_map));
+    EXPECT_NE(nullptr, dirs13);
+    EXPECT_EQ(full_s3_path, dirs13->path());
+
+    auto& dirs14 = GetTmpRemoteDir(CreateTmpFileMgr(
+        "/tmp/local-buffer-dir, " + s3_path + ":100", true, &fake_hdfs_conn_map));
+    EXPECT_NE(nullptr, dirs14);
+    EXPECT_EQ(full_s3_path, dirs14->path());
+    EXPECT_EQ(100, dirs14->bytes_limit());
+
+    auto& dirs15 = GetTmpRemoteDir(CreateTmpFileMgr(
+        "/tmp/local-buffer-dir, " + s3_path + ":1KB:1", true, &fake_hdfs_conn_map));
+    EXPECT_NE(nullptr, dirs15);
+    EXPECT_EQ(full_s3_path, dirs15->path());
+    EXPECT_EQ(1024, dirs15->bytes_limit());
+  }
+
+  // Failure cases for parsing S3 paths, S3 paths don't allow the port number.
+  auto& dirs16 = GetTmpRemoteDir(CreateTmpFileMgr(
+      "/tmp/local-buffer-dir, s3a://fake_host:1234/for-parsing-test-only:1KB:1", true,
+      &fake_hdfs_conn_map));
+  EXPECT_EQ(nullptr, dirs16);
+
+  auto& dirs17 = GetTmpRemoteDir(CreateTmpFileMgr(
+      "/tmp/local-buffer-dir, s3a://fake_host:1234/for-parsing-test-only:1KB", true,
+      &fake_hdfs_conn_map));
+  EXPECT_EQ(nullptr, dirs17);
+
+  auto& dirs18 = GetTmpRemoteDir(CreateTmpFileMgr(
+      "/tmp/local-buffer-dir, s3a://fake_host:1234/for-parsing-test-only", true,
+      &fake_hdfs_conn_map));
+  EXPECT_EQ(nullptr, dirs18);
+}
+
 // Test compression buffer memory management for reads and writes.
 TEST_F(TmpFileMgrTest, TestCompressBufferManagement) {
   FLAGS_disk_spill_encryption = false;
@@ -1125,18 +1291,18 @@ TEST_F(TmpFileMgrTest, TestDirectoryPriorityParsing) {
                        "/tmp/tmp-file-mgr-test2::2,/tmp/tmp-file-mgr-test4:99999999:4,"
                        "/tmp/tmp-file-mgr-test5:200tb:5,/tmp/tmp-file-mgr-test1:5g:1"));
   EXPECT_EQ(6, dirs.size());
-  EXPECT_EQ(5 * GIGABYTE, dirs[0].bytes_limit);
-  EXPECT_EQ(1, dirs[0].priority);
-  EXPECT_EQ(numeric_limits<int64_t>::max(), dirs[1].bytes_limit);
-  EXPECT_EQ(2, dirs[1].priority);
-  EXPECT_EQ(1234, dirs[2].bytes_limit);
-  EXPECT_EQ(3, dirs[2].priority);
-  EXPECT_EQ(99999999, dirs[3].bytes_limit);
-  EXPECT_EQ(4, dirs[3].priority);
-  EXPECT_EQ(200 * TERABYTE, dirs[4].bytes_limit);
-  EXPECT_EQ(5, dirs[4].priority);
-  EXPECT_EQ(100 * MEGABYTE, dirs[5].bytes_limit);
-  EXPECT_EQ(numeric_limits<int>::max(), dirs[5].priority);
+  EXPECT_EQ(5 * GIGABYTE, dirs[0]->bytes_limit());
+  EXPECT_EQ(1, dirs[0]->priority());
+  EXPECT_EQ(numeric_limits<int64_t>::max(), dirs[1]->bytes_limit());
+  EXPECT_EQ(2, dirs[1]->priority());
+  EXPECT_EQ(1234, dirs[2]->bytes_limit());
+  EXPECT_EQ(3, dirs[2]->priority());
+  EXPECT_EQ(99999999, dirs[3]->bytes_limit());
+  EXPECT_EQ(4, dirs[3]->priority());
+  EXPECT_EQ(200 * TERABYTE, dirs[4]->bytes_limit());
+  EXPECT_EQ(5, dirs[4]->priority());
+  EXPECT_EQ(100 * MEGABYTE, dirs[5]->bytes_limit());
+  EXPECT_EQ(numeric_limits<int>::max(), dirs[5]->priority());
 
   // Various invalid limit formats result in the directory getting skipped.
   // Include a valid dir on the end to ensure that we don't short-circuit all
@@ -1147,9 +1313,9 @@ TEST_F(TmpFileMgrTest, TestDirectoryPriorityParsing) {
                        "/tmp/tmp-file-mgr-test5::p0,/tmp/tmp-file-mgr-test6::10%,"
                        "/tmp/tmp-file-mgr-test1:100:-1"));
   EXPECT_EQ(1, dirs2.size());
-  EXPECT_EQ("/tmp/tmp-file-mgr-test1/impala-scratch", dirs2[0].path);
-  EXPECT_EQ(100, dirs2[0].bytes_limit);
-  EXPECT_EQ(-1, dirs2[0].priority);
+  EXPECT_EQ("/tmp/tmp-file-mgr-test1/impala-scratch", dirs2[0]->path());
+  EXPECT_EQ(100, dirs2[0]->bytes_limit());
+  EXPECT_EQ(-1, dirs2[0]->priority());
 }
 
 // Tests that when TmpFileGroup is constructed, the priority based index ranges are
@@ -1688,7 +1854,7 @@ void TmpFileMgrTest::TestTmpFileBufferPoolHelper(TmpFileMgr& tmp_file_mgr,
   write_range->SetDiskFile(file3->GetWriteFile());
   SetWriteRangeContext(write_range.get(), file_group);
   // The space hasn't been reserved for file3 because the pool size is 2K, and there are
-  // two files in use, we use the AsyncWriteRange() function to wait untial a file is
+  // two files in use, we use the AsyncWriteRange() function to wait until a file is
   // available to be evicted, then finish the writing of file3.
   EXPECT_FALSE(file3->GetWriteFile()->IsSpaceReserved());
   ASSERT_OK(tmp_file_mgr.AsyncWriteRange(write_range.get(), file3));
@@ -1818,4 +1984,81 @@ TEST_F(TmpFileMgrTest, TestRemoteRemoveBuffer) {
   file_group.Close();
   test_env_->TearDownQueries();
 }
+
+/// Test config a non-existent remote path.
+TEST_F(TmpFileMgrTest, TestRemoteUploadToNonExistentPath) {
+  TmpFileMgr tmp_file_mgr;
+  vector<string> tmp_dirs{{LOCAL_BUFFER_PATH}};
+  RemoveAndCreateDirs(tmp_dirs);
+  // Create a non-existent remote path.
+  string random_name = lexical_cast<string>(random_generator()());
+  string remote_file_path_to_crt = "s3a://non-existent_" + random_name + "/tmp/test";
+  tmp_dirs.push_back(remote_file_path_to_crt);
+  // Failed to initialize because unable to get a connection to the non-existent path
+  // during verification.
+  EXPECT_FALSE(tmp_file_mgr.InitCustom(tmp_dirs, true, "", false, metrics_.get()).ok());
+  test_env_->TearDownQueries();
+}
+
+/// Test upload failure and lead to query cancelled due to timeout.
+TEST_F(TmpFileMgrTest, TestRemoteUploadFailed) {
+  TmpFileMgr tmp_file_mgr;
+  int64_t buffer_limit = 2048;
+  int64_t alloc_size = 512;
+  // The file size should be aligned with the FLAGS_remote_tmp_file_size.
+  int64_t file_size = 1024;
+  int64_t per_file_page_num = file_size / alloc_size;
+  // Set the timeout value to the minimum, 1 second. Because otherwise, the testcase
+  // needs to wait for the default timeout interval, which is much larger than 1, before
+  // the testcase can end and exit.
+  FLAGS_wait_for_spill_buffer_timeout_s = 1;
+  // The file size is the same as file_size, so each write uses one file.
+  FLAGS_remote_tmp_file_size = "1KB";
+  vector<string> tmp_create_dirs{{LOCAL_BUFFER_PATH}};
+  vector<string> tmp_dirs{{Substitute(LOCAL_BUFFER_PATH + ":$0", buffer_limit)}};
+  RemoveAndCreateDirs(tmp_create_dirs);
+  tmp_dirs.push_back(HDFS_LOCAL_URL);
+
+  ASSERT_OK(tmp_file_mgr.InitCustom(tmp_dirs, true, "", false, metrics_.get()));
+  TUniqueId id;
+  TmpFileGroup file_group(&tmp_file_mgr, io_mgr(), profile_, id);
+  uint8_t* data = (uint8_t*)malloc(alloc_size);
+  ASSERT_TRUE(data != nullptr);
+  MemRange data_mem_range(data, alloc_size);
+
+  int write_times = buffer_limit / alloc_size;
+
+  // We set invalid remote paths to the tmp files, so that the upload of
+  // the tmp files would fail. Because all of the uploads fail, the query
+  // should be cancelled in the end due to timeout (the local buffer is full,
+  // the later file waits for the buffer till timeout happens).
+  for (int i = 0; i < write_times + 1; i++) {
+    // Set the remote paths of current TmpFile to invalid paths to create upload failures.
+    if (i != write_times && ((i + 1) % per_file_page_num == 0)) {
+      int cur_tmp_file_idx = i / per_file_page_num;
+      SetInvalidRemotePath(&tmp_file_mgr, &file_group, cur_tmp_file_idx);
+    }
+    unique_ptr<TmpWriteHandle> handle;
+    WriteRange::WriteDoneCallback callback = [this, i, write_times](
+                                                 const Status& status) {
+      // The first several pages have been successfully written because the local
+      // buffer isn't full. However, because all of the uploads fail, the existing
+      // local buffer can't be evicted and would be full eventually. In the end, the
+      // query is cancelled because of the timeout.
+      if (i < write_times) {
+        EXPECT_OK(status);
+      } else {
+        EXPECT_EQ(status.code(), TErrorCode::CANCELLED_INTERNALLY);
+      }
+      SignalCallback(status);
+    };
+    EXPECT_OK(file_group.Write(data_mem_range, callback, &handle));
+    WaitForWrite(handle.get());
+    WaitForCallbacks(1);
+  }
+
+  file_group.Close();
+  test_env_->TearDownQueries();
+}
+
 } // namespace impala
