@@ -17,13 +17,21 @@
 
 #include "scheduling/cluster-membership-mgr.h"
 
+#include <boost/algorithm/string/join.hpp>
+#include <gutil/strings/split.h>
+#include <gutil/strings/stringpiece.h>
+
 #include "common/logging.h"
 #include "common/names.h"
-#include "gen-cpp/Frontend_types.h"
+#include "runtime/exec-env.h"
 #include "service/impala-server.h"
 #include "util/metrics.h"
+#include "util/string-parser.h"
 #include "util/test-info.h"
 
+DECLARE_int32(num_expected_executors);
+DECLARE_string(expected_executor_group_sets);
+
 namespace {
 using namespace impala;
 
@@ -52,6 +60,13 @@ static const string LIVE_EXEC_GROUP_KEY("cluster-membership.executor-groups.tota
 static const string HEALTHY_EXEC_GROUP_KEY(
     "cluster-membership.executor-groups.total-healthy");
 static const string TOTAL_BACKENDS_KEY("cluster-membership.backends.total");
+// Per group set metrics
+static const string LIVE_EXEC_GROUP_KEY_FORMAT(
+    "cluster-membership.group-set.executor-groups.total.$0");
+static const string HEALTHY_EXEC_GROUP_KEY_FORMAT(
+    "cluster-membership.group-set.executor-groups.total-healthy.$0");
+static const string TOTAL_BACKENDS_KEY_FORMAT(
+    "cluster-membership.group-set.backends.total.$0");
 
 ClusterMembershipMgr::ClusterMembershipMgr(
     string local_backend_id, StatestoreSubscriber* subscriber, MetricGroup* metrics)
@@ -59,16 +74,38 @@ ClusterMembershipMgr::ClusterMembershipMgr(
     current_membership_(std::make_shared<const Snapshot>()),
     statestore_subscriber_(subscriber),
     local_backend_id_(move(local_backend_id)) {
-  DCHECK(metrics != nullptr);
-  MetricGroup* metric_grp = metrics->GetOrCreateChildGroup("cluster-membership");
-  total_live_executor_groups_ = metric_grp->AddCounter(LIVE_EXEC_GROUP_KEY, 0);
-  total_healthy_executor_groups_ = metric_grp->AddCounter(HEALTHY_EXEC_GROUP_KEY, 0);
-  total_backends_ = metric_grp->AddCounter(TOTAL_BACKENDS_KEY, 0);
+  Status status = PopulateExpectedExecGroupSets(expected_exec_group_sets_);
+  if(!status.ok()) {
+    LOG(FATAL) << "Error populating expected executor group sets: " << status;
+  }
+  InitMetrics(metrics);
   // Register the metric update function as a callback.
   RegisterUpdateCallbackFn([this](
       ClusterMembershipMgr::SnapshotPtr snapshot) { this->UpdateMetrics(snapshot); });
 }
 
+void ClusterMembershipMgr::InitMetrics(MetricGroup* metrics) {
+  DCHECK(metrics != nullptr);
+  MetricGroup* metric_grp = metrics->GetOrCreateChildGroup("cluster-membership");
+  aggregated_group_set_metrics_.total_live_executor_groups_ =
+      metric_grp->AddCounter(LIVE_EXEC_GROUP_KEY, 0);
+  aggregated_group_set_metrics_.total_healthy_executor_groups_ =
+      metric_grp->AddCounter(HEALTHY_EXEC_GROUP_KEY, 0);
+  aggregated_group_set_metrics_.total_backends_ =
+      metric_grp->AddCounter(TOTAL_BACKENDS_KEY, 0);
+
+  for (auto& set : expected_exec_group_sets_) {
+    GroupSetMetrics grp_set_metrics;
+    grp_set_metrics.total_live_executor_groups_ =
+        metric_grp->AddCounter(LIVE_EXEC_GROUP_KEY_FORMAT, 0, set.exec_group_name_prefix);
+    grp_set_metrics.total_healthy_executor_groups_ = metric_grp->AddCounter(
+        HEALTHY_EXEC_GROUP_KEY_FORMAT, 0, set.exec_group_name_prefix);
+    grp_set_metrics.total_backends_ =
+        metric_grp->AddCounter(TOTAL_BACKENDS_KEY_FORMAT, 0, set.exec_group_name_prefix);
+    per_group_set_metrics_.insert({set.exec_group_name_prefix, grp_set_metrics});
+  }
+}
+
 Status ClusterMembershipMgr::Init() {
   LOG(INFO) << "Starting cluster membership manager";
   if (statestore_subscriber_ == nullptr) {
@@ -553,9 +590,35 @@ void ClusterMembershipMgr::UpdateMetrics(const SnapshotPtr& new_state){
     }
   }
   DCHECK_GE(total_live_executor_groups, healthy_executor_groups);
-  total_live_executor_groups_->SetValue(total_live_executor_groups);
-  total_healthy_executor_groups_->SetValue(healthy_executor_groups);
-  total_backends_->SetValue(new_state->current_backends.size());
+  aggregated_group_set_metrics_.total_live_executor_groups_->SetValue(
+      total_live_executor_groups);
+  aggregated_group_set_metrics_.total_healthy_executor_groups_->SetValue(
+      healthy_executor_groups);
+  aggregated_group_set_metrics_.total_backends_->SetValue(
+      new_state->current_backends.size());
+
+  for (auto& set : expected_exec_group_sets_) {
+    int total_backends = 0;
+    int64_t total_live_exec_groups = 0;
+    int64_t healthy_exec_groups = 0;
+    StringPiece prefix(set.exec_group_name_prefix);
+    for (const auto& group_it : new_state->executor_groups) {
+      StringPiece name(group_it.first);
+      if (!name.starts_with(prefix)) continue;
+      const ExecutorGroup& group = group_it.second;
+      if (group.IsHealthy()) {
+        ++healthy_exec_groups;
+      }
+      if (group.NumHosts() > 0) {
+        ++total_live_exec_groups;
+        total_backends += group.NumExecutors();
+      }
+    }
+    auto& grp_metrics = per_group_set_metrics_[set.exec_group_name_prefix];
+    grp_metrics.total_live_executor_groups_->SetValue(total_live_exec_groups);
+    grp_metrics.total_healthy_executor_groups_->SetValue(healthy_exec_groups);
+    grp_metrics.total_backends_->SetValue(total_backends);
+  }
 }
 
 bool ClusterMembershipMgr::IsBackendInExecutorGroups(
@@ -568,48 +631,119 @@ bool ClusterMembershipMgr::IsBackendInExecutorGroups(
   return false;
 }
 
-/// Helper method to populate a thrift request object for cluster membership
-/// using a supplied snapshot from the cluster membership manager.
-///
-/// The frontend uses cluster membership information to determine whether it expects the
-/// scheduler to assign local or remote reads. It also uses the number of executors to
-/// determine the join type (partitioned vs broadcast). For the default executor group, we
-/// assume that local reads are preferred and will include the hostnames and IP addresses
-/// in the update to the frontend. For non-default executor groups, we assume that we will
-/// read data remotely and will only send the number of executors in the largest healthy
-/// group.
+/// For the default executor group, we assume that local reads are preferred and will
+/// include the hostnames and IP addresses in the update to the frontend. For non-default
+/// executor groups, we assume that we will read data remotely and will only send the
+/// number of executors in the largest healthy group. When expected exec group sets are
+/// specified we apply the aforementioned steps for each group set.
 void PopulateExecutorMembershipRequest(ClusterMembershipMgr::SnapshotPtr& snapshot,
+    const vector<TExecutorGroupSet>& expected_exec_group_sets,
     TUpdateExecutorMembershipRequest& update_req) {
-  const ExecutorGroup* group = nullptr;
-  bool is_default_group = false;
+  vector<TExecutorGroupSet> exec_group_sets;
   auto default_it =
       snapshot->executor_groups.find(ImpalaServer::DEFAULT_EXECUTOR_GROUP_NAME);
   if (default_it != snapshot->executor_groups.end()) {
-    is_default_group = true;
-    group = &(default_it->second);
-  } else {
-    int max_num_executors = 0;
-    // Find largest healthy group.
-    for (const auto& it : snapshot->executor_groups) {
-      if (!it.second.IsHealthy()) continue;
-      int num_executors = it.second.NumExecutors();
-      if (num_executors > max_num_executors) {
-        max_num_executors = num_executors;
-        group = &(it.second);
-      }
-    }
-  }
-  if (group) {
+    exec_group_sets.emplace_back();
+    exec_group_sets.back().__set_expected_num_executors(FLAGS_num_expected_executors);
+    const ExecutorGroup* group = &(default_it->second);
     for (const auto& backend : group->GetAllExecutorDescriptors()) {
       if (backend.is_executor()) {
-        if (is_default_group) {
-          update_req.hostnames.insert(backend.address().hostname());
-          update_req.ip_addresses.insert(backend.ip_address());
-        }
-        update_req.num_executors++;
+        update_req.hostnames.insert(backend.address().hostname());
+        update_req.ip_addresses.insert(backend.ip_address());
+        exec_group_sets.back().curr_num_executors++;
       }
     }
+  } else {
+    if (expected_exec_group_sets.empty()) {
+      // Add a default exec group set if no expected group sets were specified.
+      exec_group_sets.emplace_back();
+      exec_group_sets.back().__set_expected_num_executors(FLAGS_num_expected_executors);
+    } else {
+      exec_group_sets.insert(exec_group_sets.begin(), expected_exec_group_sets.begin(),
+          expected_exec_group_sets.end());
+    }
+    int matching_exec_groups_found = 0;
+    for (auto& set : exec_group_sets) {
+      int max_num_executors = -1;
+      StringPiece prefix(set.exec_group_name_prefix);
+      DCHECK(!prefix.empty() || exec_group_sets.size() == 1)
+          << "An empty group set prefix can only exist if no executor group sets are "
+             "specified";
+      for (const auto& it : snapshot->executor_groups) {
+        StringPiece name(it.first);
+        if (!prefix.empty() && !name.starts_with(prefix)) continue;
+        matching_exec_groups_found++;
+        if (!it.second.IsHealthy()) continue;
+        int num_executors = it.second.NumExecutors();
+        if (num_executors > max_num_executors) {
+          max_num_executors = num_executors;
+          set.curr_num_executors = num_executors;
+        }
+      }
+    }
+    if (matching_exec_groups_found != snapshot->executor_groups.size()) {
+      vector<string> group_sets;
+      for (const auto& set : exec_group_sets) {
+        group_sets.push_back(set.exec_group_name_prefix);
+      }
+      vector<string> group_names;
+      for (const auto& it : snapshot->executor_groups) {
+        group_names.push_back(it.first);
+      }
+      LOG(WARNING) << "Some executor groups either do not match expected group sets or "
+                   "match to more than one set. Expected group sets: "
+                << boost::algorithm::join(group_sets, ",") << " Current executor groups: "
+                << boost::algorithm::join(group_names, ",");
+    }
   }
+  update_req.__set_exec_group_sets(exec_group_sets);
+}
+
+Status ClusterMembershipMgr::PopulateExpectedExecGroupSets(
+    std::vector<TExecutorGroupSet>& expected_exec_group_sets) {
+  expected_exec_group_sets.clear();
+  std::unordered_set<string> parsed_group_prefixes;
+  vector<StringPiece> groups;
+  groups = strings::Split(FLAGS_expected_executor_group_sets, ",", strings::SkipEmpty());
+  if (groups.empty()) return Status::OK();
+
+  // Name and expected group size are separated by ':'.
+  for (const StringPiece& group : groups) {
+    int colon_idx = group.find_first_of(':');
+    string group_prefix = group.substr(0, colon_idx).as_string();
+    if (group_prefix.empty()) {
+      return Status(Substitute(
+          "Executor group set prefix cannot be empty for input: $0", group.ToString()));
+    }
+    if (parsed_group_prefixes.find(group_prefix) != parsed_group_prefixes.end()) {
+      return Status(Substitute(
+          "Executor group set prefix specified multiple times: $0", group.ToString()));
+    }
+    if (colon_idx != StringPiece::npos) {
+      StringParser::ParseResult result;
+      int64_t expected_num_executors = StringParser::StringToInt<int64_t>(
+          group.data() + colon_idx + 1, group.length() - colon_idx - 1, &result);
+      if (result != StringParser::PARSE_SUCCESS) {
+        return Status(
+            Substitute("Failed to parse expected executor group set size for input: $0",
+                group.ToString()));
+      }
+      expected_exec_group_sets.emplace_back();
+      expected_exec_group_sets.back().__set_exec_group_name_prefix(group_prefix);
+      expected_exec_group_sets.back().__set_expected_num_executors(
+          expected_num_executors);
+      parsed_group_prefixes.insert(group_prefix);
+    } else {
+      return Status(
+          Substitute("Invalid executor group set format: $0", group.ToString()));
+    }
+  }
+  // sort by increasing order expected group size.
+  sort(expected_exec_group_sets.begin(), expected_exec_group_sets.end(),
+      [](const TExecutorGroupSet& first, const TExecutorGroupSet& second) {
+        return first.expected_num_executors < second.expected_num_executors;
+      });
+  return Status::OK();
 }
 
 } // end namespace impala
