@@ -25,6 +25,7 @@
 #include <vector>
 
 #include "common/status.h"
+#include "gen-cpp/Frontend_types.h"
 #include "scheduling/executor-blacklist.h"
 #include "scheduling/executor-group.h"
 #include "statestore/statestore-subscriber.h"
@@ -170,7 +171,17 @@ class ClusterMembershipMgr {
   /// queries.
   const ExecutorGroup* GetEmptyExecutorGroup() { return &empty_exec_group_; }
 
+  const std::vector<TExecutorGroupSet>& GetExpectedExecGroupSets() {
+    return expected_exec_group_sets_;
+  }
+
  private:
+  struct GroupSetMetrics {
+    IntCounter* total_live_executor_groups_ = nullptr;
+    IntCounter* total_healthy_executor_groups_ = nullptr;
+    IntCounter* total_backends_ = nullptr;
+  };
+
   /// Serializes and adds the local backend descriptor to 'subscriber_topic_updates'.
   void AddLocalBackendToStatestore(const BackendDescriptorPB& local_be_desc,
       std::vector<TTopicDelta>* subscriber_topic_updates);
@@ -201,7 +212,7 @@ class ClusterMembershipMgr {
   /// Updates the membership metrics. Is registered as an updated callback function to
   /// receive any membership updates. The only exception is that this is called directly
   /// in BlacklistExecutor() where updates are not required to be sent to external
-  /// listeners.
+  /// listeners. InitMetrics() must have been called before registering this method.
   void UpdateMetrics(const SnapshotPtr& new_state);
 
   /// Returns true if the 'be_desc' is in any of the 'executor_groups', false otherwise.
@@ -210,18 +221,35 @@ class ClusterMembershipMgr {
   bool IsBackendInExecutorGroups(
       const BackendDescriptorPB& be_desc, const ExecutorGroups& executor_groups);
 
+  /// Parses the --expected_executor_group_sets startup flag and populates
+  /// 'expected_exec_group_sets' with the group name prefix and expected group size. Also
+  /// sorts it in increasing order by the expected group size.
+  static Status PopulateExpectedExecGroupSets(
+      std::vector<TExecutorGroupSet>& expected_exec_group_sets);
+
+  /// Helper method to initialize metrics. Also initializes metrics for all group sets if
+  /// expected group sets are specified.
+  void InitMetrics(MetricGroup* metrics);
+
   /// An empty group used for scheduling coordinator only queries.
   const ExecutorGroup empty_exec_group_;
 
+  /// Info about expected executor group sets parsed from num_expected_executor_groups
+  /// flag in Init() and sorted by the expected group size in increasing order.
+  std::vector<TExecutorGroupSet> expected_exec_group_sets_;
+
   /// Ensures that only one thread is processing a membership update at a time, either
   /// from a statestore update or a blacklisting decision. Must be taken before any other
   /// locks in this class.
   std::mutex update_membership_lock_;
 
-  /// Membership metrics
-  IntCounter* total_live_executor_groups_ = nullptr;
-  IntCounter* total_healthy_executor_groups_ = nullptr;
-  IntCounter* total_backends_ = nullptr;
+  /// Membership metrics for all executor groups regardless of which group set they belong
+  /// to. The value for total backends also includes the coordinators.
+  GroupSetMetrics aggregated_group_set_metrics_;
+
+  /// Maps from the group set prefix to its group set metrics. Is empty if group sets are
+  /// not used.
+  std::unordered_map<string, GroupSetMetrics> per_group_set_metrics_;
 
   /// The snapshot of the current cluster membership. When receiving changes to the
   /// executors configuration from the statestore we will make a copy of the stored
@@ -257,9 +285,17 @@ class ClusterMembershipMgr {
   mutable std::mutex callback_fn_lock_;
 
   friend class impala::test::SchedulerWrapper;
+  friend class ClusterMembershipMgrUnitTest_TestPopulateExpectedExecGroupSets_Test;
 };
 
+/// Helper method to populate a thrift request object 'update_req' for cluster membership
+/// using a supplied 'snapshot' from the cluster membership manager and the
+/// 'expected_exec_group_sets' sorted in increasing order by their expected group size.
+/// The frontend uses cluster membership information to determine whether it expects the
+/// scheduler to assign local or remote reads. It also uses the number of executors to
+/// determine the join type (partitioned vs broadcast).
 void PopulateExecutorMembershipRequest(ClusterMembershipMgr::SnapshotPtr& snapshot,
+    const std::vector<TExecutorGroupSet>& expected_exec_group_sets,
     TUpdateExecutorMembershipRequest& update_req);
 
 } // end namespace impala
