@@ -136,6 +136,9 @@ Status Scheduler::GenerateScanRanges(const vector<TFileSplitGeneratorSpec>& spec
       hdfs_scan_range.__set_partition_path_hash(spec.partition_path_hash);
       TScanRange scan_range;
       scan_range.__set_hdfs_file_split(hdfs_scan_range);
+      if (spec.file_desc.__isset.file_metadata) {
+        scan_range.__set_file_metadata(spec.file_desc.file_metadata);
+      }
       TScanRangeLocationList scan_range_list;
       scan_range_list.__set_scan_range(scan_range);
 
@@ -409,7 +412,20 @@ void Scheduler::CreateCollocatedAndScanInstances(const ExecutorConfig& executor_
   DCHECK(has_union || scan_node_ids.size() == 1) << "This method may need revisiting "
       << "for plans with no union and multiple scans per fragment";
   vector<NetworkAddressPB> scan_hosts;
-  GetScanHosts(executor_config.coord_desc, scan_node_ids, *fragment_state, &scan_hosts);
+  GetScanHosts(scan_node_ids, *fragment_state, &scan_hosts);
+  if (scan_hosts.empty()) {
+    // None of the scan nodes have any scan ranges; run it on a random executor.
+    // TODO TODO: the TODO below seems partially stale
+    // TODO: we'll need to revisit this strategy once we can partition joins
+    // (in which case this fragment might be executing a right outer join
+    // with a large build table)
+    vector<BackendDescriptorPB> all_executors =
+        executor_config.group.GetAllExecutorDescriptors();
+    int idx = std::uniform_int_distribution<int>(0, all_executors.size() - 1)(
+        *state->rng());
+    const BackendDescriptorPB& be_desc = all_executors[idx];
+    scan_hosts.push_back(be_desc.address());
+  }
   for (const NetworkAddressPB& host_addr : scan_hosts) {
     // Ensure that the num instances is at least as many as input fragments. We don't
     // want to increment if there were already some instances from the input fragment,
@@ -784,9 +800,8 @@ std::vector<TPlanNodeId> Scheduler::FindScanNodes(const TPlan& plan) {
   return FindNodes(plan, SCAN_NODE_TYPES);
 }
 
-void Scheduler::GetScanHosts(const BackendDescriptorPB& coord_desc,
-    const vector<TPlanNodeId>& scan_ids, const FragmentScheduleState& fragment_state,
-    vector<NetworkAddressPB>* scan_hosts) {
+void Scheduler::GetScanHosts(const vector<TPlanNodeId>& scan_ids,
+    const FragmentScheduleState& fragment_state, vector<NetworkAddressPB>* scan_hosts) {
   for (const TPlanNodeId& scan_id : scan_ids) {
     // Get the list of impalad host from scan_range_assignment_
     for (const FragmentScanRangeAssignment::value_type& scan_range_assignment :
@@ -796,14 +811,6 @@ void Scheduler::GetScanHosts(const BackendDescriptorPB& coord_desc,
         scan_hosts->push_back(scan_range_assignment.first);
       }
     }
-
-    if (scan_hosts->empty()) {
-      // this scan node doesn't have any scan ranges; run it on the coordinator
-      // TODO: we'll need to revisit this strategy once we can partition joins
-      // (in which case this fragment might be executing a right outer join
-      // with a large build table)
-      scan_hosts->push_back(coord_desc.address());
-    }
   }
 }
 
@@ -821,11 +828,13 @@ Status Scheduler::Schedule(const ExecutorConfig& executor_config, ScheduleState*
 
 bool Scheduler::IsCoordinatorOnlyQuery(const TQueryExecRequest& exec_request) {
   DCHECK_GT(exec_request.plan_exec_info.size(), 0);
+  int64_t num_parallel_plans = exec_request.plan_exec_info.size();
   const TPlanExecInfo& plan_exec_info = exec_request.plan_exec_info[0];
   int64_t num_fragments = plan_exec_info.fragments.size();
   DCHECK_GT(num_fragments, 0);
   auto type = plan_exec_info.fragments[0].partition.type;
-  return num_fragments == 1 && type == TPartitionType::UNPARTITIONED;
+  return num_parallel_plans == 1 && num_fragments == 1
+      && type == TPartitionType::UNPARTITIONED;
 }
 
 void Scheduler::ComputeBackendExecParams(
@@ -1120,6 +1129,9 @@ void TScanRangeToScanRangePB(const TScanRange& tscan_range, ScanRangePB* scan_ra
   if (tscan_range.__isset.kudu_scan_token) {
     scan_range_pb->set_kudu_scan_token(tscan_range.kudu_scan_token);
   }
+  if (tscan_range.__isset.file_metadata) {
+    scan_range_pb->set_file_metadata(tscan_range.file_metadata);
+  }
 }
 
 void Scheduler::AssignmentCtx::RecordScanRangeAssignment(
