@@ -239,7 +239,8 @@ Status ClientRequestState::Exec() {
     case TStmtType::QUERY:
     case TStmtType::DML:
       DCHECK(exec_request_->__isset.query_exec_request);
-      RETURN_IF_ERROR(ExecAsyncQueryOrDmlRequest(exec_request_->query_exec_request));
+      RETURN_IF_ERROR(
+          ExecQueryOrDmlRequest(exec_request_->query_exec_request, true /*async*/));
       break;
     case TStmtType::EXPLAIN: {
       request_result_set_.reset(new vector<TResultRow>(
@@ -258,35 +259,7 @@ Status ClientRequestState::Exec() {
     }
     case TStmtType::LOAD: {
       DCHECK(exec_request_->__isset.load_data_request);
-      TLoadDataResp response;
-      RETURN_IF_ERROR(
-          frontend_->LoadData(exec_request_->load_data_request, &response));
-      request_result_set_.reset(new vector<TResultRow>);
-      request_result_set_->push_back(response.load_summary);
-
-      // Now refresh the table metadata.
-      TCatalogOpRequest reset_req;
-      reset_req.__set_sync_ddl(exec_request_->query_options.sync_ddl);
-      reset_req.__set_op_type(TCatalogOpType::RESET_METADATA);
-      reset_req.__set_reset_metadata_params(TResetMetadataRequest());
-      reset_req.reset_metadata_params.__set_header(TCatalogServiceRequestHeader());
-      reset_req.reset_metadata_params.header.__set_want_minimal_response(
-          FLAGS_use_local_catalog);
-      reset_req.reset_metadata_params.__set_is_refresh(true);
-      reset_req.reset_metadata_params.__set_table_name(
-          exec_request_->load_data_request.table_name);
-      if (exec_request_->load_data_request.__isset.partition_spec) {
-        reset_req.reset_metadata_params.__set_partition_spec(
-            exec_request_->load_data_request.partition_spec);
-      }
-      reset_req.reset_metadata_params.__set_sync_ddl(
-          exec_request_->query_options.sync_ddl);
-      catalog_op_executor_.reset(
-          new CatalogOpExecutor(ExecEnv::GetInstance(), frontend_, server_profile_));
-      RETURN_IF_ERROR(catalog_op_executor_->Exec(reset_req));
-      RETURN_IF_ERROR(parent_server_->ProcessCatalogUpdateResult(
-          *catalog_op_executor_->update_catalog_result(),
-          exec_request_->query_options.sync_ddl));
+      LOG_AND_RETURN_IF_ERROR(ExecLoadDataRequest());
       break;
     }
     case TStmtType::SET: {
@@ -512,8 +485,8 @@ Status ClientRequestState::ExecLocalCatalogOp(
   }
 }
 
-Status ClientRequestState::ExecAsyncQueryOrDmlRequest(
-    const TQueryExecRequest& query_exec_request) {
+Status ClientRequestState::ExecQueryOrDmlRequest(
+    const TQueryExecRequest& query_exec_request, bool isAsync) {
   // we always need at least one plan fragment
   DCHECK(query_exec_request.plan_exec_info.size() > 0);
 
@@ -574,11 +547,18 @@ Status ClientRequestState::ExecAsyncQueryOrDmlRequest(
     // Don't start executing the query if Cancel() was called concurrently with Exec().
     if (is_cancelled_) return Status::CANCELLED;
   }
-  // Don't transition to PENDING inside the FinishExecQueryOrDmlRequest thread because
-  // the query should be in the PENDING state before the Exec RPC returns.
-  UpdateNonErrorExecState(ExecState::PENDING);
-  RETURN_IF_ERROR(Thread::Create("query-exec-state", "async-exec-thread",
-      &ClientRequestState::FinishExecQueryOrDmlRequest, this, &async_exec_thread_, true));
+  if (isAsync) {
+    // Don't transition to PENDING inside the FinishExecQueryOrDmlRequest thread because
+    // the query should be in the PENDING state before the Exec RPC returns.
+    UpdateNonErrorExecState(ExecState::PENDING);
+    RETURN_IF_ERROR(Thread::Create("query-exec-state", "async-exec-thread",
+        &ClientRequestState::FinishExecQueryOrDmlRequest, this, &async_exec_thread_,
+        true));
+  } else {
+    // Update query_status_ as necessary.
+    FinishExecQueryOrDmlRequest();
+    return query_status_;
+  }
   return Status::OK();
 }
 
@@ -640,10 +620,7 @@ void ClientRequestState::FinishExecQueryOrDmlRequest() {
   UpdateNonErrorExecState(ExecState::RUNNING);
 }
 
-Status ClientRequestState::ExecDdlRequest() {
-  string op_type = catalog_op_type() == TCatalogOpType::DDL ?
-      PrintThriftEnum(ddl_type()) : PrintThriftEnum(catalog_op_type());
-  summary_profile_->AddInfoString("DDL Type", op_type);
+Status ClientRequestState::ExecDdlRequestImplSync() {
 
   if (catalog_op_type() != TCatalogOpType::DDL &&
       catalog_op_type() != TCatalogOpType::RESET_METADATA) {
@@ -683,12 +660,34 @@ Status ClientRequestState::ExecDdlRequest() {
     return Status::OK();
   }
 
+  DCHECK(false) << "Not handled sync exec ddl request.";
+  return Status::OK();
+}
+
+void ClientRequestState::ExecDdlRequestImpl(bool exec_in_worker_thread) {
+  bool is_CTAS = (catalog_op_type() == TCatalogOpType::DDL
+      && ddl_type() == TDdlType::CREATE_TABLE_AS_SELECT);
+
   catalog_op_executor_.reset(
       new CatalogOpExecutor(ExecEnv::GetInstance(), frontend_, server_profile_));
+
+  // Indirectly check if running in thread async_exec_thread_.
+  if (exec_in_worker_thread) {
+    DCHECK(exec_state() == ExecState::PENDING);
+
+    // 1. For any non-CTAS DDLs, transition to RUNNING
+    // 2. For CTAS DDLs, transition to RUNNING during FinishExecQueryOrDmlRequest()
+    //    called by ExecQueryOrDmlRequest().
+    if (!is_CTAS) UpdateNonErrorExecState(ExecState::RUNNING);
+  }
+
+  // Optionally wait with a debug action before Exec() below.
+  DebugActionNoFail(exec_request_->query_options, "CRS_DELAY_BEFORE_CATALOG_OP_EXEC");
+
   Status status = catalog_op_executor_->Exec(exec_request_->catalog_op_request);
   {
     lock_guard<mutex> l(lock_);
-    RETURN_IF_ERROR(UpdateQueryStatus(status));
+    RETURN_VOID_IF_ERROR(UpdateQueryStatus(status));
   }
 
   // If this is a CTAS request, there will usually be more work to do
@@ -700,27 +699,142 @@ Status ClientRequestState::ExecDdlRequest() {
       !catalog_op_executor_->ddl_exec_response()->new_table_created) {
     DCHECK(exec_request_->catalog_op_request.
         ddl_params.create_table_params.if_not_exists);
-    return Status::OK();
+    return;
   }
 
   // Add newly created table to catalog cache.
-  RETURN_IF_ERROR(parent_server_->ProcessCatalogUpdateResult(
+  status = parent_server_->ProcessCatalogUpdateResult(
       *catalog_op_executor_->update_catalog_result(),
-      exec_request_->query_options.sync_ddl));
+      exec_request_->query_options.sync_ddl);
+  {
+    lock_guard<mutex> l(lock_);
+    RETURN_VOID_IF_ERROR(UpdateQueryStatus(status));
+  }
 
-  if (catalog_op_type() == TCatalogOpType::DDL &&
-      ddl_type() == TDdlType::CREATE_TABLE_AS_SELECT) {
+  if (is_CTAS) {
     // At this point, the remainder of the CTAS request executes
     // like a normal DML request. As with other DML requests, it will
     // wait for another catalog update if any partitions were altered as a result
     // of the operation.
     DCHECK(exec_request_->__isset.query_exec_request);
-    RETURN_IF_ERROR(ExecAsyncQueryOrDmlRequest(exec_request_->query_exec_request));
+    RETURN_VOID_IF_ERROR(
+        ExecQueryOrDmlRequest(exec_request_->query_exec_request, !exec_in_worker_thread));
   }
 
-  // Set the results to be reported to the client.
-  SetResultSet(catalog_op_executor_->ddl_exec_response());
-  return Status::OK();
+  // Set the results to be reported to the client. Do this under lock to avoid races
+  // with ImpalaServer::GetResultSetMetadata().
+  {
+    lock_guard<mutex> l(lock_);
+    SetResultSet(catalog_op_executor_->ddl_exec_response());
+  }
+}
+
+bool ClientRequestState::ShouldRunExecDdlAsync() {
+  // Local catalog op DDL will run synchronously.
+  if (catalog_op_type() != TCatalogOpType::DDL
+      && catalog_op_type() != TCatalogOpType::RESET_METADATA) {
+    return false;
+  }
+
+  // The exec DDL part of compute stats will run synchronously.
+  if (ddl_type() == TDdlType::COMPUTE_STATS) return false;
+
+  return true;
+}
+
+Status ClientRequestState::ExecDdlRequest() {
+  string op_type = catalog_op_type() == TCatalogOpType::DDL ?
+      PrintThriftEnum(ddl_type()) : PrintThriftEnum(catalog_op_type());
+  bool async_ddl = ShouldRunExecDdlAsync();
+  bool async_ddl_enabled = exec_request_->query_options.enable_async_ddl_execution;
+  string exec_mode = (async_ddl && async_ddl_enabled) ? "asynchronous" : "synchronous";
+
+  summary_profile_->AddInfoString("DDL Type", op_type);
+  summary_profile_->AddInfoString("DDL execution mode", exec_mode);
+  VLOG_QUERY << "DDL exec mode=" << exec_mode;
+
+  if (!async_ddl) return ExecDdlRequestImplSync();
+
+  if (async_ddl_enabled) {
+    // Transition the exec state out of INITIALIZED to PENDING to make available the
+    // runtime profile for the DDL. Later on in ExecDdlRequestImpl(), the state
+    // further transitions to RUNNING.
+    UpdateNonErrorExecState(ExecState::PENDING);
+    return Thread::Create("impala-server", "async_exec_thread_",
+        &ClientRequestState::ExecDdlRequestImpl, this, true /*exec in a worker thread*/,
+        &async_exec_thread_);
+  } else {
+    ExecDdlRequestImpl(false /*exec in the same thread as the caller*/);
+    return query_status_;
+  }
+}
+
+void ClientRequestState::ExecLoadDataRequestImpl(bool exec_in_worker_thread) {
+  if (exec_in_worker_thread) {
+    DCHECK(exec_state() == ExecState::PENDING);
+    UpdateNonErrorExecState(ExecState::RUNNING);
+  }
+  DebugActionNoFail(
+      exec_request_->query_options, "CRS_DELAY_BEFORE_LOAD_DATA");
+
+  TLoadDataResp response;
+  Status status = frontend_->LoadData(exec_request_->load_data_request, &response);
+  {
+    lock_guard<mutex> l(lock_);
+    RETURN_VOID_IF_ERROR(UpdateQueryStatus(status));
+  }
+
+  request_result_set_.reset(new vector<TResultRow>);
+  request_result_set_->push_back(response.load_summary);
+
+  // Now refresh the table metadata.
+  TCatalogOpRequest reset_req;
+  reset_req.__set_sync_ddl(exec_request_->query_options.sync_ddl);
+  reset_req.__set_op_type(TCatalogOpType::RESET_METADATA);
+  reset_req.__set_reset_metadata_params(TResetMetadataRequest());
+  reset_req.reset_metadata_params.__set_header(TCatalogServiceRequestHeader());
+  reset_req.reset_metadata_params.header.__set_want_minimal_response(
+      FLAGS_use_local_catalog);
+  reset_req.reset_metadata_params.__set_is_refresh(true);
+  reset_req.reset_metadata_params.__set_table_name(
+      exec_request_->load_data_request.table_name);
+  if (exec_request_->load_data_request.__isset.partition_spec) {
+    reset_req.reset_metadata_params.__set_partition_spec(
+        exec_request_->load_data_request.partition_spec);
+  }
+  reset_req.reset_metadata_params.__set_sync_ddl(
+      exec_request_->query_options.sync_ddl);
+  catalog_op_executor_.reset(
+      new CatalogOpExecutor(ExecEnv::GetInstance(), frontend_, server_profile_));
+
+  status = catalog_op_executor_->Exec(reset_req);
+  {
+    lock_guard<mutex> l(lock_);
+    RETURN_VOID_IF_ERROR(UpdateQueryStatus(status));
+  }
+
+  status = parent_server_->ProcessCatalogUpdateResult(
+      *catalog_op_executor_->update_catalog_result(),
+      exec_request_->query_options.sync_ddl);
+  {
+    lock_guard<mutex> l(lock_);
+    RETURN_VOID_IF_ERROR(UpdateQueryStatus(status));
+  }
+}
+
+
+Status ClientRequestState::ExecLoadDataRequest() {
+  if (exec_request_->query_options.enable_async_load_data_execution) {
+    // Transition the exec state out of INITIALIZED to PENDING to make available the
+    // runtime profile for the DDL.
+    UpdateNonErrorExecState(ExecState::PENDING);
+    return Thread::Create("impala-server", "async_exec_thread_",
+        &ClientRequestState::ExecLoadDataRequestImpl, this, true, &async_exec_thread_);
+  }
+
+  // sync exection
+  ExecLoadDataRequestImpl(false /* not use a worker thread */);
+  return query_status_;
 }
 
 Status ClientRequestState::ExecShutdownRequest() {
@@ -822,7 +936,11 @@ Status ClientRequestState::Finalize(bool check_inflight, const Status* cause) {
   }
 
   // If the transaction didn't get committed by this point then we should just abort it.
-  if (InTransaction()) AbortTransaction();
+  if (InTransaction()) {
+    AbortTransaction();
+  } else if (InKuduTransaction()) {
+    AbortKuduTransaction();
+  }
 
   UpdateEndTime();
 
@@ -946,9 +1064,24 @@ Status ClientRequestState::WaitInternal() {
   }
   if (!child_queries.empty()) query_events_->MarkEvent("Child queries finished");
 
+  bool isCTAS = catalog_op_type() == TCatalogOpType::DDL
+      && ddl_type() == TDdlType::CREATE_TABLE_AS_SELECT;
+
   if (GetCoordinator() != NULL) {
-    RETURN_IF_ERROR(GetCoordinator()->Wait());
+    Status status = GetCoordinator()->Wait();
+    if (UNLIKELY(!status.ok())) {
+      if (InKuduTransaction()) AbortKuduTransaction();
+      return status;
+    }
     RETURN_IF_ERROR(UpdateCatalog());
+  } else {
+    // When the coordinator is not available for CTAS that requires a coordinator, check
+    // further if the query has been cancelled. If so, return immediately as there will
+    // be no query result available (IMPALA-11006).
+    if (isCTAS) {
+      lock_guard<mutex> l(lock_);
+      if (is_cancelled_) return Status::CANCELLED;
+    }
   }
 
   if (catalog_op_type() == TCatalogOpType::DDL &&
@@ -960,8 +1093,7 @@ Status ClientRequestState::WaitInternal() {
     // Queries that do not return a result are finished at this point. This includes
     // DML operations.
     eos_.Store(true);
-  } else if (catalog_op_type() == TCatalogOpType::DDL &&
-      ddl_type() == TDdlType::CREATE_TABLE_AS_SELECT) {
+  } else if (isCTAS) {
     SetCreateTableAsSelectResultSet();
   }
   // Rows are available now (for SELECT statement), so start the 'wait' timer that tracks
@@ -1006,7 +1138,7 @@ Status ClientRequestState::RestartFetch() {
 void ClientRequestState::UpdateNonErrorExecState(ExecState new_state) {
   lock_guard<mutex> l(lock_);
   ExecState old_state = exec_state();
-  static string error_msg = "Illegal state transition: $0 -> $1, query_id=$3";
+  static string error_msg = "Illegal state transition: $0 -> $1, query_id=$2";
   switch (new_state) {
     case ExecState::PENDING:
       DCHECK(old_state == ExecState::INITIALIZED)
@@ -1230,8 +1362,12 @@ Status ClientRequestState::Cancel(
           || retry_state() == RetryState::RETRYING);
     }
 
-    admission_control_client_->CancelAdmission();
-    is_cancelled_ = true;
+    // To avoid recalling RemoteAdmissionControlClient::CancelAdmission() since it will
+    // send extra RPC.
+    if (!is_cancelled_) {
+      admission_control_client_->CancelAdmission();
+      is_cancelled_ = true;
+    }
   } // Release lock_ before doing cancellation work.
 
   // Cancel and close child queries before cancelling parent. 'lock_' should not be held
@@ -1332,6 +1468,17 @@ Status ClientRequestState::UpdateCatalog() {
       RETURN_IF_ERROR(parent_server_->ProcessCatalogUpdateResult(resp.result,
           exec_request_->query_options.sync_ddl));
     }
+  } else if (InKuduTransaction()) {
+    // Commit the Kudu transaction. Clear transaction state if it's successful.
+    // Otherwise, abort the Kudu transaction and clear transaction state.
+    // Note that TQueryExecRequest.finalize_params is not set for inserting rows to Kudu
+    // table.
+    Status status = CommitKuduTransaction();
+    if (UNLIKELY(!status.ok())) {
+      AbortKuduTransaction();
+      LOG(ERROR) << "ERROR Finalizing DML: " << status.GetDetail();
+      return status;
+    }
   }
   query_events_->MarkEvent("DML Metastore update finished");
   return Status::OK();
@@ -1610,6 +1757,38 @@ void ClientRequestState::ClearTransactionState() {
   transaction_closed_ = true;
 }
 
+bool ClientRequestState::InKuduTransaction() const {
+  // If Kudu transaction is opened, TQueryExecRequest.query_ctx.is_kudu_transactional
+  // is set as true by Frontend.doCreateExecRequest().
+  DCHECK(exec_request_ != nullptr);
+  return (exec_request_->query_exec_request.query_ctx.is_kudu_transactional
+      && !transaction_closed_);
+}
+
+void ClientRequestState::AbortKuduTransaction() {
+  DCHECK(InKuduTransaction());
+  if (frontend_->AbortKuduTransaction(query_ctx_.query_id).ok()) {
+    query_events_->MarkEvent("Kudu transaction aborted");
+  } else {
+    VLOG(1) << Substitute("Unable to abort Kudu transaction with query-id: $0",
+        PrintId(query_ctx_.query_id));
+  }
+  transaction_closed_ = true;
+}
+
+Status ClientRequestState::CommitKuduTransaction() {
+  DCHECK(InKuduTransaction());
+  Status status = frontend_->CommitKuduTransaction(query_ctx_.query_id);
+  if (status.ok()) {
+    query_events_->MarkEvent("Kudu transaction committed");
+    transaction_closed_ = true;
+  } else {
+    VLOG(1) << Substitute("Unable to commit Kudu transaction with query-id: $0",
+        PrintId(query_ctx_.query_id));
+  }
+  return status;
+}
+
 void ClientRequestState::LogQueryEvents() {
   // Wait until the results are available. This guarantees the completion of non QUERY
   // statements like DDL/DML etc. Query events are logged if the query reaches a FINISHED
