@@ -494,6 +494,9 @@ class ClientRequestState {
   /// execution in the following cases:
   /// 1. exec_request().stmt_type == QUERY or DML.
   /// 2. CTAS query for creating a table that does not exist.
+  /// 3. DDLs that are not the following:
+  ///    a. Local catalog operations requiring no catalog service access;
+  ///    b. Compute Stats.
   std::unique_ptr<Thread> async_exec_thread_;
 
   /// Set by the async-exec-thread after successful admission. Accessed through
@@ -672,11 +675,13 @@ class ClientRequestState {
   /// actively processed. Takes expiration_data_lock_.
   void MarkActive();
 
-  /// Sets up profile and pre-execution counters, creates the query schedule, and spawns
-  /// a thread that calls FinishExecQueryOrDmlRequest() which contains the core logic of
-  /// executing a QUERY or DML execution request.
+  /// Sets up profile and pre-execution counters, creates the query schedule, and calls
+  /// FinishExecQueryOrDmlRequest() which contains the core logic of executing a QUERY or
+  /// DML execution request. When 'async' is true, spawn a thread to run
+  /// FinishExecQueryOrDmlRequest(). Otherwise, the method runs in the same thread as the
+  /// caller.
   /// Non-blocking.
-  Status ExecAsyncQueryOrDmlRequest(const TQueryExecRequest& query_exec_request)
+  Status ExecQueryOrDmlRequest(const TQueryExecRequest& query_exec_request, bool async)
       WARN_UNUSED_RESULT;
 
   /// Submits the exec request to the admission controller and on successful admission,
@@ -689,6 +694,28 @@ class ClientRequestState {
   /// queries (e.g., compute stats) or dml (e.g., create table as select)
   Status ExecDdlRequest() WARN_UNUSED_RESULT;
 
+  /// A helper function to execute certain ddl requests synchronously.
+  Status ExecDdlRequestImplSync() WARN_UNUSED_RESULT;
+
+  /// A helper function to execute certain ddl requests optionally
+  /// asynchronously. 'exec_in_worker_thread' indicates whether the execution of
+  /// DDL is done in a worker thread or not. This flag is set to true when the
+  /// execution of the DDL request runs in 'async_exec_thread_', and set to false
+  /// otherwise.
+  void ExecDdlRequestImpl(bool exec_in_worker_thread);
+
+  /// Decide whether to call ExecDdlRequestImpl() or
+  /// ExecDdlRequestImplSync() for ExecDdlRequest(). Return true to call
+  /// ExecDdlRequestImpl() and false to call ExecDdlRequestImplSync().
+  bool ShouldRunExecDdlAsync();
+
+  /// The logic of executing a load data statement which runs ExecLoadDataRequestImpl()
+  /// in async_exec_thread_ by default.
+  Status ExecLoadDataRequest() WARN_UNUSED_RESULT;
+
+  /// Core logic of executing a load data statement.
+  void ExecLoadDataRequestImpl(bool exec_in_worker_thread);
+
   /// Executes a shut down request.
   Status ExecShutdownRequest() WARN_UNUSED_RESULT;
 
@@ -703,7 +730,8 @@ class ClientRequestState {
 
   /// Gather and publish all required updates to the metastore.
   /// For transactional queries:
-  /// If everything goes well the transaction is committed by the Catalogd.
+  /// If everything goes well the Hive transaction is committed by the Catalogd,
+  /// but the Kudu transaction is committed by this function.
   /// If an error occurs the transaction gets aborted by this function. Either way
   /// the transaction will be closed when this function returns.
   Status UpdateCatalog() WARN_UNUSED_RESULT;
@@ -748,19 +776,28 @@ class ClientRequestState {
   /// 'SET' and all of them for 'SET ALL'
   void PopulateResultForSet(bool is_set_all);
 
-  /// Returns the transaction id for this client request. 'InTransaction()' must be
+  /// Returns the Hive transaction id for this client request. 'InTransaction()' must be
   /// true when invoked.
   int64_t GetTransactionId() const;
 
-  /// Returns true if there is an open transaction for this client request.
+  /// Returns true if there is an open Hive transaction for this client request.
   bool InTransaction() const;
 
-  /// Aborts the transaction of this client request.
+  /// Aborts the Hive transaction of this client request.
   void AbortTransaction();
 
-  /// Invoke this function when the transaction is committed or aborted.
+  /// Invoke this function when the Hive transaction is committed or aborted.
   void ClearTransactionState();
 
+  /// Returns true if there is an open Kudu transaction for this client request.
+  bool InKuduTransaction() const;
+
+  /// Aborts the Kudu transaction of this client request.
+  void AbortKuduTransaction();
+
+  /// Commits the Kudu transaction of this client request.
+  Status CommitKuduTransaction();
+
   /// helper that logs the audit record for this query id. Takes the query_status
   /// as input parameter so that it operates on the same status polled in the
   /// beginning of LogQueryEvents().
