@@ -74,6 +74,11 @@ DEFINE_string(authorized_proxy_group_config, "",
     "all users. For example: hue=group1,group2;admin=*");
 DEFINE_string(authorized_proxy_group_config_delimiter, ",",
     "Specifies the delimiter used in authorized_proxy_group_config. ");
+DEFINE_bool(enable_shell_based_groups_mapping_support, false,
+    "Enables support for Hadoop groups mapping "
+    "org.apache.hadoop.security.ShellBasedUnixGroupsMapping. By default this support "
+    "is not enabled as it can lead to many process getting spawned to fetch groups for "
+    "user using shell command.");
 DEFINE_string(kudu_master_hosts, "", "Specifies the default Kudu master(s). The given "
     "value should be a comma separated list of hostnames or IP addresses; ports are "
     "optional.");
@@ -115,7 +120,9 @@ Frontend::Frontend() {
     {"unregisterTransaction", "(J)V", &unregister_txn_},
     {"getSaml2Redirect", "([B)[B", &get_saml2_redirect_id_},
     {"validateSaml2Response", "([B)[B", &validate_saml2_response_id_},
-    {"validateSaml2Bearer", "([B)Ljava/lang/String;", &validate_saml2_bearer_id_}
+    {"validateSaml2Bearer", "([B)Ljava/lang/String;", &validate_saml2_bearer_id_},
+    {"abortKuduTransaction", "([B)V", &abort_kudu_txn_},
+    {"commitKuduTransaction", "([B)V", &commit_kudu_txn_}
   };
 
   JNIEnv* jni_env = JniUtil::GetJNIEnv();
@@ -343,3 +350,11 @@ Status Frontend::ValidateSaml2Bearer(
   return JniUtil::CallJniMethod(
       fe_, validate_saml2_bearer_id_, request, user);
 }
+
+Status Frontend::AbortKuduTransaction(const TUniqueId& query_id) {
+  return JniUtil::CallJniMethod(fe_, abort_kudu_txn_, query_id);
+}
+
+Status Frontend::CommitKuduTransaction(const TUniqueId& query_id) {
+  return JniUtil::CallJniMethod(fe_, commit_kudu_txn_, query_id);
+}
