@@ -556,7 +556,8 @@ void ImpalaServer::ExecuteStatement(TExecuteStatementResp& return_val,
 void ImpalaServer::ExecutePlannedStatement(
       TExecuteStatementResp& return_val,
       const TExecutePlannedStatementReq& request) {
-  VLOG_QUERY << "ExecutePlannedStatement(): request=" << ThriftDebugString(request);
+  VLOG_QUERY << "ExecutePlannedStatement(): request=" << RedactedDebugString(request);
+  VLOG(3) << "ExecutePlannedStatement(): plan=" << ThriftDebugString(request.plan);
   const ThriftServer::ConnectionContext* connection_context =
       ThriftServer::GetThreadConnectionContext();
   // This RPC is only supported on the external frontend service and should only be
@@ -1247,7 +1248,8 @@ void ImpalaServer::GetExecutorMembership(
 
   // Populate an instance of TUpdateExecutorMembershipRequest
   // with the field values retrieved from membership_snapshot
-  PopulateExecutorMembershipRequest(membership_snapshot, return_val.executor_membership);
+  PopulateExecutorMembershipRequest(membership_snapshot,
+      cluster_membership_mgr->GetExpectedExecGroupSets(), return_val.executor_membership);
 
   return_val.status.__set_statusCode(thrift::TStatusCode::SUCCESS_STATUS);
   VLOG_RPC << "GetExecutorMembership(): return_val=" << ThriftDebugString(return_val);
