@@ -886,6 +886,10 @@ class ImpalaServer : public ImpalaServiceIf,
   /// Initializes the backend descriptor in 'be_desc' with the local backend information.
   void BuildLocalBackendDescriptorInternal(BackendDescriptorPB* be_desc);
 
+  /// Converts a type to a string. The only complex type supported is array which
+  /// is returned as string (see IMPALA-11041).
+  std::string ColumnTypeToBeeswaxTypeString(const TColumnType& type);
+
   /// Snapshot of a query's state, archived in the query log. Not mutated after
   /// construction.
   struct QueryStateRecord {
@@ -1626,5 +1630,9 @@ class ImpalaServer : public ImpalaServiceIf,
   /// client requests and running finstances. Set before 'shutting_down_' and updated
   /// atomically if a new shutdown command with a shorter deadline comes in.
   AtomicInt64 shutdown_deadline_{0};
+
+  /// Stores the last version number for the admission heartbeat that was sent.
+  /// Incremented every time a new admission heartbeat is sent.
+  int64_t admission_heartbeat_version_ = 0;
 };
 }
