@@ -98,6 +98,24 @@ const string PrintQueryOptionValue(const impala::TCompressionCodec& compression_
   }
 }
 
+std::ostream& impala::operator<<(std::ostream& out,
+    const std::set<impala::TRuntimeFilterType::type>& filter_types) {
+  bool first = true;
+  for (const auto& t : filter_types) {
+    if (!first) out << ",";
+    out << t;
+    first = false;
+  }
+  return out;
+}
+
+const string PrintQueryOptionValue(
+    const set<impala::TRuntimeFilterType::type>& filter_types) {
+  stringstream val;
+  val << filter_types;
+  return val.str();
+}
+
 void impala::TQueryOptionsToMap(const TQueryOptions& query_options,
     map<string, string>* configuration) {
 #define QUERY_OPT_FN(NAME, ENUM, LEVEL)\
@@ -442,9 +460,9 @@ Status impala::SetQueryOption(const string& key, const string& value,
         break;
       }
       case TImpalaQueryOptions::PARQUET_FALLBACK_SCHEMA_RESOLUTION: {
-        TParquetFallbackSchemaResolution::type enum_type;
+        TSchemaResolutionStrategy::type enum_type;
         RETURN_IF_ERROR(GetThriftEnum(value, "parquet fallback schema resolution",
-            _TParquetFallbackSchemaResolution_VALUES_TO_NAMES, &enum_type));
+            _TSchemaResolutionStrategy_VALUES_TO_NAMES, &enum_type));
         query_options->__set_parquet_fallback_schema_resolution(enum_type);
         break;
       }
@@ -510,10 +528,25 @@ Status impala::SetQueryOption(const string& key, const string& value,
         query_options->__set_parquet_dictionary_filtering(IsTrue(value));
         break;
       }
+      case TImpalaQueryOptions::PARQUET_BLOOM_FILTERING: {
+        query_options->__set_parquet_bloom_filtering(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::PARQUET_BLOOM_FILTER_WRITE: {
+        TParquetBloomFilterWrite::type enum_type;
+        RETURN_IF_ERROR(GetThriftEnum(value, "Parquet Bloom filter write",
+           _TParquetBloomFilterWrite_VALUES_TO_NAMES, &enum_type));
+        query_options->__set_parquet_bloom_filter_write(enum_type);
+        break;
+      }
       case TImpalaQueryOptions::PARQUET_READ_STATISTICS: {
         query_options->__set_parquet_read_statistics(IsTrue(value));
         break;
       }
+      case TImpalaQueryOptions::ORC_READ_STATISTICS: {
+        query_options->__set_orc_read_statistics(IsTrue(value));
+        break;
+      }
       case TImpalaQueryOptions::DEFAULT_JOIN_DISTRIBUTION_MODE: {
         TJoinDistributionMode::type enum_type;
         RETURN_IF_ERROR(GetThriftEnum(value, "default join distribution mode",
@@ -932,11 +965,23 @@ Status impala::SetQueryOption(const string& key, const string& value,
         break;
       }
       case TImpalaQueryOptions::ENABLED_RUNTIME_FILTER_TYPES: {
-        // Parse the enabled runtime filter types and validate it.
-        TEnabledRuntimeFilterTypes::type enum_type;
-        RETURN_IF_ERROR(GetThriftEnum(value, "enabled runtime filter types",
-            _TEnabledRuntimeFilterTypes_VALUES_TO_NAMES, &enum_type));
-        query_options->__set_enabled_runtime_filter_types(enum_type);
+        set<TRuntimeFilterType::type> filter_types;
+        if (iequals(value, "all")) {
+          for (const auto& kv : _TRuntimeFilterType_VALUES_TO_NAMES) {
+            filter_types.insert(static_cast<TRuntimeFilterType::type>(kv.first));
+          }
+        } else {
+          // Parse and verify the enabled runtime filter types.
+          vector<string> str_types;
+          split(str_types, value, is_any_of(","), token_compress_on);
+          for (const auto& t : str_types) {
+            TRuntimeFilterType::type filter_type;
+            RETURN_IF_ERROR(GetThriftEnum(t, "runtime filter type",
+                _TRuntimeFilterType_VALUES_TO_NAMES, &filter_type));
+            filter_types.insert(filter_type);
+          }
+        }
+        query_options->__set_enabled_runtime_filter_types(filter_types);
         break;
       }
       case TImpalaQueryOptions::ASYNC_CODEGEN: {
@@ -1076,6 +1121,114 @@ Status impala::SetQueryOption(const string& key, const string& value,
         query_options->__set_default_ndv_scale(scale);
         break;
       }
+      case TImpalaQueryOptions::KUDU_REPLICA_SELECTION: {
+        // Parse the kudu replica selection and validate it.
+        TKuduReplicaSelection::type enum_type;
+        RETURN_IF_ERROR(GetThriftEnum(value, "kudu replica selection",
+            _TKuduReplicaSelection_VALUES_TO_NAMES, &enum_type));
+        query_options->__set_kudu_replica_selection(enum_type);
+        break;
+      }
+      case TImpalaQueryOptions::DELETE_STATS_IN_TRUNCATE: {
+        query_options->__set_delete_stats_in_truncate(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::MINMAX_FILTER_SORTED_COLUMNS: {
+        query_options->__set_minmax_filter_sorted_columns(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::MINMAX_FILTER_FAST_CODE_PATH: {
+        TMinmaxFilterFastCodePathMode::type enum_type;
+        RETURN_IF_ERROR(GetThriftEnum(value, "minmax filter fast code path type",
+            _TMinmaxFilterFastCodePathMode_VALUES_TO_NAMES, &enum_type));
+        query_options->__set_minmax_filter_fast_code_path(enum_type);
+        break;
+      }
+      case TImpalaQueryOptions::ENABLE_KUDU_TRANSACTION: {
+        query_options->__set_enable_kudu_transaction(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::MINMAX_FILTER_PARTITION_COLUMNS: {
+        query_options->__set_minmax_filter_partition_columns(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::ENABLE_ASYNC_DDL_EXECUTION: {
+        query_options->__set_enable_async_ddl_execution(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::PARQUET_LATE_MATERIALIZATION_THRESHOLD: {
+        StringParser::ParseResult result;
+        const int32_t threshold =
+            StringParser::StringToInt<int32_t>(value.c_str(), value.length(), &result);
+        if (value == nullptr || result != StringParser::PARSE_SUCCESS || threshold < -1 ||
+            threshold == 0) {
+          return Status(Substitute("Invalid parquet late materialization threshold: "
+              "'$0'. Use -1 to disable the feature, and values > 0 to specify the "
+              "minimum skip length.", value));
+        }
+        query_options->__set_parquet_late_materialization_threshold(threshold);
+        break;
+      }
+      case TImpalaQueryOptions::PARQUET_DICTIONARY_RUNTIME_FILTER_ENTRY_LIMIT: {
+        StringParser::ParseResult result;
+        const int32_t limit =
+            StringParser::StringToInt<int32_t>(value.c_str(), value.length(), &result);
+        if (value == nullptr || result != StringParser::PARSE_SUCCESS || limit < 0) {
+          return Status(Substitute("Invalid parquet parquet dictionary runtime filter "
+              "entry limit '$0'. Only integer value 0 and above is allowed.", value));
+        }
+        query_options->__set_parquet_dictionary_runtime_filter_entry_limit(limit);
+        break;
+      }
+      case TImpalaQueryOptions::ENABLE_ASYNC_LOAD_DATA_EXECUTION: {
+        query_options->__set_enable_async_load_data_execution(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::ABORT_JAVA_UDF_ON_EXCEPTION: {
+        query_options->__set_abort_java_udf_on_exception(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::ORC_ASYNC_READ: {
+        query_options->__set_orc_async_read(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::RUNTIME_IN_LIST_FILTER_ENTRY_LIMIT: {
+        StringParser::ParseResult result;
+        const int32_t limit =
+            StringParser::StringToInt<int32_t>(value.c_str(), value.length(), &result);
+        if (value == nullptr || result != StringParser::PARSE_SUCCESS || limit < 0) {
+          return Status(Substitute("Invalid runtime in-list filter entry limit '$0'. "
+              "Only integer value 0 and above is allowed.", value));
+        }
+        query_options->__set_runtime_in_list_filter_entry_limit(limit);
+      }
+      case TImpalaQueryOptions::LOCK_MAX_WAIT_TIME_S: {
+        StringParser::ParseResult result;
+        const int32_t lock_max_wait =
+            StringParser::StringToInt<int32_t>(value.c_str(), value.length(), &result);
+        if (value == nullptr || result != StringParser::PARSE_SUCCESS ||
+            lock_max_wait < 0) {
+          return Status(Substitute("Invalid value for LOCK_MAX_WAIT_TIME_S: '$0'. "
+              "Only integer value 0 and above is allowed.", value));
+        }
+        query_options->__set_lock_max_wait_time_s(lock_max_wait);
+        break;
+      }
+      case TImpalaQueryOptions::ENABLE_REPLAN: {
+        query_options->__set_enable_replan(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::TEST_REPLAN: {
+        query_options->__set_test_replan(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::ORC_SCHEMA_RESOLUTION: {
+        TSchemaResolutionStrategy::type enum_type;
+        RETURN_IF_ERROR(GetThriftEnum(value, "orc schema resolution",
+            _TSchemaResolutionStrategy_VALUES_TO_NAMES, &enum_type));
+        query_options->__set_orc_schema_resolution(enum_type);
+        break;
+      }
       default:
         if (IsRemovedQueryOption(key)) {
           LOG(WARNING) << "Ignoring attempt to set removed query option '" << key << "'";
