@@ -21,9 +21,13 @@
 #include <boost/scoped_ptr.hpp>
 
 #include "exprs/scalar-expr-evaluator.h"
+#include "exprs/slot-ref.h"
 #include "rpc/thrift-util.h"
+#include "runtime/descriptors.h"
 #include "runtime/raw-value.h"
 #include "runtime/row-batch.h"
+#include "runtime/tuple.h"
+#include "runtime/tuple-row.h"
 #include "runtime/types.h"
 #include "service/hs2-util.h"
 #include "util/bit-util.h"
@@ -186,10 +190,21 @@ Status AsciiQueryResultSet::AddRows(const vector<ScalarExprEvaluator*>& expr_eva
     for (int i = 0; i < num_col; ++i) {
       // ODBC-187 - ODBC can only take "\t" as the delimiter
       out_stream << (i > 0 ? "\t" : "");
-      DCHECK_EQ(1, metadata_.columns[i].columnType.types.size());
-      RawValue::PrintValue(expr_evals[i]->GetValue(it.Get()),
-          ColumnType::FromThrift(metadata_.columns[i].columnType), scales[i],
-          &out_stream);
+
+      if (metadata_.columns[i].columnType.types.size() == 1) {
+        RawValue::PrintValue(expr_evals[i]->GetValue(it.Get()),
+            ColumnType::FromThrift(metadata_.columns[i].columnType), scales[i],
+            &out_stream);
+      } else if (metadata_.columns[i].columnType.types.size() > 1) {
+        ColumnType col_type = ColumnType::FromThrift(metadata_.columns[i].columnType);
+        // TODO: Implement map (IMPALA-10918) type.
+        DCHECK(col_type.IsArrayType());
+        if (col_type.IsArrayType()) {
+          PrintArrayValue(expr_evals[i], it.Get(), scales[i], &out_stream);
+        }
+      } else {
+        DCHECK(false);
+      }
     }
     result_set_->push_back(out_stream.str());
     out_stream.str("");
@@ -197,6 +212,19 @@ Status AsciiQueryResultSet::AddRows(const vector<ScalarExprEvaluator*>& expr_eva
   return Status::OK();
 }
 
+void QueryResultSet::PrintArrayValue(ScalarExprEvaluator* expr_eval,
+    const TupleRow* row, int scale, stringstream *stream) {
+  const ScalarExpr& scalar_expr = expr_eval->root();
+  // Currently scalar_expr can be only a slot ref as no functions return arrays.
+  DCHECK(scalar_expr.IsSlotRef());
+  const TupleDescriptor* item_tuple_desc = scalar_expr.GetCollectionTupleDesc();
+  DCHECK(item_tuple_desc != nullptr);
+  const CollectionValue* array_val =
+      static_cast<const CollectionValue*>(expr_eval->GetValue(row));
+
+  RawValue::PrintArrayValue(array_val, item_tuple_desc, scale, stream);
+}
+
 Status AsciiQueryResultSet::AddOneRow(const TResultRow& row) {
   int num_col = row.colVals.size();
   DCHECK_EQ(num_col, metadata_.columns.size());
@@ -323,7 +351,13 @@ int HS2ColumnarResultSet::AddRows(
   for (int j = 0; j < metadata_.columns.size(); ++j) {
     ThriftTColumn* from = &o->result_set_->columns[j];
     ThriftTColumn* to = &result_set_->columns[j];
-    switch (metadata_.columns[j].columnType.types[0].scalar_type.type) {
+    const TColumnType& colType = metadata_.columns[j].columnType;
+    TPrimitiveType::type primitiveType = colType.types[0].scalar_type.type;
+    if (colType.types[0].type != TTypeNodeType::SCALAR) {
+      DCHECK(from->__isset.stringVal);
+      primitiveType = TPrimitiveType::STRING;
+    }
+    switch (primitiveType) {
       case TPrimitiveType::NULL_TYPE:
       case TPrimitiveType::BOOLEAN:
         StitchNulls(
@@ -402,45 +436,53 @@ int64_t HS2ColumnarResultSet::ByteSize(int start_idx, int num_rows) {
 
 void HS2ColumnarResultSet::InitColumns() {
   result_set_->__isset.columns = true;
-  for (const TColumn& col : metadata_.columns) {
-    DCHECK(col.columnType.types.size() == 1)
-        << "Structured columns unsupported in HS2 interface";
-    ThriftTColumn column;
-    switch (col.columnType.types[0].scalar_type.type) {
-      case TPrimitiveType::NULL_TYPE:
-      case TPrimitiveType::BOOLEAN:
-        column.__isset.boolVal = true;
-        break;
-      case TPrimitiveType::TINYINT:
-        column.__isset.byteVal = true;
-        break;
-      case TPrimitiveType::SMALLINT:
-        column.__isset.i16Val = true;
-        break;
-      case TPrimitiveType::INT:
-        column.__isset.i32Val = true;
-        break;
-      case TPrimitiveType::BIGINT:
-        column.__isset.i64Val = true;
-        break;
-      case TPrimitiveType::FLOAT:
-      case TPrimitiveType::DOUBLE:
-        column.__isset.doubleVal = true;
-        break;
-      case TPrimitiveType::TIMESTAMP:
-      case TPrimitiveType::DATE:
-      case TPrimitiveType::DECIMAL:
-      case TPrimitiveType::VARCHAR:
-      case TPrimitiveType::CHAR:
-      case TPrimitiveType::STRING:
-        column.__isset.stringVal = true;
-        break;
-      default:
-        DCHECK(false) << "Unhandled column type: "
-                      << TypeToString(
-                             ThriftToType(col.columnType.types[0].scalar_type.type));
+  for (const TColumn& col_input : metadata_.columns) {
+    const vector<TTypeNode>& type_nodes = col_input.columnType.types;
+    DCHECK(type_nodes.size() > 0);
+    ThriftTColumn col_output;
+    if (type_nodes[0].type == TTypeNodeType::STRUCT
+        || type_nodes[0].type == TTypeNodeType::ARRAY) {
+      // Return structs and arrays as string.
+      col_output.__isset.stringVal = true;
+    } else {
+      DCHECK(type_nodes.size() == 1);
+      DCHECK(type_nodes[0].__isset.scalar_type);
+      TPrimitiveType::type input_type = type_nodes[0].scalar_type.type;
+      switch (input_type) {
+        case TPrimitiveType::NULL_TYPE:
+        case TPrimitiveType::BOOLEAN:
+          col_output.__isset.boolVal = true;
+          break;
+        case TPrimitiveType::TINYINT:
+          col_output.__isset.byteVal = true;
+          break;
+        case TPrimitiveType::SMALLINT:
+          col_output.__isset.i16Val = true;
+          break;
+        case TPrimitiveType::INT:
+          col_output.__isset.i32Val = true;
+          break;
+        case TPrimitiveType::BIGINT:
+          col_output.__isset.i64Val = true;
+          break;
+        case TPrimitiveType::FLOAT:
+        case TPrimitiveType::DOUBLE:
+          col_output.__isset.doubleVal = true;
+          break;
+        case TPrimitiveType::TIMESTAMP:
+        case TPrimitiveType::DATE:
+        case TPrimitiveType::DECIMAL:
+        case TPrimitiveType::VARCHAR:
+        case TPrimitiveType::CHAR:
+        case TPrimitiveType::STRING:
+          col_output.__isset.stringVal = true;
+          break;
+        default:
+          DCHECK(false) << "Unhandled column type: "
+              << TypeToString(ThriftToType(input_type));
+      }
     }
-    result_set_->columns.push_back(column);
+    result_set_->columns.push_back(col_output);
   }
 }
 
