@@ -31,7 +31,6 @@
 DECLARE_bool(load_catalog_in_background);
 DECLARE_bool(load_auth_to_local_rules);
 DECLARE_bool(enable_stats_extrapolation);
-DECLARE_bool(enable_orc_scanner);
 DECLARE_bool(use_local_catalog);
 DECLARE_int32(local_catalog_cache_expiration_s);
 DECLARE_int32(local_catalog_cache_mb);
@@ -48,6 +47,7 @@ DECLARE_string(principal);
 DECLARE_string(local_library_dir);
 DECLARE_string(server_name);
 DECLARE_string(authorized_proxy_group_config);
+DECLARE_bool(enable_shell_based_groups_mapping_support);
 DECLARE_string(catalog_topic_mode);
 DECLARE_string(kudu_master_hosts);
 DECLARE_string(reserved_words_version);
@@ -91,6 +91,15 @@ DECLARE_int64(topic_update_tbl_max_wait_time_ms);
 DECLARE_int32(catalog_max_lock_skipped_topic_updates);
 DECLARE_string(scratch_dirs);
 DECLARE_int32(max_wait_time_for_sync_ddl_s);
+DECLARE_bool(start_hms_server);
+DECLARE_int32(hms_port);
+DECLARE_bool(fallback_to_hms_on_errors);
+DECLARE_bool(enable_catalogd_hms_cache);
+DECLARE_string(kudu_sasl_protocol_name);
+DECLARE_bool(invalidate_hms_cache_on_ddls);
+DECLARE_bool(hms_event_incremental_refresh_transactional_table);
+DECLARE_bool(auto_check_compaction);
+DECLARE_bool(enable_sync_to_latest_event_on_ddls);
 
 // HS2 SAML2.0 configuration
 // Defined here because TAG_FLAG caused issues in global-flags.cc
@@ -165,6 +174,18 @@ DEFINE_bool(allow_ordinals_in_having, false,
     "supported in Impala 3.x and earlier. We intend to disable it since 4.0. So it "
     "defaults to be false. See IMPALA-7844.");
 
+DEFINE_int32(warn_catalog_response_size_mb, 50,
+    "Threshold in MB to log a warning for large catalogd response size.");
+
+DEFINE_int32(warn_catalog_response_duration_s, 60,
+    "Threshold in seconds to log a warning for slow catalogd response.");
+
+DEFINE_string(startup_filesystem_check_directories, "/",
+    "Comma separated list of directories to list on startup to verify access to the "
+    "filesystem. The default is to list the root of the filesytem. This can be "
+    "specified to a subdirectory to avoid accesses to the root of the filesystem. "
+    "To disable the startup check, specify the empty string.");
+
 namespace impala {
 
 Status GetConfigFromCommand(const string& flag_cmd, string& result) {
@@ -178,7 +199,6 @@ Status GetConfigFromCommand(const string& flag_cmd, string& result) {
 
 Status PopulateThriftBackendGflags(TBackendGflags& cfg) {
   cfg.__set_load_catalog_in_background(FLAGS_load_catalog_in_background);
-  cfg.__set_enable_orc_scanner(FLAGS_enable_orc_scanner);
   cfg.__set_use_local_catalog(FLAGS_use_local_catalog);
   cfg.__set_local_catalog_cache_mb(FLAGS_local_catalog_cache_mb);
   cfg.__set_local_catalog_cache_expiration_s(
@@ -211,6 +231,8 @@ Status PopulateThriftBackendGflags(TBackendGflags& cfg) {
   cfg.__set_max_filter_error_rate(FLAGS_max_filter_error_rate);
   cfg.__set_min_buffer_size(FLAGS_min_buffer_size);
   cfg.__set_authorized_proxy_group_config(FLAGS_authorized_proxy_group_config);
+  cfg.__set_enable_shell_based_groups_mapping_support(
+      FLAGS_enable_shell_based_groups_mapping_support);
   cfg.__set_disable_catalog_data_ops_debug_only(
       FLAGS_disable_catalog_data_ops_debug_only);
   cfg.__set_catalog_topic_mode(FLAGS_catalog_topic_mode);
@@ -285,6 +307,21 @@ Status PopulateThriftBackendGflags(TBackendGflags& cfg) {
   cfg.__set_scratch_dirs(FLAGS_scratch_dirs);
   cfg.__set_max_wait_time_for_sync_ddl_s(FLAGS_max_wait_time_for_sync_ddl_s);
   cfg.__set_allow_ordinals_in_having(FLAGS_allow_ordinals_in_having);
+  cfg.__set_start_hms_server(FLAGS_start_hms_server);
+  cfg.__set_hms_port(FLAGS_hms_port);
+  cfg.__set_fallback_to_hms_on_errors(FLAGS_fallback_to_hms_on_errors);
+  cfg.__set_enable_catalogd_hms_cache(FLAGS_enable_catalogd_hms_cache);
+  cfg.__set_kudu_sasl_protocol_name(FLAGS_kudu_sasl_protocol_name);
+  cfg.__set_warn_catalog_response_size_mb(FLAGS_warn_catalog_response_size_mb);
+  cfg.__set_warn_catalog_response_duration_s(FLAGS_warn_catalog_response_duration_s);
+  cfg.__set_invalidate_hms_cache_on_ddls(FLAGS_invalidate_hms_cache_on_ddls);
+  cfg.__set_startup_filesystem_check_directories(
+      FLAGS_startup_filesystem_check_directories);
+  cfg.__set_hms_event_incremental_refresh_transactional_table(
+      FLAGS_hms_event_incremental_refresh_transactional_table);
+  cfg.__set_auto_check_compaction(FLAGS_auto_check_compaction);
+  cfg.__set_enable_sync_to_latest_event_on_ddls(
+      FLAGS_enable_sync_to_latest_event_on_ddls);
   return Status::OK();
 }
 
