@@ -36,6 +36,7 @@
 #include "runtime/bufferpool/buffer-pool.h"
 #include "util/cpu-info.h"
 #include "util/hash-util.h"
+#include "util/impala-bloom-filter-buffer-allocator.h"
 
 namespace kudu {
 namespace rpc {
@@ -63,46 +64,6 @@ struct TestData;
 
 namespace impala {
 
-// Buffer allocator to allocate and de-allocate memory for the BlockBloomFilter
-// from buffer pool.
-class ImpalaBloomFilterBufferAllocator : public kudu::BlockBloomFilterBufferAllocatorIf {
- public:
-  // Default constructor, which is defined to support the virtual function Clone().
-  // It uses kudu::DefaultBlockBloomFilterBufferAllocator to allocate/de-allocate
-  // memory. Since Clone function is only used for internal testing, so that
-  // memory allocation don't need to be tracked.
-  ImpalaBloomFilterBufferAllocator();
-
-  // Constructor with client handle of the buffer pool, which is created for
-  // runtime filters in runtime-filter-bank.
-  explicit ImpalaBloomFilterBufferAllocator(BufferPool::ClientHandle* client);
-
-  ~ImpalaBloomFilterBufferAllocator() override;
-
-  kudu::Status AllocateBuffer(size_t bytes, void** ptr) override;
-  void FreeBuffer(void* ptr) override;
-
-  // This virtual function is only defined for Kudu internal testing.
-  // Impala code should not hit this function.
-  std::shared_ptr<kudu::BlockBloomFilterBufferAllocatorIf> Clone() const override {
-    LOG(DFATAL) << "Unsupported code path.";
-    return std::make_shared<ImpalaBloomFilterBufferAllocator>();
-  }
-
-  bool IsAllocated() { return is_allocated_; }
-
- private:
-  void Close();
-
-  /// Bufferpool client and handle used for allocating and freeing directory memory.
-  /// Client is not owned by the buffer allocator.
-  BufferPool::ClientHandle* buffer_pool_client_;
-  BufferPool::BufferHandle buffer_handle_;
-  bool is_allocated_;
-
-  DISALLOW_COPY_AND_ASSIGN(ImpalaBloomFilterBufferAllocator);
-};
-
 /// A BloomFilter stores sets of items and offers a query operation indicating whether
 /// or not that item is in the set. The BloomFilter functionality is implemented in
 /// kudu::BlockBloomFilter class (see source at be/src/kudu/util/block_bloom_filter.h),
