@@ -18,6 +18,7 @@
 #include <algorithm>
 
 #include "gutil/strings/substitute.h"
+#include "util/bit-util.h"
 #include "util/string-util.h"
 
 #include "common/names.h"
@@ -91,4 +92,55 @@ const uint8_t* FindEndOfIdentifier(const uint8_t* start, const uint8_t* end) {
   return end;
 }
 
+int FindUtf8PosForward(const uint8_t* ptr, const int len, int index) {
+  DCHECK_GE(index, 0);
+  int pos = 0;
+  while (index > 0 && pos < len) {
+    // Counting malformed UTF8 characters.
+    while (!BitUtil::IsUtf8StartByte(ptr[pos]) && index > 0 && pos < len) {
+      ++pos;
+      --index;
+    }
+    if (index == 0 || pos == len) break;
+    pos += BitUtil::NumBytesInUTF8Encoding(ptr[pos]);
+    --index;
+  }
+  if (pos >= len) return len;
+  return pos;
+}
+
+int FindUtf8PosBackward(const uint8_t* ptr, const int len, int index) {
+  DCHECK_GE(index, 0);
+  int pos = len - 1;
+  int last_pos = len;
+  while (pos >= 0) {
+    // Point to the start byte of the last character.
+    while (pos >= 0 && !BitUtil::IsUtf8StartByte(ptr[pos])) --pos;
+    if (pos < 0) {
+      // Can't find any legal characters. Count each byte from last_pos as one character.
+      // Note that index is 0-based.
+      if (index < last_pos) return last_pos - index - 1;
+      return -1;
+    }
+    // Get bytes length of the located character.
+    int bytes_len = BitUtil::NumBytesInUTF8Encoding(ptr[pos]);
+    // If there are not enough bytes after the first byte, i.e. last_pos-pos < bytes_len,
+    // we consider the bytes belong to a malformed character, and count them as one
+    // character.
+    int malformed_bytes = max(last_pos - pos - bytes_len, 0);
+    if (index < malformed_bytes) {
+      // Count each malformed bytes as one character.
+      return last_pos - index - 1;
+    }
+    // We found a legal character and 'malformed_bytes' malformed characters.
+    // At this point, index >= malformed_bytes. So the lowest value of the updated index
+    // is -1, which means 'pos' points at what we want.
+    index -= malformed_bytes + 1;
+    if (index < 0) return pos;
+    last_pos = pos;
+    --pos;
+  }
+  DCHECK_EQ(pos, -1);
+  return -1;
+}
 }
