@@ -22,6 +22,7 @@
 #include <boost/functional/hash.hpp>
 
 #include "gen-cpp/Types_types.h" // for TUniqueId
+#include "gen-cpp/common.pb.h" // for UniqueIdPB
 
 namespace impala {
 
@@ -32,6 +33,13 @@ inline std::size_t hash_value(const TUniqueId& id) {
   return seed;
 }
 
+inline std::size_t hash_value(const UniqueIdPB& id) {
+  std::size_t seed = 0;
+  boost::hash_combine(seed, id.lo());
+  boost::hash_combine(seed, id.hi());
+  return seed;
+}
+
 } // namespace impala
 
 /// Hash function for std:: containers
@@ -44,4 +52,11 @@ struct hash<impala::TUniqueId> {
   }
 };
 
+template <>
+struct hash<impala::UniqueIdPB> {
+  std::size_t operator()(const impala::UniqueIdPB& id) const {
+    return impala::hash_value(id);
+  }
+};
+
 } // namespace std
