@@ -21,6 +21,7 @@
 #include <boost/filesystem.hpp>
 #include <boost/lexical_cast.hpp>
 #include <gutil/strings/substitute.h>
+#include <openssl/crypto.h>
 #include <openssl/ssl.h>
 
 #include "common/init.h"
@@ -44,7 +45,8 @@ DECLARE_string(webserver_private_key_password_cmd);
 DECLARE_string(webserver_x_frame_options);
 DECLARE_string(ssl_cipher_list);
 DECLARE_string(ssl_minimum_version);
-DECLARE_bool(ldap_passwords_in_clear_ok);
+DECLARE_bool(webserver_ldap_passwords_in_clear_ok);
+DECLARE_bool(cookie_require_secure);
 
 #include "common/names.h"
 
@@ -277,7 +279,7 @@ TEST(Webserver, SslCipherSuite) {
       Substitute("$0/be/src/testutil/server-key-password.pem", getenv("IMPALA_HOME")));
   auto cmd = ScopedFlagSetter<string>::Make(
       &FLAGS_webserver_private_key_password_cmd, "echo password");
-
+#ifndef __aarch64__
   {
     auto ciphers = ScopedFlagSetter<string>::Make(
         &FLAGS_ssl_cipher_list, "not_a_cipher");
@@ -285,7 +287,7 @@ TEST(Webserver, SslCipherSuite) {
     Webserver webserver("", FLAGS_webserver_port, &metrics);
     ASSERT_FALSE(webserver.Start().ok());
   }
-
+#endif
   {
     auto ciphers = ScopedFlagSetter<string>::Make(
         &FLAGS_ssl_cipher_list, "AES128-SHA");
@@ -375,7 +377,8 @@ TEST(Webserver, TestWithSpnego) {
 
   gflags::FlagSaver saver;
   FLAGS_webserver_require_spnego = true;
-  FLAGS_ldap_passwords_in_clear_ok = true;
+  FLAGS_webserver_ldap_passwords_in_clear_ok = true;
+  FLAGS_cookie_require_secure = false;
 
   MetricGroup metrics("webserver-test");
   Webserver webserver("", FLAGS_webserver_port, &metrics);
@@ -442,15 +445,20 @@ TEST(Webserver, StartWithPasswordFileTest) {
 
   MetricGroup metrics("webserver-test");
   Webserver webserver("", FLAGS_webserver_port, &metrics);
-  ASSERT_OK(webserver.Start());
+  if (FIPS_mode()) {
+    ASSERT_FALSE(webserver.Start().ok());
+  } else {
+    ASSERT_OK(webserver.Start());
 
-  // Don't expect HTTP requests to work without a password
-  stringstream contents;
-  ASSERT_ERROR_MSG(HttpGet("localhost", FLAGS_webserver_port, "/", &contents),
-      "Unexpected status code: 401");
+    // Don't expect HTTP requests to work without a password
+    stringstream contents;
+    ASSERT_ERROR_MSG(HttpGet("localhost", FLAGS_webserver_port, "/", &contents),
+        "Unexpected status code: 401");
+  }
 }
 
 TEST(Webserver, StartWithMissingPasswordFileTest) {
+  if (FIPS_mode()) return;
   stringstream password_file;
   password_file << getenv("IMPALA_HOME") << "/be/src/testutil/doesntexist";
   auto password =
@@ -533,7 +541,7 @@ TEST(Webserver, Options) {
   ASSERT_OK(webserver.Start());
   stringstream contents;
   ASSERT_OK(HttpGet("localhost", FLAGS_webserver_port, "/", &contents, 200, "OPTIONS"));
-  ASSERT_FALSE(contents.str().find("Allow: GET, POST, HEAD, OPTIONS, PROPFIND, MKCOL")
+  ASSERT_FALSE(contents.str().find("Allow: GET, POST, HEAD, OPTIONS")
       == string::npos);
 }
 
