@@ -41,8 +41,8 @@
 #include "kudu/util/env.h"
 #include "kudu/util/logging.h"
 #include "kudu/util/net/sockaddr.h"
-#include "rpc/authentication.h"
 #include "rpc/authentication-util.h"
+#include "rpc/authentication.h"
 #include "rpc/thrift-util.h"
 #include "runtime/exec-env.h"
 #include "service/impala-server.h"
@@ -52,6 +52,7 @@
 #include "util/cpu-info.h"
 #include "util/debug-util.h"
 #include "util/disk-info.h"
+#include "util/jwt-util.h"
 #include "util/mem-info.h"
 #include "util/metrics.h"
 #include "util/os-info.h"
@@ -142,6 +143,10 @@ DEFINE_bool(webserver_ldap_passwords_in_clear_ok, false,
     "(Advanced) If true, allows the webserver to start with LDAP authentication even if "
     "SSL is not enabled, a potentially insecure configuration.");
 
+DEFINE_bool(disable_content_security_policy_header, false,
+    "If true then the webserver will not add the Content-Security-Policy "
+    "HTTP header to HTTP responses");
+
 DECLARE_bool(enable_ldap_auth);
 DECLARE_string(hostname);
 DECLARE_bool(is_coordinator);
@@ -150,6 +155,10 @@ DECLARE_string(ssl_minimum_version);
 DECLARE_string(ssl_cipher_list);
 DECLARE_string(trusted_domain);
 DECLARE_bool(trusted_domain_use_xff_header);
+DECLARE_bool(jwt_token_auth);
+DECLARE_bool(jwt_validate_signature);
+DECLARE_string(jwt_custom_claim_username);
+DECLARE_string(trusted_auth_header);
 
 static const char* DOC_FOLDER = "/www/";
 static const int DOC_FOLDER_LEN = strlen(DOC_FOLDER);
@@ -164,6 +173,11 @@ static const char* ERROR_KEY = "__error_msg__";
 
 static const char* CRLF = "\r\n";
 
+// The value to be returned in the Content-Security-Policy header.
+static const char* CSP_HEADER = "default-src 'self'; style-src 'self' 'unsafe-inline'; "
+                                "script-src 'self' 'unsafe-inline'; "
+                                "img-src 'self' data:;";
+
 // Returns $IMPALA_HOME if set, otherwise /tmp/impala_www
 const char* GetDefaultDocumentRoot() {
   stringstream ss;
@@ -191,6 +205,8 @@ string HttpStatusCodeToString(HttpStatusCode code) {
   switch (code) {
     case HttpStatusCode::Ok:
       return "200 OK";
+    case HttpStatusCode::TemporaryRedirect:
+      return "307 Temporary Redirect";
     case HttpStatusCode::BadRequest:
       return "400 Bad Request";
     case HttpStatusCode::AuthenticationRequired:
@@ -221,6 +237,9 @@ void SendResponse(struct sq_connection* connection, const string& response_code_
     oss << h << CRLF;
   }
   oss << "X-Frame-Options: " << FLAGS_webserver_x_frame_options << CRLF;
+  if (!FLAGS_disable_content_security_policy_header) {
+    oss << "Content-Security-Policy: " << CSP_HEADER << CRLF;
+  }
   oss << "Content-Type: " << content_type << CRLF;
   oss << "Content-Length: " << content.size() << CRLF;
   oss << CRLF;
@@ -234,6 +253,7 @@ void SendResponse(struct sq_connection* connection, const string& response_code_
 // Return the address of the remote user from the squeasel request info.
 kudu::Sockaddr GetRemoteAddress(const struct sq_request_info* req) {
   struct sockaddr_in addr;
+  memset(&addr, 0, sizeof(addr));
   addr.sin_family = AF_INET;
   addr.sin_port = NetworkByteOrder::FromHost16(req->remote_port);
   addr.sin_addr.s_addr = NetworkByteOrder::FromHost32(req->remote_ip);
@@ -282,7 +302,9 @@ Webserver::Webserver(const string& interface, const int port, MetricGroup* metri
     error_handler_(UrlHandler(
         bind<void>(&Webserver::ErrorHandler, this, _1, _2), "error.tmpl", false)),
     use_cookies_(FLAGS_max_cookie_lifetime_s > 0),
-    check_trusted_domain_(!FLAGS_trusted_domain.empty()) {
+    check_trusted_domain_(!FLAGS_trusted_domain.empty()),
+    check_trusted_auth_header_(!FLAGS_trusted_auth_header.empty()),
+    use_jwt_(FLAGS_jwt_token_auth) {
   http_address_ = MakeNetworkAddress(interface.empty() ? "0.0.0.0" : interface, port);
   Init();
 
@@ -309,6 +331,17 @@ Webserver::Webserver(const string& interface, const int port, MetricGroup* metri
     total_trusted_domain_check_success_ =
         metrics->AddCounter("impala.webserver.total-trusted-domain-check-success", 0);
   }
+  if (check_trusted_auth_header_
+      && (auth_mode_ == AuthMode::SPNEGO || auth_mode_ == AuthMode::LDAP)) {
+    total_trusted_auth_header_check_success_ = metrics->AddCounter(
+        "impala.webserver.total-trusted-auth-header-check-success", 0);
+  }
+  if (use_jwt_) {
+    total_jwt_token_auth_success_ =
+        metrics->AddCounter("impala.webserver.total-jwt-token-auth-success", 0);
+    total_jwt_token_auth_failure_ =
+        metrics->AddCounter("impala.webserver.total-jwt-token-auth-failure", 0);
+  }
 }
 
 Webserver::~Webserver() {
@@ -600,8 +633,36 @@ sq_callback_result_t Webserver::BeginRequestCallback(struct sq_connection* conne
   }
 
   vector<string> response_headers;
-  bool authenticated = auth_mode_ != AuthMode::SPNEGO && auth_mode_ != AuthMode::LDAP;
-  // Try authenticating with a cookie first, if enabled.
+  bool authenticated = false;
+  // Try authenticating with JWT token first, if enabled.
+  if (use_jwt_) {
+    const char* auth_value = nullptr;
+    const char* value = sq_get_header(connection, "Authorization");
+    if (value != nullptr) auth_value = StripLeadingWhiteSpace(value);
+    // Check Authorization header with the Bearer authentication scheme as:
+    // Authorization: Bearer <token>
+    // A well-formed JWT consists of three concatenated Base64url-encoded strings,
+    // separated by dots (.).
+    if (auth_value != nullptr && strncasecmp(auth_value, "Bearer ", 7) == 0
+        && strchr(auth_value, '.') != nullptr) {
+      string jwt_token = string(auth_value + 7);
+      StripWhiteSpace(&jwt_token);
+      if (!jwt_token.empty()) {
+        if (JWTTokenAuth(jwt_token, connection, request_info)) {
+          total_jwt_token_auth_success_->Increment(1);
+          authenticated = true;
+          // TODO: cookies are not added, but are not needed right now
+        } else {
+          LOG(INFO) << "Invalid JWT token provided: " << jwt_token;
+          total_jwt_token_auth_failure_->Increment(1);
+        }
+      }
+    }
+  }
+  if (!authenticated) {
+    authenticated = auth_mode_ != AuthMode::SPNEGO && auth_mode_ != AuthMode::LDAP;
+  }
+  // Try authenticating with a cookie, if enabled.
   if (!authenticated && use_cookies_) {
     const char* cookie_header = sq_get_header(connection, "Cookie");
     string username;
@@ -640,6 +701,21 @@ sq_callback_result_t Webserver::BeginRequestCallback(struct sq_connection* conne
     }
   }
 
+  if (!authenticated && check_trusted_auth_header_) {
+    const char* auth_header =
+        sq_get_header(connection, FLAGS_trusted_auth_header.c_str());
+    if (auth_header != nullptr) {
+      string err_msg;
+      if (GetUsernameFromAuthHeader(connection, request_info, err_msg)) {
+        total_trusted_auth_header_check_success_->Increment(1);
+        authenticated = true;
+        AddCookie(request_info, &response_headers);
+      } else {
+        LOG(ERROR) << "Found trusted auth header but " << err_msg;
+      }
+    }
+  }
+
   if (!authenticated) {
     if (auth_mode_ == AuthMode::SPNEGO) {
       sq_callback_result_t spnego_result =
@@ -691,6 +767,9 @@ sq_callback_result_t Webserver::BeginRequestCallback(struct sq_connection* conne
     req.query_string = request_info->query_string;
     BuildArgumentMap(request_info->query_string, &req.parsed_args);
   }
+  if (request_info->remote_user != nullptr) {
+    req.source_user = request_info->remote_user;
+  }
 
   HttpStatusCode response = HttpStatusCode::Ok;
   ContentType content_type = HTML;
@@ -753,7 +832,10 @@ sq_callback_result_t Webserver::BeginRequestCallback(struct sq_connection* conne
 
   // The output of this page is accumulated into this stringstream.
   stringstream output;
-  if (!url_handler->use_templates()) {
+  if (strncmp("HEAD", request_info->request_method, 4) == 0) {
+    // For a HEAD call do not generate the response body.
+    VLOG(4) << "Not generating output for HEAD call on " << request_info->uri;
+  } else if (!url_handler->use_templates()) {
     content_type = PLAIN;
     url_handler->raw_callback()(req, &output, &response);
   } else {
@@ -807,25 +889,67 @@ sq_callback_result_t Webserver::HandleSpnego(struct sq_connection* connection,
   return SQ_CONTINUE_HANDLING;
 }
 
-bool Webserver::TrustedDomainCheck(const string& origin, struct sq_connection* connection,
-    struct sq_request_info* request_info) {
-  if (!IsTrustedDomain(origin, FLAGS_trusted_domain)) return false;
+bool Webserver::GetUsernameFromAuthHeader(struct sq_connection* connection,
+    struct sq_request_info* request_info, string& err_msg) {
   const char* authz_header = sq_get_header(connection, "Authorization");
   if (!authz_header) {
-    LOG(ERROR) << "Passed TrustedDomainCheck but no Authorization header provided.";
+    err_msg = "no Authorization header provided.";
     return false;
   }
 
   string base64;
   if (!TryStripPrefixString(authz_header, "Basic ", &base64)) {
-    LOG(ERROR) << "Passed TrustedDomainCheck but No Basic authentication provided.";
+    err_msg = "no Basic authentication provided.";
     return false;
   }
   string username, password;
   Status status = BasicAuthExtractCredentials(base64, username, password);
   if (!status.ok()) {
-    LOG(ERROR) << "Error parsing basic authentication token from: "
-               << GetRemoteAddress(request_info).ToString() << " Error: " << status;
+    err_msg = Substitute("error parsing basic authentication token from: $0, Error: $1",
+        GetRemoteAddress(request_info).ToString(), status.GetDetail());
+    return false;
+  }
+  request_info->remote_user = strdup(username.c_str());
+  return true;
+}
+
+bool Webserver::TrustedDomainCheck(const string& origin, struct sq_connection* connection,
+    struct sq_request_info* request_info) {
+  if (!IsTrustedDomain(origin, FLAGS_trusted_domain)) return false;
+
+  string err_msg;
+  if (!GetUsernameFromAuthHeader(connection, request_info, err_msg)) {
+    LOG(ERROR) << "Passed TrustedDomainCheck but " << err_msg;
+    return false;
+  }
+  return true;
+}
+
+bool Webserver::JWTTokenAuth(const std::string& jwt_token,
+    struct sq_connection* connection, struct sq_request_info* request_info) {
+  JWTHelper::UniqueJWTDecodedToken decoded_token;
+  Status status = JWTHelper::Decode(jwt_token, decoded_token);
+  if (!status.ok()) {
+    LOG(ERROR) << "Error decoding JWT token in Authorization header, "
+               << "Error: " << status;
+    return false;
+  }
+  if (FLAGS_jwt_validate_signature) {
+    status = JWTHelper::GetInstance()->Verify(decoded_token.get());
+    if (!status.ok()) {
+      LOG(ERROR) << "Error verifying JWT token in Authorization header, "
+                 << "Error: " << status;
+      return false;
+    }
+  }
+
+  DCHECK(!FLAGS_jwt_custom_claim_username.empty());
+  string username;
+  status = JWTHelper::GetCustomClaimUsername(
+      decoded_token.get(), FLAGS_jwt_custom_claim_username, username);
+  if (!status.ok()) {
+    LOG(ERROR) << "Cannot retrieve username from JWT token in Authorization header, "
+               << "Error: " << status;
     return false;
   }
   request_info->remote_user = strdup(username.c_str());
