@@ -145,6 +145,12 @@ mkdir -p hive-site-without-hms
 rm -f hive-site-without-hms/hive-site.xml
 ln -s "${CONFIG_DIR}/hive-site_without_hms.xml" hive-site-without-hms/hive-site.xml
 
+export HIVE_VARIANT=events_cleanup
+$IMPALA_HOME/bin/generate_xml_config.py hive-site.xml.py hive-site_events_cleanup.xml
+mkdir -p hive-site-events-cleanup
+rm -f hive-site-events-cleanup/hive-site.xml
+ln -s "${CONFIG_DIR}/hive-site_events_cleanup.xml" hive-site-events-cleanup/hive-site.xml
+
 export HIVE_VARIANT=ranger_auth
 HIVE_RANGER_CONF_DIR=hive-site-ranger-auth
 $IMPALA_HOME/bin/generate_xml_config.py hive-site.xml.py hive-site_ranger_auth.xml
