@@ -63,19 +63,28 @@ fi
 # variable from a previously-sourced impala-config.sh overrides the new value.        #
 #######################################################################################
 
+# The current Impala version that will be embedded in the Impala binary. This is
+# also used to find the Impala frontend jar files, so the version must match
+# the version in our Maven pom.xml files. This is validated via
+# bin/validate-java-pom-version.sh during the build.
+# WARNING: If changing this value, also run these commands:
+# cd ${IMPALA_HOME}/java
+# mvn versions:set -DnewVersion=YOUR_NEW_VERSION
+export IMPALA_VERSION=4.1.2-RELEASE
+
 # The unique build id of the toolchain to use if bootstrapping. This is generated by the
 # native-toolchain build when publishing its build artifacts. This should be changed when
 # moving to a different build of the toolchain, e.g. when a version is bumped or a
 # compile option is changed. The build id can be found in the output of the toolchain
 # build jobs, it is constructed from the build number and toolchain git hash prefix.
-export IMPALA_TOOLCHAIN_BUILD_ID=10-425e7fd862
+export IMPALA_TOOLCHAIN_BUILD_ID=144-57806de424
 # Versions of toolchain dependencies.
 # -----------------------------------
 export IMPALA_AVRO_VERSION=1.7.4-p5
 unset IMPALA_AVRO_URL
 export IMPALA_BINUTILS_VERSION=2.28
 unset IMPALA_BINUTILS_URL
-export IMPALA_BOOST_VERSION=1.61.0-p2
+export IMPALA_BOOST_VERSION=1.74.0-p1
 unset IMPALA_BOOST_URL
 export IMPALA_BREAKPAD_VERSION=97a98836768f8f0154f8f86e5e14c2bb7e74132e-p2
 unset IMPALA_BREAKPAD_URL
@@ -83,10 +92,12 @@ export IMPALA_BZIP2_VERSION=1.0.8-p2
 unset IMPALA_BZIP2_URL
 export IMPALA_CCTZ_VERSION=2.2
 unset IMPALA_CCTZ_URL
-export IMPALA_CMAKE_VERSION=3.14.3
+export IMPALA_CMAKE_VERSION=3.22.2
 unset IMPALA_CMAKE_URL
 export IMPALA_CRCUTIL_VERSION=440ba7babeff77ffad992df3a10c767f184e946e-p2
 unset IMPALA_CRCUTIL_URL
+export IMPALA_CURL_VERSION=7.78.0
+unset IMPALA_CURL_URL
 export IMPALA_CYRUS_SASL_VERSION=2.1.23
 unset IMPALA_CYRUS_SASL_URL
 export IMPALA_FLATBUFFERS_VERSION=1.6.0
@@ -97,12 +108,14 @@ export IMPALA_GDB_VERSION=7.9.1-p1
 unset IMPALA_GDB_URL
 export IMPALA_GFLAGS_VERSION=2.2.0-p2
 unset IMPALA_GFLAGS_URL
-export IMPALA_GLOG_VERSION=0.3.4-p3
+export IMPALA_GLOG_VERSION=0.3.5-p3
 unset IMPALA_GLOG_URL
-export IMPALA_GPERFTOOLS_VERSION=2.5-p2
+export IMPALA_GPERFTOOLS_VERSION=2.5-p4
 unset IMPALA_GPERFTOOLS_URL
 export IMPALA_GTEST_VERSION=1.6.0
 unset IMPALA_GTEST_URL
+export IMPALA_JWT_CPP_VERSION=0.5.0
+unset IMPALA_JWT_CPP_URL
 export IMPALA_LIBEV_VERSION=4.20
 unset IMPALA_LIBEV_URL
 export IMPALA_LIBUNWIND_VERSION=1.3-rc1-p3
@@ -132,11 +145,13 @@ export IMPALA_OPENLDAP_VERSION=2.4.47
 unset IMPALA_OPENLDAP_URL
 export IMPALA_OPENSSL_VERSION=1.0.2l
 unset IMPALA_OPENSSL_URL
-export IMPALA_ORC_VERSION=1.6.2-p7
+export IMPALA_ORC_VERSION=1.7.0-p14
 unset IMPALA_ORC_URL
-export IMPALA_PROTOBUF_VERSION=3.5.1
+export IMPALA_PROTOBUF_VERSION=3.14.0
 unset IMPALA_PROTOBUF_URL
-export IMPALA_POSTGRES_JDBC_DRIVER_VERSION=42.2.14
+export IMPALA_PROTOBUF_CLANG_VERSION=3.14.0-clangcompat-p2
+unset IMPALA_PROTOBUF_CLANG_URL
+export IMPALA_POSTGRES_JDBC_DRIVER_VERSION=42.3.3
 unset IMPALA_POSTGRES_JDBC_DRIVER_URL
 export IMPALA_PYTHON_VERSION=2.7.16
 unset IMPALA_PYTHON_URL
@@ -153,10 +168,12 @@ export IMPALA_TPC_DS_VERSION=2.1.0
 unset IMPALA_TPC_DS_URL
 export IMPALA_TPC_H_VERSION=2.17.0
 unset IMPALA_TPC_H_URL
-export IMPALA_THRIFT_VERSION=0.11.0-p4
+export IMPALA_THRIFT_VERSION=0.11.0-p5
 unset IMPALA_THRIFT_URL
 export IMPALA_ZLIB_VERSION=1.2.11
 unset IMPALA_ZLIB_URL
+export IMPALA_CALLONCEHACK_VERSION=1.0.0
+unset IMPALA_CALLONCEHACK_URL
 
 if [[ $OSTYPE == "darwin"* ]]; then
   IMPALA_CYRUS_SASL_VERSION=2.1.26
@@ -170,30 +187,33 @@ fi
 : ${IMPALA_TOOLCHAIN_HOST:=native-toolchain.s3.amazonaws.com}
 export IMPALA_TOOLCHAIN_HOST
 
-#TODO: On next CDP_BUILD_NUMBER bump check if the following issues are resolved
-# and adjust existing tests that mentions them:
-# * HIVE-23995
-# * HIVE-24175
-export CDP_BUILD_NUMBER=11920537
+export CDP_BUILD_NUMBER=23144489
 export CDP_MAVEN_REPOSITORY=\
 "https://${IMPALA_TOOLCHAIN_HOST}/build/cdp_components/${CDP_BUILD_NUMBER}/maven"
-export CDP_AVRO_JAVA_VERSION=1.8.2.7.2.9.0-146
-export CDP_HADOOP_VERSION=3.1.1.7.2.9.0-146
-export CDP_HBASE_VERSION=2.2.6.7.2.9.0-146
-export CDP_HIVE_VERSION=3.1.3000.7.2.9.0-146
-export CDP_ICEBERG_VERSION=0.9.1.7.2.9.0-146
-export CDP_KNOX_VERSION=1.3.0.7.2.9.0-146
-export CDP_OZONE_VERSION=1.0.0.7.2.9.0-146
-export CDP_PARQUET_VERSION=1.10.99.7.2.9.0-146
-export CDP_RANGER_VERSION=2.1.0.7.2.9.0-146
-export CDP_TEZ_VERSION=0.9.1.7.2.9.0-146
-export CDP_GCS_VERSION=2.1.2.7.2.9.0-146
+export CDP_AVRO_JAVA_VERSION=1.8.2.7.2.15.0-88
+export CDP_HADOOP_VERSION=3.1.1.7.2.15.0-88
+export CDP_HBASE_VERSION=2.4.6.7.2.15.0-88
+export CDP_HIVE_VERSION=3.1.3000.7.2.15.0-88
+export CDP_ICEBERG_VERSION=0.9.1.7.2.15.0-88
+export CDP_KNOX_VERSION=1.3.0.7.2.15.0-88
+export CDP_OZONE_VERSION=1.1.0.7.2.15.0-88
+export CDP_PARQUET_VERSION=1.10.99.7.2.15.0-88
+export CDP_RANGER_VERSION=2.1.0.7.2.15.0-88
+export CDP_TEZ_VERSION=0.9.1.7.2.15.0-88
+export CDP_GCS_VERSION=2.1.2.7.2.15.0-88
+
+# Ref: https://infra.apache.org/release-download-pages.html#closer
+: ${APACHE_MIRROR:="https://www.apache.org/dyn/closer.cgi"}
+export APACHE_MIRROR
+export APACHE_HIVE_VERSION=3.1.2
+export APACHE_HIVE_STORAGE_API_VERSION=2.7.0
 
 export ARCH_NAME=$(uname -p)
 
 export IMPALA_HUDI_VERSION=0.5.0-incubating
 export IMPALA_KITE_VERSION=1.1.0
 export IMPALA_ORC_JAVA_VERSION=1.6.2
+export IMPALA_COS_VERSION=3.1.0-5.9.3
 
 # When IMPALA_(CDP_COMPONENT)_URL are overridden, they may contain '$(platform_label)'
 # which will be substituted for the CDP platform label in bootstrap_toolchain.py
@@ -230,6 +250,9 @@ export CDP_ICEBERG_URL=${CDP_ICEBERG_URL-}
 export CDP_RANGER_URL=${CDP_RANGER_URL-}
 export CDP_TEZ_URL=${CDP_TEZ_URL-}
 
+export APACHE_HIVE_URL=${APACHE_HIVE_URL-}
+export APACHE_HIVE_SOURCE_URL=${APACHE_HIVE_SOURCE_URL-}
+
 export CDP_COMPONENTS_HOME="$IMPALA_TOOLCHAIN/cdp_components-$CDP_BUILD_NUMBER"
 export CDH_MAJOR_VERSION=7
 export IMPALA_AVRO_JAVA_VERSION=${CDP_AVRO_JAVA_VERSION}
@@ -238,9 +261,6 @@ export IMPALA_HADOOP_URL=${CDP_HADOOP_URL-}
 export HADOOP_HOME="$CDP_COMPONENTS_HOME/hadoop-${IMPALA_HADOOP_VERSION}/"
 export IMPALA_HBASE_VERSION=${CDP_HBASE_VERSION}
 export IMPALA_HBASE_URL=${CDP_HBASE_URL-}
-export IMPALA_HIVE_VERSION=${HIVE_VERSION_OVERRIDE:-"$CDP_HIVE_VERSION"}
-export IMPALA_HIVE_URL=${CDP_HIVE_URL-}
-export IMPALA_HIVE_SOURCE_URL=${CDP_HIVE_SOURCE_URL-}
 export IMPALA_ICEBERG_VERSION=${CDP_ICEBERG_VERSION}
 export IMPALA_ICEBERG_URL=${CDP_ICEBERG_URL-}
 export IMPALA_KNOX_VERSION=${CDP_KNOX_VERSION}
@@ -250,9 +270,29 @@ export IMPALA_RANGER_VERSION=${CDP_RANGER_VERSION}
 export IMPALA_RANGER_URL=${CDP_RANGER_URL-}
 export IMPALA_TEZ_VERSION=${CDP_TEZ_VERSION}
 export IMPALA_TEZ_URL=${CDP_TEZ_URL-}
-export IMPALA_HIVE_STORAGE_API_VERSION=${HIVE_STORAGE_API_VERSION_OVERRIDE:-"2.3.0.$IMPALA_HIVE_VERSION"}
 export IMPALA_GCS_VERSION=${CDP_GCS_VERSION}
 
+export APACHE_COMPONENTS_HOME="$IMPALA_TOOLCHAIN/apache_components"
+export USE_APACHE_HIVE=${USE_APACHE_HIVE-false}
+if $USE_APACHE_HIVE; then
+  # When USE_APACHE_HIVE is set we use the apache hive version to build as well as deploy
+  # in the minicluster
+  export IMPALA_HIVE_DIST_TYPE="apache-hive"
+  export IMPALA_HIVE_VERSION=${APACHE_HIVE_VERSION}
+  export IMPALA_HIVE_URL=${APACHE_HIVE_URL-}
+  export IMPALA_HIVE_SOURCE_URL=${APACHE_HIVE_SOURCE_URL-}
+  export IMPALA_HIVE_STORAGE_API_VERSION=${APACHE_HIVE_STORAGE_API_VERSION}
+else
+  # CDP hive version is used to build and deploy in minicluster when USE_APACHE_HIVE is
+  # false
+  export IMPALA_HIVE_DIST_TYPE="hive"
+  export IMPALA_HIVE_VERSION=${HIVE_VERSION_OVERRIDE:-"$CDP_HIVE_VERSION"}
+  export IMPALA_HIVE_URL=${CDP_HIVE_URL-}
+  export IMPALA_HIVE_SOURCE_URL=${CDP_HIVE_SOURCE_URL-}
+  export IMPALA_HIVE_STORAGE_API_VERSION=${HIVE_STORAGE_API_VERSION_OVERRIDE:-\
+"2.3.0.$IMPALA_HIVE_VERSION"}
+fi
+
 # Extract the first component of the hive version.
 # Allow overriding of Hive source location in case we want to build Impala without
 # a complete Hive build. This is used by various tests and scripts to enable and
@@ -353,6 +393,10 @@ export GOOGLE_CLOUD_PROJECT_ID="${GOOGLE_CLOUD_PROJECT_ID-}"
 export GOOGLE_CLOUD_SERVICE_ACCOUNT="${GOOGLE_CLOUD_SERVICE_ACCOUNT-}"
 export GOOGLE_APPLICATION_CREDENTIALS="${GOOGLE_APPLICATION_CREDENTIALS-}"
 export GCS_BUCKET="${GCS_BUCKET-}"
+export COS_SECRET_ID="${COS_SECRET_ID-}"
+export COS_SECRET_KEY="${COS_SECRET_KEY-}"
+export COS_REGION="${COS_REGION-}"
+export COS_BUCKET="${COS_BUCKET-}"
 export HDFS_REPLICATION="${HDFS_REPLICATION-3}"
 export ISILON_NAMENODE="${ISILON_NAMENODE-}"
 # Internal and external interfaces that test cluster services will listen on. The
@@ -367,16 +411,26 @@ export LOCAL_FS="file:${WAREHOUSE_LOCATION_PREFIX}"
 export IMPALA_CLUSTER_NODES_DIR="${IMPALA_CLUSTER_NODES_DIR-$IMPALA_HOME/testdata/cluster/cdh$CDH_MAJOR_VERSION}"
 
 ESCAPED_IMPALA_HOME=$(sed "s/[^0-9a-zA-Z]/_/g" <<< "$IMPALA_HOME")
-export HIVE_HOME=${HIVE_HOME_OVERRIDE:-"$CDP_COMPONENTS_HOME/apache-hive-${IMPALA_HIVE_VERSION}-bin"}
-export HIVE_SRC_DIR=${HIVE_SRC_DIR_OVERRIDE:-"${CDP_COMPONENTS_HOME}/hive-${IMPALA_HIVE_VERSION}"}
+if $USE_APACHE_HIVE; then
+  export HIVE_HOME="$APACHE_COMPONENTS_HOME/apache-hive-${IMPALA_HIVE_VERSION}-bin"
+  export HIVE_SRC_DIR="$APACHE_COMPONENTS_HOME/apache-hive-${IMPALA_HIVE_VERSION}-src"
+  # if apache hive is being used change the metastore db name, so we don't have to
+  # format the metastore db everytime we switch between hive versions
+  export METASTORE_DB=${METASTORE_DB-"$(cut -c-59 <<< HMS$ESCAPED_IMPALA_HOME)_apache"}
+else
+  export HIVE_HOME=${HIVE_HOME_OVERRIDE:-\
+"$CDP_COMPONENTS_HOME/apache-hive-${IMPALA_HIVE_VERSION}-bin"}
+  export HIVE_SRC_DIR=${HIVE_SRC_DIR_OVERRIDE:-\
+"${CDP_COMPONENTS_HOME}/hive-${IMPALA_HIVE_VERSION}"}
+  # Previously, there were multiple configurations and the "_cdp" included below
+  # allowed the two to be distinct. We keep this "_cdp" for historical reasons.
+  export METASTORE_DB=${METASTORE_DB-"$(cut -c-59 <<< HMS$ESCAPED_IMPALA_HOME)_cdp"}
+fi
 # Set the path to the hive_metastore.thrift which is used to build thrift code
 export HIVE_METASTORE_THRIFT_DIR=${HIVE_METASTORE_THRIFT_DIR_OVERRIDE:-\
 "$HIVE_SRC_DIR/standalone-metastore/src/main/thrift"}
 export TEZ_HOME="$CDP_COMPONENTS_HOME/tez-${IMPALA_TEZ_VERSION}-minimal"
 export HBASE_HOME="$CDP_COMPONENTS_HOME/hbase-${IMPALA_HBASE_VERSION}/"
-# Previously, there were multiple configurations and the "_cdp" included below
-# allowed the two to be distinct. We keep this "_cdp" for historical reasons.
-export METASTORE_DB=${METASTORE_DB-"$(cut -c-59 <<< HMS$ESCAPED_IMPALA_HOME)_cdp"}
 # Set the Hive binaries in the path
 export PATH="$HIVE_HOME/bin:$PATH"
 
@@ -506,6 +560,26 @@ elif [ "${TARGET_FILESYSTEM}" = "gs" ]; then
   fi
   DEFAULT_FS="gs://${GCS_BUCKET}"
   export DEFAULT_FS
+elif [ "${TARGET_FILESYSTEM}" = "cosn" ]; then
+  # Basic error checking
+  if [[ "${COS_SECRET_ID}" = "" ]]; then
+    echo "COS_SECRET_ID cannot be an empty string for COS"
+    return 1
+  fi
+  if [[ "${COS_SECRET_KEY}" = "" ]]; then
+    echo "COS_SECRET_KEY cannot be an empty string for COS"
+    return 1
+  fi
+  if [[ "${COS_REGION}" = "" ]]; then
+    echo "COS_REGION cannot be an empty string for COS"
+    return 1
+  fi
+  if [[ "${COS_BUCKET}" = "" ]]; then
+    echo "COS_BUCKET cannot be an empty string for COS"
+    return 1
+  fi
+  DEFAULT_FS="cosn://${COS_BUCKET}"
+  export DEFAULT_FS
 elif [ "${TARGET_FILESYSTEM}" = "isilon" ]; then
   if [ "${ISILON_NAMENODE}" = "" ]; then
     echo "In order to access the Isilon filesystem, ISILON_NAMENODE"
@@ -666,7 +740,7 @@ fi
 # overall build type) and does not apply when using a local Kudu build.
 export USE_KUDU_DEBUG_BUILD=${USE_KUDU_DEBUG_BUILD-false}
 
-export IMPALA_KUDU_VERSION=${IMPALA_KUDU_VERSION-"b5e7362e69"}
+export IMPALA_KUDU_VERSION=${IMPALA_KUDU_VERSION-"1.16.0"}
 export IMPALA_KUDU_HOME=${IMPALA_TOOLCHAIN_PACKAGES_HOME}/kudu-$IMPALA_KUDU_VERSION
 export IMPALA_KUDU_JAVA_HOME=\
 ${IMPALA_TOOLCHAIN_PACKAGES_HOME}/kudu-${IMPALA_KUDU_VERSION}/java
@@ -713,6 +787,7 @@ CLASSPATH="$IMPALA_FE_DIR/src/test/resources:$CLASSPATH"
 # A marker in the environment to prove that we really did source this file
 export IMPALA_CONFIG_SOURCED=1
 
+echo "IMPALA_VERSION          = $IMPALA_VERSION"
 echo "IMPALA_HOME             = $IMPALA_HOME"
 echo "HADOOP_HOME             = $HADOOP_HOME"
 echo "HADOOP_CONF_DIR         = $HADOOP_CONF_DIR"
@@ -739,6 +814,10 @@ echo "IMPALA_MAVEN_OPTIONS    = $IMPALA_MAVEN_OPTIONS"
 echo "IMPALA_TOOLCHAIN_HOST   = $IMPALA_TOOLCHAIN_HOST"
 echo "CDP_BUILD_NUMBER        = $CDP_BUILD_NUMBER"
 echo "CDP_COMPONENTS_HOME     = $CDP_COMPONENTS_HOME"
+if $USE_APACHE_HIVE; then
+  echo "APACHE_MIRROR           = $APACHE_MIRROR"
+  echo "APACHE_COMPONENTS_HOME  = $APACHE_COMPONENTS_HOME"
+fi
 echo "IMPALA_HADOOP_VERSION   = $IMPALA_HADOOP_VERSION"
 echo "IMPALA_AVRO_JAVA_VERSION= $IMPALA_AVRO_JAVA_VERSION"
 echo "IMPALA_PARQUET_VERSION  = $IMPALA_PARQUET_VERSION"
@@ -749,6 +828,7 @@ echo "IMPALA_KUDU_VERSION     = $IMPALA_KUDU_VERSION"
 echo "IMPALA_RANGER_VERSION   = $IMPALA_RANGER_VERSION"
 echo "IMPALA_ICEBERG_VERSION  = $IMPALA_ICEBERG_VERSION"
 echo "IMPALA_GCS_VERSION      = $IMPALA_GCS_VERSION"
+echo "IMPALA_COS_VERSION      = $IMPALA_COS_VERSION"
 
 # Kerberos things.  If the cluster exists and is kerberized, source
 # the required environment.  This is required for any hadoop tool to
