@@ -21,8 +21,8 @@
 # Note: for internal (aka pre-release) versions, the version should have
 # "-INTERNAL" appended. Parts of the code will look for this to distinguish
 # between released and internal versions.
-VERSION=4.0.0-RELEASE
-GIT_HASH=a702d2d4cf7e2062476e1057ebcbafd5e6fdcf02
+VERSION=${IMPALA_VERSION}
+GIT_HASH=1d7b63102ebc8974e8133c964917ea8052148088
 if [ -z $GIT_HASH ]
 then
   GIT_HASH="Could not obtain git hash"
