@@ -75,6 +75,7 @@ BUILD_UBSAN=0
 BUILD_UBSAN_FULL=0
 BUILD_TSAN=0
 BUILD_TSAN_FULL=0
+BUILD_DEBUG_NOOPT=0
 BUILD_SHARED_LIBS=0
 # Export MAKE_CMD so it is visible in scripts that invoke make, e.g. copy-udfs-udas.sh
 export MAKE_CMD=make
@@ -85,6 +86,8 @@ export MAKE_CMD=make
 : ${CMAKE_BUILD_TYPE:=Debug}
 
 # parse command line options
+# Note: if you add a new build type, please also add it to 'VALID_BUILD_TYPES' in
+# tests/common/environ.py.
 while [ -n "$*" ]
 do
   case "$1" in
@@ -145,9 +148,12 @@ do
     -tsan)
       BUILD_TSAN=1
       ;;
-     -full_tsan)
+    -full_tsan)
       BUILD_TSAN_FULL=1
       ;;
+    -debug_noopt)
+      BUILD_DEBUG_NOOPT=1
+      ;;
     -testpairwise)
       EXPLORATION_STRATEGY=pairwise
       ;;
@@ -226,6 +232,10 @@ do
       echo "[-full_ubsan] : Undefined behavior sanitizer build, including code generated"\
            "by cross-compilation to LLVM IR. Much slower queries than plain -ubsan"\
            "[Default: False]"
+      echo "[-debug_noopt] : Debug build without optimizations applied. The regular"\
+           "debug build applies basic optimizations, but even these optimizations may"\
+           "impact debuggability, so this is an option to omit the optimizations."\
+           "[Default: False]"
       echo "[-skiptests] : Skips execution of all tests"
       echo "[-notests] : Skips building and execution of all tests"
       echo "[-start_minicluster] : Start test cluster including Impala and all"\
@@ -325,6 +335,10 @@ if [[ ${BUILD_TSAN_FULL} -eq 1 ]]; then
   CMAKE_BUILD_TYPE_LIST+=(TSAN_FULL)
   export TSAN_FULL=1
 fi
+if [[ ${BUILD_DEBUG_NOOPT} -eq 1 ]]; then
+  CMAKE_BUILD_TYPE_LIST+=(DEBUG_NOOPT)
+  export DEBUG_NOOPT=1
+fi
 if [[ -n "${CMAKE_BUILD_TYPE_LIST:+1}" ]]; then
   if [[ ${#CMAKE_BUILD_TYPE_LIST[@]} -gt 1 ]]; then
     echo "ERROR: more than one CMake build type defined: ${CMAKE_BUILD_TYPE_LIST[@]}"
@@ -419,6 +433,9 @@ bootstrap_dependencies() {
     "$IMPALA_HOME/bin/bootstrap_toolchain.py"
     echo "Toolchain bootstrap complete."
   fi
+  if [[ "${USE_APACHE_HIVE}" = true ]]; then
+    "$IMPALA_HOME/testdata/bin/patch_hive.sh"
+  fi
 }
 
 # Build the Impala frontend and its dependencies.
@@ -452,8 +469,31 @@ build_all_components() {
     fi
   fi
   ${MAKE_CMD} -j${IMPALA_BUILD_THREADS:-4} ${IMPALA_MAKE_FLAGS} ${MAKE_TARGETS}
+  save_coverage_data ${build_type}
 }
 
+save_coverage_data() {
+  local build_type=$1
+  local gcov_prefix=''
+  case $build_type in
+    CODE_COVERAGE_RELEASE)
+      gcov_prefix='gcov_release'
+      ;;
+    CODE_COVERAGE_DEBUG)
+      gcov_prefix='gcov_debug'
+      ;;
+    *)
+      # Other build types don't generate coverage data
+      return
+      ;;
+  esac
+  # Copy all '.gcno' files to ${gcov_prefix}
+  mkdir -p ${IMPALA_HOME}/${gcov_prefix}
+  pushd ${IMPALA_HOME}
+  find ./be -name '*.gcno' -exec \
+       cp --parents \{\} ./${gcov_prefix} \;
+  popd
+}
 
 # Called with the CMAKE_BUILD_TYPE as the first argument, e.g.
 #   generate_cmake_files DEBUG
@@ -604,9 +644,17 @@ if [[ "$BUILD_RELEASE_AND_DEBUG" -eq 1 ]]; then
   # Build the standard release and debug builds. We can't do this for arbitrary build
   # types because many build types reuse the same be/build/debug and be/build/release
   # trees.
-  build_all_components RELEASE 1
+  CMAKE_BUILD_TYPE=RELEASE
+  if [[ ${CODE_COVERAGE} -eq 1 ]]; then
+    CMAKE_BUILD_TYPE=CODE_COVERAGE_RELEASE
+  fi
+  build_all_components ${CMAKE_BUILD_TYPE} 1
   # Avoid rebuilding targets that are independent of the build type.
-  build_all_components DEBUG 0
+  CMAKE_BUILD_TYPE=DEBUG
+  if [[ ${CODE_COVERAGE} -eq 1 ]]; then
+    CMAKE_BUILD_TYPE=CODE_COVERAGE_DEBUG
+  fi
+  build_all_components ${CMAKE_BUILD_TYPE} 0
 else
   build_all_components $CMAKE_BUILD_TYPE 1
 fi
