@@ -68,7 +68,7 @@ error_codes = (
 
   ("PARQUET_WRONG_PRECISION", 17,
     "File '$0' column '$1' has a precision that does not match the table metadata "
-    " precision. File metadata precision: $2, table metadata precision: $3."),
+    "precision. File metadata precision: $2, table metadata precision: $3."),
 
   ("PARQUET_BAD_CONVERTED_TYPE", 18,
    "File '$0' column '$1' does not have converted type set to DECIMAL"),
@@ -471,6 +471,12 @@ error_codes = (
 
   ("LOCAL_DISK_FAULTY", 152,
    "Query execution failure caused by local disk IO fatal error on backend: $0."),
+
+  ("JWKS_PARSE_ERROR", 153, "Error parsing JWKS: $0."),
+
+  ("JWT_VERIFY_FAILED", 154, "Error verifying JWT Token: $0."),
+
+  ("PARQUET_ROWS_SKIPPING", 155, "Couldn't skip rows in column '$0' in file '$1'."),
 )
 
 import sys
