@@ -27,7 +27,7 @@ import org.apache.impala.common.AnalysisException;
 import org.apache.impala.thrift.TAlterTableParams;
 import org.apache.impala.thrift.TAlterTableSetRowFormatParams;
 import org.apache.impala.thrift.TAlterTableType;
-
+import org.apache.hadoop.hive.metastore.api.StorageDescriptor;
 /**
  * Represents an ALTER TABLE [PARTITION partitionSet] SET ROW FORMAT statement.
  */
@@ -73,16 +73,16 @@ public class AlterTableSetRowFormatStmt extends AlterTableSetStmt {
       for (FeFsPartition partition: partitionSet_.getPartitions()) {
         if (partition.getFileFormat() != HdfsFileFormat.TEXT &&
             partition.getFileFormat() != HdfsFileFormat.SEQUENCE_FILE) {
-          throw new AnalysisException(String.format("ALTER TABLE SET ROW FORMAT is " +
-              "only supported on TEXT or SEQUENCE file formats.  " +
-              "Conflicting partition/format: %1$s / %2$s", partition.getPartitionName(),
-              HdfsFileFormat.fromHdfsInputFormatClass(
-                  partition.getFileFormat().inputFormat()).name()));
+          throw new AnalysisException(String.format("ALTER TABLE SET ROW FORMAT is "
+                  + "only supported on TEXT or SEQUENCE file formats.  "
+                  + "Conflicting partition/format: %1$s / %2$s",
+              partition.getPartitionName(), partition.getFileFormat().name()));
         }
       }
     } else {
+      StorageDescriptor sd = ((FeFsTable) tbl).getMetaStoreTable().getSd();
       HdfsFileFormat format = HdfsFileFormat.fromHdfsInputFormatClass(
-          ((FeFsTable) tbl).getMetaStoreTable().getSd().getInputFormat());
+          sd.getInputFormat(), sd.getSerdeInfo().getSerializationLib());
       if (format != HdfsFileFormat.TEXT &&
           format != HdfsFileFormat.SEQUENCE_FILE) {
         throw new AnalysisException(String.format("ALTER TABLE SET ROW FORMAT is " +
