@@ -393,6 +393,9 @@ public class AnalysisContext {
     public boolean requiresAcidComplexScanRewrite() {
       return canRewriteStatement() && analyzer_.hasTopLevelAcidCollectionTableRef();
     }
+    public boolean requiresZippingUnnestRewrite() {
+      return canRewriteStatement() && isZippingUnnestInSelectList(stmt_);
+    }
     public boolean requiresExprRewrite() {
       return isQueryStmt() || isInsertStmt() || isCreateTableAsSelectStmt()
           || isUpdateStmt() || isDeleteStmt();
@@ -406,6 +409,19 @@ public class AnalysisContext {
     }
     public void setUserHasProfileAccess(boolean value) { userHasProfileAccess_ = value; }
     public boolean userHasProfileAccess() { return userHasProfileAccess_; }
+
+    private boolean isZippingUnnestInSelectList(StatementBase stmt) {
+      if (!(stmt instanceof SelectStmt)) return false;
+      if (!stmt.analyzer_.getTableRefsFromUnnestExpr().isEmpty()) return true;
+      SelectStmt selectStmt = (SelectStmt)stmt;
+      for (TableRef tblRef : selectStmt.fromClause_.getTableRefs()) {
+        if (tblRef instanceof InlineViewRef &&
+            isZippingUnnestInSelectList(((InlineViewRef)tblRef).getViewStmt())) {
+          return true;
+        }
+      }
+      return false;
+    }
   }
 
   public Analyzer createAnalyzer(StmtTableCache stmtTableCache) {
@@ -419,6 +435,13 @@ public class AnalysisContext {
     return result;
   }
 
+  public AnalysisResult analyzeAndAuthorize(StatementBase stmt,
+      StmtTableCache stmtTableCache, AuthorizationChecker authzChecker)
+      throws ImpalaException {
+    return analyzeAndAuthorize(
+        stmt, stmtTableCache, authzChecker, false /*disableAuthorization*/);
+  }
+
   /**
    * Analyzes and authorizes the given statement using the provided table cache and
    * authorization checker.
@@ -426,8 +449,8 @@ public class AnalysisContext {
    * reveal the existence/absence of objects the user is not authorized to see.
    */
   public AnalysisResult analyzeAndAuthorize(StatementBase stmt,
-      StmtTableCache stmtTableCache, AuthorizationChecker authzChecker)
-      throws ImpalaException {
+      StmtTableCache stmtTableCache, AuthorizationChecker authzChecker,
+      boolean disableAuthorization) throws ImpalaException {
     // TODO: Clean up the creation/setting of the analysis result.
     analysisResult_ = new AnalysisResult();
     analysisResult_.stmt_ = stmt;
@@ -448,17 +471,24 @@ public class AnalysisContext {
     } finally {
       authzChecker.postAnalyze(authzCtx);
     }
-    timeline_.markEvent("Analysis finished");
+    long durationMs = timeline_.markEvent("Analysis finished") / 1000000;
+    LOG.info("Analysis took {} ms", durationMs);
 
     // Authorize statement and record exception. Authorization relies on information
     // collected during analysis.
     AuthorizationException authException = null;
-    try {
-      authzChecker.authorize(authzCtx, analysisResult_, catalog_);
-    } catch (AuthorizationException e) {
-      authException = e;
-    } finally {
-      authzChecker.postAuthorize(authzCtx, authException == null);
+    if (!disableAuthorization) {
+      try {
+        if (analysisResult_.getAnalyzer().encounteredMVAuthException()) {
+          throw new AuthorizationException(
+            analysisResult_.getAnalyzer().getMVAuthExceptionMsg());
+        }
+        authzChecker.authorize(authzCtx, analysisResult_, catalog_);
+      } catch (AuthorizationException e) {
+        authException = e;
+      } finally {
+        authzChecker.postAuthorize(authzCtx, authException == null);
+      }
     }
 
     // AuthorizationExceptions take precedence over AnalysisExceptions so as not
@@ -533,15 +563,22 @@ public class AnalysisContext {
       new StmtRewriter.AcidRewriter().rewrite(analysisResult_);
       shouldReAnalyze = true;
     }
+    if (analysisResult_.requiresZippingUnnestRewrite()) {
+      new StmtRewriter.ZippingUnnestRewriter().rewrite(analysisResult_);
+      shouldReAnalyze = true;
+    }
     if (!shouldReAnalyze) return;
 
     // For SetOperationStmt we must replace the query statement with the rewritten version
-    // before re-analysis.
+    // before re-analysis and set the explain flag of the rewritten version if the
+    // original is explain statement.
     if (analysisResult_.requiresSetOperationRewrite()) {
       if (analysisResult_.isSetOperationStmt()) {
         if (((SetOperationStmt) analysisResult_.getStmt()).hasRewrittenStmt()) {
+          boolean isExplain = analysisResult_.isExplainStmt();
           analysisResult_.stmt_ =
             ((SetOperationStmt) analysisResult_.getStmt()).getRewrittenStmt();
+          if (isExplain) analysisResult_.stmt_.setIsExplain();
         }
       }
     }
