@@ -25,6 +25,8 @@ import org.apache.avro.Schema;
 import org.apache.avro.SchemaParseException;
 import org.apache.hadoop.hive.metastore.api.SQLForeignKey;
 import org.apache.hadoop.hive.metastore.api.SQLPrimaryKey;
+import org.apache.iceberg.TableProperties;
+import org.apache.iceberg.mr.Catalogs;
 import org.apache.impala.authorization.AuthorizationConfig;
 import org.apache.impala.catalog.KuduTable;
 import org.apache.impala.catalog.IcebergTable;
@@ -34,9 +36,12 @@ import org.apache.impala.common.AnalysisException;
 import org.apache.impala.common.ImpalaRuntimeException;
 import org.apache.impala.common.RuntimeEnv;
 import org.apache.impala.service.BackendConfig;
+import org.apache.impala.thrift.TCompressionCodec;
 import org.apache.impala.thrift.TCreateTableParams;
+import org.apache.impala.thrift.THdfsCompression;
 import org.apache.impala.thrift.THdfsFileFormat;
 import org.apache.impala.thrift.TIcebergCatalog;
+import org.apache.impala.thrift.TIcebergFileFormat;
 import org.apache.impala.thrift.TIcebergPartitionTransformType;
 import org.apache.impala.thrift.TSortingOrder;
 import org.apache.impala.thrift.TTableName;
@@ -53,6 +58,7 @@ import com.google.common.base.Preconditions;
 import com.google.common.base.Strings;
 import com.google.common.collect.Maps;
 import com.google.common.primitives.Ints;
+import com.google.common.primitives.Longs;
 
 /**
  * Represents a CREATE TABLE statement.
@@ -129,11 +135,11 @@ public class CreateTableStmt extends StatementBase {
   Map<String, String> getSerdeProperties() { return tableDef_.getSerdeProperties(); }
   public THdfsFileFormat getFileFormat() { return tableDef_.getFileFormat(); }
   RowFormat getRowFormat() { return tableDef_.getRowFormat(); }
-  private void putGeneratedKuduProperty(String key, String value) {
-    tableDef_.putGeneratedKuduProperty(key, value);
+  private void putGeneratedProperty(String key, String value) {
+    tableDef_.putGeneratedProperty(key, value);
   }
   public Map<String, String> getGeneratedKuduProperties() {
-    return tableDef_.getGeneratedKuduProperties();
+    return tableDef_.getGeneratedProperties();
   }
 
   // Only exposed for ToSqlUtils. Returns the list of primary keys declared by the user
@@ -262,6 +268,26 @@ public class CreateTableStmt extends StatementBase {
         THdfsFileFormat.ICEBERG) {
       throw new AnalysisException("Table requires at least 1 column");
     }
+    if (getRowFormat() != null) {
+      String fieldDelimiter = getRowFormat().getFieldDelimiter();
+      String lineDelimiter = getRowFormat().getLineDelimiter();
+      String escapeChar = getRowFormat().getEscapeChar();
+      if (getFileFormat() != THdfsFileFormat.TEXT
+          && getFileFormat() != THdfsFileFormat.SEQUENCE_FILE) {
+        if (fieldDelimiter != null) {
+          analyzer.addWarning("'ROW FORMAT DELIMITED FIELDS TERMINATED BY '"
+              + fieldDelimiter + "'' is ignored.");
+        }
+        if (lineDelimiter != null) {
+          analyzer.addWarning("'ROW FORMAT DELIMITED LINES TERMINATED BY '"
+              + lineDelimiter + "'' is ignored.");
+        }
+        if (escapeChar != null) {
+          analyzer.addWarning(
+              "'ROW FORMAT DELIMITED ESCAPED BY '" + escapeChar + "'' is ignored.");
+        }
+      }
+    }
     if (getFileFormat() == THdfsFileFormat.AVRO) {
       setColumnDefs(analyzeAvroSchema(analyzer));
       if (getColumnDefs().isEmpty()) {
@@ -278,7 +304,7 @@ public class CreateTableStmt extends StatementBase {
       List<IcebergPartitionSpec> iceSpec = tableDef_.getIcebergPartitionSpecs();
       if (iceSpec != null && !iceSpec.isEmpty()) {
         throw new AnalysisException(
-            "PARTITION BY SPEC is only valid for Iceberg tables.");
+            "PARTITIONED BY SPEC is only valid for Iceberg tables.");
       }
     }
 
@@ -314,8 +340,7 @@ public class CreateTableStmt extends StatementBase {
     }
 
     analyzeKuduTableProperties(analyzer);
-    if (isExternal() && !Boolean.parseBoolean(getTblProperties().get(
-        Table.TBL_PROP_EXTERNAL_TABLE_PURGE))) {
+    if (isExternalWithNoPurge()) {
       // this is an external table
       analyzeExternalKuduTableParams(analyzer);
     } else {
@@ -352,7 +377,7 @@ public class CreateTableStmt extends StatementBase {
       throw new AnalysisException("Invalid storage handler specified for Kudu table: " +
           handler);
     }
-    putGeneratedKuduProperty(KuduTable.KEY_STORAGE_HANDLER,
+    putGeneratedProperty(KuduTable.KEY_STORAGE_HANDLER,
         KuduTable.KUDU_STORAGE_HANDLER);
 
     String kuduMasters = getKuduMasters(analyzer);
@@ -361,7 +386,7 @@ public class CreateTableStmt extends StatementBase {
           "Table property '%s' is required when the impalad startup flag " +
           "-kudu_master_hosts is not used.", KuduTable.KEY_MASTER_HOSTS));
     }
-    putGeneratedKuduProperty(KuduTable.KEY_MASTER_HOSTS, kuduMasters);
+    putGeneratedProperty(KuduTable.KEY_MASTER_HOSTS, kuduMasters);
 
     // TODO: Find out what is creating a directory in HDFS and stop doing that. Kudu
     //       tables shouldn't have HDFS dirs: IMPALA-3570
@@ -477,7 +502,7 @@ public class CreateTableStmt extends StatementBase {
       throw new AnalysisException(String.format("Cannot analyze Kudu table '%s': %s",
           getTbl(), e.getMessage()));
     }
-    putGeneratedKuduProperty(KuduTable.KEY_TABLE_NAME,
+    putGeneratedProperty(KuduTable.KEY_TABLE_NAME,
         KuduUtil.getDefaultKuduTableName(getDb(), getTbl(), isHMSIntegrationEnabled));
   }
 
@@ -601,29 +626,88 @@ public class CreateTableStmt extends StatementBase {
       throw new AnalysisException("Invalid storage handler " +
           "specified for Iceberg format: " + handler);
     }
-    putGeneratedKuduProperty(IcebergTable.KEY_STORAGE_HANDLER,
+    putGeneratedProperty(IcebergTable.KEY_STORAGE_HANDLER,
         IcebergTable.ICEBERG_STORAGE_HANDLER);
+    putGeneratedProperty(TableProperties.ENGINE_HIVE_ENABLED, "true");
 
     String fileformat = getTblProperties().get(IcebergTable.ICEBERG_FILE_FORMAT);
-    if (fileformat != null && IcebergUtil.getIcebergFileFormat(fileformat) == null) {
+    TIcebergFileFormat icebergFileFormat = IcebergUtil.getIcebergFileFormat(fileformat);
+    if (fileformat != null && icebergFileFormat == null) {
       throw new AnalysisException("Invalid fileformat for Iceberg table: " + fileformat);
     }
     if (fileformat == null || fileformat.isEmpty()) {
-      putGeneratedKuduProperty(IcebergTable.ICEBERG_FILE_FORMAT, "parquet");
+      putGeneratedProperty(IcebergTable.ICEBERG_FILE_FORMAT, "parquet");
     }
 
+    validateIcebergParquetCompressionCodec(icebergFileFormat);
+    validateIcebergParquetRowGroupSize(icebergFileFormat);
+    validateIcebergParquetPageSize(icebergFileFormat,
+        IcebergTable.PARQUET_PLAIN_PAGE_SIZE, "page size");
+    validateIcebergParquetPageSize(icebergFileFormat,
+        IcebergTable.PARQUET_DICT_PAGE_SIZE, "dictionary page size");
+
     // Determine the Iceberg catalog being used. The default catalog is HiveCatalog.
     String catalogStr = getTblProperties().get(IcebergTable.ICEBERG_CATALOG);
     TIcebergCatalog catalog;
     if (catalogStr == null || catalogStr.isEmpty()) {
       catalog = TIcebergCatalog.HIVE_CATALOG;
-      putGeneratedKuduProperty(IcebergTable.ICEBERG_CATALOG, "hive.catalog");
     } else {
       catalog = IcebergUtil.getTIcebergCatalog(catalogStr);
     }
     validateIcebergTableProperties(catalog);
   }
 
+  private void validateIcebergParquetCompressionCodec(
+      TIcebergFileFormat icebergFileFormat) throws AnalysisException {
+    if (icebergFileFormat != TIcebergFileFormat.PARQUET) {
+      if (getTblProperties().containsKey(IcebergTable.PARQUET_COMPRESSION_CODEC)) {
+          throw new AnalysisException(IcebergTable.PARQUET_COMPRESSION_CODEC +
+              " should be set only for parquet file format");
+      }
+      if (getTblProperties().containsKey(IcebergTable.PARQUET_COMPRESSION_LEVEL)) {
+          throw new AnalysisException(IcebergTable.PARQUET_COMPRESSION_LEVEL +
+              " should be set only for parquet file format");
+      }
+    } else {
+      StringBuilder errMsg = new StringBuilder();
+      if (IcebergUtil.parseParquetCompressionCodec(true, getTblProperties(), errMsg)
+          == null) {
+        throw new AnalysisException(errMsg.toString());
+      }
+    }
+  }
+
+  private void validateIcebergParquetRowGroupSize(TIcebergFileFormat icebergFileFormat)
+      throws AnalysisException {
+    if (getTblProperties().containsKey(IcebergTable.PARQUET_ROW_GROUP_SIZE)) {
+      if (icebergFileFormat != TIcebergFileFormat.PARQUET) {
+        throw new AnalysisException(IcebergTable.PARQUET_ROW_GROUP_SIZE +
+            " should be set only for parquet file format");
+      }
+    }
+
+    StringBuilder errMsg = new StringBuilder();
+    if (IcebergUtil.parseParquetRowGroupSize(getTblProperties(), errMsg) == null) {
+      throw new AnalysisException(errMsg.toString());
+    }
+  }
+
+  private void validateIcebergParquetPageSize(TIcebergFileFormat icebergFileFormat,
+      String pageSizeTblProp, String descr) throws AnalysisException {
+    if (getTblProperties().containsKey(pageSizeTblProp)) {
+      if (icebergFileFormat != TIcebergFileFormat.PARQUET) {
+        throw new AnalysisException(pageSizeTblProp +
+            " should be set only for parquet file format");
+      }
+    }
+
+    StringBuilder errMsg = new StringBuilder();
+    if (IcebergUtil.parseParquetPageSize(getTblProperties(), pageSizeTblProp, descr,
+        errMsg) == null) {
+      throw new AnalysisException(errMsg.toString());
+    }
+  }
+
   private void validateIcebergTableProperties(TIcebergCatalog catalog)
       throws AnalysisException {
     // Metadata location is only used by HiveCatalog, but we shouldn't allow setting this
@@ -639,6 +723,8 @@ public class CreateTableStmt extends StatementBase {
       break;
       case HADOOP_TABLES: validateTableInHadoopTables();
       break;
+      case CATALOGS: validateTableInCatalogs();
+      break;
       default: throw new AnalysisException(String.format(
           "Unknown Iceberg catalog type: %s", catalog));
     }
@@ -649,6 +735,19 @@ public class CreateTableStmt extends StatementBase {
       throw new AnalysisException(String.format("%s cannot be set for Iceberg table " +
           "stored in hive.catalog", IcebergTable.ICEBERG_CATALOG_LOCATION));
     }
+    if (isExternalWithNoPurge()) {
+      String tableId = getTblProperties().get(IcebergTable.ICEBERG_TABLE_IDENTIFIER);
+      if (tableId == null || tableId.isEmpty()) {
+        tableId = getTblProperties().get(Catalogs.NAME);
+      }
+      if (tableId == null || tableId.isEmpty()) {
+        throw new AnalysisException(String.format("Table property '%s' is necessary " +
+            "for external Iceberg tables stored in hive.catalog. " +
+            "For creating a completely new Iceberg table, use 'CREATE TABLE' " +
+            "(no EXTERNAL keyword).",
+            IcebergTable.ICEBERG_TABLE_IDENTIFIER));
+      }
+    }
   }
 
   private void validateTableInHadoopCatalog() throws AnalysisException {
@@ -670,6 +769,18 @@ public class CreateTableStmt extends StatementBase {
       throw new AnalysisException(String.format("%s cannot be set for Iceberg table " +
           "stored in hadoop.tables", IcebergTable.ICEBERG_CATALOG_LOCATION));
     }
+    if (isExternalWithNoPurge() && getLocation() == null) {
+      throw new AnalysisException("Set LOCATION for external Iceberg tables " +
+          "stored in hadoop.tables. For creating a completely new Iceberg table, use " +
+          "'CREATE TABLE' (no EXTERNAL keyword).");
+    }
+  }
+
+  private void validateTableInCatalogs() {
+    String tableId = getTblProperties().get(IcebergTable.ICEBERG_TABLE_IDENTIFIER);
+    if (tableId != null && !tableId.isEmpty()) {
+      putGeneratedProperty(Catalogs.NAME, tableId);
+    }
   }
 
   /**
@@ -735,4 +846,12 @@ public class CreateTableStmt extends StatementBase {
     getColumnDefs().addAll(getPartitionColumnDefs());
     getPartitionColumnDefs().clear();
   }
+
+  /**
+   * @return true for external tables that don't have "external.table.purge" set to true.
+   */
+  private boolean isExternalWithNoPurge() {
+    return isExternal() && !Boolean.parseBoolean(getTblProperties().get(
+      Table.TBL_PROP_EXTERNAL_TABLE_PURGE));
+  }
 }
