@@ -413,6 +413,9 @@ abstract public class Expr extends TreeNode<Expr> implements ParseNode, Cloneabl
   // For exprs of type Predicate, this keeps track of predicate hints
   protected List<PlanHint> predicateHints_;
 
+  // Is codegen disabled for this expression ?
+  private boolean isCodegenDisabled_ = false;
+
   protected Expr() {
     type_ = Type.INVALID;
     selectivity_ = -1.0;
@@ -441,6 +444,7 @@ abstract public class Expr extends TreeNode<Expr> implements ParseNode, Cloneabl
       predicateHints_ = new ArrayList<>();
       predicateHints_.addAll(other.predicateHints_);
     }
+    isCodegenDisabled_ = other.isCodegenDisabled_;
   }
 
   public boolean isAnalyzed() { return isAnalyzed_; }
@@ -462,6 +466,12 @@ abstract public class Expr extends TreeNode<Expr> implements ParseNode, Cloneabl
   public boolean isAuxExpr() { return isAuxExpr_; }
   public void setIsAuxExpr() { isAuxExpr_ = true; }
   public Function getFn() { return fn_; }
+  public void disableCodegen() {
+    isCodegenDisabled_ = true;
+    for (Expr child : children_) {
+      child.disableCodegen();
+    }
+  }
 
   /**
    * Perform semantic analysis of node and all of its children.
@@ -856,6 +866,7 @@ abstract public class Expr extends TreeNode<Expr> implements ParseNode, Cloneabl
     msg.type = type_.toThrift();
     msg.is_constant = isConstant_;
     msg.num_children = children_.size();
+    msg.setIs_codegen_disabled(isCodegenDisabled_);
     if (fn_ != null) {
       TFunction thriftFn = fn_.toThrift();
       thriftFn.setLast_modified_time(fn_.getLastModifiedTime());
@@ -1131,14 +1142,16 @@ abstract public class Expr extends TreeNode<Expr> implements ParseNode, Cloneabl
       Expr substExpr = smap.get(this);
       if (substExpr != null) return substExpr.clone();
     }
+    substituteImplOnChildren(smap, analyzer);
+    resetAnalysisState();
+    return this;
+  }
+
+  protected final void substituteImplOnChildren(ExprSubstitutionMap smap,
+      Analyzer analyzer) {
     for (int i = 0; i < children_.size(); ++i) {
       children_.set(i, children_.get(i).substituteImpl(smap, analyzer));
     }
-    // SlotRefs must remain analyzed to support substitution across query blocks. All
-    // other exprs must be analyzed again after the substitution to add implicit casts
-    // and for resolving their correct function signature.
-    if (!(this instanceof SlotRef)) resetAnalysisState();
-    return this;
   }
 
   /**
@@ -1628,6 +1641,30 @@ abstract public class Expr extends TreeNode<Expr> implements ParseNode, Cloneabl
     return this;
   }
 
+  /**
+   * Returns the source expression for this expression. Traverses the source
+   * exprs of intermediate slot descriptors to resolve materialization points
+   * (e.g., aggregations). Returns null if there are multiple source Exprs
+   * mapped to the expression at any given point.
+   */
+  public Expr findSrcExpr() {
+    // If the source expression is a constant expression, it won't have a scanSlotRef
+    // and we can return this.
+    if (isConstant()) {
+      return this;
+    }
+    SlotRef slotRef = unwrapSlotRef(false);
+    if (slotRef == null) return null;
+    SlotDescriptor slotDesc = slotRef.getDesc();
+    if (slotDesc.isScanSlot()) return slotRef;
+    if (slotDesc.getSourceExprs().size() == 1) {
+      return slotDesc.getSourceExprs().get(0).findSrcExpr();
+    }
+    // No known source expr, or there are several source exprs meaning the slot is
+    // has no single source table.
+    return null;
+  }
+
   /**
    * Returns the descriptor of the scan slot that directly or indirectly produces
    * the values of 'this' SlotRef. Traverses the source exprs of intermediate slot
@@ -1635,16 +1672,15 @@ abstract public class Expr extends TreeNode<Expr> implements ParseNode, Cloneabl
    * Returns null if 'e' or any source expr of 'e' is not a SlotRef or cast SlotRef.
    */
   public SlotDescriptor findSrcScanSlot() {
-    SlotRef slotRef = unwrapSlotRef(false);
-    if (slotRef == null) return null;
-    SlotDescriptor slotDesc = slotRef.getDesc();
-    if (slotDesc.isScanSlot()) return slotDesc;
-    if (slotDesc.getSourceExprs().size() == 1) {
-      return slotDesc.getSourceExprs().get(0).findSrcScanSlot();
+    Expr sourceExpr = findSrcExpr();
+    if (sourceExpr == null) {
+      return null;
     }
-    // No known source expr, or there are several source exprs meaning the slot is
-    // has no single source table.
-    return null;
+    SlotRef slotRef = sourceExpr.unwrapSlotRef(false);
+    if (slotRef == null) {
+      return null;
+    }
+    return slotRef.getDesc();
   }
 
   /**
@@ -1855,4 +1891,26 @@ abstract public class Expr extends TreeNode<Expr> implements ParseNode, Cloneabl
     }
     return hasChanges;
   }
+
+  /**
+   * A slot descriptor may be associated with more than 1 source expression.
+   * This method returns the first source expr in that case or null if there
+   * are no source exprs.
+   */
+  public Expr getSlotDescFirstSourceExpr() {
+    SlotRef slotRef = unwrapSlotRef(false);
+    if (slotRef == null) return null;
+    SlotDescriptor slotDesc = slotRef.getDesc();
+    if (slotDesc.getSourceExprs().size() >= 1) {
+      return slotDesc.getSourceExprs().get(0);
+    }
+    return null;
+  }
+
+  /**
+   * Returns true if 'this' is a constant.
+   */
+  public boolean shouldConvertToCNF() {
+    return isConstant();
+  }
 }
