@@ -20,7 +20,6 @@ package org.apache.impala.analysis;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Set;
-import java.util.stream.Collectors;
 
 import org.apache.impala.authorization.AuthorizationContext;
 import org.apache.impala.authorization.TableMask;
@@ -32,6 +31,7 @@ import org.apache.impala.catalog.StructField;
 import org.apache.impala.catalog.StructType;
 import org.apache.impala.common.AnalysisException;
 import org.apache.impala.common.InternalException;
+import org.apache.impala.common.Pair;
 import org.apache.impala.rewrite.ExprRewriter;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -78,6 +78,10 @@ public class InlineViewRef extends TableRef {
   // Whether this is an inline view generated for table masking.
   private boolean isTableMaskingView_ = false;
 
+  // Whether this is an inline view generated for a non-correlated scalar subquery
+  // returning at most one value.
+  private boolean isNonCorrelatedScalarSubquery_ = false;
+
   // END: Members that need to be reset()
   /////////////////////////////////////////
 
@@ -156,12 +160,13 @@ public class InlineViewRef extends TableRef {
     List<Column> columns = tableMask.getRequiredColumns();
     List<SelectListItem> items = Lists.newArrayListWithCapacity(columns.size());
     for (Column col: columns) {
-      Preconditions.checkState(!col.getType().isComplexType(),
-          "Complex type columns should not be registered to table mask");
       items.add(new SelectListItem(
           tableMask.createColumnMask(col.getName(), col.getType(), authzCtx),
           /*alias*/ col.getName()));
     }
+    if (tableMask.hasComplexColumnMask()) {
+      throw new AnalysisException("Column masking is not supported for complex types");
+    }
     if (columns.isEmpty()) {
       // No columns so use "SELECT 1 FROM tbl" to make a valid statement.
       items.add(new SelectListItem(NumericLiteral.create(1), /*alias*/null));
@@ -227,6 +232,7 @@ public class InlineViewRef extends TableRef {
         queryStmt_.hasLimit() || queryStmt_.hasOffset());
     queryStmt_.getMaterializedTupleIds(materializedTupleIds_);
     desc_ = analyzer.registerTableRef(this);
+    desc_.setSourceView(this);
     isAnalyzed_ = true;  // true now that we have assigned desc
 
     // For constant selects we materialize its exprs into a tuple.
@@ -251,24 +257,15 @@ public class InlineViewRef extends TableRef {
     for (int i = 0; i < getColLabels().size(); ++i) {
       String colName = getColLabels().get(i).toLowerCase();
       Expr colExpr = queryStmt_.getResultExprs().get(i);
-      Path p = new Path(desc_, Lists.newArrayList(colName));
-      Preconditions.checkState(p.resolve());
-      SlotDescriptor slotDesc = analyzer.registerSlotRef(p);
-      slotDesc.setSourceExpr(colExpr);
-      slotDesc.setStats(ColumnStats.fromExpr(colExpr));
-      SlotRef slotRef = new SlotRef(slotDesc);
-      smap_.put(slotRef, colExpr);
-      baseTblSmap_.put(slotRef, queryStmt_.getBaseTblResultExprs().get(i));
-      if (createAuxPredicate(colExpr)) {
-        analyzer.createAuxEqPredicate(new SlotRef(slotDesc), colExpr.clone());
-      }
+      Expr baseTableExpr =  queryStmt_.getBaseTblResultExprs().get(i);
+      addColumnToSubstitutionMaps(analyzer, colName, colExpr, baseTableExpr);
     }
     if (LOG.isTraceEnabled()) {
       LOG.trace("inline view " + getUniqueAlias() + " smap: " + smap_.debugString());
       LOG.trace("inline view " + getUniqueAlias() + " baseTblSmap: " +
           baseTblSmap_.debugString());
+      Preconditions.checkState(baseTblSmap_.checkComposedFrom(smap_));
     }
-    Preconditions.checkState(baseTblSmap_.checkComposedFrom(smap_));
 
     analyzeTableSample(analyzer);
     analyzeHints(analyzer);
@@ -276,12 +273,149 @@ public class InlineViewRef extends TableRef {
     analyzeJoin(analyzer);
   }
 
+  private void addColumnToSubstitutionMaps(
+      Analyzer analyzer, String colName, Expr colExpr, Expr baseTableExpr)
+      throws AnalysisException {
+    Path p = new Path(desc_, Lists.newArrayList(colName));
+    Preconditions.checkState(p.resolve());
+    SlotDescriptor slotDesc = analyzer.registerSlotRef(p, false);
+    slotDesc.setSourceExpr(colExpr);
+    slotDesc.setStats(ColumnStats.fromExpr(colExpr));
+    SlotRef slotRef = new SlotRef(slotDesc);
+    smap_.put(slotRef, colExpr);
+    baseTblSmap_.put(slotRef, baseTableExpr);
+    if (createAuxPredicate(colExpr)) {
+      analyzer.createAuxEqPredicate(new SlotRef(slotDesc), colExpr.clone());
+    }
+
+    if (colExpr.getType().isArrayType()) {
+      // Calling registerSlotRef() above created a new SlotDescriptor + TupleDescriptor
+      // hierarchy for Array types. Walk through this hiararchy and add all slot refs
+      // to smap_ and baseTblSmap_.
+      // Source must be a SlotRef
+      SlotRef srcSlotRef = (SlotRef) colExpr;
+      SlotRef baseTableSlotRef = (SlotRef) baseTableExpr;
+      SlotDescriptor srcSlotDesc = srcSlotRef.getDesc();
+      SlotDescriptor baseTableSlotDesc = baseTableSlotRef.getDesc();
+      TupleDescriptor itemTupleDesc = slotDesc.getItemTupleDesc();
+      TupleDescriptor srcItemTupleDesc = srcSlotDesc.getItemTupleDesc();
+      TupleDescriptor baseTableItemTupleDesc = baseTableSlotDesc.getItemTupleDesc();
+      // We don't recurse deeper and only add the immediate item child to the
+      // substitution map. This is enough both for collections in select list and in
+      // from clause.
+      if (itemTupleDesc != null) {
+        Preconditions.checkState(srcItemTupleDesc != null);
+        Preconditions.checkState(baseTableItemTupleDesc != null);
+        // Assume that there is only a single slot as struct in nested in
+        // arrays is not yet supported in select lists.
+        Preconditions.checkState(itemTupleDesc.getSlots().size() == 1);
+        Preconditions.checkState(srcItemTupleDesc.getSlots().size() == 1);
+        Preconditions.checkState(baseTableItemTupleDesc.getSlots().size() == 1);
+        SlotDescriptor itemSlotDesc = itemTupleDesc.getSlots().get(0);
+        SlotDescriptor srcItemSlotDesc = srcItemTupleDesc.getSlots().get(0);
+        SlotDescriptor baseTableItemSlotDesc = baseTableItemTupleDesc.getSlots().get(0);
+        SlotRef itemSlotRef = new SlotRef(itemSlotDesc);
+        SlotRef srcItemSlotRef = new SlotRef(srcItemSlotDesc);
+        SlotRef beseTableItemSlotRef = new SlotRef(baseTableItemSlotDesc);
+        smap_.put(itemSlotRef, srcItemSlotRef);
+        baseTblSmap_.put(itemSlotRef, beseTableItemSlotRef);
+        if (createAuxPredicate(colExpr)) {
+          analyzer.createAuxEqPredicate(
+              new SlotRef(itemSlotDesc), srcItemSlotRef.clone());
+        }
+      }
+    }
+
+    if (colExpr.getType().isStructType()) {
+      Preconditions.checkState(colExpr instanceof SlotRef);
+      Preconditions.checkState(baseTableExpr instanceof SlotRef);
+      putStructMembersIntoSmap(smap_, slotDesc, (SlotRef) colExpr);
+      putStructMembersIntoSmap(baseTblSmap_, slotDesc, (SlotRef) baseTableExpr);
+      createAuxPredicatesForStructMembers(analyzer, slotDesc, (SlotRef) colExpr);
+    }
+  }
+
+  // Puts the fields of 'rhsStruct' into 'smap' as right hand side (mapped) values,
+  // recursively. The keys (left hand side values) are constructed based on the
+  // corresponding slot descriptors in the itemTupleDesc of 'lhsSlotDesc'.
+  private void putStructMembersIntoSmap(ExprSubstitutionMap smap,
+      SlotDescriptor lhsSlotDesc, SlotRef rhsStruct) {
+    for (Pair<SlotDescriptor, SlotRef> pair :
+        getStructSlotDescSlotRefPairs(lhsSlotDesc, rhsStruct)) {
+      SlotDescriptor lhs = pair.first;
+      SlotRef rhs = pair.second;
+      smap.put(new SlotRef(lhs), rhs);
+    }
+  }
+
+  private void createAuxPredicatesForStructMembers(Analyzer analyzer,
+      SlotDescriptor structSlotDesc, SlotRef structExpr) {
+    for (Pair<SlotDescriptor, SlotRef> pair :
+        getStructSlotDescSlotRefPairs(structSlotDesc, structExpr)) {
+      SlotDescriptor structMemberSlotDesc = pair.first;
+      SlotRef structMemberExpr = pair.second;
+
+      if (createAuxPredicate(structMemberExpr)) {
+        analyzer.createAuxEqPredicate(
+            new SlotRef(structMemberSlotDesc), structMemberExpr.clone());
+      }
+    }
+  }
+
+  /**
+   * Given a slot desc and a SlotRef expression, both referring to the same struct,
+   * returns a list of corresponding SlotDescriptor/SlotRef pairs of the struct members,
+   * recursively.
+   */
+  private List<Pair<SlotDescriptor, SlotRef>> getStructSlotDescSlotRefPairs(
+      SlotDescriptor structSlotDesc, SlotRef structExpr) {
+    Preconditions.checkState(structSlotDesc.getType().isStructType());
+    Preconditions.checkState(structExpr.getType().isStructType());
+    Preconditions.checkState(structSlotDesc.getType().equals(structExpr.getType()));
+
+    List<Pair<SlotDescriptor, SlotRef>> result = new ArrayList<>();
+
+    TupleDescriptor lhsItemTupleDesc = structSlotDesc.getItemTupleDesc();
+    Preconditions.checkNotNull(lhsItemTupleDesc);
+    List<SlotDescriptor> lhsChildSlotDescs = lhsItemTupleDesc.getSlots();
+    Preconditions.checkState(
+        lhsChildSlotDescs.size() == structExpr.getChildren().size());
+    for (int i = 0; i < lhsChildSlotDescs.size(); i++) {
+      SlotDescriptor lhsChildSlotDesc = lhsChildSlotDescs.get(i);
+
+      Expr rhsChild = structExpr.getChildren().get(i);
+      Preconditions.checkState(rhsChild instanceof SlotRef);
+      SlotRef rhsChildSlotRef = (SlotRef) rhsChild;
+
+      List<String> lhsRawPath = lhsChildSlotDesc.getPath().getRawPath();
+      Path rhsPath = rhsChildSlotRef.getResolvedPath();
+
+      // The path can be null in the case of the sorting tuple.
+      if (rhsPath != null) {
+        List<String> rhsRawPath = rhsPath.getRawPath();
+
+        // Check that the children come in the same order on both lhs and rhs. If not, the
+        // last part of the paths would be different.
+        Preconditions.checkState(lhsRawPath.get(lhsRawPath.size() - 1)
+            .equals(rhsRawPath.get(rhsRawPath.size() - 1)));
+
+        result.add(new Pair(lhsChildSlotDesc, rhsChildSlotRef));
+
+        if (rhsChildSlotRef.getType().isStructType()) {
+          result.addAll(
+              getStructSlotDescSlotRefPairs(lhsChildSlotDesc, rhsChildSlotRef));
+        }
+      }
+    }
+    return result;
+  }
+
   /**
    * Checks if an auxiliary predicate should be created for an expr. Returns False if the
    * inline view has a SELECT stmt with analytic functions and the expr is not in the
    * common partition exprs of all the analytic functions computed by this inline view.
    */
-  public boolean createAuxPredicate(Expr e) {
+  private boolean createAuxPredicate(Expr e) {
     if (!(queryStmt_ instanceof SelectStmt)
         || !((SelectStmt) queryStmt_).hasAnalyticInfo()) {
       return true;
@@ -389,6 +523,11 @@ public class InlineViewRef extends TableRef {
     return queryStmt_.getColLabels();
   }
 
+  @Override
+  public List<Column> getColumnsInHiveOrder() {
+    return view_.getColumnsInHiveOrder();
+  }
+
   public FeView getView() { return view_; }
 
   public boolean isTableMaskingView() { return isTableMaskingView_; }
@@ -446,4 +585,11 @@ public class InlineViewRef extends TableRef {
     }
     return sql.toString();
   }
+
+  public void setIsNonCorrelatedScalarSubquery() {
+    isNonCorrelatedScalarSubquery_ = true;
+  }
+  public boolean isNonCorrelatedScalarSubquery() {
+    return isNonCorrelatedScalarSubquery_;
+  }
 }
