@@ -18,6 +18,7 @@
 package org.apache.impala.analysis;
 
 import java.io.IOException;
+import java.nio.ByteBuffer;
 import java.util.ArrayList;
 import java.util.HashSet;
 import java.util.List;
@@ -38,6 +39,7 @@ import org.apache.impala.catalog.HdfsFileFormat;
 import org.apache.impala.catalog.HdfsTable;
 import org.apache.impala.catalog.IcebergColumn;
 import org.apache.impala.catalog.KuduColumn;
+import org.apache.impala.catalog.MaterializedViewHdfsTable;
 import org.apache.impala.catalog.PrunablePartition;
 import org.apache.impala.catalog.Type;
 import org.apache.impala.catalog.View;
@@ -51,6 +53,7 @@ import org.apache.impala.rewrite.ExprRewriter;
 import org.apache.impala.thrift.TIcebergPartitionTransformType;
 import org.apache.impala.thrift.TSortingOrder;
 import org.apache.impala.util.IcebergUtil;
+import org.apache.thrift.TBaseHelper;
 
 import com.google.common.base.Joiner;
 import com.google.common.base.Preconditions;
@@ -199,6 +202,10 @@ public class InsertStmt extends StatementBase {
   // Set by the Frontend if the target table is transactional.
   private long writeId_ = -1;
 
+  // Serialized metadata of transaction object which is set by the Frontend if the
+  // target table is Kudu table and Kudu's transaction is enabled.
+  private java.nio.ByteBuffer kuduTxnToken_ = null;
+
   // END: Members that need to be reset()
   /////////////////////////////////////////
 
@@ -257,6 +264,7 @@ public class InsertStmt extends StatementBase {
     table_ = other.table_;
     isUpsert_ = other.isUpsert_;
     writeId_ = other.writeId_;
+    kuduTxnToken_ = org.apache.thrift.TBaseHelper.copyBinary(other.kuduTxnToken_);
   }
 
   @Override
@@ -279,6 +287,7 @@ public class InsertStmt extends StatementBase {
     mentionedColumns_.clear();
     primaryKeyExprs_.clear();
     writeId_ = -1;
+    kuduTxnToken_ = null;
   }
 
   @Override
@@ -452,7 +461,7 @@ public class InsertStmt extends StatementBase {
     }
 
     // We do not support (in|up)serting into views and iceberg table
-    if (table_ instanceof FeView) {
+    if (table_ instanceof FeView || table_ instanceof MaterializedViewHdfsTable) {
       throw new AnalysisException(
           String.format("Impala does not support %sing into views: %s", getOpName(),
               table_.getFullName()));
@@ -642,7 +651,7 @@ public class InsertStmt extends StatementBase {
   /**
    * Returns true if all the partition key values of the target table are static.
    */
-  private boolean isStaticPartitionTarget() {
+  public boolean isStaticPartitionTarget() {
     if (partitionKeyValues_ == null) return false;
     for (PartitionKeyValue pkv : partitionKeyValues_) {
       if (pkv.isDynamic()) return false;
@@ -650,6 +659,10 @@ public class InsertStmt extends StatementBase {
     return true;
   }
 
+  public List<PartitionKeyValue> getPartitionKeyValues() {
+    return partitionKeyValues_;
+  }
+
   private boolean isIcebergTarget() {
     return table_ instanceof FeIcebergTable;
   }
@@ -869,9 +882,14 @@ public class InsertStmt extends StatementBase {
       }
     }
 
-    if (isIcebergTarget()) {
-      Preconditions.checkState(
-          partitionKeyExprs_.size() == icebergPartSpec.getIcebergPartitionFieldsSize());
+    if (isIcebergTarget() && icebergPartSpec.hasPartitionFields()) {
+      int parts = 0;
+      for (IcebergPartitionField pField : icebergPartSpec.getIcebergPartitionFields()) {
+        if (pField.getTransformType() != TIcebergPartitionTransformType.VOID) {
+          ++parts;
+        }
+      }
+      Preconditions.checkState(partitionKeyExprs_.size() == parts);
     }
     else if (isKuduTable) {
       Preconditions.checkState(
@@ -952,6 +970,7 @@ public class InsertStmt extends StatementBase {
       IcebergPartitionSpec icebergPartSpec) throws AnalysisException {
     if (!icebergPartSpec.hasPartitionFields()) return;
     for (IcebergPartitionField partField : icebergPartSpec.getIcebergPartitionFields()) {
+      if (partField.getTransformType() == TIcebergPartitionTransformType.VOID) continue;
       for (int i = 0; i < selectListExprs.size(); ++i) {
         IcebergColumn targetColumn = (IcebergColumn)selectExprTargetColumns.get(i);
         if (targetColumn.getFieldId() != partField.getSourceId()) continue;
@@ -1107,8 +1126,13 @@ public class InsertStmt extends StatementBase {
   public TableName getTargetTableName() { return targetTableName_; }
   public FeTable getTargetTable() { return table_; }
   public void setTargetTable(FeTable table) { this.table_ = table; }
+  public boolean isTargetTableKuduTable() { return (table_ instanceof FeKuduTable); }
   public void setMaxTableSinks(int maxTableSinks) { this.maxTableSinks_ = maxTableSinks; }
   public void setWriteId(long writeId) { this.writeId_ = writeId; }
+  public void setKuduTransactionToken(byte[] kuduTxnToken) {
+    Preconditions.checkNotNull(kuduTxnToken);
+    kuduTxnToken_ = java.nio.ByteBuffer.wrap(kuduTxnToken.clone());
+  }
   public boolean isOverwrite() { return overwrite_; }
   public TSortingOrder getSortingOrder() { return sortingOrder_; }
 
@@ -1125,6 +1149,9 @@ public class InsertStmt extends StatementBase {
   public List<Expr> getPrimaryKeyExprs() { return primaryKeyExprs_; }
   public List<Expr> getSortExprs() { return sortExprs_; }
   public long getWriteId() { return writeId_; }
+  public byte[] getKuduTransactionToken() {
+    return kuduTxnToken_ == null ? null : kuduTxnToken_.array();
+  }
 
   // Clustering is enabled by default. If the table has a 'sort.columns' property and the
   // query has a 'noclustered' hint, we issue a warning during analysis and ignore the
@@ -1146,7 +1173,7 @@ public class InsertStmt extends StatementBase {
     return TableSink.create(table_, isUpsert_ ? TableSink.Op.UPSERT : TableSink.Op.INSERT,
         partitionKeyExprs_, resultExprs_, mentionedColumns_, overwrite_,
         requiresClustering(), new Pair<>(sortColumns_, sortingOrder_), writeId_,
-        maxTableSinks_);
+        kuduTxnToken_, maxTableSinks_);
   }
 
   /**
