@@ -131,6 +131,7 @@ public class Path {
 
   // Registered table alias that this path is rooted at, if any.
   // Null if the path is rooted at a catalog table/view.
+  // Note that this is also set for STAR path of "v.*" when "v" is a catalog table/view.
   private final TupleDescriptor rootDesc_;
 
   // Catalog table that this resolved path is rooted at, if any.
@@ -142,7 +143,9 @@ public class Path {
   private final Path rootPath_;
 
   // List of matched types and field positions set during resolution. The matched
-  // types/positions describe the physical path through the schema tree.
+  // types/positions describe the physical path through the schema tree of a catalog
+  // table/view. Empty if the path corresponds to a catalog table/view, e.g. when it's a
+  // TABLE_REF or when it's a STAR on a table/view.
   private final List<Type> matchedTypes_ = new ArrayList<>();
   private final List<Integer> matchedPositions_ = new ArrayList<>();
 
@@ -465,6 +468,26 @@ public class Path {
     return absolutePath_;
   }
 
+  /**
+   * If 'prefix' is a prefix of this path, returns a list with the elements of this path
+   * that would remain after removing the prefix; otherwise returns null.
+   *
+   * Uses 'getFullyQualifiedRawPath()' to obtain the elements
+   * of the paths.
+   */
+  public List<String> getRawPathWithoutPrefix(Path prefix) {
+    final List<String> prefixPath = prefix.getFullyQualifiedRawPath();
+    final List<String> thisPath = this.getFullyQualifiedRawPath();
+    return getRawPathWithoutPrefix(thisPath, prefixPath);
+  }
+
+  public static List<String> getRawPathWithoutPrefix(List<String> path,
+      List<String> prefix) {
+    if (prefix.size() > path.size()) return null;
+    if (!prefix.equals(path.subList(0, prefix.size()))) return null;
+    return path.subList(prefix.size(), path.size());
+  }
+
   /**
    * Converts table schema path to file schema path. Well, it's actually somewhere between
    * the two because the first column is offsetted with the number of partitions.
