@@ -19,16 +19,25 @@ package org.apache.impala.analysis;
 
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.Comparator;
 import java.util.Iterator;
+import java.util.LinkedHashMap;
+import java.util.HashSet;
 import java.util.List;
+import java.util.Map;
 import java.util.Set;
+import java.util.stream.Stream;
+import java.util.stream.Collectors;
 
 import org.apache.impala.analysis.Path.PathType;
+import org.apache.impala.analysis.TableRef.ZippingUnnestType;
 import org.apache.impala.authorization.Privilege;
+import org.apache.impala.catalog.ArrayType;
 import org.apache.impala.catalog.Column;
 import org.apache.impala.catalog.FeKuduTable;
 import org.apache.impala.catalog.FeTable;
 import org.apache.impala.catalog.FeView;
+import org.apache.impala.catalog.MapType;
 import org.apache.impala.catalog.StructField;
 import org.apache.impala.catalog.StructType;
 import org.apache.impala.catalog.TableLoadingException;
@@ -145,6 +154,8 @@ public class SelectStmt extends QueryStmt {
   public boolean hasHavingClause() { return havingClause_ != null; }
   public ExprSubstitutionMap getBaseTblSmap() { return baseTblSmap_; }
 
+  public void addToFromClause(TableRef ref) { fromClause_.add(ref); }
+
   /**
    * A simple limit statement has a limit but no order-by,
    * group-by, aggregates or analytic functions. Joins are
@@ -277,12 +288,16 @@ public class SelectStmt extends QueryStmt {
       // Start out with table refs to establish aliases.
       fromClause_.analyze(analyzer_);
 
+      registerStructSlotRefPathsWithAnalyzer();
+
       analyzeSelectClause();
       verifyResultExprs();
       registerViewColumnPrivileges();
       analyzeWhereClause();
       createSortInfo(analyzer_);
 
+      setZippingUnnestSlotRefsFromViews();
+
       // Analyze aggregation-relevant components of the select block (Group By
       // clause, select list, Order By clause), substitute AVG with SUM/COUNT,
       // create the AggregationInfo, including the agg output tuple, and transform
@@ -315,6 +330,76 @@ public class SelectStmt extends QueryStmt {
       analyzer_.setSimpleLimitStatus(checkSimpleLimitStmt());
     }
 
+    /* Register paths that point to structs. This is to unify expressions and allow
+     * embedded (struct member) expressions to share the tuple memory of their enclosing
+     * expressions.
+     */
+    private void registerStructSlotRefPathsWithAnalyzer() throws AnalysisException {
+      Preconditions.checkNotNull(selectList_);
+      // Note: if we in the future include complex types in star expressions, we will have
+      // to expand star expressions here.
+      Stream<Expr> selectListExprs = selectList_.getItems().stream()
+        .filter(elem -> !elem.isStar())
+        .map(elem -> elem.getExpr());
+      Stream<Expr> nonSelectListExprs = collectExprsOutsideSelectList();
+
+      Stream<Expr> exprs = Stream.concat(selectListExprs, nonSelectListExprs);
+
+      HashSet<SlotRef> slotRefs = new HashSet<>();
+      exprs.forEach(expr -> expr.collect(SlotRef.class, slotRefs));
+
+      List<Path> paths = slotRefs.stream().map(this::slotRefToResolvedPath)
+          .filter(path -> path != null)
+          .filter(path -> path.destType().isStructType())
+          .collect(Collectors.toList());
+      // Sort paths by length in descending order so that structs that contain other
+      // structs come before their children.
+      Collections.sort(paths,
+          Comparator.<Path>comparingInt(path -> path.getMatchedTypes().size())
+          .reversed());
+      for (Path p : paths) {
+        analyzer_.registerSlotRef(p);
+      }
+    }
+
+    private Stream<Expr> collectExprsOutsideSelectList() {
+      Stream<Expr> res = Stream.empty();
+      if (whereClause_ != null) {
+        res = Stream.concat(res, Stream.of(whereClause_));
+      }
+      if (havingClause_ != null) {
+        res = Stream.concat(res, Stream.of(havingClause_));
+      }
+      if (groupingExprs_ != null) {
+        res = Stream.concat(res, groupingExprs_.stream());
+      }
+      if (sortInfo_ != null) {
+        res = Stream.concat(res, sortInfo_.getSortExprs().stream());
+      }
+      if (analyticInfo_ != null) {
+        res = Stream.concat(res,
+            analyticInfo_.getAnalyticExprs().stream()
+            .map(analyticExpr -> (Expr) analyticExpr));
+      }
+      return res;
+    }
+
+    private Path slotRefToResolvedPath(SlotRef slotRef) {
+      try {
+        Path resolvedPath = analyzer_.resolvePathWithMasking(slotRef.getRawPath(),
+            PathType.SLOT_REF);
+        return resolvedPath;
+      } catch (TableLoadingException e) {
+        // Should never happen because we only check registered table aliases.
+        Preconditions.checkState(false);
+        return null;
+      } catch (AnalysisException e) {
+        // Return null if analysis did not succeed. This means this path will not be
+        // registered here.
+        return null;
+      }
+    }
+
     private void analyzeSelectClause() throws AnalysisException {
       // Generate !empty() predicates to filter out empty collections.
       // Skip this step when analyzing a WITH-clause because CollectionTableRefs
@@ -326,6 +411,9 @@ public class SelectStmt extends QueryStmt {
       selectList_.analyzePlanHints(analyzer_);
 
       // populate resultExprs_, aliasSmap_, and colLabels_
+      // This additional map is used for performance reasons and not for finding
+      // ambiguous alias.
+      Map<String, Expr> existingAliasExprs = new LinkedHashMap<>();
       for (int i = 0; i < selectList_.getItems().size(); ++i) {
         SelectListItem item = selectList_.getItems().get(i);
         if (item.isStar()) {
@@ -361,16 +449,21 @@ public class SelectStmt extends QueryStmt {
           resultExprs_.add(item.getExpr());
           String label = item.toColumnLabel(i, analyzer_.useHiveColLabels());
           SlotRef aliasRef = new SlotRef(label);
-          Expr existingAliasExpr = aliasSmap_.get(aliasRef);
+          Expr existingAliasExpr = existingAliasExprs.get(label);
           if (existingAliasExpr != null && !existingAliasExpr.equals(item.getExpr())) {
             // If we have already seen this alias, it refers to more than one column and
             // therefore is ambiguous.
             ambiguousAliasList_.add(aliasRef);
+          } else {
+            existingAliasExprs.put(label, item.getExpr());
           }
           aliasSmap_.put(aliasRef, item.getExpr().clone());
           colLabels_.add(label);
         }
       }
+      if (LOG.isTraceEnabled()) {
+        LOG.trace("Analyzed select clause aliasSmap={}", aliasSmap_.debugString());
+      }
     }
 
     private void verifyResultExprs() throws AnalysisException {
@@ -380,19 +473,28 @@ public class SelectStmt extends QueryStmt {
         throw new AnalysisException("The star exprs expanded to an empty select list " +
             "because the referenced tables only have complex-typed columns.\n" +
             "Star exprs only expand to scalar-typed columns because " +
-            "complex-typed exprs " +
-            "are currently not supported in the select list.\n" +
+            "currently not all complex-typed exprs " +
+            "are supported in the select list.\n" +
             "Affected select statement:\n" + toSql());
       }
 
       for (Expr expr: resultExprs_) {
-        // Complex types are currently not supported in the select list because
-        // we'd need to serialize them in a meaningful way.
-        if (expr.getType().isComplexType()) {
+        if (expr.getType().isArrayType()) {
+          ArrayType arrayType = (ArrayType) expr.getType();
+          if (!arrayType.getItemType().isSupported()) {
+            throw new AnalysisException("Unsupported type '" +
+                expr.getType().toSql() + "' in '" + expr.toSql() + "'.");
+          }
+        } else if (expr.getType().isMapType()) {
           throw new AnalysisException(String.format(
-              "Expr '%s' in select list returns a complex type '%s'.\n" +
-              "Only scalar types are allowed in the select list.",
+              "Expr '%s' in select list returns a map type '%s'.\n" +
+              "Map type is not allowed in the select list.",
               expr.toSql(), expr.getType().toSql()));
+        } else if (expr.getType().isStructType()) {
+          if (!analyzer_.getQueryCtx().client_request.query_options.disable_codegen) {
+            throw new AnalysisException("Struct type in select list is not allowed " +
+                "when Codegen is ON. You might want to set DISABLE_CODEGEN=true");
+          }
         }
         if (!expr.getType().isSupported()) {
           throw new AnalysisException("Unsupported type '"
@@ -427,9 +529,80 @@ public class SelectStmt extends QueryStmt {
         throw new AnalysisException(
             "WHERE clause must not contain analytic expressions: " + e.toSql());
       }
+
+      verifyZippingUnnestSlots();
+
       analyzer_.registerConjuncts(whereClause_, false);
     }
 
+    /**
+     * Don't allow a WHERE conjunct on an array item that is part of a zipping unnest.
+     * In case there is only one zipping unnested array this restriction is not needed
+     * as the UNNEST node has to handle a single array and it's safe to do the filtering
+     * in the scanner.
+     */
+    private void verifyZippingUnnestSlots() throws AnalysisException {
+      if (analyzer_.getNumZippingUnnests() <= 1) return;
+      List<TupleId> zippingUnnestTupleIds = Lists.newArrayList(
+          analyzer_.getZippingUnnestTupleIds());
+
+      List<SlotRef> slotRefsInWhereClause = new ArrayList<>();
+      whereClause_.collect(SlotRef.class, slotRefsInWhereClause);
+      for (SlotRef slotRef : slotRefsInWhereClause) {
+        if (slotRef.isBoundByTupleIds(zippingUnnestTupleIds)) {
+          throw new AnalysisException("Not allowed to add a filter on an unnested " +
+              "array under the same select statement: " + slotRef.toSql());
+        }
+      }
+    }
+
+    /**
+     * When zipping unnest is performed using the SQL standard compliant syntax (where
+     * the unnest is in the FROM clause) the SlotRefs used for the zipping unnest are not
+     * UnnestExprs as with the other approach (where zipping unnest is in the select list)
+     * but regular SlotRefs. As a result they have to be marked so that later on anything
+     * specific for zipping unnest could be executed for them.
+     * This function identifies the SlotRefs that are for zipping unnesting and sets a
+     * flag for them. Note, only marks the SlotRefs that are originated form a view.
+     */
+    private void setZippingUnnestSlotRefsFromViews() {
+      for (TableRef tblRef : fromClause_.getTableRefs()) {
+        if (!tblRef.isFromClauseZippingUnnest()) continue;
+        Preconditions.checkState(tblRef instanceof CollectionTableRef);
+        ExprSubstitutionMap exprSubMap = getBaseTableSMapFromTableRef(tblRef);
+        if (exprSubMap == null) continue;
+
+        for (SelectListItem item : selectList_.getItems()) {
+          if (item.isStar()) continue;
+          Expr itemExpr = item.getExpr();
+          List<SlotRef> slotRefs = new ArrayList<>();
+          itemExpr.collect(SlotRef.class, slotRefs);
+          for (SlotRef slotRef : slotRefs) {
+            Expr subbedExpr = exprSubMap.get(slotRef);
+            if (subbedExpr == null || !(subbedExpr instanceof SlotRef)) continue;
+            SlotRef subbedSlotRef = (SlotRef)subbedExpr;
+            CollectionTableRef collTblRef = (CollectionTableRef)tblRef;
+            SlotRef collectionSlotRef = (SlotRef)collTblRef.getCollectionExpr();
+            // Check if 'slotRef' is originated from 'collectionSlotRef'.
+            if (subbedSlotRef.getDesc().getParent().getId() ==
+                collectionSlotRef.getDesc().getItemTupleDesc().getId()) {
+              slotRef.setIsZippingUnnest(true);
+            }
+          }
+        }
+      }
+    }
+
+    /**
+     * If 'tblRef' is originated from a view then returns the baseTblSmap from the view.
+     * Returns false otherwise.
+     */
+    private ExprSubstitutionMap getBaseTableSMapFromTableRef(TableRef tblRef) {
+      if (tblRef.getResolvedPath().getRootDesc() == null) return null;
+      if (tblRef.getResolvedPath().getRootDesc().getSourceView() == null) return null;
+      return tblRef.getResolvedPath().getRootDesc().getSourceView().getBaseTblSmap();
+    }
+
     /**
      * Generates and registers !empty() predicates to filter out empty
      * collections directly in the parent scan of collection table refs. This is
@@ -479,6 +652,11 @@ public class SelectStmt extends QueryStmt {
         // Do not generate a predicate if the parent tuple is outer joined.
         if (analyzer_.isOuterJoined(ref.getResolvedPath().getRootDesc().getId()))
           continue;
+        // Don't push down the "is not empty" predicate for zipping unnests if there are
+        // multiple zipping unnests in the FROM clause.
+        if (tblRef.isZippingUnnest() && analyzer_.getNumZippingUnnests() > 1) {
+          continue;
+        }
         IsNotEmptyPredicate isNotEmptyPred =
             new IsNotEmptyPredicate(ref.getCollectionExpr().clone());
         isNotEmptyPred.analyze(analyzer_);
@@ -553,8 +731,7 @@ public class SelectStmt extends QueryStmt {
 
     /**
      * Expand "path.*" from a resolved path, ignoring complex-typed fields
-     * because those are currently illegal in any select list (even for
-     * inline views, etc.)
+     * for backwards compatibility.
      */
     private void expandStar(Path resolvedPath)
         throws AnalysisException {
@@ -628,7 +805,7 @@ public class SelectStmt extends QueryStmt {
       if (resolvedPath.getMatchedTypes().isEmpty()) {
         Preconditions.checkState(!slotDesc.getType().isComplexType(),
             "Star expansion should only introduce scalar columns");
-        analyzer_.registerScalarColumnForMasking(slotDesc);
+        analyzer_.registerColumnForMasking(slotDesc);
       }
       resultExprs_.add(slotRef);
       colLabels_.add(relRawPath[relRawPath.length - 1]);
@@ -792,18 +969,6 @@ public class SelectStmt extends QueryStmt {
                   + groupingExprsCopy_.get(i).toSql());
         }
       }
-      // initialize groupingExprs_ with the analyzed version
-      // use the original ordinal if the analyzed expr is a INT literal
-      List<Expr> groupingExprs = new ArrayList<>();
-      for (int i = 0; i < groupingExprsCopy_.size(); ++i) {
-        Expr expr = groupingExprsCopy_.get(i);
-        if (expr instanceof NumericLiteral && Expr.IS_INT_LITERAL.apply(expr)) {
-          groupingExprs.add(groupingExprs_.get(i).clone());
-        } else {
-          groupingExprs.add(expr);
-        }
-      }
-      groupingExprs_ = groupingExprs;
 
       if (groupByClause_ != null && groupByClause_.hasGroupingSets()) {
         groupByClause_.analyzeGroupingSets(groupingExprsCopy_);
