@@ -46,7 +46,7 @@ public class SlotDescriptor {
   private Path path_;
   private Type type_;
 
-  // Tuple descriptor for collection items. Only set if type_ is an array or map.
+  // Tuple descriptor for nested items. Set if type_ is an array, map or struct.
   private TupleDescriptor itemTupleDesc_;
 
   // for SlotRef.toSql() in the absence of a path
@@ -60,6 +60,9 @@ public class SlotDescriptor {
   // (and physical layout parameters are invalid)
   private boolean isMaterialized_ = false;
 
+  // If true, then computeMemLayout() should be recursively called for itemTupleDesc_.
+  private boolean materializeRecursively_ = false;
+
   // if false, this slot cannot be NULL
   // Note: it is still possible that a SlotRef pointing to this descriptor could have a
   // NULL value if the entire tuple is NULL, for example as the result of an outer join.
@@ -89,6 +92,7 @@ public class SlotDescriptor {
     parent_ = parent;
     type_ = src.type_;
     itemTupleDesc_ = src.itemTupleDesc_;
+    if (itemTupleDesc_ != null) itemTupleDesc_.setParentSlotDesc(this);
     path_ = src.path_;
     label_ = src.label_;
     sourceExprs_ = src.sourceExprs_;
@@ -120,12 +124,24 @@ public class SlotDescriptor {
         itemTupleDesc_ == null, "Item tuple descriptor already set.");
     itemTupleDesc_ = t;
   }
+  public void clearItemTupleDesc() {
+    Preconditions.checkState(itemTupleDesc_ != null);
+    itemTupleDesc_ = null;
+  }
   public boolean isMaterialized() { return isMaterialized_; }
   public void setIsMaterialized(boolean value) {
     if (isMaterialized_ == value) return;
     isMaterialized_ = value;
     LOG.trace("Mark slot(sid={}) of tuple(tid={}) as {}materialized",
         id_, parent_.getId(), isMaterialized_ ? "" : "non-");
+    if (materializeRecursively_) {
+      Preconditions.checkState(itemTupleDesc_ != null);
+      itemTupleDesc_.materializeSlots();
+    }
+  }
+  public boolean isMaterializedRecursively() { return materializeRecursively_; }
+  public void setIsMaterializedRecursively(boolean value) {
+    materializeRecursively_ = value;
   }
   public boolean getIsNullable() { return isNullable_; }
   public void setIsNullable(boolean value) { isNullable_ = value; }
@@ -145,7 +161,8 @@ public class SlotDescriptor {
   public void setPath(Path path) {
     Preconditions.checkNotNull(path);
     Preconditions.checkState(path.isRootedAtTuple());
-    Preconditions.checkState(path.getRootDesc() == parent_);
+    Preconditions.checkState(path.getRootDesc() == parent_ ||
+        parent_.getType().isStructType());
     path_ = path;
     type_ = path_.destType();
     label_ = Joiner.on(".").join(path.getRawPath());
@@ -173,6 +190,66 @@ public class SlotDescriptor {
     return stats_;
   }
 
+  private void getEnclosingStructSlotAndTupleDescs(List<SlotDescriptor> slotDescs,
+      List<TupleDescriptor> tupleDescs) {
+    TupleDescriptor tupleDesc = getParent();
+    while (tupleDesc != null) {
+      if (tupleDescs != null) tupleDescs.add(tupleDesc);
+
+      final SlotDescriptor parentStructSlotDesc = tupleDesc.getParentSlotDesc();
+      if (parentStructSlotDesc != null && slotDescs != null) {
+        slotDescs.add(parentStructSlotDesc);
+      }
+
+      tupleDesc = parentStructSlotDesc != null ? parentStructSlotDesc.getParent() : null;
+    }
+  }
+
+  /**
+   * Returns the slot descs of the structs that contain this slot desc, recursively.
+   * For an example struct 'outer: <middle: <inner: <i: int>>>', called for the slot desc
+   * of 'i', the returned list will contain the slot descs of 'inner', 'middle' and
+   * 'outer'.
+   */
+  public List<SlotDescriptor> getEnclosingStructSlotDescs() {
+    List<SlotDescriptor> result = new ArrayList<>();
+    getEnclosingStructSlotAndTupleDescs(result, null);
+    return result;
+  }
+
+  /**
+   * Returns the tuple descs enclosing this slot desc, recursively.
+   * For an example struct 'outer: <middle: <inner: <i: int>>>', called for the slot desc
+   * of 'i', the returned list will contain the 'itemTupleDesc_'s of 'inner', 'middle'
+   * and 'outer' as well as the tuple desc of the main tuple (the 'parent_' of the slot
+   * desc of 'outer').
+   */
+  public List<TupleDescriptor> getEnclosingTupleDescs() {
+    List<TupleDescriptor> result = new ArrayList<>();
+    getEnclosingStructSlotAndTupleDescs(null, result);
+    return result;
+  }
+
+  /**
+   * Returns the size of the slot without null indicators.
+   *
+   * Takes materialisation into account: returns 0 for non-materialised slots. This is
+   * most relevant for structs as it is possible that only a subset of the fields are
+   * materialised and the size of the struct varies according to this.
+   */
+  public int getMaterializedSlotSize() {
+    if (!isMaterialized()) return 0;
+    if (!getType().isStructType()) return getType().getSlotSize();
+
+    Preconditions.checkNotNull(itemTupleDesc_);
+    int size = 0;
+    for (SlotDescriptor d : itemTupleDesc_.getSlots()) {
+      size += d.getMaterializedSlotSize();
+    }
+
+    return size;
+  }
+
   /**
    * Checks if this descriptor describes  an array "pos" pseudo-column.
    *
@@ -239,8 +316,8 @@ public class SlotDescriptor {
     Preconditions.checkState(path_.isResolved());
 
     List<Integer> materializedPath = Lists.newArrayList(path_.getAbsolutePath());
-    // For scalar types, the materialized path is the same as path_
-    if (type_.isScalarType()) return materializedPath;
+    // For scalar types and structs the materialized path is the same as path_
+    if (type_.isScalarType() || type_.isStructType()) return materializedPath;
     Preconditions.checkState(type_.isCollectionType());
     Preconditions.checkState(path_.getFirstCollectionIndex() != -1);
     // Truncate materializedPath after first collection element
@@ -311,12 +388,15 @@ public class SlotDescriptor {
   public String debugString() {
     String pathStr = (path_ == null) ? "null" : path_.toString();
     String typeStr = (type_ == null ? "null" : type_.toString());
+    String parentTupleId = parent_ == null ?
+        "null" : String.valueOf(parent_.getId().asInt());
     return MoreObjects.toStringHelper(this)
         .add("id", id_.asInt())
         .add("path", pathStr)
         .add("label", label_)
         .add("type", typeStr)
         .add("materialized", isMaterialized_)
+        .add("materializeRecursively", materializeRecursively_)
         .add("byteSize", byteSize_)
         .add("byteOffset", byteOffset_)
         .add("nullable", isNullable_)
@@ -324,6 +404,8 @@ public class SlotDescriptor {
         .add("nullIndicatorBit", nullIndicatorBit_)
         .add("slotIdx", slotIdx_)
         .add("stats", stats_)
+        .add("itemTupleDesc", itemTupleDesc_)
+        .add("parent_tuple_id", parentTupleId)
         .toString();
   }
 
