@@ -21,15 +21,18 @@ import static org.apache.impala.analysis.ToSqlOptions.DEFAULT;
 
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import org.apache.impala.analysis.TimeTravelSpec.Kind;
 import org.apache.impala.authorization.Privilege;
 import org.apache.impala.catalog.Column;
 import org.apache.impala.catalog.FeFsTable;
+import org.apache.impala.catalog.FeIcebergTable;
 import org.apache.impala.catalog.FeTable;
 import org.apache.impala.common.AnalysisException;
 import org.apache.impala.planner.JoinNode.DistributionMode;
@@ -50,7 +53,7 @@ import com.google.common.collect.Lists;
  * The analysis of table refs follows a two-step process:
  *
  * 1. Resolution: A table ref's path is resolved and then the generic TableRef is
- * replaced by a concrete table ref (a BaseTableRef, CollectionTabeRef or ViewRef)
+ * replaced by a concrete table ref (a BaseTableRef, CollectionTableRef or ViewRef)
  * in the originating stmt and that is given the resolved path. This step is driven by
  * Analyzer.resolveTableRef().
  *
@@ -108,6 +111,15 @@ public class TableRef extends StmtNode {
   // TODO: Move join-specific members out of TableRef.
   private DistributionMode distrMode_ = DistributionMode.NONE;
 
+  public enum ZippingUnnestType {
+    NONE,
+    FROM_CLAUSE_ZIPPING_UNNEST,
+    SELECT_LIST_ZIPPING_UNNEST
+  }
+
+  // Indicates if this TableRef is for the purpose of zipping unnest for arrays.
+  protected ZippingUnnestType zippingUnnestType_ = ZippingUnnestType.NONE;
+
   /////////////////////////////////////////
   // BEGIN: Members that need to be reset()
 
@@ -144,8 +156,12 @@ public class TableRef extends StmtNode {
   // columns via the view.
   protected boolean exposeNestedColumnsByTableMaskView_ = false;
 
-  // Scalar columns referenced in the query. Used in resolving column mask.
-  protected Map<String, Column> scalarColumns_ = new LinkedHashMap<>();
+  // Columns referenced in the query. Used in resolving column mask.
+  protected Map<String, Column> columns_ = new HashMap<>();
+
+  // Time travel spec of this table ref. It contains information specified in the
+  // FOR SYSTEM_TIME AS OF <timestamp> or FOR SYSTEM_TIME AS OF <version> clause.
+  protected TimeTravelSpec timeTravelSpec_;
 
   // END: Members that need to be reset()
   /////////////////////////////////////////
@@ -170,20 +186,25 @@ public class TableRef extends StmtNode {
   }
 
   public TableRef(List<String> path, String alias, TableSampleClause tableSample) {
-    this(path, alias, tableSample, Privilege.SELECT, false);
+    this(path, alias, tableSample, null);
+  }
+
+  public TableRef(List<String> path, String alias, TableSampleClause tableSample,
+      TimeTravelSpec timeTravel) {
+    this(path, alias, tableSample, timeTravel, Privilege.SELECT, false);
   }
 
   public TableRef(List<String> path, String alias, Privilege priv) {
-    this(path, alias, null, priv, false);
+    this(path, alias, null, null, priv, false);
   }
 
   public TableRef(List<String> path, String alias, Privilege priv,
       boolean requireGrantOption) {
-    this(path, alias, null, priv, requireGrantOption);
+    this(path, alias, null, null, priv, requireGrantOption);
   }
 
   public TableRef(List<String> path, String alias, TableSampleClause sampleParams,
-      Privilege priv, boolean requireGrantOption) {
+      TimeTravelSpec timeTravel, Privilege priv, boolean requireGrantOption) {
     rawPath_ = path;
     if (alias != null) {
       aliases_ = new String[] { alias.toLowerCase() };
@@ -198,6 +219,7 @@ public class TableRef extends StmtNode {
     replicaPreference_ = null;
     randomReplica_ = false;
     convertLimitToSampleHintPercent_ = -1;
+    timeTravelSpec_ = timeTravel;
   }
 
   /**
@@ -209,6 +231,8 @@ public class TableRef extends StmtNode {
     aliases_ = other.aliases_;
     hasExplicitAlias_ = other.hasExplicitAlias_;
     sampleParams_ = other.sampleParams_;
+    timeTravelSpec_ = other.timeTravelSpec_ != null ?
+                      other.timeTravelSpec_.clone() : null;
     priv_ = other.priv_;
     requireGrantOption_ = other.requireGrantOption_;
     joinOp_ = other.joinOp_;
@@ -231,8 +255,9 @@ public class TableRef extends StmtNode {
     correlatedTupleIds_ = Lists.newArrayList(other.correlatedTupleIds_);
     desc_ = other.desc_;
     exposeNestedColumnsByTableMaskView_ = other.exposeNestedColumnsByTableMaskView_;
-    scalarColumns_ = new LinkedHashMap<>(other.scalarColumns_);
+    columns_ = new LinkedHashMap<>(other.columns_);
     isHidden_ = other.isHidden_;
+    zippingUnnestType_ = other.zippingUnnestType_;
   }
 
   @Override
@@ -290,6 +315,8 @@ public class TableRef extends StmtNode {
    */
   public boolean isRelative() { return false; }
 
+  public boolean isCollectionInSelectList() { return false; }
+
   /**
    * Indicates if this TableRef directly or indirectly references another TableRef from
    * an outer query block.
@@ -331,6 +358,7 @@ public class TableRef extends StmtNode {
     return resolvedPath_.getRootTable();
   }
   public TableSampleClause getSampleParams() { return sampleParams_; }
+  public TimeTravelSpec getTimeTravelSpec() { return timeTravelSpec_; }
   public Privilege getPrivilege() { return priv_; }
   public boolean requireGrantOption() { return requireGrantOption_; }
   public List<PlanHint> getJoinHints() { return joinHints_; }
@@ -376,6 +404,16 @@ public class TableRef extends StmtNode {
   public boolean isAnalyzed() { return isAnalyzed_; }
   public boolean isResolved() { return !getClass().equals(TableRef.class); }
 
+  public boolean isFromClauseZippingUnnest() {
+    return zippingUnnestType_ == ZippingUnnestType.FROM_CLAUSE_ZIPPING_UNNEST;
+  }
+  public boolean isZippingUnnest() {
+    return zippingUnnestType_ != ZippingUnnestType.NONE;
+  }
+  public ZippingUnnestType getZippingUnnestType() { return zippingUnnestType_; }
+  public void setZippingUnnestType(ZippingUnnestType t) { zippingUnnestType_ = t; }
+
+
   /**
    * This method should only be called after the TableRef has been analyzed.
    */
@@ -431,6 +469,20 @@ public class TableRef extends StmtNode {
     }
   }
 
+  protected void analyzeTimeTravel(Analyzer analyzer) throws AnalysisException {
+    if (timeTravelSpec_ != null) {
+      if (!(getTable() instanceof FeIcebergTable)) {
+        throw new AnalysisException(String.format(
+            "FOR %s AS OF clause is only supported for Iceberg tables. " +
+            "%s is not an Iceberg table.",
+            timeTravelSpec_.getKind() == Kind.TIME_AS_OF ? "SYSTEM_TIME" :
+                                                           "SYSTEM_VERSION",
+            getTable().getFullName()));
+      }
+      timeTravelSpec_.analyze(analyzer);
+    }
+  }
+
   protected void analyzeHints(Analyzer analyzer) throws AnalysisException {
     // We prefer adding warnings over throwing exceptions here to maintain view
     // compatibility with Hive.
@@ -677,6 +729,7 @@ public class TableRef extends StmtNode {
 
   @Override
   public String toSql(ToSqlOptions options) {
+    if (isZippingUnnest()) return "";
     if (joinOp_ == null) {
       // prepend "," if we're part of a sequence of table refs w/o an
       // explicit JOIN clause
@@ -717,16 +770,43 @@ public class TableRef extends StmtNode {
     allMaterializedTupleIds_.clear();
     correlatedTupleIds_.clear();
     desc_ = null;
+    if (timeTravelSpec_ != null) timeTravelSpec_.reset();
   }
 
   public boolean isTableMaskingView() { return false; }
 
-  public void registerScalarColumn(Column column) {
-    scalarColumns_.put(column.getName(), column);
+  public void registerColumn(Column column) {
+    columns_.put(column.getName(), column);
   }
 
-  public List<Column> getScalarColumns() {
-    return new ArrayList<>(scalarColumns_.values());
+  /**
+   * @return an unmodifiable list of all columns, but with partition columns at the end of
+   * the list rather than the beginning. This is equivalent to the order in which Hive
+   * enumerates columns.
+   */
+  public List<Column> getColumnsInHiveOrder() {
+    return getTable().getColumnsInHiveOrder();
+  }
+
+  public List<Column> getSelectedColumnsInHiveOrder() {
+    // Map from column name to the Column object (null if not selected).
+    // Use LinkedHashMap to preserve the order.
+    Map<String, Column> colSelection = new LinkedHashMap<>();
+    for (Column c : getColumnsInHiveOrder()) {
+      colSelection.put(c.getName(), null);
+    }
+    // Update 'colSelection' with selected columns. Virtual columns will also be added.
+    for (String colName : columns_.keySet()) {
+      colSelection.put(colName, columns_.get(colName));
+    }
+    List<Column> res = new ArrayList<>();
+    for (Column c : colSelection.values()) {
+      if (c != null) res.add(c);
+    }
+    // Make sure not missing any columns
+    Preconditions.checkState(res.size() == columns_.size(),
+        "missing columns: " + res.size() + " != " + columns_.size());
+    return res;
   }
 
   void migratePropertiesTo(TableRef other) {
@@ -736,6 +816,7 @@ public class TableRef extends StmtNode {
     other.joinOp_ = joinOp_;
     other.joinHints_ = joinHints_;
     other.tableHints_ = tableHints_;
+    other.timeTravelSpec_ = timeTravelSpec_;
     // Clear properties. Don't clear aliases_ since it's still used in resolving slots
     // in the query block of 'other'.
     onClause_ = null;
@@ -743,5 +824,6 @@ public class TableRef extends StmtNode {
     joinOp_ = null;
     joinHints_ = new ArrayList<>();
     tableHints_ = new ArrayList<>();
+    timeTravelSpec_ = null;
   }
 }
