@@ -419,7 +419,7 @@ public class ToSqlUtils {
         properties.remove(StatsSetupConst.DO_NOT_UPDATE_STATS);
         properties.remove(IcebergTable.METADATA_LOCATION);
 
-        // Fill "PARTITION BY SPEC" part if the Iceberg table is partitioned.
+        // Fill "PARTITIONED BY SPEC" part if the Iceberg table is partitioned.
         FeIcebergTable feIcebergTable= (FeIcebergTable)table;
         if (!feIcebergTable.getPartitionSpecs().isEmpty()) {
           IcebergPartitionSpec latestPartitionSpec =
@@ -431,7 +431,8 @@ public class ToSqlUtils {
       }
 
       String inputFormat = msTable.getSd().getInputFormat();
-      format = HdfsFileFormat.fromHdfsInputFormatClass(inputFormat);
+      String serDeLib = msTable.getSd().getSerdeInfo().getSerializationLib();
+      format = HdfsFileFormat.fromHdfsInputFormatClass(inputFormat, serDeLib);
       compression = HdfsCompression.fromHdfsInputFormatClass(inputFormat);
       try {
         primaryKeySql = ((FeFsTable) table).getPrimaryKeyColumnNames();
@@ -503,7 +504,7 @@ public class ToSqlUtils {
           Joiner.on(", \n  ").join(sortProperties.first)));
     }
     if (icebergPartitions != null && !icebergPartitions.isEmpty()) {
-      sb.append("PARTITION BY SPEC\n");
+      sb.append("PARTITIONED BY SPEC\n");
       sb.append(icebergPartitions);
       sb.append("\n");
     }
