@@ -19,11 +19,13 @@ package org.apache.impala.authorization.ranger;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
+import com.google.common.collect.Lists;
 import org.apache.commons.lang.StringUtils;
 import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.impala.analysis.AnalysisContext.AnalysisResult;
 import org.apache.impala.authorization.Authorizable;
 import org.apache.impala.authorization.Authorizable.Type;
+import org.apache.impala.authorization.AuthorizableTable;
 import org.apache.impala.authorization.AuthorizationChecker;
 import org.apache.impala.authorization.AuthorizationConfig;
 import org.apache.impala.authorization.AuthorizationContext;
@@ -90,6 +92,7 @@ public class RangerAuthorizationChecker extends BaseAuthorizationChecker {
     RangerAuthorizationContext rangerAuthzCtx = (RangerAuthorizationContext) authzCtx;
     List<RangerAccessResourceImpl> resources = new ArrayList<>();
     Authorizable authorizable = request.getAuthorizable();
+    Privilege privilege = request.getPrivilege();
     switch (authorizable.getType()) {
       case SERVER:
         // Hive service definition does not have a concept of server. So we define
@@ -128,11 +131,11 @@ public class RangerAuthorizationChecker extends BaseAuthorizationChecker {
         // For example any column in foo.bar table:
         // access type: RangerPolicyEngine.ANY_ACCESS
         // resources: [server=server1, database=foo, table=bar]
-        if (request.getPrivilege() != Privilege.ANY ||
+        if (privilege != Privilege.ANY ||
           !DefaultAuthorizableFactory.ALL.equals(authorizable.getTableName())) {
           builder.table(authorizable.getTableName());
         }
-        if (request.getPrivilege() != Privilege.ANY ||
+        if (privilege != Privilege.ANY ||
           !DefaultAuthorizableFactory.ALL.equals(authorizable.getColumnName())) {
           builder.column(authorizable.getColumnName());
         }
@@ -156,15 +159,15 @@ public class RangerAuthorizationChecker extends BaseAuthorizationChecker {
     }
 
     for (RangerAccessResourceImpl resource: resources) {
-      if (request.getPrivilege() == Privilege.ANY) {
-        if (!authorizeResource(rangerAuthzCtx, resource, user, request.getPrivilege(),
-            ((RangerAuthorizationContext) authzCtx).getAuditHandler())) {
+      if (privilege == Privilege.ANY) {
+        if (!authorizeResource(rangerAuthzCtx, user, resource, authorizable, privilege,
+            rangerAuthzCtx.getAuditHandler())) {
           return false;
         }
       } else {
-        boolean authorized = request.getPrivilege().hasAnyOf() ?
-            authorizeAny(rangerAuthzCtx, resource, user, request.getPrivilege()) :
-            authorizeAll(rangerAuthzCtx, resource, user, request.getPrivilege());
+        boolean authorized = privilege.hasAnyOf() ?
+            authorizeAny(rangerAuthzCtx, resource, authorizable, user, privilege) :
+            authorizeAll(rangerAuthzCtx, resource, authorizable, user, privilege);
         if (!authorized) {
           return false;
         }
@@ -174,9 +177,14 @@ public class RangerAuthorizationChecker extends BaseAuthorizationChecker {
   }
 
   @Override
-  public void postAuthorize(AuthorizationContext authzCtx) {
+  public void postAuthorize(AuthorizationContext authzCtx, boolean authzOk) {
     Preconditions.checkArgument(authzCtx instanceof RangerAuthorizationContext);
-    super.postAuthorize(authzCtx);
+    super.postAuthorize(authzCtx, authzOk);
+    // Apply the deduplicated column masking events to update the List of all
+    // AuthzAuditEvent's only if the authorization is successful.
+    if (authzOk) {
+      ((RangerAuthorizationContext) authzCtx).applyDeduplicatedAuthzEvents();
+    }
     RangerBufferAuditHandler auditHandler =
         ((RangerAuthorizationContext) authzCtx).getAuditHandler();
     auditHandler.flush();
@@ -187,15 +195,18 @@ public class RangerAuthorizationChecker extends BaseAuthorizationChecker {
       List<PrivilegeRequest> privilegeRequests)
       throws AuthorizationException, InternalException {
     boolean isColumnMaskingEnabled = BackendConfig.INSTANCE.isColumnMaskingEnabled();
+    boolean isRowFilteringEnabled = BackendConfig.INSTANCE.isRowFilteringEnabled();
+    if (isColumnMaskingEnabled && isRowFilteringEnabled) return;
     for (PrivilegeRequest request : privilegeRequests) {
       if (!isColumnMaskingEnabled
           && request.getAuthorizable().getType() == Type.COLUMN) {
-        authorizeColumnMask(user,
+        throwIfColumnMaskingRequired(user,
             request.getAuthorizable().getDbName(),
             request.getAuthorizable().getTableName(),
             request.getAuthorizable().getColumnName());
-      } else if (request.getAuthorizable().getType() == Type.TABLE) {
-        authorizeRowFilter(user,
+      } else if (!isRowFilteringEnabled
+          && request.getAuthorizable().getType() == Type.TABLE) {
+        throwIfRowFilteringRequired(user,
             request.getAuthorizable().getDbName(),
             request.getAuthorizable().getTableName());
       }
@@ -215,8 +226,7 @@ public class RangerAuthorizationChecker extends BaseAuthorizationChecker {
 
   @Override
   public AuthorizationContext createAuthorizationContext(boolean doAudits,
-      String sqlStmt, TSessionState sessionState, Optional<EventSequence> timeline)
-      throws InternalException {
+      String sqlStmt, TSessionState sessionState, Optional<EventSequence> timeline) {
     RangerAuthorizationContext authzCtx =
         new RangerAuthorizationContext(sessionState, timeline);
     if (doAudits) {
@@ -270,13 +280,13 @@ public class RangerAuthorizationChecker extends BaseAuthorizationChecker {
   }
 
   /**
-   * This method checks if column mask is enabled on the given columns and deny access
-   * when column mask is enabled by throwing an {@link AuthorizationException}. This is
-   * to prevent data leak when Hive has column mask enabled but not in Impala.
+   * This method throws an {@link AuthorizationException} if column mask is enabled on the
+   * given column. This is used to prevent data leak when Hive has column mask enabled but
+   * it's disabled in Impala.
    */
-  private void authorizeColumnMask(User user, String dbName, String tableName,
+  private void throwIfColumnMaskingRequired(User user, String dbName, String tableName,
       String columnName) throws InternalException, AuthorizationException {
-    if (evalColumnMask(user, dbName, tableName, columnName).isMaskEnabled()) {
+    if (evalColumnMask(user, dbName, tableName, columnName, null).isMaskEnabled()) {
       throw new AuthorizationException(String.format(
           "Column masking is disabled by --enable_column_masking flag. Can't access " +
               "column %s.%s.%s that has column masking policy.",
@@ -288,23 +298,86 @@ public class RangerAuthorizationChecker extends BaseAuthorizationChecker {
   public boolean needsMaskingOrFiltering(User user, String dbName, String tableName,
       List<String> requiredColumns) throws InternalException {
     for (String column: requiredColumns) {
-      if (evalColumnMask(user, dbName, tableName, column).isMaskEnabled()) {
+      if (evalColumnMask(user, dbName, tableName, column, null)
+          .isMaskEnabled()) {
         return true;
       }
     }
-    return false;
+    return needsRowFiltering(user, dbName, tableName);
+  }
+
+  @Override
+  public boolean needsRowFiltering(User user, String dbName, String tableName)
+      throws InternalException {
+    return evalRowFilter(user, dbName, tableName, null).isRowFilterEnabled();
+  }
+
+  /**
+   * Util method for removing stale audit events of column masking and row filtering
+   * policies. See comments below for cases we need this.
+   *
+   * TODO: Revisit RangerBufferAuditHandler and compare it to
+   *  org.apache.ranger.authorization.hive.authorizer.RangerHiveAuditHandler to see
+   *  whether we can avoid generating stale audit events there.
+   * TODO: Do we really need this for row-filtering?
+   */
+  private void removeStaleAudits(RangerBufferAuditHandler auditHandler,
+      int numAuthzAuditEventsBefore) {
+    // When a user adds an "Unmasked" policy for 'columnName' that retains the original
+    // value, accessResult.getMaskType() would be "MASK_NONE". We do not need to log such
+    // an event. Removing such an event also makes the logged audits consistent when
+    // there is at least one "Unmasked" policy. Recall that this method is the only place
+    // evalColumnMask() is called with a non-null RangerBufferAuditHandler to produce
+    // AuthzAuditEvent's. When an "Unmasked" policy is the only column masking policy
+    // involved in this query, this method would not be invoked because isMaskEnabled()
+    // of the corresponding RangerAccessResult is false in needsMaskingOrFiltering(),
+    // whereas this method will be called when there are policies of other types. If we
+    // do not remove the event of the "Unmasked" policy, we will have one more logged
+    // event in the latter case where there are policies other than "Unmasked".
+
+    // Moreover, notice that there will be an AuthzAuditEvent added to
+    // 'auditHandler.getAuthzEvents()' as long as there exists a column masking policy
+    // associated with 'dbName', 'tableName', and 'columnName', even though the policy
+    // does NOT apply to 'user'. In this case, accessResult.isMaskEnabled() would be
+    // false, and accessResult.getPolicyId() would be -1. In addition, the field of
+    // 'accessResult' would be 0 for that AuthzAuditEvent. An AuthzAuditEvent with
+    // 'accessResult' equal to 0 will be considered as an indication of a failed access
+    // request in RangerBufferAuditHandler#flush() even though there is no failed access
+    // request at all for the current query and thus we need to filter out this
+    // AuthzAuditEvent. If we did not filter it out, other AuthzAuditEvent's
+    // corresponding to successful/allowed access requests in the same query will not be
+    // logged since only the first AuthzAuditEvent with 'accessResult' equal to 0 is
+    // logged in RangerBufferAuditHandler#flush().
+    List<AuthzAuditEvent> auditEvents = auditHandler.getAuthzEvents();
+    Preconditions.checkState(auditEvents.size() - numAuthzAuditEventsBefore <= 1);
+    if (auditEvents.size() > numAuthzAuditEventsBefore) {
+      // Recall that the same instance of RangerAuthorizationContext is passed to
+      // createColumnMask() every time this method is called. Thus the same instance of
+      // RangerBufferAuditHandler is provided for evalColumnMask() to log the event.
+      // Since there will be exactly one event added to 'auditEvents' when there is a
+      // corresponding column masking policy, i.e., accessResult.isMaskEnabled()
+      // evaluates to true, we only need to process the last event on 'auditEvents'.
+      auditHandler.getAuthzEvents().remove(auditEvents.size() - 1);
+    }
   }
 
   @Override
   public String createColumnMask(User user, String dbName, String tableName,
-      String columnName) throws InternalException {
-    RangerAccessResult accessResult = evalColumnMask(user, dbName, tableName,
-        columnName);
-    // No column masking policies, return the original column.
-    if (!accessResult.isMaskEnabled()) return columnName;
+      String columnName, AuthorizationContext rangerCtx) throws InternalException {
+    RangerBufferAuditHandler auditHandler = (rangerCtx == null) ? null:
+        ((RangerAuthorizationContext) rangerCtx).getAuditHandler();
+    int numAuthzAuditEventsBefore = (auditHandler == null) ? 0 :
+        auditHandler.getAuthzEvents().size();
+    RangerAccessResult accessResult = evalColumnMask(user, dbName, tableName, columnName,
+        auditHandler);
+    if (!accessResult.isMaskEnabled() && auditHandler != null) {
+      // No column masking policies, remove any possible stale audit events and
+      // return the original column.
+      removeStaleAudits(auditHandler, numAuthzAuditEventsBefore);
+      return columnName;
+    }
     String maskType = accessResult.getMaskType();
     RangerServiceDef.RangerDataMaskTypeDef maskTypeDef = accessResult.getMaskTypeDef();
-    Preconditions.checkNotNull(maskType);
     // The expression used to replace the original column.
     String maskedColumn = columnName;
     // The expression of the mask type. Column names are referenced by "{col}".
@@ -331,17 +404,48 @@ public class RangerAuthorizationChecker extends BaseAuthorizationChecker {
     } else if (StringUtils.isNotEmpty(transformer)) {
       maskedColumn = transformer.replace("{col}", columnName);
     }
-    LOG.info("dbName: {}, tableName: {}, column: {}, maskType: {}, columnTransformer: {}",
+    LOG.trace(
+        "dbName: {}, tableName: {}, column: {}, maskType: {}, columnTransformer: {}",
         dbName, tableName, columnName, maskType, maskedColumn);
     return maskedColumn;
   }
 
+  @Override
+  public String createRowFilter(User user, String dbName, String tableName,
+      AuthorizationContext rangerCtx) throws InternalException {
+    RangerBufferAuditHandler auditHandler = (rangerCtx == null) ? null :
+        ((RangerAuthorizationContext) rangerCtx).getAuditHandler();
+    int numAuthzAuditEventsBefore = (auditHandler == null) ? 0 :
+        auditHandler.getAuthzEvents().size();
+    RangerAccessResult accessResult = evalRowFilter(user, dbName, tableName,
+        auditHandler);
+    if (!accessResult.isRowFilterEnabled() && auditHandler != null) {
+      // No row filtering policies, remove any possible stale audit events.
+      removeStaleAudits(auditHandler, numAuthzAuditEventsBefore);
+      return null;
+    }
+    String filter = accessResult.getFilterExpr();
+    LOG.trace("dbName: {}, tableName: {}, rowFilter: {}", dbName, tableName, filter);
+    return filter;
+  }
+
+  @Override
+  public void postAnalyze(AuthorizationContext authzCtx) {
+    Preconditions.checkArgument(authzCtx instanceof RangerAuthorizationContext);
+    ((RangerAuthorizationContext) authzCtx).stashTableMaskingAuditEvents(plugin_);
+  }
+
   /**
-   * Evaluate column masking policies on the given column and returns the result.
-   * A RangerAccessResult contains the matched policy details and the masked column.
+   * Evaluate column masking policies on the given column and returns the result,
+   * a RangerAccessResult contains the matched policy details and the masked column.
+   * Note that Ranger will add an AuthzAuditEvent to auditHandler.getAuthzEvents() as
+   * long as there exists a policy for the given column even though the policy does not
+   * apply to 'user'.
    */
   private RangerAccessResult evalColumnMask(User user, String dbName,
-      String tableName, String columnName) throws InternalException {
+      String tableName, String columnName, RangerBufferAuditHandler auditHandler)
+      throws InternalException {
+    Preconditions.checkNotNull(user);
     RangerAccessResourceImpl resource = new RangerImpalaResourceBuilder()
         .database(dbName)
         .table(tableName)
@@ -349,31 +453,48 @@ public class RangerAuthorizationChecker extends BaseAuthorizationChecker {
         .build();
     RangerAccessRequest req = new RangerAccessRequestImpl(resource,
         SELECT_ACCESS_TYPE, user.getShortName(), getUserGroups(user));
-    return plugin_.evalDataMaskPolicies(req, null);
+    // The method evalDataMaskPolicies() only checks whether there is a corresponding
+    // column masking policy on the Ranger server and thus does not check whether the
+    // requesting user/group is granted the necessary privilege on the specified
+    // resource. No AnalysisException or AuthorizationException will be thrown if the
+    // requesting user/group does not possess the necessary privilege.
+    return plugin_.evalDataMaskPolicies(req, auditHandler);
   }
 
   /**
-   * This method checks if row filter is enabled on the given tables and deny access
-   * when row filter is enabled by throwing an {@link AuthorizationException} . This is
-   * to prevent data leak when Hive has row filter enabled but not in Impala.
+   * Evaluate row filtering policies on the given table and returns the result,
+   * a RangerAccessResult contains the matched policy details and the filter string.
    */
-  private void authorizeRowFilter(User user, String dbName, String tableName)
-      throws InternalException, AuthorizationException {
+  private RangerAccessResult evalRowFilter(User user, String dbName, String tableName,
+      RangerBufferAuditHandler auditHandler) throws InternalException {
+    Preconditions.checkNotNull(user);
     RangerAccessResourceImpl resource = new RangerImpalaResourceBuilder()
         .database(dbName)
         .table(tableName)
         .build();
     RangerAccessRequest req = new RangerAccessRequestImpl(resource,
         SELECT_ACCESS_TYPE, user.getShortName(), getUserGroups(user));
-    if (plugin_.evalRowFilterPolicies(req, null).isRowFilterEnabled()) {
+    return plugin_.evalRowFilterPolicies(req, auditHandler);
+  }
+
+  /**
+   * This method throws an {@link AuthorizationException} if row filter is enabled on the
+   * given tables. This is used to prevent data leak when Hive has row filter enabled but
+   * it's disabled in Impala.
+   */
+  private void throwIfRowFilteringRequired(User user, String dbName, String tableName)
+      throws InternalException, AuthorizationException {
+    if (evalRowFilter(user, dbName, tableName, null).isRowFilterEnabled()) {
       throw new AuthorizationException(String.format(
-          "Impala does not support row filtering yet. Row filtering is enabled " +
-              "on table: %s.%s", dbName, tableName));
+          "Row filtering is disabled by --enable_row_filtering flag. Can't access " +
+              "table %s.%s that has row filtering policy.",
+          dbName, tableName));
     }
   }
 
   @Override
   public Set<String> getUserGroups(User user) throws InternalException {
+    Preconditions.checkNotNull(user);
     UserGroupInformation ugi;
     if (RuntimeEnv.INSTANCE.isTestEnv() ||
         BackendConfig.INSTANCE.useCustomizedUserGroupsMapperForRanger()) {
@@ -386,23 +507,28 @@ public class RangerAuthorizationChecker extends BaseAuthorizationChecker {
   }
 
   private boolean authorizeAny(RangerAuthorizationContext authzCtx,
-      RangerAccessResourceImpl resource, User user, Privilege privilege)
-      throws InternalException {
+      RangerAccessResourceImpl resource, Authorizable authorizable, User user,
+      Privilege privilege) throws InternalException {
     boolean authorized = false;
     RangerBufferAuditHandler originalAuditHandler = authzCtx.getAuditHandler();
     // Use a temporary audit handler instead of the original audit handler
     // so that we know all the audit events generated by the temporary audit
     // handler are contained to the given resource.
-    RangerBufferAuditHandler tmpAuditHandler = originalAuditHandler == null ?
+    // Recall that in some case, e.g., the DESCRIBE statement, 'originalAuditHandler'
+    // could be null, resulting in 'tmpAuditHandler' being null as well.
+    RangerBufferAuditHandler tmpAuditHandler =
+        (originalAuditHandler == null || !authzCtx.getRetainAudits()) ?
         null : new RangerBufferAuditHandler(originalAuditHandler);
     for (Privilege impliedPrivilege: privilege.getImpliedPrivileges()) {
-      if (authorizeResource(authzCtx, resource, user, impliedPrivilege,
+      if (authorizeResource(authzCtx, user, resource, authorizable, impliedPrivilege,
           tmpAuditHandler)) {
         authorized = true;
         break;
       }
     }
-    if (originalAuditHandler != null) {
+    // 'tmpAuditHandler' could be null if 'originalAuditHandler' is null or
+    // authzCtx.getRetainAudits() is false.
+    if (originalAuditHandler != null && tmpAuditHandler != null) {
       updateAuditEvents(tmpAuditHandler, originalAuditHandler, true /*any*/,
           privilege);
     }
@@ -410,23 +536,31 @@ public class RangerAuthorizationChecker extends BaseAuthorizationChecker {
   }
 
   private boolean authorizeAll(RangerAuthorizationContext authzCtx,
-      RangerAccessResourceImpl resource, User user, Privilege privilege)
-      throws InternalException {
+      RangerAccessResourceImpl resource, Authorizable authorizable, User user,
+      Privilege privilege) throws InternalException {
     boolean authorized = true;
     RangerBufferAuditHandler originalAuditHandler = authzCtx.getAuditHandler();
     // Use a temporary audit handler instead of the original audit handler
     // so that we know all the audit events generated by the temporary audit
     // handler are contained to the given resource.
-    RangerBufferAuditHandler tmpAuditHandler = originalAuditHandler == null ?
+    // Recall that if 'min_privilege_set_for_show_stmts' is specified when the impalad's
+    // are started, this method will be called with the RangerBufferAuditHandler in
+    // 'authzCtx' being null for the SHOW DATABASES and SHOW TABLES statements. Refer to
+    // BaseAuthorizationChecker#hasAccess(User user, PrivilegeRequest request) for
+    // further details.
+    RangerBufferAuditHandler tmpAuditHandler =
+        (originalAuditHandler == null || !authzCtx.getRetainAudits()) ?
         null : new RangerBufferAuditHandler(originalAuditHandler);
     for (Privilege impliedPrivilege: privilege.getImpliedPrivileges()) {
-      if (!authorizeResource(authzCtx, resource, user, impliedPrivilege,
+      if (!authorizeResource(authzCtx, user, resource, authorizable, impliedPrivilege,
           tmpAuditHandler)) {
         authorized = false;
         break;
       }
     }
-    if (originalAuditHandler != null) {
+    // 'tmpAuditHandler' could be null if 'originalAuditHandler' is null or
+    // authzCtx.getRetainAudits() is false.
+    if (originalAuditHandler != null && tmpAuditHandler != null) {
       updateAuditEvents(tmpAuditHandler, originalAuditHandler, false /*not any*/,
           privilege);
     }
@@ -461,8 +595,8 @@ public class RangerAuthorizationChecker extends BaseAuthorizationChecker {
     }
   }
 
-  private boolean authorizeResource(RangerAuthorizationContext authzCtx,
-      RangerAccessResourceImpl resource, User user, Privilege privilege,
+  private boolean authorizeResource(RangerAuthorizationContext authzCtx, User user,
+      RangerAccessResourceImpl resource, Authorizable authorizable, Privilege privilege,
       RangerBufferAuditHandler auditHandler) throws InternalException {
     String accessType;
     if (privilege == Privilege.ANY) {
@@ -481,7 +615,82 @@ public class RangerAuthorizationChecker extends BaseAuthorizationChecker {
           authzCtx.getSessionState().getNetwork_address().getHostname());
     }
     RangerAccessResult authorized = plugin_.isAccessAllowed(request, auditHandler);
-    return authorized != null && authorized.getIsAllowed();
+    if (authorized == null || !authorized.getIsAllowed()) return false;
+    // We are done if don't need to block updates on tables that have column-masking or
+    // row-filtering policies.
+    if (!plugin_.blockUpdateIfTableMaskSpecified() || !privilege.impliesUpdate()
+        || (authorizable.getType() != Type.TABLE
+           && authorizable.getType() != Type.COLUMN)) {
+      return true;
+    }
+    return authorizeByTableMasking(request, user, authorizable, authorized, privilege,
+        auditHandler);
+  }
+
+  /**
+   * Blocks the update request if any row-filtering or column masking policy exists on the
+   * resource(table/column). Appends a deny audit if any exists.
+   * Returns true if the request is authorized.
+   */
+  private boolean authorizeByTableMasking(RangerAccessRequestImpl request, User user,
+      Authorizable authorizable, RangerAccessResult accessResult, Privilege privilege,
+      RangerBufferAuditHandler auditHandler) throws InternalException {
+    Preconditions.checkNotNull(accessResult, "accessResult is null!");
+    Preconditions.checkState(accessResult.getIsAllowed(),
+        "update should be allowed before checking this");
+    String originalAccessType = request.getAccessType();
+    // Row-filtering/Column-masking policies are defined only for SELECT requests.
+    request.setAccessType(SELECT_ACCESS_TYPE);
+    // Check if row filtering is enabled for the table/view.
+    if (authorizable.getType() == Type.TABLE) {
+      RangerAccessResult rowFilterResult = plugin_.evalRowFilterPolicies(
+          request, /*resultProcessor*/null);
+      if (rowFilterResult != null && rowFilterResult.isRowFilterEnabled()) {
+        LOG.trace("Deny {} on {} due to row filtering policy {}",
+            privilege, authorizable.getName(), rowFilterResult.getPolicyId());
+        accessResult.setIsAllowed(false);
+        accessResult.setPolicyId(rowFilterResult.getPolicyId());
+        accessResult.setReason("User does not have access to all rows of the table");
+      } else {
+        LOG.trace("No row filtering policy found on {}.", authorizable.getName());
+      }
+    }
+    // Check if masking is enabled for any column in the table/view.
+    if (accessResult.getIsAllowed()) {
+      List<String> columns;
+      if (authorizable.getType() == Type.TABLE) {
+        // Check all columns.
+        columns = ((AuthorizableTable) authorizable).getColumns();
+        LOG.trace("Checking mask policies on {} columns of table {}", columns.size(),
+            authorizable.getFullTableName());
+      } else {
+        columns = Lists.newArrayList(authorizable.getColumnName());
+      }
+      for (String column : columns) {
+        RangerAccessResult columnMaskResult = evalColumnMask(user,
+            authorizable.getDbName(), authorizable.getTableName(), column,
+            /*auditHandler*/null);
+        if (columnMaskResult != null && columnMaskResult.isMaskEnabled()) {
+          LOG.trace("Deny {} on {} due to column masking policy {}",
+              privilege, authorizable.getName(), columnMaskResult.getPolicyId());
+          accessResult.setIsAllowed(false);
+          accessResult.setPolicyId(columnMaskResult.getPolicyId());
+          accessResult.setReason("User does not have access to unmasked column values");
+          break;
+        } else {
+          LOG.trace("No column masking policy found on column {} of {}.", column,
+              authorizable.getFullTableName());
+        }
+      }
+    }
+    // Set back the original access type. The request object is still referenced by the
+    // access result.
+    request.setAccessType(originalAccessType);
+    // Only add deny audits.
+    if (!accessResult.getIsAllowed() && auditHandler != null) {
+      auditHandler.processResult(accessResult);
+    }
+    return accessResult.getIsAllowed();
   }
 
   @VisibleForTesting
