@@ -21,8 +21,10 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.Comparator;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.atomic.AtomicReference;
 
@@ -91,6 +93,15 @@ public abstract class Catalog implements AutoCloseable {
   // Cache of data sources.
   protected final CatalogObjectCache<DataSource> dataSources_;
 
+  // Cache of transaction to write id mapping for the open transactions which are detected
+  // by the catalogd via events processor. The entries get cleaned up on receiving commit
+  // transaction or abort transaction events.
+  // We need this mapping because not all the write ids for a commit event can be
+  // retrieved from HMS. We can fetch write id for write events via getAllWriteEventInfo.
+  // However, we don't have API to fetch write ids for DDL events.
+  protected final ConcurrentHashMap<Long, Set<TableWriteId>> txnToWriteIds_ =
+      new ConcurrentHashMap<>();
+
   // Cache of known HDFS cache pools. Allows for checking the existence
   // of pools without hitting HDFS.
   protected final CatalogObjectCache<HdfsCachePool> hdfsCachePools_ =
@@ -699,11 +710,14 @@ public abstract class Catalog implements AutoCloseable {
    * 'releaseTableLock()'.
    * @param dbName Name of the DB where the particular table is.
    * @param tableName Name of the table where the lock is acquired.
+   * @param lockMaxWaitTime Maximum wait time on the ACID lock.
    * @throws TransactionException
    */
-  public long lockTableStandalone(String dbName, String tableName, HeartbeatContext ctx)
+  public long lockTableStandalone(String dbName, String tableName, HeartbeatContext ctx,
+      int lockMaxWaitTime)
       throws TransactionException {
-    return lockTableInternal(dbName, tableName, 0L, DataOperationType.NO_TXN, ctx);
+    return lockTableInternal(dbName, tableName, 0L, DataOperationType.NO_TXN, ctx,
+        lockMaxWaitTime);
   }
 
   /**
@@ -713,13 +727,16 @@ public abstract class Catalog implements AutoCloseable {
    * @param dbName Name of the DB where the particular table is.
    * @param tableName Name of the table where the lock is acquired.
    * @param transaction the transaction that needs to lock the table.
+   * @param lockMaxWaitTime Maximum wait time on the ACID lock.
    * @throws TransactionException
    */
   public void lockTableInTransaction(String dbName, String tableName,
-      Transaction transaction, DataOperationType opType, HeartbeatContext ctx)
+      Transaction transaction, DataOperationType opType, HeartbeatContext ctx,
+      int lockMaxWaitTime)
       throws TransactionException {
     Preconditions.checkState(transaction.getId() > 0);
-    lockTableInternal(dbName, tableName, transaction.getId(), opType, ctx);
+    lockTableInternal(dbName, tableName, transaction.getId(), opType, ctx,
+        lockMaxWaitTime);
   }
 
   /**
@@ -728,10 +745,12 @@ public abstract class Catalog implements AutoCloseable {
    * @param dbName Name of the DB where the particular table is.
    * @param tableName Name of the table where the lock is acquired.
    * @param txnId id of the transaction, 0 for standalone locks.
+   * @param lockMaxWaitTime Maximum wait time on the ACID lock.
    * @throws TransactionException
    */
   private long lockTableInternal(String dbName, String tableName, long txnId,
-      DataOperationType opType, HeartbeatContext ctx) throws TransactionException {
+      DataOperationType opType, HeartbeatContext ctx, int lockMaxWaitTime)
+      throws TransactionException {
     Preconditions.checkState(txnId >= 0);
     LockComponent lockComponent = new LockComponent();
     lockComponent.setDbname(dbName);
@@ -742,7 +761,8 @@ public abstract class Catalog implements AutoCloseable {
     List<LockComponent> lockComponents = Arrays.asList(lockComponent);
     long lockId = -1L;
     try (MetaStoreClient client = metaStoreClientPool_.getClient()) {
-      lockId = MetastoreShim.acquireLock(client.getHiveClient(), txnId, lockComponents);
+      lockId = MetastoreShim.acquireLock(client.getHiveClient(), txnId, lockComponents,
+          lockMaxWaitTime);
       if (txnId == 0L) transactionKeepalive_.addLock(lockId, ctx);
     }
     return lockId;
@@ -758,4 +778,40 @@ public abstract class Catalog implements AutoCloseable {
       MetastoreShim.releaseLock(client.getHiveClient(), lockId);
     }
   }
+
+  /**
+   * Returns write ids for an open txn from the Catalog. If there is no write id
+   * associated with the txnId, it returns empty set.
+   */
+  public Set<TableWriteId> getWriteIds(Long txnId) {
+    Preconditions.checkNotNull(txnId);
+    return Collections.unmodifiableSet(txnToWriteIds_.getOrDefault(txnId,
+        Collections.emptySet()));
+  }
+
+  /**
+   * Adds a mapping from txnId to tableWriteId to the Catalog.
+   */
+  public void addWriteId(Long txnId, TableWriteId tableWriteId) {
+    Preconditions.checkNotNull(txnId);
+    Preconditions.checkNotNull(tableWriteId);
+    txnToWriteIds_.computeIfAbsent(txnId, k -> new HashSet<>()).add(tableWriteId);
+  }
+
+  /**
+   * Removes and returns all write id records for a transaction. If there is no write id
+   * associated with the txnId, it returns empty set.
+   */
+  public Set<TableWriteId> removeWriteIds(Long txnId) {
+    Preconditions.checkNotNull(txnId);
+    Set<TableWriteId> resultSet = txnToWriteIds_.remove(txnId);
+    return resultSet != null ? resultSet : Collections.emptySet();
+  }
+
+  /**
+   * Clears all write id records.
+   */
+  public void clearWriteIds() {
+    txnToWriteIds_.clear();
+  }
 }
