@@ -48,6 +48,7 @@ import org.apache.impala.thrift.TNetworkAddress;
 import org.apache.impala.thrift.TPartitionKeyValue;
 import org.apache.impala.thrift.TResultSet;
 import org.apache.impala.thrift.TResultSetMetadata;
+import org.apache.impala.thrift.TSortingOrder;
 import org.apache.impala.thrift.TTableStats;
 import org.apache.impala.util.ListMap;
 import org.apache.impala.util.TAccessLevelUtil;
@@ -208,6 +209,16 @@ public interface FeFsTable extends FeTable {
     return tableFs;
   }
 
+  public static FileSystem getFileSystem(Path filePath) throws CatalogException {
+    FileSystem tableFs;
+    try {
+      tableFs = filePath.getFileSystem(CONF);
+    } catch (IOException e) {
+      throw new CatalogException("Invalid path: " + filePath.toString(), e);
+    }
+    return tableFs;
+  }
+
   /**
    * @return  List of primary keys column names, useful for toSqlUtils. In local
    * catalog mode, this causes load of constraints.
@@ -288,6 +299,63 @@ public interface FeFsTable extends FeTable {
    */
   ListMap<TNetworkAddress> getHostIndex();
 
+  /**
+   * Check if 'col_name' appears in the list of sort-by columns by searching the
+   * 'sort.columns' table property and return the index in the list if so. Return
+   * -1 otherwise.
+   */
+  default int getSortByColumnIndex(String col_name) {
+    // Get the names of all sort by columns (specified in the SORT BY clause in
+    // CREATE TABLE DDL) from TBLPROPERTIES.
+    Map<String, String> parameters = getMetaStoreTable().getParameters();
+    if (parameters == null) return -1;
+    String sort_by_columns_string = parameters.get("sort.columns");
+    if (sort_by_columns_string == null) return -1;
+    String[] sort_by_columns = sort_by_columns_string.split(",");
+    if (sort_by_columns == null) return -1;
+    for (int i = 0; i < sort_by_columns.length; i++) {
+      if (sort_by_columns[i].equals(col_name)) return i;
+    }
+    return -1;
+  }
+
+  /**
+   * Check if 'col_name' names the leading sort-by column.
+   */
+  default boolean isLeadingSortByColumn(String col_name) {
+    return getSortByColumnIndex(col_name) == 0;
+  }
+
+  /**
+   * Check if 'col_name' appears in the list of sort-by columns.
+   */
+  default boolean isSortByColumn(String col_name) {
+    return getSortByColumnIndex(col_name) >= 0;
+  }
+
+  /**
+   * Return the sort order for the sort by columns if exists. Return null otherwise.
+   */
+  default TSortingOrder getSortOrderForSortByColumn() {
+    Map<String, String> parameters = getMetaStoreTable().getParameters();
+    if (parameters == null) return null;
+    String sortOrder = parameters.get("sort.order");
+    if (sortOrder == null) return null;
+    if (sortOrder.equals("LEXICAL")) return TSortingOrder.LEXICAL;
+    if (sortOrder.equals("ZORDER")) return TSortingOrder.ZORDER;
+    return null;
+  }
+
+  /**
+   * Return true if the sort order for the sort by columns is lexical. Return false
+   * otherwise.
+   */
+  default boolean IsLexicalSortByColumn() {
+    TSortingOrder sortOrder = getSortOrderForSortByColumn();
+    if (sortOrder == null) return false;
+    return sortOrder == TSortingOrder.LEXICAL;
+  }
+
   /**
    * Utility functions for operating on FeFsTable. When we move fully to Java 8,
    * these can become default methods of the interface.
@@ -365,6 +433,10 @@ public interface FeFsTable extends FeTable {
       resultSchema.addToColumns(new TColumn("Partition", Type.STRING.toThrift()));
       result.setRows(new ArrayList<>());
 
+      if (table instanceof FeIcebergTable) {
+        return getIcebergTableFiles((FeIcebergTable) table, result);
+      }
+
       List<? extends FeFsPartition> orderedPartitions;
       if (partitionSet == null) {
         orderedPartitions = Lists.newArrayList(FeCatalogUtils.loadAllPartitions(table));
@@ -388,6 +460,24 @@ public interface FeFsTable extends FeTable {
       return result;
     }
 
+    /**
+     * Get file info for the given fe iceberg table.
+     */
+    private static TResultSet getIcebergTableFiles(FeIcebergTable table,
+        TResultSet result) {
+      List<FileDescriptor> orderedFds = Lists
+          .newArrayList(table.getPathHashToFileDescMap().values());
+      Collections.sort(orderedFds);
+      for (FileDescriptor fd : orderedFds) {
+        TResultRowBuilder rowBuilder = new TResultRowBuilder();
+        rowBuilder.add(table.getLocation() + "/" + fd.getRelativePath());
+        rowBuilder.add(PrintUtils.printBytes(fd.getFileLength()));
+        rowBuilder.add("");
+        result.addToRows(rowBuilder.get());
+      }
+      return result;
+    }
+
     /**
      * Selects a random sample of files from the given list of partitions such that the
      * sum of file sizes is at least 'percentBytes' percent of the total number of bytes
