@@ -35,14 +35,17 @@ import org.apache.iceberg.DataFile;
 import org.apache.iceberg.PartitionField;
 import org.apache.iceberg.PartitionSpec;
 import org.apache.iceberg.Schema;
-import org.apache.iceberg.TableMetadata;
 import org.apache.impala.analysis.IcebergPartitionField;
 import org.apache.impala.analysis.IcebergPartitionSpec;
 import org.apache.impala.analysis.LiteralExpr;
+import org.apache.impala.catalog.HdfsPartition.FileDescriptor;
 import org.apache.impala.common.FileSystemUtil;
 import org.apache.impala.common.Reference;
 import org.apache.impala.compat.HdfsShim;
 import org.apache.impala.thrift.TColumn;
+import org.apache.impala.thrift.TCompressionCodec;
+import org.apache.impala.thrift.THdfsCompression;
+import org.apache.impala.thrift.THdfsFileDesc;
 import org.apache.impala.thrift.THdfsTable;
 import org.apache.impala.thrift.THdfsPartition;
 import org.apache.impala.thrift.TIcebergCatalog;
@@ -56,12 +59,16 @@ import org.apache.impala.util.ListMap;
 import org.apache.impala.util.TResultRowBuilder;
 
 import com.google.common.base.Preconditions;
+import com.google.common.primitives.Ints;
+
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * Frontend interface for interacting with an Iceberg-backed table.
  */
 public interface FeIcebergTable extends FeFsTable {
-
+  final static Logger LOG = LoggerFactory.getLogger(FeIcebergTable.class);
   /**
    * FileDescriptor map
    */
@@ -77,6 +84,11 @@ public interface FeIcebergTable extends FeFsTable {
    */
   TIcebergCatalog getIcebergCatalog();
 
+  /**
+   * Returns the cached Iceberg Table object that stores the metadata loaded by Iceberg.
+   */
+  org.apache.iceberg.Table getIcebergApiTable();
+
   /**
    * Return Iceberg catalog location, we use this location to load metadata from Iceberg
    * When using 'hadoop.tables', this value equals to table location
@@ -89,6 +101,26 @@ public interface FeIcebergTable extends FeFsTable {
    */
   TIcebergFileFormat getIcebergFileFormat();
 
+  /**
+   * Return iceberg parquet compression codec from table properties
+   */
+  TCompressionCodec getIcebergParquetCompressionCodec();
+
+  /**
+   * Return iceberg parquet row group size in bytes from table properties
+   */
+  long getIcebergParquetRowGroupSize();
+
+  /**
+   * Return iceberg parquet plain page size in bytes from table properties
+   */
+  long getIcebergParquetPlainPageSize();
+
+  /**
+   * Return iceberg parquet dictionary page size in bytes from table properties
+   */
+  long getIcebergParquetDictPageSize();
+
   /**
    * Return the table location of Iceberg table
    * When using 'hadoop.tables', this value is a normal table location
@@ -114,7 +146,9 @@ public interface FeIcebergTable extends FeFsTable {
   /**
    * @return the Iceberg schema.
    */
-  Schema getIcebergSchema();
+  default Schema getIcebergSchema() {
+    return getIcebergApiTable().schema();
+  }
 
   @Override
   default boolean isCacheable() {
@@ -221,10 +255,29 @@ public interface FeIcebergTable extends FeFsTable {
     return getFeFsTable().getHostIndex();
   }
 
+  @Override /* FeTable */
+  default boolean isComputedPartitionColumn(Column col) {
+    Preconditions.checkState(col instanceof IcebergColumn);
+    IcebergColumn iceCol = (IcebergColumn)col;
+    IcebergPartitionSpec spec = getDefaultPartitionSpec();
+    if (spec == null || !spec.hasPartitionFields()) return false;
+
+    for (IcebergPartitionField partField : spec.getIcebergPartitionFields()) {
+      if (iceCol.getFieldId() == partField.getSourceId()) return true;
+    }
+    return false;
+  }
+
   /**
-   * Current snapshot id of the table.
+   * Returns the current snapshot id of the Iceberg API table if it exists, otherwise
+   * returns -1.
    */
-  long snapshotId();
+  default long snapshotId() {
+    if (getIcebergApiTable() != null && getIcebergApiTable().currentSnapshot() != null) {
+      return getIcebergApiTable().currentSnapshot().snapshotId();
+    }
+    return -1;
+  }
 
   /**
    * Utility functions
@@ -251,10 +304,9 @@ public interface FeIcebergTable extends FeFsTable {
       resultSchema.addToColumns(new TColumn("Field Partition Transform",
           Type.STRING.toThrift()));
 
-      TableMetadata metadata = IcebergUtil.getIcebergTableMetadata(table);
-      if (!metadata.specs().isEmpty()) {
+      if (!table.getIcebergApiTable().specs().isEmpty()) {
         // Just show the current PartitionSpec from Iceberg table metadata
-        PartitionSpec latestSpec = metadata.spec();
+        PartitionSpec latestSpec = table.getIcebergApiTable().spec();
         HashMap<String, Integer> transformParams =
             IcebergUtil.getPartitionTransformParams(latestSpec);
         for(PartitionField field : latestSpec.fields()) {
@@ -294,15 +346,60 @@ public interface FeIcebergTable extends FeFsTable {
     }
 
     /**
-     * Get iceberg table file format from hms table properties
+     * Get iceberg parquet compression codec from hms table properties
      */
-    public static TIcebergFileFormat getIcebergFileFormat(
+    public static TCompressionCodec getIcebergParquetCompressionCodec(
         org.apache.hadoop.hive.metastore.api.Table msTable) {
-      TIcebergFileFormat fileFormat = IcebergUtil.getIcebergFileFormat(
-          msTable.getParameters().get(IcebergTable.ICEBERG_FILE_FORMAT));
-      return fileFormat == null ? TIcebergFileFormat.PARQUET : fileFormat;
+      THdfsCompression codec = IcebergUtil.getIcebergParquetCompressionCodec(
+          msTable.getParameters().get(IcebergTable.PARQUET_COMPRESSION_CODEC));
+      if (codec == null) codec = IcebergTable.DEFAULT_PARQUET_COMPRESSION_CODEC;
+      TCompressionCodec compression = new TCompressionCodec(codec);
+
+      // Compression level is interesting only if ZSTD codec is used.
+      if (codec == THdfsCompression.ZSTD) {
+        int clevel = IcebergTable.DEFAULT_PARQUET_ZSTD_COMPRESSION_LEVEL;
+
+        String clevelTblProp = msTable.getParameters().get(
+            IcebergTable.PARQUET_COMPRESSION_LEVEL);
+        if (clevelTblProp != null) {
+          Integer cl = Ints.tryParse(clevelTblProp);
+          if (cl != null && cl >= IcebergTable.MIN_PARQUET_COMPRESSION_LEVEL &&
+              cl <= IcebergTable.MAX_PARQUET_COMPRESSION_LEVEL) {
+            clevel = cl;
+          }
+        }
+        compression.setCompression_level(clevel);
+      }
+
+      return compression;
     }
 
+    /**
+     * Get iceberg parquet row group size from hms table properties
+     */
+    public static long getIcebergParquetRowGroupSize(
+        org.apache.hadoop.hive.metastore.api.Table msTable) {
+      return IcebergUtil.getIcebergParquetRowGroupSize(
+          msTable.getParameters().get(IcebergTable.PARQUET_ROW_GROUP_SIZE));
+    }
+
+    /**
+     * Get iceberg parquet plain page size from hms table properties
+     */
+    public static long getIcebergParquetPlainPageSize(
+        org.apache.hadoop.hive.metastore.api.Table msTable) {
+      return IcebergUtil.getIcebergParquetPageSize(
+          msTable.getParameters().get(IcebergTable.PARQUET_PLAIN_PAGE_SIZE));
+    }
+
+    /**
+     * Get iceberg parquet dictionary page size from hms table properties
+     */
+    public static long getIcebergParquetDictPageSize(
+        org.apache.hadoop.hive.metastore.api.Table msTable) {
+      return IcebergUtil.getIcebergParquetPageSize(
+          msTable.getParameters().get(IcebergTable.PARQUET_DICT_PAGE_SIZE));
+    }
 
     public static TIcebergTable getTIcebergTable(FeIcebergTable icebergTable) {
       TIcebergTable tIcebergTable = new TIcebergTable();
@@ -314,19 +411,59 @@ public interface FeIcebergTable extends FeFsTable {
       tIcebergTable.setDefault_partition_spec_id(
           icebergTable.getDefaultPartitionSpecId());
 
+      tIcebergTable.setPath_hash_to_file_descriptor(
+          convertPathHashToFileDescMap(icebergTable));
+
+      tIcebergTable.setCatalog_snapshot_id(icebergTable.snapshotId());
+      tIcebergTable.setParquet_compression_codec(
+          icebergTable.getIcebergParquetCompressionCodec());
+      tIcebergTable.setParquet_row_group_size(
+          icebergTable.getIcebergParquetRowGroupSize());
+      tIcebergTable.setParquet_plain_page_size(
+          icebergTable.getIcebergParquetPlainPageSize());
+      tIcebergTable.setParquet_dict_page_size(
+          icebergTable.getIcebergParquetDictPageSize());
+      return tIcebergTable;
+    }
+
+    public static Map<String, THdfsFileDesc> convertPathHashToFileDescMap(
+        FeIcebergTable icebergTable) {
+      Map<String, THdfsFileDesc> ret = new HashMap<>();
       for (Map.Entry<String, HdfsPartition.FileDescriptor> entry :
           icebergTable.getPathHashToFileDescMap().entrySet()) {
-        tIcebergTable.putToPath_hash_to_file_descriptor(entry.getKey(),
-          entry.getValue().toThrift());
+        ret.put(entry.getKey(), entry.getValue().toThrift());
       }
-      tIcebergTable.setSnapshot_id(icebergTable.snapshotId());
-      return tIcebergTable;
+      return ret;
+    }
+
+    /**
+     * Load the file descriptors from the thrift-encoded 'tFileDescMap'. Optionally
+     * translate the file descriptors with the given 'networkAddresses'/'hostIndex'.
+     */
+    public static Map<String, FileDescriptor> loadFileDescMapFromThrift(
+        Map<String, THdfsFileDesc> tFileDescMap,
+        List<TNetworkAddress> networkAddresses,
+        ListMap<TNetworkAddress> hostIndex) {
+      Map<String, FileDescriptor> fileDescMap = new HashMap<>();
+      if (tFileDescMap == null) return fileDescMap;
+      for (Map.Entry<String, THdfsFileDesc> entry : tFileDescMap.entrySet()) {
+        FileDescriptor fd = FileDescriptor.fromThrift(entry.getValue());
+        Preconditions.checkNotNull(fd);
+        if (networkAddresses == null) {
+          fileDescMap.put(entry.getKey(), fd);
+        } else {
+          Preconditions.checkNotNull(hostIndex);
+          fileDescMap.put(entry.getKey(),
+              fd.cloneWithNewHostIndex(networkAddresses, hostIndex));
+        }
+      }
+      return fileDescMap;
     }
 
     /**
      * Get FileDescriptor by data file location
      */
-    private static HdfsPartition.FileDescriptor getFileDescriptor(Path fileLoc,
+    public static HdfsPartition.FileDescriptor getFileDescriptor(Path fileLoc,
         Path tableLoc, ListMap<TNetworkAddress> hostIndex) throws IOException {
       FileSystem fs = FileSystemUtil.getFileSystemForPath(tableLoc);
       FileStatus fileStatus = fs.getFileStatus(fileLoc);
@@ -355,20 +492,45 @@ public interface FeIcebergTable extends FeFsTable {
     }
 
     /**
-     * Get all FileDescriptor from iceberg table without any predicates.
+     * Returns the FileDescriptors loaded by the internal HdfsTable. To avoid returning
+     * the metadata files the resulset is limited to the files that are tracked by
+     * Iceberg. Both the HdfsBaseDir and the DataFile path can contain the scheme in their
+     * path, using org.apache.hadoop.fs.Path to normalize the paths.
      */
     public static Map<String, HdfsPartition.FileDescriptor> loadAllPartition(
-        FeIcebergTable table) throws IOException, TableLoadingException {
-      // Empty predicates
-      List<DataFile> dataFileList = IcebergUtil.getIcebergDataFiles(table,
-          new ArrayList<>());
-
+        IcebergTable table) throws IOException, TableLoadingException {
+      Map<String, HdfsPartition.FileDescriptor> hdfsFileDescMap = new HashMap<>();
+      Collection<HdfsPartition> partitions =
+          ((HdfsTable)table.getFeFsTable()).partitionMap_.values();
+      for (HdfsPartition partition : partitions) {
+        for (FileDescriptor fileDesc : partition.getFileDescriptors()) {
+            Path path = new Path(table.getHdfsBaseDir() + Path.SEPARATOR +
+                fileDesc.getRelativePath());
+            hdfsFileDescMap.put(path.toUri().getPath(), fileDesc);
+        }
+      }
       Map<String, HdfsPartition.FileDescriptor> fileDescMap = new HashMap<>();
-      for (DataFile file : dataFileList) {
-        HdfsPartition.FileDescriptor fileDesc = getFileDescriptor(
-            new Path(file.path().toString()),
-            new Path(table.getIcebergTableLocation()), table.getHostIndex());
-        fileDescMap.put(IcebergUtil.getDataFilePathHash(file), fileDesc);
+      List<DataFile> dataFileList = IcebergUtil.getIcebergDataFiles(table,
+          new ArrayList<>(), /*timeTravelSpecl=*/null).first;
+      for (DataFile dataFile : dataFileList) {
+          Path path = new Path(dataFile.path().toString());
+          if (hdfsFileDescMap.containsKey(path.toUri().getPath())) {
+            String pathHash = IcebergUtil.getDataFilePathHash(dataFile);
+            HdfsPartition.FileDescriptor fsFd = hdfsFileDescMap.get(
+                path.toUri().getPath());
+            HdfsPartition.FileDescriptor iceFd = fsFd.cloneWithFileMetadata(
+                IcebergUtil.createIcebergMetadata(table, dataFile));
+            fileDescMap.put(pathHash, iceFd);
+          } else {
+            LOG.warn("Iceberg DataFile '{}' cannot be found in the HDFS recursive file "
+                + "listing results.", path.toString());
+            HdfsPartition.FileDescriptor fileDesc = getFileDescriptor(
+                new Path(dataFile.path().toString()),
+                new Path(table.getIcebergTableLocation()), table.getHostIndex());
+            HdfsPartition.FileDescriptor iceFd = fileDesc.cloneWithFileMetadata(
+                IcebergUtil.createIcebergMetadata(table, dataFile));
+            fileDescMap.put(IcebergUtil.getDataFilePathHash(dataFile), iceFd);
+          }
       }
       return fileDescMap;
     }
@@ -377,9 +539,9 @@ public interface FeIcebergTable extends FeFsTable {
      * Get iceberg partition spec by iceberg table metadata
      */
     public static List<IcebergPartitionSpec> loadPartitionSpecByIceberg(
-        TableMetadata metadata) throws TableLoadingException {
+        FeIcebergTable table) throws TableLoadingException {
       List<IcebergPartitionSpec> ret = new ArrayList<>();
-      for (PartitionSpec spec : metadata.specs()) {
+      for (PartitionSpec spec : table.getIcebergApiTable().specs().values()) {
         ret.add(convertPartitionSpec(spec));
       }
       return ret;
@@ -403,7 +565,7 @@ public interface FeIcebergTable extends FeFsTable {
       List<IcebergPartitionSpec> specs = feIcebergTable.getPartitionSpecs();
       Preconditions.checkState(specs != null);
       if (specs.isEmpty()) return null;
-      int defaultSpecId = feIcebergTable.getDefaultPartitionSpecId();
+      int defaultSpecId = feIcebergTable.getIcebergApiTable().spec().specId();
       Preconditions.checkState(specs.size() > defaultSpecId);
       return specs.get(defaultSpecId);
     }
