@@ -110,6 +110,12 @@ public interface FeTable {
 
   boolean isClusteringColumn(Column c);
 
+  /**
+   * Return true when the column is used in a computed partition, e.g. in Iceberg
+   * partition transforms.
+   */
+  default boolean isComputedPartitionColumn(Column c) { return false; }
+
   /**
    * Case-insensitive lookup.
    *
