@@ -51,6 +51,7 @@ import org.apache.impala.compat.MetastoreShim;
 import org.apache.impala.fb.FbCompression;
 import org.apache.impala.fb.FbFileBlock;
 import org.apache.impala.fb.FbFileDesc;
+import org.apache.impala.fb.FbFileMetadata;
 import org.apache.impala.thrift.CatalogObjectsConstants;
 import org.apache.impala.thrift.TAccessLevel;
 import org.apache.impala.thrift.TCatalogObject;
@@ -113,10 +114,26 @@ public class HdfsPartition extends CatalogObjectImpl
     // Internal representation of a file descriptor using a FlatBuffer.
     private final FbFileDesc fbFileDescriptor_;
 
-    private FileDescriptor(FbFileDesc fileDescData) { fbFileDescriptor_ = fileDescData; }
+    // Internal representation of additional file metadata, e.g. Iceberg metadata.
+    private final FbFileMetadata fbFileMetadata_;
+
+    private FileDescriptor(FbFileDesc fileDescData) {
+      fbFileDescriptor_ = fileDescData;
+      fbFileMetadata_ = null;
+    }
+
+    private FileDescriptor(FbFileDesc fileDescData, FbFileMetadata fileMetadata) {
+      fbFileDescriptor_ = fileDescData;
+      fbFileMetadata_ = fileMetadata;
+    }
 
     public static FileDescriptor fromThrift(THdfsFileDesc desc) {
       ByteBuffer bb = ByteBuffer.wrap(desc.getFile_desc_data());
+      if (desc.isSetFile_metadata()) {
+        ByteBuffer bbMd = ByteBuffer.wrap(desc.getFile_metadata());
+        return new FileDescriptor(FbFileDesc.getRootAsFbFileDesc(bb),
+                                  FbFileMetadata.getRootAsFbFileMetadata(bbMd));
+      }
       return new FileDescriptor(FbFileDesc.getRootAsFbFileDesc(bb));
     }
 
@@ -141,11 +158,15 @@ public class HdfsPartition extends CatalogObjectImpl
           int origHostIdx = FileBlock.getReplicaHostIdx(it, j);
           boolean isCached = FileBlock.isReplicaCached(it, j);
           TNetworkAddress origHost = origIndex.get(origHostIdx);
-          int newHostIdx = dstIndex.getIndex(origHost);
+          int newHostIdx = dstIndex.getOrAddIndex(origHost);
           it.mutateReplicaHostIdxs(j, FileBlock.makeReplicaIdx(isCached, newHostIdx));
         }
       }
-      return new FileDescriptor(cloned);
+      return new FileDescriptor(cloned, fbFileMetadata_);
+    }
+
+    public FileDescriptor cloneWithFileMetadata(FbFileMetadata fileMetadata) {
+      return new FileDescriptor(fbFileDescriptor_, fileMetadata);
     }
 
     /**
@@ -178,7 +199,7 @@ public class HdfsPartition extends CatalogObjectImpl
         if (isEc) {
           fbFileBlockOffsets[blockIdx++] = FileBlock.createFbFileBlock(fbb,
               loc.getOffset(), loc.getLength(),
-              (short) hostIndex.getIndex(REMOTE_NETWORK_ADDRESS));
+              (short) hostIndex.getOrAddIndex(REMOTE_NETWORK_ADDRESS));
         } else {
           fbFileBlockOffsets[blockIdx++] =
               FileBlock.createFbFileBlock(fbb, loc, hostIndex, numUnknownDiskIds);
@@ -254,10 +275,17 @@ public class HdfsPartition extends CatalogObjectImpl
       return fbFileDescriptor_.fileBlocks(idx);
     }
 
+    public FbFileMetadata getFbFileMetadata() {
+      return fbFileMetadata_;
+    }
+
     public THdfsFileDesc toThrift() {
       THdfsFileDesc fd = new THdfsFileDesc();
       ByteBuffer bb = fbFileDescriptor_.getByteBuffer();
       fd.setFile_desc_data(bb);
+      if (fbFileMetadata_ != null) {
+        fd.setFile_metadata(fbFileMetadata_.getByteBuffer());
+      }
       return fd;
     }
 
@@ -399,7 +427,7 @@ public class HdfsPartition extends CatalogObjectImpl
       // map it to the corresponding hostname from getHosts().
       for (int i = 0; i < loc.getNames().length; ++i) {
         TNetworkAddress networkAddress = BlockReplica.parseLocation(loc.getNames()[i]);
-        short replicaIdx = (short) hostIndex.getIndex(networkAddress);
+        short replicaIdx = (short) hostIndex.getOrAddIndex(networkAddress);
         boolean isReplicaCached = cachedHosts.contains(loc.getHosts()[i]);
         replicaIdx = makeReplicaIdx(isReplicaCached, replicaIdx);
         fbb.addShort(replicaIdx);
@@ -683,6 +711,15 @@ public class HdfsPartition extends CatalogObjectImpl
   // it's not used in coordinators.
   private final InFlightEvents inFlightEvents_;
 
+  // event id in hive metastore which pertains to the creation of this partition object.
+  // if this partition is not created by this catalogd or if the events processing is
+  // not active this is set to -1.
+  private final long createEventId_;
+
+  // The last committed compaction ID
+  // -1 means there is no previous compaction event or compaction is not supported
+  private final long lastCompactionId_;
+
   /**
    * Constructor.  Needed for third party extensions that want to use their own builder
    * to construct the object.
@@ -701,10 +738,10 @@ public class HdfsPartition extends CatalogObjectImpl
         partitionKeyValues, fileFormatDescriptor, encodedFileDescriptors,
         encodedInsertFileDescriptors, encodedDeleteFileDescriptors, location,
         isMarkedCached, accessLevel, hmsParameters, cachedMsPartitionDescriptor,
-        partitionStats, hasIncrementalStats, numRows, writeId, inFlightEvents);
+        partitionStats, hasIncrementalStats, numRows, writeId,
+        inFlightEvents, /*createEventId=*/-1L, /*lastCompactionId*/-1L);
   }
 
-
   protected HdfsPartition(HdfsTable table, long id, long prevId, String partName,
       List<LiteralExpr> partitionKeyValues, HdfsStorageDescriptor fileFormatDescriptor,
       @Nonnull ImmutableList<byte[]> encodedFileDescriptors,
@@ -714,7 +751,7 @@ public class HdfsPartition extends CatalogObjectImpl
       boolean isMarkedCached, TAccessLevel accessLevel, Map<String, String> hmsParameters,
       CachedHmsPartitionDescriptor cachedMsPartitionDescriptor,
       byte[] partitionStats, boolean hasIncrementalStats, long numRows, long writeId,
-      InFlightEvents inFlightEvents) {
+      InFlightEvents inFlightEvents, long createEventId, long lastCompactionId) {
     table_ = table;
     id_ = id;
     prevId_ = prevId;
@@ -733,6 +770,8 @@ public class HdfsPartition extends CatalogObjectImpl
     numRows_ = numRows;
     writeId_ = writeId;
     inFlightEvents_ = inFlightEvents;
+    createEventId_ = createEventId;
+    lastCompactionId_ = lastCompactionId;
     if (partName == null && id_ != CatalogObjectsConstants.PROTOTYPE_PARTITION_ID) {
       partName_ = FeCatalogUtils.getPartitionName(this);
     } else {
@@ -740,6 +779,8 @@ public class HdfsPartition extends CatalogObjectImpl
     }
   }
 
+  public long getCreateEventId() { return createEventId_; }
+
   @Override // FeFsPartition
   public HdfsStorageDescriptor getInputFormatDescriptor() {
     return fileFormatDescriptor_;
@@ -862,7 +903,8 @@ public class HdfsPartition extends CatalogObjectImpl
    * Insert events. If false, remove version number from list of versions for in-flight
    * DDL events.
    * @param versionNumber when isInsertEvent is true, it's eventId to remove
-   *                      when isInsertEvent is false, it's version number to remove
+   *                      when isInsertEvent is false, it's version number to
+   *                      remove.
    * @return true if the versionNumber was removed, false if it didn't exist
    */
   public boolean removeFromVersionsForInflightEvents(
@@ -870,7 +912,12 @@ public class HdfsPartition extends CatalogObjectImpl
     Preconditions.checkState(table_.isWriteLockedByCurrentThread(),
         "removeFromVersionsForInflightEvents called without holding the table lock on "
             + "partition " + getPartitionName() + " of table " + table_.getFullName());
-    return inFlightEvents_.remove(isInsertEvent, versionNumber);
+    boolean ret = inFlightEvents_.remove(isInsertEvent, versionNumber);
+    if (!ret) {
+      LOG.trace("Failed to remove in-flight version number {}: in-flight events: {}",
+          versionNumber, inFlightEvents_.print());
+    }
+    return ret;
   }
 
   /**
@@ -884,12 +931,15 @@ public class HdfsPartition extends CatalogObjectImpl
     Preconditions.checkState(table_.isWriteLockedByCurrentThread(),
         "addToVersionsForInflightEvents called without holding the table lock on "
             + "partition " + getPartitionName() + " of table " + table_.getFullName());
-    if (!inFlightEvents_.add(isInsertEvent, versionNumber)) {
+    boolean added = inFlightEvents_.add(isInsertEvent, versionNumber);
+    if (!added) {
       LOG.warn(String.format("Could not add %s version to the partition %s of table %s. "
           + "This could cause unnecessary refresh of the partition when the event is"
           + "received by the Events processor.", versionNumber, getPartitionName(),
           getTable().getFullName()));
     }
+    LOG.trace("{} {} to in-flight list {}",
+        (added ? "Added" : "Could not add"), versionNumber, inFlightEvents_.print());
   }
 
   @Override // FeFsPartition
@@ -919,6 +969,10 @@ public class HdfsPartition extends CatalogObjectImpl
     return Lists.transform(encodedDeleteFileDescriptors_, FileDescriptor.FROM_BYTES);
   }
 
+  public long getLastCompactionId() {
+    return lastCompactionId_;
+  }
+
   /**
    * Returns a set of fully qualified file names in the partition.
    */
@@ -1143,6 +1197,10 @@ public class HdfsPartition extends CatalogObjectImpl
         .build();
   }
 
+  protected HdfsPartitionLocationCompressor.Location getLocationStruct() {
+    return location_;
+  }
+
   public static class Builder {
     // For the meaning of these fields, see field comments of HdfsPartition.
     private HdfsTable table_;
@@ -1163,7 +1221,12 @@ public class HdfsPartition extends CatalogObjectImpl
     private boolean hasIncrementalStats_ = false;
     private long numRows_ = -1;
     private long writeId_ = -1L;
-    private InFlightEvents inFlightEvents_ = new InFlightEvents(20);
+    // event id in metastore which pertains to the creation of this partition. Defaults
+    // to -1 if the partition was not created by this catalogd or if events processing
+    // is not active.
+    private long createEventId_ = -1L;
+    private long lastCompactionId_ = -1L;
+    private InFlightEvents inFlightEvents_ = new InFlightEvents();
 
     @Nullable
     private HdfsPartition oldInstance_ = null;
@@ -1185,6 +1248,10 @@ public class HdfsPartition extends CatalogObjectImpl
       this(partition.table_);
       oldInstance_ = partition;
       prevId_ = oldInstance_.id_;
+      copyFromPartition(partition);
+    }
+
+    public Builder copyFromPartition(HdfsPartition partition) {
       partitionKeyValues_ = partition.partitionKeyValues_;
       fileFormatDescriptor_ = partition.fileFormatDescriptor_;
       setFileDescriptors(partition);
@@ -1202,6 +1269,8 @@ public class HdfsPartition extends CatalogObjectImpl
       }
       // Take over the in-flight events
       inFlightEvents_ = partition.inFlightEvents_;
+      lastCompactionId_ = partition.lastCompactionId_;
+      return this;
     }
 
     public HdfsPartition build() {
@@ -1223,7 +1292,8 @@ public class HdfsPartition extends CatalogObjectImpl
           fileFormatDescriptor_, encodedFileDescriptors_, encodedInsertFileDescriptors_,
           encodedDeleteFileDescriptors_, location_, isMarkedCached_, accessLevel_,
           hmsParameters_, cachedMsPartitionDescriptor_, partitionStats_,
-          hasIncrementalStats_, numRows_, writeId_, inFlightEvents_);
+          hasIncrementalStats_, numRows_, writeId_, inFlightEvents_, createEventId_,
+          lastCompactionId_);
     }
 
     public Builder setId(long id) {
@@ -1231,6 +1301,11 @@ public class HdfsPartition extends CatalogObjectImpl
       return this;
     }
 
+    public Builder setCreateEventId(long eventId) {
+      createEventId_ = eventId;
+      return this;
+    }
+
     public Builder setPrevId(long prevId) {
       prevId_ = prevId;
       return this;
@@ -1479,6 +1554,13 @@ public class HdfsPartition extends CatalogObjectImpl
       return FeCatalogUtils.getPartitionName(this);
     }
 
+    public Builder setLastCompactionId(long compactionId) {
+      lastCompactionId_ = compactionId;
+      return this;
+    }
+
+    public long getLastCompactionId() { return lastCompactionId_; }
+
     /**
      * Adds a version number to the in-flight events of this partition
      * @param isInsertEvent if true, add eventId to list of eventIds for in-flight Insert
@@ -1516,8 +1598,12 @@ public class HdfsPartition extends CatalogObjectImpl
           MetastoreEventPropertyKey.CATALOG_VERSION.getKey())) {
         return;
       }
-      inFlightEvents_.add(false, Long.parseLong(
+      boolean added = inFlightEvents_.add(false, Long.parseLong(
           hmsParameters_.get(MetastoreEventPropertyKey.CATALOG_VERSION.getKey())));
+      LOG.trace("{} {} to inflight events {}",
+          (added ? "Added" : "Could not add"), Long.parseLong(
+              hmsParameters_.get(MetastoreEventPropertyKey.CATALOG_VERSION.getKey())),
+          inFlightEvents_.print());
     }
 
     private List<FileDescriptor> fdsFromThrift(List<THdfsFileDesc> tFileDescs) {
@@ -1627,7 +1713,8 @@ public class HdfsPartition extends CatalogObjectImpl
           && partitionStats_ == oldInstance.partitionStats_
           && hasIncrementalStats_ == oldInstance.hasIncrementalStats_
           && numRows_ == oldInstance.numRows_
-          && writeId_ == oldInstance.writeId_);
+          && writeId_ == oldInstance.writeId_
+          && lastCompactionId_ == oldInstance.lastCompactionId_);
     }
   }
 
