@@ -27,21 +27,22 @@ import org.apache.hadoop.hive.common.StatsSetupConst;
 import org.apache.hadoop.hive.metastore.api.hive_metastoreConstants;
 import org.apache.hadoop.hive.metastore.IMetaStoreClient;
 import org.apache.hadoop.hive.metastore.TableType;
-import org.apache.iceberg.Schema;
-import org.apache.iceberg.TableMetadata;
-import org.apache.iceberg.types.Types;
 import org.apache.impala.analysis.IcebergPartitionField;
 import org.apache.impala.analysis.IcebergPartitionSpec;
 import org.apache.impala.analysis.IcebergPartitionTransform;
 import org.apache.impala.catalog.HdfsPartition.FileDescriptor;
 import org.apache.impala.thrift.TCatalogObjectType;
-import org.apache.impala.thrift.THdfsFileDesc;
+import org.apache.impala.thrift.TCompressionCodec;
+import org.apache.impala.thrift.TGetPartialCatalogObjectRequest;
+import org.apache.impala.thrift.TGetPartialCatalogObjectResponse;
+import org.apache.impala.thrift.THdfsCompression;
 import org.apache.impala.thrift.THdfsTable;
 import org.apache.impala.thrift.TIcebergCatalog;
 import org.apache.impala.thrift.TIcebergFileFormat;
 import org.apache.impala.thrift.TIcebergPartitionField;
 import org.apache.impala.thrift.TIcebergPartitionSpec;
 import org.apache.impala.thrift.TIcebergTable;
+import org.apache.impala.thrift.TPartialPartitionInfo;
 import org.apache.impala.thrift.TTable;
 import org.apache.impala.thrift.TTableDescriptor;
 import org.apache.impala.thrift.TTableType;
@@ -68,7 +69,7 @@ public class IcebergTable extends Table implements FeIcebergTable {
       "org.apache.iceberg.mr.hive.HiveIcebergStorageHandler";
 
   // Iceberg file format key in tblproperties
-  public static final String ICEBERG_FILE_FORMAT = "iceberg.file_format";
+  public static final String ICEBERG_FILE_FORMAT = "write.format.default";
 
   // Iceberg catalog type key in tblproperties
   public static final String ICEBERG_CATALOG = "iceberg.catalog";
@@ -86,12 +87,61 @@ public class IcebergTable extends Table implements FeIcebergTable {
   // table metadata. This property is only valid for tables in 'hive.catalog'.
   public static final String METADATA_LOCATION = "metadata_location";
 
+  // Parquet compression codec and compression level table properties.
+  public static final String PARQUET_COMPRESSION_CODEC =
+      "write.parquet.compression-codec";
+  public static final String PARQUET_COMPRESSION_LEVEL =
+      "write.parquet.compression-level";
+
+  // Default values for parquet compression codec.
+  public static final THdfsCompression DEFAULT_PARQUET_COMPRESSION_CODEC =
+      THdfsCompression.SNAPPY;
+  // Default values for parquet compression level (used with ZSTD codec).
+  public static final int DEFAULT_PARQUET_ZSTD_COMPRESSION_LEVEL = 3;
+  // Valid range for parquet compression level.
+  public static final int MIN_PARQUET_COMPRESSION_LEVEL = 1;
+  public static final int MAX_PARQUET_COMPRESSION_LEVEL = 22;
+
+  // Parquet row group size table property.
+  public static final String PARQUET_ROW_GROUP_SIZE =
+      "write.parquet.row-group-size-bytes";
+  // 0 means that the table property should be ignored.
+  public static final long UNSET_PARQUET_ROW_GROUP_SIZE = 0;
+  // Valid range for parquet row group size is [8MB, 2047MB]
+  // (see HDFS_MIN_FILE_SIZE defined in hdfs-parquet-table-writer.h)
+  public static final long MIN_PARQUET_ROW_GROUP_SIZE = 8 * 1024 * 1024;
+  public static final long MAX_PARQUET_ROW_GROUP_SIZE = 2047 * 1024 * 1024;
+
+  // Parquet plain page size table property.
+  public static final String PARQUET_PLAIN_PAGE_SIZE = "write.parquet.page-size-bytes";
+  // Parquet dictionary page size table property.
+  public static final String PARQUET_DICT_PAGE_SIZE = "write.parquet.dict-size-bytes";
+  // 0 means that the table property should be ignored.
+  public static final long UNSET_PARQUET_PAGE_SIZE = 0;
+  // Valid range for parquet plain and dictionary page size [64K, 1GB]
+  // (see DEFAULT_DATA_PAGE_SIZE and MAX_DATA_PAGE_SIZE defined in
+  // hdfs-parquet-table-writer.h)
+  public static final long MIN_PARQUET_PAGE_SIZE = 64 * 1024;
+  public static final long MAX_PARQUET_PAGE_SIZE = 1024 * 1024 * 1024;
+
   // Iceberg catalog type dependend on table properties
   private TIcebergCatalog icebergCatalog_;
 
   // Iceberg file format dependend on table properties
   private TIcebergFileFormat icebergFileFormat_;
 
+  // Iceberg parquet compression codec dependent on table properties
+  private TCompressionCodec icebergParquetCompressionCodec_;
+
+  // Iceberg parquet row group size dependent on table property
+  private long icebergParquetRowGroupSize_;
+
+  // Iceberg parquet plain page size dependent on table property
+  private long icebergParquetPlainPageSize_;
+
+  // Iceberg parquet dictionary page size dependent on table property
+  private long icebergParquetDictPageSize_;
+
   // The iceberg file system table location
   private String icebergTableLocation_;
 
@@ -102,23 +152,28 @@ public class IcebergTable extends Table implements FeIcebergTable {
   // last item of the list is the latest.
   private int defaultPartitionSpecId_;
 
-  // Schema of the iceberg table.
-  private Schema icebergSchema_;
-
   // Key is the DataFile path hash, value is FileDescriptor transformed from DataFile
   private Map<String, FileDescriptor> pathHashToFileDescMap_;
 
   // Treat iceberg table as a non-partitioned hdfs table in backend
   private HdfsTable hdfsTable_;
 
-  private long snapshotId_ = -1;
+  // Cached Iceberg API table object.
+  private org.apache.iceberg.Table icebergApiTable_;
+
+  // The snapshot id cached in the CatalogD, necessary to syncronize the caches.
+  private long catalogSnapshotId_ = -1;
 
   protected IcebergTable(org.apache.hadoop.hive.metastore.api.Table msTable,
       Db db, String name, String owner) {
     super(msTable, db, name, owner);
     icebergTableLocation_ = msTable.getSd().getLocation();
     icebergCatalog_ = IcebergUtil.getTIcebergCatalog(msTable);
-    icebergFileFormat_ = Utils.getIcebergFileFormat(msTable);
+    icebergFileFormat_ = IcebergUtil.getIcebergFileFormat(msTable);
+    icebergParquetCompressionCodec_ = Utils.getIcebergParquetCompressionCodec(msTable);
+    icebergParquetRowGroupSize_ = Utils.getIcebergParquetRowGroupSize(msTable);
+    icebergParquetPlainPageSize_ = Utils.getIcebergParquetPlainPageSize(msTable);
+    icebergParquetDictPageSize_ = Utils.getIcebergParquetDictPageSize(msTable);
     hdfsTable_ = new HdfsTable(msTable, db, name, owner);
   }
 
@@ -145,6 +200,11 @@ public class IcebergTable extends Table implements FeIcebergTable {
     return hdfsTable_;
   }
 
+  @Override
+  public org.apache.iceberg.Table getIcebergApiTable() {
+    return icebergApiTable_;
+  }
+
   @Override
   public TCatalogObjectType getCatalogObjectType() {
     return TCatalogObjectType.TABLE;
@@ -163,11 +223,6 @@ public class IcebergTable extends Table implements FeIcebergTable {
     return ICEBERG_STORAGE_HANDLER;
   }
 
-  @Override
-  public Schema getIcebergSchema() {
-    return icebergSchema_;
-  }
-
   public static boolean isIcebergStorageHandler(String handler) {
     return handler != null && handler.equals(ICEBERG_STORAGE_HANDLER);
   }
@@ -191,6 +246,26 @@ public class IcebergTable extends Table implements FeIcebergTable {
     return icebergFileFormat_;
   }
 
+  @Override
+  public TCompressionCodec getIcebergParquetCompressionCodec() {
+    return icebergParquetCompressionCodec_;
+  }
+
+  @Override
+  public long getIcebergParquetRowGroupSize() {
+    return icebergParquetRowGroupSize_;
+  }
+
+  @Override
+  public long getIcebergParquetPlainPageSize() {
+    return icebergParquetPlainPageSize_;
+  }
+
+  @Override
+  public long getIcebergParquetDictPageSize() {
+    return icebergParquetDictPageSize_;
+  }
+
   @Override
   public String getIcebergTableLocation() {
     return icebergTableLocation_;
@@ -213,7 +288,9 @@ public class IcebergTable extends Table implements FeIcebergTable {
   }
 
   @Override
-  public int getDefaultPartitionSpecId() { return defaultPartitionSpecId_; }
+  public int getDefaultPartitionSpecId() {
+    return defaultPartitionSpecId_;
+  }
 
   @Override
   public Map<String, FileDescriptor> getPathHashToFileDescMap() {
@@ -222,7 +299,7 @@ public class IcebergTable extends Table implements FeIcebergTable {
 
   @Override
   public long snapshotId() {
-    return snapshotId_;
+    return catalogSnapshotId_;
   }
 
   @Override
@@ -255,20 +332,23 @@ public class IcebergTable extends Table implements FeIcebergTable {
       final Timer.Context ctxStorageLdTime =
           getMetrics().getTimer(Table.LOAD_DURATION_STORAGE_METADATA).time();
       try {
-        TableMetadata metadata = IcebergUtil.getIcebergTableMetadata(this);
-        if (metadata.currentSnapshot() != null) {
-            snapshotId_ = metadata.currentSnapshot().snapshotId();
-        }
-        loadSchemaFromIceberg(metadata);
+        icebergApiTable_ = IcebergUtil.loadTable(this);
+        catalogSnapshotId_ = FeIcebergTable.super.snapshotId();
+        loadSchemaFromIceberg();
         // Loading hdfs table after loaded schema from Iceberg,
         // in case we create external Iceberg table skipping column info in sql.
-        icebergFileFormat_ = Utils.getIcebergFileFormat(msTbl);
-        hdfsTable_.load(false, msClient, msTable_, true, true, false, null, null, reason);
+        icebergFileFormat_ = IcebergUtil.getIcebergFileFormat(msTbl);
+        icebergParquetCompressionCodec_ = Utils.getIcebergParquetCompressionCodec(msTbl);
+        icebergParquetRowGroupSize_ = Utils.getIcebergParquetRowGroupSize(msTbl);
+        icebergParquetPlainPageSize_ = Utils.getIcebergParquetPlainPageSize(msTbl);
+        icebergParquetDictPageSize_ = Utils.getIcebergParquetDictPageSize(msTbl);
+        hdfsTable_
+            .load(false, msClient, msTable_, true, true, false, null, null,null, reason);
         pathHashToFileDescMap_ = Utils.loadAllPartition(this);
         loadAllColumnStats(msClient);
       } catch (Exception e) {
-        throw new TableLoadingException("Error loading metadata for Iceberg table " +
-            icebergTableLocation_, e);
+        throw new IcebergTableLoadingException("Error loading metadata for Iceberg table "
+            + icebergTableLocation_, e);
       } finally {
         storageMetadataLoadTime_ = ctxStorageLdTime.stop();
       }
@@ -295,11 +375,10 @@ public class IcebergTable extends Table implements FeIcebergTable {
   /**
    * Load schema and partitioning schemes directly from Iceberg.
    */
-  public void loadSchemaFromIceberg(TableMetadata metadata) throws TableLoadingException {
-    icebergSchema_ = metadata.schema();
+  public void loadSchemaFromIceberg() throws TableLoadingException {
     loadSchema();
-    partitionSpecs_ = Utils.loadPartitionSpecByIceberg(metadata);
-    defaultPartitionSpecId_ = metadata.defaultSpecId();
+    partitionSpecs_ = Utils.loadPartitionSpecByIceberg(this);
+    defaultPartitionSpecId_ = icebergApiTable_.spec().specId();
   }
 
   /**
@@ -307,8 +386,9 @@ public class IcebergTable extends Table implements FeIcebergTable {
    */
   private void loadSchema() throws TableLoadingException {
     clearColumns();
-    msTable_.getSd().setCols(IcebergSchemaConverter.convertToHiveSchema(icebergSchema_));
-    for (Column col : IcebergSchemaConverter.convertToImpalaSchema(icebergSchema_)) {
+    msTable_.getSd().setCols(IcebergSchemaConverter.convertToHiveSchema(
+        getIcebergSchema()));
+    for (Column col : IcebergSchemaConverter.convertToImpalaSchema(getIcebergSchema())) {
       addColumn(col);
     }
   }
@@ -329,14 +409,21 @@ public class IcebergTable extends Table implements FeIcebergTable {
     super.loadFromThrift(thriftTable);
     TIcebergTable ticeberg = thriftTable.getIceberg_table();
     icebergTableLocation_ = ticeberg.getTable_location();
+    icebergParquetCompressionCodec_ = ticeberg.getParquet_compression_codec();
+    icebergParquetRowGroupSize_ = ticeberg.getParquet_row_group_size();
+    icebergParquetPlainPageSize_ = ticeberg.getParquet_plain_page_size();
+    icebergParquetDictPageSize_ = ticeberg.getParquet_dict_page_size();
     partitionSpecs_ = loadPartitionBySpecsFromThrift(ticeberg.getPartition_spec());
     defaultPartitionSpecId_ = ticeberg.getDefault_partition_spec_id();
-    pathHashToFileDescMap_ = loadFileDescFromThrift(
-        ticeberg.getPath_hash_to_file_descriptor());
-    snapshotId_ = ticeberg.getSnapshot_id();
+    // Load file descriptors for the Iceberg snapshot. We are using the same host index,
+    // so there's no need for translation.
+    catalogSnapshotId_ = ticeberg.catalog_snapshot_id;
+    // The Iceberg API table needs to be available and cached even when loaded through
+    // thrift.
+    icebergApiTable_ = IcebergUtil.loadTable(this);
+    pathHashToFileDescMap_ = FeIcebergTable.Utils.loadFileDescMapFromThrift(
+        ticeberg.getPath_hash_to_file_descriptor(), null, null);
     hdfsTable_.loadFromThrift(thriftTable);
-    TableMetadata metadata = IcebergUtil.getIcebergTableMetadata(this);
-    icebergSchema_ = metadata.schema();
   }
 
   private List<IcebergPartitionSpec> loadPartitionBySpecsFromThrift(
@@ -366,16 +453,6 @@ public class IcebergTable extends Table implements FeIcebergTable {
     return ret;
   }
 
-  private Map<String, FileDescriptor> loadFileDescFromThrift(
-      Map<String, THdfsFileDesc> tFileDescMap) {
-    Map<String, FileDescriptor> fileDescMap = new HashMap<>();
-    if (tFileDescMap == null) return fileDescMap;
-    for (Map.Entry<String, THdfsFileDesc> entry : tFileDescMap.entrySet()) {
-      fileDescMap.put(entry.getKey(), FileDescriptor.fromThrift(entry.getValue()));
-    }
-    return fileDescMap;
-  }
-
   @Override
   public TTableDescriptor toThriftDescriptor(int tableId,
       Set<Long> referencedPartitions) {
@@ -395,4 +472,21 @@ public class IcebergTable extends Table implements FeIcebergTable {
     }
     return hdfsTable;
   }
+
+  @Override
+  public TGetPartialCatalogObjectResponse getPartialInfo(
+      TGetPartialCatalogObjectRequest req) throws CatalogException {
+    Preconditions.checkState(isLoaded(), "unloaded table: %s", getFullName());
+    Map<HdfsPartition, TPartialPartitionInfo> missingPartialInfos = new HashMap<>();
+    TGetPartialCatalogObjectResponse resp =
+        getHdfsTable().getPartialInfo(req, missingPartialInfos);
+    if (req.table_info_selector.want_iceberg_table) {
+      resp.table_info.setIceberg_table(Utils.getTIcebergTable(this));
+      if (!resp.table_info.isSetNetwork_addresses()) {
+        resp.table_info.setNetwork_addresses(getHostIndex().getList());
+      }
+      resp.table_info.iceberg_table.setCatalog_snapshot_id(catalogSnapshotId_);
+    }
+    return resp;
+  }
 }
