@@ -57,11 +57,12 @@ import org.apache.impala.thrift.TTableInfoSelector;
 import org.apache.impala.thrift.TTableStats;
 import org.apache.impala.util.AcidUtils;
 import org.apache.impala.util.HdfsCachingUtil;
-import org.apache.log4j.Logger;
 
 import com.codahale.metrics.Timer;
 import com.google.common.base.Preconditions;
 import com.google.common.collect.Lists;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * Base class for table metadata.
@@ -73,7 +74,7 @@ import com.google.common.collect.Lists;
  * a key range into a fixed number of buckets).
  */
 public abstract class Table extends CatalogObjectImpl implements FeTable {
-  private static final Logger LOG = Logger.getLogger(Table.class);
+  private static final Logger LOG = LoggerFactory.getLogger(Table.class);
   protected org.apache.hadoop.hive.metastore.api.Table msTable_;
   protected final Db db_;
   protected final String name_;
@@ -137,6 +138,10 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
   // impalad.
   protected long lastUsedTime_;
 
+  // Represents the event id in the metastore which pertains to the creation of this
+  // table. Defaults to -1 for a preexisting table or if events processing is not active.
+  protected volatile long createEventId_ = -1;
+
   // tracks the in-flight metastore events for this table. Used by Events processor to
   // avoid unnecessary refresh when the event is received
   private final InFlightEvents inFlightEvents = new InFlightEvents();
@@ -163,6 +168,9 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
   public static final String LOAD_DURATION_ALL_COLUMN_STATS =
       "load-duration.all-column-stats";
 
+  // metric key for the number of in-flight events for this table.
+  public static final String NUMBER_OF_INFLIGHT_EVENTS = "num-inflight-events";
+
   // Table property key for storing the time of the last DDL operation.
   public static final String TBL_PROP_LAST_DDL_TIME = "transient_lastDdlTime";
 
@@ -176,6 +184,13 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
   // Table property key to determined if HMS translated a managed table to external table
   public static final String TBL_PROP_EXTERNAL_TABLE_PURGE = "external.table.purge";
 
+  // this field represents the last event id in metastore upto which this table is
+  // synced. It is used if the flag sync_to_latest_event_on_ddls is set to true.
+  // Making it as volatile so that read and write of this variable are thread safe.
+  // As an example, EventProcessor can check if it needs to process a table event or
+  // not by reading this flag and without acquiring read lock on table object
+  protected volatile long lastSyncedEventId_ = -1;
+
   protected Table(org.apache.hadoop.hive.metastore.api.Table msTable, Db db,
       String name, String owner) {
     msTable_ = msTable;
@@ -187,6 +202,34 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
     initMetrics();
   }
 
+  // TODO: Get rid of get and createEventId
+  // once we implement the logic of setting
+  // last synced id in full table reload
+  public long getCreateEventId() { return createEventId_; }
+
+  public void setCreateEventId(long eventId) {
+    // TODO: Add a preconditions check for eventId < lastSycnedEventId
+    createEventId_ = eventId;
+    LOG.debug("createEventId_ for table: {} set to: {}", getFullName(), createEventId_);
+    // TODO: Should we reset lastSyncedEvent Id if it is less than event Id?
+    // If we don't reset it - we may start syncing table from an event id which
+    // is less than create event id
+    if (lastSyncedEventId_ < eventId) {
+      setLastSyncedEventId(eventId);
+    }
+  }
+
+  public long getLastSyncedEventId() {
+    return lastSyncedEventId_;
+  }
+
+  public void setLastSyncedEventId(long eventId) {
+    // TODO: Add a preconditions check for eventId >= createEventId_
+    LOG.debug("lastSyncedEventId_ for table: {} set from {} to {}", getFullName(),
+        lastSyncedEventId_, eventId);
+    lastSyncedEventId_ = eventId;
+  }
+
   /**
    * Returns if the given HMS table is an external table (uses table type if
    * available or else uses table properties). Implementation is based on org.apache
@@ -324,6 +367,7 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
     metrics_.addTimer(LOAD_DURATION_STORAGE_METADATA);
     metrics_.addTimer(HMS_LOAD_TBL_SCHEMA);
     metrics_.addTimer(LOAD_DURATION_ALL_COLUMN_STATS);
+    metrics_.addCounter(NUMBER_OF_INFLIGHT_EVENTS);
   }
 
   public Metrics getMetrics() { return metrics_; }
@@ -433,7 +477,13 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
     // Create a table of appropriate type
     if (MetadataOp.TABLE_TYPE_VIEW.equals(
           MetastoreShim.mapToInternalTableType(msTbl.getTableType()))) {
-      table = new View(msTbl, db, msTbl.getTableName(), msTbl.getOwner());
+      if (msTbl.getTableType().equalsIgnoreCase("MATERIALIZED_VIEW") &&
+          HdfsFileFormat.isHdfsInputFormatClass(msTbl.getSd().getInputFormat())) {
+        table = new MaterializedViewHdfsTable(msTbl, db, msTbl.getTableName(),
+            msTbl.getOwner());
+      } else {
+        table = new View(msTbl, db, msTbl.getTableName(), msTbl.getOwner());
+      }
     } else if (HBaseTable.isHBaseTable(msTbl)) {
       table = new HBaseTable(msTbl, db, msTbl.getTableName(), msTbl.getOwner());
     } else if (KuduTable.isKuduTable(msTbl)) {
@@ -660,10 +710,13 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
       // is done while we continue to hold the table lock.
       resp.table_info.setHms_table(getMetaStoreTable().deepCopy());
     }
-    if (selector.want_stats_for_column_names != null) {
-      List<ColumnStatisticsObj> statsList = Lists.newArrayListWithCapacity(
-          selector.want_stats_for_column_names.size());
-      for (String colName: selector.want_stats_for_column_names) {
+    if (selector.want_stats_for_column_names != null ||
+        selector.want_stats_for_all_columns) {
+      List<String> colList = selector.want_stats_for_all_columns ? getColumnNames() :
+          selector.want_stats_for_column_names;
+      List<ColumnStatisticsObj> statsList =
+          Lists.newArrayListWithCapacity(colList.size());
+      for (String colName: colList) {
         Column col = getColumn(colName);
         if (col == null) continue;
 
@@ -691,6 +744,7 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
     }
     return resp;
   }
+
   /**
    * @see FeCatalogUtils#parseColumnType(FieldSchema, String)
    */
@@ -868,7 +922,11 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
     Preconditions.checkState(isWriteLockedByCurrentThread(),
         "removeFromVersionsForInFlightEvents called without taking the table lock on "
             + getFullName());
-    return inFlightEvents.remove(isInsertEvent, versionNumber);
+    boolean removed = inFlightEvents.remove(isInsertEvent, versionNumber);
+    if (removed) {
+      metrics_.getCounter(NUMBER_OF_INFLIGHT_EVENTS).dec();
+    }
+    return removed;
   }
 
   /**
@@ -892,6 +950,8 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
       LOG.warn(String.format("Could not add %s version to the table %s. This could "
           + "cause unnecessary refresh of the table when the event is received by the "
               + "Events processor.", versionNumber, getFullName()));
+    } else {
+      metrics_.getCounter(NUMBER_OF_INFLIGHT_EVENTS).inc();
     }
   }
 
