@@ -20,6 +20,7 @@ package org.apache.impala.catalog;
 import java.util.ArrayList;
 import java.util.List;
 
+import org.apache.impala.analysis.CollectionStructType;
 import org.apache.impala.analysis.CreateTableStmt;
 import org.apache.impala.analysis.Parser;
 import org.apache.impala.analysis.StatementBase;
@@ -234,6 +235,9 @@ public abstract class Type {
   public boolean isMapType() { return this instanceof MapType; }
   public boolean isArrayType() { return this instanceof ArrayType; }
   public boolean isStructType() { return this instanceof StructType; }
+  public boolean isCollectionStructType() {
+    return this instanceof CollectionStructType;
+  }
 
   /**
    * Returns true if Impala supports this type in the metdata. It does not mean we
@@ -346,13 +350,16 @@ public abstract class Type {
    * casting between any two types other than both decimals.
    *
    *
-   * TODO: Support non-scalar types.
+   * TODO: Support map and struct types.
    */
   public static Type getAssignmentCompatibleType(
       Type t1, Type t2, boolean strict, boolean strictDecimal) {
     if (t1.isScalarType() && t2.isScalarType()) {
       return ScalarType.getAssignmentCompatibleType(
           (ScalarType) t1, (ScalarType) t2, strict, strictDecimal);
+    } else if (t1.isArrayType() && t2.isArrayType()) {
+      // Only support exact match for array types.
+      if (t1.equals(t2)) return t2;
     }
     return ScalarType.INVALID;
   }
