@@ -26,6 +26,7 @@ import java.util.Set;
 
 import com.google.common.base.Preconditions;
 
+import org.apache.iceberg.CatalogProperties;
 import org.apache.iceberg.PartitionSpec;
 import org.apache.iceberg.Schema;
 import org.apache.iceberg.catalog.Namespace;
@@ -46,6 +47,7 @@ import org.apache.impala.catalog.HdfsStorageDescriptor;
 import org.apache.impala.catalog.HdfsTable;
 import org.apache.impala.catalog.IcebergColumn;
 import org.apache.impala.catalog.IcebergStructField;
+import org.apache.impala.catalog.IcebergTable;
 import org.apache.impala.catalog.HdfsPartition.FileDescriptor;
 import org.apache.impala.catalog.local.LocalDb;
 import org.apache.impala.catalog.local.LocalFsTable;
@@ -54,6 +56,7 @@ import org.apache.impala.catalog.StructType;
 import org.apache.impala.thrift.CatalogObjectsConstants;
 import org.apache.impala.thrift.TCatalogObjectType;
 import org.apache.impala.thrift.TColumn;
+import org.apache.impala.thrift.TCompressionCodec;
 import org.apache.impala.thrift.THdfsPartition;
 import org.apache.impala.thrift.THdfsStorageDescriptor;
 import org.apache.impala.thrift.THdfsTable;
@@ -73,6 +76,10 @@ public class IcebergCtasTarget extends CtasTargetTable implements FeIcebergTable
   private Schema iceSchema_;
   private List<IcebergPartitionSpec> partitionSpecs_ = new ArrayList<>();
   private TIcebergFileFormat icebergFileFormat_;
+  private TCompressionCodec icebergParquetCompressionCodec_;
+  private long icebergParquetRowGroupSize_;
+  private long icebergParquetPlainPageSize_;
+  private long icebergParquetDictPageSize_;
   private TIcebergCatalog icebergCatalog_;
   private String icebergTableLocation_;
   private String icebergCatalogLocation_;
@@ -87,7 +94,11 @@ public class IcebergCtasTarget extends CtasTargetTable implements FeIcebergTable
     createPartitionSpec(partSpec);
     icebergCatalog_ = IcebergUtil.getTIcebergCatalog(msTbl);
     setLocations();
-    icebergFileFormat_ = Utils.getIcebergFileFormat(msTbl);
+    icebergFileFormat_ = IcebergUtil.getIcebergFileFormat(msTbl);
+    icebergParquetCompressionCodec_ = Utils.getIcebergParquetCompressionCodec(msTbl);
+    icebergParquetRowGroupSize_ = Utils.getIcebergParquetRowGroupSize(msTbl);
+    icebergParquetPlainPageSize_ = Utils.getIcebergParquetPlainPageSize(msTbl);
+    icebergParquetDictPageSize_ = Utils.getIcebergParquetDictPageSize(msTbl);
     hdfsSd_ = HdfsStorageDescriptor.fromStorageDescriptor(name_, msTable_.getSd());
   }
 
@@ -134,8 +145,15 @@ public class IcebergCtasTarget extends CtasTargetTable implements FeIcebergTable
   private void setLocations() {
     Preconditions.checkState(msTable_ != null);
     Preconditions.checkState(icebergCatalog_ != null);
-    if (icebergCatalog_ == TIcebergCatalog.HADOOP_CATALOG) {
-      icebergCatalogLocation_ = IcebergUtil.getIcebergCatalogLocation(msTable_);
+    TIcebergCatalog underlyingCatalog = IcebergUtil.getUnderlyingCatalog(msTable_);
+    if (underlyingCatalog == TIcebergCatalog.HADOOP_CATALOG) {
+      if (icebergCatalog_ == TIcebergCatalog.CATALOGS) {
+        String catName = msTable_.getParameters().get(IcebergTable.ICEBERG_CATALOG);
+        icebergCatalogLocation_ = IcebergCatalogs.getInstance().getCatalogProperty(
+            catName, CatalogProperties.WAREHOUSE_LOCATION);
+      } else {
+        icebergCatalogLocation_ = IcebergUtil.getIcebergCatalogLocation(msTable_);
+      }
       TableIdentifier tId = IcebergUtil.getIcebergTableIdentifier(msTable_);
       Namespace ns = tId.namespace();
       List<String> components = new ArrayList<>();
@@ -146,7 +164,8 @@ public class IcebergCtasTarget extends CtasTargetTable implements FeIcebergTable
       return;
     }
     Preconditions.checkState(icebergCatalog_ == TIcebergCatalog.HADOOP_TABLES ||
-                             icebergCatalog_ == TIcebergCatalog.HIVE_CATALOG);
+                             icebergCatalog_ == TIcebergCatalog.HIVE_CATALOG ||
+                             icebergCatalog_ == TIcebergCatalog.CATALOGS);
     icebergTableLocation_ = msTable_.getSd().getLocation();
     icebergCatalogLocation_ = icebergTableLocation_;
   }
@@ -185,6 +204,26 @@ public class IcebergCtasTarget extends CtasTargetTable implements FeIcebergTable
     return icebergFileFormat_;
   }
 
+  @Override
+  public TCompressionCodec getIcebergParquetCompressionCodec() {
+    return icebergParquetCompressionCodec_;
+  }
+
+  @Override
+  public long getIcebergParquetRowGroupSize() {
+    return icebergParquetRowGroupSize_;
+  }
+
+  @Override
+  public long getIcebergParquetPlainPageSize() {
+    return icebergParquetPlainPageSize_;
+  }
+
+  @Override
+  public long getIcebergParquetDictPageSize() {
+    return icebergParquetDictPageSize_;
+  }
+
   @Override
   public String getIcebergTableLocation() {
     return icebergTableLocation_;
@@ -215,6 +254,11 @@ public class IcebergCtasTarget extends CtasTargetTable implements FeIcebergTable
     return -1;
   }
 
+  @Override
+  public org.apache.iceberg.Table getIcebergApiTable() {
+    return null;
+  }
+
   public void addColumn(IcebergColumn col) {
     colsByPos_.add(col);
     colsByName_.put(col.getName().toLowerCase(), col);
