@@ -33,7 +33,9 @@ import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicLong;
 import java.util.concurrent.atomic.AtomicReference;
+import java.util.stream.Collectors;
 
+import com.google.common.annotations.VisibleForTesting;
 import com.google.common.collect.ImmutableCollection;
 import com.google.common.collect.ImmutableMap;
 import org.apache.commons.lang3.StringUtils;
@@ -61,8 +63,10 @@ import org.apache.impala.catalog.ImpaladCatalog.ObjectUpdateSequencer;
 import org.apache.impala.catalog.Principal;
 import org.apache.impala.catalog.PrincipalPrivilege;
 import org.apache.impala.catalog.SqlConstraints;
+import org.apache.impala.catalog.local.LocalIcebergTable.TableParams;
 import org.apache.impala.common.InternalException;
 import org.apache.impala.common.Pair;
+import org.apache.impala.service.BackendConfig;
 import org.apache.impala.service.FeSupport;
 import org.apache.impala.service.FrontendProfile;
 import org.apache.impala.service.MetadataOp;
@@ -82,6 +86,7 @@ import org.apache.impala.thrift.TGetPartialCatalogObjectResponse;
 import org.apache.impala.thrift.THdfsFileDesc;
 import org.apache.impala.thrift.TNetworkAddress;
 import org.apache.impala.thrift.TPartialPartitionInfo;
+import org.apache.impala.thrift.TPartialTableInfo;
 import org.apache.impala.thrift.TTable;
 import org.apache.impala.thrift.TTableInfoSelector;
 import org.apache.impala.thrift.TUniqueId;
@@ -89,17 +94,19 @@ import org.apache.impala.thrift.TUnit;
 import org.apache.impala.thrift.TUpdateCatalogCacheRequest;
 import org.apache.impala.thrift.TUpdateCatalogCacheResponse;
 import org.apache.impala.thrift.TValidWriteIdList;
+import org.apache.impala.util.AcidUtils;
+import org.apache.impala.util.IcebergUtil;
 import org.apache.impala.util.ListMap;
-import org.apache.impala.util.TByteBuffer;
+import org.apache.impala.util.ThreadNameAnnotator;
 import org.apache.thrift.TDeserializer;
 import org.apache.thrift.TException;
 import org.apache.thrift.TSerializer;
 import org.apache.thrift.protocol.TBinaryProtocol;
+import org.apache.thrift.transport.TByteBuffer;
 import org.ehcache.sizeof.SizeOf;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Objects;
 import com.google.common.base.Preconditions;
 import com.google.common.base.Stopwatch;
@@ -505,7 +512,8 @@ public class CatalogdMetaProvider implements MetaProvider {
     Stopwatch sw = Stopwatch.createStarted();
     boolean hit = false;
     boolean isPiggybacked = false;
-    try {
+    try (ThreadNameAnnotator tna = new ThreadNameAnnotator(
+        "LoadWithCaching for " + itemString)) {
       CompletableFuture<Object> f = new CompletableFuture<Object>();
       // NOTE: the Cache ensures that this is an atomic operation of either returning
       // an existing value or inserting our own. Only one thread can think it is the
@@ -602,7 +610,7 @@ public class CatalogdMetaProvider implements MetaProvider {
             req.catalog_info_selector.want_db_names = true;
             TGetPartialCatalogObjectResponse resp = sendRequest(req);
             checkResponse(resp.catalog_info != null && resp.catalog_info.db_names != null,
-                req, "missing table names");
+                req, "missing database names");
             return ImmutableList.copyOf(resp.catalog_info.db_names);
           }
     });
@@ -891,6 +899,18 @@ public class CatalogdMetaProvider implements MetaProvider {
     // Load what we can from the cache.
     Map<PartitionRef, PartitionMetadata> refToMeta = loadPartitionsFromCache(refImpl,
         hostIndex, partitionRefs);
+    if (BackendConfig.INSTANCE.isAutoCheckCompaction()) {
+      // If any partitions are stale after compaction, they will be removed from refToMeta
+      List<PartitionRef> stalePartitions = directProvider_.checkLatestCompaction(
+          refImpl.dbName_, refImpl.tableName_, refImpl, refToMeta);
+      cache_.invalidateAll(stalePartitions.stream()
+          .map(PartitionRefImpl.class::cast)
+          .map(PartitionRefImpl::getId)
+          .map(PartitionCacheKey::new)
+          .collect(Collectors.toList()));
+      LOG.debug("Checked the latest compaction id for {}.{}", refImpl.dbName_,
+          refImpl.tableName_);
+    }
 
     final int numHits = refToMeta.size();
     final int numMisses = partitionRefs.size() - numHits;
@@ -937,6 +957,9 @@ public class CatalogdMetaProvider implements MetaProvider {
     req.table_info_selector.partition_ids = ids;
     req.table_info_selector.want_partition_metadata = true;
     req.table_info_selector.want_partition_files = true;
+    if (BackendConfig.INSTANCE.isAutoCheckCompaction()) {
+      req.table_info_selector.valid_write_ids = table.validWriteIds_;
+    }
     // TODO(todd): fetch incremental stats on-demand for compute-incremental-stats.
     req.table_info_selector.want_partition_stats = true;
     TGetPartialCatalogObjectResponse resp = sendRequest(req);
@@ -988,7 +1011,8 @@ public class CatalogdMetaProvider implements MetaProvider {
       PartitionMetadataImpl metaImpl = new PartitionMetadataImpl(part.getHms_parameters(),
           part.write_id, hdfsStorageDescriptor,
           fds, insertFds, deleteFds, part.getPartition_stats(),
-          part.has_incremental_stats, part.is_marked_cached, location);
+          part.has_incremental_stats, part.is_marked_cached, location,
+          part.last_compaction_id);
 
       checkResponse(partRef != null, req, "returned unexpected partition id %s", part.id);
 
@@ -1001,6 +1025,49 @@ public class CatalogdMetaProvider implements MetaProvider {
     return ret;
   }
 
+  @Override
+  public TPartialTableInfo loadIcebergTable(final TableMetaRef table) throws TException {
+    Preconditions.checkArgument(table instanceof TableMetaRefImpl);
+    TableMetaRefImpl tableRef = (TableMetaRefImpl)table;
+    String itemStr = "iceberg metadata for " + tableRef.dbName_ + "."
+        + tableRef.tableName_;
+    IcebergMetaCacheKey cacheKey = new IcebergMetaCacheKey(tableRef);
+    return loadWithCaching(itemStr, TABLE_METADATA_CACHE_CATEGORY, cacheKey,
+        new Callable<TPartialTableInfo>() {
+          @Override
+          public TPartialTableInfo call() throws Exception {
+            TGetPartialCatalogObjectRequest req = newReqForTable(table);
+            req.table_info_selector.want_iceberg_table = true;
+            TGetPartialCatalogObjectResponse resp = sendRequest(req);
+            checkResponse(resp.table_info != null &&
+                resp.table_info.iceberg_table != null, req,
+                "missing Iceberg table metadata");
+            return resp.getTable_info();
+          }
+    });
+  }
+
+  @Override
+  public org.apache.iceberg.Table loadIcebergApiTable(final TableMetaRef table,
+      TableParams params, Table msTable) throws TException {
+    Preconditions.checkArgument(table instanceof TableMetaRefImpl);
+    TableMetaRefImpl tableRef = (TableMetaRefImpl)table;
+    String itemStr = "iceberg api table for " + tableRef.dbName_ + "."
+        + tableRef.tableName_;
+    IcebergApiTableCacheKey cacheKey = new IcebergApiTableCacheKey(tableRef);
+    return loadWithCaching(itemStr, TABLE_METADATA_CACHE_CATEGORY, cacheKey,
+        new Callable<org.apache.iceberg.Table>() {
+          @Override
+          public org.apache.iceberg.Table call() throws Exception {
+            return IcebergUtil.loadTable(
+                params.getIcebergCatalog(),
+                IcebergUtil.getIcebergTableIdentifier(msTable),
+                params.getIcebergCatalogLocation(),
+                msTable.getParameters());
+          }
+        });
+    }
+
   private ImmutableList<FileDescriptor> convertThriftFdList(List<THdfsFileDesc> thriftFds,
       List<TNetworkAddress> networkAddresses, ListMap<TNetworkAddress> hostIndex) {
     List<FileDescriptor> fds = Lists.newArrayListWithCapacity(thriftFds.size());
@@ -1531,12 +1598,13 @@ public class CatalogdMetaProvider implements MetaProvider {
     private final byte[] partitionStats_;
     private final boolean hasIncrementalStats_;
     private final boolean isMarkedCached_;
+    private final long lastCompactionId_;
 
     public PartitionMetadataImpl(Map<String, String> hmsParameters, long writeId,
         HdfsStorageDescriptor hdfsStorageDescriptor, ImmutableList<FileDescriptor> fds,
         ImmutableList<FileDescriptor> insertFds, ImmutableList<FileDescriptor> deleteFds,
         byte[] partitionStats, boolean hasIncrementalStats, boolean isMarkedCached,
-        HdfsPartitionLocationCompressor.Location location) {
+        HdfsPartitionLocationCompressor.Location location, long lastCompactionId) {
       this.hmsParameters_ = hmsParameters;
       this.writeId_ = writeId;
       this.hdfsStorageDescriptor_ = hdfsStorageDescriptor;
@@ -1547,6 +1615,7 @@ public class CatalogdMetaProvider implements MetaProvider {
       this.partitionStats_ = partitionStats;
       this.hasIncrementalStats_ = hasIncrementalStats;
       this.isMarkedCached_ = isMarkedCached;
+      this.lastCompactionId_ = lastCompactionId;
     }
 
     /**
@@ -1564,7 +1633,7 @@ public class CatalogdMetaProvider implements MetaProvider {
           deleteFds_, origIndex, dstIndex);
       return new PartitionMetadataImpl(hmsParameters_, writeId_, hdfsStorageDescriptor_,
           fds, insertFds, deleteFds, partitionStats_, hasIncrementalStats_,
-          isMarkedCached_, location_);
+          isMarkedCached_, location_, lastCompactionId_);
     }
 
     private static ImmutableList<FileDescriptor> cloneFdsRelativeToHostIndex(
@@ -1619,6 +1688,9 @@ public class CatalogdMetaProvider implements MetaProvider {
 
     @Override
     public boolean isMarkedCached() { return isMarkedCached_; }
+
+    @Override
+    public long getLastCompactionId() { return lastCompactionId_; }
   }
 
   /**
@@ -1681,6 +1753,12 @@ public class CatalogdMetaProvider implements MetaProvider {
       return String.format("TableMetaRef %s.%s@%d", dbName_, tableName_, catalogVersion_);
     }
 
+    @Override
+    public boolean isPartitioned() {
+      return msTable_.getPartitionKeysSize() != 0;
+    }
+
+    @Override
     public boolean isMarkedCached() { return isMarkedCached_; }
 
     public HdfsPartitionLocationCompressor getPartitionLocationCompressor() {
@@ -1690,6 +1768,11 @@ public class CatalogdMetaProvider implements MetaProvider {
     public List<String> getPartitionPrefixes() {
       return partitionLocationCompressor_.getPrefixes();
     }
+
+    @Override
+    public boolean isTransactional() {
+      return AcidUtils.isTransactionalTable(msTable_.getParameters());
+    }
   }
 
   /**
@@ -1873,6 +1956,30 @@ public class CatalogdMetaProvider implements MetaProvider {
     }
   }
 
+  /**
+   * Cache key for an entry storing Iceberg table metadata.
+   *
+   * Values for these keys are 'TPartialTableInfo' objects.
+   */
+  private static class IcebergMetaCacheKey extends VersionedTableCacheKey {
+
+    public IcebergMetaCacheKey(TableMetaRefImpl table) {
+      super(table);
+    }
+  }
+
+  /**
+   * Cache key for an entry storing Iceberg API metadata.
+   *
+   * Values for these keys are 'org.apache.iceberg.Table' objects.
+   */
+  private static class IcebergApiTableCacheKey extends VersionedTableCacheKey {
+
+    public IcebergApiTableCacheKey(TableMetaRefImpl table) {
+      super(table);
+    }
+  }
+
   @VisibleForTesting
   static class SizeOfWeigher implements Weigher<Object, Object> {
     // Bypass flyweight objects like small boxed integers, Boolean.TRUE, enums, etc.
