@@ -46,13 +46,16 @@ import org.apache.impala.catalog.HdfsStorageDescriptor;
 import org.apache.impala.catalog.MetaStoreClientPool;
 import org.apache.impala.catalog.MetaStoreClientPool.MetaStoreClient;
 import org.apache.impala.catalog.SqlConstraints;
+import org.apache.impala.catalog.local.LocalIcebergTable.TableParams;
 import org.apache.impala.common.Pair;
 import org.apache.impala.compat.MetastoreShim;
 import org.apache.impala.service.BackendConfig;
 import org.apache.impala.thrift.TBackendGflags;
 import org.apache.impala.thrift.TBriefTableMeta;
 import org.apache.impala.thrift.TNetworkAddress;
+import org.apache.impala.thrift.TPartialTableInfo;
 import org.apache.impala.thrift.TValidWriteIdList;
+import org.apache.impala.util.AcidUtils;
 import org.apache.impala.util.ListMap;
 import org.apache.impala.util.MetaStoreUtil;
 import org.apache.thrift.TException;
@@ -65,12 +68,15 @@ import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 import com.google.errorprone.annotations.Immutable;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * Metadata provider which calls out directly to the source systems
  * (filesystem, HMS, etc) with no caching.
  */
 class DirectMetaProvider implements MetaProvider {
+  private final static Logger LOG = LoggerFactory.getLogger(DirectMetaProvider.class);
   private static MetaStoreClientPool msClientPool_;
 
   DirectMetaProvider() {
@@ -406,7 +412,9 @@ class DirectMetaProvider implements MetaProvider {
     public Map<String, String> getHmsParameters() { return msPartition_.getParameters(); }
 
     @Override
-    public long getWriteId() { return msPartition_.getWriteId(); }
+    public long getWriteId() {
+      return MetastoreShim.getWriteIdFromMSPartition(msPartition_);
+    }
 
     @Override
     public HdfsStorageDescriptor getInputFormatDescriptor() {
@@ -463,6 +471,12 @@ class DirectMetaProvider implements MetaProvider {
       throw new UnsupportedOperationException("Hdfs caching not supported with " +
           "DirectMetaProvider implementation");
     }
+
+    @Override
+    public long getLastCompactionId() {
+      throw new UnsupportedOperationException("Compaction id is not provided with " +
+          "DirectMetaProvider implementation");
+    }
   }
 
   private class TableMetaRefImpl implements TableMetaRef {
@@ -477,7 +491,8 @@ class DirectMetaProvider implements MetaProvider {
       this.msTable_ = msTable;
     }
 
-    private boolean isPartitioned() {
+    @Override
+    public boolean isPartitioned() {
       return msTable_.getPartitionKeysSize() != 0;
     }
 
@@ -491,6 +506,11 @@ class DirectMetaProvider implements MetaProvider {
     public List<String> getPartitionPrefixes() {
       return Collections.emptyList();
     }
+
+    @Override
+    public boolean isTransactional() {
+      return AcidUtils.isTransactionalTable(msTable_.getParameters());
+    }
   }
 
   @Override
@@ -498,4 +518,32 @@ class DirectMetaProvider implements MetaProvider {
     throw new NotImplementedException(
         "getValidWriteIdList() is not implemented for DirectMetaProvider");
   }
+
+  /**
+   * Fetches the latest compaction id from HMS and compares with partition metadata in
+   * cache. If a partition is stale due to compaction, removes it from metas.
+   */
+  public List<PartitionRef> checkLatestCompaction(String dbName, String tableName,
+      TableMetaRef table, Map<PartitionRef, PartitionMetadata> metas) throws TException {
+    Preconditions.checkNotNull(table, "TableMetaRef must be non-null");
+    Preconditions.checkNotNull(metas, "Partition map must be non-null");
+
+    List<PartitionRef> stalePartitions = MetastoreShim.checkLatestCompaction(
+        msClientPool_, dbName, tableName, table, metas,
+        PartitionRefImpl.UNPARTITIONED_NAME);
+    return stalePartitions;
+  }
+
+  @Override
+  public TPartialTableInfo loadIcebergTable(final TableMetaRef table) throws TException {
+    throw new NotImplementedException(
+        "loadIcebergTable() is not implemented for DirectMetaProvider");
+  }
+
+  @Override
+  public org.apache.iceberg.Table loadIcebergApiTable(final TableMetaRef table,
+      TableParams param, Table msTable) throws TException {
+    throw new NotImplementedException(
+        "loadIcebergApiTable() is not implemented for DirectMetaProvider");
+  }
 }
