@@ -34,9 +34,11 @@ import org.apache.impala.catalog.HdfsPartition.FileDescriptor;
 import org.apache.impala.catalog.HdfsPartitionLocationCompressor;
 import org.apache.impala.catalog.HdfsStorageDescriptor;
 import org.apache.impala.catalog.SqlConstraints;
+import org.apache.impala.catalog.local.LocalIcebergTable.TableParams;
 import org.apache.impala.common.Pair;
 import org.apache.impala.thrift.TBriefTableMeta;
 import org.apache.impala.thrift.TNetworkAddress;
+import org.apache.impala.thrift.TPartialTableInfo;
 import org.apache.impala.thrift.TValidWriteIdList;
 import org.apache.impala.util.ListMap;
 import org.apache.thrift.TException;
@@ -123,6 +125,18 @@ public interface MetaProvider {
   List<ColumnStatisticsObj> loadTableColumnStatistics(TableMetaRef table,
       List<String> colNames) throws TException;
 
+  /**
+   * Loads Iceberg related table metadata.
+   */
+  public TPartialTableInfo loadIcebergTable(
+      final TableMetaRef table) throws TException;
+
+  /**
+   * Loads the Iceberg API table metadata through the Iceberg library.
+   */
+  public org.apache.iceberg.Table loadIcebergApiTable(
+      final TableMetaRef table, TableParams param, Table msTable) throws TException;
+
   /**
    * Reference to a table as returned by loadTable(). This reference must be passed
    * back to other functions to fetch more details about the table. Implementations
@@ -132,6 +146,8 @@ public interface MetaProvider {
   interface TableMetaRef {
     boolean isMarkedCached();
     List<String> getPartitionPrefixes();
+    boolean isPartitioned();
+    boolean isTransactional();
   }
 
   /**
@@ -157,6 +173,7 @@ public interface MetaProvider {
     byte[] getPartitionStats();
     boolean hasIncrementalStats();
     boolean isMarkedCached();
+    long getLastCompactionId();
   }
 
   public TValidWriteIdList getValidWriteIdList(TableMetaRef ref);
