@@ -65,8 +65,11 @@ public class FileSystemUtil {
   public static final String SCHEME_HDFS = "hdfs";
   public static final String SCHEME_S3A = "s3a";
   public static final String SCHEME_O3FS = "o3fs";
+  public static final String SCHEME_OFS = "ofs";
   public static final String SCHEME_ALLUXIO = "alluxio";
   public static final String SCHEME_GCS = "gs";
+  public static final String SCHEME_COS = "cosn";
+  public static final String SCHEME_SFS = "sfs";
 
   /**
    * Set containing all FileSystem scheme that known to supports storage UUIDs in
@@ -76,6 +79,7 @@ public class FileSystemUtil {
       ImmutableSet.<String>builder()
           .add(SCHEME_HDFS)
           .add(SCHEME_O3FS)
+          .add(SCHEME_OFS)
           .add(SCHEME_ALLUXIO)
           .build();
 
@@ -91,7 +95,9 @@ public class FileSystemUtil {
           .add(SCHEME_HDFS)
           .add(SCHEME_S3A)
           .add(SCHEME_O3FS)
+          .add(SCHEME_OFS)
           .add(SCHEME_GCS)
+          .add(SCHEME_COS)
           .build();
 
   /**
@@ -105,6 +111,7 @@ public class FileSystemUtil {
           .add(SCHEME_HDFS)
           .add(SCHEME_S3A)
           .add(SCHEME_GCS)
+          .add(SCHEME_COS)
           .build();
 
   /**
@@ -118,7 +125,9 @@ public class FileSystemUtil {
           .add(SCHEME_HDFS)
           .add(SCHEME_S3A)
           .add(SCHEME_O3FS)
+          .add(SCHEME_OFS)
           .add(SCHEME_GCS)
+          .add(SCHEME_COS)
           .build();
 
   /**
@@ -407,6 +416,13 @@ public class FileSystemUtil {
     return hasScheme(fs, SCHEME_GCS);
   }
 
+  /**
+   * Returns true iff the filesystem is a CosFileSystem.
+   */
+  public static boolean isCOSFileSystem(FileSystem fs) {
+    return hasScheme(fs, SCHEME_COS);
+  }
+
   /**
    * Returns true iff the filesystem is AdlFileSystem.
    */
@@ -472,14 +488,14 @@ public class FileSystemUtil {
    * Returns true iff the filesystem is a OzoneFileSystem.
    */
   public static boolean isOzoneFileSystem(FileSystem fs) {
-    return hasScheme(fs, SCHEME_O3FS);
+    return hasScheme(fs, SCHEME_O3FS) || hasScheme(fs, SCHEME_OFS);
   }
 
   /**
    * Returns true iff the path is on OzoneFileSystem.
    */
   public static boolean isOzoneFileSystem(Path path) throws IOException {
-    return hasScheme(path, SCHEME_O3FS);
+    return hasScheme(path, SCHEME_O3FS) || hasScheme(path, SCHEME_OFS);
   }
 
   /**
@@ -515,7 +531,9 @@ public class FileSystemUtil {
     S3,
     OZONE,
     ALLUXIO,
-    GCS;
+    GCS,
+    COS,
+    SFS;
 
     private static final Map<String, FsType> SCHEME_TO_FS_MAPPING =
         ImmutableMap.<String, FsType>builder()
@@ -526,8 +544,10 @@ public class FileSystemUtil {
             .put(SCHEME_HDFS, HDFS)
             .put(SCHEME_S3A, S3)
             .put(SCHEME_O3FS, OZONE)
+            .put(SCHEME_OFS, OZONE)
             .put(SCHEME_ALLUXIO, ALLUXIO)
             .put(SCHEME_GCS, GCS)
+            .put(SCHEME_COS, COS)
             .build();
 
     /**
@@ -538,6 +558,9 @@ public class FileSystemUtil {
      * hdfs, s3a, etc.)
      */
     public static FsType getFsType(String scheme) {
+      if(scheme.startsWith("sfs+")) {
+        return SFS;
+      }
       return SCHEME_TO_FS_MAPPING.get(scheme);
     }
   }
@@ -701,7 +724,7 @@ public class FileSystemUtil {
           return listFiles(fs, p, true, debugAction);
         }
         DebugUtils.executeDebugAction(debugAction, DebugUtils.REFRESH_HDFS_LISTING_DELAY);
-        return new FilterIterator(p, new RecursingIterator<>(fs, p,
+        return new FilterIterator(p, new RecursingIterator<>(fs, p, debugAction,
             FileSystemUtil::listStatusIterator));
       }
       DebugUtils.executeDebugAction(debugAction, DebugUtils.REFRESH_HDFS_LISTING_DELAY);
@@ -725,7 +748,7 @@ public class FileSystemUtil {
       if (hasRecursiveListFiles(fs)) {
         baseIterator = fs.listFiles(p, recursive);
       } else {
-        baseIterator = new RecursingIterator<>(fs, p,
+        baseIterator = new RecursingIterator<>(fs, p, debugAction,
             FileSystemUtil::listLocatedStatusIterator);
       }
       return new FilterIterator(p, baseIterator);
@@ -903,17 +926,22 @@ public class FileSystemUtil {
     private final BiFunctionWithException<FileSystem, Path, RemoteIterator<T>>
         newIterFunc_;
     private final FileSystem fs_;
+    private final String debugAction_;
     private final Stack<RemoteIterator<T>> iters_ = new Stack<>();
     private RemoteIterator<T> curIter_;
     private T curFile_;
 
-    private RecursingIterator(FileSystem fs, Path startPath,
+    private RecursingIterator(FileSystem fs, Path startPath, String debugAction,
         BiFunctionWithException<FileSystem, Path, RemoteIterator<T>> newIterFunc)
         throws IOException {
       this.fs_ = Preconditions.checkNotNull(fs);
+      this.debugAction_ = debugAction;
       this.newIterFunc_ = Preconditions.checkNotNull(newIterFunc);
       Preconditions.checkNotNull(startPath);
       curIter_ = newIterFunc.apply(fs, startPath);
+      LOG.trace("listed start path: {}", startPath);
+      DebugUtils.executeDebugAction(debugAction,
+          DebugUtils.REFRESH_PAUSE_AFTER_HDFS_REMOTE_ITERATOR_CREATION);
     }
 
     @Override
@@ -955,6 +983,13 @@ public class FileSystemUtil {
      * @throws IOException if any IO error occurs
      */
     private void handleFileStat(T fileStatus) throws IOException {
+      LOG.trace("handleFileStat: {}", fileStatus.getPath());
+      if (isIgnoredDir(fileStatus.getPath())) {
+        LOG.debug("Ignoring {} since it is either a hidden directory or a temporary "
+            + "staging directory", fileStatus.getPath());
+        curFile_ = null;
+        return;
+      }
       if (fileStatus.isFile()) {
         curFile_ = fileStatus;
         return;
@@ -964,6 +999,9 @@ public class FileSystemUtil {
       iters_.push(curIter_);
       curIter_ = subIter;
       curFile_ = fileStatus;
+      LOG.trace("listed sub dir: {}", fileStatus.getPath());
+      DebugUtils.executeDebugAction(debugAction_,
+          DebugUtils.REFRESH_PAUSE_AFTER_HDFS_REMOTE_ITERATOR_CREATION);
     }
 
     @Override
