@@ -587,7 +587,7 @@ public class AnalyticEvalNode extends PlanNode {
       return falseStatus;
     }
     List<Expr> lhsSourceExprs = ((SlotRef) lhs).getDesc().getSourceExprs();
-    if (lhsSourceExprs.size() > 1 ||
+    if (lhsSourceExprs.size() != 1 ||
           !(lhsSourceExprs.get(0) instanceof AnalyticExpr)) {
       return falseStatus;
     }
