@@ -931,7 +931,9 @@ public class DistributedPlanner {
     // if there is a limit, we need to transfer it from the pre-aggregation
     // node in the child fragment to the merge aggregation node in the parent
     long limit = node.getLimit();
-    node.unsetLimit();
+    if (node.getMultiAggInfo().hasAggregateExprs() || !node.getConjuncts().isEmpty()) {
+      node.unsetLimit();
+    }
     node.unsetNeedsFinalize();
 
     // place a merge aggregation step in a new fragment
@@ -940,6 +942,10 @@ public class DistributedPlanner {
         mergeFragment.getPlanRoot(), node.getMultiAggInfo(), AggPhase.FIRST_MERGE);
     mergeAggNode.init(ctx_.getRootAnalyzer());
     mergeAggNode.setLimit(limit);
+    // Carry the IsNonCorrelatedSclarSubquery_ flag to the merge node. This flag is
+    // applicable regardless of the partition scheme for the children since it is a
+    // logical property.
+    mergeAggNode.setIsNonCorrelatedScalarSubquery(node.isNonCorrelatedScalarSubquery());
     // Merge of non-grouping agg only processes one tuple per Impala daemon - codegen
     // will cost more than benefit.
     if (!hasGrouping) {
