@@ -46,6 +46,7 @@ import org.apache.impala.catalog.Type;
 import org.apache.impala.common.ImpalaRuntimeException;
 import org.apache.impala.service.BackendConfig;
 import org.apache.impala.thrift.TExplainLevel;
+import org.apache.impala.thrift.TKuduReplicaSelection;
 import org.apache.impala.thrift.TKuduScanNode;
 import org.apache.impala.thrift.TNetworkAddress;
 import org.apache.impala.thrift.TPlanNode;
@@ -55,6 +56,7 @@ import org.apache.impala.thrift.TScanRange;
 import org.apache.impala.thrift.TScanRangeLocation;
 import org.apache.impala.thrift.TScanRangeLocationList;
 import org.apache.impala.thrift.TScanRangeSpec;
+import org.apache.impala.util.ExprUtil;
 import org.apache.impala.util.KuduUtil;
 import org.apache.impala.util.ExecutorMembershipSnapshot;
 import org.apache.kudu.ColumnSchema;
@@ -65,6 +67,7 @@ import org.apache.kudu.client.KuduPredicate.ComparisonOp;
 import org.apache.kudu.client.KuduScanToken;
 import org.apache.kudu.client.KuduScanToken.KuduScanTokenBuilder;
 import org.apache.kudu.client.LocatedTablet;
+import org.apache.kudu.consensus.Metadata.RaftPeerPB.Role;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -98,6 +101,9 @@ public class KuduScanNode extends ScanNode {
   // Set in computeNodeResourceProfile().
   private boolean useMtScanNode_;
 
+  // True if the query option of replica selection is set as leader-only.
+  private boolean replicaSelectionLeaderOnly_ = false;
+
   // Indexes for the set of hosts that will be used for the query.
   // From analyzer.getHostIndex().getIndex(address)
   private final Set<Integer> hostIndexSet_ = new HashSet<>();
@@ -217,6 +223,8 @@ public class KuduScanNode extends ScanNode {
       throws ImpalaRuntimeException {
     scanRangeSpecs_ = new TScanRangeSpec();
 
+    replicaSelectionLeaderOnly_ = (analyzer.getQueryOptions().getKudu_replica_selection()
+        == TKuduReplicaSelection.LEADER_ONLY);
     List<KuduScanToken> scanTokens = createScanTokens(analyzer, client, rpcTable);
     for (KuduScanToken token: scanTokens) {
       LocatedTablet tablet = token.getTablet();
@@ -228,10 +236,17 @@ public class KuduScanNode extends ScanNode {
       }
 
       for (LocatedTablet.Replica replica: tablet.getReplicas()) {
+        // Skip non-leader replicas if query option KUDU_REPLICA_SELECTION is set as
+        // LEADER_ONLY.
+        if (replicaSelectionLeaderOnly_
+            && !replica.getRole().equals(Role.LEADER.toString())) {
+          continue;
+        }
+
         TNetworkAddress address =
             new TNetworkAddress(replica.getRpcHost(), replica.getRpcPort());
         // Use the network address to look up the host in the global list
-        Integer hostIndex = analyzer.getHostIndex().getIndex(address);
+        Integer hostIndex = analyzer.getHostIndex().getOrAddIndex(address);
         locations.add(new TScanRangeLocation(hostIndex));
         hostIndexSet_.add(hostIndex);
       }
@@ -289,7 +304,8 @@ public class KuduScanNode extends ScanNode {
   protected void computeNumNodes(Analyzer analyzer) {
     ExecutorMembershipSnapshot cluster = ExecutorMembershipSnapshot.getCluster();
     final int maxInstancesPerNode = getMaxInstancesPerNode(analyzer);
-    final int maxPossibleInstances = cluster.numExecutors() * maxInstancesPerNode;
+    final int maxPossibleInstances =
+        analyzer.numExecutorsForPlanning() * maxInstancesPerNode;
     int totalNodes = 0;
     int totalInstances = 0;
     int numLocalRanges = 0;
@@ -332,10 +348,12 @@ public class KuduScanNode extends ScanNode {
         // hosts that hold replica for those ranges.
         int numLocalNodes = Math.min(numLocalRanges, localRangeCounts.size());
         // The remote ranges are round-robined across all the impalads.
-        int numRemoteNodes = Math.min(numRemoteRanges, cluster.numExecutors());
+        int numRemoteNodes =
+            Math.min(numRemoteRanges, analyzer.numExecutorsForPlanning());
         // The local and remote assignments may overlap, but we don't know by how much
         // so conservatively assume no overlap.
-        totalNodes = Math.min(numLocalNodes + numRemoteNodes, cluster.numExecutors());
+        totalNodes =
+            Math.min(numLocalNodes + numRemoteNodes, analyzer.numExecutorsForPlanning());
 
         int numLocalInstances = Math.min(numLocalRanges, totalLocalParallelism);
         totalInstances = Math.min(numLocalInstances + numRemoteRanges,
@@ -406,8 +424,10 @@ public class KuduScanNode extends ScanNode {
     StringBuilder result = new StringBuilder();
 
     String aliasStr = desc_.hasExplicitAlias() ? " " + desc_.getAlias() : "";
-    result.append(String.format("%s%s:%s [%s%s]\n", prefix, id_.toString(), displayName_,
-        kuduTable_.getFullName(), aliasStr));
+    result.append(
+        String.format(replicaSelectionLeaderOnly_ ? "%s%s:%s [%s%s, LEADER-only]\n" :
+                                                    "%s%s:%s [%s%s]\n",
+            prefix, id_.toString(), displayName_, kuduTable_.getFullName(), aliasStr));
 
     switch (detailLevel) {
       case MINIMAL: break;
@@ -540,7 +560,7 @@ public class KuduScanNode extends ScanNode {
         try {
           // TODO: Simplify when Impala supports a 64-bit TIMESTAMP type.
           kuduPredicate = KuduPredicate.newComparisonPredicate(column, op,
-              KuduUtil.timestampToUnixTimeMicros(analyzer, literal));
+              ExprUtil.utcTimestampToUnixTimeMicros(analyzer, literal));
         } catch (Exception e) {
           LOG.info("Exception converting Kudu timestamp predicate: " + expr.toSql(), e);
           return false;
@@ -651,7 +671,7 @@ public class KuduScanNode extends ScanNode {
       case TIMESTAMP: {
         try {
           // TODO: Simplify when Impala supports a 64-bit TIMESTAMP type.
-          return KuduUtil.timestampToUnixTimeMicros(analyzer, e);
+          return ExprUtil.utcTimestampToUnixTimeMicros(analyzer, e);
         } catch (Exception ex) {
           LOG.info("Exception converting Kudu timestamp expr: " + e.toSql(), ex);
         }
