@@ -21,16 +21,19 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.Iterator;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
 
 import org.apache.impala.analysis.Analyzer;
 import org.apache.impala.analysis.BinaryPredicate;
+import org.apache.impala.analysis.CollectionTableRef;
 import org.apache.impala.analysis.Expr;
 import org.apache.impala.analysis.ExprId;
 import org.apache.impala.analysis.ExprSubstitutionMap;
 import org.apache.impala.analysis.SlotDescriptor;
+import org.apache.impala.analysis.SlotRef;
 import org.apache.impala.analysis.ToSqlOptions;
 import org.apache.impala.analysis.TupleDescriptor;
 import org.apache.impala.analysis.TupleId;
@@ -386,6 +389,7 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
       for (int i = children_.size() - 1; i >= 1; --i) {
         PlanNode child = getChild(i);
         if (fragment_ != child.fragment_) {
+          if (detailLevel == TExplainLevel.VERBOSE) continue;
           // we're crossing a fragment boundary
           expBuilder.append(
               child.fragment_.getExplainString(
@@ -494,6 +498,30 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
     msg.num_children = numChildren;
   }
 
+  /**
+   * If there are more than one array to be zipping unnested in the 'analyzer' then
+   * removes the conjuncts related to the unnested array item from the 'conjuncts' list.
+   * This could be useful to prevent e.g. ScanNode or SingularRowSrc to pick up the
+   * conjuncts for zipping unnested arrays and let the UnnestNode to take care of them.
+   */
+  public static void removeZippingUnnestConjuncts(List<Expr> conjuncts,
+      Analyzer analyzer) {
+    Set<TupleId> zippingUnnestTupleIds = analyzer.getZippingUnnestTupleIds();
+
+    Iterator<Expr> it = conjuncts.iterator();
+    while(it.hasNext()) {
+      Expr e = it.next();
+      List<SlotRef> slotRefs = new ArrayList<>();
+      e.collect(SlotRef.class, slotRefs);
+      for (SlotRef slotRef : slotRefs) {
+        if (zippingUnnestTupleIds.contains(slotRef.getDesc().getParent().getId())) {
+          it.remove();
+          break;
+        }
+      }
+    }
+  }
+
   /**
    * Computes the full internal state, including smap and planner-relevant statistics
    * (calls computeStats()), marks all slots referenced by this node as materialized
@@ -514,10 +542,15 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
    */
   protected void assignConjuncts(Analyzer analyzer) {
     List<Expr> unassigned = analyzer.getUnassignedConjuncts(this);
+    if (!shouldPickUpZippingUnnestConjuncts()) {
+      removeZippingUnnestConjuncts(unassigned, analyzer);
+    }
     conjuncts_.addAll(unassigned);
     analyzer.markConjunctsAssigned(unassigned);
   }
 
+  protected boolean shouldPickUpZippingUnnestConjuncts() { return false; }
+
   /**
    * Apply the provided conjuncts to the this node, returning the new root of
    * the plan tree. Also add any slot equivalences for tupleIds that have not
@@ -542,12 +575,23 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
     // Check if this is an inferred identity predicate i.e for c1 = c2 both
     // sides are pointing to the same source slot. In such cases it is wrong
     // to add the predicate to the SELECT node because it will incorrectly
-    // eliminate rows with NULL values.
+    // eliminate rows with NULL values. We also check if both sides are pointing
+    // to equal constant values.
     for (Expr e : conjuncts) {
       if (e instanceof BinaryPredicate && ((BinaryPredicate) e).isInferred()) {
-        SlotDescriptor lhs = ((BinaryPredicate) e).getChild(0).findSrcScanSlot();
-        SlotDescriptor rhs = ((BinaryPredicate) e).getChild(1).findSrcScanSlot();
-        if (lhs != null && rhs != null && lhs.equals(rhs)) continue;
+        Expr lhsSrcExpr = ((BinaryPredicate) e).getChild(0).findSrcExpr();
+        Expr rhsSrcExpr  = ((BinaryPredicate) e).getChild(1).findSrcExpr();
+        if (lhsSrcExpr != null && rhsSrcExpr != null) {
+          if (lhsSrcExpr.isConstant() && rhsSrcExpr.isConstant() &&
+              lhsSrcExpr.equals(rhsSrcExpr)) {
+            continue;
+          }
+          if (lhsSrcExpr instanceof SlotRef && rhsSrcExpr instanceof SlotRef) {
+            SlotRef lhsSlotRef = (SlotRef) lhsSrcExpr;
+            SlotRef rhsSlotRef = (SlotRef) rhsSrcExpr;
+            if (lhsSlotRef.getDesc().equals(rhsSlotRef.getDesc())) continue;
+          }
+        }
       }
       finalConjuncts.add(e);
     }
@@ -1008,8 +1052,12 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
     while (!remaining.isEmpty()) {
       double smallestCost = Float.MAX_VALUE;
       T bestConjunct =  null;
+      // IMPALA-9338: The bestConjunctIdx indicates the index of the bestConjunct in
+      // the remaining list.
+      int bestConjunctIdx = 0;
       double backoffExp = 1.0 / (double) (sortedConjuncts.size() + 1);
-      for (T e : remaining) {
+      for (int i = 0; i < remaining.size(); i++) {
+        T e = remaining.get(i);
         double sel = Math.pow(e.hasSelectivity() ? e.getSelectivity() : defaultSel,
             backoffExp);
 
@@ -1021,17 +1069,18 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
         if (cost < smallestCost) {
           smallestCost = cost;
           bestConjunct = e;
+          bestConjunctIdx = i;
         } else if (cost == smallestCost) {
           // Break ties based on toSql() to get a consistent display in explain plans.
           if (e.toSql().compareTo(bestConjunct.toSql()) < 0) {
-            smallestCost = cost;
             bestConjunct = e;
+            bestConjunctIdx = i;
           }
         }
       }
 
       sortedConjuncts.add(bestConjunct);
-      remaining.remove(bestConjunct);
+      remaining.remove(bestConjunctIdx);
       totalCost -= bestConjunct.getCost();
     }
 
