@@ -18,6 +18,7 @@
 package org.apache.impala.planner;
 
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashMap;
@@ -46,7 +47,11 @@ import org.apache.impala.analysis.TupleId;
 import org.apache.impala.analysis.TupleIsNullPredicate;
 import org.apache.impala.catalog.FeTable;
 import org.apache.impala.catalog.Column;
+import org.apache.impala.catalog.FeIcebergTable;
+import org.apache.impala.catalog.HdfsFileFormat;
 import org.apache.impala.catalog.KuduColumn;
+import org.apache.impala.catalog.PrimitiveType;
+import org.apache.impala.catalog.ScalarType;
 import org.apache.impala.catalog.Type;
 import org.apache.impala.common.AnalysisException;
 import org.apache.impala.common.IdGenerator;
@@ -55,13 +60,14 @@ import org.apache.impala.planner.JoinNode.DistributionMode;
 import org.apache.impala.service.BackendConfig;
 import org.apache.impala.service.FeSupport;
 import org.apache.impala.thrift.TColumnValue;
-import org.apache.impala.thrift.TEnabledRuntimeFilterTypes;
+import org.apache.impala.thrift.TExplainLevel;
 import org.apache.impala.thrift.TQueryOptions;
 import org.apache.impala.thrift.TRuntimeFilterDesc;
 import org.apache.impala.thrift.TRuntimeFilterMode;
 import org.apache.impala.thrift.TRuntimeFilterTargetDesc;
 import org.apache.impala.thrift.TRuntimeFilterType;
 import org.apache.impala.util.BitUtil;
+import org.apache.impala.util.TColumnValueUtil;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -103,6 +109,14 @@ public final class RuntimeFilterGenerator {
   // Should be in sync with corresponding values in runtime-filter-bank.cc.
   private static final long MIN_BLOOM_FILTER_SIZE = 4 * 1024;
   private static final long MAX_BLOOM_FILTER_SIZE = 512 * 1024 * 1024;
+  // Should be in sync with the corresponding value in in-list-filter.h.
+  private static final long IN_LIST_FILTER_STRING_SET_MAX_TOTAL_LENGTH = 4 * 1024 * 1024;
+
+  private static final Set<PrimitiveType> IN_LIST_FILTER_SUPPORTED_TYPES =
+      new HashSet<>(Arrays.asList(
+          PrimitiveType.TINYINT, PrimitiveType.SMALLINT, PrimitiveType.INT,
+          PrimitiveType.BIGINT, PrimitiveType.DATE, PrimitiveType.STRING,
+          PrimitiveType.CHAR, PrimitiveType.VARCHAR));
 
   // Map of base table tuple ids to a list of runtime filters that
   // can be applied at the corresponding scan nodes.
@@ -115,21 +129,24 @@ public final class RuntimeFilterGenerator {
 
   /**
    * Internal class that encapsulates the max, min and default sizes used for creating
-   * bloom filter objects.
+   * bloom filter objects, and entry limit for in-list filters.
    */
   private class FilterSizeLimits {
-    // Maximum filter size, in bytes, rounded up to a power of two.
+    // Maximum bloom filter size, in bytes, rounded up to a power of two.
     public final long maxVal;
 
-    // Minimum filter size, in bytes, rounded up to a power of two.
+    // Minimum bloom filter size, in bytes, rounded up to a power of two.
     public final long minVal;
 
-    // Pre-computed default filter size, in bytes, rounded up to a power of two.
+    // Pre-computed default bloom filter size, in bytes, rounded up to a power of two.
     public final long defaultVal;
 
-    // Target false positive probability, between 0 and 1 exclusive.
+    // Target false positive probability for bloom filters, between 0 and 1 exclusive.
     public final double targetFpp;
 
+    // Maximum entry size for in-list filters.
+    public final long inListFilterEntryLimit;
+
     public FilterSizeLimits(TQueryOptions tQueryOptions) {
       // Round up all limits to a power of two and make sure filter size is more
       // than the min buffer size that can be allocated by the buffer pool.
@@ -152,15 +169,17 @@ public final class RuntimeFilterGenerator {
       targetFpp = tQueryOptions.isSetRuntime_filter_error_rate() ?
           tQueryOptions.getRuntime_filter_error_rate() :
           BackendConfig.INSTANCE.getMaxFilterErrorRate();
-    }
-  };
 
-  // Contains size limits for bloom filters.
-  private FilterSizeLimits bloomFilterSizeLimits_;
+      inListFilterEntryLimit = tQueryOptions.getRuntime_in_list_filter_entry_limit();
+    }
+  }
+
+  // Contains size limits for runtime filters.
+  final private FilterSizeLimits filterSizeLimits_;
 
   private RuntimeFilterGenerator(TQueryOptions tQueryOptions) {
-    bloomFilterSizeLimits_ = new FilterSizeLimits(tQueryOptions);
-  };
+    filterSizeLimits_ = new FilterSizeLimits(tQueryOptions);
+  }
 
   /**
    * Internal representation of a runtime filter. A runtime filter is generated from
@@ -229,14 +248,17 @@ public final class RuntimeFilterGenerator {
       // The low and high value of the column on which the filter is applied
       public final TColumnValue lowValue;
       public final TColumnValue highValue;
+      // Indicates if the data is actually stored in the data files
+      public final boolean isColumnInDataFile;
 
       public RuntimeFilterTarget(ScanNode targetNode, Expr targetExpr,
-          boolean isBoundByPartitionColumns, boolean isLocalTarget, TColumnValue lowValue,
-          TColumnValue highValue) {
+          boolean isBoundByPartitionColumns, boolean isColumnInDataFile,
+          boolean isLocalTarget, TColumnValue lowValue, TColumnValue highValue) {
         Preconditions.checkState(targetExpr.isBoundByTupleIds(targetNode.getTupleIds()));
         node = targetNode;
         expr = targetExpr;
         this.isBoundByPartitionColumns = isBoundByPartitionColumns;
+        this.isColumnInDataFile = isColumnInDataFile;
         this.isLocalTarget = isLocalTarget;
         this.lowValue = lowValue;
         this.highValue = highValue;
@@ -253,6 +275,9 @@ public final class RuntimeFilterGenerator {
         tFilterTarget.setTarget_expr_slotids(tSlotIds);
         tFilterTarget.setIs_bound_by_partition_columns(isBoundByPartitionColumns);
         tFilterTarget.setIs_local_target(isLocalTarget);
+        if (node instanceof HdfsScanNode) {
+          tFilterTarget.setIs_column_in_data_file(isColumnInDataFile);
+        }
         if (node instanceof KuduScanNode) {
           // assignRuntimeFilters() only assigns KuduScanNode targets if the target expr
           // is a slot ref, possibly with an implicit cast, pointing to a column.
@@ -263,6 +288,21 @@ public final class RuntimeFilterGenerator {
         }
         tFilterTarget.setLow_value(lowValue);
         tFilterTarget.setHigh_value(highValue);
+        boolean minMaxValuePresent = false;
+        if (node instanceof HdfsScanNode) {
+          // Set the min/max value present flag to avoid repeated check of the presence
+          // of the two values during execution.
+          SlotRef slotRefInScan = expr.unwrapSlotRef(true);
+          if (slotRefInScan != null) {
+            Column col = slotRefInScan.getDesc().getColumn();
+            if (col != null) {
+              Type col_type = col.getType();
+              minMaxValuePresent = TColumnValueUtil.isFieldSet(col_type, lowValue)
+                  && TColumnValueUtil.isFieldSet(col_type, highValue);
+            }
+          }
+        }
+        tFilterTarget.setIs_min_max_value_present(minMaxValuePresent);
         return tFilterTarget;
       }
 
@@ -272,9 +312,10 @@ public final class RuntimeFilterGenerator {
         return output.append("Target Id: " + node.getId() + " ")
             .append("Target expr: " + expr.debugString() + " ")
             .append("Partition columns: " + isBoundByPartitionColumns)
+            .append("Is column stored in data files: " + isColumnInDataFile)
             .append("Is local: " + isLocalTarget)
-            .append("lowValue: " + lowValue.toString())
-            .append("highValue: " + highValue.toString())
+            .append("lowValue: " + (lowValue != null ? lowValue.toString() : -1))
+            .append("highValue: " + (highValue != null ? highValue.toString() : -1))
             .toString();
       }
     }
@@ -319,6 +360,7 @@ public final class RuntimeFilterGenerator {
       tFilter.setNdv_estimate(ndvEstimate_);
       tFilter.setHas_local_targets(hasLocalTargets_);
       tFilter.setHas_remote_targets(hasRemoteTargets_);
+      tFilter.setCompareOp(exprCmpOp_.getThriftOp());
       boolean appliedOnPartitionColumns = true;
       for (int i = 0; i < targets_.size(); ++i) {
         RuntimeFilterTarget target = targets_.get(i);
@@ -345,18 +387,70 @@ public final class RuntimeFilterGenerator {
       Preconditions.checkNotNull(idGen);
       Preconditions.checkNotNull(joinPredicate);
       Preconditions.checkNotNull(filterSrcNode);
-      // Only consider binary equality predicates
-      if (type == TRuntimeFilterType.BLOOM
-          && !Predicate.isEquivalencePredicate(joinPredicate)) {
-        return null;
+      // Only consider binary equality predicates under hash joins for runtime bloom
+      // filters and in-list filters.
+      if (type == TRuntimeFilterType.BLOOM || type == TRuntimeFilterType.IN_LIST) {
+        if (!Predicate.isEquivalencePredicate(joinPredicate)
+            || filterSrcNode instanceof NestedLoopJoinNode) {
+          return null;
+        }
       }
-      if (type == TRuntimeFilterType.MIN_MAX
-          && !Predicate.isSqlEquivalencePredicate(joinPredicate)) {
-        return null;
+      if (type == TRuntimeFilterType.IN_LIST) {
+        PrimitiveType lhsType = joinPredicate.getChild(0).getType().getPrimitiveType();
+        PrimitiveType rhsType = joinPredicate.getChild(1).getType().getPrimitiveType();
+        Preconditions.checkState(lhsType == rhsType, "Unanalyzed equivalence pred!");
+        if (!IN_LIST_FILTER_SUPPORTED_TYPES.contains(lhsType)) {
+          return null;
+        }
       }
-      BinaryPredicate normalizedJoinConjunct = SingleNodePlanner.getNormalizedEqPred(
+
+      BinaryPredicate normalizedJoinConjunct = null;
+      if (type == TRuntimeFilterType.MIN_MAX) {
+        if (filterSrcNode instanceof HashJoinNode) {
+          if (!Predicate.isSqlEquivalencePredicate(joinPredicate)) {
+            return null;
+          } else {
+            normalizedJoinConjunct = SingleNodePlanner.getNormalizedEqPred(joinPredicate,
+                filterSrcNode.getChild(0).getTupleIds(),
+                filterSrcNode.getChild(1).getTupleIds(), analyzer);
+          }
+        } else if (filterSrcNode instanceof NestedLoopJoinNode) {
+          if (Predicate.isSingleRangePredicate(joinPredicate)) {
+            PlanNode child1 = filterSrcNode.getChild(1);
+            if (child1 instanceof ExchangeNode) {
+              child1 = child1.getChild(0);
+            }
+            // When immediate or the indirect child below an Exchange is an
+            // AggregationNode that implements a non-correlated scalar subquery
+            // returning at most one value, a min/max filter can be generated.
+            if (!(child1 instanceof AggregationNode)
+                || !((AggregationNode) (child1)).isNonCorrelatedScalarSubquery()) {
+              return null;
+            }
+          } else {
+            return null;
+          }
+          normalizedJoinConjunct = SingleNodePlanner.getNormalizedSingleRangePred(
+                joinPredicate, filterSrcNode.getChild(0).getTupleIds(),
+                filterSrcNode.getChild(1).getTupleIds(), analyzer);
+          if (normalizedJoinConjunct != null) {
+            // MinMaxFilter can't handle range predicates with decimals stored
+            // in __int128_t for the following reason:
+            //  1) Both numeric_limits<__int128_t>::min() and
+            //     numeric_limits<__int128_t>::max() return 0.
+            if (normalizedJoinConjunct.getChild(0).getType().isDecimal()) {
+              ScalarType decimalType =
+                  (ScalarType) (normalizedJoinConjunct.getChild(0).getType());
+              if (decimalType.storageBytesForDecimal() == 16) return null;
+            }
+          }
+        }
+      } else {
+        normalizedJoinConjunct = SingleNodePlanner.getNormalizedEqPred(
           joinPredicate, filterSrcNode.getChild(0).getTupleIds(),
           filterSrcNode.getChild(1).getTupleIds(), analyzer);
+      }
+
       if (normalizedJoinConjunct == null) return null;
 
       // Ensure that the target expr does not contain TupleIsNull predicates as these
@@ -577,13 +671,25 @@ public final class RuntimeFilterGenerator {
     }
 
     /**
-     * Sets the filter size (in bytes) required for a bloom filter to achieve the
-     * configured maximum false-positive rate based on the expected NDV. Also bounds the
-     * filter size between the max and minimum filter sizes supplied to it by
-     * 'filterSizeLimits'.
+     * Sets the filter size (in bytes) based on the filter type.
+     * For bloom filters, the size should be able to achieve the configured maximum
+     * false-positive rate based on the expected NDV. Also bounds the filter size between
+     * the max and minimum filter sizes supplied to it by 'filterSizeLimits'.
+     * For min-max filters, we ignore the size since each filter only keeps two values.
+     * For in-list filters, the size is calculated based on the data types.
      */
     private void calculateFilterSize(FilterSizeLimits filterSizeLimits) {
       if (type_ == TRuntimeFilterType.MIN_MAX) return;
+      if (type_ == TRuntimeFilterType.IN_LIST) {
+        Type colType = srcExpr_.getType();
+        if (colType.isStringType()) {
+          filterSizeBytes_ = IN_LIST_FILTER_STRING_SET_MAX_TOTAL_LENGTH;
+        } else {
+          filterSizeBytes_ =
+              filterSizeLimits.inListFilterEntryLimit * colType.getSlotSize();
+        }
+        return;
+      }
       if (ndvEstimate_ == -1) {
         filterSizeBytes_ = filterSizeLimits.defaultVal;
         return;
@@ -607,6 +713,7 @@ public final class RuntimeFilterGenerator {
     public String debugString() {
       StringBuilder output = new StringBuilder();
       return output.append("FilterID: " + id_ + " ")
+          .append("Type: " + type_ + " ")
           .append("Source: " + src_.getId() + " ")
           .append("SrcExpr: " + getSrcExpr().debugString() +  " ")
           .append("Target(s): ")
@@ -650,8 +757,13 @@ public final class RuntimeFilterGenerator {
         if (numBloomFilters >= maxNumBloomFilters) continue;
         ++numBloomFilters;
       }
-      filter.setIsBroadcast(
-          filter.src_.getDistributionMode() == DistributionMode.BROADCAST);
+      DistributionMode distMode = filter.src_.getDistributionMode();
+      filter.setIsBroadcast(distMode == DistributionMode.BROADCAST);
+      if (filter.getType() == TRuntimeFilterType.IN_LIST
+          && distMode == DistributionMode.PARTITIONED) {
+        LOG.trace("Skip IN-list filter on partitioned join: {}", filter.debugString());
+        continue;
+      }
       filter.computeHasLocalTargets();
       if (LOG.isTraceEnabled()) LOG.trace("Runtime filter: " + filter.debugString());
       filter.assignToPlanNodes();
@@ -698,8 +810,8 @@ public final class RuntimeFilterGenerator {
    * (scan) nodes. Filters that cannot be assigned to a scan node are discarded.
    */
   private void generateFilters(PlannerContext ctx, PlanNode root) {
-    if (root instanceof HashJoinNode) {
-      HashJoinNode joinNode = (HashJoinNode) root;
+    if (root instanceof HashJoinNode || root instanceof NestedLoopJoinNode) {
+      JoinNode joinNode = (JoinNode) root;
       List<Expr> joinConjuncts = new ArrayList<>();
       if (!joinNode.getJoinOp().isLeftOuterJoin()
           && !joinNode.getJoinOp().isFullOuterJoin()
@@ -712,11 +824,14 @@ public final class RuntimeFilterGenerator {
       joinConjuncts.addAll(joinNode.getConjuncts());
 
       List<RuntimeFilter> filters = new ArrayList<>();
+      Set<TRuntimeFilterType> enabledRuntimeFilterTypes =
+          ctx.getQueryOptions().getEnabled_runtime_filter_types();
       for (TRuntimeFilterType filterType : TRuntimeFilterType.values()) {
+        if (!enabledRuntimeFilterTypes.contains(filterType)) continue;
         for (Expr conjunct : joinConjuncts) {
           RuntimeFilter filter =
               RuntimeFilter.create(filterIdGenerator, ctx.getRootAnalyzer(), conjunct,
-                  joinNode, filterType, bloomFilterSizeLimits_,
+                  joinNode, filterType, filterSizeLimits_,
                   /* isTimestampTruncation */ false);
           if (filter != null) {
             registerRuntimeFilter(filter);
@@ -730,7 +845,7 @@ public final class RuntimeFilterGenerator {
               && conjunct.getChild(1).getType().isTimestamp()) {
             RuntimeFilter filter2 =
                 RuntimeFilter.create(filterIdGenerator, ctx.getRootAnalyzer(), conjunct,
-                    joinNode, filterType, bloomFilterSizeLimits_,
+                    joinNode, filterType, filterSizeLimits_,
                     /* isTimestampTruncation */ true);
             if (filter2 == null) continue;
             registerRuntimeFilter(filter2);
@@ -797,6 +912,13 @@ public final class RuntimeFilterGenerator {
     runtimeFilter.markFinalized();
   }
 
+  public static boolean enableOverlapFilter(TQueryOptions queryOptions) {
+    return queryOptions.parquet_read_statistics
+        && ((queryOptions.isMinmax_filter_sorted_columns()
+                || queryOptions.getMinmax_filter_threshold() > 0.0)
+            || queryOptions.isMinmax_filter_partition_columns());
+  }
+
   /**
    * Assigns runtime filters to a specific scan node 'scanNode'.
    * The assigned filters are the ones for which 'scanNode' can be used as a destination
@@ -808,8 +930,8 @@ public final class RuntimeFilterGenerator {
    *    to 'scanNode' if the filter is produced within the same fragment that contains the
    *    scan node.
    * 3. Only Hdfs and Kudu scan nodes are supported:
-   *     a. If the target is an HdfsScanNode, the filter must be type BLOOM for non
-   *        Parquet tables, or type BLOOM and/or MIN_MAX for Parquet tables.
+   *     a. If the target is an HdfsScanNode, the filter must be type BLOOM/IN_LIST for
+   *        non Parquet tables, or type BLOOM/MIN_MAX/IN_LIST for Parquet tables.
    *     b. If the target is a KuduScanNode, the filter could be type MIN_MAX, and/or
    *        BLOOM, the target must be a slot ref on a column, and the comp op cannot
    *        be 'not distinct'.
@@ -825,14 +947,13 @@ public final class RuntimeFilterGenerator {
     Analyzer analyzer = ctx.getRootAnalyzer();
     boolean disableRowRuntimeFiltering =
         ctx.getQueryOptions().isDisable_row_runtime_filtering();
-    boolean disable_overlap_filter =
-        ctx.getQueryOptions().getMinmax_filter_threshold() == 0.0;
+    boolean enable_overlap_filter = enableOverlapFilter(ctx.getQueryOptions());
     TRuntimeFilterMode runtimeFilterMode = ctx.getQueryOptions().getRuntime_filter_mode();
-    TEnabledRuntimeFilterTypes enabledRuntimeFilterTypes =
+    Set<TRuntimeFilterType> enabledRuntimeFilterTypes =
         ctx.getQueryOptions().getEnabled_runtime_filter_types();
 
     // Init the overlap predicate for the hdfs scan node.
-    if (scanNode instanceof HdfsScanNode && !disable_overlap_filter) {
+    if (scanNode instanceof HdfsScanNode && enable_overlap_filter) {
       ((HdfsScanNode) scanNode).initOverlapPredicate(analyzer);
     }
 
@@ -843,6 +964,8 @@ public final class RuntimeFilterGenerator {
       boolean isBoundByPartitionColumns = isBoundByPartitionColumns(analyzer, targetExpr,
           scanNode);
       if (disableRowRuntimeFiltering && !isBoundByPartitionColumns) continue;
+      boolean isColumnInDataFile = isColumnInDataFile(scanNode.getTupleDesc().getTable(),
+          isBoundByPartitionColumns);
       boolean isLocalTarget = isLocalTarget(filter, scanNode);
       if (runtimeFilterMode == TRuntimeFilterMode.LOCAL && !isLocalTarget) continue;
 
@@ -852,35 +975,30 @@ public final class RuntimeFilterGenerator {
           continue;
         }
         if (filter.getType() == TRuntimeFilterType.MIN_MAX) {
-          boolean allow_min_max =
-              enabledRuntimeFilterTypes == TEnabledRuntimeFilterTypes.MIN_MAX
-              || enabledRuntimeFilterTypes == TEnabledRuntimeFilterTypes.ALL;
-          if (!allow_min_max) {
+          Preconditions.checkState(
+              enabledRuntimeFilterTypes.contains(TRuntimeFilterType.MIN_MAX),
+              "MIN_MAX filters should not be generated");
+          if (!enable_overlap_filter) continue;
+          // Try to compute an overlap predicate for the filter. This predicate will be
+          // used to filter out partitions, or row groups, pages or rows in Parquet data
+          // files.
+          if (!((HdfsScanNode) scanNode)
+                   .tryToComputeOverlapPredicate(
+                       analyzer, filter, targetExpr, isBoundByPartitionColumns)) {
             continue;
           }
-          // TODO: Apply min/max filters on partition columns.
-          if (isBoundByPartitionColumns) {
-            continue;
-          }
-          if (!disable_overlap_filter) {
-            // If the filter is not defined on partition columns, try to compute
-            // an overlap predicate for it. This predicate will be used to filter
-            // out row groups or pages in Parquet data files.
-            if (!((HdfsScanNode) scanNode)
-                .tryToComputeOverlapPredicate(analyzer, filter, targetExpr)) {
-              continue;
-            }
-          } else {
+        } else if (filter.getType() == TRuntimeFilterType.IN_LIST) {
+          // Only assign IN-list filters on ORC tables.
+          if (!((HdfsScanNode) scanNode).getFileFormats().contains(HdfsFileFormat.ORC)) {
             continue;
           }
         }
       } else {
-        Preconditions.checkState(scanNode instanceof KuduScanNode);
+        // assign filters to KuduScanNode
         if (filter.getType() == TRuntimeFilterType.BLOOM) {
-          if (enabledRuntimeFilterTypes != TEnabledRuntimeFilterTypes.BLOOM
-              && enabledRuntimeFilterTypes != TEnabledRuntimeFilterTypes.ALL) {
-            continue;
-          }
+          Preconditions.checkState(
+              enabledRuntimeFilterTypes.contains(TRuntimeFilterType.BLOOM),
+              "BLOOM filters should not be generated!");
           // TODO: Support Kudu VARCHAR Bloom Filter
           if (targetExpr.getType().isVarchar()) continue;
           // Kudu only supports targeting a single column, not general exprs, so the
@@ -895,12 +1013,10 @@ public final class RuntimeFilterGenerator {
           }
           SlotRef slotRef = (SlotRef) targetExpr;
           if (slotRef.getDesc().getColumn() == null) continue;
-        } else {
-          Preconditions.checkState(filter.getType() == TRuntimeFilterType.MIN_MAX);
-          if (enabledRuntimeFilterTypes != TEnabledRuntimeFilterTypes.MIN_MAX
-              && enabledRuntimeFilterTypes != TEnabledRuntimeFilterTypes.ALL) {
-            continue;
-          }
+        } else if (filter.getType() == TRuntimeFilterType.MIN_MAX) {
+          Preconditions.checkState(
+              enabledRuntimeFilterTypes.contains(TRuntimeFilterType.MIN_MAX),
+              "MIN_MAX filters should not be generated!");
           // TODO: IMPALA-9580: Support Kudu VARCHAR Min/Max Filters
           if (targetExpr.getType().isVarchar()) continue;
           SlotRef slotRef = targetExpr.unwrapSlotRef(true);
@@ -914,6 +1030,13 @@ public final class RuntimeFilterGenerator {
               || filter.getExprCompOp() == Operator.NOT_DISTINCT) {
             continue;
           }
+        } else {
+          Preconditions.checkState(filter.getType() == TRuntimeFilterType.IN_LIST);
+          Preconditions.checkState(
+              enabledRuntimeFilterTypes.contains(TRuntimeFilterType.IN_LIST),
+              "IN_LIST filters should not be generated!");
+          // TODO(IMPALA-11066): Runtime IN-list filters for Kudu tables
+          continue;
         }
       }
       TColumnValue lowValue = null;
@@ -930,12 +1053,13 @@ public final class RuntimeFilterGenerator {
       }
       RuntimeFilter.RuntimeFilterTarget target =
           new RuntimeFilter.RuntimeFilterTarget(scanNode, targetExpr,
-              isBoundByPartitionColumns, isLocalTarget, lowValue, highValue);
+              isBoundByPartitionColumns, isColumnInDataFile, isLocalTarget,
+              lowValue, highValue);
       filter.addTarget(target);
     }
 
     // finalize the overlap predicate for the hdfs scan node.
-    if (scanNode instanceof HdfsScanNode && !disable_overlap_filter) {
+    if (scanNode instanceof HdfsScanNode && enable_overlap_filter) {
       ((HdfsScanNode) scanNode).finalizeOverlapPredicate();
     }
   }
@@ -955,19 +1079,33 @@ public final class RuntimeFilterGenerator {
     Preconditions.checkState(targetExpr.isBoundByTupleIds(targetNode.getTupleIds()));
     TupleDescriptor baseTblDesc = targetNode.getTupleDesc();
     FeTable tbl = baseTblDesc.getTable();
-    if (tbl.getNumClusteringCols() == 0) return false;
+    if (tbl.getNumClusteringCols() == 0 && !(tbl instanceof FeIcebergTable)) return false;
     List<SlotId> sids = new ArrayList<>();
     targetExpr.getIds(null, sids);
     for (SlotId sid : sids) {
-      SlotDescriptor slotDesc = analyzer.getSlotDesc(sid);
-      if (slotDesc.getColumn() == null
-          || slotDesc.getColumn().getPosition() >= tbl.getNumClusteringCols()) {
+      Column col = analyzer.getSlotDesc(sid).getColumn();
+      if (col == null) return false;
+      if (!tbl.isClusteringColumn(col) && !tbl.isComputedPartitionColumn(col)) {
         return false;
       }
     }
     return true;
   }
 
+  /**
+   * Return true if the column is actually stored in the data file. E.g. partition
+   * columns are not stored in most cases.
+   */
+  private static boolean isColumnInDataFile(FeTable tbl,
+      boolean isBoundByPartitionColumns) {
+    // Non-partition values are always sotred in the data files.
+    if (!isBoundByPartitionColumns) return true;
+    // Iceberg uses hidden partitioning which means all columns are stored in the data
+    // files.
+    if (tbl instanceof FeIcebergTable) return true;
+    return false;
+  }
+
   /**
    * Computes the target expr for a specified runtime filter 'filter' to be applied at
    * the scan node with target tuple descriptor 'targetTid'.
