@@ -245,6 +245,10 @@ public class UnionNode extends PlanNode {
 
     for (int i = 0; i < children_.size(); i++) {
       if (!isChildPassthrough(analyzer, children_.get(i), resultExprLists_.get(i))) {
+        for (Expr expr : resultExprLists_.get(i)) {
+          Preconditions.checkState(!expr.getType().isCollectionType(),
+              "only pass-through UNION ALL is supported for array columns");
+        }
         newResultExprLists.add(resultExprLists_.get(i));
         newChildren.add(children_.get(i));
       }
@@ -295,7 +299,12 @@ public class UnionNode extends PlanNode {
       Preconditions.checkState(exprList.size() == slots.size());
       List<Expr> newExprList = new ArrayList<>();
       for (int i = 0; i < exprList.size(); ++i) {
-        if (slots.get(i).isMaterialized()) newExprList.add(exprList.get(i));
+        if (slots.get(i).isMaterialized()) {
+          Expr constExpr = exprList.get(i);
+          // Disable codegen for const expressions which will only ever be evaluated once.
+          if (!isInSubplan_) constExpr.disableCodegen();
+          newExprList.add(constExpr);
+        }
       }
       materializedConstExprLists_.add(newExprList);
     }
