@@ -23,8 +23,10 @@ import static org.apache.hadoop.fs.CommonConfigurationKeysPublic
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.security.authentication.util.KerberosName;
 import org.apache.impala.analysis.SqlScanner;
+import org.apache.impala.common.PrintUtils;
 import org.apache.impala.thrift.TBackendGflags;
 
+import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 import com.google.common.base.Strings;
 
@@ -37,8 +39,14 @@ public class BackendConfig {
 
   private TBackendGflags backendCfg_;
 
+  // Thresholds in warning slow or large catalog responses.
+  private long warnCatalogResponseSize_;
+  private long warnCatalogResponseDurationMs_;
+
   private BackendConfig(TBackendGflags cfg) {
     backendCfg_ = cfg;
+    warnCatalogResponseSize_ = cfg.warn_catalog_response_size_mb * 1024L * 1024L;
+    warnCatalogResponseDurationMs_ = cfg.warn_catalog_response_duration_s * 1000L;
   }
 
   public static void create(TBackendGflags cfg) {
@@ -73,6 +81,8 @@ public class BackendConfig {
   }
   public int getKuduClientTimeoutMs() { return backendCfg_.kudu_operation_timeout_ms; }
 
+  public String getKuduSaslProtocolName() { return backendCfg_.kudu_sasl_protocol_name; }
+
   public String getImpalaBuildVersion() { return backendCfg_.impala_build_version; }
   public int getImpalaLogLevel() { return backendCfg_.impala_log_lvl; }
   public int getNonImpalaJavaVlogLevel() { return backendCfg_.non_impala_java_vlog; }
@@ -93,6 +103,10 @@ public class BackendConfig {
     return !Strings.isNullOrEmpty(backendCfg_.authorized_proxy_group_config);
   }
 
+  public boolean isShellBasedGroupsMappingEnabled() {
+    return backendCfg_.enable_shell_based_groups_mapping_support;
+  }
+
   public boolean disableCatalogDataOpsDebugOnly() {
     return backendCfg_.disable_catalog_data_ops_debug_only;
   }
@@ -131,10 +145,6 @@ public class BackendConfig {
 
   public boolean isInsertEventsEnabled() { return backendCfg_.enable_insert_events; }
 
-  public boolean isOrcScannerEnabled() {
-    return backendCfg_.enable_orc_scanner;
-  }
-
   /**
    * Returns the value of the `authorization_factory_class` flag or `null` if
    * the flag was not specified.
@@ -261,6 +271,12 @@ public class BackendConfig {
     backendCfg_.allow_ordinals_in_having = allow_ordinals_in_having;
   }
 
+  public long getWarnCatalogResponseSize() { return warnCatalogResponseSize_; }
+
+  public long getWarnCatalogResponseDurationMs() {
+    return warnCatalogResponseDurationMs_;
+  }
+
   // Inits the auth_to_local configuration in the static KerberosName class.
   private static void initAuthToLocal() {
     // If auth_to_local is enabled, we read the configuration hadoop.security.auth_to_local
@@ -288,4 +304,55 @@ public class BackendConfig {
   public int getMaxWaitTimeForSyncDdlSecs() {
     return backendCfg_.max_wait_time_for_sync_ddl_s;
   }
+
+  public boolean startHmsServer() {
+    return backendCfg_.start_hms_server;
+  }
+
+  public int getHMSPort() {
+    return backendCfg_.hms_port;
+  }
+
+  public boolean fallbackToHMSOnErrors() {
+    return backendCfg_.fallback_to_hms_on_errors;
+  }
+
+  @VisibleForTesting
+  public void setEnableCatalogdHMSCache(boolean flag) {
+    backendCfg_.enable_catalogd_hms_cache = flag;
+  }
+
+  public boolean enableCatalogdHMSCache() {
+    return backendCfg_.enable_catalogd_hms_cache;
+  }
+
+  public boolean invalidateCatalogdHMSCacheOnDDLs() {
+    return backendCfg_.invalidate_hms_cache_on_ddls;
+  }
+
+  public String getStartupFilesystemCheckDirectories() {
+    return backendCfg_.startup_filesystem_check_directories;
+  }
+
+  public boolean getHMSEventIncrementalRefreshTransactionalTable() {
+    return backendCfg_.hms_event_incremental_refresh_transactional_table;
+  }
+
+  public boolean isAutoCheckCompaction() {
+    return backendCfg_.auto_check_compaction;
+  }
+
+  @VisibleForTesting
+  public void setInvalidateCatalogdHMSCacheOnDDLs(boolean flag) {
+    backendCfg_.invalidate_hms_cache_on_ddls = flag;
+  }
+
+  public boolean enableSyncToLatestEventOnDdls() {
+    return backendCfg_.enable_sync_to_latest_event_on_ddls;
+  }
+
+  @VisibleForTesting
+  public void setEnableSyncToLatestEventOnDdls(boolean flag) {
+    backendCfg_.enable_sync_to_latest_event_on_ddls = flag;
+  }
 }
