@@ -90,6 +90,7 @@ import org.apache.impala.thrift.TShowStatsOp;
 import org.apache.impala.thrift.TShowStatsParams;
 import org.apache.impala.thrift.TDescribeHistoryParams;
 import org.apache.impala.thrift.TTableName;
+import org.apache.impala.thrift.TUniqueId;
 import org.apache.impala.thrift.TUpdateCatalogCacheRequest;
 import org.apache.impala.thrift.TUpdateExecutorMembershipRequest;
 import org.apache.impala.thrift.TWrappedHttpRequest;
@@ -109,6 +110,7 @@ import org.slf4j.LoggerFactory;
 
 import java.io.File;
 import java.io.IOException;
+import java.lang.IllegalArgumentException;
 import java.util.Collections;
 import java.util.Enumeration;
 import java.util.List;
@@ -334,9 +336,7 @@ public class JniFrontend {
     Preconditions.checkNotNull(frontend_);
     TDescribeHistoryParams params = new TDescribeHistoryParams();
     JniUtil.deserializeThrift(protocolFactory_, params, thriftParams);
-    TGetTableHistoryResult result = frontend_.getTableHistory(
-        params.getTable_name().getDb_name(), params.getTable_name().getTable_name());
-
+    TGetTableHistoryResult result = frontend_.getTableHistory(params);
     TSerializer serializer = new TSerializer(protocolFactory_);
     try {
       return serializer.serialize(result);
@@ -763,6 +763,32 @@ public class JniFrontend {
     return frontend_.getSaml2Client().validateBearer(webContext);
   }
 
+  /**
+   * Aborts a Kudu transaction.
+   * @param queryId the id of the query.
+   * @throws TransactionException, ImpalaException
+   */
+  public void abortKuduTransaction(byte[] thriftQueryId)
+      throws TransactionException, ImpalaException {
+    Preconditions.checkNotNull(frontend_);
+    TUniqueId queryId = new TUniqueId();
+    JniUtil.deserializeThrift(protocolFactory_, queryId, thriftQueryId);
+    this.frontend_.abortKuduTransaction(queryId);
+  }
+
+  /**
+   * Commits a Kudu transaction.
+   * @param queryId the id of the query.
+   * @throws TransactionException, ImpalaException
+   */
+  public void commitKuduTransaction(byte[] thriftQueryId)
+      throws TransactionException, ImpalaException {
+    Preconditions.checkNotNull(frontend_);
+    TUniqueId queryId = new TUniqueId();
+    JniUtil.deserializeThrift(protocolFactory_, queryId, thriftQueryId);
+    this.frontend_.commitKuduTransaction(queryId);
+  }
+
   /**
    * Returns an error string describing configuration issue with the groups mapping
    * provider implementation.
@@ -795,7 +821,8 @@ public class JniFrontend {
     output.append(checkLogFilePermission());
     output.append(checkFileSystem(CONF));
     output.append(checkShortCircuitRead(CONF));
-    if (BackendConfig.INSTANCE.isAuthorizedProxyGroupEnabled()) {
+    if (BackendConfig.INSTANCE.isAuthorizedProxyGroupEnabled() &&
+        !BackendConfig.INSTANCE.isShellBasedGroupsMappingEnabled()) {
       output.append(checkGroupsMappingProvider(CONF));
     }
     return output.toString();
@@ -898,12 +925,35 @@ public class JniFrontend {
       return "couldn't retrieve FileSystem:\n" + e.getMessage();
     }
 
-    try {
-      FileSystemUtil.getTotalNumVisibleFiles(new Path("/"));
-    } catch (IOException e) {
-      return "Could not read the root directory at " +
-          CONF.get(CommonConfigurationKeysPublic.FS_DEFAULT_NAME_KEY) +
-          ". Error was: \n" + e.getMessage();
+    String startup_filesystem_check_dirs =
+        BackendConfig.INSTANCE.getStartupFilesystemCheckDirectories();
+    String[] path_strings = startup_filesystem_check_dirs.split(",");
+    for (String path_string : path_strings) {
+      // Skip empty strings (or if there are no paths to check)
+      if (path_string.isEmpty()) {
+        continue;
+      }
+      try {
+        LOG.info("Verifying access to " + path_string);
+        Path path = new Path(path_string);
+        FileSystem fs = path.getFileSystem(CONF);
+        if (!fs.exists(path)) {
+          return "Invalid path specified for startup_filesystem_check_directories: " +
+              path_string + " does not exist.";
+        }
+        if (!fs.isDirectory(path)) {
+          return "Invalid path specified for startup_filesystem_check_directories: " +
+              path_string + " is not a directory.";
+        }
+        FileSystemUtil.getTotalNumVisibleFiles(path);
+        LOG.info("Successfully listed " + path_string);
+      } catch (IllegalArgumentException e) {
+        return "Invalid path specified for startup_filesystem_check_directories: " +
+            path_string + ". Error was: \n" + e.getMessage();
+      } catch (IOException e) {
+        return "Could not read the path at " + path_string +
+            ". Error was: \n" + e.getMessage();
+      }
     }
     return "";
   }
