@@ -18,8 +18,7 @@ package org.apache.impala.util;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
-import com.google.common.collect.Lists;
-import com.google.common.collect.Range;
+import com.google.common.base.Stopwatch;
 import com.google.errorprone.annotations.Immutable;
 
 import java.io.IOException;
@@ -34,30 +33,38 @@ import org.apache.impala.catalog.CatalogException;
 import org.apache.impala.catalog.CatalogServiceCatalog;
 import org.apache.impala.catalog.Column;
 import org.apache.impala.catalog.FileMetadataLoader.LoadStats;
+import org.apache.impala.catalog.HdfsPartition;
 import org.apache.impala.catalog.HdfsPartition.FileDescriptor;
 import org.apache.impala.catalog.HdfsTable;
+import org.apache.impala.catalog.MetaStoreClientPool;
 import org.apache.impala.catalog.ScalarType;
 import org.apache.impala.catalog.StructField;
 import org.apache.impala.catalog.StructType;
 import org.apache.impala.common.FileSystemUtil;
 import org.apache.impala.common.Pair;
+import org.apache.impala.common.PrintUtils;
 import org.apache.impala.common.Reference;
+import org.apache.impala.compat.MetastoreShim;
 import org.apache.impala.thrift.THdfsFileDesc;
 import org.apache.impala.thrift.TPartialPartitionInfo;
 import org.apache.impala.thrift.TTransactionalType;
+import org.apache.thrift.TException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import java.util.ArrayList;
 import java.util.Comparator;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.concurrent.TimeUnit;
 import java.util.function.Predicate;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
+import java.util.stream.Collectors;
 
 import javax.annotation.Nullable;
 
@@ -697,7 +704,7 @@ public class AcidUtils {
     // if the provided table id does not match with what CatalogService has we return
     // -1 indicating that cached table is stale.
     if (tableId != CatalogServiceCatalog.TABLE_ID_UNAVAILABLE
-        && tbl.getMetaStoreTable().getId() != tableId) {
+        && MetastoreShim.getTableId(tbl.getMetaStoreTable()) != tableId) {
       return -1;
     }
     return compare(tbl.getValidWriteIds(), validWriteIdList);
@@ -766,4 +773,22 @@ public class AcidUtils {
     }
     return invalidIds[invalidIds.length-1] == hwm;
   }
+
+  /**
+   * This method fetches latest compaction id from HMS and compares it with the cached
+   * compaction id. It returns list of partitions that need to refresh file metadata due
+   * to compactions. If none of the partitions need to refresh, it returns empty list.
+   */
+  public static List<HdfsPartition.Builder> getPartitionsForRefreshingFileMetadata(
+      CatalogServiceCatalog catalog, HdfsTable hdfsTable) throws CatalogException {
+    Stopwatch sw = Stopwatch.createStarted();
+    Preconditions.checkState(hdfsTable.isReadLockedByCurrentThread());
+
+    List<HdfsPartition.Builder> partBuilders = MetastoreShim
+        .getPartitionsForRefreshingFileMetadata(catalog, hdfsTable);
+    LOG.debug("Checked the latest compaction id for {}. Time taken: {}",
+        hdfsTable.getFullName(),
+        PrintUtils.printTimeMs(sw.stop().elapsed(TimeUnit.MILLISECONDS)));
+    return partBuilders;
+  }
 }
