@@ -17,6 +17,7 @@
 
 package org.apache.impala.util;
 
+import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.time.Instant;
 import java.time.LocalDateTime;
@@ -25,38 +26,52 @@ import java.time.ZoneOffset;
 import java.time.temporal.ChronoUnit;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.HashMap;
 import java.util.Collections;
+import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
+import java.util.Map;
+import java.util.Set;
 
 import com.google.common.base.Preconditions;
+import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.Maps;
 import com.google.common.hash.Hasher;
 import com.google.common.hash.Hashing;
+import com.google.common.primitives.Ints;
+import com.google.common.primitives.Longs;
+import com.google.flatbuffers.FlatBufferBuilder;
 
-import org.apache.impala.catalog.IcebergStructField;
 import org.apache.impala.common.Pair;
-import org.apache.iceberg.BaseTable;
-import org.apache.iceberg.UpdateSchema;
+import org.apache.impala.fb.FbFileMetadata;
+import org.apache.impala.fb.FbIcebergDataFileFormat;
+import org.apache.impala.fb.FbIcebergMetadata;
+import org.apache.impala.fb.FbIcebergPartitionTransformValue;
+import org.apache.impala.fb.FbIcebergTransformType;
+import org.apache.iceberg.Transaction;
 import org.apache.iceberg.catalog.TableIdentifier;
 import org.apache.iceberg.DataFile;
+import org.apache.iceberg.FileFormat;
 import org.apache.iceberg.FileScanTask;
 import org.apache.iceberg.TableScan;
 import org.apache.iceberg.expressions.UnboundPredicate;
+import org.apache.iceberg.io.CloseableIterable;
 import org.apache.iceberg.PartitionField;
 import org.apache.iceberg.PartitionSpec;
 import org.apache.iceberg.Schema;
+import org.apache.iceberg.Snapshot;
 import org.apache.iceberg.StructLike;
 import org.apache.iceberg.Table;
-import org.apache.iceberg.TableMetadata;
+import org.apache.iceberg.mr.Catalogs;
 import org.apache.iceberg.transforms.PartitionSpecVisitor;
-import org.apache.iceberg.transforms.Transform;
 import org.apache.iceberg.types.Conversions;
 import org.apache.iceberg.types.Type;
 import org.apache.iceberg.types.Types;
 import org.apache.impala.analysis.IcebergPartitionField;
 import org.apache.impala.analysis.IcebergPartitionSpec;
 import org.apache.impala.analysis.IcebergPartitionTransform;
+import org.apache.impala.analysis.TimeTravelSpec;
+import org.apache.impala.analysis.TimeTravelSpec.Kind;
 import org.apache.impala.catalog.Catalog;
 import org.apache.impala.catalog.FeIcebergTable;
 import org.apache.impala.catalog.HdfsFileFormat;
@@ -66,14 +81,15 @@ import org.apache.impala.catalog.iceberg.IcebergHadoopCatalog;
 import org.apache.impala.catalog.iceberg.IcebergHadoopTables;
 import org.apache.impala.catalog.iceberg.IcebergHiveCatalog;
 import org.apache.impala.catalog.iceberg.IcebergCatalog;
+import org.apache.impala.catalog.iceberg.IcebergCatalogs;
 import org.apache.impala.common.ImpalaRuntimeException;
-import org.apache.impala.thrift.TCreateTableParams;
+import org.apache.impala.thrift.TCompressionCodec;
+import org.apache.impala.thrift.THdfsCompression;
 import org.apache.impala.thrift.THdfsFileFormat;
 import org.apache.impala.thrift.TIcebergCatalog;
 import org.apache.impala.thrift.TIcebergFileFormat;
 import org.apache.impala.thrift.TIcebergPartitionField;
 import org.apache.impala.thrift.TIcebergPartitionSpec;
-import org.apache.impala.thrift.TIcebergPartitionTransform;
 import org.apache.impala.thrift.TIcebergPartitionTransformType;
 
 public class IcebergUtil {
@@ -100,6 +116,7 @@ public class IcebergUtil {
       case HADOOP_TABLES: return IcebergHadoopTables.getInstance();
       case HIVE_CATALOG: return IcebergHiveCatalog.getInstance();
       case HADOOP_CATALOG: return new IcebergHadoopCatalog(location);
+      case CATALOGS: return IcebergCatalogs.getInstance();
       default: throw new ImpalaRuntimeException (
           "Unexpected catalog type: " + catalog.toString());
     }
@@ -110,17 +127,17 @@ public class IcebergUtil {
    */
   public static Table loadTable(FeIcebergTable feTable) throws TableLoadingException {
     return loadTable(feTable.getIcebergCatalog(), getIcebergTableIdentifier(feTable),
-        feTable.getIcebergCatalogLocation());
+        feTable.getIcebergCatalogLocation(), feTable.getMetaStoreTable().getParameters());
   }
 
   /**
    * Helper method to load native Iceberg table.
    */
   public static Table loadTable(TIcebergCatalog catalog, TableIdentifier tableId,
-      String location) throws TableLoadingException {
+      String location, Map<String, String> tableProps) throws TableLoadingException {
     try {
       IcebergCatalog cat = getIcebergCatalog(catalog, location);
-      return cat.loadTable(tableId, location);
+      return cat.loadTable(tableId, location, tableProps);
     } catch (ImpalaRuntimeException e) {
       throw new TableLoadingException(String.format(
           "Failed to load Iceberg table: %s at location: %s",
@@ -128,26 +145,6 @@ public class IcebergUtil {
     }
   }
 
-  /**
-   * Get TableMetadata by FeIcebergTable
-   */
-  public static TableMetadata getIcebergTableMetadata(FeIcebergTable table)
-      throws TableLoadingException {
-    BaseTable iceTable = (BaseTable)IcebergUtil.loadTable(table);
-    return iceTable.operations().current();
-  }
-
-  /**
-   * Get TableMetadata by related info tableName is table full name, usually
-   * database.table
-   */
-  public static TableMetadata getIcebergTableMetadata(TIcebergCatalog catalog,
-      TableIdentifier tableId, String location) throws TableLoadingException {
-    BaseTable baseTable = (BaseTable)IcebergUtil.loadTable(catalog,
-        tableId, location);
-    return baseTable.operations().current();
-  }
-
   /**
    * Get Iceberg table identifier by table property
    */
@@ -158,6 +155,10 @@ public class IcebergUtil {
   public static TableIdentifier getIcebergTableIdentifier(
       org.apache.hadoop.hive.metastore.api.Table msTable) {
     String name = msTable.getParameters().get(IcebergTable.ICEBERG_TABLE_IDENTIFIER);
+    if (name == null || name.isEmpty()) {
+      // Iceberg's Catalogs API uses table property 'name' for the table id.
+      name = msTable.getParameters().get(Catalogs.NAME);
+    }
     if (name == null || name.isEmpty()) {
       return TableIdentifier.of(msTable.getDbName(), msTable.getTableName());
     }
@@ -170,12 +171,12 @@ public class IcebergUtil {
   }
 
   /**
-   * Get Iceberg UpdateSchema from 'feTable', usually use UpdateSchema to update Iceberg
+   * Get Iceberg Transaction for 'feTable', usually use Transaction to update Iceberg
    * table schema.
    */
-  public static UpdateSchema getIcebergUpdateSchema(FeIcebergTable feTable)
+  public static Transaction getIcebergTransaction(FeIcebergTable feTable)
       throws TableLoadingException, ImpalaRuntimeException {
-    return getIcebergCatalog(feTable).loadTable(feTable).updateSchema();
+    return feTable.getIcebergApiTable().newTransaction();
   }
 
   /**
@@ -207,6 +208,8 @@ public class IcebergUtil {
       } else if (transformType == TIcebergPartitionTransformType.TRUNCATE) {
         builder.truncate(partitionField.getField_name(),
             partitionField.getTransform().getTransform_param());
+      } else if (transformType == TIcebergPartitionTransformType.VOID) {
+        builder.alwaysNull(partitionField.getField_name());
       } else {
         throw new ImpalaRuntimeException(String.format("Skip partition: %s, %s",
             partitionField.getField_name(), transformType));
@@ -215,15 +218,29 @@ public class IcebergUtil {
     return builder.build();
   }
 
+  /**
+   * Returns true if 'msTable' uses HiveCatalog.
+   */
+  public static boolean isHiveCatalog(
+      org.apache.hadoop.hive.metastore.api.Table msTable) {
+    TIcebergCatalog tCat = getTIcebergCatalog(msTable);
+    if (tCat == TIcebergCatalog.HIVE_CATALOG) return true;
+    if (tCat == TIcebergCatalog.CATALOGS) {
+      String catName = msTable.getParameters().get(IcebergTable.ICEBERG_CATALOG);
+      tCat = IcebergCatalogs.getInstance().getUnderlyingCatalogType(catName);
+      return tCat == TIcebergCatalog.HIVE_CATALOG;
+    }
+    return false;
+  }
+
   /**
    * Get iceberg table catalog type from hms table properties
    * use HiveCatalog as default
    */
   public static TIcebergCatalog getTIcebergCatalog(
       org.apache.hadoop.hive.metastore.api.Table msTable) {
-    TIcebergCatalog catalog = getTIcebergCatalog(
+    return getTIcebergCatalog(
         msTable.getParameters().get(IcebergTable.ICEBERG_CATALOG));
-    return catalog == null ? TIcebergCatalog.HIVE_CATALOG : catalog;
   }
 
   /**
@@ -234,10 +251,33 @@ public class IcebergUtil {
       return TIcebergCatalog.HADOOP_TABLES;
     } else if ("hadoop.catalog".equalsIgnoreCase(catalog)) {
       return TIcebergCatalog.HADOOP_CATALOG;
-    } else if ("hive.catalog".equalsIgnoreCase(catalog)) {
+    } else if ("hive.catalog".equalsIgnoreCase(catalog) ||
+               catalog == null) {
       return TIcebergCatalog.HIVE_CATALOG;
     }
-    return null;
+    return TIcebergCatalog.CATALOGS;
+  }
+
+  /**
+   * Return the underlying Iceberg catalog when Iceberg Catalogs is being used, simply
+   * return the Iceberg catalog otherwise.
+   */
+  public static TIcebergCatalog getUnderlyingCatalog(
+      org.apache.hadoop.hive.metastore.api.Table msTable) {
+    return getUnderlyingCatalog(
+        msTable.getParameters().get(IcebergTable.ICEBERG_CATALOG));
+  }
+
+  /**
+   * Return the underlying Iceberg catalog when Iceberg Catalogs is being used, simply
+   * return the Iceberg catalog otherwise.
+   */
+  public static TIcebergCatalog getUnderlyingCatalog(String catalog) {
+    TIcebergCatalog tCat = getTIcebergCatalog(catalog);
+    if (tCat == TIcebergCatalog.CATALOGS) {
+      return IcebergCatalogs.getInstance().getUnderlyingCatalogType(catalog);
+    }
+    return tCat;
   }
 
   /**
@@ -249,6 +289,23 @@ public class IcebergUtil {
     return msTable.getParameters().get(IcebergTable.ICEBERG_CATALOG_LOCATION);
   }
 
+  /**
+   * Get iceberg table file format from hms table properties
+   */
+  public static TIcebergFileFormat getIcebergFileFormat(
+      org.apache.hadoop.hive.metastore.api.Table msTable) {
+    TIcebergFileFormat fileFormat = null;
+    Map<String, String> params = msTable.getParameters();
+    if (params.containsKey(IcebergTable.ICEBERG_FILE_FORMAT)) {
+      fileFormat = IcebergUtil.getIcebergFileFormat(
+          params.get(IcebergTable.ICEBERG_FILE_FORMAT));
+    } else {
+      // Accept "iceberg.file_format" for backward compatibility.
+      fileFormat = IcebergUtil.getIcebergFileFormat(params.get("iceberg.file_format"));
+    }
+    return fileFormat == null ? TIcebergFileFormat.PARQUET : fileFormat;
+  }
+
   /**
    * Get TIcebergFileFormat from a string, usually from table properties.
    * Returns PARQUET when 'format' is null. Returns null for invalid formats.
@@ -262,6 +319,47 @@ public class IcebergUtil {
     return null;
   }
 
+  /**
+   * Map from parquet compression codec names to a compression type.
+   * The list of parquet supported compression codecs was taken from
+   * hdfs-parquet-table-writer.cc.
+   */
+  public static final ImmutableMap<String, THdfsCompression> PARQUET_CODEC_MAP =
+      ImmutableMap.<String, THdfsCompression>builder().
+          put("none", THdfsCompression.NONE).
+          put("gzip", THdfsCompression.GZIP).
+          put("snappy", THdfsCompression.SNAPPY).
+          put("lz4", THdfsCompression.LZ4).
+          put("zstd", THdfsCompression.ZSTD).
+          build();
+
+  public static THdfsCompression getIcebergParquetCompressionCodec(String codec) {
+    if (codec == null) return IcebergTable.DEFAULT_PARQUET_COMPRESSION_CODEC;
+    return PARQUET_CODEC_MAP.get(codec.toLowerCase());
+  }
+
+  public static long getIcebergParquetRowGroupSize(String rowGroupSize) {
+    if (rowGroupSize == null) return IcebergTable.UNSET_PARQUET_ROW_GROUP_SIZE;
+
+    Long rgSize = Longs.tryParse(rowGroupSize);
+    if (rgSize == null || rgSize < IcebergTable.MIN_PARQUET_ROW_GROUP_SIZE ||
+        rgSize > IcebergTable.MAX_PARQUET_ROW_GROUP_SIZE) {
+      return IcebergTable.UNSET_PARQUET_ROW_GROUP_SIZE;
+    }
+    return rgSize;
+  }
+
+  public static long getIcebergParquetPageSize(String pageSize) {
+    if (pageSize == null) return IcebergTable.UNSET_PARQUET_PAGE_SIZE;
+
+    Long pSize = Longs.tryParse(pageSize);
+    if (pSize == null || pSize < IcebergTable.MIN_PARQUET_PAGE_SIZE ||
+        pSize > IcebergTable.MAX_PARQUET_PAGE_SIZE) {
+      return IcebergTable.UNSET_PARQUET_PAGE_SIZE;
+    }
+    return pSize;
+  }
+
   public static IcebergPartitionTransform getPartitionTransform(
       PartitionField field, HashMap<String, Integer> transformParams)
       throws TableLoadingException {
@@ -284,24 +382,24 @@ public class IcebergUtil {
 
   public static TIcebergPartitionTransformType getPartitionTransformType(
       String transformType) throws TableLoadingException {
+    Preconditions.checkNotNull(transformType);
     transformType = transformType.toUpperCase();
     if ("IDENTITY".equals(transformType)) {
       return TIcebergPartitionTransformType.IDENTITY;
-    } else if ("HOUR".equals(transformType)) {
-      return TIcebergPartitionTransformType.HOUR;
-    } else if ("DAY".equals(transformType)) {
-      return TIcebergPartitionTransformType.DAY;
-    } else if ("MONTH".equals(transformType)) {
-      return TIcebergPartitionTransformType.MONTH;
-    } else if ("YEAR".equals(transformType)) {
-      return TIcebergPartitionTransformType.YEAR;
     } else if (transformType != null && transformType.startsWith("BUCKET")) {
       return TIcebergPartitionTransformType.BUCKET;
     } else if (transformType != null && transformType.startsWith("TRUNCATE")) {
       return TIcebergPartitionTransformType.TRUNCATE;
-    } else {
-      throw new TableLoadingException("Unsupported iceberg partition type: " +
-      transformType);
+    }
+    switch (transformType) {
+      case "HOUR":  case "HOURS":  return TIcebergPartitionTransformType.HOUR;
+      case "DAY":   case "DAYS":   return TIcebergPartitionTransformType.DAY;
+      case "MONTH": case "MONTHS": return TIcebergPartitionTransformType.MONTH;
+      case "YEAR":  case "YEARS":  return TIcebergPartitionTransformType.YEAR;
+      case "VOID": return TIcebergPartitionTransformType.VOID;
+      default:
+        throw new TableLoadingException("Unsupported iceberg partition type: " +
+            transformType);
     }
   }
 
@@ -324,7 +422,7 @@ public class IcebergUtil {
   public static HashMap<String, Integer> getPartitionTransformParams(PartitionSpec spec)
       throws TableLoadingException {
     List<Pair<String, Integer>> transformParams = PartitionSpecVisitor.visit(
-        spec.schema(), spec, new PartitionSpecVisitor<Pair<String, Integer>>() {
+        spec, new PartitionSpecVisitor<Pair<String, Integer>>() {
           @Override
           public Pair<String, Integer> identity(String sourceName, int sourceId) {
             String mappingKey = getPartitonTransformMappingKey(sourceId,
@@ -375,6 +473,14 @@ public class IcebergUtil {
                 TIcebergPartitionTransformType.HOUR);
             return new Pair<String, Integer>(mappingKey, null);
           }
+
+          @Override
+          public Pair<String, Integer> alwaysNull(int fieldId, String sourceName,
+              int sourceId) {
+            String mappingKey = getPartitonTransformMappingKey(sourceId,
+                TIcebergPartitionTransformType.VOID);
+            return new Pair<String, Integer>(mappingKey, null);
+          }
         });
     // Move the content of the List into a HashMap for faster querying in the future.
     HashMap<String, Integer> result = Maps.newHashMap();
@@ -412,22 +518,61 @@ public class IcebergUtil {
   }
 
   /**
-   * Get iceberg data file by file system table location and iceberg predicates
+   * Returns a Pair, the first element is the DataFiles by file system table location and
+   * iceberg predicates, the second element is whether or not a DeleteFile exists for the
+   * DataFiles in the first element.
    */
-  public static List<DataFile> getIcebergDataFiles(FeIcebergTable table,
-      List<UnboundPredicate> predicates) throws TableLoadingException {
-    if (table.snapshotId() == -1) return Collections.emptyList();
-    BaseTable baseTable =  (BaseTable)IcebergUtil.loadTable(table);
-    TableScan scan = baseTable.newScan().useSnapshot(table.snapshotId());
+  public static Pair<List<DataFile>, Boolean> getIcebergDataFiles(FeIcebergTable table,
+      List<UnboundPredicate> predicates, TimeTravelSpec timeTravelSpec)
+        throws TableLoadingException {
+    if (table.snapshotId() == -1) return new Pair<>(Collections.emptyList(), false);
+
+    TableScan scan = createScanAsOf(table, timeTravelSpec);
     for (UnboundPredicate predicate : predicates) {
       scan = scan.filter(predicate);
     }
 
     List<DataFile> dataFileList = new ArrayList<>();
-    for (FileScanTask task : scan.planFiles()) {
-      dataFileList.add(task.file());
+    Boolean hasDeleteFile = false;
+    try (CloseableIterable<FileScanTask> fileScanTasks = scan.planFiles()) {
+      for (FileScanTask task : fileScanTasks) {
+        if (!task.deletes().isEmpty()) {
+          hasDeleteFile = true;
+        }
+        dataFileList.add(task.file());
+      }
+    } catch (IOException e) {
+      throw new TableLoadingException("Data file list collection failed.", e);
     }
-    return dataFileList;
+    return new Pair<>(dataFileList, hasDeleteFile);
+  }
+
+  private static TableScan createScanAsOf(FeIcebergTable table,
+      TimeTravelSpec timeTravelSpec) {
+    TableScan scan = table.getIcebergApiTable().newScan();
+    if (timeTravelSpec == null) {
+      scan = scan.useSnapshot(table.snapshotId());
+    } else {
+      if (timeTravelSpec.getKind() == Kind.TIME_AS_OF) {
+        scan = createScanAsOfTime(timeTravelSpec, scan);
+      } else {
+        Preconditions.checkState(timeTravelSpec.getKind() == Kind.VERSION_AS_OF);
+        scan = scan.useSnapshot(timeTravelSpec.getAsOfVersion());
+      }
+    }
+    return scan;
+  }
+
+  private static TableScan createScanAsOfTime(TimeTravelSpec timeTravelSpec,
+      TableScan scan) {
+    Preconditions.checkState(timeTravelSpec.getKind() == Kind.TIME_AS_OF);
+    try {
+      scan = scan.asOfTime(timeTravelSpec.getAsOfMillis());
+    } catch (IllegalArgumentException e) {
+      throw new IllegalArgumentException(
+          "Cannot find a snapshot older than " + timeTravelSpec.toTimeString());
+    }
+    return scan;
   }
 
   /**
@@ -445,11 +590,11 @@ public class IcebergUtil {
   public static org.apache.iceberg.FileFormat fbFileFormatToIcebergFileFormat(
       byte fbFileFormat) throws ImpalaRuntimeException {
     switch (fbFileFormat){
-      case org.apache.impala.fb.FbFileFormat.PARQUET:
+      case org.apache.impala.fb.FbIcebergDataFileFormat.PARQUET:
           return org.apache.iceberg.FileFormat.PARQUET;
       default:
           throw new ImpalaRuntimeException(String.format("Unexpected file format: %s",
-              org.apache.impala.fb.FbFileFormat.name(fbFileFormat)));
+              org.apache.impala.fb.FbIcebergDataFileFormat.name(fbFileFormat)));
     }
   }
 
@@ -515,14 +660,20 @@ public class IcebergUtil {
 
     PartitionData data = new PartitionData(spec.getIcebergPartitionFieldsSize());
     String[] partitions = path.split("/", -1);
-    for (int i = 0; i < partitions.length; i += 1) {
+    int path_i = 0;
+    for (int i = 0; i < spec.getIcebergPartitionFieldsSize(); ++i) {
       IcebergPartitionField field = spec.getIcebergPartitionFields().get(i);
-      String[] parts = partitions[i].split("=", 2);
+      if (field.getTransformType() == TIcebergPartitionTransformType.VOID) {
+        continue;
+      }
+      String[] parts = partitions[path_i].split("=", 2);
       Preconditions.checkArgument(parts.length == 2 && parts[0] != null &&
-          field.getFieldName().equals(parts[0]), "Invalid partition: %s", partitions[i]);
+          field.getFieldName().equals(parts[0]), "Invalid partition: %s",
+          partitions[path_i]);
       TIcebergPartitionTransformType transformType = field.getTransformType();
       data.set(i, getPartitionValue(
           partitionType.fields().get(i).type(), transformType, parts[1]));
+      path_i += 1;
     }
     return data;
   }
@@ -548,6 +699,7 @@ public class IcebergUtil {
       case YEAR: return parseYearToTransformYear(stringValue);
       case MONTH: return parseMonthToTransformMonth(stringValue);
       case HOUR: return parseHourToTransformHour(stringValue);
+      case VOID: return null;
     }
     throw new ImpalaRuntimeException("Unexpected partition transform: " + transformType);
   }
@@ -595,4 +747,235 @@ public class IcebergUtil {
         ZoneOffset.UTC);
     return (int)ChronoUnit.HOURS.between(EPOCH, datetime);
   }
+
+  public static TCompressionCodec parseParquetCompressionCodec(
+      boolean onCreateTbl, Map<String, String> tblProperties, StringBuilder errMsg) {
+    String codecTblProp = tblProperties.get(IcebergTable.PARQUET_COMPRESSION_CODEC);
+    THdfsCompression codec = getIcebergParquetCompressionCodec(codecTblProp);
+    if (codec == null) {
+      errMsg.append("Invalid parquet compression codec for Iceberg table: ")
+          .append(codecTblProp);
+      return null;
+    }
+
+    TCompressionCodec compressionCodec = new TCompressionCodec();
+    if (tblProperties.containsKey(IcebergTable.PARQUET_COMPRESSION_CODEC)) {
+      compressionCodec.setCodec(codec);
+    }
+
+    if (onCreateTbl && codec != THdfsCompression.ZSTD) {
+      if (tblProperties.containsKey(IcebergTable.PARQUET_COMPRESSION_LEVEL)) {
+        errMsg.append("Parquet compression level cannot be set for codec ")
+          .append(codec)
+          .append(". Only ZSTD codec supports compression level table property.");
+        return null;
+      }
+    } else if (tblProperties.containsKey(IcebergTable.PARQUET_COMPRESSION_LEVEL)) {
+      String clevelTblProp = tblProperties.get(IcebergTable.PARQUET_COMPRESSION_LEVEL);
+      Integer clevel = Ints.tryParse(clevelTblProp);
+      if (clevel == null) {
+        errMsg.append("Invalid parquet compression level for Iceberg table: ")
+            .append(clevelTblProp);
+        return null;
+      } else if (clevel < IcebergTable.MIN_PARQUET_COMPRESSION_LEVEL ||
+          clevel > IcebergTable.MAX_PARQUET_COMPRESSION_LEVEL) {
+        errMsg.append("Parquet compression level for Iceberg table should fall in " +
+            "the range of [")
+            .append(String.valueOf(IcebergTable.MIN_PARQUET_COMPRESSION_LEVEL))
+            .append("..")
+            .append(String.valueOf(IcebergTable.MAX_PARQUET_COMPRESSION_LEVEL))
+            .append("]");
+        return null;
+      }
+      compressionCodec.setCompression_level(clevel);
+    }
+    return compressionCodec;
+  }
+
+  public static Long parseParquetRowGroupSize(Map<String, String> tblProperties,
+      StringBuilder errMsg) {
+    if (tblProperties.containsKey(IcebergTable.PARQUET_ROW_GROUP_SIZE)) {
+      String propVal = tblProperties.get(IcebergTable.PARQUET_ROW_GROUP_SIZE);
+      Long rowGroupSize = Longs.tryParse(propVal);
+      if (rowGroupSize == null) {
+        errMsg.append("Invalid parquet row group size for Iceberg table: ")
+            .append(propVal);
+        return null;
+      } else if (rowGroupSize < IcebergTable.MIN_PARQUET_ROW_GROUP_SIZE ||
+          rowGroupSize > IcebergTable.MAX_PARQUET_ROW_GROUP_SIZE) {
+        errMsg.append("Parquet row group size for Iceberg table should ")
+            .append("fall in the range of [")
+            .append(String.valueOf(IcebergTable.MIN_PARQUET_ROW_GROUP_SIZE))
+            .append("..")
+            .append(String.valueOf(IcebergTable.MAX_PARQUET_ROW_GROUP_SIZE))
+            .append("]");
+        return null;
+      }
+      return rowGroupSize;
+    }
+    return IcebergTable.UNSET_PARQUET_ROW_GROUP_SIZE;
+  }
+
+  public static Long parseParquetPageSize(Map<String, String> tblProperties,
+      String property, String descr, StringBuilder errMsg) {
+    if (tblProperties.containsKey(property)) {
+      String propVal = tblProperties.get(property);
+      Long pageSize = Longs.tryParse(propVal);
+      if (pageSize == null) {
+        errMsg.append("Invalid parquet ")
+            .append(descr)
+            .append(" for Iceberg table: ")
+            .append(propVal);
+        return null;
+      } else if (pageSize < IcebergTable.MIN_PARQUET_PAGE_SIZE ||
+          pageSize > IcebergTable.MAX_PARQUET_PAGE_SIZE) {
+        errMsg.append("Parquet ")
+            .append(descr)
+            .append(" for Iceberg table should fall in the range of [")
+            .append(String.valueOf(IcebergTable.MIN_PARQUET_PAGE_SIZE))
+            .append("..")
+            .append(String.valueOf(IcebergTable.MAX_PARQUET_PAGE_SIZE))
+            .append("]");
+        return null;
+      }
+      return pageSize;
+    }
+    return IcebergTable.UNSET_PARQUET_PAGE_SIZE;
+  }
+
+  public static Set<Long> currentAncestorIds(Table table) {
+    Set<Long> ret = new HashSet<>();
+    Snapshot snapshot = table.currentSnapshot();
+    while (snapshot != null) {
+      ret.add(snapshot.snapshotId());
+      Long parentId = snapshot.parentId();
+      if (parentId == null) break;
+      snapshot = table.snapshot(parentId);
+    }
+    return ret;
+  }
+
+  /**
+   * Extracts metadata from Iceberg data file object 'df'. Such metadata is the file
+   * format of the data file, also the partition information the data file belongs.
+   * It creates a flatbuffer so it can be passed between machines and processes without
+   * further de/serialization.
+   */
+  public static FbFileMetadata createIcebergMetadata(FeIcebergTable feTbl, DataFile df) {
+    FlatBufferBuilder fbb = new FlatBufferBuilder(1);
+    int iceOffset = createIcebergMetadata(feTbl, fbb, df);
+    fbb.finish(FbFileMetadata.createFbFileMetadata(fbb, iceOffset));
+    ByteBuffer bb = fbb.dataBuffer().slice();
+    ByteBuffer compressedBb = ByteBuffer.allocate(bb.capacity());
+    compressedBb.put(bb);
+    return FbFileMetadata.getRootAsFbFileMetadata((ByteBuffer)compressedBb.flip());
+  }
+
+  private static int createIcebergMetadata(FeIcebergTable feTbl, FlatBufferBuilder fbb,
+      DataFile df) {
+    int partKeysOffset = -1;
+    PartitionSpec spec = feTbl.getIcebergApiTable().specs().get(df.specId());
+    if (spec != null && !spec.fields().isEmpty()) {
+      partKeysOffset = createPartitionKeys(feTbl, fbb, spec, df);
+    }
+    FbIcebergMetadata.startFbIcebergMetadata(fbb);
+    byte fileFormat = -1;
+    if (df.format() == FileFormat.PARQUET) fileFormat = FbIcebergDataFileFormat.PARQUET;
+    else if (df.format() == FileFormat.ORC) fileFormat = FbIcebergDataFileFormat.ORC;
+    else if (df.format() == FileFormat.AVRO) fileFormat = FbIcebergDataFileFormat.AVRO;
+    if (fileFormat != -1) {
+      FbIcebergMetadata.addFileFormat(fbb, fileFormat);
+    }
+    if (partKeysOffset != -1) {
+      FbIcebergMetadata.addPartitionKeys(fbb, partKeysOffset);
+    }
+    return FbIcebergMetadata.endFbIcebergMetadata(fbb);
+  }
+
+  private static int createPartitionKeys(FeIcebergTable feTbl, FlatBufferBuilder fbb,
+      PartitionSpec spec, DataFile df) {
+    Preconditions.checkState(spec.fields().size() == df.partition().size());
+    int[] partitionKeyOffsets = new int[spec.fields().size()];
+    for (int i = 0; i < spec.fields().size(); ++i) {
+      partitionKeyOffsets[i] =
+          createPartitionTransformValue(feTbl, fbb, spec, df, i);
+    }
+    return FbIcebergMetadata.createPartitionKeysVector(fbb, partitionKeyOffsets);
+  }
+
+  private static int createPartitionTransformValue(FeIcebergTable feTbl,
+      FlatBufferBuilder fbb, PartitionSpec spec, DataFile df, int fieldIndex) {
+    PartitionField field = spec.fields().get(fieldIndex);
+    Pair<Byte, Integer> transform = getFbTransform(spec.schema(), field);
+    int valueOffset = -1;
+    if (transform.first != FbIcebergTransformType.VOID) {
+      Object partValue = df.partition().get(fieldIndex, Object.class);
+      String partValueString;
+      if (partValue != null) {
+        partValueString = partValue.toString();
+      } else {
+        partValueString = feTbl.getNullPartitionKeyValue();
+      }
+      valueOffset = fbb.createString(partValueString);
+    }
+    FbIcebergPartitionTransformValue.startFbIcebergPartitionTransformValue(fbb);
+    FbIcebergPartitionTransformValue.addTransformType(fbb, transform.first);
+    if (transform.second != null) {
+      FbIcebergPartitionTransformValue.addTransformParam(fbb, transform.second);
+    }
+    if (valueOffset != -1) {
+      FbIcebergPartitionTransformValue.addTransformValue(fbb, valueOffset);
+    }
+    FbIcebergPartitionTransformValue.addSourceId(fbb, field.sourceId());
+    return FbIcebergPartitionTransformValue.endFbIcebergPartitionTransformValue(fbb);
+  }
+
+  private static Pair<Byte, Integer> getFbTransform(Schema schema,
+      PartitionField field) {
+    return PartitionSpecVisitor.visit(
+      schema, field, new PartitionSpecVisitor<Pair<Byte, Integer>>() {
+      @Override
+      public Pair<Byte, Integer> identity(String sourceName, int sourceId) {
+        return new Pair<Byte, Integer>(FbIcebergTransformType.IDENTITY, null);
+      }
+
+      @Override
+      public Pair<Byte, Integer> bucket(String sourceName, int sourceId,
+          int numBuckets) {
+        return new Pair<Byte, Integer>(FbIcebergTransformType.BUCKET, numBuckets);
+      }
+
+      @Override
+      public Pair<Byte, Integer> truncate(String sourceName, int sourceId,
+          int width) {
+        return new Pair<Byte, Integer>(FbIcebergTransformType.TRUNCATE, width);
+      }
+
+      @Override
+      public Pair<Byte, Integer> year(String sourceName, int sourceId) {
+        return new Pair<Byte, Integer>(FbIcebergTransformType.YEAR, null);
+      }
+
+      @Override
+      public Pair<Byte, Integer> month(String sourceName, int sourceId) {
+        return new Pair<Byte, Integer>(FbIcebergTransformType.MONTH, null);
+      }
+
+      @Override
+      public Pair<Byte, Integer> day(String sourceName, int sourceId) {
+        return new Pair<Byte, Integer>(FbIcebergTransformType.DAY, null);
+      }
+
+      @Override
+      public Pair<Byte, Integer> hour(String sourceName, int sourceId) {
+        return new Pair<Byte, Integer>(FbIcebergTransformType.HOUR, null);
+      }
+
+      @Override
+      public Pair<Byte, Integer> alwaysNull(int fieldId, String sourceName,
+          int sourceId) {
+        return new Pair<Byte, Integer>(FbIcebergTransformType.VOID, null);
+      }
+    });
+  }
 }
