@@ -28,9 +28,6 @@ import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.hadoop.yarn.api.records.QueueACL;
 import org.apache.hadoop.yarn.conf.YarnConfiguration;
-import org.apache.thrift.TException;
-import org.apache.thrift.TSerializer;
-import org.apache.thrift.protocol.TBinaryProtocol;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -80,8 +77,6 @@ import com.google.common.collect.Lists;
 public class RequestPoolService {
   final static Logger LOG = LoggerFactory.getLogger(RequestPoolService.class);
 
-  private final static TBinaryProtocol.Factory protocolFactory_ =
-      new TBinaryProtocol.Factory();
   // Used to ensure start() has been called before any other methods can be used.
   private final AtomicBoolean running_;
 
@@ -152,6 +147,9 @@ public class RequestPoolService {
   // URL of the configuration file.
   private final URL confUrl_;
 
+  // Reference of single instance of RequestPoolService.
+  private static RequestPoolService single_instance_ = null;
+
   /**
    * Updates the configuration when the file changes. The file is confUrl_
    * and it will exist when this is created (or RequestPoolService will not start). If
@@ -169,6 +167,35 @@ public class RequestPoolService {
     }
   }
 
+  /**
+   * Static method to create singleton instance of RequestPoolService class.
+   * This API is called by backend code through JNI, or called by unit-test code.
+   */
+  public static RequestPoolService getInstance(
+      final String fsAllocationPath, final String sitePath, boolean isTest) {
+    // For frontend and backend tests, different request pools could be created with
+    // different configurations in one process so we have to allow multiple instances
+    // to be created for frontend and backend tests.
+    if (isTest) return (new RequestPoolService(fsAllocationPath, sitePath));
+
+    if (single_instance_ == null) {
+      single_instance_ = new RequestPoolService(fsAllocationPath, sitePath);
+    }
+    return single_instance_;
+  }
+
+  /**
+   * Static method to return singleton instance of RequestPoolService class.
+   * This API is called by frontend Java code. An instance should be already created
+   * by backend before this API is called except only default pool is used.
+   */
+  public static RequestPoolService getInstance() {
+    if (single_instance_ == null) {
+      LOG.info("Default pool only, scheduler allocation is not specified.");
+    }
+    return single_instance_;
+  }
+
   /**
    * Creates a RequestPoolService instance with a configuration containing the specified
    * fair-scheduler.xml and llama-site.xml.
@@ -176,7 +203,7 @@ public class RequestPoolService {
    * @param fsAllocationPath path to the fair scheduler allocation file.
    * @param sitePath path to the configuration file.
    */
-  RequestPoolService(final String fsAllocationPath, final String sitePath) {
+  private RequestPoolService(final String fsAllocationPath, final String sitePath) {
     Preconditions.checkNotNull(fsAllocationPath);
     running_ = new AtomicBoolean(false);
     allocationConf_ = new AtomicReference<>();
@@ -274,31 +301,12 @@ public class RequestPoolService {
    * Resolves a user and pool to the pool specified by the allocation placement policy
    * and checks if the user is authorized to submit requests.
    *
-   * @param thriftResolvePoolParams Serialized {@link TResolveRequestPoolParams}
-   * @return serialized {@link TResolveRequestPoolResult}
+   * @param resolvePoolParams {@link TResolveRequestPoolParams}
+   * @return {@link TResolveRequestPoolResult}
    */
-  @SuppressWarnings("unused") // called from C++
-  public byte[] resolveRequestPool(byte[] thriftResolvePoolParams)
-      throws ImpalaException {
-    TResolveRequestPoolParams resolvePoolParams = new TResolveRequestPoolParams();
-    JniUtil.deserializeThrift(protocolFactory_, resolvePoolParams,
-        thriftResolvePoolParams);
-    TResolveRequestPoolResult result = resolveRequestPool(resolvePoolParams);
-    if (LOG.isTraceEnabled()) {
-      LOG.trace("resolveRequestPool(pool={}, user={}): resolved_pool={}, has_access={}",
-          resolvePoolParams.getRequested_pool(), resolvePoolParams.getUser(),
-          result.resolved_pool, result.has_access);
-    }
-    try {
-      return new TSerializer(protocolFactory_).serialize(result);
-    } catch (TException e) {
-      throw new InternalException(e.getMessage());
-    }
-  }
-
-  @VisibleForTesting
-  TResolveRequestPoolResult resolveRequestPool(
+  public TResolveRequestPoolResult resolveRequestPool(
       TResolveRequestPoolParams resolvePoolParams) throws InternalException {
+    Preconditions.checkState(running_.get());
     String requestedPool = resolvePoolParams.getRequested_pool();
     String user = resolvePoolParams.getUser();
     TResolveRequestPoolResult result = new TResolveRequestPoolResult();
@@ -335,31 +343,22 @@ public class RequestPoolService {
       result.setHas_access(hasAccess(pool, user));
       result.setStatus(new TStatus(TErrorCode.OK, Lists.newArrayList()));
     }
+    if (LOG.isTraceEnabled()) {
+      LOG.trace("resolveRequestPool(pool={}, user={}): resolved_pool={}, has_access={}",
+          resolvePoolParams.getRequested_pool(), resolvePoolParams.getUser(),
+          result.resolved_pool, result.has_access);
+    }
     return result;
   }
 
   /**
    * Gets the pool configuration values for the specified pool.
    *
-   * @param thriftPoolConfigParams Serialized {@link TPoolConfigParams}
-   * @return serialized {@link TPoolConfig}
+   * @param pool name.
+   * @return {@link TPoolConfig}
    */
-  @SuppressWarnings("unused") // called from C++
-  public byte[] getPoolConfig(byte[] thriftPoolConfigParams) throws ImpalaException {
+  public TPoolConfig getPoolConfig(String pool) {
     Preconditions.checkState(running_.get());
-    TPoolConfigParams poolConfigParams = new TPoolConfigParams();
-    JniUtil.deserializeThrift(protocolFactory_, poolConfigParams,
-        thriftPoolConfigParams);
-    TPoolConfig result = getPoolConfig(poolConfigParams.getPool());
-    try {
-      return new TSerializer(protocolFactory_).serialize(result);
-    } catch (TException e) {
-      throw new InternalException(e.getMessage());
-    }
-  }
-
-  @VisibleForTesting
-  TPoolConfig getPoolConfig(String pool) {
     TPoolConfig result = new TPoolConfig();
     long maxMemoryMb = allocationConf_.get().getMaxResources(pool).getMemory();
     result.setMax_mem_resources(
