@@ -67,10 +67,8 @@ public class CardinalityTest extends PlannerTestBase {
     verifyCardinality(
         "SELECT id FROM functional.alltypes WHERE int_col = 1", 7300/10);
 
-    // Assume classic 0.1 selectivity for other operators
-    // IMPALA-7560 says this should be revised.
     verifyCardinality(
-        "SELECT id FROM functional.alltypes WHERE int_col != 1", 730);
+        "SELECT id FROM functional.alltypes WHERE int_col != 1", 6570);
 
     // IMPALA-7601 says the following should be revised.
     verifyCardinality(
@@ -103,6 +101,10 @@ public class CardinalityTest extends PlannerTestBase {
         "SELECT COUNT(*) FROM functional.alltypes GROUP BY id", 7300);
     verifyCardinality(
         "SELECT COUNT(*) FROM functional.alltypes GROUP BY bool_col", 2);
+
+    // Regression test for IMPALA-11301.
+    verifyCardinality(
+        "SELECT * FROM tpcds_parquet.date_dim WHERE d_current_day != 'a'", 36525);
   }
 
   /**
@@ -117,9 +119,11 @@ public class CardinalityTest extends PlannerTestBase {
     // f repeats for 5 rows, so NDV=7, 26/7 =~ 4
     verifyCardinality("SELECT null_int FROM functional.nullrows WHERE group_str = 'x'",
         4);
-    // null_str is all nulls, NDV = 1, selectivity = 1/1, cardinality = 26
+    // null_str is all nulls, NDV = 1, selectivity =0, cardinality = 0
+    // PlanNode#applySelectivity, If cardinality is 0, set cardinality to 1
+    // IMPALA-8647: don't round cardinality down to zero for safety.
     verifyCardinality(
-          "SELECT null_int FROM functional.nullrows WHERE null_str = 'x'", 26);
+          "SELECT null_int FROM functional.nullrows WHERE null_str = 'x'", 1);
   }
 
   @Test
@@ -366,11 +370,11 @@ public class CardinalityTest extends PlannerTestBase {
     // The estimated number of rows caps the output cardinality.
     verifyApproxCardinality(
         "select distinct id, int_col from functional_parquet.alltypes",
-        12760, true, ImmutableSet.of(),
+        12400, true, ImmutableSet.of(),
         pathToFirstAggregationNode, AggregationNode.class);
     verifyApproxCardinality(
         "select distinct id, int_col from functional_parquet.alltypes",
-        12760, true, ImmutableSet.of(),
+        12400, true, ImmutableSet.of(),
         pathToSecondAggregationNode, AggregationNode.class);
     // No column stats available and row estimation disabled - no estimate is possible.
     verifyCardinality(
@@ -396,6 +400,12 @@ public class CardinalityTest extends PlannerTestBase {
     verifyApproxCardinality("SELECT SUM(int_col) OVER() int_col "
         + "FROM functional_parquet.alltypestiny", 742, true,
         ImmutableSet.of(), path, AnalyticEvalNode.class);
+
+    // Regression test for IMPALA-11301. row_number() is (incorrectly) assumed to have
+    // NDV=1, which was leading to selectivity=0.0 in rn != 5. Will break if someone
+    // implements correct ndv estimates for analytic functions.
+    verifyCardinality("SELECT * FROM (SELECT *, row_number() OVER(order by id) "
+        + "as rn FROM functional.alltypestiny) v where rn != 5", 4);
   }
 
   @Test
@@ -892,18 +902,18 @@ public class CardinalityTest extends PlannerTestBase {
     verifyApproxMemoryEstimate("SELECT COUNT(*),MAX(B.BIGINT_COL) FROM " +
         "FUNCTIONAL.ALLTYPES A , FUNCTIONAL.ALLTYPESSMALL B, " +
         "FUNCTIONAL.NULLROWS C GROUP BY A.ID, B.TIMESTAMP_COL,C.BOOL_NULLS",
-        2190000*(4+16+1+8+8+16), true, true, ImmutableSet.of(),
+        2190000*(4+16+1+8+8+12), true, true, ImmutableSet.of(),
         pathToFirstAggregationNode, AggregationNode.class);
     verifyApproxMemoryEstimate("SELECT COUNT(*),MAX(B.BIGINT_COL) FROM " +
         "FUNCTIONAL.ALLTYPES A , FUNCTIONAL.ALLTYPESSMALL B, " +
         "FUNCTIONAL.NULLROWS C GROUP BY A.ID, B.TIMESTAMP_COL,C.BOOL_NULLS",
-        2190000*(4+16+1+8+8+16), true, false, ImmutableSet.of(),
+        2190000*(4+16+1+8+8+12), true, false, ImmutableSet.of(),
         pathToSecondAggregationNode, AggregationNode.class);
     // create a single node plan.
     verifyApproxMemoryEstimate("SELECT COUNT(*),MAX(B.BIGINT_COL) FROM " +
         "FUNCTIONAL.ALLTYPES A , FUNCTIONAL.ALLTYPESSMALL B, " +
         "FUNCTIONAL.NULLROWS C GROUP BY A.ID, B.TIMESTAMP_COL,C.BOOL_NULLS",
-        2190000*(4+16+1+8+8+16), false, false, ImmutableSet.of(),
+        2190000*(4+16+1+8+8+12), false, false, ImmutableSet.of(),
         pathToAggregationNode, AggregationNode.class);
 
     List<Integer> countFirstAggregationNode = Arrays.asList();
@@ -951,13 +961,13 @@ public class CardinalityTest extends PlannerTestBase {
     // FUNCTIONAL.ALLTYPES.DATE_STRING_COL's Ndv is 736 and avgRowSize is 8
     // Cardinality estimate of subquery is 5372800
     // BitUtil.roundUpToPowerOf2((long) Math.ceil(3 * Cardinality / 2)) is 8388608
-    // Size of Bucket is 16.
+    // Size of Bucket is 12.
     verifyApproxMemoryEstimate("SELECT A.ID FROM (SELECT A1.ID,B1.DATE_STRING_COL " +
         "IDB FROM FUNCTIONAL.ALLTYPES A1 , FUNCTIONAL.ALLTYPES B1 GROUP BY A1.ID," +
         "B1.DATE_STRING_COL) A JOIN (SELECT A1.ID,B1.DATE_STRING_COL IDB FROM " +
         "FUNCTIONAL.ALLTYPES A1,FUNCTIONAL.ALLTYPES B1 GROUP BY " +
         "A1.ID,B1.DATE_STRING_COL) B ON A.ID = B.ID AND A.IDB=B.IDB",
-        5372800*(4+8+4+8) + 8388608*16, true, true, ImmutableSet.of(),
+        5372800*(4+8+4+8) + 8388608*12, true, true, ImmutableSet.of(),
         pathToJoinNode, HashJoinNode.class);
     // create a single node plan.
     verifyApproxMemoryEstimate("SELECT A.ID FROM (SELECT A1.ID,B1.DATE_STRING_COL " +
@@ -965,27 +975,27 @@ public class CardinalityTest extends PlannerTestBase {
         "B1.DATE_STRING_COL) A JOIN (SELECT A1.ID,B1.DATE_STRING_COL IDB FROM " +
         "FUNCTIONAL.ALLTYPES A1,FUNCTIONAL.ALLTYPES B1 GROUP BY " +
         "A1.ID,B1.DATE_STRING_COL) B ON A.ID = B.ID AND A.IDB=B.IDB",
-        5372800*(4+8+4+8) + 8388608*16, false, false, ImmutableSet.of(),
+        5372800*(4+8+4+8) + 8388608*12, false, false, ImmutableSet.of(),
         pathToJoinNodeSg, HashJoinNode.class);
 
     // FUNCTIONAL.ALLTYPES.ID's Ndv is 7300 and avgRowSize is 4
     // FUNCTIONAL.ALLTYPES.DATE_STRING_COL's Ndv is 736 and avgRowSize is 8
     // Cardinality estimate of subquery is 53290000
     // BitUtil.roundUpToPowerOf2((long) Math.ceil(3 * Cardinality / 2)) is 134217728
-    // Size of Bucket is 16. Size of Duplicatenode is 24
+    // Size of Bucket is 12. Size of Duplicatenode is 16
     // Ndv estimate of subquery is 5372800
     verifyApproxMemoryEstimate("SELECT A.ID FROM (SELECT A1.ID,B1.DATE_STRING_COL " +
         "IDB FROM FUNCTIONAL.ALLTYPES A1 , FUNCTIONAL.ALLTYPES B1) A JOIN " +
         "(SELECT A1.ID,B1.DATE_STRING_COL IDB FROM FUNCTIONAL.ALLTYPES A1," +
         "FUNCTIONAL.ALLTYPES B1) B ON A.ID = B.ID AND A.IDB=B.IDB",
-        53290000L*(4L+8L+4L+8L)+134217728L*16L+(53290000L-5372800L)*24L, true, true,
+        53290000L*(4L+8L+4L+8L)+134217728L*12L+(53290000L-5372800L)*16L, true, true,
         ImmutableSet.of(), pathToJoinNode, HashJoinNode.class);
     // create a single node plan.
     verifyApproxMemoryEstimate("SELECT A.ID FROM (SELECT A1.ID,B1.DATE_STRING_COL " +
         "IDB FROM FUNCTIONAL.ALLTYPES A1 , FUNCTIONAL.ALLTYPES B1) A JOIN " +
         "(SELECT A1.ID,B1.DATE_STRING_COL IDB FROM FUNCTIONAL.ALLTYPES A1," +
         "FUNCTIONAL.ALLTYPES B1) B ON A.ID = B.ID AND A.IDB=B.IDB",
-        53290000L*(4L+8L+4L+8L)+134217728L*16L+(53290000L-5372800L)*24L, false, false,
+        53290000L*(4L+8L+4L+8L)+134217728L*12L+(53290000L-5372800L)*16L, false, false,
         ImmutableSet.of(), pathToJoinNodeSg, HashJoinNode.class);
   }
 
