@@ -32,10 +32,12 @@ import org.apache.impala.datagenerator.HBaseTestDataRegionAssignment;
 import org.apache.impala.service.Frontend.PlanCtx;
 import org.apache.impala.testutil.TestUtils;
 import org.apache.impala.testutil.TestUtils.IgnoreValueFilter;
-import org.apache.impala.thrift.TEnabledRuntimeFilterTypes;
+import org.apache.impala.thrift.TRuntimeFilterType;
 import org.apache.impala.thrift.TExecRequest;
 import org.apache.impala.thrift.TExplainLevel;
+import org.apache.impala.thrift.TExplainResult;
 import org.apache.impala.thrift.TJoinDistributionMode;
+import org.apache.impala.thrift.TKuduReplicaSelection;
 import org.apache.impala.thrift.TQueryCtx;
 import org.apache.impala.thrift.TQueryOptions;
 import org.apache.impala.thrift.TRuntimeFilterMode;
@@ -340,7 +342,9 @@ public class PlannerTest extends PlannerTestBase {
 
   @Test
   public void testNestedCollections() {
-    runPlannerTestFile("nested-collections");
+    TQueryOptions options = new TQueryOptions();
+    options.setMinmax_filter_sorted_columns(false);
+    runPlannerTestFile("nested-collections", options);
   }
 
   @Test
@@ -348,6 +352,14 @@ public class PlannerTest extends PlannerTestBase {
     runPlannerTestFile("complex-types-file-formats");
   }
 
+  @Test
+  public void testZippingUnnest() {
+    addTestDb("test_zipping_unnest_db", "For creating views for zipping unnest queries.");
+    addTestView("create view test_zipping_unnest_db.view_arrays as " +
+        "select id, arr1, arr2 from functional_parquet.complextypes_arrays");
+    runPlannerTestFile("zipping-unnest");
+  }
+
   @Test
   public void testJoins() {
     TQueryOptions options = defaultQueryOptions();
@@ -551,7 +563,8 @@ public class PlannerTest extends PlannerTestBase {
   @Test
   public void testTpchNested() {
     TQueryOptions options = new TQueryOptions();
-    runPlannerTestFile("tpch-nested", "tpch_nested_parquet",
+    options.setMinmax_filter_sorted_columns(false);
+    runPlannerTestFile("tpch-nested", "tpch_nested_parquet", options,
         ImmutableSet.of(PlannerTestOption.INCLUDE_RESOURCE_HEADER,
             PlannerTestOption.VALIDATE_RESOURCES,
             PlannerTestOption.VALIDATE_CARDINALITY));
@@ -608,17 +621,23 @@ public class PlannerTest extends PlannerTestBase {
     runPlannerTestFile("disable-runtime-overlap-filter", options);
 
     options.setMinmax_filter_threshold(1.0);
-    options.setEnabled_runtime_filter_types(TEnabledRuntimeFilterTypes.BLOOM);
+    options.unsetEnabled_runtime_filter_types();
+    options.addToEnabled_runtime_filter_types(TRuntimeFilterType.BLOOM);
     runPlannerTestFile("disable-runtime-overlap-filter", options);
   }
 
   @Test
   public void testRuntimeFilterQueryOptions() {
-    runPlannerTestFile("runtime-filter-query-options");
+    runPlannerTestFile("runtime-filter-query-options",
+        ImmutableSet.of(
+            PlannerTestOption.DO_NOT_VALIDATE_ROWCOUNT_ESTIMATION_FOR_PARTITIONS));
   }
 
   @Test
   public void testBloomFilterAssignment() {
+    TQueryOptions options = defaultQueryOptions();
+    options.setMinmax_filter_sorted_columns(false);
+    options.setMinmax_filter_partition_columns(false);
     runPlannerTestFile("bloom-filter-assignment",
         ImmutableSet.of(
             PlannerTestOption.DO_NOT_VALIDATE_ROWCOUNT_ESTIMATION_FOR_PARTITIONS));
@@ -654,7 +673,9 @@ public class PlannerTest extends PlannerTestBase {
   @Test
   public void testKudu() {
     TQueryOptions options = defaultQueryOptions();
-    options.setEnabled_runtime_filter_types(TEnabledRuntimeFilterTypes.ALL);
+    options.unsetEnabled_runtime_filter_types();
+    options.addToEnabled_runtime_filter_types(TRuntimeFilterType.BLOOM);
+    options.addToEnabled_runtime_filter_types(TRuntimeFilterType.MIN_MAX);
     addTestDb("kudu_planner_test", "Test DB for Kudu Planner.");
     addTestTable("CREATE EXTERNAL TABLE kudu_planner_test.no_stats STORED AS KUDU " +
         "TBLPROPERTIES ('kudu.table_name' = 'impala::functional_kudu.alltypes');");
@@ -669,7 +690,9 @@ public class PlannerTest extends PlannerTestBase {
   @Test
   public void testKuduUpdate() {
     TQueryOptions options = defaultQueryOptions();
-    options.setEnabled_runtime_filter_types(TEnabledRuntimeFilterTypes.ALL);
+    options.unsetEnabled_runtime_filter_types();
+    options.addToEnabled_runtime_filter_types(TRuntimeFilterType.BLOOM);
+    options.addToEnabled_runtime_filter_types(TRuntimeFilterType.MIN_MAX);
     runPlannerTestFile("kudu-update", options);
   }
 
@@ -685,10 +708,23 @@ public class PlannerTest extends PlannerTestBase {
     runPlannerTestFile("kudu-selectivity", options);
   }
 
+  @Test
+  public void testKuduReplicaSelection() {
+    TQueryOptions options = defaultQueryOptions();
+    options.setExplain_level(TExplainLevel.VERBOSE);
+    options.setKudu_replica_selection(TKuduReplicaSelection.LEADER_ONLY);
+    runPlannerTestFile("kudu-replica-selection-leader-only", options);
+
+    options.setKudu_replica_selection(TKuduReplicaSelection.CLOSEST_REPLICA);
+    runPlannerTestFile("kudu-replica-selection-closest-replica", options);
+  }
+
   @Test
   public void testKuduTpch() {
     TQueryOptions options = defaultQueryOptions();
-    options.setEnabled_runtime_filter_types(TEnabledRuntimeFilterTypes.ALL);
+    options.unsetEnabled_runtime_filter_types();
+    options.addToEnabled_runtime_filter_types(TRuntimeFilterType.BLOOM);
+    options.addToEnabled_runtime_filter_types(TRuntimeFilterType.MIN_MAX);
     runPlannerTestFile("tpch-kudu", options,
         ImmutableSet.of(PlannerTestOption.INCLUDE_RESOURCE_HEADER,
             PlannerTestOption.VALIDATE_RESOURCES));
@@ -756,6 +792,83 @@ public class PlannerTest extends PlannerTestBase {
     Assert.assertEquals(actualMtDop, expectedMtDop);
   }
 
+  @Test
+  public void testOnlyNeededStructFieldsMaterialized() throws ImpalaException {
+    // Tests that if a struct is selected in an inline view but only a subset of its
+    // fields are selected in the top level query, only the selected fields are
+    // materialised, not the whole struct.
+    // Also tests that selecting the whole struct or fields from an inline view or
+    // directly from the table give the same row size.
+
+    // For comlex types in the select list, we have to turn codegen off.
+    TQueryOptions queryOpts = defaultQueryOptions();
+    queryOpts.setDisable_codegen(true);
+
+    String queryWholeStruct =
+        "select outer_struct from functional_orc_def.complextypes_nested_structs";
+    int rowSizeWholeStruct = getRowSize(queryWholeStruct, queryOpts);
+
+    String queryWholeStructFromInlineView =
+        "with sub as (" +
+          "select id, outer_struct from functional_orc_def.complextypes_nested_structs)" +
+        "select sub.outer_struct from sub";
+    int rowSizeWholeStructFromInlineView = getRowSize(
+        queryWholeStructFromInlineView, queryOpts);
+
+    String queryOneField =
+        "select outer_struct.str from functional_orc_def.complextypes_nested_structs";
+    int rowSizeOneField = getRowSize(queryOneField, queryOpts);
+
+    String queryOneFieldFromInlineView =
+        "with sub as (" +
+          "select id, outer_struct from functional_orc_def.complextypes_nested_structs)" +
+        "select sub.outer_struct.str from sub";
+    int rowSizeOneFieldFromInlineView = getRowSize(
+        queryOneFieldFromInlineView, queryOpts);
+
+    String queryTwoFields =
+        "select outer_struct.str, outer_struct.inner_struct1 " +
+        "from functional_orc_def.complextypes_nested_structs";
+    int rowSizeTwoFields = getRowSize(queryTwoFields, queryOpts);
+
+    String queryTwoFieldsFromInlineView =
+        "with sub as (" +
+          "select id, outer_struct from functional_orc_def.complextypes_nested_structs)" +
+        "select sub.outer_struct.str, sub.outer_struct.inner_struct1 from sub";
+    int rowSizeTwoFieldsFromInlineView = getRowSize(
+        queryTwoFieldsFromInlineView, queryOpts);
+
+    Assert.assertEquals(rowSizeWholeStruct, rowSizeWholeStructFromInlineView);
+    Assert.assertEquals(rowSizeOneField, rowSizeOneFieldFromInlineView);
+    Assert.assertEquals(rowSizeTwoFields, rowSizeTwoFieldsFromInlineView);
+
+    Assert.assertTrue(rowSizeOneField < rowSizeTwoFields);
+    Assert.assertTrue(rowSizeTwoFields < rowSizeWholeStruct);
+  }
+
+  @Test
+  public void testStructFieldSlotSharedWithStruct() throws ImpalaException {
+    // Tests that in the case where a struct and one of its fields are both present in the
+    // select list, no extra slot is generated in the row for the struct field but the
+    // memory of the struct is reused, i.e. the row size is the same as when only the
+    // struct is queried.
+
+    // For comlex types in the select list, we have to turn codegen off.
+    TQueryOptions queryOpts = defaultQueryOptions();
+    queryOpts.setDisable_codegen(true);
+
+    String queryWithoutField =
+        "select id, outer_struct from functional_orc_def.complextypes_nested_structs";
+    int rowSizeWithoutField = getRowSize(queryWithoutField, queryOpts);
+
+    String queryWithField =
+        "select id, outer_struct, outer_struct.str " +
+        "from functional_orc_def.complextypes_nested_structs";
+    int rowSizeWithField = getRowSize(queryWithField, queryOpts);
+
+    Assert.assertEquals(rowSizeWithoutField, rowSizeWithField);
+  }
+
   @Test
   public void testResourceRequirements() {
     // Tests the resource requirement computation from the planner.
@@ -877,7 +990,8 @@ public class PlannerTest extends PlannerTestBase {
     TQueryOptions options = defaultQueryOptions();
     options.setExplain_level(TExplainLevel.EXTENDED);
     options.setDisable_hdfs_num_rows_estimate(false);
-    options.setEnabled_runtime_filter_types(TEnabledRuntimeFilterTypes.MIN_MAX);
+    options.unsetEnabled_runtime_filter_types();
+    options.addToEnabled_runtime_filter_types(TRuntimeFilterType.MIN_MAX);
     runPlannerTestFile("min-max-runtime-filters-hdfs-num-rows-est-enabled", options,
         ImmutableSet.of(
             PlannerTestOption.DO_NOT_VALIDATE_ROWCOUNT_ESTIMATION_FOR_PARTITIONS));
@@ -888,7 +1002,9 @@ public class PlannerTest extends PlannerTestBase {
     TQueryOptions options = defaultQueryOptions();
     options.setExplain_level(TExplainLevel.EXTENDED);
     options.setDisable_hdfs_num_rows_estimate(true);
-    options.setEnabled_runtime_filter_types(TEnabledRuntimeFilterTypes.MIN_MAX);
+    options.setMinmax_filter_partition_columns(false);
+    options.unsetEnabled_runtime_filter_types();
+    options.addToEnabled_runtime_filter_types(TRuntimeFilterType.MIN_MAX);
     runPlannerTestFile("min-max-runtime-filters", options);
   }
 
@@ -1132,7 +1248,9 @@ public class PlannerTest extends PlannerTestBase {
    */
   @Test
   public void testLimitPushdownPartitionedTopN() {
-    runPlannerTestFile("limit-pushdown-partitioned-top-n");
+    TQueryOptions options = defaultQueryOptions();
+    options.setDisable_hdfs_num_rows_estimate(true); // Needed to test IMPALA-11443.
+    runPlannerTestFile("limit-pushdown-partitioned-top-n", options);
   }
 
   /**
@@ -1164,4 +1282,17 @@ public class PlannerTest extends PlannerTestBase {
     runPlannerTestFile("tpcds-dist-method", "tpcds");
   }
 
+  @Test
+  public void testOrcStatsAgg() {
+    runPlannerTestFile("orc-stats-agg");
+  }
+
+  /**
+   * Test EXPLAIN_LEVEL=VERBOSE is displayed properly with MT_DOP>0
+   */
+  @Test
+  public void testExplainVerboseMtDop() {
+    runPlannerTestFile("explain-verbose-mt_dop", "tpcds_parquet",
+        ImmutableSet.of(PlannerTestOption.INCLUDE_RESOURCE_HEADER));
+  }
 }
