@@ -18,6 +18,7 @@
 package org.apache.impala.planner;
 
 import static org.junit.Assert.fail;
+import org.junit.Assert;
 
 import java.io.FileWriter;
 import java.io.IOException;
@@ -50,6 +51,7 @@ import org.apache.impala.thrift.ImpalaInternalServiceConstants;
 import org.apache.impala.thrift.QueryConstants;
 import org.apache.impala.thrift.TDescriptorTable;
 import org.apache.impala.thrift.TExecRequest;
+import org.apache.impala.thrift.TExecutorGroupSet;
 import org.apache.impala.thrift.TExplainLevel;
 import org.apache.impala.thrift.THBaseKeyRange;
 import org.apache.impala.thrift.THdfsFileSplit;
@@ -106,7 +108,11 @@ public class PlannerTestBase extends FrontendTestBase {
     TUpdateExecutorMembershipRequest updateReq = new TUpdateExecutorMembershipRequest();
     updateReq.setIp_addresses(Sets.newHashSet("127.0.0.1"));
     updateReq.setHostnames(Sets.newHashSet("localhost"));
-    updateReq.setNum_executors(3);
+    TExecutorGroupSet group_set = new TExecutorGroupSet();
+    group_set.curr_num_executors = 3;
+    group_set.expected_num_executors = 20; // default num_expected_executors startup flag
+    updateReq.setExec_group_sets(new ArrayList<TExecutorGroupSet>());
+    updateReq.getExec_group_sets().add(group_set);
     ExecutorMembershipSnapshot.update(updateReq);
 
     kuduClient_ = new KuduClient.KuduClientBuilder("127.0.0.1:7051").build();
@@ -798,6 +804,30 @@ public class PlannerTestBase extends FrontendTestBase {
     return builder.toString();
   }
 
+  protected int getRowSize(String query, TQueryOptions queryOptions) {
+    TQueryCtx queryCtx = TestUtils.createQueryContext(Catalog.DEFAULT_DB,
+        System.getProperty("user.name"));
+    queryCtx.client_request.setStmt(query);
+    PlanCtx planCtx = new PlanCtx(queryCtx);
+    queryCtx.client_request.query_options = queryOptions;
+
+    try {
+      TExecRequest execRequest = frontend_.createExecRequest(planCtx);
+    } catch (ImpalaException e) {
+      Assert.fail("Failed to create exec request for '" + query + "': " + e.getMessage());
+    }
+
+    String explainStr = planCtx.getExplainString();
+
+    Pattern rowSizePattern = Pattern.compile("row-size=([0-9]*)B");
+    Matcher m = rowSizePattern.matcher(explainStr);
+    boolean matchFound = m.find();
+    Assert.assertTrue("Row size not found in plan.", matchFound);
+    String rowSizeStr = m.group(1);
+    return Integer.valueOf(rowSizeStr);
+  }
+
+
   /**
    * Assorted binary options that alter the behaviour of planner tests, generally
    * enabling additional more-detailed checks.
