@@ -61,7 +61,10 @@ CONFIG.update({
   # The default value of this configuration is 1h which will block
   # the test for long time. Overriding this configuration helps with
   # the runtime of the test.
-  'hive.repl.bootstrap.dump.open.txn.timeout': '120s'
+  'hive.repl.bootstrap.dump.open.txn.timeout': '120s',
+
+  # allow both hs2 and hs2-http protocols
+  'hive.server2.transport.mode': 'all'
 })
 
 if variant == 'changed_external_dir':
@@ -75,6 +78,12 @@ elif variant == 'ranger_auth':
     'hive.metastore.pre.event.listeners':
         'org.apache.hadoop.hive.ql.security.authorization.plugin.metastore.HiveMetaStoreAuthorizer',
   })
+elif variant == 'events_cleanup':
+  # HMS configs needed for regression test for IMPALA-11028
+  CONFIG.update({
+    'hive.metastore.event.db.listener.timetolive': '60s',
+    'hive.metastore.event.db.listener.clean.interval': '10s'
+  })
 
 # HBase-related configs.
 # Impala processes need to connect to zookeeper on INTERNAL_LISTEN_HOST for HBase.
@@ -100,7 +109,7 @@ if kerberize:
   #   hive.metastore.kerberos.keytab.file
   #   hive.metastore.kerberos.principal
 
-# Enable Tez and ACID for Hive 3
+# Enable Tez, ACID and proleptic Gregorian calendar DATE types for Hive 3
 if hive_major_version >= 3:
   CONFIG.update({
    'hive.tez.container.size': '512',
@@ -146,7 +155,17 @@ if hive_major_version >= 3:
 
    # Due to HIVE-23102 Hive will wait for at least this amount of time for compactions
    # (the default value is 5 mins which is way too long). Setting it to 2 seconds.
-   'hive.compactor.wait.timeout': '2000'
+   'hive.compactor.wait.timeout': '2000',
+
+   # Since HIVE-22589, Hive uses Julian Calendar for writing dates before 1582-10-15,
+   # whereas Impala uses proleptic Gregorian Calendar. This affects the results Impala
+   # gets when querying tables written by Hive.
+   'hive.avro.proleptic.gregorian': 'true',
+   'hive.avro.proleptic.gregorian.default': 'true',
+   'hive.parquet.date.proleptic.gregorian': 'true',
+   'hive.parquet.date.proleptic.gregorian.default': 'true',
+   'orc.proleptic.gregorian': 'true',
+   'orc.proleptic.gregorian.default': 'true'
   })
 else:
   CONFIG.update({
@@ -167,6 +186,15 @@ CONFIG.update({
  'hive.metastore.dml.events': 'true',
 })
 
+# Add Iceberg catalog configurations.
+CONFIG.update({
+  # Hive catalog:
+  'iceberg.catalog.ice_hive_cat.type': 'hive',
+  # Hadoop catalog:
+  'iceberg.catalog.ice_hadoop_cat.type': 'hadoop',
+  'iceberg.catalog.ice_hadoop_cat.warehouse': '${WAREHOUSE_LOCATION_PREFIX}/test-warehouse/ice_hadoop_cat',
+})
+
 if variant == 'without_hms_config':
   CONFIG.clear()
 
