@@ -25,15 +25,18 @@ import sys
 import warnings
 import base64
 import datetime
+from collections import namedtuple
 
 from six.moves import urllib, http_client
 
 from thrift.transport.TTransport import TTransportBase
 from shell_exceptions import HttpError
-from cookie_util import get_first_matching_cookie, get_cookie_expiry
+from cookie_util import get_all_matching_cookies, get_cookie_expiry
 
 import six
 
+# Declare namedtuple for Cookie with named fields - cookie and expiry_time
+Cookie = namedtuple('Cookie', ['cookie', 'expiry_time'])
 
 # This was taken from THttpClient.py in Thrift to allow making changes Impala needs.
 # The current changes that have been applied:
@@ -49,20 +52,21 @@ class ImpalaHttpClient(TTransportBase):
   MIN_REQUEST_SIZE_FOR_EXPECT = 1024
 
   def __init__(self, uri_or_host, port=None, path=None, cafile=None, cert_file=None,
-      key_file=None, ssl_context=None, auth_cookie_names=None):
+      key_file=None, ssl_context=None, http_cookie_names=None, socket_timeout_s=None):
     """ImpalaHttpClient supports two different types of construction:
 
     ImpalaHttpClient(host, port, path) - deprecated
     ImpalaHttpClient(uri, [port=<n>, path=<s>, cafile=<filename>, cert_file=<filename>,
-        key_file=<filename>, ssl_context=<context>], auth_cookie_names=<cookienamelist>])
+        key_file=<filename>, ssl_context=<context>], http_cookie_names=<cookienamelist>])
 
     Only the second supports https.  To properly authenticate against the server,
     provide the client's identity by specifying cert_file and key_file.  To properly
     authenticate the server, specify either cafile or ssl_context with a CA defined.
     NOTE: if both cafile and ssl_context are defined, ssl_context will override cafile.
-    auth_cookie_names is used to specify the list of possible cookie names used for
-    cookie-based authentication. If there's only one name in the cookie name list, a str
-    value can be specified instead of the list.
+    http_cookie_names is used to specify a comma-separated list of possible cookie names
+    used for cookie-based authentication or session management. If a cookie with one of
+    these names is returned in an http response by the server or an intermediate proxy
+    then it will be included in each subsequent request for the same connection.
     """
     if port is not None:
       warnings.warn(
@@ -106,13 +110,19 @@ class ImpalaHttpClient(TTransportBase):
       self.proxy_auth = self.basic_proxy_auth_header(parsed)
     else:
       self.realhost = self.realport = self.proxy_auth = None
-    self.__auth_cookie_names = auth_cookie_names
-    self.__auth_cookie = None
-    self.__auth_cookie_expiry = None
+    if (not http_cookie_names) or (str(http_cookie_names).strip() == ""):
+      self.__http_cookie_dict = None
+    else:
+      # Build a dictionary that maps cookie name to namedtuple.
+      cookie_names = http_cookie_names.split(',')
+      self.__http_cookie_dict = \
+        {cn: Cookie(cookie=None, expiry_time=None) for cn in cookie_names}
+    # Set __are_matching_cookies_found as True if matching cookies are found in response.
+    self.__are_matching_cookies_found = False
     self.__wbuf = BytesIO()
     self.__http = None
     self.__http_response = None
-    self.__timeout = None
+    self.__timeout = socket_timeout_s
     self.__custom_headers = None
 
   @staticmethod
@@ -158,26 +168,43 @@ class ImpalaHttpClient(TTransportBase):
   def setCustomHeaders(self, headers):
     self.__custom_headers = headers
 
-  def updateAuthCookie(self, headers):
-    if self.__auth_cookie_names:
-      c = get_first_matching_cookie(self.__auth_cookie_names, self.path, headers)
-      if c:
-        self.__auth_cookie = c
-        self.__auth_cookie_expiry = get_cookie_expiry(c)
+  # Extract cookies from response and save those cookies for which the cookie names
+  # are in the cookie name list specified in the __init__().
+  def extractHttpCookiesFromResponse(self, headers):
+    if self.__http_cookie_dict:
+      matching_cookies = \
+          get_all_matching_cookies(self.__http_cookie_dict.keys(), self.path, headers)
+      if matching_cookies:
+        self.__are_matching_cookies_found = True
+        for c in matching_cookies:
+          self.__http_cookie_dict[c.key] = Cookie(c, get_cookie_expiry(c))
 
-  def getAuthCookie(self):
-    if self.__auth_cookie and self.__auth_cookie_expiry and \
-        self.__auth_cookie_expiry <= datetime.datetime.now():
-      self.__auth_cookie = None
-      self.__auth_cookie_expiry = None
-    return self.__auth_cookie
+  # Return the value as a cookie list for Cookie header. It's a list of name-value
+  # pairs in the form of <cookie-name>=<cookie-value>. Pairs in the list are separated by
+  # a semicolon and a space ('; ').
+  def getHttpCookieHeaderForRequest(self):
+    if (self.__http_cookie_dict is None) or not self.__are_matching_cookies_found:
+      return None
+    cookie_headers = []
+    for cn, c_tuple in self.__http_cookie_dict.items():
+      if c_tuple.cookie:
+        if c_tuple.expiry_time and c_tuple.expiry_time <= datetime.datetime.now():
+          self.__http_cookie_dict[cn] = Cookie(cookie=None, expiry_time=None)
+        else:
+          cookie_header = c_tuple.cookie.output(attrs=['value'], header='').strip()
+          cookie_headers.append(cookie_header)
+    if not cookie_headers:
+      self.__are_matching_cookies_found = False
+      return None
+    else:
+      return '; '.join(cookie_headers)
 
-  def addAuthCookieToRequestHeaders(self):
-    auth_cookie = self.getAuthCookie()
-    if auth_cookie:
-      cookie_header = auth_cookie.output(attrs=['value'], header='').strip()
-      if cookie_header:
-        self.__http.putheader('Cookie', cookie_header)
+  # Add HTTP cookie headers based on the saved cookies.
+  def addHttpCookiesToRequestHeaders(self):
+    if self.__http_cookie_dict:
+      cookie_headers = self.getHttpCookieHeaderForRequest()
+      if cookie_headers:
+        self.__http.putheader('Cookie', cookie_headers)
 
   def read(self, sz):
     return self.__http_response.read(sz)
@@ -228,7 +255,7 @@ class ImpalaHttpClient(TTransportBase):
       for key, val in six.iteritems(self.__custom_headers):
         self.__http.putheader(key, val)
 
-    self.addAuthCookieToRequestHeaders()
+    self.addHttpCookiesToRequestHeaders()
     self.__http.endheaders()
 
     # Write payload
@@ -239,10 +266,10 @@ class ImpalaHttpClient(TTransportBase):
     self.code = self.__http_response.status
     self.message = self.__http_response.reason
     self.headers = self.__http_response.msg
-    self.updateAuthCookie(self.headers)
+    self.extractHttpCookiesFromResponse(self.headers)
 
     if self.code >= 300:
       # Report any http response code that is not 1XX (informational response) or
       # 2XX (successful).
-      body = self.readBody()
+      body = self.readBody().decode('utf-8')
       raise HttpError(self.code, self.message, body, self.headers)
