@@ -40,7 +40,7 @@ from Status.ttypes import TStatus
 from TCLIService.TCLIService import (TExecuteStatementReq, TOpenSessionReq,
     TCloseSessionReq, TProtocolVersion, TStatusCode, TGetOperationStatusReq,
     TOperationState, TFetchResultsReq, TFetchOrientation, TGetLogReq,
-    TGetResultSetMetadataReq, TTypeId, TCancelOperationReq)
+    TGetResultSetMetadataReq, TTypeId, TCancelOperationReq, TCloseOperationReq)
 from ImpalaHttpClient import ImpalaHttpClient
 from thrift.protocol import TBinaryProtocol
 from thrift_sasl import TSaslClientTransport
@@ -132,7 +132,7 @@ class ImpalaClient(object):
                kerberos_service_name="impala", use_ssl=False, ca_cert=None, user=None,
                ldap_password=None, use_ldap=False, client_connect_timeout_ms=60000,
                verbose=True, use_http_base_transport=False, http_path=None,
-               auth_cookie_names=None):
+               http_cookie_names=None, http_socket_timeout_s=None):
     self.connected = False
     self.impalad_host = impalad[0]
     self.impalad_port = int(impalad[1])
@@ -146,12 +146,13 @@ class ImpalaClient(object):
     self.user, self.ldap_password = user, ldap_password
     self.use_ldap = use_ldap
     self.client_connect_timeout_ms = int(client_connect_timeout_ms)
+    self.http_socket_timeout_s = http_socket_timeout_s
     self.default_query_options = {}
     self.query_option_levels = {}
     self.fetch_size = fetch_size
     self.use_http_base_transport = use_http_base_transport
     self.http_path = http_path
-    self.auth_cookie_names = auth_cookie_names
+    self.http_cookie_names = http_cookie_names
     # This is set from ImpalaShell's signal handler when a query is cancelled
     # from command line via CTRL+C. It is used to suppress error messages of
     # query cancellation.
@@ -380,6 +381,13 @@ class ImpalaClient(object):
       print("Warning: --connect_timeout_ms is currently ignored with HTTP transport.",
             file=sys.stderr)
 
+    # Notes on http socket timeout:
+    # https://docs.python.org/3/library/socket.html#socket-timeouts
+    # Having a default timeout of 'None' (blocking mode) could result in hang like
+    # symptoms in case of a problematic remote endpoint. It's better to have a finite
+    # timeout so that in case of any connection errors, the client retries have a better
+    # chance of succeeding.
+
     # HTTP server implemententations do not support SPNEGO yet.
     # TODO: when we add support for Kerberos+HTTP, we need to re-enable the automatic
     # kerberos retry logic in impala_shell.py that was disabled for HTTP because of
@@ -401,10 +409,12 @@ class ImpalaClient(object):
         ssl_ctx.verify_mode = ssl.CERT_NONE
       url = "https://{0}/{1}".format(host_and_port, self.http_path)
       transport = ImpalaHttpClient(url, ssl_context=ssl_ctx,
-                                   auth_cookie_names=self.auth_cookie_names)
+                                   http_cookie_names=self.http_cookie_names,
+                                   socket_timeout_s=self.http_socket_timeout_s)
     else:
       url = "http://{0}/{1}".format(host_and_port, self.http_path)
-      transport = ImpalaHttpClient(url, auth_cookie_names=self.auth_cookie_names)
+      transport = ImpalaHttpClient(url, http_cookie_names=self.http_cookie_names,
+                                   socket_timeout_s=self.http_socket_timeout_s)
 
     if self.use_ldap:
       # Set the BASIC auth header
@@ -671,6 +681,28 @@ class ImpalaHS2Client(ImpalaClient):
     # TODO: ensure it's closed if needed
     self.session_handle = resp.sessionHandle
 
+    self._populate_query_options()
+
+  def close_connection(self):
+    if self.session_handle is not None:
+      # Attempt to close session explicitly. Do not fail if there is an error
+      # doing so. We still need to close the transport and we can rely on the
+      # server to clean up the session.
+      try:
+        req = TCloseSessionReq(self.session_handle)
+
+        def CloseSession():
+          return self.imp_service.CloseSession(req)
+        # CloseSession rpcs don't need retries since we catch all exceptions and close
+        # transport.
+        resp = self._do_hs2_rpc(CloseSession)
+        self._check_hs2_rpc_status(resp.status)
+      except Exception as e:
+        print("Warning: close session RPC failed: {0}, {1}".format(str(e), type(e)))
+      self.session_handle = None
+    self._close_transport()
+
+  def _populate_query_options(self):
     # List all of the query options and their levels.
     # Retrying "set all" should be idempotent
     num_tries = 1
@@ -709,28 +741,10 @@ class ImpalaHS2Client(ImpalaClient):
       finally:
         if set_all_handle is not None:
           self.close_query(set_all_handle)
+
       time.sleep(self._get_sleep_interval_for_retries(num_tries))
       num_tries += 1
 
-  def close_connection(self):
-    if self.session_handle is not None:
-      # Attempt to close session explicitly. Do not fail if there is an error
-      # doing so. We still need to close the transport and we can rely on the
-      # server to clean up the session.
-      try:
-        req = TCloseSessionReq(self.session_handle)
-
-        def CloseSession():
-          return self.imp_service.CloseSession(req)
-        # CloseSession rpcs don't need retries since we catch all exceptions and close
-        # transport.
-        resp = self._do_hs2_rpc(CloseSession)
-        self._check_hs2_rpc_status(resp.status)
-      except Exception as e:
-        print("Warning: close session RPC failed: {0}, {1}".format(str(e), type(e)))
-      self.session_handle = None
-    self._close_transport()
-
   def _ping_impala_service(self):
     req = TPingImpalaHS2ServiceReq(self.session_handle)
 
@@ -826,9 +840,12 @@ class ImpalaHS2Client(ImpalaClient):
       # for the display code. This is somewhat inefficient, but performance is comparable
       # to the old Beeswax code.
       yield self._transpose(col_value_converters, resp.results.columns)
-      if not resp.hasMoreRows:
+      if not self._hasMoreRows(resp, col_value_converters):
         return
 
+  def _hasMoreRows(self, resp, col_value_converters):
+    return resp.hasMoreRows
+
   def _transpose(self, col_value_converters, columns):
     """Transpose the columns from a TFetchResultsResp into the row format returned
     by fetch() with all the values converted into their string representations for
@@ -845,11 +862,19 @@ class ImpalaHS2Client(ImpalaClient):
       # Skip stringification if not needed. This makes large extracts of tpch.orders
       # ~8% faster according to benchmarks.
       if stringifier is None:
-        for row_idx, row in enumerate(rows):
-          row[col_idx] = 'NULL' if is_null[row_idx] else tcol.values[row_idx]
+        bitset_len = min(len(is_null), len(rows))
+        for current_row in xrange(bitset_len):
+          rows[current_row][col_idx] = 'NULL' if is_null[current_row] \
+              else tcol.values[current_row]
+        for current_row in xrange(bitset_len, len(rows)):
+          rows[current_row][col_idx] = tcol.values[current_row]
       else:
-        for row_idx, row in enumerate(rows):
-          row[col_idx] = 'NULL' if is_null[row_idx] else stringifier(tcol.values[row_idx])
+        bitset_len = min(len(is_null), len(rows))
+        for current_row in xrange(bitset_len):
+          rows[current_row][col_idx] = 'NULL' if is_null[current_row] \
+              else stringifier(tcol.values[current_row])
+        for current_row in xrange(bitset_len, len(rows)):
+          rows[current_row][col_idx] = stringifier(tcol.values[current_row])
     return rows
 
   def close_dml(self, last_query_handle):
@@ -1070,6 +1095,48 @@ class RpcStatus:
   ERROR = 1
 
 
+class StrictHS2Client(ImpalaHS2Client):
+  """HS2 client. Uses the HS2 protocol without Impala-specific extensions.
+  This can be used to connect with HiveServer2 directly."""
+  def __init__(self, *args, **kwargs):
+    super(StrictHS2Client, self).__init__(*args, **kwargs)
+
+  def close_dml(self, last_query_handle):
+    self.close_query(last_query_handle)
+    return (None, None)
+
+  def close_query(self, last_query_handle):
+    # Set a member in the handle to make sure that it is idempotent
+    if last_query_handle.is_closed:
+      return True
+    req = TCloseOperationReq(last_query_handle)
+
+    def CloseOperation():
+      return self.imp_service.CloseOperation(req)
+    resp = self._do_hs2_rpc(CloseOperation, retry_on_error=False)
+    last_query_handle.is_closed = True
+    return self._is_hs2_nonerror_status(resp.status.statusCode)
+
+  def _ping_impala_service(self):
+    return ("N/A", "N/A")
+
+  def get_warning_log(self, last_query_handle):
+    return ""
+
+  def get_error_log(self, last_query_handle):
+    return ""
+
+  def get_runtime_profile(self, last_query_handle):
+    return None, None
+
+  def _populate_query_options(self):
+    return
+
+  def _hasMoreRows(self, resp, col_value_converters):
+    tcol = col_value_converters[0][0](resp.results.columns[0])
+    return len(tcol.values)
+
+
 class ImpalaBeeswaxClient(ImpalaClient):
   """Legacy Beeswax client. Uses the Beeswax protocol plus Impala-specific extensions.
   TODO: remove once we've phased out beeswax."""
