@@ -39,7 +39,8 @@ import textwrap
 import time
 import traceback
 
-from impala_client import ImpalaHS2Client, ImpalaBeeswaxClient, QueryOptionLevels
+from impala_client import ImpalaHS2Client, StrictHS2Client, \
+    ImpalaBeeswaxClient, QueryOptionLevels
 from impala_shell_config_defaults import impala_shell_defaults
 from option_parser import get_option_parser, get_config_from_file
 from shell_output import (DelimitedOutputFormatter, OutputStream, PrettyOutputFormatter,
@@ -66,7 +67,8 @@ except Exception:
 DEFAULT_BEESWAX_PORT = 21000
 DEFAULT_HS2_PORT = 21050
 DEFAULT_HS2_HTTP_PORT = 28000
-
+DEFAULT_STRICT_HS2_PORT = 11050
+DEFAULT_STRICT_HS2_HTTP_PORT = 10001
 
 def strip_comments(sql):
   """sqlparse default implementation of strip comments has a bad performance when parsing
@@ -183,10 +185,20 @@ class ImpalaShell(cmd.Cmd, object):
     self.use_ssl = options.ssl
     self.ca_cert = options.ca_cert
     self.user = options.user
-    self.ldap_password = options.ldap_password
     self.ldap_password_cmd = options.ldap_password_cmd
-    self.use_ldap = options.use_ldap
+    self.strict_hs2_protocol = options.strict_hs2_protocol
+    self.ldap_password = options.ldap_password
+    # When running tests in strict mode, the server uses the ldap
+    # protocol but can allow any password.
+    if options.use_ldap_test_password:
+      self.ldap_password = 'password'
+    self.use_ldap = options.use_ldap or \
+        (self.strict_hs2_protocol and not self.use_kerberos)
     self.client_connect_timeout_ms = options.client_connect_timeout_ms
+    self.http_socket_timeout_s = None
+    if (options.http_socket_timeout_s != 'None' and
+          options.http_socket_timeout_s is not None):
+        self.http_socket_timeout_s = float(options.http_socket_timeout_s)
     self.verbose = options.verbose
     self.prompt = ImpalaShell.DISCONNECTED_PROMPT
     self.server_version = ImpalaShell.UNKNOWN_SERVER_VERSION
@@ -223,13 +235,27 @@ class ImpalaShell(cmd.Cmd, object):
     # Tracks query handle of the last query executed. Used by the 'profile' command.
     self.last_query_handle = None;
 
-    self.live_summary = options.live_summary
-    self.live_progress = options.live_progress
+    # live_summary and live_progress are turned off in strict_hs2_protocol mode
+    if options.strict_hs2_protocol:
+      if options.live_summary:
+        warning = "WARNING: Unable to track live summary with strict_hs2_protocol"
+        print(warning, file=sys.stderr)
+      if options.live_progress:
+        warning = "WARNING: Unable to track live progress with strict_hs2_protocol"
+        print(warning, file=sys.stderr)
+
+      # do not allow live_progress or live_summary to be changed.
+      self.VALID_SHELL_OPTIONS['LIVE_PROGRESS'] = (lambda x: x in (), "live_progress")
+      self.VALID_SHELL_OPTIONS['LIVE_SUMMARY'] = (lambda x: x in (), "live_summary")
+
+    self.live_summary = options.live_summary and not options.strict_hs2_protocol
+    self.live_progress = options.live_progress and not options.strict_hs2_protocol
 
     self.ignore_query_failure = options.ignore_query_failure
 
     self.http_path = options.http_path
     self.fetch_size = options.fetch_size
+    self.http_cookie_names = options.http_cookie_names
 
     # Due to a readline bug in centos/rhel7, importing it causes control characters to be
     # printed. This breaks any scripting against the shell in non-interactive mode. Since
@@ -561,20 +587,37 @@ class ImpalaShell(cmd.Cmd, object):
 
   def _new_impala_client(self):
     protocol = options.protocol.lower()
+    if options.strict_hs2_protocol:
+      assert protocol == 'hs2' or protocol == 'hs2-http'
+      if protocol == 'hs2':
+        return StrictHS2Client(self.impalad, self.fetch_size, self.kerberos_host_fqdn,
+                          self.use_kerberos, self.kerberos_service_name, self.use_ssl,
+                          self.ca_cert, self.user, self.ldap_password, True,
+                          self.client_connect_timeout_ms, self.verbose,
+                          use_http_base_transport=False, http_path=self.http_path,
+                          http_cookie_names=None)
+      elif protocol == 'hs2-http':
+        return StrictHS2Client(self.impalad, self.fetch_size, self.kerberos_host_fqdn,
+                          self.use_kerberos, self.kerberos_service_name, self.use_ssl,
+                          self.ca_cert, self.user, self.ldap_password, self.use_ldap,
+                          self.client_connect_timeout_ms, self.verbose,
+                          use_http_base_transport=True, http_path=self.http_path,
+                          http_cookie_names=self.http_cookie_names)
     if protocol == 'hs2':
       return ImpalaHS2Client(self.impalad, self.fetch_size, self.kerberos_host_fqdn,
                           self.use_kerberos, self.kerberos_service_name, self.use_ssl,
                           self.ca_cert, self.user, self.ldap_password, self.use_ldap,
                           self.client_connect_timeout_ms, self.verbose,
                           use_http_base_transport=False, http_path=self.http_path,
-                          auth_cookie_names=None)
+                          http_cookie_names=None)
     elif protocol == 'hs2-http':
       return ImpalaHS2Client(self.impalad, self.fetch_size, self.kerberos_host_fqdn,
                           self.use_kerberos, self.kerberos_service_name, self.use_ssl,
                           self.ca_cert, self.user, self.ldap_password, self.use_ldap,
                           self.client_connect_timeout_ms, self.verbose,
                           use_http_base_transport=True, http_path=self.http_path,
-                          auth_cookie_names='impala.auth')
+                          http_cookie_names=self.http_cookie_names,
+                          http_socket_timeout_s=self.http_socket_timeout_s)
     elif protocol == 'beeswax':
       return ImpalaBeeswaxClient(self.impalad, self.fetch_size, self.kerberos_host_fqdn,
                           self.use_kerberos, self.kerberos_service_name, self.use_ssl,
@@ -884,7 +927,16 @@ class ImpalaShell(cmd.Cmd, object):
             "<hostname[:port]>", file=sys.stderr)
       return CmdStatus.ERROR
     elif len(host_port) == 1:
-      if protocol == 'hs2':
+      if options.strict_hs2_protocol:
+        if protocol == 'hs2':
+          port = str(DEFAULT_STRICT_HS2_PORT)
+        elif protocol == 'hs2-http':
+          port = str(DEFAULT_STRICT_HS2_HTTP_PORT)
+        else:
+          print("Invalid protocol specified for 'strict_hs2_protocol' option: %s"
+              % protocol, file=sys.stderr)
+          raise FatalShellException()
+      elif protocol == 'hs2':
         port = str(DEFAULT_HS2_PORT)
       elif protocol == 'hs2-http':
         port = str(DEFAULT_HS2_HTTP_PORT)
@@ -1194,7 +1246,7 @@ class ImpalaShell(cmd.Cmd, object):
           return
 
         if self.live_progress and progress.total_scan_ranges > 0:
-          val = ((summary.progress.num_completed_scan_ranges * 100) /
+          val = ((summary.progress.num_completed_scan_ranges * 100) //
                  summary.progress.total_scan_ranges)
           fragment_text = "[%s%s] %s%%\n" % ("#" * val, " " * (100 - val), val)
           data += fragment_text
@@ -1276,6 +1328,10 @@ class ImpalaShell(cmd.Cmd, object):
         # retrieve the error log
         warning_log = self.imp_client.get_warning_log(self.last_query_handle)
 
+        # Flush the row output. This is important so that the row output will not
+        # come after the "Fetch X row(s)" message.
+        self.output_stream.flush()
+
       end_time = time.time()
 
       if warning_log:
@@ -1291,8 +1347,12 @@ class ImpalaShell(cmd.Cmd, object):
       else:
         error_report = ""
 
-      self._print_if_verbose("%s %d row(s)%s in %2.2fs" %\
-          (verb, num_rows, error_report, time_elapsed))
+      if num_rows is not None:
+        self._print_if_verbose("%s %d row(s)%s in %2.2fs" %
+            (verb, num_rows, error_report, time_elapsed))
+      else:
+        self._print_if_verbose("Time elapsed: %2.2fs" %
+            (time_elapsed))
 
       if not is_dml:
         self.imp_client.close_query(self.last_query_handle)
@@ -1448,7 +1508,10 @@ class ImpalaShell(cmd.Cmd, object):
     if self.readline and self.readline.get_current_history_length() > 0:
       for index in xrange(1, self.readline.get_current_history_length() + 1):
         cmd = self.readline.get_history_item(index)
-        print('[%d]: %s' % (index, cmd.decode('utf-8', 'replace')), file=sys.stderr)
+        if sys.version_info.major == 2:
+          print('[%d]: %s' % (index, cmd.decode('utf-8', 'replace')), file=sys.stderr)
+        else:
+          print('[%d]: %s' % (index, cmd), file=sys.stderr)
     else:
       print(READLINE_UNAVAILABLE_ERROR, file=sys.stderr)
 
@@ -2012,6 +2075,13 @@ def impala_shell_main():
       print('Error opening output file for writing: %s' % e, file=sys.stderr)
       raise FatalShellException()
 
+  if options.http_socket_timeout_s is not None:
+    if (options.http_socket_timeout_s != 'None' and
+          float(options.http_socket_timeout_s) < 0):
+        print("http_socket_timeout_s must be a nonnegative floating point number"
+              " expressing seconds, or None", file=sys.stderr)
+        raise FatalShellException()
+
   options.variables = parse_variables(options.keyval)
 
   # Override query_options from config file with those specified on the command line.
