@@ -271,12 +271,26 @@ def get_option_parser(defaults):
                     help="Timeout in milliseconds after which impala-shell will time out"
                     " if it fails to connect to Impala server. Set to 0 to disable any"
                     " timeout.")
+  parser.add_option("--http_socket_timeout_s",
+                    help="Timeout in seconds after which the socket will time out"
+                    " if the associated operation cannot be completed. Set to None to"
+                    " disable any timeout. Only supported for hs2-http mode.")
   parser.add_option("--protocol", dest="protocol", default="hs2",
                     help="Protocol to use for client/server connection. Valid inputs are "
                          "['hs2', 'hs2-http', 'beeswax']. 'hs2-http' uses HTTP transport "
                          "to speak to the coordinator while 'hs2' and 'beeswax' use the "
                          "binary TCP based transport. Beeswax support is deprecated "
                          "and will be removed in the future.")
+  parser.add_option("--strict_hs2_protocol", dest="strict_hs2_protocol",
+                    action="store_true",
+                    help="True if the hs2 connection is using the strict hs2 protocol."
+                         "Only useful if connecting straight to hs2 instead of Impala."
+                         "The default hs2 port is 11050 and the default hs2 http port "
+                         "is 10001.")
+  parser.add_option("--use_ldap_test_password", dest="use_ldap_test_password",
+                    action="store_true",
+                    help="True if need to use the default LDAP password. This is needed "
+                         "when running tests in strict mode.")
   parser.add_option("--http_path", dest="http_path", default="cliservice",
                     help="Default http path on the coordinator to connect to. The final "
                     "connection URL looks like <http(s)>://<coordinator-host>:<port>/"
@@ -296,6 +310,13 @@ def get_option_parser(defaults):
                     "batches (assuming the default batch size). Note that if result "
                     "spooling is disabled only a single row batch can be fetched at a "
                     "time regardless of the specified fetch_size.")
+  parser.add_option("--http_cookie_names", dest="http_cookie_names",
+                    default="impala.auth,impala.session.id",
+                    help="A comma-separated list of HTTP cookie names that are supported "
+                    "by the impala-shell. If a cookie with one of these names is "
+                    "returned in an http response by the server or an intermediate proxy "
+                    "then it will be included in each subsequent request for the same "
+                    "connection.")
 
 
   # add default values to the help text
@@ -338,6 +359,13 @@ def get_option_parser(defaults):
   if '--live_progress' in sys.argv and '--disable_live_progress' in sys.argv:
     parser.error("options --live_progress and --disable_live_progress are mutually "
                  "exclusive")
+
+  if '--strict_hs2_protocol' in sys.argv:
+    if '--live_progress' in sys.argv:
+      parser.error("options --strict_hs2_protocol does not support --live_progress")
+    if '--live_summary' in sys.argv:
+      parser.error("options --strict_hs2_protocol does not support --live_summary")
+
   if '--verbose' in sys.argv and '--quiet' in sys.argv:
     parser.error("options --verbose and --quiet are mutually exclusive")
 
