@@ -119,8 +119,8 @@ fi
 TIMEOUT_PID=$!
 
 SCHEMA_MISMATCH_ERROR="A schema change has been detected in the metadata, "
-SCHEMA_MISMATCH_ERROR+="but it cannot be loaded on Isilon, s3, gcs or local filesystem, "
-SCHEMA_MISMATCH_ERROR+="and the filesystem is ${TARGET_FILESYSTEM}".
+SCHEMA_MISMATCH_ERROR+="but it cannot be loaded on Isilon, s3, gcs, cos or local "
+SCHEMA_MISMATCH_ERROR+="filesystem, and the filesystem is ${TARGET_FILESYSTEM}".
 
 if [[ $SKIP_METADATA_LOAD -eq 0  && "$SNAPSHOT_FILE" = "" ]]; then
   run-step "Generating HBase data" create-hbase.log \
@@ -134,9 +134,10 @@ elif [ $SKIP_SNAPSHOT_LOAD -eq 0 ]; then
   # Don't skip the metadata load if a schema change is detected.
   if ! ${IMPALA_HOME}/testdata/bin/check-schema-diff.sh; then
     if [[ "${TARGET_FILESYSTEM}" == "isilon" || "${TARGET_FILESYSTEM}" == "s3" || \
-          "${TARGET_FILESYSTEM}" == "local" || "${TARGET_FILESYSTEM}" == "gs" ]] ; then
+          "${TARGET_FILESYSTEM}" == "local" || "${TARGET_FILESYSTEM}" == "gs" || \
+          "${TARGET_FILESYSTEM}" == "cosn" ]] ; then
       echo "ERROR in $0 at line $LINENO: A schema change has been detected in the"
-      echo "metadata, but it cannot be loaded on isilon, s3, gcs or local and the"
+      echo "metadata, but it cannot be loaded on isilon, s3, gcs, cos or local and the"
       echo "target file system is ${TARGET_FILESYSTEM}.  Exiting."
       # Generate an explicit JUnitXML symptom report here for easier triaging
       ${IMPALA_HOME}/bin/generate_junitxml.py --phase=dataload \
@@ -335,19 +336,21 @@ function setup-ranger {
     "${RANGER_SETUP_DIR}/impala_group_owner.json.template" > \
     "${RANGER_SETUP_DIR}/impala_group_owner.json"
 
-  export GROUP_ID_OWNER=$(wget -qO - --auth-no-challenge --user=admin --password=admin \
+  GROUP_ID_OWNER=$(wget -qO - --auth-no-challenge --user=admin --password=admin \
     --post-file="${RANGER_SETUP_DIR}/impala_group_owner.json" \
     --header="accept:application/json" \
     --header="Content-Type:application/json" \
     http://localhost:6080/service/xusers/secure/groups |
     python -c "import sys, json; print json.load(sys.stdin)['id']")
+  export GROUP_ID_OWNER
 
-  export GROUP_ID_NON_OWNER=$(wget -qO - --auth-no-challenge --user=admin \
+  GROUP_ID_NON_OWNER=$(wget -qO - --auth-no-challenge --user=admin \
     --password=admin --post-file="${RANGER_SETUP_DIR}/impala_group_non_owner.json" \
     --header="accept:application/json" \
     --header="Content-Type:application/json" \
     http://localhost:6080/service/xusers/secure/groups |
     python -c "import sys, json; print json.load(sys.stdin)['id']")
+  export GROUP_ID_NON_OWNER
 
   perl -wpl -e 's/\$\{([^}]+)\}/defined $ENV{$1} ? $ENV{$1} : $&/eg' \
     "${RANGER_SETUP_DIR}/impala_user_owner.json.template" > \
@@ -382,9 +385,10 @@ function setup-ranger {
     --header="Content-Type:application/json" \
     http://localhost:6080/service/public/v2/api/service
 
-  curl -u admin:admin -H "Accept: application/json" -H "Content-Type: application/json" \
-    -X PUT http://localhost:6080/service/public/v2/api/policy/4 \
-    -d @"${RANGER_SETUP_DIR}/policy_4_revised.json"
+  curl -f -u admin:admin -H "Accept: application/json" \
+    -H "Content-Type: application/json" \
+    -X PUT http://localhost:6080/service/public/v2/api/policy/5 \
+    -d @"${RANGER_SETUP_DIR}/policy_5_revised.json"
 }
 
 function copy-and-load-dependent-tables {
