@@ -426,7 +426,7 @@ def avro_schema(columns):
   return json.dumps(record)
 
 def build_compression_codec_statement(codec, compression_type, file_format):
-  codec = AVRO_COMPRESSION_MAP[codec] if file_format == 'avro' else COMPRESSION_MAP[codec]
+  codec = (AVRO_COMPRESSION_MAP if file_format == 'avro' else COMPRESSION_MAP).get(codec)
   if not codec:
     return str()
   return (AVRO_COMPRESSION_CODEC % codec) if file_format == 'avro' else (
@@ -688,6 +688,14 @@ def generate_statements(output_name, test_vectors, sections,
           create_file_format == 'orc' and
           'transactional' not in tblproperties):
         tblproperties['transactional'] = 'true'
+      if create_file_format == 'orc' and create_codec != 'def':
+        # The default value of 'orc.compress' is ZLIB which corresponds to 'def'.
+        # We just need it for non-def codec.
+        # The original codec name can be used except snap.
+        if create_codec == 'snap':
+          tblproperties['orc.compress'] = 'SNAPPY'
+        else:
+          tblproperties['orc.compress'] = create_codec
 
       hdfs_location = '{0}.{1}{2}'.format(db_name, table_name, db_suffix)
       # hdfs file names for functional datasets are stored
@@ -698,7 +706,8 @@ def generate_statements(output_name, test_vectors, sections,
         hdfs_location = hdfs_location.split('.')[-1]
       # Transactional tables need to be put under the 'managed' directory.
       if is_transactional(tblproperties):
-        hdfs_location = os.path.join('managed', hdfs_location)
+        db_location = '{0}{1}.db'.format(db_name, db_suffix)
+        hdfs_location = os.path.join('managed', db_location, hdfs_location)
       data_path = os.path.join(options.hive_warehouse_dir, hdfs_location)
 
       output = impala_create
@@ -775,7 +784,7 @@ def generate_statements(output_name, test_vectors, sections,
       if not force_reload and hdfs_location in existing_tables:
         print 'HDFS path:', data_path, 'contains data. Data loading can be skipped.'
       else:
-        print 'HDFS path:', data_path, 'does not exists or is empty. Data will be loaded.'
+        print 'HDFS path:', data_path, 'does not exist or is empty. Data will be loaded.'
         if not db_suffix:
           if load:
             hive_output.load_base.append(build_load_statement(load, db_name,
@@ -810,10 +819,8 @@ def generate_statements(output_name, test_vectors, sections,
         (output_name, file_format, codec, compression_type))
 
   if hbase_output:
-    hbase_output.create.append("exit")
     hbase_output.write_to_file('load-' + output_name + '-hbase-generated.create')
   if hbase_post_load:
-    hbase_post_load.load.append("exit")
     hbase_post_load.write_to_file('post-load-' + output_name + '-hbase-generated.sql')
   impala_invalidate.write_to_file("invalidate-" + output_name + "-impala-generated.sql")
 
