@@ -28,7 +28,9 @@ from subprocess import check_call
 from getpass import getuser
 from tests.common.custom_cluster_test_suite import CustomClusterTestSuite
 from tests.common.skip import (SkipIfS3, SkipIfABFS, SkipIfADLS, SkipIfIsilon,
-                               SkipIfLocal, SkipIfHive2, SkipIfGCS)
+                               SkipIfLocal, SkipIfHive2, SkipIfGCS, SkipIfCOS)
+from tests.common.test_dimensions import (create_client_protocol_dimension,
+    create_exec_option_dimension, create_orc_dimension)
 from tests.util.hdfs_util import NAMENODE
 from tests.util.calculation_util import get_random_id
 
@@ -1071,6 +1073,35 @@ class TestRanger(CustomClusterTestSuite):
       for i in range(policy_cnt):
         TestRanger._remove_policy(unique_name + str(i))
 
+  @pytest.mark.execute_serially
+  @CustomClusterTestSuite.with_args(
+    impalad_args=IMPALAD_ARGS, catalogd_args=CATALOGD_ARGS)
+  def test_block_metadata_update(self, vector, unique_name):
+    """Test that the metadata update operation on a table by a requesting user is denied
+       if there exists a column masking policy defined on any column in the table for the
+       requesting user even when the table metadata (e.g., list of columns) have been
+       invalidated immediately before the requesting user tries to invalidate the table
+       metadata again. This test would have failed if we did not load the table metadata
+       for ResetMetadataStmt."""
+    user = getuser()
+    admin_client = self.create_impala_client()
+    non_owner_client = self.create_impala_client()
+    try:
+      TestRanger._add_column_masking_policy(
+          unique_name, user, "functional", "alltypestiny", "id",
+          "CUSTOM", "id * 100")
+      self.execute_query_expect_success(admin_client,
+          "invalidate metadata functional.alltypestiny", user=ADMIN)
+      admin_client.execute("grant all on server to user {0}".format(user))
+      result = self.execute_query_expect_failure(
+          non_owner_client, "invalidate metadata functional.alltypestiny", user=user)
+      assert "User '{0}' does not have privileges to execute " \
+          "'INVALIDATE METADATA/REFRESH' on: functional.alltypestiny".format(user) \
+          in str(result)
+    finally:
+      TestRanger._remove_policy(unique_name)
+      admin_client.execute("revoke all on server from user {0}".format(user))
+
   @pytest.mark.execute_serially
   @CustomClusterTestSuite.with_args(
     impalad_args=IMPALAD_ARGS, catalogd_args=CATALOGD_ARGS)
@@ -1356,6 +1387,7 @@ class TestRanger(CustomClusterTestSuite):
   @SkipIfLocal.hive
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfHive2.ranger_auth
   @CustomClusterTestSuite.with_args()
   def test_hive_with_ranger_setup(self, vector):
@@ -1573,3 +1605,68 @@ class TestRangerColumnMaskingTpchNested(CustomClusterTestSuite):
         for col in tbl_cols[tbl]:
           policy_name = "%s_%s_mask" % (tbl, col)
           TestRanger._remove_policy(policy_name)
+
+
+class TestRangerColumnMaskingComplexTypesInSelectList(CustomClusterTestSuite):
+  """
+  Tests Ranger policies when complex types are given in the select list. The reason
+  this is a separate class is that directly querying complex types works only on HS2
+  while some tests in TestRanger needs Beeswax interface otherwise some of them fails.
+  """
+
+  @classmethod
+  def get_workload(cls):
+    return 'functional-query'
+
+  @classmethod
+  def add_test_dimensions(cls):
+    super(TestRangerColumnMaskingComplexTypesInSelectList, cls).add_test_dimensions()
+    cls.ImpalaTestMatrix.add_dimension(create_client_protocol_dimension())
+    cls.ImpalaTestMatrix.add_dimension(create_orc_dimension(cls.get_workload()))
+    cls.ImpalaTestMatrix.add_constraint(lambda v:
+        v.get_value('protocol') == 'hs2')
+    cls.ImpalaTestMatrix.add_dimension(create_exec_option_dimension(
+        disable_codegen_options=[True]))
+
+  @classmethod
+  def add_custom_cluster_constraints(cls):
+    # Do not call the super() implementation, because this class needs to relax
+    # the set of constraints. The usual constraints only run on uncompressed text.
+    # This disables that constraint to let us run against only ORC.
+    return
+
+  @pytest.mark.execute_serially
+  @CustomClusterTestSuite.with_args(
+    impalad_args=IMPALAD_ARGS, catalogd_args=CATALOGD_ARGS)
+  def test_column_masking_with_structs_in_select_list(self, vector, unique_name):
+    user = getuser()
+    db = "functional_orc_def"
+    # Create another client for admin user since current user doesn't have privileges to
+    # create/drop databases or refresh authorization.
+    admin_client = self.create_impala_client()
+    policy_cnt = 0
+    try:
+      # Add a policy on a primitive column of a table which contains nested columns.
+      TestRanger._add_column_masking_policy(
+          unique_name + str(policy_cnt), user, "functional_orc_def",
+          "complextypes_structs", "str", "MASK_NULL")
+      policy_cnt += 1
+      TestRanger._add_column_masking_policy(
+          unique_name + str(policy_cnt), user, "functional_orc_def",
+          "complextypes_structs", "tiny_struct", "MASK_NULL")
+      policy_cnt += 1
+      TestRanger._add_column_masking_policy(
+          unique_name + str(policy_cnt), user, "functional_orc_def",
+          "complextypestbl", "id", "MASK_NULL")
+      policy_cnt += 1
+      TestRanger._add_column_masking_policy(
+          unique_name + str(policy_cnt), user, "functional_orc_def",
+          "complextypestbl", "int_array_array", "MASK_NULL")
+      policy_cnt += 1
+      self.execute_query_expect_success(admin_client, "refresh authorization",
+          user=ADMIN)
+      self.run_test_case("QueryTest/ranger_column_masking_struct_in_select_list", vector,
+          use_db=db)
+    finally:
+      for i in range(policy_cnt):
+        TestRanger._remove_policy(unique_name + str(i))
