@@ -30,6 +30,7 @@ import requests
 import socket
 import subprocess
 import time
+import string
 from functools import wraps
 from getpass import getuser
 from random import choice
@@ -65,6 +66,7 @@ from tests.util.filesystem_utils import (
     IS_ABFS,
     IS_ADLS,
     IS_GCS,
+    IS_COS,
     IS_HDFS,
     S3_BUCKET_NAME,
     S3GUARD_ENABLED,
@@ -111,6 +113,10 @@ IMPALAD_HS2_HOST_PORT =\
     IMPALAD_HOSTNAME + ":" + pytest.config.option.impalad_hs2_port
 IMPALAD_HS2_HTTP_HOST_PORT =\
     IMPALAD_HOSTNAME + ":" + pytest.config.option.impalad_hs2_http_port
+STRICT_HS2_HOST_PORT =\
+    IMPALAD_HOSTNAME + ":" + pytest.config.option.strict_hs2_port
+STRICT_HS2_HTTP_HOST_PORT =\
+    IMPALAD_HOSTNAME + ":" + pytest.config.option.strict_hs2_http_port
 HIVE_HS2_HOST_PORT = pytest.config.option.hive_server2
 WORKLOAD_DIR = os.environ['IMPALA_WORKLOAD_DIR']
 HDFS_CONF = HdfsConfig(pytest.config.option.minicluster_xml_conf)
@@ -151,6 +157,24 @@ class ImpalaTestSuite(BaseTestSuite):
     # to work with the HS2 client can add HS2 in addition to or instead of beeswax.
     cls.ImpalaTestMatrix.add_dimension(ImpalaTestDimension('protocol', 'beeswax'))
 
+  @staticmethod
+  def create_hive_client(port):
+    """
+    Creates a HMS client to a external running metastore service at the provided port
+    """
+    trans_type = 'buffered'
+    if pytest.config.option.use_kerberos:
+      trans_type = 'kerberos'
+    hive_transport = create_transport(
+      host=pytest.config.option.metastore_server.split(':')[0],
+      port=port,
+      service=pytest.config.option.hive_service_name,
+      transport_type=trans_type)
+    protocol = TBinaryProtocol.TBinaryProtocol(hive_transport)
+    hive_client = ThriftHiveMetastore.Client(protocol)
+    hive_transport.open()
+    return hive_client, hive_transport
+
   @classmethod
   def setup_class(cls):
     """Setup section that runs before each test suite"""
@@ -200,6 +224,7 @@ class ImpalaTestSuite(BaseTestSuite):
     #     ADLS: uses a mixture of azure-data-lake-store-python and the HDFS CLI (TODO:
     #           this should completely switch to the HDFS CLI once we test it)
     #     GCS:  uses the HDFS CLI
+    #     COS:  uses the HDFS CLI
     #
     # 'hdfs_client' is a HDFS-specific client library, and it only works when running on
     # HDFS. When using 'hdfs_client', the test must be skipped on everything other than
@@ -223,6 +248,9 @@ class ImpalaTestSuite(BaseTestSuite):
     elif IS_GCS:
       # GCS is implemented via HDFS command line client
       cls.filesystem_client = HadoopFsCommandLineClient("GCS")
+    elif IS_COS:
+      # COS is implemented via HDFS command line client
+      cls.filesystem_client = HadoopFsCommandLineClient("COS")
 
     # Override the shell history path so that commands run by any tests
     # don't write any history into the developer's file.
@@ -898,6 +926,22 @@ class ImpalaTestSuite(BaseTestSuite):
     assert (impala_results is not None) and (hive_results is not None)
     assert compare(impala_results, hive_results)
 
+  def exec_with_jdbc(self, stmt):
+    """Pass 'stmt' to IMPALA via Impala JDBC client and execute it"""
+    # execute_using_jdbc expects a Query object. Convert the query string into a Query
+    # object
+    query = Query()
+    query.query_str = stmt
+    # Run the statement targeting Impala
+    exec_opts = JdbcQueryExecConfig(impalad=IMPALAD_HS2_HOST_PORT, transport='NOSASL')
+    return execute_using_jdbc(query, exec_opts).data
+
+  def exec_with_jdbc_and_compare_result(self, stmt, expected):
+    """Execute 'stmt' via Impala JDBC client and compare the result with 'expected'"""
+    result = self.exec_with_jdbc(stmt)
+    # Check the results
+    assert (result is not None) and (result == expected)
+
   def load_query_test_file(self, workload, file_name, valid_section_names=None,
       encoding=None):
     """
@@ -995,10 +1039,10 @@ class ImpalaTestSuite(BaseTestSuite):
       tf_dimensions = ImpalaTestDimension('table_format', *table_formats)
     else:
       tf_dimensions = load_table_info_dimension(cls.get_workload(), exploration_strategy)
-    # If 'skip_hbase' is specified or the filesystem is isilon, s3, GCS(gs) or local,
-    # we don't need the hbase dimension.
+    # If 'skip_hbase' is specified or the filesystem is isilon, s3, GCS(gs), COS(cosn) or
+    # local, we don't need the hbase dimension.
     if pytest.config.option.skip_hbase or TARGET_FILESYSTEM.lower() \
-        in ['s3', 'isilon', 'local', 'abfs', 'adls', 'gs']:
+        in ['s3', 'isilon', 'local', 'abfs', 'adls', 'gs', 'cosn']:
       for tf_dimension in tf_dimensions:
         if tf_dimension.value.file_format == "hbase":
           tf_dimensions.remove(tf_dimension)
@@ -1092,6 +1136,19 @@ class ImpalaTestSuite(BaseTestSuite):
         continue
     raise Exception("DB {0} didn't show up after {1}s", db_name, timeout_s)
 
+  def confirm_db_exists(self, db_name):
+    """Confirm the database with 'db_name' is present in the impalad's local catalog.
+       Fail if the db is not present"""
+    # This will throw an exception if the database is not present.
+    self.client.execute("describe database `{db_name}`".format(db_name=db_name))
+    return
+
+  def confirm_table_exists(self, db_name, tbl_name):
+    """Confirms if the table exists. The describe table command will fail if the table
+       does not exist."""
+    self.client.execute("describe `{0}`.`{1}`".format(db_name, tbl_name))
+    return
+
   def wait_for_table_to_appear(self, db_name, table_name, timeout_s):
     """Wait until the table with 'table_name' in 'db_name' is present in the
     impalad's local catalog. Fail after timeout_s if the doesn't appear."""
@@ -1185,3 +1242,11 @@ class ImpalaTestSuite(BaseTestSuite):
         LOG.info("Expected log lines could not be found, sleeping before retrying: %s",
             str(e))
         time.sleep(1)
+
+  @staticmethod
+  def get_random_name(prefix='', length=5):
+    """
+    Gets a random name used to create unique database or table
+    """
+    assert length > 0
+    return prefix + ''.join(choice(string.ascii_lowercase) for i in range(length))
