@@ -32,6 +32,7 @@ from tests.util.filesystem_utils import (
     IS_ABFS,
     IS_ADLS,
     IS_GCS,
+    IS_COS,
     IS_EC,
     IS_HDFS,
     IS_ISILON,
@@ -128,6 +129,28 @@ class SkipIfGCS:
       reason="Flakiness due to unpredictable listing times on GCS.")
 
 
+class SkipIfCOS:
+
+  # These are skipped due to product limitations.
+  caching = pytest.mark.skipif(IS_COS, reason="SET CACHED not implemented for COS")
+  hive = pytest.mark.skipif(IS_COS, reason="Hive doesn't work with COS")
+  hdfs_block_size = pytest.mark.skipif(IS_COS, reason="COS uses it's own block size")
+  hdfs_acls = pytest.mark.skipif(IS_COS, reason="HDFS acls are not supported on COS")
+  jira = partial(pytest.mark.skipif, IS_COS)
+  hdfs_encryption = pytest.mark.skipif(IS_COS,
+      reason="HDFS encryption is not supported with COS")
+
+  # These need test infra work to re-enable.
+  udfs = pytest.mark.skipif(IS_COS, reason="udas/udfs not copied to COS")
+  datasrc = pytest.mark.skipif(IS_COS, reason="data sources not copied to COS")
+  hbase = pytest.mark.skipif(IS_COS, reason="HBase not started with COS")
+  qualified_path = pytest.mark.skipif(IS_COS,
+      reason="Tests rely on HDFS qualified paths, IMPALA-1872")
+  variable_listing_times = pytest.mark.skipif(IS_COS,
+      reason="Flakiness due to unpredictable listing times on COS.")
+  eventually_consistent = pytest.mark.skipif(IS_COS,
+      reason="The client is slow to realize changes to file metadata")
+
 class SkipIfKudu:
   no_hybrid_clock = pytest.mark.skipif(
       get_kudu_master_flag("--use_hybrid_clock") == "false",
@@ -232,11 +255,10 @@ class SkipIfDockerizedCluster:
       IS_DOCKERIZED_TEST_CLUSTER, reason="Daemon would need to access host filesystem.")
   insert_acls = pytest.mark.skipif(IS_DOCKERIZED_TEST_CLUSTER,
       reason="IMPALA-8384: insert ACL tests are broken on dockerised minicluster.")
+  jira = partial(pytest.mark.skipif, IS_DOCKERIZED_TEST_CLUSTER)
 
 
 class SkipIfHive3:
-  kudu_hms_notifications_not_supported = pytest.mark.skipif(HIVE_MAJOR_VERSION >= 3,
-      reason="Kudu is not tested with Hive 3 notifications yet, see IMPALA-8751.")
   col_stat_separated_by_engine = pytest.mark.skipif(HIVE_MAJOR_VERSION >= 3,
       reason="Hive 3 separates column statistics by engine")
   without_hms_not_supported = pytest.mark.skipif(HIVE_MAJOR_VERSION >= 3,
@@ -302,6 +324,11 @@ class SkipIfCatalogV2:
       IMPALA_TEST_CLUSTER_PROPERTIES.is_catalog_v2_cluster(),
       reason="Table isn't invalidated with Local catalog and enabled hms_event_polling.")
 
+  @classmethod
+  def hms_event_polling_disabled(self):
+    return pytest.mark.skipif(
+      not IMPALA_TEST_CLUSTER_PROPERTIES.is_event_polling_enabled(),
+      reason="Test expects event polling to be enabled.")
 
 class SkipIfOS:
   redhat6 = pytest.mark.skipif(IS_REDHAT_6_DERIVATIVE,
