@@ -22,9 +22,26 @@ import os
 from itertools import product
 
 from tests.common.test_vector import ImpalaTestDimension, ImpalaTestVector
+from tests.util.filesystem_utils import (
+    IS_HDFS)
 
 WORKLOAD_DIR = os.environ['IMPALA_WORKLOAD_DIR']
 
+# Map from the test dimension file_format string to the SQL "STORED AS" or "STORED BY"
+# argument.
+FILE_FORMAT_TO_STORED_AS_MAP = {
+  'text': 'TEXTFILE',
+  'seq': 'SEQUENCEFILE',
+  'rc': 'RCFILE',
+  'orc': 'ORC',
+  'parquet': 'PARQUET',
+  'hudiparquet': 'HUDIPARQUET',
+  'avro': 'AVRO',
+  'hbase': "'org.apache.hadoop.hive.hbase.HBaseStorageHandler'",
+  'kudu': "KUDU",
+  'iceberg': "ICEBERG"
+}
+
 # Describes the configuration used to execute a single tests. Contains both the details
 # of what specific table format to target along with the exec options (num_nodes, etc)
 # to use when running the query.
@@ -107,6 +124,11 @@ def create_parquet_dimension(workload):
       TableFormatInfo.create_from_string(dataset, 'parquet/none'))
 
 
+def create_orc_dimension(workload):
+  dataset = get_dataset_from_workload(workload)
+  return ImpalaTestDimension('table_format',
+      TableFormatInfo.create_from_string(dataset, 'orc/def'))
+
 def create_avro_snappy_dimension(workload):
   dataset = get_dataset_from_workload(workload)
   return ImpalaTestDimension('table_format',
@@ -129,6 +151,19 @@ def create_client_protocol_dimension():
   return ImpalaTestDimension('protocol', 'beeswax', 'hs2', 'hs2-http')
 
 
+def create_client_protocol_strict_dimension():
+  # only support strict dimensions if the file system is HDFS, since that is
+  # where the hive cluster is run.
+  if IS_HDFS:
+    return ImpalaTestDimension('strict_hs2_protocol', False, True)
+  else:
+    return create_client_protocol_no_strict_dimension()
+
+
+def create_client_protocol_no_strict_dimension():
+  return ImpalaTestDimension('strict_hs2_protocol', False)
+
+
 def hs2_parquet_constraint(v):
   """Constraint function, used to only run HS2 against Parquet format, because file format
   and the client protocol are orthogonal."""
@@ -144,6 +179,13 @@ def hs2_text_constraint(v):
           v.get_value('table_format').file_format == 'text' and
           v.get_value('table_format').compression_codec == 'none')
 
+
+def orc_schema_resolution_constraint(v):
+  """ Constraint to use multiple orc_schema_resolution only in case of orc files"""
+  file_format = v.get_value('table_format').file_format
+  orc_schema_resolution = v.get_value('orc_schema_resolution')
+  return file_format == 'orc' or orc_schema_resolution == 0
+
 # Common sets of values for the exec option vectors
 ALL_BATCH_SIZES = [0]
 
@@ -177,7 +219,8 @@ def create_exec_option_dimension(cluster_sizes=ALL_CLUSTER_SIZES,
       'batch_size': batch_sizes,
       'disable_codegen': disable_codegen_options,
       'disable_codegen_rows_threshold': disable_codegen_rows_threshold_options,
-      'num_nodes': cluster_sizes}
+      'num_nodes': cluster_sizes,
+      'test_replan': [1]}
 
   if sync_ddl is not None:
     exec_option_dimensions['sync_ddl'] = sync_ddl
