@@ -49,7 +49,7 @@ class TestCatalogWait(CustomClusterTestSuite):
 
     # On startup, expect only two executors to be registered.
     self._start_impala_cluster(["--catalog_init_delays=0,0,200000"],
-                               expected_num_executors=2,
+                               expected_num_impalads=2,
                                expected_subscribers=4)
 
     # Expect that impalad[2] is not ready.
