@@ -25,7 +25,7 @@ from subprocess import check_call
 from tests.util.filesystem_utils import get_fs_path
 from tests.common.custom_cluster_test_suite import CustomClusterTestSuite
 from tests.common.skip import (SkipIf, SkipIfS3, SkipIfABFS, SkipIfADLS,
-                               SkipIfIsilon, SkipIfGCS, SkipIfLocal)
+                               SkipIfIsilon, SkipIfGCS, SkipIfCOS, SkipIfLocal)
 
 LOG = logging.getLogger('test_coordinators')
 LOG.setLevel(level=logging.DEBUG)
@@ -275,6 +275,7 @@ class TestCoordinators(CustomClusterTestSuite):
       self.client = client
 
       client.execute("SET EXPLAIN_LEVEL=2")
+      client.execute("SET TEST_REPLAN=0")
 
       # Ensure that the plan generated always uses only the executor nodes for scanning
       # Multi-partition table
@@ -321,6 +322,7 @@ class TestCoordinators(CustomClusterTestSuite):
 
   @SkipIfS3.hbase
   @SkipIfGCS.hbase
+  @SkipIfCOS.hbase
   @SkipIfABFS.hbase
   @SkipIfADLS.hbase
   @SkipIfIsilon.hbase
