@@ -22,6 +22,7 @@ from tests.util.concurrent_workload import ConcurrentWorkload
 
 import json
 import logging
+import os
 import pytest
 from time import sleep
 
@@ -30,6 +31,9 @@ LOG = logging.getLogger("test_auto_scaling")
 # Non-trivial query that gets scheduled on all executors within a group.
 TEST_QUERY = "select count(*) from functional.alltypes where month + random() < 3"
 
+DEFAULT_RESOURCE_POOL = "default-pool"
+
+
 class TestExecutorGroups(CustomClusterTestSuite):
   """This class contains tests that exercise the logic related to scaling clusters up and
   down by adding and removing groups of executors. All tests start with a base cluster
@@ -43,17 +47,18 @@ class TestExecutorGroups(CustomClusterTestSuite):
     method.func_dict["cluster_size"] = 1
     method.func_dict["num_exclusive_coordinators"] = 1
     self.num_groups = 1
-    self.num_executors = 1
+    self.num_impalads = 1
     super(TestExecutorGroups, self).setup_method(method)
     self.coordinator = self.cluster.impalads[0]
 
-  def _group_name(self, name):
+  def _group_name(self, resource_pool, name_suffix):
     # By convention, group names must start with their associated resource pool name
-    # followed by a "-". Tests in this class all use the default resource pool.
-    return "default-pool-%s" % name
+    # followed by a "-". Tests in this class mostly use the default resource pool.
+    return "%s-%s" % (resource_pool, name_suffix)
 
   def _add_executor_group(self, name_suffix, min_size, num_executors=0,
-                          admission_control_slots=0):
+                          admission_control_slots=0, extra_args=None,
+                          resource_pool=DEFAULT_RESOURCE_POOL):
     """Adds an executor group to the cluster. 'min_size' specifies the minimum size for
     the new group to be considered healthy. 'num_executors' specifies the number of
     executors to start and defaults to 'min_size' but can be different from 'min_size' to
@@ -63,19 +68,37 @@ class TestExecutorGroups(CustomClusterTestSuite):
     self.num_groups += 1
     if num_executors == 0:
       num_executors = min_size
-    self.num_executors += num_executors
-    name = self._group_name(name_suffix)
+    self.num_impalads += num_executors
+    name = self._group_name(resource_pool, name_suffix)
     LOG.info("Adding %s executors to group %s with minimum size %s" %
              (num_executors, name, min_size))
     cluster_args = ["--impalad_args=-admission_control_slots=%s" %
                     admission_control_slots]
     if len(name_suffix) > 0:
       cluster_args.append("--impalad_args=-executor_groups=%s:%s" % (name, min_size))
+    if extra_args:
+      cluster_args.append("--impalad_args=%s" % extra_args)
     self._start_impala_cluster(options=cluster_args,
                                cluster_size=num_executors,
                                num_coordinators=0,
                                add_executors=True,
-                               expected_num_executors=self.num_executors)
+                               expected_num_impalads=self.num_impalads)
+
+  def _restart_coordinators(self, num_coordinators, extra_args=None):
+    """Restarts the coordinator spawned in setup_method and enables the caller to start
+    more than one coordinator by specifying 'num_coordinators'"""
+    LOG.info("Adding a coordinator")
+    cluster_args = ["--impalad_args=-executor_groups=coordinator"]
+    if extra_args:
+      cluster_args.append("--impalad_args=%s" % extra_args)
+    self._start_impala_cluster(options=cluster_args,
+                               cluster_size=num_coordinators,
+                               num_coordinators=num_coordinators,
+                               add_executors=False,
+                               expected_num_impalads=num_coordinators,
+                               use_exclusive_coordinators=True)
+    self.coordinator = self.cluster.impalads[0]
+    self.num_impalads = num_coordinators
 
   def _get_total_admitted_queries(self):
     """Returns the total number of queries that have been admitted to the default resource
@@ -98,22 +121,32 @@ class TestExecutorGroups(CustomClusterTestSuite):
       return self.coordinator.service.wait_for_metric_value(
         "cluster-membership.executor-groups.total", num_exec_grps, timeout=30)
 
-  def _get_num_executor_groups(self, only_healthy=False):
+  def _get_num_executor_groups(self, only_healthy=False, exec_group_set_prefix=None):
     """Returns the number of executor groups with at least one executor. If
-    'only_healthy' is True, only the number of healthy executor groups is returned."""
-    if only_healthy:
-      return self.coordinator.service.get_metric_value(
-        "cluster-membership.executor-groups.total-healthy")
+    'only_healthy' is True, only the number of healthy executor groups is returned.
+    If exec_group_set_prefix is used, it returns the metric corresponding to that
+    executor group set."""
+    metric_name = ""
+    if exec_group_set_prefix is None:
+      if only_healthy:
+        metric_name = "cluster-membership.executor-groups.total-healthy"
+      else:
+        metric_name = "cluster-membership.executor-groups.total"
     else:
-      return self.coordinator.service.get_metric_value(
-        "cluster-membership.executor-groups.total")
+      if only_healthy:
+        metric_name = "cluster-membership.group-set.executor-groups.total-healthy.{0}"\
+          .format(exec_group_set_prefix)
+      else:
+        metric_name = "cluster-membership.group-set.executor-groups.total.{0}"\
+          .format(exec_group_set_prefix)
+    return self.coordinator.service.get_metric_value(metric_name)
 
-  def _get_num_queries_executing_for_exec_group(self, group_name_suffix):
-    """Returns the number of queries running on the executor group 'group_name_suffix'.
+  def _get_num_queries_executing_for_exec_group(self, group_name_prefix):
+    """Returns the number of queries running on the executor group 'group_name_prefix'.
     None is returned if the group has no executors or does not exist."""
     METRIC_PREFIX = "admission-controller.executor-group.num-queries-executing.{0}"
     return self.coordinator.service.get_metric_value(
-      METRIC_PREFIX.format(self._group_name(group_name_suffix)))
+      METRIC_PREFIX.format(self._group_name(DEFAULT_RESOURCE_POOL, group_name_prefix)))
 
   def _assert_eventually_in_profile(self, query_handle, expected_str):
     """Assert with a timeout of 60 sec and a polling interval of 1 sec that the
@@ -438,7 +471,7 @@ class TestExecutorGroups(CustomClusterTestSuite):
     client.cancel(q3)
     self.coordinator.service.wait_for_metric_value(
       "admission-controller.executor-group.num-queries-executing.{0}".format(
-        self._group_name(group_names[0])), 0, timeout=30)
+        self._group_name(DEFAULT_RESOURCE_POOL, group_names[0])), 0, timeout=30)
 
   @pytest.mark.execute_serially
   def test_join_strategy_single_executor(self):
@@ -475,12 +508,12 @@ class TestExecutorGroups(CustomClusterTestSuite):
                                                    timeout=20)
     assert_broadcast_join()
 
-    # Kill a second executor. The group becomes unhealthy but we cache its last healthy
-    # size and will continue to pick a broadcast join.
+    # Kill a second executor. The group becomes unhealthy and we go back to using the
+    # expected size to plan which would result in a hash join
     self.cluster.impalads[-2].kill()
     self.coordinator.service.wait_for_metric_value("cluster-membership.backends.total", 1,
                                                    timeout=20)
-    assert_broadcast_join()
+    assert_hash_join()
 
   @pytest.mark.execute_serially
   def test_join_strategy_multiple_executors(self):
@@ -513,3 +546,226 @@ class TestExecutorGroups(CustomClusterTestSuite):
     self.coordinator.service.wait_for_metric_value("cluster-membership.backends.total", 2,
                                                    timeout=20)
     assert_hash_join()
+
+  @pytest.mark.execute_serially
+  def test_admission_control_with_multiple_coords(self):
+    """This test verifies that host level metrics like the num of admission slots used
+    and memory admitted is disseminated correctly across the cluster and accounted for
+    while making admission decisions. We run a query that takes up all of a particular
+    resource (slots or memory) and check if attempting to run a query on the other
+    coordinator results in queuing."""
+    # A long running query that runs on every executor
+    QUERY = "select * from functional_parquet.alltypes \
+                 where month < 3 and id + random() < sleep(100);"
+    # default_pool_mem_limit is set to enable mem based admission.
+    self._restart_coordinators(num_coordinators=2,
+                               extra_args="-default_pool_mem_limit=100g")
+    # Create fresh clients
+    second_coord_client = self.create_client_for_nth_impalad(1)
+    self.create_impala_clients()
+    # Add an exec group with a 4gb mem_limit.
+    self._add_executor_group("group1", 2, admission_control_slots=2,
+                             extra_args="-mem_limit=4g")
+    assert self._get_num_executor_groups(only_healthy=True) == 1
+    second_coord_client.set_configuration({'mt_dop': '2'})
+    handle_for_second = second_coord_client.execute_async(QUERY)
+    # Verify that the first coordinator knows about the query running on the second
+    self.coordinator.service.wait_for_metric_value(
+      "admission-controller.agg-num-running.default-pool", 1, timeout=30)
+    handle_for_first = self.execute_query_async(TEST_QUERY)
+    self.coordinator.service.wait_for_metric_value(
+      "admission-controller.local-num-queued.default-pool", 1, timeout=30)
+    profile = self.client.get_runtime_profile(handle_for_first)
+    assert "queue reason: Not enough admission control slots available on host" in \
+           profile, profile
+    self.close_query(handle_for_first)
+    second_coord_client.close_query(handle_for_second)
+    # Wait for first coordinator to get the admission update.
+    self.coordinator.service.wait_for_metric_value(
+      "admission-controller.agg-num-running.default-pool", 0, timeout=30)
+    # Now verify that mem based admission also works as intended. A max of mem_reserved
+    # and mem_admitted is used for this. Since mem_limit is being used here, both will be
+    # identical but this will at least test that code path as a sanity check.
+    second_coord_client.clear_configuration()
+    second_coord_client.set_configuration({'mem_limit': '4g'})
+    handle_for_second = second_coord_client.execute_async(QUERY)
+    # Verify that the first coordinator knows about the query running on the second
+    self.coordinator.service.wait_for_metric_value(
+      "admission-controller.agg-num-running.default-pool", 1, timeout=30)
+    handle_for_first = self.execute_query_async(TEST_QUERY)
+    self.coordinator.service.wait_for_metric_value(
+      "admission-controller.local-num-queued.default-pool", 1, timeout=30)
+    profile = self.client.get_runtime_profile(handle_for_first)
+    assert "queue reason: Not enough memory available on host" in profile, profile
+    self.close_query(handle_for_first)
+    second_coord_client.close_query(handle_for_second)
+
+  @pytest.mark.execute_serially
+  def test_admission_control_with_multiple_coords_and_exec_groups(self):
+    """This test verifies that admission control accounting works when using multiple
+    coordinators and multiple executor groups mapped to different resource pools and
+    having different sizes."""
+    # A long running query that runs on every executor
+    LONG_QUERY = "select * from functional_parquet.alltypes \
+                 where month < 3 and id + random() < sleep(100);"
+    # The path to resources directory which contains the admission control config files.
+    RESOURCES_DIR = os.path.join(os.environ['IMPALA_HOME'], "fe", "src", "test",
+                                 "resources")
+    fs_allocation_path = os.path.join(RESOURCES_DIR, "fair-scheduler-allocation.xml")
+    llama_site_path = os.path.join(RESOURCES_DIR, "llama-site-empty.xml")
+    # Start with a regular admission config with multiple pools and no resource limits.
+    self._restart_coordinators(num_coordinators=2,
+         extra_args="-vmodule admission-controller=3 "
+                    "-expected_executor_group_sets=root.queue1:2,root.queue2:1 "
+                    "-fair_scheduler_allocation_path %s "
+                    "-llama_site_path %s" % (
+                      fs_allocation_path, llama_site_path))
+
+    # Create fresh clients
+    second_coord_client = self.create_client_for_nth_impalad(1)
+    self.create_impala_clients()
+    # Add an exec group with a single admission slot and 2 executors.
+    self._add_executor_group("group", 2, admission_control_slots=1,
+                             resource_pool="root.queue1", extra_args="-mem_limit=2g")
+    # Add an exec group with a single admission slot and only 1 executor.
+    self._add_executor_group("group", 1, admission_control_slots=1,
+                             resource_pool="root.queue2", extra_args="-mem_limit=2g")
+    assert self._get_num_executor_groups(only_healthy=True) == 2
+    assert self._get_num_executor_groups(only_healthy=True,
+                                         exec_group_set_prefix="root.queue1") == 1
+    assert self._get_num_executor_groups(only_healthy=True,
+                                         exec_group_set_prefix="root.queue2") == 1
+
+    # Execute a long running query on group 'queue1'
+    self.client.set_configuration({'request_pool': 'queue1'})
+    handle_long_running_queue1 = self.execute_query_async(LONG_QUERY)
+    self.coordinator.service.wait_for_metric_value(
+      "admission-controller.executor-group.num-queries-executing.root.queue1-group",
+      1, timeout=30)
+    profile = self.client.get_runtime_profile(handle_long_running_queue1)
+    "Executor Group: root.queue1-group" in profile
+
+    # Try to execute another query on group 'queue1'. This one should queue.
+    handle_queued_query_queue1 = self.execute_query_async(TEST_QUERY)
+    self.coordinator.service.wait_for_metric_value(
+      "admission-controller.local-num-queued.root.queue1", 1, timeout=30)
+    profile = self.client.get_runtime_profile(handle_queued_query_queue1)
+    assert "queue reason: Not enough admission control slots available on host" in \
+           profile, profile
+
+    # Execute a query on group 'queue2'. This one will run as its running in another pool.
+    result = self.execute_query_expect_success(self.client, TEST_QUERY,
+                                               query_options={'request_pool': 'queue2'})
+    assert "Executor Group: root.queue2-group" in str(result.runtime_profile)
+
+    # Verify that multiple coordinators' accounting still works correctly in case of
+    # multiple executor groups.
+
+    # Run a query in group 'queue2' on the second coordinator
+    second_coord_client.set_configuration({'request_pool': 'queue2'})
+    second_coord_client.execute_async(LONG_QUERY)
+    # Verify that the first coordinator knows about the query running on the second
+    self.coordinator.service.wait_for_metric_value(
+      "admission-controller.agg-num-running.root.queue2", 1, timeout=30)
+
+    # Check that attempting to run another query in 'queue2' will queue the query.
+    self.client.set_configuration({'request_pool': 'queue2'})
+    handle_queued_query_queue2 = self.execute_query_async(TEST_QUERY)
+    self.coordinator.service.wait_for_metric_value(
+      "admission-controller.local-num-queued.root.queue2", 1, timeout=30)
+    profile = self.client.get_runtime_profile(handle_queued_query_queue2)
+    assert "queue reason: Not enough admission control slots available on host" in \
+           profile, profile
+
+    self.client.close()
+    second_coord_client.close()
+
+  @pytest.mark.execute_serially
+  def test_query_assignment_with_two_exec_groups(self):
+    """This test verifies that query assignment works with two executor groups with
+    different number of executors and memory limit in each."""
+    # A small query with estimated memory per host of 10MB that can run on the small
+    # executor group
+    SMALL_QUERY = "select count(*) from tpcds_parquet.date_dim where d_year=2022;"
+    # A large query with estimated memory per host of 132MB that can only run on
+    # the large executor group.
+    LARGE_QUERY = "select * from tpcds_parquet.store_sales where ss_item_sk = 1 limit 50;"
+    # The path to resources directory which contains the admission control config files.
+    RESOURCES_DIR = os.path.join(os.environ['IMPALA_HOME'], "fe", "src", "test",
+                                 "resources")
+    # Define two group sets: small and large
+    fs_allocation_path = os.path.join(RESOURCES_DIR, "fair-scheduler-2-groups.xml")
+    # Define the min-query-mem-limit and max-query-mem-limit properties of the two sets:
+    # small: [0, 64MB]
+    # large: [64MB+1Byte, 8PB]
+    llama_site_path = os.path.join(RESOURCES_DIR, "llama-site-2-groups.xml")
+    # Start with a regular admission config with multiple pools and no resource limits.
+    self._restart_coordinators(num_coordinators=1,
+         extra_args="-vmodule admission-controller=3 "
+                    "-expected_executor_group_sets=root.small:2,root.large:3 "
+                    "-fair_scheduler_allocation_path %s "
+                    "-llama_site_path %s" % (
+                      fs_allocation_path, llama_site_path))
+
+    # Create fresh client
+    self.create_impala_clients()
+    # Add an exec group with a single admission slot and 2 executors.
+    self._add_executor_group("group", 2, admission_control_slots=1,
+                             resource_pool="root.small", extra_args="-mem_limit=2g")
+    # Add another exec group with a single admission slot and 3 executors.
+    self._add_executor_group("group", 3, admission_control_slots=1,
+                             resource_pool="root.large", extra_args="-mem_limit=2g")
+    assert self._get_num_executor_groups(only_healthy=True) == 2
+    assert self._get_num_executor_groups(only_healthy=True,
+                                         exec_group_set_prefix="root.small") == 1
+    assert self._get_num_executor_groups(only_healthy=True,
+                                         exec_group_set_prefix="root.large") == 1
+
+    # Expect to run the small query on the small group
+    result = self.execute_query_expect_success(self.client, SMALL_QUERY)
+    assert "Executor Group: root.small-group" in str(result.runtime_profile)
+
+    # Expect to run the large query on the large group
+    result = self.execute_query_expect_success(self.client, LARGE_QUERY)
+    assert "Executor Group: root.large-group" in str(result.runtime_profile)
+
+    # Force to run the large query on the small group should fail
+    self.client.set_configuration({'request_pool': 'small'})
+    result = self.execute_query_expect_failure(self.client, LARGE_QUERY)
+    assert "The query does not fit any executor group set" in str(result)
+
+    self.client.close()
+
+  @pytest.mark.execute_serially
+  def test_per_exec_group_set_metrics(self):
+    """This test verifies that the metrics for each exec group set are updated
+    appropriately."""
+    self._restart_coordinators(num_coordinators=1,
+         extra_args="-expected_executor_group_sets=root.queue1:2,root.queue2:1")
+
+    # Add an unhealthy exec group in queue1 group set
+    self._add_executor_group("group", 2, num_executors=1,
+                             resource_pool="root.queue1", extra_args="-mem_limit=2g")
+    assert self._get_num_executor_groups(only_healthy=True,
+                                         exec_group_set_prefix="root.queue1") == 0
+    assert self._get_num_executor_groups(exec_group_set_prefix="root.queue1") == 1
+    assert self.coordinator.service.get_metric_value(
+      "cluster-membership.group-set.backends.total.root.queue1") == 1
+
+    # Add another executor to the previous group to make healthy again
+    self._add_executor_group("group", 2, num_executors=1,
+                             resource_pool="root.queue1", extra_args="-mem_limit=2g")
+    assert self._get_num_executor_groups(only_healthy=True,
+                                         exec_group_set_prefix="root.queue1") == 1
+    assert self._get_num_executor_groups(exec_group_set_prefix="root.queue1") == 1
+    assert self.coordinator.service.get_metric_value(
+      "cluster-membership.group-set.backends.total.root.queue1") == 2
+
+    # Add a healthy exec group in queue2 group set
+    self._add_executor_group("group", 1,
+                             resource_pool="root.queue2", extra_args="-mem_limit=2g")
+    assert self._get_num_executor_groups(only_healthy=True,
+                                         exec_group_set_prefix="root.queue2") == 1
+    assert self._get_num_executor_groups(exec_group_set_prefix="root.queue2") == 1
+    assert self.coordinator.service.get_metric_value(
+      "cluster-membership.group-set.backends.total.root.queue2") == 1
