@@ -20,8 +20,9 @@
 import pytest
 
 from tests.common.custom_cluster_test_suite import CustomClusterTestSuite
+from tests.util.event_processor_utils import EventProcessorUtils
 from tests.common.environ import HIVE_MAJOR_VERSION
-from tests.common.skip import SkipIfS3, SkipIfGCS
+from tests.common.skip import SkipIfS3, SkipIfGCS, SkipIfCOS
 from tests.common.test_dimensions import create_exec_option_dimension
 from tests.common.test_result_verifier import verify_query_result_is_equal
 from tests.util.filesystem_utils import get_fs_path
@@ -53,6 +54,7 @@ class TestParquetInterop(CustomClusterTestSuite):
 
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @pytest.mark.execute_serially
   @CustomClusterTestSuite.with_args("-convert_legacy_hive_parquet_utc_timestamps=true "
       "-hdfs_zone_info_zip=%s" % get_fs_path("/test-warehouse/tzdb/2017c.zip"))
@@ -96,8 +98,10 @@ class TestParquetInterop(CustomClusterTestSuite):
           .format(codec, external, hive_table, tblproperties, impala_table))
 
       # Make sure Impala's metadata is in sync.
-      if cluster_properties.is_catalog_v2_cluster():
-        self.wait_for_table_to_appear(unique_database, hive_table, timeout_s=10)
+      if cluster_properties.is_event_polling_enabled():
+        assert EventProcessorUtils.get_event_processor_status() == "ACTIVE"
+        EventProcessorUtils.wait_for_event_processing(self)
+        self.confirm_table_exists(unique_database, "t1_hive")
       else:
         self.client.execute("invalidate metadata {0}".format(hive_table))
 
