@@ -27,7 +27,7 @@ import time
 from multiprocessing.pool import ThreadPool
 
 from tests.common.custom_cluster_test_suite import CustomClusterTestSuite
-from tests.common.skip import (SkipIfHive2, SkipIfS3, SkipIfABFS, SkipIfGCS,
+from tests.common.skip import (SkipIfHive2, SkipIfS3, SkipIfABFS, SkipIfGCS, SkipIfCOS,
                                SkipIfADLS, SkipIfIsilon, SkipIfLocal)
 from tests.util.filesystem_utils import WAREHOUSE
 
@@ -273,8 +273,9 @@ class TestLocalCatalogRetries(CustomClusterTestSuite):
           q = random.choice(queries)
           attempt += 1
           try:
+            print 'Attempt', attempt, 'client', str(client)
             ret = self.execute_query_unchecked(client, q)
-          except Exception, e:
+          except Exception as e:
             if 'InconsistentMetadataFetchException' in str(e):
               with inconsistent_seen_lock:
                 inconsistent_seen[0] += 1
@@ -287,7 +288,8 @@ class TestLocalCatalogRetries(CustomClusterTestSuite):
         t.start()
       for t in threads:
         # When there are failures, they're observed quickly.
-        t.join(30)
+        # 600s is enough for 200 attempts.
+        t.join(600)
 
       assert failed_queries.empty(),\
           "Failed query count non zero: %s" % list(failed_queries.queue)
@@ -318,7 +320,8 @@ class TestLocalCatalogRetries(CustomClusterTestSuite):
 
   @pytest.mark.execute_serially
   @CustomClusterTestSuite.with_args(
-      impalad_args="--use_local_catalog=true --local_catalog_max_fetch_retries=0",
+      impalad_args="--use_local_catalog=true --local_catalog_max_fetch_retries=0"
+                   " --inject_latency_before_catalog_fetch_ms=500",
       catalogd_args="--catalog_topic_mode=minimal")
   def test_replan_limit(self):
     """
@@ -326,7 +329,8 @@ class TestLocalCatalogRetries(CustomClusterTestSuite):
     an inconsistent metadata exception when running concurrent reads/writes
     is seen. With the max retries set to 0, no retries are expected and with
     the concurrent read/write workload, an inconsistent metadata exception is
-    expected.
+    expected. Setting inject_latency_before_catalog_fetch_ms to increases the
+    possibility of a stale request which throws the expected exception.
     """
     queries = [
       'refresh functional.alltypes',
@@ -444,7 +448,6 @@ class TestLocalCatalogRetries(CustomClusterTestSuite):
     for i in t.imap_unordered(do_table, xrange(NUM_ITERS)):
       pass
 
-
 class TestObservability(CustomClusterTestSuite):
   def get_catalog_cache_metrics(self, impalad):
     """ Returns catalog cache metrics as a dict by scraping the json metrics page on the
@@ -536,6 +539,7 @@ class TestFullAcid(CustomClusterTestSuite):
   @SkipIfABFS.hive
   @SkipIfADLS.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfIsilon.hive
   @SkipIfLocal.hive
   @pytest.mark.execute_serially
@@ -558,39 +562,40 @@ class TestReusePartitionMetadata(CustomClusterTestSuite):
     # Make sure the table is unloaded either in catalogd or coordinator.
     self.execute_query("invalidate metadata %s.alltypes" % unique_database)
     # First time: misses all(24) partitions.
-    self.check_missing_partitions(unique_database, 24)
+    self.check_missing_partitions(unique_database, 24, 24)
     # Second time: hits all(24) partitions.
-    self.check_missing_partitions(unique_database, 0)
+    self.check_missing_partitions(unique_database, 0, 24)
 
     # Alter comment on the table. Partition metadata should be reusable.
     self.execute_query(
         "comment on table %s.alltypes is null" % unique_database)
-    self.check_missing_partitions(unique_database, 0)
+    self.check_missing_partitions(unique_database, 0, 24)
 
     # Refresh one partition. Although table version bumps, metadata cache of other
     # partitions should be reusable.
     self.execute_query(
         "refresh %s.alltypes partition(year=2009, month=1)" % unique_database)
-    self.check_missing_partitions(unique_database, 1)
+    self.check_missing_partitions(unique_database, 1, 24)
 
     # Drop one partition. Although table version bumps, metadata cache of existing
     # partitions should be reusable.
     self.execute_query(
         "alter table %s.alltypes drop partition(year=2009, month=1)" % unique_database)
-    self.check_missing_partitions(unique_database, 0)
+    self.check_missing_partitions(unique_database, 0, 23)
 
     # Add back one partition. The partition meta is loaded in catalogd but not the
     # coordinator. So we still miss its meta. For other partitions, we can reuse them.
     self.execute_query(
         "insert into %s.alltypes partition(year=2009, month=1) "
         "select 0,true,0,0,0,0,0,0,'a','a',NULL" % unique_database)
-    self.check_missing_partitions(unique_database, 1)
+    self.check_missing_partitions(unique_database, 1, 24)
 
-  def check_missing_partitions(self, unique_database, partition_misses):
+  def check_missing_partitions(self, unique_database, partition_misses, total_partitions):
     """Helper method for checking number of missing partitions while selecting
      all partitions of the alltypes table"""
     ret = self.execute_query_expect_success(
         self.client, "explain select count(*) from %s.alltypes" % unique_database)
+    assert ("partitions=%d" % total_partitions) in ret.get_data()
     match = re.search(r"CatalogFetch.Partitions.Misses: (\d+)", ret.runtime_profile)
     assert len(match.groups()) == 1
     assert match.group(1) == str(partition_misses)
