@@ -34,7 +34,7 @@ from tests.beeswax.impala_beeswax import ImpalaBeeswaxException
 from tests.common.impala_test_suite import ImpalaTestSuite
 from tests.common.custom_cluster_test_suite import CustomClusterTestSuite
 from tests.common.errors import Timeout
-from tests.common.skip import SkipIfEC, SkipIfBuildType, SkipIfGCS
+from tests.common.skip import SkipIfEC, SkipIfBuildType, SkipIfGCS, SkipIfCOS
 from tests.common.skip import SkipIfNotHdfsMinicluster
 
 # The BE krpc port of the impalad to simulate rpc or disk errors in tests.
@@ -76,6 +76,11 @@ class TestQueryRetries(CustomClusterTestSuite):
         union all
         select count(*) from functional.alltypes where bool_col = sleep(50)"""
 
+  # A simple count query with predicate. The predicate is needed so that the planner does
+  # not create the optimized count(star) query plan.
+  _count_query = "select count(*) from tpch_parquet.lineitem where l_orderkey < 50"
+  _count_query_result = "55"
+
   @classmethod
   def get_workload(cls):
     return 'functional-query'
@@ -122,6 +127,7 @@ class TestQueryRetries(CustomClusterTestSuite):
     # state since it asserts that no queries are in flight.
     self.client.close_query(handle)
     self.__validate_web_ui_state()
+    self.__validate_memz()
 
   @pytest.mark.execute_serially
   @CustomClusterTestSuite.with_args(
@@ -170,6 +176,7 @@ class TestQueryRetries(CustomClusterTestSuite):
     # state since it asserts that no queries are in flight.
     self.client.close_query(handle)
     self.__validate_web_ui_state()
+    self.__validate_memz()
 
     # Assert that the web ui shows all queries are complete.
     completed_queries = self.cluster.get_first_impalad().service.get_completed_queries()
@@ -232,6 +239,7 @@ class TestQueryRetries(CustomClusterTestSuite):
 
     # Validate the state of the Web UI.
     self.__validate_web_ui_state()
+    self.__validate_memz()
 
   @pytest.mark.execute_serially
   @CustomClusterTestSuite.with_args(
@@ -246,7 +254,7 @@ class TestQueryRetries(CustomClusterTestSuite):
 
     # Kill an impalad, and run a query. The query should be retried.
     killed_impalad = self.__kill_random_impalad()
-    query = "select count(*) from tpch_parquet.lineitem"
+    query = self._count_query
     handle = self.execute_query_async(query,
         query_options={'retry_failed_queries': 'true'})
     self.wait_for_state(handle, self.client.QUERY_STATES['FINISHED'], 60)
@@ -258,7 +266,7 @@ class TestQueryRetries(CustomClusterTestSuite):
     results = self.client.fetch(query, handle)
     assert results.success
     assert len(results.data) == 1
-    assert "6001215" in results.data[0]
+    assert self._count_query_result in results.data[0]
 
     # The runtime profile of the retried query.
     retried_runtime_profile = self.client.get_runtime_profile(handle)
@@ -282,8 +290,10 @@ class TestQueryRetries(CustomClusterTestSuite):
     # state since it asserts that no queries are in flight.
     self.client.close_query(handle)
     self.__validate_web_ui_state()
+    self.__validate_memz()
 
   @SkipIfGCS.jira(reason="IMPALA-10562")
+  @SkipIfCOS.jira(reason="IMPALA-10562")
   @SkipIfBuildType.not_dev_build
   @pytest.mark.execute_serially
   @CustomClusterTestSuite.with_args(
@@ -305,7 +315,7 @@ class TestQueryRetries(CustomClusterTestSuite):
     # and the query should be retried. Add delay before admission so that the 2nd node
     # is removed from the blacklist before scheduler makes schedule for the retried
     # query.
-    query = "select count(*) from tpch_parquet.lineitem"
+    query = self._count_query
     handle = self.execute_query_async(query,
         query_options={'retry_failed_queries': 'true',
                        'debug_action': 'AC_BEFORE_ADMISSION:SLEEP@18000'})
@@ -318,7 +328,7 @@ class TestQueryRetries(CustomClusterTestSuite):
     results = self.client.fetch(query, handle)
     assert results.success
     assert len(results.data) == 1
-    assert "6001215" in results.data[0]
+    assert self._count_query_result in results.data[0]
 
     # The runtime profile of the retried query.
     retried_runtime_profile = self.client.get_runtime_profile(handle)
@@ -350,6 +360,7 @@ class TestQueryRetries(CustomClusterTestSuite):
     self.__validate_web_ui_state()
 
   @SkipIfGCS.jira(reason="IMPALA-10562")
+  @SkipIfCOS.jira(reason="IMPALA-10562")
   @SkipIfBuildType.not_dev_build
   @pytest.mark.execute_serially
   @CustomClusterTestSuite.with_args(
@@ -368,7 +379,7 @@ class TestQueryRetries(CustomClusterTestSuite):
     rpc_not_accessible_impalad = self.cluster.impalads[1]
     assert rpc_not_accessible_impalad.service.krpc_port == FAILED_KRPC_PORT
 
-    query = "select count(*) from tpch_parquet.lineitem"
+    query = self._count_query
     handle = self.execute_query_async(query,
         query_options={'retry_failed_queries': 'true',
                        'debug_action': 'AC_BEFORE_ADMISSION:SLEEP@18000'})
@@ -472,6 +483,7 @@ class TestQueryRetries(CustomClusterTestSuite):
     # Validate the state of the web ui. The query must be closed before validating the
     # state since it asserts that no queries are in flight.
     self.__validate_web_ui_state()
+    self.__validate_memz()
 
   @pytest.mark.execute_serially
   def test_retry_fetched_rows(self):
@@ -690,7 +702,7 @@ class TestQueryRetries(CustomClusterTestSuite):
 
     # Kill an impalad, and run a query. The query should be retried.
     self.cluster.impalads[1].kill()
-    query = "select count(*) from tpch_parquet.lineitem"
+    query = self._count_query
     handle = self.execute_query_async(query,
         query_options={'retry_failed_queries': 'true'})
     self.wait_for_state(handle, self.client.QUERY_STATES['FINISHED'], 60)
@@ -715,6 +727,7 @@ class TestQueryRetries(CustomClusterTestSuite):
       assert False
     except Exception, e:
         assert "Cancelled" in str(e)
+    self.__validate_memz()
 
   @pytest.mark.execute_serially
   @CustomClusterTestSuite.with_args(
@@ -728,7 +741,7 @@ class TestQueryRetries(CustomClusterTestSuite):
 
     # Kill an impalad, and run a query. The query should be retried.
     self.cluster.impalads[1].kill()
-    query = "select count(*) from tpch_parquet.lineitem"
+    query = self._count_query
     handle = self.execute_query_async(query,
         query_options={'retry_failed_queries': 'true'})
     self.__wait_until_retry_state(handle, 'RETRYING')
@@ -744,6 +757,34 @@ class TestQueryRetries(CustomClusterTestSuite):
     self.client.close_query(handle)
     time.sleep(2)
     assert self.cluster.impalads[0].get_pid() is not None, "Coordinator crashed"
+    self.__validate_memz()
+
+  @pytest.mark.execute_serially
+  @CustomClusterTestSuite.with_args(
+      impalad_args="--debug_actions=RETRY_DELAY_GET_QUERY_DRIVER:SLEEP@2000",
+      statestored_args="--statestore_heartbeat_frequency_ms=60000")
+  def test_retry_query_close_before_getting_query_driver(self):
+    """Trigger a query retry, and then close the retried query before getting
+    the query driver. Validate that it doesn't crash the impalad.
+    Set a really high statestore heartbeat frequency so that killed impalads are not
+    removed from the cluster membership."""
+
+    # Kill an impalad, and run a query. The query should be retried.
+    self.cluster.impalads[1].kill()
+    query = self._count_query
+    handle = self.execute_query_async(query,
+        query_options={'retry_failed_queries': 'true'})
+
+    time.sleep(1)
+    # close the query
+    self.client.close_query(handle)
+
+    time.sleep(2)
+    impala_service = self.cluster.get_first_impalad().service
+    self.assert_eventually(60, 0.1,
+        lambda: impala_service.get_num_in_flight_queries() == 0,
+        lambda: "in-flight queries: %d" % impala_service.get_num_in_flight_queries())
+    self.__validate_memz()
 
   @pytest.mark.execute_serially
   @CustomClusterTestSuite.with_args(
@@ -754,7 +795,7 @@ class TestQueryRetries(CustomClusterTestSuite):
 
     # Kill an impalad, and run a query. The query should be retried.
     self.cluster.impalads[1].kill()
-    query = "select count(*) from tpch_parquet.lineitem"
+    query = self._count_query
     self.hs2_client.set_configuration({'retry_failed_queries': 'true'})
     self.hs2_client.set_configuration_option('impala.resultset.cache.size', '1024')
     self.hs2_client.execute_async(query)
@@ -770,6 +811,7 @@ class TestQueryRetries(CustomClusterTestSuite):
     self.assert_eventually(60, 0.1,
         lambda: impala_service.get_num_in_flight_queries() == 0,
         lambda: "in-flight queries: %d" % impala_service.get_num_in_flight_queries())
+    self.__validate_memz()
 
   @pytest.mark.execute_serially
   @CustomClusterTestSuite.with_args(
@@ -780,7 +822,7 @@ class TestQueryRetries(CustomClusterTestSuite):
 
     # Kill an impalad, and run a query. The query should be retried.
     self.cluster.impalads[1].kill()
-    query = "select count(*) from tpch_parquet.lineitem"
+    query = self._count_query
     self.execute_query_async(query, query_options={'retry_failed_queries': 'true'})
     # The number of in-flight queries is 0 at the beginning, then 1 when the original
     # query is submitted. It's 2 when the retried query is registered. Although the retry
@@ -794,6 +836,7 @@ class TestQueryRetries(CustomClusterTestSuite):
     self.assert_eventually(60, 0.1,
         lambda: impala_service.get_num_in_flight_queries() == 0,
         lambda: "in-flight queries: %d" % impala_service.get_num_in_flight_queries())
+    self.__validate_memz()
 
   @pytest.mark.execute_serially
   @CustomClusterTestSuite.with_args(
@@ -809,7 +852,7 @@ class TestQueryRetries(CustomClusterTestSuite):
 
     # Kill an impalad, and run a query. The query should be retried.
     self.cluster.impalads[1].kill()
-    query = "select count(*) from tpch_parquet.lineitem"
+    query = self._count_query
     handle = self.execute_query_async(query,
         query_options={'retry_failed_queries': 'true', 'query_timeout_s': '1'})
     self.wait_for_state(handle, self.client.QUERY_STATES['EXCEPTION'], 60)
@@ -848,7 +891,7 @@ class TestQueryRetries(CustomClusterTestSuite):
 
     # Kill an impalad, and run a query. The query should be retried.
     self.cluster.impalads[1].kill()
-    query = "select count(*) from tpch_parquet.lineitem"
+    query = self._count_query
     client = self.cluster.get_first_impalad().service.create_beeswax_client()
     client.set_configuration({'retry_failed_queries': 'true'})
     handle = client.execute_async(query)
@@ -878,7 +921,7 @@ class TestQueryRetries(CustomClusterTestSuite):
     """Test query retries with the HS2 protocol. Enable the results set cache as well and
     test that query retries work with the results cache."""
     self.cluster.impalads[1].kill()
-    query = "select count(*) from tpch_parquet.lineitem"
+    query = self._count_query
     self.hs2_client.set_configuration({'retry_failed_queries': 'true'})
     self.hs2_client.set_configuration_option('impala.resultset.cache.size', '1024')
     handle = self.hs2_client.execute_async(query)
@@ -887,7 +930,7 @@ class TestQueryRetries(CustomClusterTestSuite):
     results = self.hs2_client.fetch(query, handle)
     assert results.success
     assert len(results.data) == 1
-    assert int(results.data[0]) == 6001215
+    assert results.data[0] == self._count_query_result
 
     # Validate the live exec summary.
     retried_query_id = \
@@ -1085,6 +1128,17 @@ class TestQueryRetries(CustomClusterTestSuite):
           return query_id_search.group(1)
     return None
 
+  def __exist_queries_in_web_ui_memz(self):
+    memz_breakdown = self.cluster.get_first_impalad() \
+      .service.get_debug_webpage_json('memz')['detailed']
+    query = re.compile("Query\([0-9a-f]{16}:[0-9a-f]{16}")
+    return query.search(memz_breakdown)
+
+  def __validate_memz(self):
+    # Validate that all queries are released
+    self.assert_eventually(60, 0.1,
+        lambda: self.__exist_queries_in_web_ui_memz() is None)
+
 
 # Tests that verify the query-retries are properly triggered by disk IO failure.
 # Coordinator adds an executor node to its blacklist if that node reports query
