@@ -17,7 +17,7 @@
 
 from tests.common.impala_test_suite import ImpalaTestSuite
 from tests.common.skip import (SkipIfS3, SkipIfABFS, SkipIfADLS, SkipIfLocal,
-                               SkipIfGCS, SkipIfCatalogV2)
+                               SkipIfGCS, SkipIfCOS, SkipIfCatalogV2)
 from tests.common.test_dimensions import (
     create_single_exec_option_dimension,
     create_uncompressed_text_dimension)
@@ -30,6 +30,7 @@ TBL_LOC = '%s/%s' % (WAREHOUSE, TEST_TBL)
 
 @SkipIfS3.hdfs_acls
 @SkipIfGCS.hdfs_acls
+@SkipIfCOS.hdfs_acls
 @SkipIfABFS.hdfs_acls
 @SkipIfADLS.hdfs_acls
 @SkipIfLocal.hdfs_client
