@@ -21,11 +21,12 @@ import os
 import pytest
 import time
 
-from hive_metastore.ttypes import CommitTxnRequest, OpenTxnRequest
+from hive_metastore.ttypes import CommitTxnRequest, LockType, OpenTxnRequest
 from subprocess import check_call
 from tests.common.impala_test_suite import ImpalaTestSuite
 from tests.common.skip import (SkipIf, SkipIfHive2, SkipIfCatalogV2, SkipIfS3, SkipIfABFS,
-                               SkipIfADLS, SkipIfIsilon, SkipIfGCS, SkipIfLocal)
+                               SkipIfADLS, SkipIfIsilon, SkipIfGCS, SkipIfCOS,
+                               SkipIfLocal)
 from tests.common.test_dimensions import create_single_exec_option_dimension
 from tests.util.acid_txn import AcidTxn
 
@@ -47,6 +48,7 @@ class TestAcid(ImpalaTestSuite):
   @SkipIfHive2.acid
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfABFS.hive
   @SkipIfADLS.hive
   @SkipIfIsilon.hive
@@ -65,6 +67,7 @@ class TestAcid(ImpalaTestSuite):
   @SkipIfHive2.acid
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfABFS.hive
   @SkipIfADLS.hive
   @SkipIfIsilon.hive
@@ -75,6 +78,7 @@ class TestAcid(ImpalaTestSuite):
   @SkipIfHive2.acid
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfABFS.hive
   @SkipIfADLS.hive
   @SkipIfIsilon.hive
@@ -85,6 +89,7 @@ class TestAcid(ImpalaTestSuite):
   @SkipIfHive2.acid
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfABFS.hive
   @SkipIfADLS.hive
   @SkipIfIsilon.hive
@@ -97,6 +102,7 @@ class TestAcid(ImpalaTestSuite):
   @SkipIfHive2.acid
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfABFS.hive
   @SkipIfADLS.hive
   @SkipIfIsilon.hive
@@ -107,6 +113,7 @@ class TestAcid(ImpalaTestSuite):
   @SkipIfHive2.acid
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfABFS.hive
   @SkipIfADLS.hive
   @SkipIfIsilon.hive
@@ -127,6 +134,7 @@ class TestAcid(ImpalaTestSuite):
   @SkipIfCatalogV2.hms_event_polling_enabled()
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfABFS.hive
   @SkipIfADLS.hive
   @SkipIfIsilon.hive
@@ -141,6 +149,7 @@ class TestAcid(ImpalaTestSuite):
   @SkipIfHive2.acid
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfABFS.hive
   @SkipIfADLS.hive
   @SkipIfIsilon.hive
@@ -164,6 +173,7 @@ class TestAcid(ImpalaTestSuite):
   @SkipIfHive2.acid
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfABFS.hive
   @SkipIfADLS.hive
   @SkipIfIsilon.hive
@@ -181,6 +191,7 @@ class TestAcid(ImpalaTestSuite):
 
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfABFS.hive
   @SkipIfADLS.hive
   @SkipIfIsilon.hive
@@ -200,6 +211,7 @@ class TestAcid(ImpalaTestSuite):
   @SkipIfHive2.acid
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfABFS.hive
   @SkipIfADLS.hive
   @SkipIfIsilon.hive
@@ -215,6 +227,7 @@ class TestAcid(ImpalaTestSuite):
   @SkipIfHive2.acid
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfABFS.hive
   @SkipIfADLS.hive
   @SkipIfIsilon.hive
@@ -291,9 +304,56 @@ class TestAcid(ImpalaTestSuite):
     commit_req.txnid = txn_id
     return self.hive_client.commit_txn(commit_req)
 
+  @SkipIfS3.hive
+  @SkipIfGCS.hive
+  @SkipIfCOS.hive
+  @SkipIfABFS.hive
+  @SkipIfADLS.hive
+  @SkipIfIsilon.hive
+  @SkipIfLocal.hive
+  def test_lock_timings(self, vector, unique_database):
+    def elapsed_time_for_query(query):
+      t_start = time.time()
+      self.execute_query_expect_failure(self.client, query)
+      return time.time() - t_start
+
+    tbl_name = "test_lock"
+    tbl = "{0}.{1}".format(unique_database, tbl_name)
+    self.execute_query("create table {} (i int) tblproperties"
+        "('transactional'='true',"
+        "'transactional_properties'='insert_only')".format(tbl))
+    acid_util = AcidTxn(self.hive_client)
+    lock_resp = acid_util.lock(0, unique_database, tbl_name, LockType.EXCLUSIVE)
+    try:
+      if self.exploration_strategy() == 'exhaustive':
+        elapsed = elapsed_time_for_query("insert into {} values (1)".format(tbl))
+        assert elapsed > 300 and elapsed < 310
+      self.execute_query("set lock_max_wait_time_s=20")
+      elapsed = elapsed_time_for_query("insert into {} values (1)".format(tbl))
+      assert elapsed > 20 and elapsed < 28
+
+      self.execute_query("set lock_max_wait_time_s=0")
+      elapsed = elapsed_time_for_query("insert into {} values (1)".format(tbl))
+      assert elapsed < 8
+
+      self.execute_query("set lock_max_wait_time_s=10")
+      elapsed = elapsed_time_for_query("insert into {} values (1)".format(tbl))
+      assert elapsed > 10 and elapsed < 18
+
+      self.execute_query("set lock_max_wait_time_s=2")
+      elapsed = elapsed_time_for_query("truncate table {}".format(tbl))
+      assert elapsed > 2 and elapsed < 10
+
+      self.execute_query("set lock_max_wait_time_s=5")
+      elapsed = elapsed_time_for_query("drop table {}".format(tbl))
+      assert elapsed > 5 and elapsed < 13
+    finally:
+      acid_util.unlock(lock_resp.lockid)
+
   @SkipIfHive2.acid
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfABFS.hive
   @SkipIfADLS.hive
   @SkipIfIsilon.hive
