@@ -20,7 +20,8 @@
 import pytest
 from tests.common.file_utils import create_table_and_copy_files
 from tests.common.impala_test_suite import ImpalaTestSuite
-from tests.common.skip import SkipIfS3, SkipIfABFS, SkipIfADLS, SkipIfLocal, SkipIfGCS
+from tests.common.skip import (SkipIfS3, SkipIfABFS, SkipIfADLS, SkipIfLocal, SkipIfGCS,
+                               SkipIfCOS)
 from tests.common.test_dimensions import (create_exec_option_dimension_from_dict,
     create_client_protocol_dimension, hs2_parquet_constraint)
 from tests.shell.util import ImpalaShell
@@ -56,8 +57,7 @@ class TestDateQueries(ImpalaTestSuite):
       #  - Hive2 uses Julian Calendar for writing dates before 1582-10-15, whereas Impala
       #    uses proleptic Gregorian Calendar. This affects the results Impala gets when
       #    querying avro tables written by Hive2.
-      #  - Since HIVE-22589, Hive3 also uses Julian Calendar for dates before 1582-10-15
-      #    by default.
+      #  - Hive3 on the other hand uses proleptic Gregorian Calendar to write dates.
       self.run_test_case('QueryTest/avro_date', vector)
     else:
       self.run_test_case('QueryTest/date', vector)
@@ -72,6 +72,7 @@ class TestDateQueries(ImpalaTestSuite):
 
   @SkipIfS3.qualified_path
   @SkipIfGCS.qualified_path
+  @SkipIfCOS.qualified_path
   @SkipIfABFS.qualified_path
   @SkipIfADLS.qualified_path
   @SkipIfLocal.qualified_path
