@@ -27,6 +27,7 @@ from tests.common.skip import (
     SkipIfIsilon,
     SkipIfS3,
     SkipIfGCS,
+    SkipIfCOS,
     SkipIfABFS,
     SkipIfADLS,
     SkipIfEC,
@@ -34,6 +35,9 @@ from tests.common.skip import (
     SkipIfLocal,
     SkipIfNotHdfsMinicluster
     )
+from tests.common.test_dimensions import (create_exec_option_dimension,
+    create_exec_option_dimension_from_dict, create_client_protocol_dimension,
+    create_orc_dimension, orc_schema_resolution_constraint)
 from tests.common.test_vector import ImpalaTestDimension
 from tests.util.filesystem_utils import WAREHOUSE, get_fs_path, IS_HDFS
 
@@ -44,6 +48,13 @@ class TestNestedTypes(ImpalaTestSuite):
   def get_workload(self):
     return 'functional-query'
 
+  @staticmethod
+  def orc_schema_resolution_constraint(vector):
+    """ Constraint to use multiple orc_schema_resolution only in case of orc files"""
+    file_format = vector.get_value('table_format').file_format
+    orc_schema_resolution = vector.get_value('orc_schema_resolution')
+    return file_format == 'orc' or orc_schema_resolution == 0
+
   @classmethod
   def add_test_dimensions(cls):
     super(TestNestedTypes, cls).add_test_dimensions()
@@ -51,6 +62,8 @@ class TestNestedTypes(ImpalaTestSuite):
         v.get_value('table_format').file_format in ['parquet', 'orc'])
     cls.ImpalaTestMatrix.add_dimension(
         ImpalaTestDimension('mt_dop', 0, 2))
+    cls.ImpalaTestMatrix.add_dimension(ImpalaTestDimension('orc_schema_resolution', 0, 1))
+    cls.ImpalaTestMatrix.add_constraint(orc_schema_resolution_constraint)
 
   def test_scanner_basic(self, vector):
     """Queries that do not materialize arrays."""
@@ -112,6 +125,148 @@ class TestNestedTypes(ImpalaTestSuite):
                        use_db='tpch_nested' + db_suffix)
 
 
+class TestNestedStructsInSelectList(ImpalaTestSuite):
+  """Functional tests for nested structs provided in the select list."""
+  @classmethod
+  def get_workload(self):
+    return 'functional-query'
+
+  @classmethod
+  def add_test_dimensions(cls):
+    super(TestNestedStructsInSelectList, cls).add_test_dimensions()
+    cls.ImpalaTestMatrix.add_constraint(lambda v:
+        v.get_value('table_format').file_format in ['parquet', 'orc'])
+    cls.ImpalaTestMatrix.add_dimension(
+        ImpalaTestDimension('mt_dop', 0, 2))
+    cls.ImpalaTestMatrix.add_dimension(
+        create_exec_option_dimension_from_dict({
+            'disable_codegen': ['False', 'True']}))
+    cls.ImpalaTestMatrix.add_dimension(create_client_protocol_dimension())
+    cls.ImpalaTestMatrix.add_constraint(lambda v:
+        v.get_value('protocol') == 'hs2')
+    cls.ImpalaTestMatrix.add_dimension(ImpalaTestDimension('orc_schema_resolution', 0, 1))
+    cls.ImpalaTestMatrix.add_constraint(orc_schema_resolution_constraint)
+
+  def test_struct_in_select_list(self, vector, unique_database):
+    """Queries where a struct column is in the select list"""
+    if vector.get_value('table_format').file_format == 'parquet':
+      pytest.skip()
+    if vector.get_value('exec_option')['disable_codegen'] == 'False':
+      pytest.skip()
+    self.run_test_case('QueryTest/struct-in-select-list', vector, unique_database)
+
+  def test_nested_struct_in_select_list(self, vector, unique_database):
+    """Queries where a nested struct column is in the select list"""
+    if vector.get_value('table_format').file_format == 'parquet':
+      pytest.skip()
+    if vector.get_value('exec_option')['disable_codegen'] == 'False':
+      pytest.skip()
+    self.run_test_case('QueryTest/nested-struct-in-select-list', vector, unique_database)
+
+
+class TestNestedTArraysInSelectList(ImpalaTestSuite):
+  """Functional tests for nested arrays provided in the select list."""
+  @classmethod
+  def get_workload(self):
+    return 'functional-query'
+
+  @classmethod
+  def add_test_dimensions(cls):
+    super(TestNestedTArraysInSelectList, cls).add_test_dimensions()
+    cls.ImpalaTestMatrix.add_constraint(lambda v:
+        v.get_value('table_format').file_format in ['parquet', 'orc'])
+    cls.ImpalaTestMatrix.add_dimension(
+        ImpalaTestDimension('mt_dop', 0, 2))
+    cls.ImpalaTestMatrix.add_dimension(
+        create_exec_option_dimension_from_dict({
+            'disable_codegen': ['False', 'True']}))
+    cls.ImpalaTestMatrix.add_dimension(create_client_protocol_dimension())
+    cls.ImpalaTestMatrix.add_dimension(ImpalaTestDimension('orc_schema_resolution', 0, 1))
+    cls.ImpalaTestMatrix.add_constraint(orc_schema_resolution_constraint)
+
+  def test_array_in_select_list(self, vector, unique_database):
+    """Queries where an array column is in the select list"""
+    self.run_test_case('QueryTest/nested-array-in-select-list', vector)
+
+
+# Moved this to a separate test class from TestNestedTypesInSelectList because this needs
+# a narrower test vector.
+class TestNestedTypesInSelectListWithBeeswax(ImpalaTestSuite):
+  """Functional tests for nested types provided in the select list."""
+  @classmethod
+  def get_workload(self):
+    return 'functional-query'
+
+  @classmethod
+  def add_test_dimensions(cls):
+    super(TestNestedTypesInSelectListWithBeeswax, cls).add_test_dimensions()
+    cls.ImpalaTestMatrix.add_dimension(create_client_protocol_dimension())
+    cls.ImpalaTestMatrix.add_constraint(lambda v:
+        v.get_value('protocol') == 'beeswax')
+    cls.ImpalaTestMatrix.add_dimension(create_orc_dimension(cls.get_workload()))
+    cls.ImpalaTestMatrix.add_dimension(create_exec_option_dimension(
+        disable_codegen_options=[True]))
+
+  def test_struct_with_beeswax(self, vector):
+    expected_err = "Returning struct types is not supported through the beeswax " + \
+        "interface"
+    err = self.execute_query_expect_failure(self.client,
+        "select tiny_struct from functional_orc_def.complextypes_structs",
+        vector.get_value('exec_option'))
+    assert expected_err in str(err)
+
+
+class TestComputeStatsWithNestedTypes(ImpalaTestSuite):
+  """Functional tests for running compute stats on tables that have nested types in the
+  columns."""
+  @classmethod
+  def get_workload(self):
+    return 'functional-query'
+
+  @classmethod
+  def add_test_dimensions(cls):
+    super(TestComputeStatsWithNestedTypes, cls).add_test_dimensions()
+    cls.ImpalaTestMatrix.add_constraint(lambda v:
+        v.get_value('table_format').file_format in ['parquet', 'orc'])
+    cls.ImpalaTestMatrix.add_dimension(ImpalaTestDimension('orc_schema_resolution', 0, 1))
+    cls.ImpalaTestMatrix.add_constraint(orc_schema_resolution_constraint)
+
+  def test_compute_stats_with_structs(self, vector):
+    """COMPUTE STATS and SHOW COLUMN STATS for tables with structs"""
+    self.run_test_case('QueryTest/compute-stats-with-structs', vector)
+
+
+class TestZippingUnnest(ImpalaTestSuite):
+  """Functional tests for zipping unnest functionality."""
+  @classmethod
+  def get_workload(self):
+    return 'functional-query'
+
+  @classmethod
+  def add_test_dimensions(cls):
+    super(TestZippingUnnest, cls).add_test_dimensions()
+    cls.ImpalaTestMatrix.add_constraint(lambda v:
+        v.get_value('table_format').file_format in ['parquet', 'orc'])
+    cls.ImpalaTestMatrix.add_dimension(ImpalaTestDimension('orc_schema_resolution', 0, 1))
+    cls.ImpalaTestMatrix.add_constraint(orc_schema_resolution_constraint)
+
+  def test_zipping_unnest_in_from_clause(self, vector):
+    """Queries where zipping unnest is executed by providing UNNEST() in the from clause.
+    """
+    self.run_test_case('QueryTest/zipping-unnest-in-from-clause', vector)
+
+  def test_zipping_unnest_in_select_list(self, vector):
+    """Queries where zipping unnest is executed by providing UNNEST() in the select list.
+    """
+    self.run_test_case('QueryTest/zipping-unnest-in-select-list', vector)
+
+  def test_zipping_unnest_from_view(self, vector, unique_database):
+    """Zipping unnest queries where views are involved."""
+    if vector.get_value('table_format').file_format == 'orc':
+      pytest.skip('No need to run this test for multiple file formats.')
+    self.run_test_case('QueryTest/zipping-unnest-from-view', vector,
+        use_db=unique_database)
+
 class TestNestedTypesNoMtDop(ImpalaTestSuite):
   """Functional tests for nested types that do not need to be run with mt_dop > 0."""
   @classmethod
@@ -123,6 +278,8 @@ class TestNestedTypesNoMtDop(ImpalaTestSuite):
     super(TestNestedTypesNoMtDop, cls).add_test_dimensions()
     cls.ImpalaTestMatrix.add_constraint(lambda v:
         v.get_value('table_format').file_format in ['parquet', 'orc'])
+    cls.ImpalaTestMatrix.add_dimension(ImpalaTestDimension('orc_schema_resolution', 0, 1))
+    cls.ImpalaTestMatrix.add_constraint(orc_schema_resolution_constraint)
 
   def test_tpch(self, vector):
     """Queries over the larger nested TPCH dataset."""
@@ -155,13 +312,13 @@ class TestNestedTypesNoMtDop(ImpalaTestSuite):
   def test_parquet_stats(self, vector):
     """Queries that test evaluation of Parquet row group statistics."""
     if vector.get_value('table_format').file_format == 'orc':
-      pytest.skip('Predicate push down on ORC stripe statistics is not supported' +
-                  '(IMPALA-6505)')
+      pytest.skip('This test is specific to Parquet')
     self.run_test_case('QueryTest/nested-types-parquet-stats', vector)
 
   @SkipIfIsilon.hive
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfABFS.hive
   @SkipIfADLS.hive
   @SkipIfLocal.hive
@@ -221,6 +378,7 @@ class TestNestedTypesNoMtDop(ImpalaTestSuite):
   @SkipIfLocal.hive
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfHive2.acid
   def test_partitioned_table_acid(self, vector, unique_database):
     """IMPALA-6370: Test that a partitioned table with nested types can be scanned."""
@@ -679,6 +837,8 @@ class TestMaxNestingDepth(ImpalaTestSuite):
     super(TestMaxNestingDepth, cls).add_test_dimensions()
     cls.ImpalaTestMatrix.add_constraint(lambda v:
         v.get_value('table_format').file_format in ['parquet', 'orc'])
+    cls.ImpalaTestMatrix.add_dimension(ImpalaTestDimension('orc_schema_resolution', 0, 1))
+    cls.ImpalaTestMatrix.add_constraint(orc_schema_resolution_constraint)
 
   def test_max_nesting_depth(self, vector, unique_database):
     """Tests that Impala can scan Parquet and ORC files having complex types of
@@ -723,6 +883,7 @@ class TestMaxNestingDepth(ImpalaTestSuite):
   @SkipIfIsilon.hive
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfABFS.hive
   @SkipIfADLS.hive
   @SkipIfLocal.hive
