@@ -17,6 +17,7 @@
 #
 
 from copy import deepcopy
+import os
 import pytest
 import re
 import time
@@ -75,9 +76,7 @@ class TestRuntimeFilters(ImpalaTestSuite):
   def test_basic_filters(self, vector):
     new_vector = deepcopy(vector)
     new_vector.get_value('exec_option')['mt_dop'] = vector.get_value('mt_dop')
-    if 'kudu' in str(vector.get_value('table_format')):
-      self.execute_query("SET ENABLED_RUNTIME_FILTER_TYPES=ALL")
-    self.run_test_case('QueryTest/runtime_filters', vector,
+    self.run_test_case('QueryTest/runtime_filters', new_vector,
         test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS' : str(WAIT_TIME_MS)})
 
   def test_wait_time(self, vector):
@@ -189,10 +188,26 @@ class TestBloomFilters(ImpalaTestSuite):
       add_exec_option_dimension(cls, "async_codegen", 1)
 
   def test_bloom_filters(self, vector):
-    if 'kudu' in str(vector.get_value('table_format')):
-      self.execute_query("SET ENABLED_RUNTIME_FILTER_TYPES=BLOOM")
+    self.execute_query("SET ENABLED_RUNTIME_FILTER_TYPES=BLOOM")
     self.run_test_case('QueryTest/bloom_filters', vector)
 
+  def test_iceberg_dictionary_runtime_filter(self, vector, unique_database):
+    self.execute_query("SET ENABLED_RUNTIME_FILTER_TYPES=BLOOM")
+    self.execute_query("SET PARQUET_READ_STATISTICS=false;")
+    self.execute_query("SET MINMAX_FILTER_THRESHOLD = 0.0;")
+    self.execute_query("SET MINMAX_FILTER_SORTED_COLUMNS=FALSE;")
+    self.execute_query("SET MINMAX_FILTER_PARTITION_COLUMNS=FALSE;")
+    self.run_test_case('QueryTest/iceberg-dictionary-runtime-filter', vector,
+      unique_database, test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS)})
+
+  def test_parquet_dictionary_runtime_filter(self, vector, unique_database):
+    self.execute_query("SET ENABLED_RUNTIME_FILTER_TYPES=BLOOM")
+    self.execute_query("SET PARQUET_READ_STATISTICS=false;")
+    self.execute_query("SET MINMAX_FILTER_THRESHOLD = 0.0;")
+    self.execute_query("SET MINMAX_FILTER_SORTED_COLUMNS=FALSE;")
+    self.execute_query("SET MINMAX_FILTER_PARTITION_COLUMNS=FALSE;")
+    self.run_test_case('QueryTest/parquet-dictionary-runtime-filter', vector,
+      unique_database, test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS)})
 
 @SkipIfLocal.multiple_impalad
 class TestMinMaxFilters(ImpalaTestSuite):
@@ -286,10 +301,66 @@ class TestOverlapMinMaxFilters(ImpalaTestSuite):
 
   def test_overlap_min_max_filters(self, vector, unique_database):
     self.execute_query("SET MINMAX_FILTER_THRESHOLD=0.5")
+    # disable min/max filters on partition columns and allow min/max filters
+    # on sorted columns (by default).
+    self.execute_query("SET MINMAX_FILTER_PARTITION_COLUMNS=false")
     self.run_test_case('QueryTest/overlap_min_max_filters', vector, unique_database,
         test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS)})
+    self.execute_query("SET MINMAX_FILTER_PARTITION_COLUMNS=true")
 
-# Apply both Bloom filter and Minmax filters
+  def test_overlap_min_max_filters_on_sorted_columns(self, vector, unique_database):
+    self.run_test_case('QueryTest/overlap_min_max_filters_on_sorted_columns', vector,
+                       unique_database,
+        test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS)})
+
+  def test_overlap_min_max_filters_on_partition_columns(self, vector, unique_database):
+    self.run_test_case('QueryTest/overlap_min_max_filters_on_partition_columns', vector,
+                       unique_database,
+        test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS)})
+
+  @SkipIfLocal.hdfs_client
+  def test_partition_column_in_parquet_data_file(self, vector, unique_database):
+    """IMPALA-11147: Test that runtime min/max filters still work on data files that
+    contain the partitioning columns."""
+    tbl_name = "part_col_in_data_file"
+    self.execute_query("CREATE TABLE {0}.{1} (i INT) PARTITIONED BY (d DATE) "
+                       "STORED AS PARQUET".format(unique_database, tbl_name))
+    tbl_loc = self._get_table_location("{0}.{1}".format(unique_database, tbl_name),
+        vector)
+    self.filesystem_client.make_dir(tbl_loc[tbl_loc.find('test-warehouse'):] +
+        '/d=2022-02-22/')
+    self.filesystem_client.copy_from_local(os.environ['IMPALA_HOME'] +
+        '/testdata/data/partition_col_in_parquet.parquet', tbl_loc + '/d=2022-02-22/')
+    self.execute_query("ALTER TABLE {0}.{1} RECOVER PARTITIONS".format(
+        unique_database, tbl_name))
+    self.execute_query("SET PARQUET_FALLBACK_SCHEMA_RESOLUTION=NAME")
+    self.execute_query("SET ENABLED_RUNTIME_FILTER_TYPES=MIN_MAX")
+    self.execute_query("select * from {0}.{1} t1, {0}.{1} t2 where t1.d=t2.d and t2.i=2".
+        format(unique_database, tbl_name))
+
+
+class TestInListFilters(ImpalaTestSuite):
+  @classmethod
+  def get_workload(cls):
+    return 'functional-query'
+
+  @classmethod
+  def add_test_dimensions(cls):
+    super(TestInListFilters, cls).add_test_dimensions()
+    # Currently, IN-list filters are only implemented for orc.
+    cls.ImpalaTestMatrix.add_constraint(
+        lambda v: v.get_value('table_format').file_format in ['orc'])
+    # Enable query option ASYNC_CODEGEN for slow build
+    if build_runs_slowly:
+      add_exec_option_dimension(cls, "async_codegen", 1)
+
+  def test_in_list_filters(self, vector):
+    vector.get_value('exec_option')['enabled_runtime_filter_types'] = 'in_list'
+    vector.get_value('exec_option')['runtime_filter_wait_time_ms'] = WAIT_TIME_MS
+    self.run_test_case('QueryTest/in_list_filters', vector)
+
+
+# Apply Bloom filter, Minmax filter and IN-list filters
 class TestAllRuntimeFilters(ImpalaTestSuite):
   @classmethod
   def get_workload(cls):
@@ -307,6 +378,9 @@ class TestAllRuntimeFilters(ImpalaTestSuite):
 
   def test_all_runtime_filters(self, vector):
     self.execute_query("SET ENABLED_RUNTIME_FILTER_TYPES=ALL")
+    # Disable generating min/max filters for sorted columns
+    self.execute_query("SET MINMAX_FILTER_SORTED_COLUMNS=false")
+    self.execute_query("SET MINMAX_FILTER_PARTITION_COLUMNS=false")
     self.run_test_case('QueryTest/all_runtime_filters', vector,
                        test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS)})
 
@@ -337,6 +411,10 @@ class TestRuntimeRowFilters(ImpalaTestSuite):
   def test_row_filters(self, vector):
     new_vector = deepcopy(vector)
     new_vector.get_value('exec_option')['mt_dop'] = vector.get_value('mt_dop')
+    # Disable generating min/max filters for sorted columns
+    self.execute_query("SET MINMAX_FILTER_SORTED_COLUMNS=false")
+    self.execute_query("SET MINMAX_FILTER_PARTITION_COLUMNS=false")
+    self.execute_query("SET PARQUET_DICTIONARY_RUNTIME_FILTER_ENTRY_LIMIT=0")
     self.run_test_case('QueryTest/runtime_row_filters', new_vector,
                        test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS)})
 
