@@ -31,12 +31,12 @@ from parquet.ttypes import ConvertedType
 from subprocess import check_call
 
 from testdata.common import widetable
-from tests.common.file_utils import create_table_and_copy_files
 from tests.common.impala_test_suite import ImpalaTestSuite, LOG
 from tests.common.skip import (
     SkipIf,
     SkipIfS3,
     SkipIfGCS,
+    SkipIfCOS,
     SkipIfABFS,
     SkipIfADLS,
     SkipIfEC,
@@ -91,12 +91,14 @@ class TestScannersAllTableFormats(ImpalaTestSuite):
         ImpalaTestDimension('batch_size', *TestScannersAllTableFormats.BATCH_SIZES))
     cls.ImpalaTestMatrix.add_dimension(
         ImpalaTestDimension('debug_action', *DEBUG_ACTION_DIMS))
+    cls.ImpalaTestMatrix.add_dimension(ImpalaTestDimension('mt_dop', *MT_DOP_VALUES))
 
   def test_scanners(self, vector):
     new_vector = deepcopy(vector)
     # Copy over test dimensions to the matching query options.
     new_vector.get_value('exec_option')['batch_size'] = vector.get_value('batch_size')
     new_vector.get_value('exec_option')['debug_action'] = vector.get_value('debug_action')
+    new_vector.get_value('exec_option')['mt_dop'] = vector.get_value('mt_dop')
     self.run_test_case('QueryTest/scanners', new_vector)
 
   def test_many_nulls(self, vector):
@@ -107,6 +109,7 @@ class TestScannersAllTableFormats(ImpalaTestSuite):
     new_vector = deepcopy(vector)
     new_vector.get_value('exec_option')['batch_size'] = vector.get_value('batch_size')
     new_vector.get_value('exec_option')['debug_action'] = vector.get_value('debug_action')
+    new_vector.get_value('exec_option')['mt_dop'] = vector.get_value('mt_dop')
     self.run_test_case('QueryTest/scanners-many-nulls', new_vector)
 
   def test_hdfs_scanner_profile(self, vector):
@@ -357,6 +360,9 @@ class TestIceberg(ImpalaTestSuite):
   def test_iceberg_query(self, vector):
     self.run_test_case('QueryTest/iceberg-query', vector)
 
+  def test_iceberg_old_fileformat(self, vector, unique_database):
+    self.run_test_case('QueryTest/iceberg-old-fileformat', vector, use_db=unique_database)
+
   def test_iceberg_profile(self, vector, unique_database):
     self.run_test_case('QueryTest/iceberg-profile', vector, use_db=unique_database)
 
@@ -385,6 +391,10 @@ class TestParquet(ImpalaTestSuite):
     new_vector.get_value('exec_option')['abort_on_error'] = 1
     self.run_test_case('QueryTest/parquet-abort-on-error', new_vector)
 
+  def test_default_scale(self, vector, unique_database):
+    create_table_from_parquet(self.client, unique_database, "no_scale")
+    self.run_test_case('QueryTest/default-scale', vector, unique_database)
+
   def test_timestamp_out_of_range(self, vector, unique_database):
     """IMPALA-4363: Test scanning parquet files with an out of range timestamp.
        Also tests IMPALA-7595: Test Parquet timestamp columns where the time part
@@ -468,6 +478,7 @@ class TestParquet(ImpalaTestSuite):
   @SkipIfLocal.hive
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   def test_multi_compression_types(self, vector, unique_database):
     """IMPALA-5448: Tests that parquet splits with multi compression types are counted
     correctly. Cases tested:
@@ -585,8 +596,18 @@ class TestParquet(ImpalaTestSuite):
     self.run_test_case('QueryTest/parquet-num-values-def-levels-mismatch',
         vector, unique_database)
 
+    """IMPALA-11134: Impala returns "Couldn't skip rows in file" error for old
+    (possibly corrupt) Parquet file where there are more def levels than num_values"""
+    create_table_from_parquet(self.client, unique_database,
+        "too_many_def_levels")
+    result = self.client.execute("select i_item_id from {0}."
+        "too_many_def_levels where i_item_sk = 350963".format(unique_database))
+    assert len(result.data) == 1
+    assert "AAAAAAAACPKFFAAA" in result.data
+
   @SkipIfS3.hdfs_block_size
   @SkipIfGCS.hdfs_block_size
+  @SkipIfCOS.hdfs_block_size
   @SkipIfABFS.hdfs_block_size
   @SkipIfADLS.hdfs_block_size
   @SkipIfIsilon.hdfs_block_size
@@ -645,6 +666,7 @@ class TestParquet(ImpalaTestSuite):
 
   @SkipIfS3.hdfs_block_size
   @SkipIfGCS.hdfs_block_size
+  @SkipIfCOS.hdfs_block_size
   @SkipIfABFS.hdfs_block_size
   @SkipIfADLS.hdfs_block_size
   @SkipIfIsilon.hdfs_block_size
@@ -669,6 +691,11 @@ class TestParquet(ImpalaTestSuite):
       assert len(num_rows_read_list) == 7
       assert len(ranges_complete_list) == 7
 
+      # Extract the host for each fragment instance. The first is the coordinator
+      # fragment instance.
+      host_list = re.findall(r'host=(\S+:[0-9]*)', result.runtime_profile)
+      assert len(host_list) == 7
+
       total_rows_read = 0
       # Skip the Averaged Fragment; it comes first in the runtime profile.
       for num_row_read in num_rows_read_list[1:]:
@@ -676,15 +703,21 @@ class TestParquet(ImpalaTestSuite):
       assert total_rows_read == TOTAL_ROWS
 
       # Again skip the Averaged Fragment; it comes first in the runtime profile.
-      # With mt_dop 2, every backend will have 2 instances which are printed consecutively
-      # in the profile.
-      for i in range(1, len(ranges_complete_list), 2):
-        assert int(ranges_complete_list[i]) + int(ranges_complete_list[i + 1]) == 2
+      # With mt_dop 2, every backend will have 2 instances.
+      ranges_per_host = {}
+      for i in range(1, 7):
+        host = host_list[i]
+        if host not in ranges_per_host:
+          ranges_per_host[host] = 0
+        ranges_per_host[host] += int(ranges_complete_list[i])
+      for host in ranges_per_host:
+        assert ranges_per_host[host] == 2
     finally:
       self.client.clear_configuration()
 
   @SkipIfS3.hdfs_block_size
   @SkipIfGCS.hdfs_block_size
+  @SkipIfCOS.hdfs_block_size
   @SkipIfABFS.hdfs_block_size
   @SkipIfADLS.hdfs_block_size
   @SkipIfIsilon.hdfs_block_size
@@ -702,6 +735,7 @@ class TestParquet(ImpalaTestSuite):
 
   @SkipIfS3.hdfs_block_size
   @SkipIfGCS.hdfs_block_size
+  @SkipIfCOS.hdfs_block_size
   @SkipIfABFS.hdfs_block_size
   @SkipIfADLS.hdfs_block_size
   @SkipIfIsilon.hdfs_block_size
@@ -1052,6 +1086,61 @@ class TestParquet(ImpalaTestSuite):
         else:
           assert summary.total_num_values == 11
 
+  def test_decimal_precision_and_scale_widening(self, vector, unique_database):
+    """IMPALA-7087: Tests that Parquet files stored with a lower precision or scale than
+       the table metadata can be read by Impala.
+    """
+    # The file binary_decimal_precision_widening is written with schema (decimal(9,2),
+    # decimal(18,2), decimal(38,2))
+    binary_decimal_test_files =\
+        ["testdata/data/binary_decimal_precision_and_scale_widening.parquet"]
+
+    # Test reading Parquet files when the table has a higher precision than the file
+    create_table_and_copy_files(self.client, """create table if not exists {db}.{tbl}
+        (small_dec decimal(38,2), med_dec decimal(38,2), large_dec decimal(38,2))
+        STORED AS PARQUET""", unique_database, "binary_decimal_precision_widening",
+        binary_decimal_test_files)
+
+    # Test reading Parquet files when the table has a higher scale than the file
+    create_table_and_copy_files(self.client, """create table if not exists {db}.{tbl}
+        (small_dec decimal(9,4), med_dec decimal(18,4), large_dec decimal(38,4))
+        STORED AS PARQUET""", unique_database, "binary_decimal_scale_widening",
+        binary_decimal_test_files)
+
+    # Test reading Parquet files when the table has a higher precision and scale than the
+    # file
+    create_table_and_copy_files(self.client, """create table if not exists {db}.{tbl}
+        (small_dec decimal(38,4), med_dec decimal(38,4), large_dec decimal(38,4))
+        STORED AS PARQUET""", unique_database,
+        "binary_decimal_precision_and_scale_widening", binary_decimal_test_files)
+
+    # Test Parquet precision and scale widening when decimals are stored as INT32
+    create_table_and_copy_files(self.client, """create table if not exists {db}.{tbl}
+        (team string, score decimal(12, 6)) STORED AS PARQUET""", unique_database,
+        "int32_decimal_precision_and_scale_widening",
+        ["testdata/data/decimal_stored_as_int32.parquet"])
+
+    # Test Parquet precision and scale widening when decimals are stored as INT64
+    create_table_and_copy_files(self.client, """create table if not exists {db}.{tbl}
+        (team string, score decimal(32, 8)) STORED AS PARQUET""", unique_database,
+        "int64_decimal_precision_and_scale_widening",
+        ["testdata/data/decimal_stored_as_int64.parquet"])
+
+    # Unlike the file binary_decimal_precision_and_scale_widening.parquet, all the values
+    # in binary_decimal_no_dictionary.parquet cannot be converted to a higher scale
+    # without overflowing
+    create_table_and_copy_files(self.client, """create table if not exists {db}.{tbl}
+        (small_dec decimal(9,4), med_dec decimal(18,4), large_dec decimal(38,4))
+        STORED AS PARQUET""", unique_database, "scale_overflow",
+        ["testdata/data/binary_decimal_no_dictionary.parquet"])
+
+    self.run_test_case("QueryTest/parquet-decimal-precision-and-scale-widening", vector,
+                       unique_database)
+
+  def test_decimal_precision_and_scale_altering(self, vector, unique_database):
+    self.run_test_case(
+        "QueryTest/parquet-decimal-precision-and-scale-altering", vector, unique_database)
+
 
 # We use various scan range lengths to exercise corner cases in the HDFS scanner more
 # thoroughly. In particular, it will exercise:
@@ -1309,6 +1398,7 @@ class TestTextScanRangeLengths(ImpalaTestSuite):
 # Missing Coverage: No coverage for truncated files errors or scans.
 @SkipIfS3.hive
 @SkipIfGCS.hive
+@SkipIfCOS.hive
 @SkipIfABFS.hive
 @SkipIfADLS.hive
 @SkipIfIsilon.hive
@@ -1390,9 +1480,11 @@ class TestOrc(ImpalaTestSuite):
     super(TestOrc, cls).add_test_dimensions()
     cls.ImpalaTestMatrix.add_constraint(
       lambda v: v.get_value('table_format').file_format == 'orc')
+    cls.ImpalaTestMatrix.add_dimension(ImpalaTestDimension('orc_schema_resolution', 0, 1))
 
   @SkipIfS3.hdfs_block_size
   @SkipIfGCS.hdfs_block_size
+  @SkipIfCOS.hdfs_block_size
   @SkipIfABFS.hdfs_block_size
   @SkipIfADLS.hdfs_block_size
   @SkipIfEC.fix_later
@@ -1437,7 +1529,8 @@ class TestOrc(ImpalaTestSuite):
   def _misaligned_orc_stripes_helper(
           self, table_name, rows_in_table, num_scanners_with_no_reads=0):
     """Checks if 'num_scanners_with_no_reads' indicates the expected number of scanners
-    that don't read anything because the underlying file is poorly formatted
+    that don't read anything because the underlying file is poorly formatted.
+    Additionally, test that select count(star) match with expected number of rows.
     """
     query = 'select * from %s' % table_name
     result = self.client.execute(query)
@@ -1458,6 +1551,11 @@ class TestOrc(ImpalaTestSuite):
       total += int(n)
     assert total == num_scanners_with_no_reads
 
+    # Test that select count(star) match with expected number of rows.
+    query = 'select count(*) from %s' % table_name
+    result = self.client.execute(query)
+    assert int(result.data[0]) == rows_in_table
+
   # Skip this test on non-HDFS filesystems, because orc-type-check.test contains Hive
   # queries that hang in some cases (IMPALA-9345). It would be possible to separate
   # the tests that use Hive and run most tests on S3, but I think that running these on
@@ -1468,6 +1566,7 @@ class TestOrc(ImpalaTestSuite):
   @SkipIfLocal.hive
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfHive3.non_acid
   def test_type_conversions_hive2(self, vector, unique_database):
     # Create "illtypes" tables whose columns can't match the underlining ORC file's.
@@ -1518,6 +1617,7 @@ class TestOrc(ImpalaTestSuite):
   @SkipIfLocal.hive
   @SkipIfS3.hive
   @SkipIfGCS.hive
+  @SkipIfCOS.hive
   @SkipIfHive2.acid
   def test_type_conversions_hive3(self, vector, unique_database):
     # Create "illtypes" tables whose columns can't match the underlining ORC file's.
@@ -1581,6 +1681,16 @@ class TestOrc(ImpalaTestSuite):
       self.run_test_case('DataErrorsTest/orc-out-of-range-timestamp',
                          new_vector, unique_database)
 
+  def test_orc_timestamp_with_local_timezone(self, vector, unique_database):
+      """Test scanning of ORC file that contains 'timstamp with local timezone'."""
+      test_files = ["testdata/data/timestamp_with_local_timezone.orc"]
+      create_table_and_copy_files(self.client,
+          "create table {db}.{tbl} "
+          "(id int, user string, action string, event_time timestamp) "
+          "stored as orc", unique_database, "timestamp_with_local_timezone", test_files)
+      self.run_test_case("QueryTest/orc_timestamp_with_local_timezone", vector,
+          unique_database)
+
   def _run_invalid_schema_test(self, unique_database, test_name, expected_error):
     """Copies 'test_name'.orc to a table and runs a simple query. These tests should
        cause an error during the processing of the ORC schema, so the file's columns do
@@ -1591,13 +1701,13 @@ class TestOrc(ImpalaTestSuite):
         "CREATE TABLE {db}.{tbl} (id BIGINT) STORED AS ORC",
         unique_database, test_name, test_files)
     err = self.execute_query_expect_failure(self.client,
-        "select count(*) from {0}.{1}".format(unique_database, test_name))
+        "select count(id) from {0}.{1}".format(unique_database, test_name))
     assert expected_error in str(err)
 
   def test_invalid_schema(self, vector, unique_database):
     """Test scanning of ORC file with malformed schema."""
     self._run_invalid_schema_test(unique_database, "corrupt_schema",
-        "Encountered parse error during schema selection")
+        "Encountered parse error in tail of ORC file")
     self._run_invalid_schema_test(unique_database, "corrupt_root_type",
         "Root of the selected type returned by the ORC lib is not STRUCT: boolean.")
 
@@ -1627,6 +1737,66 @@ class TestOrc(ImpalaTestSuite):
 
     self.run_test_case('QueryTest/hive2-pre-gregorian-date-orc', vector, unique_database)
 
+  @SkipIfABFS.hive
+  @SkipIfADLS.hive
+  @SkipIfIsilon.hive
+  @SkipIfLocal.hive
+  @SkipIfS3.hive
+  @SkipIfGCS.hive
+  @SkipIfCOS.hive
+  def test_missing_field_orc(self, unique_database):
+    # Test scanning orc files with missing fields in file meta.
+    orc_tbl_name = unique_database + ".missing_field_orc"
+    self.client.execute("create table %s (f0 int) stored as orc" % orc_tbl_name)
+    self.run_stmt_in_hive("insert into table %s select 1" % orc_tbl_name)
+    self.client.execute("refresh %s" % orc_tbl_name)
+
+    self.client.execute("alter table %s add columns(f1 int)" % orc_tbl_name)
+    result = self.client.execute("select f1 from %s " % orc_tbl_name)
+    assert result.data == ['NULL']
+
+    self.client.execute("alter table %s add columns(f2 STRUCT<s0:STRING, s1:STRING>)"
+                        % orc_tbl_name)
+    result = self.client.execute("select f2.s0 from %s " % orc_tbl_name)
+    assert result.data == ['NULL']
+
+    orc_tbl_name = unique_database + ".missing_field_full_txn_test"
+    self.client.execute("create table %s(f0 int) stored as orc "
+                        "tblproperties('transactional'='true')" % orc_tbl_name)
+    self.run_stmt_in_hive("insert into %s values(0)" % orc_tbl_name)
+    self.run_stmt_in_hive("alter table %s add columns(f1 int)" % orc_tbl_name)
+    self.run_stmt_in_hive("insert into %s values(1,1)" % orc_tbl_name)
+    self.client.execute("refresh %s" % orc_tbl_name)
+    result = self.client.execute("select f1 from %s" % orc_tbl_name)
+    assert len(result.data) == 2
+    assert '1' in result.data
+    assert 'NULL' in result.data
+
+    # TODO: add a test case for Iceberg tables once IMPALA-10542 is done.
+    # orc_tbl_name = unique_database + ".missing_field_iceberg_test"
+    # self.client.execute("create table %s (f0 int) stored as iceberg "
+    #                     "tblproperties('write.format.default' = 'orc')"
+    #                     % orc_tbl_name)
+    # self.run_stmt_in_hive("insert into %s values(0)" % orc_tbl_name)
+    # self.run_stmt_in_hive("alter table %s add columns(f1 int)" % orc_tbl_name)
+    # self.run_stmt_in_hive("insert into %s values(1,1)" % orc_tbl_name)
+    # self.client.execute("refresh %s" % orc_tbl_name)
+    # result = self.client.execute("select f1 from %s" % orc_tbl_name)
+    # assert len(result.data) == 2
+    # assert '1' in result.data
+    # assert 'NULL' in result.data
+
+    orc_tbl_name = unique_database + ".lineitem_orc_ext"
+    test_file = "/test-warehouse/tpch.lineitem_orc_def"
+    create_sql = "create external table %s like tpch_orc_def.lineitem " \
+                 "location '%s'" % (orc_tbl_name, test_file)
+    self.client.execute(create_sql)
+    self.client.execute("alter table %s add columns (new_col int)" % orc_tbl_name)
+    result = self.execute_query("select count(*) from %s where new_col is null"
+                                % orc_tbl_name)
+    assert len(result.data) == 1
+    assert '6001215' in result.data
+
 
 class TestScannerReservation(ImpalaTestSuite):
   @classmethod
