@@ -73,6 +73,9 @@ class TestTpcdsQuery(ImpalaTestSuite):
   def test_tpcds_q8(self, vector):
     self.run_test_case(self.get_workload() + '-q8', vector)
 
+  def test_tpcds_q9(self, vector):
+    self.run_test_case(self.get_workload() + '-q9', vector)
+
   def test_tpcds_q10a(self, vector):
     self.run_test_case(self.get_workload() + '-q10a', vector)
 
@@ -85,6 +88,12 @@ class TestTpcdsQuery(ImpalaTestSuite):
   def test_tpcds_q13(self, vector):
     self.run_test_case(self.get_workload() + '-q13', vector)
 
+  def test_tpcds_q14_1(self, vector):
+    self.run_test_case(self.get_workload() + '-q14-1', vector)
+
+  def test_tpcds_q14_2(self, vector):
+    self.run_test_case(self.get_workload() + '-q14-2', vector)
+
   def test_tpcds_q15(self, vector):
     self.run_test_case(self.get_workload() + '-q15', vector)
 
@@ -106,12 +115,30 @@ class TestTpcdsQuery(ImpalaTestSuite):
   def test_tpcds_q21(self, vector):
     self.run_test_case(self.get_workload() + '-q21', vector)
 
+  def test_tpcds_q23_1(self, vector):
+    self.run_test_case(self.get_workload() + '-q23-1', vector)
+
+  def test_tpcds_q23_2(self, vector):
+    self.run_test_case(self.get_workload() + '-q23-2', vector)
+
+  def test_tpcds_q24_1(self, vector):
+    self.run_test_case(self.get_workload() + '-q24-1', vector)
+
+  def test_tpcds_q24_2(self, vector):
+    self.run_test_case(self.get_workload() + '-q24-2', vector)
+
   def test_tpcds_q25(self, vector):
     self.run_test_case(self.get_workload() + '-q25', vector)
 
+  def test_tpcds_q26(self, vector):
+    self.run_test_case(self.get_workload() + '-q26', vector)
+
   def test_tpcds_q29(self, vector):
     self.run_test_case(self.get_workload() + '-q29', vector)
 
+  def test_tpcds_q30(self, vector):
+    self.run_test_case(self.get_workload() + '-q30', vector)
+
   def test_tpcds_q32(self, vector):
     self.run_test_case(self.get_workload() + '-q32', vector)
 
@@ -124,6 +151,9 @@ class TestTpcdsQuery(ImpalaTestSuite):
   def test_tpcds_q37(self, vector):
     self.run_test_case(self.get_workload() + '-q37', vector)
 
+  def test_tpcds_q38(self, vector):
+    self.run_test_case(self.get_workload() + '-q38', vector)
+
   def test_tpcds_q39_1(self, vector):
     self.run_test_case(self.get_workload() + '-q39-1', vector)
 
@@ -142,9 +172,18 @@ class TestTpcdsQuery(ImpalaTestSuite):
   def test_tpcds_q43(self, vector):
     self.run_test_case(self.get_workload() + '-q43', vector)
 
+  def test_tpcds_q44(self, vector):
+    self.run_test_case(self.get_workload() + '-q44', vector)
+
   def test_tpcds_q46(self, vector):
     self.run_test_case(self.get_workload() + '-q46', vector)
 
+  def test_tpcds_q47(self, vector):
+    self.run_test_case(self.get_workload() + '-q47', vector)
+
+  def test_tpcds_q48(self, vector):
+    self.run_test_case(self.get_workload() + '-q48', vector)
+
   def test_tpcds_q50(self, vector):
     self.run_test_case(self.get_workload() + '-q50', vector)
 
@@ -169,6 +208,15 @@ class TestTpcdsQuery(ImpalaTestSuite):
   def test_tpcds_q56(self, vector):
     self.run_test_case(self.get_workload() + '-q56', vector)
 
+  def test_tpcds_q57(self, vector):
+    self.run_test_case(self.get_workload() + '-q57', vector)
+
+  def test_tpcds_q58(self, vector):
+    self.run_test_case(self.get_workload() + '-q58', vector)
+
+  def test_tpcds_q59(self, vector):
+    self.run_test_case(self.get_workload() + '-q59', vector)
+
   def test_tpcds_q60(self, vector):
     self.run_test_case(self.get_workload() + '-q60', vector)
 
@@ -178,6 +226,9 @@ class TestTpcdsQuery(ImpalaTestSuite):
   def test_tpcds_q62(self, vector):
     self.run_test_case(self.get_workload() + '-q62', vector)
 
+  def test_tpcds_q63(self, vector):
+    self.run_test_case(self.get_workload() + '-q63', vector)
+
   def test_tpcds_q64(self, vector):
     self.run_test_case(self.get_workload() + '-q64', vector)
 
@@ -232,15 +283,27 @@ class TestTpcdsQuery(ImpalaTestSuite):
   def test_tpcds_q82(self, vector):
     self.run_test_case(self.get_workload() + '-q82', vector)
 
+  def test_tpcds_q83(self, vector):
+    self.run_test_case(self.get_workload() + '-q83', vector)
+
   def test_tpcds_q84(self, vector):
     self.run_test_case(self.get_workload() + '-q84', vector)
 
+  def test_tpcds_q85(self, vector):
+    self.run_test_case(self.get_workload() + '-q85', vector)
+
   def test_tpcds_q86a(self, vector):
     self.run_test_case(self.get_workload() + '-q86a', vector)
 
+  def test_tpcds_q87(self, vector):
+    self.run_test_case(self.get_workload() + '-q87', vector)
+
   def test_tpcds_q88(self, vector):
     self.run_test_case(self.get_workload() + '-q88', vector)
 
+  def test_tpcds_q89(self, vector):
+    self.run_test_case(self.get_workload() + '-q89', vector)
+
   def test_tpcds_q91(self, vector):
     self.run_test_case(self.get_workload() + '-q91', vector)
 
@@ -300,6 +363,9 @@ class TestTpcdsDecimalV2Query(ImpalaTestSuite):
   def test_tpcds_q4(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q4', vector)
 
+  def test_tpcds_q5(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q5', vector)
+
   def test_tpcds_q6(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q6', vector)
 
@@ -309,6 +375,9 @@ class TestTpcdsDecimalV2Query(ImpalaTestSuite):
   def test_tpcds_q8(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q8', vector)
 
+  def test_tpcds_q9(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q9', vector)
+
   def test_tpcds_q10a(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q10a', vector)
 
@@ -321,6 +390,12 @@ class TestTpcdsDecimalV2Query(ImpalaTestSuite):
   def test_tpcds_q13(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q13', vector)
 
+  def test_tpcds_q14_1(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q14-1', vector)
+
+  def test_tpcds_q14_2(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q14-2', vector)
+
   def test_tpcds_q15(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q15', vector)
 
@@ -330,6 +405,9 @@ class TestTpcdsDecimalV2Query(ImpalaTestSuite):
   def test_tpcds_q17(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q17', vector)
 
+  def test_tpcds_q18(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q18', vector)
+
   def test_tpcds_q18a(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q18a', vector)
 
@@ -342,15 +420,45 @@ class TestTpcdsDecimalV2Query(ImpalaTestSuite):
   def test_tpcds_q21(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q21', vector)
 
+  def test_tpcds_q22(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q22', vector)
+
   def test_tpcds_q22a(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q22a', vector)
 
+  def test_tpcds_q23_1(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q23-1', vector)
+
+  def test_tpcds_q23_2(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q23-2', vector)
+
+  def test_tpcds_q24_1(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q24-1', vector)
+
+  def test_tpcds_q24_2(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q24-2', vector)
+
   def test_tpcds_q25(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q25', vector)
 
+  def test_tpcds_q26(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q26', vector)
+
+  def test_tpcds_q27(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q27', vector)
+
+  def test_tpcds_q28(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q28', vector)
+
   def test_tpcds_q29(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q29', vector)
 
+  def test_tpcds_q30(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q30', vector)
+
+  def test_tpcds_q31(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q31', vector)
+
   def test_tpcds_q32(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q32', vector)
 
@@ -360,9 +468,18 @@ class TestTpcdsDecimalV2Query(ImpalaTestSuite):
   def test_tpcds_q34(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q34', vector)
 
+  def test_tpcds_q35(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q35a', vector)
+
+  def test_tpcds_q36(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q36', vector)
+
   def test_tpcds_q37(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q37', vector)
 
+  def test_tpcds_q38(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q38', vector)
+
   def test_tpcds_q39_1(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q39-1', vector)
 
@@ -381,9 +498,24 @@ class TestTpcdsDecimalV2Query(ImpalaTestSuite):
   def test_tpcds_q43(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q43', vector)
 
+  def test_tpcds_q44(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q44', vector)
+
+  def test_tpcds_q45(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q45', vector)
+
   def test_tpcds_q46(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q46', vector)
 
+  def test_tpcds_q47(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q47', vector)
+
+  def test_tpcds_q48(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q48', vector)
+
+  def test_tpcds_q49(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q49', vector)
+
   def test_tpcds_q50(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q50', vector)
 
@@ -408,6 +540,15 @@ class TestTpcdsDecimalV2Query(ImpalaTestSuite):
   def test_tpcds_q56(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q56', vector)
 
+  def test_tpcds_q57(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q57', vector)
+
+  def test_tpcds_q58(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q58', vector)
+
+  def test_tpcds_q59(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q59', vector)
+
   def test_tpcds_q60(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q60', vector)
 
@@ -417,12 +558,21 @@ class TestTpcdsDecimalV2Query(ImpalaTestSuite):
   def test_tpcds_q62(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q62', vector)
 
+  def test_tpcds_q63(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q63', vector)
+
   def test_tpcds_q64(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q64', vector)
 
   def test_tpcds_q65(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q65', vector)
 
+  def test_tpcds_q66(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q66', vector)
+
+  def test_tpcds_q67(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q67', vector)
+
   def test_tpcds_q67a(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q67a', vector)
 
@@ -432,6 +582,9 @@ class TestTpcdsDecimalV2Query(ImpalaTestSuite):
   def test_tpcds_q69(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q69', vector)
 
+  def test_tpcds_q70(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q70', vector)
+
   def test_tpcds_q70a(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q70a', vector)
 
@@ -453,6 +606,9 @@ class TestTpcdsDecimalV2Query(ImpalaTestSuite):
   def test_tpcds_q76(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q76', vector)
 
+  def test_tpcds_q77(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q77', vector)
+
   def test_tpcds_q77a(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q77a', vector)
 
@@ -462,6 +618,9 @@ class TestTpcdsDecimalV2Query(ImpalaTestSuite):
   def test_tpcds_q79(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q79', vector)
 
+  def test_tpcds_q80(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q80', vector)
+
   def test_tpcds_q80a(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q80a', vector)
 
@@ -471,21 +630,42 @@ class TestTpcdsDecimalV2Query(ImpalaTestSuite):
   def test_tpcds_q82(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q82', vector)
 
+  def test_tpcds_q83(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q83', vector)
+
   def test_tpcds_q84(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q84', vector)
 
+  def test_tpcds_q85(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q85', vector)
+
+  def test_tpcds_q86(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q86', vector)
+
   def test_tpcds_q86a(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q86a', vector)
 
+  def test_tpcds_q87(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q87', vector)
+
   def test_tpcds_q88(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q88', vector)
 
+  def test_tpcds_q89(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q89', vector)
+
+  def test_tpcds_q90(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q90', vector)
+
   def test_tpcds_q91(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q91', vector)
 
   def test_tpcds_q92(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q92', vector)
 
+  def test_tpcds_q93(self, vector):
+    self.run_test_case(self.get_workload() + '-decimal_v2-q93', vector)
+
   def test_tpcds_q94(self, vector):
     self.run_test_case(self.get_workload() + '-decimal_v2-q94', vector)
 
