@@ -15,6 +15,7 @@
 # specific language governing permissions and limitations
 # under the License.
 
+import time
 import traceback
 
 from multiprocessing.pool import ThreadPool
@@ -39,5 +40,7 @@ class Task:
 
 def run_tasks(tasks, timeout_seconds=600):
   """Runs a list of Tasks in parallel in a thread pool."""
+  start = time.time()
   pool = ThreadPool(processes=len(tasks))
   pool.map_async(Task.run, tasks).get(timeout_seconds)
+  return time.time() - start
