@@ -20,6 +20,7 @@ from tests.common.file_utils import grep_dir
 from tests.common.skip import SkipIfBuildType, SkipIfDockerizedCluster
 from tests.common.impala_cluster import ImpalaCluster
 from tests.common.impala_test_suite import ImpalaTestSuite
+from datetime import datetime
 import itertools
 import json
 import os
@@ -29,7 +30,6 @@ import requests
 
 
 class TestWebPage(ImpalaTestSuite):
-
   ROOT_URL = "http://localhost:{0}/"
   SET_JAVA_LOGLEVEL_URL = "http://localhost:{0}/set_java_loglevel"
   RESET_JAVA_LOGLEVEL_URL = "http://localhost:{0}/reset_java_loglevel"
@@ -115,6 +115,12 @@ class TestWebPage(ImpalaTestSuite):
     assert page.status_code == requests.codes.ok
     page = requests.get("http://localhost:25020/memz")
     assert page.status_code == requests.codes.ok
+    page = requests.head("http://localhost:25000/memz")
+    assert page.status_code == requests.codes.ok
+    page = requests.head("http://localhost:25010/memz")
+    assert page.status_code == requests.codes.ok
+    page = requests.head("http://localhost:25020/memz")
+    assert page.status_code == requests.codes.ok
 
   def test_memz_shows_fragment_instance_id(self):
     """Tests that the memory breakdown on memz shows fragment instance IDs."""
@@ -161,6 +167,9 @@ class TestWebPage(ImpalaTestSuite):
     responses = []
     for port in ports_to_test:
       input_url = url.format(port)
+      response = requests.head(input_url, headers=headers)
+      assert response.status_code == requests.codes.ok, "URL: {0} Str:'{1}'\nResp:{2}"\
+        .format(input_url, string_to_search, response.text)
       response = requests.get(input_url, headers=headers)
       assert response.status_code == requests.codes.ok, "URL: {0} Str:'{1}'\nResp:{2}"\
         .format(input_url, string_to_search, response.text)
@@ -171,6 +180,7 @@ class TestWebPage(ImpalaTestSuite):
         assert string_to_search in response.text, "URL: {0} Str:'{1}'\nResp:{2}".format(
           input_url, string_to_search, response.text)
       responses.append(response)
+      assert 'Content-Security-Policy' in response.headers, "CSP header missing"
     return responses
 
   def get_debug_page(self, page_url, port=25000):
@@ -391,6 +401,9 @@ class TestWebPage(ImpalaTestSuite):
       ports_to_test = self.TEST_PORTS_WITH_SS
     for port in ports_to_test:
       input_url = url.format(port)
+      response = requests.head(input_url)
+      assert response.status_code == requests.codes.not_found, "URL: {0} Str:'{" \
+        "1}'\nResp:{2}".format(input_url, string_to_search, response.text)
       response = requests.get(input_url)
       assert response.status_code == requests.codes.not_found, "URL: {0} Str:'{" \
         "1}'\nResp:{2}".format(input_url, string_to_search, response.text)
@@ -466,7 +479,8 @@ class TestWebPage(ImpalaTestSuite):
       if expected_result in json_part['stmt']:
         check_if_contains = True
         break
-    assert check_if_contains, "No matching statement found in the jsons."
+    assert check_if_contains, "No matching statement found in the jsons at {}: {}".format(
+        datetime.now(), response_json)
 
   def __run_query_and_get_debug_page(self, query, page_url, query_options=None,
                                      expected_state=None):
@@ -779,6 +793,8 @@ class TestWebPage(ImpalaTestSuite):
     for port in self.TEST_PORTS_WITH_SS:
       page = requests.get(self.HEALTHZ_URL.format(port))
       assert page.status_code == requests.codes.ok
+      page = requests.head(self.HEALTHZ_URL.format(port))
+      assert page.status_code == requests.codes.ok
 
   def test_knox_compatibility(self):
     """Checks that the template files conform to the requirements for compatibility with
@@ -823,10 +839,11 @@ class TestWebPage(ImpalaTestSuite):
       .format("25000"), query_id)
     requests.get(cancel_query_url)
     response = requests.get(text_profile_url)
-    cancel_status = "Cancelled from Impala&apos;s debug web interface by client at"
+    cancel_status = "Cancelled from Impala&apos;s debug web interface by user: " \
+                    "&apos;anonymous&apos; at"
     assert cancel_status in response.text
     self.assert_impalad_log_contains("INFO", "Cancelled from Impala\'s debug web "
-      "interface by client at", expected_count=-1)
+      "interface by user: 'anonymous' at", expected_count=-1)
     # Session closing from the WebUI does not produce the cause message in the profile,
     # so we will skip checking the runtime profile.
     results = self.execute_query("select current_session()")
@@ -835,9 +852,11 @@ class TestWebPage(ImpalaTestSuite):
       ("25000"), session_id)
     requests.get(close_session_url)
     self.assert_impalad_log_contains("INFO", "Session closed from Impala\'s debug "
-      "web interface by client at", expected_count=-1)
+      "web interface by user: 'anonymous' at", expected_count=-1)
 
   def test_catalog_operations_endpoint(self):
     """Test to check that the /operations endpoint returns 200 OK."""
     page = requests.get("http://localhost:25020/operations")
     assert page.status_code == requests.codes.ok
+    page = requests.head("http://localhost:25020/operations")
+    assert page.status_code == requests.codes.ok
